function p$(e,t){for(var n=0;n<t.length;n++){const r=t[n];if(typeof r!="string"&&!Array.isArray(r)){for(const i in r)if(i!=="default"&&!(i in e)){const u=Object.getOwnPropertyDescriptor(r,i);u&&Object.defineProperty(e,i,u.get?u:{enumerable:!0,get:()=>r[i]})}}}return Object.freeze(Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}))}(function(){const t=document.createElement("link").relList;if(t&&t.supports&&t.supports("modulepreload"))return;for(const i of document.querySelectorAll('link[rel="modulepreload"]'))r(i);new MutationObserver(i=>{for(const u of i)if(u.type==="childList")for(const s of u.addedNodes)s.tagName==="LINK"&&s.rel==="modulepreload"&&r(s)}).observe(document,{childList:!0,subtree:!0});function n(i){const u={};return i.integrity&&(u.integrity=i.integrity),i.referrerPolicy&&(u.referrerPolicy=i.referrerPolicy),i.crossOrigin==="use-credentials"?u.credentials="include":i.crossOrigin==="anonymous"?u.credentials="omit":u.credentials="same-origin",u}function r(i){if(i.ep)return;i.ep=!0;const u=n(i);fetch(i.href,u)}})();var lf=typeof globalThis<"u"?globalThis:typeof window<"u"?window:typeof global<"u"?global:typeof self<"u"?self:{};function lt(e){return e&&e.__esModule&&Object.prototype.hasOwnProperty.call(e,"default")?e.default:e}var Pv={exports:{}},pl={};/**
 * @license React
 * react-jsx-runtime.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var gO;function v$(){if(gO)return pl;gO=1;var e=Symbol.for("react.transitional.element"),t=Symbol.for("react.fragment");function n(r,i,u){var s=null;if(u!==void 0&&(s=""+u),i.key!==void 0&&(s=""+i.key),"key"in i){u={};for(var c in i)c!=="key"&&(u[c]=i[c])}else u=i;return i=u.ref,{$$typeof:e,type:r,key:s,ref:i!==void 0?i:null,props:u}}return pl.Fragment=t,pl.jsx=n,pl.jsxs=n,pl}var bO;function y$(){return bO||(bO=1,Pv.exports=v$()),Pv.exports}var V=y$(),Rv={exports:{}},De={};/**
 * @license React
 * react.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var xO;function m$(){if(xO)return De;xO=1;var e=Symbol.for("react.transitional.element"),t=Symbol.for("react.portal"),n=Symbol.for("react.fragment"),r=Symbol.for("react.strict_mode"),i=Symbol.for("react.profiler"),u=Symbol.for("react.consumer"),s=Symbol.for("react.context"),c=Symbol.for("react.forward_ref"),d=Symbol.for("react.suspense"),h=Symbol.for("react.memo"),v=Symbol.for("react.lazy"),y=Symbol.iterator;function g(N){return N===null||typeof N!="object"?null:(N=y&&N[y]||N["@@iterator"],typeof N=="function"?N:null)}var b={isMounted:function(){return!1},enqueueForceUpdate:function(){},enqueueReplaceState:function(){},enqueueSetState:function(){}},S=Object.assign,_={};function x(N,X,ee){this.props=N,this.context=X,this.refs=_,this.updater=ee||b}x.prototype.isReactComponent={},x.prototype.setState=function(N,X){if(typeof N!="object"&&typeof N!="function"&&N!=null)throw Error("takes an object of state variables to update or a function which returns an object of state variables.");this.updater.enqueueSetState(this,N,X,"setState")},x.prototype.forceUpdate=function(N){this.updater.enqueueForceUpdate(this,N,"forceUpdate")};function E(){}E.prototype=x.prototype;function M(N,X,ee){this.props=N,this.context=X,this.refs=_,this.updater=ee||b}var A=M.prototype=new E;A.constructor=M,S(A,x.prototype),A.isPureReactComponent=!0;var C=Array.isArray,w={H:null,A:null,T:null,S:null,V:null},T=Object.prototype.hasOwnProperty;function P(N,X,ee,te,ce,_e){return ee=_e.ref,{$$typeof:e,type:N,key:X,ref:ee!==void 0?ee:null,props:_e}}function R(N,X){return P(N.type,X,void 0,void 0,void 0,N.props)}function $(N){return typeof N=="object"&&N!==null&&N.$$typeof===e}function J(N){var X={"=":"=0",":":"=2"};return"$"+N.replace(/[=:]/g,function(ee){return X[ee]})}var B=/\/+/g;function G(N,X){return typeof N=="object"&&N!==null&&N.key!=null?J(""+N.key):X.toString(36)}function W(){}function ne(N){switch(N.status){case"fulfilled":return N.value;case"rejected":throw N.reason;default:switch(typeof N.status=="string"?N.then(W,W):(N.status="pending",N.then(function(X){N.status==="pending"&&(N.status="fulfilled",N.value=X)},function(X){N.status==="pending"&&(N.status="rejected",N.reason=X)})),N.status){case"fulfilled":return N.value;case"rejected":throw N.reason}}throw N}function Q(N,X,ee,te,ce){var _e=typeof N;(_e==="undefined"||_e==="boolean")&&(N=null);var be=!1;if(N===null)be=!0;else switch(_e){case"bigint":case"string":case"number":be=!0;break;case"object":switch(N.$$typeof){case e:case t:be=!0;break;case v:return be=N._init,Q(be(N._payload),X,ee,te,ce)}}if(be)return ce=ce(N),be=te===""?"."+G(N,0):te,C(ce)?(ee="",be!=null&&(ee=be.replace(B,"$&/")+"/"),Q(ce,X,ee,"",function(fe){return fe})):ce!=null&&($(ce)&&(ce=R(ce,ee+(ce.key==null||N&&N.key===ce.key?"":(""+ce.key).replace(B,"$&/")+"/")+be)),X.push(ce)),1;be=0;var ke=te===""?".":te+":";if(C(N))for(var Pe=0;Pe<N.length;Pe++)te=N[Pe],_e=ke+G(te,Pe),be+=Q(te,X,ee,_e,ce);else if(Pe=g(N),typeof Pe=="function")for(N=Pe.call(N),Pe=0;!(te=N.next()).done;)te=te.value,_e=ke+G(te,Pe++),be+=Q(te,X,ee,_e,ce);else if(_e==="object"){if(typeof N.then=="function")return Q(ne(N),X,ee,te,ce);throw X=String(N),Error("Objects are not valid as a React child (found: "+(X==="[object Object]"?"object with keys {"+Object.keys(N).join(", ")+"}":X)+"). If you meant to render a collection of children, use an array instead.")}return be}function D(N,X,ee){if(N==null)return N;var te=[],ce=0;return Q(N,te,"","",function(_e){return X.call(ee,_e,ce++)}),te}function H(N){if(N._status===-1){var X=N._result;X=X(),X.then(function(ee){(N._status===0||N._status===-1)&&(N._status=1,N._result=ee)},function(ee){(N._status===0||N._status===-1)&&(N._status=2,N._result=ee)}),N._status===-1&&(N._status=0,N._result=X)}if(N._status===1)return N._result.default;throw N._result}var U=typeof reportError=="function"?reportError:function(N){if(typeof window=="object"&&typeof window.ErrorEvent=="function"){var X=new window.ErrorEvent("error",{bubbles:!0,cancelable:!0,message:typeof N=="object"&&N!==null&&typeof N.message=="string"?String(N.message):String(N),error:N});if(!window.dispatchEvent(X))return}else if(typeof process=="object"&&typeof process.emit=="function"){process.emit("uncaughtException",N);return}console.error(N)};function ue(){}return De.Children={map:D,forEach:function(N,X,ee){D(N,function(){X.apply(this,arguments)},ee)},count:function(N){var X=0;return D(N,function(){X++}),X},toArray:function(N){return D(N,function(X){return X})||[]},only:function(N){if(!$(N))throw Error("React.Children.only expected to receive a single React element child.");return N}},De.Component=x,De.Fragment=n,De.Profiler=i,De.PureComponent=M,De.StrictMode=r,De.Suspense=d,De.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE=w,De.__COMPILER_RUNTIME={__proto__:null,c:function(N){return w.H.useMemoCache(N)}},De.cache=function(N){return function(){return N.apply(null,arguments)}},De.cloneElement=function(N,X,ee){if(N==null)throw Error("The argument must be a React element, but you passed "+N+".");var te=S({},N.props),ce=N.key,_e=void 0;if(X!=null)for(be in X.ref!==void 0&&(_e=void 0),X.key!==void 0&&(ce=""+X.key),X)!T.call(X,be)||be==="key"||be==="__self"||be==="__source"||be==="ref"&&X.ref===void 0||(te[be]=X[be]);var be=arguments.length-2;if(be===1)te.children=ee;else if(1<be){for(var ke=Array(be),Pe=0;Pe<be;Pe++)ke[Pe]=arguments[Pe+2];te.children=ke}return P(N.type,ce,void 0,void 0,_e,te)},De.createContext=function(N){return N={$$typeof:s,_currentValue:N,_currentValue2:N,_threadCount:0,Provider:null,Consumer:null},N.Provider=N,N.Consumer={$$typeof:u,_context:N},N},De.createElement=function(N,X,ee){var te,ce={},_e=null;if(X!=null)for(te in X.key!==void 0&&(_e=""+X.key),X)T.call(X,te)&&te!=="key"&&te!=="__self"&&te!=="__source"&&(ce[te]=X[te]);var be=arguments.length-2;if(be===1)ce.children=ee;else if(1<be){for(var ke=Array(be),Pe=0;Pe<be;Pe++)ke[Pe]=arguments[Pe+2];ce.children=ke}if(N&&N.defaultProps)for(te in be=N.defaultProps,be)ce[te]===void 0&&(ce[te]=be[te]);return P(N,_e,void 0,void 0,null,ce)},De.createRef=function(){return{current:null}},De.forwardRef=function(N){return{$$typeof:c,render:N}},De.isValidElement=$,De.lazy=function(N){return{$$typeof:v,_payload:{_status:-1,_result:N},_init:H}},De.memo=function(N,X){return{$$typeof:h,type:N,compare:X===void 0?null:X}},De.startTransition=function(N){var X=w.T,ee={};w.T=ee;try{var te=N(),ce=w.S;ce!==null&&ce(ee,te),typeof te=="object"&&te!==null&&typeof te.then=="function"&&te.then(ue,U)}catch(_e){U(_e)}finally{w.T=X}},De.unstable_useCacheRefresh=function(){return w.H.useCacheRefresh()},De.use=function(N){return w.H.use(N)},De.useActionState=function(N,X,ee){return w.H.useActionState(N,X,ee)},De.useCallback=function(N,X){return w.H.useCallback(N,X)},De.useContext=function(N){return w.H.useContext(N)},De.useDebugValue=function(){},De.useDeferredValue=function(N,X){return w.H.useDeferredValue(N,X)},De.useEffect=function(N,X,ee){var te=w.H;if(typeof ee=="function")throw Error("useEffect CRUD overload is not enabled in this build of React.");return te.useEffect(N,X)},De.useId=function(){return w.H.useId()},De.useImperativeHandle=function(N,X,ee){return w.H.useImperativeHandle(N,X,ee)},De.useInsertionEffect=function(N,X){return w.H.useInsertionEffect(N,X)},De.useLayoutEffect=function(N,X){return w.H.useLayoutEffect(N,X)},De.useMemo=function(N,X){return w.H.useMemo(N,X)},De.useOptimistic=function(N,X){return w.H.useOptimistic(N,X)},De.useReducer=function(N,X,ee){return w.H.useReducer(N,X,ee)},De.useRef=function(N){return w.H.useRef(N)},De.useState=function(N){return w.H.useState(N)},De.useSyncExternalStore=function(N,X,ee){return w.H.useSyncExternalStore(N,X,ee)},De.useTransition=function(){return w.H.useTransition()},De.version="19.1.0",De}var _O;function Sx(){return _O||(_O=1,Rv.exports=m$()),Rv.exports}var q=Sx();const L=lt(q),g$=p$({__proto__:null,default:L},[q]);var Nv={exports:{}},vl={},Dv={exports:{}},kv={};/**
 * @license React
 * scheduler.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var SO;function b$(){return SO||(SO=1,function(e){function t(D,H){var U=D.length;D.push(H);e:for(;0<U;){var ue=U-1>>>1,N=D[ue];if(0<i(N,H))D[ue]=H,D[U]=N,U=ue;else break e}}function n(D){return D.length===0?null:D[0]}function r(D){if(D.length===0)return null;var H=D[0],U=D.pop();if(U!==H){D[0]=U;e:for(var ue=0,N=D.length,X=N>>>1;ue<X;){var ee=2*(ue+1)-1,te=D[ee],ce=ee+1,_e=D[ce];if(0>i(te,U))ce<N&&0>i(_e,te)?(D[ue]=_e,D[ce]=U,ue=ce):(D[ue]=te,D[ee]=U,ue=ee);else if(ce<N&&0>i(_e,U))D[ue]=_e,D[ce]=U,ue=ce;else break e}}return H}function i(D,H){var U=D.sortIndex-H.sortIndex;return U!==0?U:D.id-H.id}if(e.unstable_now=void 0,typeof performance=="object"&&typeof performance.now=="function"){var u=performance;e.unstable_now=function(){return u.now()}}else{var s=Date,c=s.now();e.unstable_now=function(){return s.now()-c}}var d=[],h=[],v=1,y=null,g=3,b=!1,S=!1,_=!1,x=!1,E=typeof setTimeout=="function"?setTimeout:null,M=typeof clearTimeout=="function"?clearTimeout:null,A=typeof setImmediate<"u"?setImmediate:null;function C(D){for(var H=n(h);H!==null;){if(H.callback===null)r(h);else if(H.startTime<=D)r(h),H.sortIndex=H.expirationTime,t(d,H);else break;H=n(h)}}function w(D){if(_=!1,C(D),!S)if(n(d)!==null)S=!0,T||(T=!0,G());else{var H=n(h);H!==null&&Q(w,H.startTime-D)}}var T=!1,P=-1,R=5,$=-1;function J(){return x?!0:!(e.unstable_now()-$<R)}function B(){if(x=!1,T){var D=e.unstable_now();$=D;var H=!0;try{e:{S=!1,_&&(_=!1,M(P),P=-1),b=!0;var U=g;try{t:{for(C(D),y=n(d);y!==null&&!(y.expirationTime>D&&J());){var ue=y.callback;if(typeof ue=="function"){y.callback=null,g=y.priorityLevel;var N=ue(y.expirationTime<=D);if(D=e.unstable_now(),typeof N=="function"){y.callback=N,C(D),H=!0;break t}y===n(d)&&r(d),C(D)}else r(d);y=n(d)}if(y!==null)H=!0;else{var X=n(h);X!==null&&Q(w,X.startTime-D),H=!1}}break e}finally{y=null,g=U,b=!1}H=void 0}}finally{H?G():T=!1}}}var G;if(typeof A=="function")G=function(){A(B)};else if(typeof MessageChannel<"u"){var W=new MessageChannel,ne=W.port2;W.port1.onmessage=B,G=function(){ne.postMessage(null)}}else G=function(){E(B,0)};function Q(D,H){P=E(function(){D(e.unstable_now())},H)}e.unstable_IdlePriority=5,e.unstable_ImmediatePriority=1,e.unstable_LowPriority=4,e.unstable_NormalPriority=3,e.unstable_Profiling=null,e.unstable_UserBlockingPriority=2,e.unstable_cancelCallback=function(D){D.callback=null},e.unstable_forceFrameRate=function(D){0>D||125<D?console.error("forceFrameRate takes a positive int between 0 and 125, forcing frame rates higher than 125 fps is not supported"):R=0<D?Math.floor(1e3/D):5},e.unstable_getCurrentPriorityLevel=function(){return g},e.unstable_next=function(D){switch(g){case 1:case 2:case 3:var H=3;break;default:H=g}var U=g;g=H;try{return D()}finally{g=U}},e.unstable_requestPaint=function(){x=!0},e.unstable_runWithPriority=function(D,H){switch(D){case 1:case 2:case 3:case 4:case 5:break;default:D=3}var U=g;g=D;try{return H()}finally{g=U}},e.unstable_scheduleCallback=function(D,H,U){var ue=e.unstable_now();switch(typeof U=="object"&&U!==null?(U=U.delay,U=typeof U=="number"&&0<U?ue+U:ue):U=ue,D){case 1:var N=-1;break;case 2:N=250;break;case 5:N=1073741823;break;case 4:N=1e4;break;default:N=5e3}return N=U+N,D={id:v++,callback:H,priorityLevel:D,startTime:U,expirationTime:N,sortIndex:-1},U>ue?(D.sortIndex=U,t(h,D),n(d)===null&&D===n(h)&&(_?(M(P),P=-1):_=!0,Q(w,U-ue))):(D.sortIndex=N,t(d,D),S||b||(S=!0,T||(T=!0,G()))),D},e.unstable_shouldYield=J,e.unstable_wrapCallback=function(D){var H=g;return function(){var U=g;g=H;try{return D.apply(this,arguments)}finally{g=U}}}}(kv)),kv}var wO;function x$(){return wO||(wO=1,Dv.exports=b$()),Dv.exports}var qv={exports:{}},Kt={};/**
 * @license React
 * react-dom.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var OO;function _$(){if(OO)return Kt;OO=1;var e=Sx();function t(d){var h="https://react.dev/errors/"+d;if(1<arguments.length){h+="?args[]="+encodeURIComponent(arguments[1]);for(var v=2;v<arguments.length;v++)h+="&args[]="+encodeURIComponent(arguments[v])}return"Minified React error #"+d+"; visit "+h+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}function n(){}var r={d:{f:n,r:function(){throw Error(t(522))},D:n,C:n,L:n,m:n,X:n,S:n,M:n},p:0,findDOMNode:null},i=Symbol.for("react.portal");function u(d,h,v){var y=3<arguments.length&&arguments[3]!==void 0?arguments[3]:null;return{$$typeof:i,key:y==null?null:""+y,children:d,containerInfo:h,implementation:v}}var s=e.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE;function c(d,h){if(d==="font")return"";if(typeof h=="string")return h==="use-credentials"?h:""}return Kt.__DOM_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE=r,Kt.createPortal=function(d,h){var v=2<arguments.length&&arguments[2]!==void 0?arguments[2]:null;if(!h||h.nodeType!==1&&h.nodeType!==9&&h.nodeType!==11)throw Error(t(299));return u(d,h,null,v)},Kt.flushSync=function(d){var h=s.T,v=r.p;try{if(s.T=null,r.p=2,d)return d()}finally{s.T=h,r.p=v,r.d.f()}},Kt.preconnect=function(d,h){typeof d=="string"&&(h?(h=h.crossOrigin,h=typeof h=="string"?h==="use-credentials"?h:"":void 0):h=null,r.d.C(d,h))},Kt.prefetchDNS=function(d){typeof d=="string"&&r.d.D(d)},Kt.preinit=function(d,h){if(typeof d=="string"&&h&&typeof h.as=="string"){var v=h.as,y=c(v,h.crossOrigin),g=typeof h.integrity=="string"?h.integrity:void 0,b=typeof h.fetchPriority=="string"?h.fetchPriority:void 0;v==="style"?r.d.S(d,typeof h.precedence=="string"?h.precedence:void 0,{crossOrigin:y,integrity:g,fetchPriority:b}):v==="script"&&r.d.X(d,{crossOrigin:y,integrity:g,fetchPriority:b,nonce:typeof h.nonce=="string"?h.nonce:void 0})}},Kt.preinitModule=function(d,h){if(typeof d=="string")if(typeof h=="object"&&h!==null){if(h.as==null||h.as==="script"){var v=c(h.as,h.crossOrigin);r.d.M(d,{crossOrigin:v,integrity:typeof h.integrity=="string"?h.integrity:void 0,nonce:typeof h.nonce=="string"?h.nonce:void 0})}}else h==null&&r.d.M(d)},Kt.preload=function(d,h){if(typeof d=="string"&&typeof h=="object"&&h!==null&&typeof h.as=="string"){var v=h.as,y=c(v,h.crossOrigin);r.d.L(d,v,{crossOrigin:y,integrity:typeof h.integrity=="string"?h.integrity:void 0,nonce:typeof h.nonce=="string"?h.nonce:void 0,type:typeof h.type=="string"?h.type:void 0,fetchPriority:typeof h.fetchPriority=="string"?h.fetchPriority:void 0,referrerPolicy:typeof h.referrerPolicy=="string"?h.referrerPolicy:void 0,imageSrcSet:typeof h.imageSrcSet=="string"?h.imageSrcSet:void 0,imageSizes:typeof h.imageSizes=="string"?h.imageSizes:void 0,media:typeof h.media=="string"?h.media:void 0})}},Kt.preloadModule=function(d,h){if(typeof d=="string")if(h){var v=c(h.as,h.crossOrigin);r.d.m(d,{as:typeof h.as=="string"&&h.as!=="script"?h.as:void 0,crossOrigin:v,integrity:typeof h.integrity=="string"?h.integrity:void 0})}else r.d.m(d)},Kt.requestFormReset=function(d){r.d.r(d)},Kt.unstable_batchedUpdates=function(d,h){return d(h)},Kt.useFormState=function(d,h,v){return s.H.useFormState(d,h,v)},Kt.useFormStatus=function(){return s.H.useHostTransitionStatus()},Kt.version="19.1.0",Kt}var AO;function D2(){if(AO)return qv.exports;AO=1;function e(){if(!(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__>"u"||typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE!="function"))try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(e)}catch(t){console.error(t)}}return e(),qv.exports=_$(),qv.exports}/**
 * @license React
 * react-dom-client.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var EO;function S$(){if(EO)return vl;EO=1;var e=x$(),t=Sx(),n=D2();function r(a){var o="https://react.dev/errors/"+a;if(1<arguments.length){o+="?args[]="+encodeURIComponent(arguments[1]);for(var l=2;l<arguments.length;l++)o+="&args[]="+encodeURIComponent(arguments[l])}return"Minified React error #"+a+"; visit "+o+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}function i(a){return!(!a||a.nodeType!==1&&a.nodeType!==9&&a.nodeType!==11)}function u(a){var o=a,l=a;if(a.alternate)for(;o.return;)o=o.return;else{a=o;do o=a,(o.flags&4098)!==0&&(l=o.return),a=o.return;while(a)}return o.tag===3?l:null}function s(a){if(a.tag===13){var o=a.memoizedState;if(o===null&&(a=a.alternate,a!==null&&(o=a.memoizedState)),o!==null)return o.dehydrated}return null}function c(a){if(u(a)!==a)throw Error(r(188))}function d(a){var o=a.alternate;if(!o){if(o=u(a),o===null)throw Error(r(188));return o!==a?null:a}for(var l=a,f=o;;){var p=l.return;if(p===null)break;var m=p.alternate;if(m===null){if(f=p.return,f!==null){l=f;continue}break}if(p.child===m.child){for(m=p.child;m;){if(m===l)return c(p),a;if(m===f)return c(p),o;m=m.sibling}throw Error(r(188))}if(l.return!==f.return)l=p,f=m;else{for(var O=!1,j=p.child;j;){if(j===l){O=!0,l=p,f=m;break}if(j===f){O=!0,f=p,l=m;break}j=j.sibling}if(!O){for(j=m.child;j;){if(j===l){O=!0,l=m,f=p;break}if(j===f){O=!0,f=m,l=p;break}j=j.sibling}if(!O)throw Error(r(189))}}if(l.alternate!==f)throw Error(r(190))}if(l.tag!==3)throw Error(r(188));return l.stateNode.current===l?a:o}function h(a){var o=a.tag;if(o===5||o===26||o===27||o===6)return a;for(a=a.child;a!==null;){if(o=h(a),o!==null)return o;a=a.sibling}return null}var v=Object.assign,y=Symbol.for("react.element"),g=Symbol.for("react.transitional.element"),b=Symbol.for("react.portal"),S=Symbol.for("react.fragment"),_=Symbol.for("react.strict_mode"),x=Symbol.for("react.profiler"),E=Symbol.for("react.provider"),M=Symbol.for("react.consumer"),A=Symbol.for("react.context"),C=Symbol.for("react.forward_ref"),w=Symbol.for("react.suspense"),T=Symbol.for("react.suspense_list"),P=Symbol.for("react.memo"),R=Symbol.for("react.lazy"),$=Symbol.for("react.activity"),J=Symbol.for("react.memo_cache_sentinel"),B=Symbol.iterator;function G(a){return a===null||typeof a!="object"?null:(a=B&&a[B]||a["@@iterator"],typeof a=="function"?a:null)}var W=Symbol.for("react.client.reference");function ne(a){if(a==null)return null;if(typeof a=="function")return a.$$typeof===W?null:a.displayName||a.name||null;if(typeof a=="string")return a;switch(a){case S:return"Fragment";case x:return"Profiler";case _:return"StrictMode";case w:return"Suspense";case T:return"SuspenseList";case $:return"Activity"}if(typeof a=="object")switch(a.$$typeof){case b:return"Portal";case A:return(a.displayName||"Context")+".Provider";case M:return(a._context.displayName||"Context")+".Consumer";case C:var o=a.render;return a=a.displayName,a||(a=o.displayName||o.name||"",a=a!==""?"ForwardRef("+a+")":"ForwardRef"),a;case P:return o=a.displayName||null,o!==null?o:ne(a.type)||"Memo";case R:o=a._payload,a=a._init;try{return ne(a(o))}catch{}}return null}var Q=Array.isArray,D=t.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,H=n.__DOM_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,U={pending:!1,data:null,method:null,action:null},ue=[],N=-1;function X(a){return{current:a}}function ee(a){0>N||(a.current=ue[N],ue[N]=null,N--)}function te(a,o){N++,ue[N]=a.current,a.current=o}var ce=X(null),_e=X(null),be=X(null),ke=X(null);function Pe(a,o){switch(te(be,o),te(_e,a),te(ce,null),o.nodeType){case 9:case 11:a=(a=o.documentElement)&&(a=a.namespaceURI)?Zw(a):0;break;default:if(a=o.tagName,o=o.namespaceURI)o=Zw(o),a=Yw(o,a);else switch(a){case"svg":a=1;break;case"math":a=2;break;default:a=0}}ee(ce),te(ce,a)}function fe(){ee(ce),ee(_e),ee(be)}function Oe(a){a.memoizedState!==null&&te(ke,a);var o=ce.current,l=Yw(o,a.type);o!==l&&(te(_e,a),te(ce,l))}function Me(a){_e.current===a&&(ee(ce),ee(_e)),ke.current===a&&(ee(ke),sl._currentValue=U)}var le=Object.prototype.hasOwnProperty,st=e.unstable_scheduleCallback,Ie=e.unstable_cancelCallback,St=e.unstable_shouldYield,wt=e.unstable_requestPaint,dt=e.unstable_now,rr=e.unstable_getCurrentPriorityLevel,xr=e.unstable_ImmediatePriority,Ia=e.unstable_UserBlockingPriority,En=e.unstable_NormalPriority,_r=e.unstable_LowPriority,Pi=e.unstable_IdlePriority,Zs=e.log,Ys=e.unstable_setDisableYieldValue,mu=null,dn=null;function ea(a){if(typeof Zs=="function"&&Ys(a),dn&&typeof dn.setStrictMode=="function")try{dn.setStrictMode(mu,a)}catch{}}var hn=Math.clz32?Math.clz32:ek,QD=Math.log,JD=Math.LN2;function ek(a){return a>>>=0,a===0?32:31-(QD(a)/JD|0)|0}var Xs=256,Ks=4194304;function Ua(a){var o=a&42;if(o!==0)return o;switch(a&-a){case 1:return 1;case 2:return 2;case 4:return 4;case 8:return 8;case 16:return 16;case 32:return 32;case 64:return 64;case 128:return 128;case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:return a&4194048;case 4194304:case 8388608:case 16777216:case 33554432:return a&62914560;case 67108864:return 67108864;case 134217728:return 134217728;case 268435456:return 268435456;case 536870912:return 536870912;case 1073741824:return 0;default:return a}}function Ws(a,o,l){var f=a.pendingLanes;if(f===0)return 0;var p=0,m=a.suspendedLanes,O=a.pingedLanes;a=a.warmLanes;var j=f&134217727;return j!==0?(f=j&~m,f!==0?p=Ua(f):(O&=j,O!==0?p=Ua(O):l||(l=j&~a,l!==0&&(p=Ua(l))))):(j=f&~m,j!==0?p=Ua(j):O!==0?p=Ua(O):l||(l=f&~a,l!==0&&(p=Ua(l)))),p===0?0:o!==0&&o!==p&&(o&m)===0&&(m=p&-p,l=o&-o,m>=l||m===32&&(l&4194048)!==0)?o:p}function gu(a,o){return(a.pendingLanes&~(a.suspendedLanes&~a.pingedLanes)&o)===0}function tk(a,o){switch(a){case 1:case 2:case 4:case 8:case 64:return o+250;case 16:case 32:case 128:case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:return o+5e3;case 4194304:case 8388608:case 16777216:case 33554432:return-1;case 67108864:case 134217728:case 268435456:case 536870912:case 1073741824:return-1;default:return-1}}function j_(){var a=Xs;return Xs<<=1,(Xs&4194048)===0&&(Xs=256),a}function C_(){var a=Ks;return Ks<<=1,(Ks&62914560)===0&&(Ks=4194304),a}function xh(a){for(var o=[],l=0;31>l;l++)o.push(a);return o}function bu(a,o){a.pendingLanes|=o,o!==268435456&&(a.suspendedLanes=0,a.pingedLanes=0,a.warmLanes=0)}function nk(a,o,l,f,p,m){var O=a.pendingLanes;a.pendingLanes=l,a.suspendedLanes=0,a.pingedLanes=0,a.warmLanes=0,a.expiredLanes&=l,a.entangledLanes&=l,a.errorRecoveryDisabledLanes&=l,a.shellSuspendCounter=0;var j=a.entanglements,k=a.expirationTimes,Y=a.hiddenUpdates;for(l=O&~l;0<l;){var ae=31-hn(l),oe=1<<ae;j[ae]=0,k[ae]=-1;var K=Y[ae];if(K!==null)for(Y[ae]=null,ae=0;ae<K.length;ae++){var F=K[ae];F!==null&&(F.lane&=-536870913)}l&=~oe}f!==0&&P_(a,f,0),m!==0&&p===0&&a.tag!==0&&(a.suspendedLanes|=m&~(O&~o))}function P_(a,o,l){a.pendingLanes|=o,a.suspendedLanes&=~o;var f=31-hn(o);a.entangledLanes|=o,a.entanglements[f]=a.entanglements[f]|1073741824|l&4194090}function R_(a,o){var l=a.entangledLanes|=o;for(a=a.entanglements;l;){var f=31-hn(l),p=1<<f;p&o|a[f]&o&&(a[f]|=o),l&=~p}}function _h(a){switch(a){case 2:a=1;break;case 8:a=4;break;case 32:a=16;break;case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:case 4194304:case 8388608:case 16777216:case 33554432:a=128;break;case 268435456:a=134217728;break;default:a=0}return a}function Sh(a){return a&=-a,2<a?8<a?(a&134217727)!==0?32:268435456:8:2}function N_(){var a=H.p;return a!==0?a:(a=window.event,a===void 0?32:dO(a.type))}function rk(a,o){var l=H.p;try{return H.p=a,o()}finally{H.p=l}}var ta=Math.random().toString(36).slice(2),Yt="__reactFiber$"+ta,an="__reactProps$"+ta,Ri="__reactContainer$"+ta,wh="__reactEvents$"+ta,ak="__reactListeners$"+ta,ik="__reactHandles$"+ta,D_="__reactResources$"+ta,xu="__reactMarker$"+ta;function Oh(a){delete a[Yt],delete a[an],delete a[wh],delete a[ak],delete a[ik]}function Ni(a){var o=a[Yt];if(o)return o;for(var l=a.parentNode;l;){if(o=l[Ri]||l[Yt]){if(l=o.alternate,o.child!==null||l!==null&&l.child!==null)for(a=Fw(a);a!==null;){if(l=a[Yt])return l;a=Fw(a)}return o}a=l,l=a.parentNode}return null}function Di(a){if(a=a[Yt]||a[Ri]){var o=a.tag;if(o===5||o===6||o===13||o===26||o===27||o===3)return a}return null}function _u(a){var o=a.tag;if(o===5||o===26||o===27||o===6)return a.stateNode;throw Error(r(33))}function ki(a){var o=a[D_];return o||(o=a[D_]={hoistableStyles:new Map,hoistableScripts:new Map}),o}function qt(a){a[xu]=!0}var k_=new Set,q_={};function Ha(a,o){qi(a,o),qi(a+"Capture",o)}function qi(a,o){for(q_[a]=o,a=0;a<o.length;a++)k_.add(o[a])}var ok=RegExp("^[:A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD][:A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD\\-.0-9\\u00B7\\u0300-\\u036F\\u203F-\\u2040]*$"),$_={},B_={};function uk(a){return le.call(B_,a)?!0:le.call($_,a)?!1:ok.test(a)?B_[a]=!0:($_[a]=!0,!1)}function Fs(a,o,l){if(uk(o))if(l===null)a.removeAttribute(o);else{switch(typeof l){case"undefined":case"function":case"symbol":a.removeAttribute(o);return;case"boolean":var f=o.toLowerCase().slice(0,5);if(f!=="data-"&&f!=="aria-"){a.removeAttribute(o);return}}a.setAttribute(o,""+l)}}function Qs(a,o,l){if(l===null)a.removeAttribute(o);else{switch(typeof l){case"undefined":case"function":case"symbol":case"boolean":a.removeAttribute(o);return}a.setAttribute(o,""+l)}}function Sr(a,o,l,f){if(f===null)a.removeAttribute(l);else{switch(typeof f){case"undefined":case"function":case"symbol":case"boolean":a.removeAttribute(l);return}a.setAttributeNS(o,l,""+f)}}var Ah,z_;function $i(a){if(Ah===void 0)try{throw Error()}catch(l){var o=l.stack.trim().match(/\n( *(at )?)/);Ah=o&&o[1]||"",z_=-1<l.stack.indexOf(`
    at`)?" (<anonymous>)":-1<l.stack.indexOf("@")?"@unknown:0:0":""}return`
`+Ah+a+z_}var Eh=!1;function Th(a,o){if(!a||Eh)return"";Eh=!0;var l=Error.prepareStackTrace;Error.prepareStackTrace=void 0;try{var f={DetermineComponentFrameRoot:function(){try{if(o){var oe=function(){throw Error()};if(Object.defineProperty(oe.prototype,"props",{set:function(){throw Error()}}),typeof Reflect=="object"&&Reflect.construct){try{Reflect.construct(oe,[])}catch(F){var K=F}Reflect.construct(a,[],oe)}else{try{oe.call()}catch(F){K=F}a.call(oe.prototype)}}else{try{throw Error()}catch(F){K=F}(oe=a())&&typeof oe.catch=="function"&&oe.catch(function(){})}}catch(F){if(F&&K&&typeof F.stack=="string")return[F.stack,K.stack]}return[null,null]}};f.DetermineComponentFrameRoot.displayName="DetermineComponentFrameRoot";var p=Object.getOwnPropertyDescriptor(f.DetermineComponentFrameRoot,"name");p&&p.configurable&&Object.defineProperty(f.DetermineComponentFrameRoot,"name",{value:"DetermineComponentFrameRoot"});var m=f.DetermineComponentFrameRoot(),O=m[0],j=m[1];if(O&&j){var k=O.split(`
`),Y=j.split(`
`);for(p=f=0;f<k.length&&!k[f].includes("DetermineComponentFrameRoot");)f++;for(;p<Y.length&&!Y[p].includes("DetermineComponentFrameRoot");)p++;if(f===k.length||p===Y.length)for(f=k.length-1,p=Y.length-1;1<=f&&0<=p&&k[f]!==Y[p];)p--;for(;1<=f&&0<=p;f--,p--)if(k[f]!==Y[p]){if(f!==1||p!==1)do if(f--,p--,0>p||k[f]!==Y[p]){var ae=`
`+k[f].replace(" at new "," at ");return a.displayName&&ae.includes("<anonymous>")&&(ae=ae.replace("<anonymous>",a.displayName)),ae}while(1<=f&&0<=p);break}}}finally{Eh=!1,Error.prepareStackTrace=l}return(l=a?a.displayName||a.name:"")?$i(l):""}function lk(a){switch(a.tag){case 26:case 27:case 5:return $i(a.type);case 16:return $i("Lazy");case 13:return $i("Suspense");case 19:return $i("SuspenseList");case 0:case 15:return Th(a.type,!1);case 11:return Th(a.type.render,!1);case 1:return Th(a.type,!0);case 31:return $i("Activity");default:return""}}function L_(a){try{var o="";do o+=lk(a),a=a.return;while(a);return o}catch(l){return`
Error generating stack: `+l.message+`
`+l.stack}}function Tn(a){switch(typeof a){case"bigint":case"boolean":case"number":case"string":case"undefined":return a;case"object":return a;default:return""}}function I_(a){var o=a.type;return(a=a.nodeName)&&a.toLowerCase()==="input"&&(o==="checkbox"||o==="radio")}function sk(a){var o=I_(a)?"checked":"value",l=Object.getOwnPropertyDescriptor(a.constructor.prototype,o),f=""+a[o];if(!a.hasOwnProperty(o)&&typeof l<"u"&&typeof l.get=="function"&&typeof l.set=="function"){var p=l.get,m=l.set;return Object.defineProperty(a,o,{configurable:!0,get:function(){return p.call(this)},set:function(O){f=""+O,m.call(this,O)}}),Object.defineProperty(a,o,{enumerable:l.enumerable}),{getValue:function(){return f},setValue:function(O){f=""+O},stopTracking:function(){a._valueTracker=null,delete a[o]}}}}function Js(a){a._valueTracker||(a._valueTracker=sk(a))}function U_(a){if(!a)return!1;var o=a._valueTracker;if(!o)return!0;var l=o.getValue(),f="";return a&&(f=I_(a)?a.checked?"true":"false":a.value),a=f,a!==l?(o.setValue(a),!0):!1}function ec(a){if(a=a||(typeof document<"u"?document:void 0),typeof a>"u")return null;try{return a.activeElement||a.body}catch{return a.body}}var ck=/[\n"\\]/g;function Mn(a){return a.replace(ck,function(o){return"\\"+o.charCodeAt(0).toString(16)+" "})}function Mh(a,o,l,f,p,m,O,j){a.name="",O!=null&&typeof O!="function"&&typeof O!="symbol"&&typeof O!="boolean"?a.type=O:a.removeAttribute("type"),o!=null?O==="number"?(o===0&&a.value===""||a.value!=o)&&(a.value=""+Tn(o)):a.value!==""+Tn(o)&&(a.value=""+Tn(o)):O!=="submit"&&O!=="reset"||a.removeAttribute("value"),o!=null?jh(a,O,Tn(o)):l!=null?jh(a,O,Tn(l)):f!=null&&a.removeAttribute("value"),p==null&&m!=null&&(a.defaultChecked=!!m),p!=null&&(a.checked=p&&typeof p!="function"&&typeof p!="symbol"),j!=null&&typeof j!="function"&&typeof j!="symbol"&&typeof j!="boolean"?a.name=""+Tn(j):a.removeAttribute("name")}function H_(a,o,l,f,p,m,O,j){if(m!=null&&typeof m!="function"&&typeof m!="symbol"&&typeof m!="boolean"&&(a.type=m),o!=null||l!=null){if(!(m!=="submit"&&m!=="reset"||o!=null))return;l=l!=null?""+Tn(l):"",o=o!=null?""+Tn(o):l,j||o===a.value||(a.value=o),a.defaultValue=o}f=f??p,f=typeof f!="function"&&typeof f!="symbol"&&!!f,a.checked=j?a.checked:!!f,a.defaultChecked=!!f,O!=null&&typeof O!="function"&&typeof O!="symbol"&&typeof O!="boolean"&&(a.name=O)}function jh(a,o,l){o==="number"&&ec(a.ownerDocument)===a||a.defaultValue===""+l||(a.defaultValue=""+l)}function Bi(a,o,l,f){if(a=a.options,o){o={};for(var p=0;p<l.length;p++)o["$"+l[p]]=!0;for(l=0;l<a.length;l++)p=o.hasOwnProperty("$"+a[l].value),a[l].selected!==p&&(a[l].selected=p),p&&f&&(a[l].defaultSelected=!0)}else{for(l=""+Tn(l),o=null,p=0;p<a.length;p++){if(a[p].value===l){a[p].selected=!0,f&&(a[p].defaultSelected=!0);return}o!==null||a[p].disabled||(o=a[p])}o!==null&&(o.selected=!0)}}function G_(a,o,l){if(o!=null&&(o=""+Tn(o),o!==a.value&&(a.value=o),l==null)){a.defaultValue!==o&&(a.defaultValue=o);return}a.defaultValue=l!=null?""+Tn(l):""}function V_(a,o,l,f){if(o==null){if(f!=null){if(l!=null)throw Error(r(92));if(Q(f)){if(1<f.length)throw Error(r(93));f=f[0]}l=f}l==null&&(l=""),o=l}l=Tn(o),a.defaultValue=l,f=a.textContent,f===l&&f!==""&&f!==null&&(a.value=f)}function zi(a,o){if(o){var l=a.firstChild;if(l&&l===a.lastChild&&l.nodeType===3){l.nodeValue=o;return}}a.textContent=o}var fk=new Set("animationIterationCount aspectRatio borderImageOutset borderImageSlice borderImageWidth boxFlex boxFlexGroup boxOrdinalGroup columnCount columns flex flexGrow flexPositive flexShrink flexNegative flexOrder gridArea gridRow gridRowEnd gridRowSpan gridRowStart gridColumn gridColumnEnd gridColumnSpan gridColumnStart fontWeight lineClamp lineHeight opacity order orphans scale tabSize widows zIndex zoom fillOpacity floodOpacity stopOpacity strokeDasharray strokeDashoffset strokeMiterlimit strokeOpacity strokeWidth MozAnimationIterationCount MozBoxFlex MozBoxFlexGroup MozLineClamp msAnimationIterationCount msFlex msZoom msFlexGrow msFlexNegative msFlexOrder msFlexPositive msFlexShrink msGridColumn msGridColumnSpan msGridRow msGridRowSpan WebkitAnimationIterationCount WebkitBoxFlex WebKitBoxFlexGroup WebkitBoxOrdinalGroup WebkitColumnCount WebkitColumns WebkitFlex WebkitFlexGrow WebkitFlexPositive WebkitFlexShrink WebkitLineClamp".split(" "));function Z_(a,o,l){var f=o.indexOf("--")===0;l==null||typeof l=="boolean"||l===""?f?a.setProperty(o,""):o==="float"?a.cssFloat="":a[o]="":f?a.setProperty(o,l):typeof l!="number"||l===0||fk.has(o)?o==="float"?a.cssFloat=l:a[o]=(""+l).trim():a[o]=l+"px"}function Y_(a,o,l){if(o!=null&&typeof o!="object")throw Error(r(62));if(a=a.style,l!=null){for(var f in l)!l.hasOwnProperty(f)||o!=null&&o.hasOwnProperty(f)||(f.indexOf("--")===0?a.setProperty(f,""):f==="float"?a.cssFloat="":a[f]="");for(var p in o)f=o[p],o.hasOwnProperty(p)&&l[p]!==f&&Z_(a,p,f)}else for(var m in o)o.hasOwnProperty(m)&&Z_(a,m,o[m])}function Ch(a){if(a.indexOf("-")===-1)return!1;switch(a){case"annotation-xml":case"color-profile":case"font-face":case"font-face-src":case"font-face-uri":case"font-face-format":case"font-face-name":case"missing-glyph":return!1;default:return!0}}var dk=new Map([["acceptCharset","accept-charset"],["htmlFor","for"],["httpEquiv","http-equiv"],["crossOrigin","crossorigin"],["accentHeight","accent-height"],["alignmentBaseline","alignment-baseline"],["arabicForm","arabic-form"],["baselineShift","baseline-shift"],["capHeight","cap-height"],["clipPath","clip-path"],["clipRule","clip-rule"],["colorInterpolation","color-interpolation"],["colorInterpolationFilters","color-interpolation-filters"],["colorProfile","color-profile"],["colorRendering","color-rendering"],["dominantBaseline","dominant-baseline"],["enableBackground","enable-background"],["fillOpacity","fill-opacity"],["fillRule","fill-rule"],["floodColor","flood-color"],["floodOpacity","flood-opacity"],["fontFamily","font-family"],["fontSize","font-size"],["fontSizeAdjust","font-size-adjust"],["fontStretch","font-stretch"],["fontStyle","font-style"],["fontVariant","font-variant"],["fontWeight","font-weight"],["glyphName","glyph-name"],["glyphOrientationHorizontal","glyph-orientation-horizontal"],["glyphOrientationVertical","glyph-orientation-vertical"],["horizAdvX","horiz-adv-x"],["horizOriginX","horiz-origin-x"],["imageRendering","image-rendering"],["letterSpacing","letter-spacing"],["lightingColor","lighting-color"],["markerEnd","marker-end"],["markerMid","marker-mid"],["markerStart","marker-start"],["overlinePosition","overline-position"],["overlineThickness","overline-thickness"],["paintOrder","paint-order"],["panose-1","panose-1"],["pointerEvents","pointer-events"],["renderingIntent","rendering-intent"],["shapeRendering","shape-rendering"],["stopColor","stop-color"],["stopOpacity","stop-opacity"],["strikethroughPosition","strikethrough-position"],["strikethroughThickness","strikethrough-thickness"],["strokeDasharray","stroke-dasharray"],["strokeDashoffset","stroke-dashoffset"],["strokeLinecap","stroke-linecap"],["strokeLinejoin","stroke-linejoin"],["strokeMiterlimit","stroke-miterlimit"],["strokeOpacity","stroke-opacity"],["strokeWidth","stroke-width"],["textAnchor","text-anchor"],["textDecoration","text-decoration"],["textRendering","text-rendering"],["transformOrigin","transform-origin"],["underlinePosition","underline-position"],["underlineThickness","underline-thickness"],["unicodeBidi","unicode-bidi"],["unicodeRange","unicode-range"],["unitsPerEm","units-per-em"],["vAlphabetic","v-alphabetic"],["vHanging","v-hanging"],["vIdeographic","v-ideographic"],["vMathematical","v-mathematical"],["vectorEffect","vector-effect"],["vertAdvY","vert-adv-y"],["vertOriginX","vert-origin-x"],["vertOriginY","vert-origin-y"],["wordSpacing","word-spacing"],["writingMode","writing-mode"],["xmlnsXlink","xmlns:xlink"],["xHeight","x-height"]]),hk=/^[\u0000-\u001F ]*j[\r\n\t]*a[\r\n\t]*v[\r\n\t]*a[\r\n\t]*s[\r\n\t]*c[\r\n\t]*r[\r\n\t]*i[\r\n\t]*p[\r\n\t]*t[\r\n\t]*:/i;function tc(a){return hk.test(""+a)?"javascript:throw new Error('React has blocked a javascript: URL as a security precaution.')":a}var Ph=null;function Rh(a){return a=a.target||a.srcElement||window,a.correspondingUseElement&&(a=a.correspondingUseElement),a.nodeType===3?a.parentNode:a}var Li=null,Ii=null;function X_(a){var o=Di(a);if(o&&(a=o.stateNode)){var l=a[an]||null;e:switch(a=o.stateNode,o.type){case"input":if(Mh(a,l.value,l.defaultValue,l.defaultValue,l.checked,l.defaultChecked,l.type,l.name),o=l.name,l.type==="radio"&&o!=null){for(l=a;l.parentNode;)l=l.parentNode;for(l=l.querySelectorAll('input[name="'+Mn(""+o)+'"][type="radio"]'),o=0;o<l.length;o++){var f=l[o];if(f!==a&&f.form===a.form){var p=f[an]||null;if(!p)throw Error(r(90));Mh(f,p.value,p.defaultValue,p.defaultValue,p.checked,p.defaultChecked,p.type,p.name)}}for(o=0;o<l.length;o++)f=l[o],f.form===a.form&&U_(f)}break e;case"textarea":G_(a,l.value,l.defaultValue);break e;case"select":o=l.value,o!=null&&Bi(a,!!l.multiple,o,!1)}}}var Nh=!1;function K_(a,o,l){if(Nh)return a(o,l);Nh=!0;try{var f=a(o);return f}finally{if(Nh=!1,(Li!==null||Ii!==null)&&(Lc(),Li&&(o=Li,a=Ii,Ii=Li=null,X_(o),a)))for(o=0;o<a.length;o++)X_(a[o])}}function Su(a,o){var l=a.stateNode;if(l===null)return null;var f=l[an]||null;if(f===null)return null;l=f[o];e:switch(o){case"onClick":case"onClickCapture":case"onDoubleClick":case"onDoubleClickCapture":case"onMouseDown":case"onMouseDownCapture":case"onMouseMove":case"onMouseMoveCapture":case"onMouseUp":case"onMouseUpCapture":case"onMouseEnter":(f=!f.disabled)||(a=a.type,f=!(a==="button"||a==="input"||a==="select"||a==="textarea")),a=!f;break e;default:a=!1}if(a)return null;if(l&&typeof l!="function")throw Error(r(231,o,typeof l));return l}var wr=!(typeof window>"u"||typeof window.document>"u"||typeof window.document.createElement>"u"),Dh=!1;if(wr)try{var wu={};Object.defineProperty(wu,"passive",{get:function(){Dh=!0}}),window.addEventListener("test",wu,wu),window.removeEventListener("test",wu,wu)}catch{Dh=!1}var na=null,kh=null,nc=null;function W_(){if(nc)return nc;var a,o=kh,l=o.length,f,p="value"in na?na.value:na.textContent,m=p.length;for(a=0;a<l&&o[a]===p[a];a++);var O=l-a;for(f=1;f<=O&&o[l-f]===p[m-f];f++);return nc=p.slice(a,1<f?1-f:void 0)}function rc(a){var o=a.keyCode;return"charCode"in a?(a=a.charCode,a===0&&o===13&&(a=13)):a=o,a===10&&(a=13),32<=a||a===13?a:0}function ac(){return!0}function F_(){return!1}function on(a){function o(l,f,p,m,O){this._reactName=l,this._targetInst=p,this.type=f,this.nativeEvent=m,this.target=O,this.currentTarget=null;for(var j in a)a.hasOwnProperty(j)&&(l=a[j],this[j]=l?l(m):m[j]);return this.isDefaultPrevented=(m.defaultPrevented!=null?m.defaultPrevented:m.returnValue===!1)?ac:F_,this.isPropagationStopped=F_,this}return v(o.prototype,{preventDefault:function(){this.defaultPrevented=!0;var l=this.nativeEvent;l&&(l.preventDefault?l.preventDefault():typeof l.returnValue!="unknown"&&(l.returnValue=!1),this.isDefaultPrevented=ac)},stopPropagation:function(){var l=this.nativeEvent;l&&(l.stopPropagation?l.stopPropagation():typeof l.cancelBubble!="unknown"&&(l.cancelBubble=!0),this.isPropagationStopped=ac)},persist:function(){},isPersistent:ac}),o}var Ga={eventPhase:0,bubbles:0,cancelable:0,timeStamp:function(a){return a.timeStamp||Date.now()},defaultPrevented:0,isTrusted:0},ic=on(Ga),Ou=v({},Ga,{view:0,detail:0}),pk=on(Ou),qh,$h,Au,oc=v({},Ou,{screenX:0,screenY:0,clientX:0,clientY:0,pageX:0,pageY:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,getModifierState:zh,button:0,buttons:0,relatedTarget:function(a){return a.relatedTarget===void 0?a.fromElement===a.srcElement?a.toElement:a.fromElement:a.relatedTarget},movementX:function(a){return"movementX"in a?a.movementX:(a!==Au&&(Au&&a.type==="mousemove"?(qh=a.screenX-Au.screenX,$h=a.screenY-Au.screenY):$h=qh=0,Au=a),qh)},movementY:function(a){return"movementY"in a?a.movementY:$h}}),Q_=on(oc),vk=v({},oc,{dataTransfer:0}),yk=on(vk),mk=v({},Ou,{relatedTarget:0}),Bh=on(mk),gk=v({},Ga,{animationName:0,elapsedTime:0,pseudoElement:0}),bk=on(gk),xk=v({},Ga,{clipboardData:function(a){return"clipboardData"in a?a.clipboardData:window.clipboardData}}),_k=on(xk),Sk=v({},Ga,{data:0}),J_=on(Sk),wk={Esc:"Escape",Spacebar:" ",Left:"ArrowLeft",Up:"ArrowUp",Right:"ArrowRight",Down:"ArrowDown",Del:"Delete",Win:"OS",Menu:"ContextMenu",Apps:"ContextMenu",Scroll:"ScrollLock",MozPrintableKey:"Unidentified"},Ok={8:"Backspace",9:"Tab",12:"Clear",13:"Enter",16:"Shift",17:"Control",18:"Alt",19:"Pause",20:"CapsLock",27:"Escape",32:" ",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"ArrowLeft",38:"ArrowUp",39:"ArrowRight",40:"ArrowDown",45:"Insert",46:"Delete",112:"F1",113:"F2",114:"F3",115:"F4",116:"F5",117:"F6",118:"F7",119:"F8",120:"F9",121:"F10",122:"F11",123:"F12",144:"NumLock",145:"ScrollLock",224:"Meta"},Ak={Alt:"altKey",Control:"ctrlKey",Meta:"metaKey",Shift:"shiftKey"};function Ek(a){var o=this.nativeEvent;return o.getModifierState?o.getModifierState(a):(a=Ak[a])?!!o[a]:!1}function zh(){return Ek}var Tk=v({},Ou,{key:function(a){if(a.key){var o=wk[a.key]||a.key;if(o!=="Unidentified")return o}return a.type==="keypress"?(a=rc(a),a===13?"Enter":String.fromCharCode(a)):a.type==="keydown"||a.type==="keyup"?Ok[a.keyCode]||"Unidentified":""},code:0,location:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,repeat:0,locale:0,getModifierState:zh,charCode:function(a){return a.type==="keypress"?rc(a):0},keyCode:function(a){return a.type==="keydown"||a.type==="keyup"?a.keyCode:0},which:function(a){return a.type==="keypress"?rc(a):a.type==="keydown"||a.type==="keyup"?a.keyCode:0}}),Mk=on(Tk),jk=v({},oc,{pointerId:0,width:0,height:0,pressure:0,tangentialPressure:0,tiltX:0,tiltY:0,twist:0,pointerType:0,isPrimary:0}),e1=on(jk),Ck=v({},Ou,{touches:0,targetTouches:0,changedTouches:0,altKey:0,metaKey:0,ctrlKey:0,shiftKey:0,getModifierState:zh}),Pk=on(Ck),Rk=v({},Ga,{propertyName:0,elapsedTime:0,pseudoElement:0}),Nk=on(Rk),Dk=v({},oc,{deltaX:function(a){return"deltaX"in a?a.deltaX:"wheelDeltaX"in a?-a.wheelDeltaX:0},deltaY:function(a){return"deltaY"in a?a.deltaY:"wheelDeltaY"in a?-a.wheelDeltaY:"wheelDelta"in a?-a.wheelDelta:0},deltaZ:0,deltaMode:0}),kk=on(Dk),qk=v({},Ga,{newState:0,oldState:0}),$k=on(qk),Bk=[9,13,27,32],Lh=wr&&"CompositionEvent"in window,Eu=null;wr&&"documentMode"in document&&(Eu=document.documentMode);var zk=wr&&"TextEvent"in window&&!Eu,t1=wr&&(!Lh||Eu&&8<Eu&&11>=Eu),n1=" ",r1=!1;function a1(a,o){switch(a){case"keyup":return Bk.indexOf(o.keyCode)!==-1;case"keydown":return o.keyCode!==229;case"keypress":case"mousedown":case"focusout":return!0;default:return!1}}function i1(a){return a=a.detail,typeof a=="object"&&"data"in a?a.data:null}var Ui=!1;function Lk(a,o){switch(a){case"compositionend":return i1(o);case"keypress":return o.which!==32?null:(r1=!0,n1);case"textInput":return a=o.data,a===n1&&r1?null:a;default:return null}}function Ik(a,o){if(Ui)return a==="compositionend"||!Lh&&a1(a,o)?(a=W_(),nc=kh=na=null,Ui=!1,a):null;switch(a){case"paste":return null;case"keypress":if(!(o.ctrlKey||o.altKey||o.metaKey)||o.ctrlKey&&o.altKey){if(o.char&&1<o.char.length)return o.char;if(o.which)return String.fromCharCode(o.which)}return null;case"compositionend":return t1&&o.locale!=="ko"?null:o.data;default:return null}}var Uk={color:!0,date:!0,datetime:!0,"datetime-local":!0,email:!0,month:!0,number:!0,password:!0,range:!0,search:!0,tel:!0,text:!0,time:!0,url:!0,week:!0};function o1(a){var o=a&&a.nodeName&&a.nodeName.toLowerCase();return o==="input"?!!Uk[a.type]:o==="textarea"}function u1(a,o,l,f){Li?Ii?Ii.push(f):Ii=[f]:Li=f,o=Zc(o,"onChange"),0<o.length&&(l=new ic("onChange","change",null,l,f),a.push({event:l,listeners:o}))}var Tu=null,Mu=null;function Hk(a){Iw(a,0)}function uc(a){var o=_u(a);if(U_(o))return a}function l1(a,o){if(a==="change")return o}var s1=!1;if(wr){var Ih;if(wr){var Uh="oninput"in document;if(!Uh){var c1=document.createElement("div");c1.setAttribute("oninput","return;"),Uh=typeof c1.oninput=="function"}Ih=Uh}else Ih=!1;s1=Ih&&(!document.documentMode||9<document.documentMode)}function f1(){Tu&&(Tu.detachEvent("onpropertychange",d1),Mu=Tu=null)}function d1(a){if(a.propertyName==="value"&&uc(Mu)){var o=[];u1(o,Mu,a,Rh(a)),K_(Hk,o)}}function Gk(a,o,l){a==="focusin"?(f1(),Tu=o,Mu=l,Tu.attachEvent("onpropertychange",d1)):a==="focusout"&&f1()}function Vk(a){if(a==="selectionchange"||a==="keyup"||a==="keydown")return uc(Mu)}function Zk(a,o){if(a==="click")return uc(o)}function Yk(a,o){if(a==="input"||a==="change")return uc(o)}function Xk(a,o){return a===o&&(a!==0||1/a===1/o)||a!==a&&o!==o}var pn=typeof Object.is=="function"?Object.is:Xk;function ju(a,o){if(pn(a,o))return!0;if(typeof a!="object"||a===null||typeof o!="object"||o===null)return!1;var l=Object.keys(a),f=Object.keys(o);if(l.length!==f.length)return!1;for(f=0;f<l.length;f++){var p=l[f];if(!le.call(o,p)||!pn(a[p],o[p]))return!1}return!0}function h1(a){for(;a&&a.firstChild;)a=a.firstChild;return a}function p1(a,o){var l=h1(a);a=0;for(var f;l;){if(l.nodeType===3){if(f=a+l.textContent.length,a<=o&&f>=o)return{node:l,offset:o-a};a=f}e:{for(;l;){if(l.nextSibling){l=l.nextSibling;break e}l=l.parentNode}l=void 0}l=h1(l)}}function v1(a,o){return a&&o?a===o?!0:a&&a.nodeType===3?!1:o&&o.nodeType===3?v1(a,o.parentNode):"contains"in a?a.contains(o):a.compareDocumentPosition?!!(a.compareDocumentPosition(o)&16):!1:!1}function y1(a){a=a!=null&&a.ownerDocument!=null&&a.ownerDocument.defaultView!=null?a.ownerDocument.defaultView:window;for(var o=ec(a.document);o instanceof a.HTMLIFrameElement;){try{var l=typeof o.contentWindow.location.href=="string"}catch{l=!1}if(l)a=o.contentWindow;else break;o=ec(a.document)}return o}function Hh(a){var o=a&&a.nodeName&&a.nodeName.toLowerCase();return o&&(o==="input"&&(a.type==="text"||a.type==="search"||a.type==="tel"||a.type==="url"||a.type==="password")||o==="textarea"||a.contentEditable==="true")}var Kk=wr&&"documentMode"in document&&11>=document.documentMode,Hi=null,Gh=null,Cu=null,Vh=!1;function m1(a,o,l){var f=l.window===l?l.document:l.nodeType===9?l:l.ownerDocument;Vh||Hi==null||Hi!==ec(f)||(f=Hi,"selectionStart"in f&&Hh(f)?f={start:f.selectionStart,end:f.selectionEnd}:(f=(f.ownerDocument&&f.ownerDocument.defaultView||window).getSelection(),f={anchorNode:f.anchorNode,anchorOffset:f.anchorOffset,focusNode:f.focusNode,focusOffset:f.focusOffset}),Cu&&ju(Cu,f)||(Cu=f,f=Zc(Gh,"onSelect"),0<f.length&&(o=new ic("onSelect","select",null,o,l),a.push({event:o,listeners:f}),o.target=Hi)))}function Va(a,o){var l={};return l[a.toLowerCase()]=o.toLowerCase(),l["Webkit"+a]="webkit"+o,l["Moz"+a]="moz"+o,l}var Gi={animationend:Va("Animation","AnimationEnd"),animationiteration:Va("Animation","AnimationIteration"),animationstart:Va("Animation","AnimationStart"),transitionrun:Va("Transition","TransitionRun"),transitionstart:Va("Transition","TransitionStart"),transitioncancel:Va("Transition","TransitionCancel"),transitionend:Va("Transition","TransitionEnd")},Zh={},g1={};wr&&(g1=document.createElement("div").style,"AnimationEvent"in window||(delete Gi.animationend.animation,delete Gi.animationiteration.animation,delete Gi.animationstart.animation),"TransitionEvent"in window||delete Gi.transitionend.transition);function Za(a){if(Zh[a])return Zh[a];if(!Gi[a])return a;var o=Gi[a],l;for(l in o)if(o.hasOwnProperty(l)&&l in g1)return Zh[a]=o[l];return a}var b1=Za("animationend"),x1=Za("animationiteration"),_1=Za("animationstart"),Wk=Za("transitionrun"),Fk=Za("transitionstart"),Qk=Za("transitioncancel"),S1=Za("transitionend"),w1=new Map,Yh="abort auxClick beforeToggle cancel canPlay canPlayThrough click close contextMenu copy cut drag dragEnd dragEnter dragExit dragLeave dragOver dragStart drop durationChange emptied encrypted ended error gotPointerCapture input invalid keyDown keyPress keyUp load loadedData loadedMetadata loadStart lostPointerCapture mouseDown mouseMove mouseOut mouseOver mouseUp paste pause play playing pointerCancel pointerDown pointerMove pointerOut pointerOver pointerUp progress rateChange reset resize seeked seeking stalled submit suspend timeUpdate touchCancel touchEnd touchStart volumeChange scroll toggle touchMove waiting wheel".split(" ");Yh.push("scrollEnd");function Hn(a,o){w1.set(a,o),Ha(o,[a])}var O1=new WeakMap;function jn(a,o){if(typeof a=="object"&&a!==null){var l=O1.get(a);return l!==void 0?l:(o={value:a,source:o,stack:L_(o)},O1.set(a,o),o)}return{value:a,source:o,stack:L_(o)}}var Cn=[],Vi=0,Xh=0;function lc(){for(var a=Vi,o=Xh=Vi=0;o<a;){var l=Cn[o];Cn[o++]=null;var f=Cn[o];Cn[o++]=null;var p=Cn[o];Cn[o++]=null;var m=Cn[o];if(Cn[o++]=null,f!==null&&p!==null){var O=f.pending;O===null?p.next=p:(p.next=O.next,O.next=p),f.pending=p}m!==0&&A1(l,p,m)}}function sc(a,o,l,f){Cn[Vi++]=a,Cn[Vi++]=o,Cn[Vi++]=l,Cn[Vi++]=f,Xh|=f,a.lanes|=f,a=a.alternate,a!==null&&(a.lanes|=f)}function Kh(a,o,l,f){return sc(a,o,l,f),cc(a)}function Zi(a,o){return sc(a,null,null,o),cc(a)}function A1(a,o,l){a.lanes|=l;var f=a.alternate;f!==null&&(f.lanes|=l);for(var p=!1,m=a.return;m!==null;)m.childLanes|=l,f=m.alternate,f!==null&&(f.childLanes|=l),m.tag===22&&(a=m.stateNode,a===null||a._visibility&1||(p=!0)),a=m,m=m.return;return a.tag===3?(m=a.stateNode,p&&o!==null&&(p=31-hn(l),a=m.hiddenUpdates,f=a[p],f===null?a[p]=[o]:f.push(o),o.lane=l|536870912),m):null}function cc(a){if(50<tl)throw tl=0,tv=null,Error(r(185));for(var o=a.return;o!==null;)a=o,o=a.return;return a.tag===3?a.stateNode:null}var Yi={};function Jk(a,o,l,f){this.tag=a,this.key=l,this.sibling=this.child=this.return=this.stateNode=this.type=this.elementType=null,this.index=0,this.refCleanup=this.ref=null,this.pendingProps=o,this.dependencies=this.memoizedState=this.updateQueue=this.memoizedProps=null,this.mode=f,this.subtreeFlags=this.flags=0,this.deletions=null,this.childLanes=this.lanes=0,this.alternate=null}function vn(a,o,l,f){return new Jk(a,o,l,f)}function Wh(a){return a=a.prototype,!(!a||!a.isReactComponent)}function Or(a,o){var l=a.alternate;return l===null?(l=vn(a.tag,o,a.key,a.mode),l.elementType=a.elementType,l.type=a.type,l.stateNode=a.stateNode,l.alternate=a,a.alternate=l):(l.pendingProps=o,l.type=a.type,l.flags=0,l.subtreeFlags=0,l.deletions=null),l.flags=a.flags&65011712,l.childLanes=a.childLanes,l.lanes=a.lanes,l.child=a.child,l.memoizedProps=a.memoizedProps,l.memoizedState=a.memoizedState,l.updateQueue=a.updateQueue,o=a.dependencies,l.dependencies=o===null?null:{lanes:o.lanes,firstContext:o.firstContext},l.sibling=a.sibling,l.index=a.index,l.ref=a.ref,l.refCleanup=a.refCleanup,l}function E1(a,o){a.flags&=65011714;var l=a.alternate;return l===null?(a.childLanes=0,a.lanes=o,a.child=null,a.subtreeFlags=0,a.memoizedProps=null,a.memoizedState=null,a.updateQueue=null,a.dependencies=null,a.stateNode=null):(a.childLanes=l.childLanes,a.lanes=l.lanes,a.child=l.child,a.subtreeFlags=0,a.deletions=null,a.memoizedProps=l.memoizedProps,a.memoizedState=l.memoizedState,a.updateQueue=l.updateQueue,a.type=l.type,o=l.dependencies,a.dependencies=o===null?null:{lanes:o.lanes,firstContext:o.firstContext}),a}function fc(a,o,l,f,p,m){var O=0;if(f=a,typeof a=="function")Wh(a)&&(O=1);else if(typeof a=="string")O=t$(a,l,ce.current)?26:a==="html"||a==="head"||a==="body"?27:5;else e:switch(a){case $:return a=vn(31,l,o,p),a.elementType=$,a.lanes=m,a;case S:return Ya(l.children,p,m,o);case _:O=8,p|=24;break;case x:return a=vn(12,l,o,p|2),a.elementType=x,a.lanes=m,a;case w:return a=vn(13,l,o,p),a.elementType=w,a.lanes=m,a;case T:return a=vn(19,l,o,p),a.elementType=T,a.lanes=m,a;default:if(typeof a=="object"&&a!==null)switch(a.$$typeof){case E:case A:O=10;break e;case M:O=9;break e;case C:O=11;break e;case P:O=14;break e;case R:O=16,f=null;break e}O=29,l=Error(r(130,a===null?"null":typeof a,"")),f=null}return o=vn(O,l,o,p),o.elementType=a,o.type=f,o.lanes=m,o}function Ya(a,o,l,f){return a=vn(7,a,f,o),a.lanes=l,a}function Fh(a,o,l){return a=vn(6,a,null,o),a.lanes=l,a}function Qh(a,o,l){return o=vn(4,a.children!==null?a.children:[],a.key,o),o.lanes=l,o.stateNode={containerInfo:a.containerInfo,pendingChildren:null,implementation:a.implementation},o}var Xi=[],Ki=0,dc=null,hc=0,Pn=[],Rn=0,Xa=null,Ar=1,Er="";function Ka(a,o){Xi[Ki++]=hc,Xi[Ki++]=dc,dc=a,hc=o}function T1(a,o,l){Pn[Rn++]=Ar,Pn[Rn++]=Er,Pn[Rn++]=Xa,Xa=a;var f=Ar;a=Er;var p=32-hn(f)-1;f&=~(1<<p),l+=1;var m=32-hn(o)+p;if(30<m){var O=p-p%5;m=(f&(1<<O)-1).toString(32),f>>=O,p-=O,Ar=1<<32-hn(o)+p|l<<p|f,Er=m+a}else Ar=1<<m|l<<p|f,Er=a}function Jh(a){a.return!==null&&(Ka(a,1),T1(a,1,0))}function ep(a){for(;a===dc;)dc=Xi[--Ki],Xi[Ki]=null,hc=Xi[--Ki],Xi[Ki]=null;for(;a===Xa;)Xa=Pn[--Rn],Pn[Rn]=null,Er=Pn[--Rn],Pn[Rn]=null,Ar=Pn[--Rn],Pn[Rn]=null}var Jt=null,bt=null,Fe=!1,Wa=null,ar=!1,tp=Error(r(519));function Fa(a){var o=Error(r(418,""));throw Nu(jn(o,a)),tp}function M1(a){var o=a.stateNode,l=a.type,f=a.memoizedProps;switch(o[Yt]=a,o[an]=f,l){case"dialog":He("cancel",o),He("close",o);break;case"iframe":case"object":case"embed":He("load",o);break;case"video":case"audio":for(l=0;l<rl.length;l++)He(rl[l],o);break;case"source":He("error",o);break;case"img":case"image":case"link":He("error",o),He("load",o);break;case"details":He("toggle",o);break;case"input":He("invalid",o),H_(o,f.value,f.defaultValue,f.checked,f.defaultChecked,f.type,f.name,!0),Js(o);break;case"select":He("invalid",o);break;case"textarea":He("invalid",o),V_(o,f.value,f.defaultValue,f.children),Js(o)}l=f.children,typeof l!="string"&&typeof l!="number"&&typeof l!="bigint"||o.textContent===""+l||f.suppressHydrationWarning===!0||Vw(o.textContent,l)?(f.popover!=null&&(He("beforetoggle",o),He("toggle",o)),f.onScroll!=null&&He("scroll",o),f.onScrollEnd!=null&&He("scrollend",o),f.onClick!=null&&(o.onclick=Yc),o=!0):o=!1,o||Fa(a)}function j1(a){for(Jt=a.return;Jt;)switch(Jt.tag){case 5:case 13:ar=!1;return;case 27:case 3:ar=!0;return;default:Jt=Jt.return}}function Pu(a){if(a!==Jt)return!1;if(!Fe)return j1(a),Fe=!0,!1;var o=a.tag,l;if((l=o!==3&&o!==27)&&((l=o===5)&&(l=a.type,l=!(l!=="form"&&l!=="button")||mv(a.type,a.memoizedProps)),l=!l),l&&bt&&Fa(a),j1(a),o===13){if(a=a.memoizedState,a=a!==null?a.dehydrated:null,!a)throw Error(r(317));e:{for(a=a.nextSibling,o=0;a;){if(a.nodeType===8)if(l=a.data,l==="/$"){if(o===0){bt=Vn(a.nextSibling);break e}o--}else l!=="$"&&l!=="$!"&&l!=="$?"||o++;a=a.nextSibling}bt=null}}else o===27?(o=bt,ga(a.type)?(a=_v,_v=null,bt=a):bt=o):bt=Jt?Vn(a.stateNode.nextSibling):null;return!0}function Ru(){bt=Jt=null,Fe=!1}function C1(){var a=Wa;return a!==null&&(sn===null?sn=a:sn.push.apply(sn,a),Wa=null),a}function Nu(a){Wa===null?Wa=[a]:Wa.push(a)}var np=X(null),Qa=null,Tr=null;function ra(a,o,l){te(np,o._currentValue),o._currentValue=l}function Mr(a){a._currentValue=np.current,ee(np)}function rp(a,o,l){for(;a!==null;){var f=a.alternate;if((a.childLanes&o)!==o?(a.childLanes|=o,f!==null&&(f.childLanes|=o)):f!==null&&(f.childLanes&o)!==o&&(f.childLanes|=o),a===l)break;a=a.return}}function ap(a,o,l,f){var p=a.child;for(p!==null&&(p.return=a);p!==null;){var m=p.dependencies;if(m!==null){var O=p.child;m=m.firstContext;e:for(;m!==null;){var j=m;m=p;for(var k=0;k<o.length;k++)if(j.context===o[k]){m.lanes|=l,j=m.alternate,j!==null&&(j.lanes|=l),rp(m.return,l,a),f||(O=null);break e}m=j.next}}else if(p.tag===18){if(O=p.return,O===null)throw Error(r(341));O.lanes|=l,m=O.alternate,m!==null&&(m.lanes|=l),rp(O,l,a),O=null}else O=p.child;if(O!==null)O.return=p;else for(O=p;O!==null;){if(O===a){O=null;break}if(p=O.sibling,p!==null){p.return=O.return,O=p;break}O=O.return}p=O}}function Du(a,o,l,f){a=null;for(var p=o,m=!1;p!==null;){if(!m){if((p.flags&524288)!==0)m=!0;else if((p.flags&262144)!==0)break}if(p.tag===10){var O=p.alternate;if(O===null)throw Error(r(387));if(O=O.memoizedProps,O!==null){var j=p.type;pn(p.pendingProps.value,O.value)||(a!==null?a.push(j):a=[j])}}else if(p===ke.current){if(O=p.alternate,O===null)throw Error(r(387));O.memoizedState.memoizedState!==p.memoizedState.memoizedState&&(a!==null?a.push(sl):a=[sl])}p=p.return}a!==null&&ap(o,a,l,f),o.flags|=262144}function pc(a){for(a=a.firstContext;a!==null;){if(!pn(a.context._currentValue,a.memoizedValue))return!0;a=a.next}return!1}function Ja(a){Qa=a,Tr=null,a=a.dependencies,a!==null&&(a.firstContext=null)}function Xt(a){return P1(Qa,a)}function vc(a,o){return Qa===null&&Ja(a),P1(a,o)}function P1(a,o){var l=o._currentValue;if(o={context:o,memoizedValue:l,next:null},Tr===null){if(a===null)throw Error(r(308));Tr=o,a.dependencies={lanes:0,firstContext:o},a.flags|=524288}else Tr=Tr.next=o;return l}var eq=typeof AbortController<"u"?AbortController:function(){var a=[],o=this.signal={aborted:!1,addEventListener:function(l,f){a.push(f)}};this.abort=function(){o.aborted=!0,a.forEach(function(l){return l()})}},tq=e.unstable_scheduleCallback,nq=e.unstable_NormalPriority,Ct={$$typeof:A,Consumer:null,Provider:null,_currentValue:null,_currentValue2:null,_threadCount:0};function ip(){return{controller:new eq,data:new Map,refCount:0}}function ku(a){a.refCount--,a.refCount===0&&tq(nq,function(){a.controller.abort()})}var qu=null,op=0,Wi=0,Fi=null;function rq(a,o){if(qu===null){var l=qu=[];op=0,Wi=lv(),Fi={status:"pending",value:void 0,then:function(f){l.push(f)}}}return op++,o.then(R1,R1),o}function R1(){if(--op===0&&qu!==null){Fi!==null&&(Fi.status="fulfilled");var a=qu;qu=null,Wi=0,Fi=null;for(var o=0;o<a.length;o++)(0,a[o])()}}function aq(a,o){var l=[],f={status:"pending",value:null,reason:null,then:function(p){l.push(p)}};return a.then(function(){f.status="fulfilled",f.value=o;for(var p=0;p<l.length;p++)(0,l[p])(o)},function(p){for(f.status="rejected",f.reason=p,p=0;p<l.length;p++)(0,l[p])(void 0)}),f}var N1=D.S;D.S=function(a,o){typeof o=="object"&&o!==null&&typeof o.then=="function"&&rq(a,o),N1!==null&&N1(a,o)};var ei=X(null);function up(){var a=ei.current;return a!==null?a:ct.pooledCache}function yc(a,o){o===null?te(ei,ei.current):te(ei,o.pool)}function D1(){var a=up();return a===null?null:{parent:Ct._currentValue,pool:a}}var $u=Error(r(460)),k1=Error(r(474)),mc=Error(r(542)),lp={then:function(){}};function q1(a){return a=a.status,a==="fulfilled"||a==="rejected"}function gc(){}function $1(a,o,l){switch(l=a[l],l===void 0?a.push(o):l!==o&&(o.then(gc,gc),o=l),o.status){case"fulfilled":return o.value;case"rejected":throw a=o.reason,z1(a),a;default:if(typeof o.status=="string")o.then(gc,gc);else{if(a=ct,a!==null&&100<a.shellSuspendCounter)throw Error(r(482));a=o,a.status="pending",a.then(function(f){if(o.status==="pending"){var p=o;p.status="fulfilled",p.value=f}},function(f){if(o.status==="pending"){var p=o;p.status="rejected",p.reason=f}})}switch(o.status){case"fulfilled":return o.value;case"rejected":throw a=o.reason,z1(a),a}throw Bu=o,$u}}var Bu=null;function B1(){if(Bu===null)throw Error(r(459));var a=Bu;return Bu=null,a}function z1(a){if(a===$u||a===mc)throw Error(r(483))}var aa=!1;function sp(a){a.updateQueue={baseState:a.memoizedState,firstBaseUpdate:null,lastBaseUpdate:null,shared:{pending:null,lanes:0,hiddenCallbacks:null},callbacks:null}}function cp(a,o){a=a.updateQueue,o.updateQueue===a&&(o.updateQueue={baseState:a.baseState,firstBaseUpdate:a.firstBaseUpdate,lastBaseUpdate:a.lastBaseUpdate,shared:a.shared,callbacks:null})}function ia(a){return{lane:a,tag:0,payload:null,callback:null,next:null}}function oa(a,o,l){var f=a.updateQueue;if(f===null)return null;if(f=f.shared,(et&2)!==0){var p=f.pending;return p===null?o.next=o:(o.next=p.next,p.next=o),f.pending=o,o=cc(a),A1(a,null,l),o}return sc(a,f,o,l),cc(a)}function zu(a,o,l){if(o=o.updateQueue,o!==null&&(o=o.shared,(l&4194048)!==0)){var f=o.lanes;f&=a.pendingLanes,l|=f,o.lanes=l,R_(a,l)}}function fp(a,o){var l=a.updateQueue,f=a.alternate;if(f!==null&&(f=f.updateQueue,l===f)){var p=null,m=null;if(l=l.firstBaseUpdate,l!==null){do{var O={lane:l.lane,tag:l.tag,payload:l.payload,callback:null,next:null};m===null?p=m=O:m=m.next=O,l=l.next}while(l!==null);m===null?p=m=o:m=m.next=o}else p=m=o;l={baseState:f.baseState,firstBaseUpdate:p,lastBaseUpdate:m,shared:f.shared,callbacks:f.callbacks},a.updateQueue=l;return}a=l.lastBaseUpdate,a===null?l.firstBaseUpdate=o:a.next=o,l.lastBaseUpdate=o}var dp=!1;function Lu(){if(dp){var a=Fi;if(a!==null)throw a}}function Iu(a,o,l,f){dp=!1;var p=a.updateQueue;aa=!1;var m=p.firstBaseUpdate,O=p.lastBaseUpdate,j=p.shared.pending;if(j!==null){p.shared.pending=null;var k=j,Y=k.next;k.next=null,O===null?m=Y:O.next=Y,O=k;var ae=a.alternate;ae!==null&&(ae=ae.updateQueue,j=ae.lastBaseUpdate,j!==O&&(j===null?ae.firstBaseUpdate=Y:j.next=Y,ae.lastBaseUpdate=k))}if(m!==null){var oe=p.baseState;O=0,ae=Y=k=null,j=m;do{var K=j.lane&-536870913,F=K!==j.lane;if(F?(Ve&K)===K:(f&K)===K){K!==0&&K===Wi&&(dp=!0),ae!==null&&(ae=ae.next={lane:0,tag:j.tag,payload:j.payload,callback:null,next:null});e:{var je=a,we=j;K=o;var at=l;switch(we.tag){case 1:if(je=we.payload,typeof je=="function"){oe=je.call(at,oe,K);break e}oe=je;break e;case 3:je.flags=je.flags&-65537|128;case 0:if(je=we.payload,K=typeof je=="function"?je.call(at,oe,K):je,K==null)break e;oe=v({},oe,K);break e;case 2:aa=!0}}K=j.callback,K!==null&&(a.flags|=64,F&&(a.flags|=8192),F=p.callbacks,F===null?p.callbacks=[K]:F.push(K))}else F={lane:K,tag:j.tag,payload:j.payload,callback:j.callback,next:null},ae===null?(Y=ae=F,k=oe):ae=ae.next=F,O|=K;if(j=j.next,j===null){if(j=p.shared.pending,j===null)break;F=j,j=F.next,F.next=null,p.lastBaseUpdate=F,p.shared.pending=null}}while(!0);ae===null&&(k=oe),p.baseState=k,p.firstBaseUpdate=Y,p.lastBaseUpdate=ae,m===null&&(p.shared.lanes=0),pa|=O,a.lanes=O,a.memoizedState=oe}}function L1(a,o){if(typeof a!="function")throw Error(r(191,a));a.call(o)}function I1(a,o){var l=a.callbacks;if(l!==null)for(a.callbacks=null,a=0;a<l.length;a++)L1(l[a],o)}var Qi=X(null),bc=X(0);function U1(a,o){a=kr,te(bc,a),te(Qi,o),kr=a|o.baseLanes}function hp(){te(bc,kr),te(Qi,Qi.current)}function pp(){kr=bc.current,ee(Qi),ee(bc)}var ua=0,$e=null,nt=null,Et=null,xc=!1,Ji=!1,ti=!1,_c=0,Uu=0,eo=null,iq=0;function Ot(){throw Error(r(321))}function vp(a,o){if(o===null)return!1;for(var l=0;l<o.length&&l<a.length;l++)if(!pn(a[l],o[l]))return!1;return!0}function yp(a,o,l,f,p,m){return ua=m,$e=o,o.memoizedState=null,o.updateQueue=null,o.lanes=0,D.H=a===null||a.memoizedState===null?AS:ES,ti=!1,m=l(f,p),ti=!1,Ji&&(m=G1(o,l,f,p)),H1(a),m}function H1(a){D.H=Tc;var o=nt!==null&&nt.next!==null;if(ua=0,Et=nt=$e=null,xc=!1,Uu=0,eo=null,o)throw Error(r(300));a===null||$t||(a=a.dependencies,a!==null&&pc(a)&&($t=!0))}function G1(a,o,l,f){$e=a;var p=0;do{if(Ji&&(eo=null),Uu=0,Ji=!1,25<=p)throw Error(r(301));if(p+=1,Et=nt=null,a.updateQueue!=null){var m=a.updateQueue;m.lastEffect=null,m.events=null,m.stores=null,m.memoCache!=null&&(m.memoCache.index=0)}D.H=dq,m=o(l,f)}while(Ji);return m}function oq(){var a=D.H,o=a.useState()[0];return o=typeof o.then=="function"?Hu(o):o,a=a.useState()[0],(nt!==null?nt.memoizedState:null)!==a&&($e.flags|=1024),o}function mp(){var a=_c!==0;return _c=0,a}function gp(a,o,l){o.updateQueue=a.updateQueue,o.flags&=-2053,a.lanes&=~l}function bp(a){if(xc){for(a=a.memoizedState;a!==null;){var o=a.queue;o!==null&&(o.pending=null),a=a.next}xc=!1}ua=0,Et=nt=$e=null,Ji=!1,Uu=_c=0,eo=null}function un(){var a={memoizedState:null,baseState:null,baseQueue:null,queue:null,next:null};return Et===null?$e.memoizedState=Et=a:Et=Et.next=a,Et}function Tt(){if(nt===null){var a=$e.alternate;a=a!==null?a.memoizedState:null}else a=nt.next;var o=Et===null?$e.memoizedState:Et.next;if(o!==null)Et=o,nt=a;else{if(a===null)throw $e.alternate===null?Error(r(467)):Error(r(310));nt=a,a={memoizedState:nt.memoizedState,baseState:nt.baseState,baseQueue:nt.baseQueue,queue:nt.queue,next:null},Et===null?$e.memoizedState=Et=a:Et=Et.next=a}return Et}function xp(){return{lastEffect:null,events:null,stores:null,memoCache:null}}function Hu(a){var o=Uu;return Uu+=1,eo===null&&(eo=[]),a=$1(eo,a,o),o=$e,(Et===null?o.memoizedState:Et.next)===null&&(o=o.alternate,D.H=o===null||o.memoizedState===null?AS:ES),a}function Sc(a){if(a!==null&&typeof a=="object"){if(typeof a.then=="function")return Hu(a);if(a.$$typeof===A)return Xt(a)}throw Error(r(438,String(a)))}function _p(a){var o=null,l=$e.updateQueue;if(l!==null&&(o=l.memoCache),o==null){var f=$e.alternate;f!==null&&(f=f.updateQueue,f!==null&&(f=f.memoCache,f!=null&&(o={data:f.data.map(function(p){return p.slice()}),index:0})))}if(o==null&&(o={data:[],index:0}),l===null&&(l=xp(),$e.updateQueue=l),l.memoCache=o,l=o.data[o.index],l===void 0)for(l=o.data[o.index]=Array(a),f=0;f<a;f++)l[f]=J;return o.index++,l}function jr(a,o){return typeof o=="function"?o(a):o}function wc(a){var o=Tt();return Sp(o,nt,a)}function Sp(a,o,l){var f=a.queue;if(f===null)throw Error(r(311));f.lastRenderedReducer=l;var p=a.baseQueue,m=f.pending;if(m!==null){if(p!==null){var O=p.next;p.next=m.next,m.next=O}o.baseQueue=p=m,f.pending=null}if(m=a.baseState,p===null)a.memoizedState=m;else{o=p.next;var j=O=null,k=null,Y=o,ae=!1;do{var oe=Y.lane&-536870913;if(oe!==Y.lane?(Ve&oe)===oe:(ua&oe)===oe){var K=Y.revertLane;if(K===0)k!==null&&(k=k.next={lane:0,revertLane:0,action:Y.action,hasEagerState:Y.hasEagerState,eagerState:Y.eagerState,next:null}),oe===Wi&&(ae=!0);else if((ua&K)===K){Y=Y.next,K===Wi&&(ae=!0);continue}else oe={lane:0,revertLane:Y.revertLane,action:Y.action,hasEagerState:Y.hasEagerState,eagerState:Y.eagerState,next:null},k===null?(j=k=oe,O=m):k=k.next=oe,$e.lanes|=K,pa|=K;oe=Y.action,ti&&l(m,oe),m=Y.hasEagerState?Y.eagerState:l(m,oe)}else K={lane:oe,revertLane:Y.revertLane,action:Y.action,hasEagerState:Y.hasEagerState,eagerState:Y.eagerState,next:null},k===null?(j=k=K,O=m):k=k.next=K,$e.lanes|=oe,pa|=oe;Y=Y.next}while(Y!==null&&Y!==o);if(k===null?O=m:k.next=j,!pn(m,a.memoizedState)&&($t=!0,ae&&(l=Fi,l!==null)))throw l;a.memoizedState=m,a.baseState=O,a.baseQueue=k,f.lastRenderedState=m}return p===null&&(f.lanes=0),[a.memoizedState,f.dispatch]}function wp(a){var o=Tt(),l=o.queue;if(l===null)throw Error(r(311));l.lastRenderedReducer=a;var f=l.dispatch,p=l.pending,m=o.memoizedState;if(p!==null){l.pending=null;var O=p=p.next;do m=a(m,O.action),O=O.next;while(O!==p);pn(m,o.memoizedState)||($t=!0),o.memoizedState=m,o.baseQueue===null&&(o.baseState=m),l.lastRenderedState=m}return[m,f]}function V1(a,o,l){var f=$e,p=Tt(),m=Fe;if(m){if(l===void 0)throw Error(r(407));l=l()}else l=o();var O=!pn((nt||p).memoizedState,l);O&&(p.memoizedState=l,$t=!0),p=p.queue;var j=X1.bind(null,f,p,a);if(Gu(2048,8,j,[a]),p.getSnapshot!==o||O||Et!==null&&Et.memoizedState.tag&1){if(f.flags|=2048,to(9,Oc(),Y1.bind(null,f,p,l,o),null),ct===null)throw Error(r(349));m||(ua&124)!==0||Z1(f,o,l)}return l}function Z1(a,o,l){a.flags|=16384,a={getSnapshot:o,value:l},o=$e.updateQueue,o===null?(o=xp(),$e.updateQueue=o,o.stores=[a]):(l=o.stores,l===null?o.stores=[a]:l.push(a))}function Y1(a,o,l,f){o.value=l,o.getSnapshot=f,K1(o)&&W1(a)}function X1(a,o,l){return l(function(){K1(o)&&W1(a)})}function K1(a){var o=a.getSnapshot;a=a.value;try{var l=o();return!pn(a,l)}catch{return!0}}function W1(a){var o=Zi(a,2);o!==null&&xn(o,a,2)}function Op(a){var o=un();if(typeof a=="function"){var l=a;if(a=l(),ti){ea(!0);try{l()}finally{ea(!1)}}}return o.memoizedState=o.baseState=a,o.queue={pending:null,lanes:0,dispatch:null,lastRenderedReducer:jr,lastRenderedState:a},o}function F1(a,o,l,f){return a.baseState=l,Sp(a,nt,typeof f=="function"?f:jr)}function uq(a,o,l,f,p){if(Ec(a))throw Error(r(485));if(a=o.action,a!==null){var m={payload:p,action:a,next:null,isTransition:!0,status:"pending",value:null,reason:null,listeners:[],then:function(O){m.listeners.push(O)}};D.T!==null?l(!0):m.isTransition=!1,f(m),l=o.pending,l===null?(m.next=o.pending=m,Q1(o,m)):(m.next=l.next,o.pending=l.next=m)}}function Q1(a,o){var l=o.action,f=o.payload,p=a.state;if(o.isTransition){var m=D.T,O={};D.T=O;try{var j=l(p,f),k=D.S;k!==null&&k(O,j),J1(a,o,j)}catch(Y){Ap(a,o,Y)}finally{D.T=m}}else try{m=l(p,f),J1(a,o,m)}catch(Y){Ap(a,o,Y)}}function J1(a,o,l){l!==null&&typeof l=="object"&&typeof l.then=="function"?l.then(function(f){eS(a,o,f)},function(f){return Ap(a,o,f)}):eS(a,o,l)}function eS(a,o,l){o.status="fulfilled",o.value=l,tS(o),a.state=l,o=a.pending,o!==null&&(l=o.next,l===o?a.pending=null:(l=l.next,o.next=l,Q1(a,l)))}function Ap(a,o,l){var f=a.pending;if(a.pending=null,f!==null){f=f.next;do o.status="rejected",o.reason=l,tS(o),o=o.next;while(o!==f)}a.action=null}function tS(a){a=a.listeners;for(var o=0;o<a.length;o++)(0,a[o])()}function nS(a,o){return o}function rS(a,o){if(Fe){var l=ct.formState;if(l!==null){e:{var f=$e;if(Fe){if(bt){t:{for(var p=bt,m=ar;p.nodeType!==8;){if(!m){p=null;break t}if(p=Vn(p.nextSibling),p===null){p=null;break t}}m=p.data,p=m==="F!"||m==="F"?p:null}if(p){bt=Vn(p.nextSibling),f=p.data==="F!";break e}}Fa(f)}f=!1}f&&(o=l[0])}}return l=un(),l.memoizedState=l.baseState=o,f={pending:null,lanes:0,dispatch:null,lastRenderedReducer:nS,lastRenderedState:o},l.queue=f,l=SS.bind(null,$e,f),f.dispatch=l,f=Op(!1),m=Cp.bind(null,$e,!1,f.queue),f=un(),p={state:o,dispatch:null,action:a,pending:null},f.queue=p,l=uq.bind(null,$e,p,m,l),p.dispatch=l,f.memoizedState=a,[o,l,!1]}function aS(a){var o=Tt();return iS(o,nt,a)}function iS(a,o,l){if(o=Sp(a,o,nS)[0],a=wc(jr)[0],typeof o=="object"&&o!==null&&typeof o.then=="function")try{var f=Hu(o)}catch(O){throw O===$u?mc:O}else f=o;o=Tt();var p=o.queue,m=p.dispatch;return l!==o.memoizedState&&($e.flags|=2048,to(9,Oc(),lq.bind(null,p,l),null)),[f,m,a]}function lq(a,o){a.action=o}function oS(a){var o=Tt(),l=nt;if(l!==null)return iS(o,l,a);Tt(),o=o.memoizedState,l=Tt();var f=l.queue.dispatch;return l.memoizedState=a,[o,f,!1]}function to(a,o,l,f){return a={tag:a,create:l,deps:f,inst:o,next:null},o=$e.updateQueue,o===null&&(o=xp(),$e.updateQueue=o),l=o.lastEffect,l===null?o.lastEffect=a.next=a:(f=l.next,l.next=a,a.next=f,o.lastEffect=a),a}function Oc(){return{destroy:void 0,resource:void 0}}function uS(){return Tt().memoizedState}function Ac(a,o,l,f){var p=un();f=f===void 0?null:f,$e.flags|=a,p.memoizedState=to(1|o,Oc(),l,f)}function Gu(a,o,l,f){var p=Tt();f=f===void 0?null:f;var m=p.memoizedState.inst;nt!==null&&f!==null&&vp(f,nt.memoizedState.deps)?p.memoizedState=to(o,m,l,f):($e.flags|=a,p.memoizedState=to(1|o,m,l,f))}function lS(a,o){Ac(8390656,8,a,o)}function sS(a,o){Gu(2048,8,a,o)}function cS(a,o){return Gu(4,2,a,o)}function fS(a,o){return Gu(4,4,a,o)}function dS(a,o){if(typeof o=="function"){a=a();var l=o(a);return function(){typeof l=="function"?l():o(null)}}if(o!=null)return a=a(),o.current=a,function(){o.current=null}}function hS(a,o,l){l=l!=null?l.concat([a]):null,Gu(4,4,dS.bind(null,o,a),l)}function Ep(){}function pS(a,o){var l=Tt();o=o===void 0?null:o;var f=l.memoizedState;return o!==null&&vp(o,f[1])?f[0]:(l.memoizedState=[a,o],a)}function vS(a,o){var l=Tt();o=o===void 0?null:o;var f=l.memoizedState;if(o!==null&&vp(o,f[1]))return f[0];if(f=a(),ti){ea(!0);try{a()}finally{ea(!1)}}return l.memoizedState=[f,o],f}function Tp(a,o,l){return l===void 0||(ua&1073741824)!==0?a.memoizedState=o:(a.memoizedState=l,a=gw(),$e.lanes|=a,pa|=a,l)}function yS(a,o,l,f){return pn(l,o)?l:Qi.current!==null?(a=Tp(a,l,f),pn(a,o)||($t=!0),a):(ua&42)===0?($t=!0,a.memoizedState=l):(a=gw(),$e.lanes|=a,pa|=a,o)}function mS(a,o,l,f,p){var m=H.p;H.p=m!==0&&8>m?m:8;var O=D.T,j={};D.T=j,Cp(a,!1,o,l);try{var k=p(),Y=D.S;if(Y!==null&&Y(j,k),k!==null&&typeof k=="object"&&typeof k.then=="function"){var ae=aq(k,f);Vu(a,o,ae,bn(a))}else Vu(a,o,f,bn(a))}catch(oe){Vu(a,o,{then:function(){},status:"rejected",reason:oe},bn())}finally{H.p=m,D.T=O}}function sq(){}function Mp(a,o,l,f){if(a.tag!==5)throw Error(r(476));var p=gS(a).queue;mS(a,p,o,U,l===null?sq:function(){return bS(a),l(f)})}function gS(a){var o=a.memoizedState;if(o!==null)return o;o={memoizedState:U,baseState:U,baseQueue:null,queue:{pending:null,lanes:0,dispatch:null,lastRenderedReducer:jr,lastRenderedState:U},next:null};var l={};return o.next={memoizedState:l,baseState:l,baseQueue:null,queue:{pending:null,lanes:0,dispatch:null,lastRenderedReducer:jr,lastRenderedState:l},next:null},a.memoizedState=o,a=a.alternate,a!==null&&(a.memoizedState=o),o}function bS(a){var o=gS(a).next.queue;Vu(a,o,{},bn())}function jp(){return Xt(sl)}function xS(){return Tt().memoizedState}function _S(){return Tt().memoizedState}function cq(a){for(var o=a.return;o!==null;){switch(o.tag){case 24:case 3:var l=bn();a=ia(l);var f=oa(o,a,l);f!==null&&(xn(f,o,l),zu(f,o,l)),o={cache:ip()},a.payload=o;return}o=o.return}}function fq(a,o,l){var f=bn();l={lane:f,revertLane:0,action:l,hasEagerState:!1,eagerState:null,next:null},Ec(a)?wS(o,l):(l=Kh(a,o,l,f),l!==null&&(xn(l,a,f),OS(l,o,f)))}function SS(a,o,l){var f=bn();Vu(a,o,l,f)}function Vu(a,o,l,f){var p={lane:f,revertLane:0,action:l,hasEagerState:!1,eagerState:null,next:null};if(Ec(a))wS(o,p);else{var m=a.alternate;if(a.lanes===0&&(m===null||m.lanes===0)&&(m=o.lastRenderedReducer,m!==null))try{var O=o.lastRenderedState,j=m(O,l);if(p.hasEagerState=!0,p.eagerState=j,pn(j,O))return sc(a,o,p,0),ct===null&&lc(),!1}catch{}finally{}if(l=Kh(a,o,p,f),l!==null)return xn(l,a,f),OS(l,o,f),!0}return!1}function Cp(a,o,l,f){if(f={lane:2,revertLane:lv(),action:f,hasEagerState:!1,eagerState:null,next:null},Ec(a)){if(o)throw Error(r(479))}else o=Kh(a,l,f,2),o!==null&&xn(o,a,2)}function Ec(a){var o=a.alternate;return a===$e||o!==null&&o===$e}function wS(a,o){Ji=xc=!0;var l=a.pending;l===null?o.next=o:(o.next=l.next,l.next=o),a.pending=o}function OS(a,o,l){if((l&4194048)!==0){var f=o.lanes;f&=a.pendingLanes,l|=f,o.lanes=l,R_(a,l)}}var Tc={readContext:Xt,use:Sc,useCallback:Ot,useContext:Ot,useEffect:Ot,useImperativeHandle:Ot,useLayoutEffect:Ot,useInsertionEffect:Ot,useMemo:Ot,useReducer:Ot,useRef:Ot,useState:Ot,useDebugValue:Ot,useDeferredValue:Ot,useTransition:Ot,useSyncExternalStore:Ot,useId:Ot,useHostTransitionStatus:Ot,useFormState:Ot,useActionState:Ot,useOptimistic:Ot,useMemoCache:Ot,useCacheRefresh:Ot},AS={readContext:Xt,use:Sc,useCallback:function(a,o){return un().memoizedState=[a,o===void 0?null:o],a},useContext:Xt,useEffect:lS,useImperativeHandle:function(a,o,l){l=l!=null?l.concat([a]):null,Ac(4194308,4,dS.bind(null,o,a),l)},useLayoutEffect:function(a,o){return Ac(4194308,4,a,o)},useInsertionEffect:function(a,o){Ac(4,2,a,o)},useMemo:function(a,o){var l=un();o=o===void 0?null:o;var f=a();if(ti){ea(!0);try{a()}finally{ea(!1)}}return l.memoizedState=[f,o],f},useReducer:function(a,o,l){var f=un();if(l!==void 0){var p=l(o);if(ti){ea(!0);try{l(o)}finally{ea(!1)}}}else p=o;return f.memoizedState=f.baseState=p,a={pending:null,lanes:0,dispatch:null,lastRenderedReducer:a,lastRenderedState:p},f.queue=a,a=a.dispatch=fq.bind(null,$e,a),[f.memoizedState,a]},useRef:function(a){var o=un();return a={current:a},o.memoizedState=a},useState:function(a){a=Op(a);var o=a.queue,l=SS.bind(null,$e,o);return o.dispatch=l,[a.memoizedState,l]},useDebugValue:Ep,useDeferredValue:function(a,o){var l=un();return Tp(l,a,o)},useTransition:function(){var a=Op(!1);return a=mS.bind(null,$e,a.queue,!0,!1),un().memoizedState=a,[!1,a]},useSyncExternalStore:function(a,o,l){var f=$e,p=un();if(Fe){if(l===void 0)throw Error(r(407));l=l()}else{if(l=o(),ct===null)throw Error(r(349));(Ve&124)!==0||Z1(f,o,l)}p.memoizedState=l;var m={value:l,getSnapshot:o};return p.queue=m,lS(X1.bind(null,f,m,a),[a]),f.flags|=2048,to(9,Oc(),Y1.bind(null,f,m,l,o),null),l},useId:function(){var a=un(),o=ct.identifierPrefix;if(Fe){var l=Er,f=Ar;l=(f&~(1<<32-hn(f)-1)).toString(32)+l,o="«"+o+"R"+l,l=_c++,0<l&&(o+="H"+l.toString(32)),o+="»"}else l=iq++,o="«"+o+"r"+l.toString(32)+"»";return a.memoizedState=o},useHostTransitionStatus:jp,useFormState:rS,useActionState:rS,useOptimistic:function(a){var o=un();o.memoizedState=o.baseState=a;var l={pending:null,lanes:0,dispatch:null,lastRenderedReducer:null,lastRenderedState:null};return o.queue=l,o=Cp.bind(null,$e,!0,l),l.dispatch=o,[a,o]},useMemoCache:_p,useCacheRefresh:function(){return un().memoizedState=cq.bind(null,$e)}},ES={readContext:Xt,use:Sc,useCallback:pS,useContext:Xt,useEffect:sS,useImperativeHandle:hS,useInsertionEffect:cS,useLayoutEffect:fS,useMemo:vS,useReducer:wc,useRef:uS,useState:function(){return wc(jr)},useDebugValue:Ep,useDeferredValue:function(a,o){var l=Tt();return yS(l,nt.memoizedState,a,o)},useTransition:function(){var a=wc(jr)[0],o=Tt().memoizedState;return[typeof a=="boolean"?a:Hu(a),o]},useSyncExternalStore:V1,useId:xS,useHostTransitionStatus:jp,useFormState:aS,useActionState:aS,useOptimistic:function(a,o){var l=Tt();return F1(l,nt,a,o)},useMemoCache:_p,useCacheRefresh:_S},dq={readContext:Xt,use:Sc,useCallback:pS,useContext:Xt,useEffect:sS,useImperativeHandle:hS,useInsertionEffect:cS,useLayoutEffect:fS,useMemo:vS,useReducer:wp,useRef:uS,useState:function(){return wp(jr)},useDebugValue:Ep,useDeferredValue:function(a,o){var l=Tt();return nt===null?Tp(l,a,o):yS(l,nt.memoizedState,a,o)},useTransition:function(){var a=wp(jr)[0],o=Tt().memoizedState;return[typeof a=="boolean"?a:Hu(a),o]},useSyncExternalStore:V1,useId:xS,useHostTransitionStatus:jp,useFormState:oS,useActionState:oS,useOptimistic:function(a,o){var l=Tt();return nt!==null?F1(l,nt,a,o):(l.baseState=a,[a,l.queue.dispatch])},useMemoCache:_p,useCacheRefresh:_S},no=null,Zu=0;function Mc(a){var o=Zu;return Zu+=1,no===null&&(no=[]),$1(no,a,o)}function Yu(a,o){o=o.props.ref,a.ref=o!==void 0?o:null}function jc(a,o){throw o.$$typeof===y?Error(r(525)):(a=Object.prototype.toString.call(o),Error(r(31,a==="[object Object]"?"object with keys {"+Object.keys(o).join(", ")+"}":a)))}function TS(a){var o=a._init;return o(a._payload)}function MS(a){function o(I,z){if(a){var Z=I.deletions;Z===null?(I.deletions=[z],I.flags|=16):Z.push(z)}}function l(I,z){if(!a)return null;for(;z!==null;)o(I,z),z=z.sibling;return null}function f(I){for(var z=new Map;I!==null;)I.key!==null?z.set(I.key,I):z.set(I.index,I),I=I.sibling;return z}function p(I,z){return I=Or(I,z),I.index=0,I.sibling=null,I}function m(I,z,Z){return I.index=Z,a?(Z=I.alternate,Z!==null?(Z=Z.index,Z<z?(I.flags|=67108866,z):Z):(I.flags|=67108866,z)):(I.flags|=1048576,z)}function O(I){return a&&I.alternate===null&&(I.flags|=67108866),I}function j(I,z,Z,ie){return z===null||z.tag!==6?(z=Fh(Z,I.mode,ie),z.return=I,z):(z=p(z,Z),z.return=I,z)}function k(I,z,Z,ie){var ye=Z.type;return ye===S?ae(I,z,Z.props.children,ie,Z.key):z!==null&&(z.elementType===ye||typeof ye=="object"&&ye!==null&&ye.$$typeof===R&&TS(ye)===z.type)?(z=p(z,Z.props),Yu(z,Z),z.return=I,z):(z=fc(Z.type,Z.key,Z.props,null,I.mode,ie),Yu(z,Z),z.return=I,z)}function Y(I,z,Z,ie){return z===null||z.tag!==4||z.stateNode.containerInfo!==Z.containerInfo||z.stateNode.implementation!==Z.implementation?(z=Qh(Z,I.mode,ie),z.return=I,z):(z=p(z,Z.children||[]),z.return=I,z)}function ae(I,z,Z,ie,ye){return z===null||z.tag!==7?(z=Ya(Z,I.mode,ie,ye),z.return=I,z):(z=p(z,Z),z.return=I,z)}function oe(I,z,Z){if(typeof z=="string"&&z!==""||typeof z=="number"||typeof z=="bigint")return z=Fh(""+z,I.mode,Z),z.return=I,z;if(typeof z=="object"&&z!==null){switch(z.$$typeof){case g:return Z=fc(z.type,z.key,z.props,null,I.mode,Z),Yu(Z,z),Z.return=I,Z;case b:return z=Qh(z,I.mode,Z),z.return=I,z;case R:var ie=z._init;return z=ie(z._payload),oe(I,z,Z)}if(Q(z)||G(z))return z=Ya(z,I.mode,Z,null),z.return=I,z;if(typeof z.then=="function")return oe(I,Mc(z),Z);if(z.$$typeof===A)return oe(I,vc(I,z),Z);jc(I,z)}return null}function K(I,z,Z,ie){var ye=z!==null?z.key:null;if(typeof Z=="string"&&Z!==""||typeof Z=="number"||typeof Z=="bigint")return ye!==null?null:j(I,z,""+Z,ie);if(typeof Z=="object"&&Z!==null){switch(Z.$$typeof){case g:return Z.key===ye?k(I,z,Z,ie):null;case b:return Z.key===ye?Y(I,z,Z,ie):null;case R:return ye=Z._init,Z=ye(Z._payload),K(I,z,Z,ie)}if(Q(Z)||G(Z))return ye!==null?null:ae(I,z,Z,ie,null);if(typeof Z.then=="function")return K(I,z,Mc(Z),ie);if(Z.$$typeof===A)return K(I,z,vc(I,Z),ie);jc(I,Z)}return null}function F(I,z,Z,ie,ye){if(typeof ie=="string"&&ie!==""||typeof ie=="number"||typeof ie=="bigint")return I=I.get(Z)||null,j(z,I,""+ie,ye);if(typeof ie=="object"&&ie!==null){switch(ie.$$typeof){case g:return I=I.get(ie.key===null?Z:ie.key)||null,k(z,I,ie,ye);case b:return I=I.get(ie.key===null?Z:ie.key)||null,Y(z,I,ie,ye);case R:var Le=ie._init;return ie=Le(ie._payload),F(I,z,Z,ie,ye)}if(Q(ie)||G(ie))return I=I.get(Z)||null,ae(z,I,ie,ye,null);if(typeof ie.then=="function")return F(I,z,Z,Mc(ie),ye);if(ie.$$typeof===A)return F(I,z,Z,vc(z,ie),ye);jc(z,ie)}return null}function je(I,z,Z,ie){for(var ye=null,Le=null,Se=z,Ae=z=0,zt=null;Se!==null&&Ae<Z.length;Ae++){Se.index>Ae?(zt=Se,Se=null):zt=Se.sibling;var Ke=K(I,Se,Z[Ae],ie);if(Ke===null){Se===null&&(Se=zt);break}a&&Se&&Ke.alternate===null&&o(I,Se),z=m(Ke,z,Ae),Le===null?ye=Ke:Le.sibling=Ke,Le=Ke,Se=zt}if(Ae===Z.length)return l(I,Se),Fe&&Ka(I,Ae),ye;if(Se===null){for(;Ae<Z.length;Ae++)Se=oe(I,Z[Ae],ie),Se!==null&&(z=m(Se,z,Ae),Le===null?ye=Se:Le.sibling=Se,Le=Se);return Fe&&Ka(I,Ae),ye}for(Se=f(Se);Ae<Z.length;Ae++)zt=F(Se,I,Ae,Z[Ae],ie),zt!==null&&(a&&zt.alternate!==null&&Se.delete(zt.key===null?Ae:zt.key),z=m(zt,z,Ae),Le===null?ye=zt:Le.sibling=zt,Le=zt);return a&&Se.forEach(function(wa){return o(I,wa)}),Fe&&Ka(I,Ae),ye}function we(I,z,Z,ie){if(Z==null)throw Error(r(151));for(var ye=null,Le=null,Se=z,Ae=z=0,zt=null,Ke=Z.next();Se!==null&&!Ke.done;Ae++,Ke=Z.next()){Se.index>Ae?(zt=Se,Se=null):zt=Se.sibling;var wa=K(I,Se,Ke.value,ie);if(wa===null){Se===null&&(Se=zt);break}a&&Se&&wa.alternate===null&&o(I,Se),z=m(wa,z,Ae),Le===null?ye=wa:Le.sibling=wa,Le=wa,Se=zt}if(Ke.done)return l(I,Se),Fe&&Ka(I,Ae),ye;if(Se===null){for(;!Ke.done;Ae++,Ke=Z.next())Ke=oe(I,Ke.value,ie),Ke!==null&&(z=m(Ke,z,Ae),Le===null?ye=Ke:Le.sibling=Ke,Le=Ke);return Fe&&Ka(I,Ae),ye}for(Se=f(Se);!Ke.done;Ae++,Ke=Z.next())Ke=F(Se,I,Ae,Ke.value,ie),Ke!==null&&(a&&Ke.alternate!==null&&Se.delete(Ke.key===null?Ae:Ke.key),z=m(Ke,z,Ae),Le===null?ye=Ke:Le.sibling=Ke,Le=Ke);return a&&Se.forEach(function(h$){return o(I,h$)}),Fe&&Ka(I,Ae),ye}function at(I,z,Z,ie){if(typeof Z=="object"&&Z!==null&&Z.type===S&&Z.key===null&&(Z=Z.props.children),typeof Z=="object"&&Z!==null){switch(Z.$$typeof){case g:e:{for(var ye=Z.key;z!==null;){if(z.key===ye){if(ye=Z.type,ye===S){if(z.tag===7){l(I,z.sibling),ie=p(z,Z.props.children),ie.return=I,I=ie;break e}}else if(z.elementType===ye||typeof ye=="object"&&ye!==null&&ye.$$typeof===R&&TS(ye)===z.type){l(I,z.sibling),ie=p(z,Z.props),Yu(ie,Z),ie.return=I,I=ie;break e}l(I,z);break}else o(I,z);z=z.sibling}Z.type===S?(ie=Ya(Z.props.children,I.mode,ie,Z.key),ie.return=I,I=ie):(ie=fc(Z.type,Z.key,Z.props,null,I.mode,ie),Yu(ie,Z),ie.return=I,I=ie)}return O(I);case b:e:{for(ye=Z.key;z!==null;){if(z.key===ye)if(z.tag===4&&z.stateNode.containerInfo===Z.containerInfo&&z.stateNode.implementation===Z.implementation){l(I,z.sibling),ie=p(z,Z.children||[]),ie.return=I,I=ie;break e}else{l(I,z);break}else o(I,z);z=z.sibling}ie=Qh(Z,I.mode,ie),ie.return=I,I=ie}return O(I);case R:return ye=Z._init,Z=ye(Z._payload),at(I,z,Z,ie)}if(Q(Z))return je(I,z,Z,ie);if(G(Z)){if(ye=G(Z),typeof ye!="function")throw Error(r(150));return Z=ye.call(Z),we(I,z,Z,ie)}if(typeof Z.then=="function")return at(I,z,Mc(Z),ie);if(Z.$$typeof===A)return at(I,z,vc(I,Z),ie);jc(I,Z)}return typeof Z=="string"&&Z!==""||typeof Z=="number"||typeof Z=="bigint"?(Z=""+Z,z!==null&&z.tag===6?(l(I,z.sibling),ie=p(z,Z),ie.return=I,I=ie):(l(I,z),ie=Fh(Z,I.mode,ie),ie.return=I,I=ie),O(I)):l(I,z)}return function(I,z,Z,ie){try{Zu=0;var ye=at(I,z,Z,ie);return no=null,ye}catch(Se){if(Se===$u||Se===mc)throw Se;var Le=vn(29,Se,null,I.mode);return Le.lanes=ie,Le.return=I,Le}finally{}}}var ro=MS(!0),jS=MS(!1),Nn=X(null),ir=null;function la(a){var o=a.alternate;te(Pt,Pt.current&1),te(Nn,a),ir===null&&(o===null||Qi.current!==null||o.memoizedState!==null)&&(ir=a)}function CS(a){if(a.tag===22){if(te(Pt,Pt.current),te(Nn,a),ir===null){var o=a.alternate;o!==null&&o.memoizedState!==null&&(ir=a)}}else sa()}function sa(){te(Pt,Pt.current),te(Nn,Nn.current)}function Cr(a){ee(Nn),ir===a&&(ir=null),ee(Pt)}var Pt=X(0);function Cc(a){for(var o=a;o!==null;){if(o.tag===13){var l=o.memoizedState;if(l!==null&&(l=l.dehydrated,l===null||l.data==="$?"||xv(l)))return o}else if(o.tag===19&&o.memoizedProps.revealOrder!==void 0){if((o.flags&128)!==0)return o}else if(o.child!==null){o.child.return=o,o=o.child;continue}if(o===a)break;for(;o.sibling===null;){if(o.return===null||o.return===a)return null;o=o.return}o.sibling.return=o.return,o=o.sibling}return null}function Pp(a,o,l,f){o=a.memoizedState,l=l(f,o),l=l==null?o:v({},o,l),a.memoizedState=l,a.lanes===0&&(a.updateQueue.baseState=l)}var Rp={enqueueSetState:function(a,o,l){a=a._reactInternals;var f=bn(),p=ia(f);p.payload=o,l!=null&&(p.callback=l),o=oa(a,p,f),o!==null&&(xn(o,a,f),zu(o,a,f))},enqueueReplaceState:function(a,o,l){a=a._reactInternals;var f=bn(),p=ia(f);p.tag=1,p.payload=o,l!=null&&(p.callback=l),o=oa(a,p,f),o!==null&&(xn(o,a,f),zu(o,a,f))},enqueueForceUpdate:function(a,o){a=a._reactInternals;var l=bn(),f=ia(l);f.tag=2,o!=null&&(f.callback=o),o=oa(a,f,l),o!==null&&(xn(o,a,l),zu(o,a,l))}};function PS(a,o,l,f,p,m,O){return a=a.stateNode,typeof a.shouldComponentUpdate=="function"?a.shouldComponentUpdate(f,m,O):o.prototype&&o.prototype.isPureReactComponent?!ju(l,f)||!ju(p,m):!0}function RS(a,o,l,f){a=o.state,typeof o.componentWillReceiveProps=="function"&&o.componentWillReceiveProps(l,f),typeof o.UNSAFE_componentWillReceiveProps=="function"&&o.UNSAFE_componentWillReceiveProps(l,f),o.state!==a&&Rp.enqueueReplaceState(o,o.state,null)}function ni(a,o){var l=o;if("ref"in o){l={};for(var f in o)f!=="ref"&&(l[f]=o[f])}if(a=a.defaultProps){l===o&&(l=v({},l));for(var p in a)l[p]===void 0&&(l[p]=a[p])}return l}var Pc=typeof reportError=="function"?reportError:function(a){if(typeof window=="object"&&typeof window.ErrorEvent=="function"){var o=new window.ErrorEvent("error",{bubbles:!0,cancelable:!0,message:typeof a=="object"&&a!==null&&typeof a.message=="string"?String(a.message):String(a),error:a});if(!window.dispatchEvent(o))return}else if(typeof process=="object"&&typeof process.emit=="function"){process.emit("uncaughtException",a);return}console.error(a)};function NS(a){Pc(a)}function DS(a){console.error(a)}function kS(a){Pc(a)}function Rc(a,o){try{var l=a.onUncaughtError;l(o.value,{componentStack:o.stack})}catch(f){setTimeout(function(){throw f})}}function qS(a,o,l){try{var f=a.onCaughtError;f(l.value,{componentStack:l.stack,errorBoundary:o.tag===1?o.stateNode:null})}catch(p){setTimeout(function(){throw p})}}function Np(a,o,l){return l=ia(l),l.tag=3,l.payload={element:null},l.callback=function(){Rc(a,o)},l}function $S(a){return a=ia(a),a.tag=3,a}function BS(a,o,l,f){var p=l.type.getDerivedStateFromError;if(typeof p=="function"){var m=f.value;a.payload=function(){return p(m)},a.callback=function(){qS(o,l,f)}}var O=l.stateNode;O!==null&&typeof O.componentDidCatch=="function"&&(a.callback=function(){qS(o,l,f),typeof p!="function"&&(va===null?va=new Set([this]):va.add(this));var j=f.stack;this.componentDidCatch(f.value,{componentStack:j!==null?j:""})})}function hq(a,o,l,f,p){if(l.flags|=32768,f!==null&&typeof f=="object"&&typeof f.then=="function"){if(o=l.alternate,o!==null&&Du(o,l,p,!0),l=Nn.current,l!==null){switch(l.tag){case 13:return ir===null?rv():l.alternate===null&&xt===0&&(xt=3),l.flags&=-257,l.flags|=65536,l.lanes=p,f===lp?l.flags|=16384:(o=l.updateQueue,o===null?l.updateQueue=new Set([f]):o.add(f),iv(a,f,p)),!1;case 22:return l.flags|=65536,f===lp?l.flags|=16384:(o=l.updateQueue,o===null?(o={transitions:null,markerInstances:null,retryQueue:new Set([f])},l.updateQueue=o):(l=o.retryQueue,l===null?o.retryQueue=new Set([f]):l.add(f)),iv(a,f,p)),!1}throw Error(r(435,l.tag))}return iv(a,f,p),rv(),!1}if(Fe)return o=Nn.current,o!==null?((o.flags&65536)===0&&(o.flags|=256),o.flags|=65536,o.lanes=p,f!==tp&&(a=Error(r(422),{cause:f}),Nu(jn(a,l)))):(f!==tp&&(o=Error(r(423),{cause:f}),Nu(jn(o,l))),a=a.current.alternate,a.flags|=65536,p&=-p,a.lanes|=p,f=jn(f,l),p=Np(a.stateNode,f,p),fp(a,p),xt!==4&&(xt=2)),!1;var m=Error(r(520),{cause:f});if(m=jn(m,l),el===null?el=[m]:el.push(m),xt!==4&&(xt=2),o===null)return!0;f=jn(f,l),l=o;do{switch(l.tag){case 3:return l.flags|=65536,a=p&-p,l.lanes|=a,a=Np(l.stateNode,f,a),fp(l,a),!1;case 1:if(o=l.type,m=l.stateNode,(l.flags&128)===0&&(typeof o.getDerivedStateFromError=="function"||m!==null&&typeof m.componentDidCatch=="function"&&(va===null||!va.has(m))))return l.flags|=65536,p&=-p,l.lanes|=p,p=$S(p),BS(p,a,l,f),fp(l,p),!1}l=l.return}while(l!==null);return!1}var zS=Error(r(461)),$t=!1;function Ut(a,o,l,f){o.child=a===null?jS(o,null,l,f):ro(o,a.child,l,f)}function LS(a,o,l,f,p){l=l.render;var m=o.ref;if("ref"in f){var O={};for(var j in f)j!=="ref"&&(O[j]=f[j])}else O=f;return Ja(o),f=yp(a,o,l,O,m,p),j=mp(),a!==null&&!$t?(gp(a,o,p),Pr(a,o,p)):(Fe&&j&&Jh(o),o.flags|=1,Ut(a,o,f,p),o.child)}function IS(a,o,l,f,p){if(a===null){var m=l.type;return typeof m=="function"&&!Wh(m)&&m.defaultProps===void 0&&l.compare===null?(o.tag=15,o.type=m,US(a,o,m,f,p)):(a=fc(l.type,null,f,o,o.mode,p),a.ref=o.ref,a.return=o,o.child=a)}if(m=a.child,!Ip(a,p)){var O=m.memoizedProps;if(l=l.compare,l=l!==null?l:ju,l(O,f)&&a.ref===o.ref)return Pr(a,o,p)}return o.flags|=1,a=Or(m,f),a.ref=o.ref,a.return=o,o.child=a}function US(a,o,l,f,p){if(a!==null){var m=a.memoizedProps;if(ju(m,f)&&a.ref===o.ref)if($t=!1,o.pendingProps=f=m,Ip(a,p))(a.flags&131072)!==0&&($t=!0);else return o.lanes=a.lanes,Pr(a,o,p)}return Dp(a,o,l,f,p)}function HS(a,o,l){var f=o.pendingProps,p=f.children,m=a!==null?a.memoizedState:null;if(f.mode==="hidden"){if((o.flags&128)!==0){if(f=m!==null?m.baseLanes|l:l,a!==null){for(p=o.child=a.child,m=0;p!==null;)m=m|p.lanes|p.childLanes,p=p.sibling;o.childLanes=m&~f}else o.childLanes=0,o.child=null;return GS(a,o,f,l)}if((l&536870912)!==0)o.memoizedState={baseLanes:0,cachePool:null},a!==null&&yc(o,m!==null?m.cachePool:null),m!==null?U1(o,m):hp(),CS(o);else return o.lanes=o.childLanes=536870912,GS(a,o,m!==null?m.baseLanes|l:l,l)}else m!==null?(yc(o,m.cachePool),U1(o,m),sa(),o.memoizedState=null):(a!==null&&yc(o,null),hp(),sa());return Ut(a,o,p,l),o.child}function GS(a,o,l,f){var p=up();return p=p===null?null:{parent:Ct._currentValue,pool:p},o.memoizedState={baseLanes:l,cachePool:p},a!==null&&yc(o,null),hp(),CS(o),a!==null&&Du(a,o,f,!0),null}function Nc(a,o){var l=o.ref;if(l===null)a!==null&&a.ref!==null&&(o.flags|=4194816);else{if(typeof l!="function"&&typeof l!="object")throw Error(r(284));(a===null||a.ref!==l)&&(o.flags|=4194816)}}function Dp(a,o,l,f,p){return Ja(o),l=yp(a,o,l,f,void 0,p),f=mp(),a!==null&&!$t?(gp(a,o,p),Pr(a,o,p)):(Fe&&f&&Jh(o),o.flags|=1,Ut(a,o,l,p),o.child)}function VS(a,o,l,f,p,m){return Ja(o),o.updateQueue=null,l=G1(o,f,l,p),H1(a),f=mp(),a!==null&&!$t?(gp(a,o,m),Pr(a,o,m)):(Fe&&f&&Jh(o),o.flags|=1,Ut(a,o,l,m),o.child)}function ZS(a,o,l,f,p){if(Ja(o),o.stateNode===null){var m=Yi,O=l.contextType;typeof O=="object"&&O!==null&&(m=Xt(O)),m=new l(f,m),o.memoizedState=m.state!==null&&m.state!==void 0?m.state:null,m.updater=Rp,o.stateNode=m,m._reactInternals=o,m=o.stateNode,m.props=f,m.state=o.memoizedState,m.refs={},sp(o),O=l.contextType,m.context=typeof O=="object"&&O!==null?Xt(O):Yi,m.state=o.memoizedState,O=l.getDerivedStateFromProps,typeof O=="function"&&(Pp(o,l,O,f),m.state=o.memoizedState),typeof l.getDerivedStateFromProps=="function"||typeof m.getSnapshotBeforeUpdate=="function"||typeof m.UNSAFE_componentWillMount!="function"&&typeof m.componentWillMount!="function"||(O=m.state,typeof m.componentWillMount=="function"&&m.componentWillMount(),typeof m.UNSAFE_componentWillMount=="function"&&m.UNSAFE_componentWillMount(),O!==m.state&&Rp.enqueueReplaceState(m,m.state,null),Iu(o,f,m,p),Lu(),m.state=o.memoizedState),typeof m.componentDidMount=="function"&&(o.flags|=4194308),f=!0}else if(a===null){m=o.stateNode;var j=o.memoizedProps,k=ni(l,j);m.props=k;var Y=m.context,ae=l.contextType;O=Yi,typeof ae=="object"&&ae!==null&&(O=Xt(ae));var oe=l.getDerivedStateFromProps;ae=typeof oe=="function"||typeof m.getSnapshotBeforeUpdate=="function",j=o.pendingProps!==j,ae||typeof m.UNSAFE_componentWillReceiveProps!="function"&&typeof m.componentWillReceiveProps!="function"||(j||Y!==O)&&RS(o,m,f,O),aa=!1;var K=o.memoizedState;m.state=K,Iu(o,f,m,p),Lu(),Y=o.memoizedState,j||K!==Y||aa?(typeof oe=="function"&&(Pp(o,l,oe,f),Y=o.memoizedState),(k=aa||PS(o,l,k,f,K,Y,O))?(ae||typeof m.UNSAFE_componentWillMount!="function"&&typeof m.componentWillMount!="function"||(typeof m.componentWillMount=="function"&&m.componentWillMount(),typeof m.UNSAFE_componentWillMount=="function"&&m.UNSAFE_componentWillMount()),typeof m.componentDidMount=="function"&&(o.flags|=4194308)):(typeof m.componentDidMount=="function"&&(o.flags|=4194308),o.memoizedProps=f,o.memoizedState=Y),m.props=f,m.state=Y,m.context=O,f=k):(typeof m.componentDidMount=="function"&&(o.flags|=4194308),f=!1)}else{m=o.stateNode,cp(a,o),O=o.memoizedProps,ae=ni(l,O),m.props=ae,oe=o.pendingProps,K=m.context,Y=l.contextType,k=Yi,typeof Y=="object"&&Y!==null&&(k=Xt(Y)),j=l.getDerivedStateFromProps,(Y=typeof j=="function"||typeof m.getSnapshotBeforeUpdate=="function")||typeof m.UNSAFE_componentWillReceiveProps!="function"&&typeof m.componentWillReceiveProps!="function"||(O!==oe||K!==k)&&RS(o,m,f,k),aa=!1,K=o.memoizedState,m.state=K,Iu(o,f,m,p),Lu();var F=o.memoizedState;O!==oe||K!==F||aa||a!==null&&a.dependencies!==null&&pc(a.dependencies)?(typeof j=="function"&&(Pp(o,l,j,f),F=o.memoizedState),(ae=aa||PS(o,l,ae,f,K,F,k)||a!==null&&a.dependencies!==null&&pc(a.dependencies))?(Y||typeof m.UNSAFE_componentWillUpdate!="function"&&typeof m.componentWillUpdate!="function"||(typeof m.componentWillUpdate=="function"&&m.componentWillUpdate(f,F,k),typeof m.UNSAFE_componentWillUpdate=="function"&&m.UNSAFE_componentWillUpdate(f,F,k)),typeof m.componentDidUpdate=="function"&&(o.flags|=4),typeof m.getSnapshotBeforeUpdate=="function"&&(o.flags|=1024)):(typeof m.componentDidUpdate!="function"||O===a.memoizedProps&&K===a.memoizedState||(o.flags|=4),typeof m.getSnapshotBeforeUpdate!="function"||O===a.memoizedProps&&K===a.memoizedState||(o.flags|=1024),o.memoizedProps=f,o.memoizedState=F),m.props=f,m.state=F,m.context=k,f=ae):(typeof m.componentDidUpdate!="function"||O===a.memoizedProps&&K===a.memoizedState||(o.flags|=4),typeof m.getSnapshotBeforeUpdate!="function"||O===a.memoizedProps&&K===a.memoizedState||(o.flags|=1024),f=!1)}return m=f,Nc(a,o),f=(o.flags&128)!==0,m||f?(m=o.stateNode,l=f&&typeof l.getDerivedStateFromError!="function"?null:m.render(),o.flags|=1,a!==null&&f?(o.child=ro(o,a.child,null,p),o.child=ro(o,null,l,p)):Ut(a,o,l,p),o.memoizedState=m.state,a=o.child):a=Pr(a,o,p),a}function YS(a,o,l,f){return Ru(),o.flags|=256,Ut(a,o,l,f),o.child}var kp={dehydrated:null,treeContext:null,retryLane:0,hydrationErrors:null};function qp(a){return{baseLanes:a,cachePool:D1()}}function $p(a,o,l){return a=a!==null?a.childLanes&~l:0,o&&(a|=Dn),a}function XS(a,o,l){var f=o.pendingProps,p=!1,m=(o.flags&128)!==0,O;if((O=m)||(O=a!==null&&a.memoizedState===null?!1:(Pt.current&2)!==0),O&&(p=!0,o.flags&=-129),O=(o.flags&32)!==0,o.flags&=-33,a===null){if(Fe){if(p?la(o):sa(),Fe){var j=bt,k;if(k=j){e:{for(k=j,j=ar;k.nodeType!==8;){if(!j){j=null;break e}if(k=Vn(k.nextSibling),k===null){j=null;break e}}j=k}j!==null?(o.memoizedState={dehydrated:j,treeContext:Xa!==null?{id:Ar,overflow:Er}:null,retryLane:536870912,hydrationErrors:null},k=vn(18,null,null,0),k.stateNode=j,k.return=o,o.child=k,Jt=o,bt=null,k=!0):k=!1}k||Fa(o)}if(j=o.memoizedState,j!==null&&(j=j.dehydrated,j!==null))return xv(j)?o.lanes=32:o.lanes=536870912,null;Cr(o)}return j=f.children,f=f.fallback,p?(sa(),p=o.mode,j=Dc({mode:"hidden",children:j},p),f=Ya(f,p,l,null),j.return=o,f.return=o,j.sibling=f,o.child=j,p=o.child,p.memoizedState=qp(l),p.childLanes=$p(a,O,l),o.memoizedState=kp,f):(la(o),Bp(o,j))}if(k=a.memoizedState,k!==null&&(j=k.dehydrated,j!==null)){if(m)o.flags&256?(la(o),o.flags&=-257,o=zp(a,o,l)):o.memoizedState!==null?(sa(),o.child=a.child,o.flags|=128,o=null):(sa(),p=f.fallback,j=o.mode,f=Dc({mode:"visible",children:f.children},j),p=Ya(p,j,l,null),p.flags|=2,f.return=o,p.return=o,f.sibling=p,o.child=f,ro(o,a.child,null,l),f=o.child,f.memoizedState=qp(l),f.childLanes=$p(a,O,l),o.memoizedState=kp,o=p);else if(la(o),xv(j)){if(O=j.nextSibling&&j.nextSibling.dataset,O)var Y=O.dgst;O=Y,f=Error(r(419)),f.stack="",f.digest=O,Nu({value:f,source:null,stack:null}),o=zp(a,o,l)}else if($t||Du(a,o,l,!1),O=(l&a.childLanes)!==0,$t||O){if(O=ct,O!==null&&(f=l&-l,f=(f&42)!==0?1:_h(f),f=(f&(O.suspendedLanes|l))!==0?0:f,f!==0&&f!==k.retryLane))throw k.retryLane=f,Zi(a,f),xn(O,a,f),zS;j.data==="$?"||rv(),o=zp(a,o,l)}else j.data==="$?"?(o.flags|=192,o.child=a.child,o=null):(a=k.treeContext,bt=Vn(j.nextSibling),Jt=o,Fe=!0,Wa=null,ar=!1,a!==null&&(Pn[Rn++]=Ar,Pn[Rn++]=Er,Pn[Rn++]=Xa,Ar=a.id,Er=a.overflow,Xa=o),o=Bp(o,f.children),o.flags|=4096);return o}return p?(sa(),p=f.fallback,j=o.mode,k=a.child,Y=k.sibling,f=Or(k,{mode:"hidden",children:f.children}),f.subtreeFlags=k.subtreeFlags&65011712,Y!==null?p=Or(Y,p):(p=Ya(p,j,l,null),p.flags|=2),p.return=o,f.return=o,f.sibling=p,o.child=f,f=p,p=o.child,j=a.child.memoizedState,j===null?j=qp(l):(k=j.cachePool,k!==null?(Y=Ct._currentValue,k=k.parent!==Y?{parent:Y,pool:Y}:k):k=D1(),j={baseLanes:j.baseLanes|l,cachePool:k}),p.memoizedState=j,p.childLanes=$p(a,O,l),o.memoizedState=kp,f):(la(o),l=a.child,a=l.sibling,l=Or(l,{mode:"visible",children:f.children}),l.return=o,l.sibling=null,a!==null&&(O=o.deletions,O===null?(o.deletions=[a],o.flags|=16):O.push(a)),o.child=l,o.memoizedState=null,l)}function Bp(a,o){return o=Dc({mode:"visible",children:o},a.mode),o.return=a,a.child=o}function Dc(a,o){return a=vn(22,a,null,o),a.lanes=0,a.stateNode={_visibility:1,_pendingMarkers:null,_retryCache:null,_transitions:null},a}function zp(a,o,l){return ro(o,a.child,null,l),a=Bp(o,o.pendingProps.children),a.flags|=2,o.memoizedState=null,a}function KS(a,o,l){a.lanes|=o;var f=a.alternate;f!==null&&(f.lanes|=o),rp(a.return,o,l)}function Lp(a,o,l,f,p){var m=a.memoizedState;m===null?a.memoizedState={isBackwards:o,rendering:null,renderingStartTime:0,last:f,tail:l,tailMode:p}:(m.isBackwards=o,m.rendering=null,m.renderingStartTime=0,m.last=f,m.tail=l,m.tailMode=p)}function WS(a,o,l){var f=o.pendingProps,p=f.revealOrder,m=f.tail;if(Ut(a,o,f.children,l),f=Pt.current,(f&2)!==0)f=f&1|2,o.flags|=128;else{if(a!==null&&(a.flags&128)!==0)e:for(a=o.child;a!==null;){if(a.tag===13)a.memoizedState!==null&&KS(a,l,o);else if(a.tag===19)KS(a,l,o);else if(a.child!==null){a.child.return=a,a=a.child;continue}if(a===o)break e;for(;a.sibling===null;){if(a.return===null||a.return===o)break e;a=a.return}a.sibling.return=a.return,a=a.sibling}f&=1}switch(te(Pt,f),p){case"forwards":for(l=o.child,p=null;l!==null;)a=l.alternate,a!==null&&Cc(a)===null&&(p=l),l=l.sibling;l=p,l===null?(p=o.child,o.child=null):(p=l.sibling,l.sibling=null),Lp(o,!1,p,l,m);break;case"backwards":for(l=null,p=o.child,o.child=null;p!==null;){if(a=p.alternate,a!==null&&Cc(a)===null){o.child=p;break}a=p.sibling,p.sibling=l,l=p,p=a}Lp(o,!0,l,null,m);break;case"together":Lp(o,!1,null,null,void 0);break;default:o.memoizedState=null}return o.child}function Pr(a,o,l){if(a!==null&&(o.dependencies=a.dependencies),pa|=o.lanes,(l&o.childLanes)===0)if(a!==null){if(Du(a,o,l,!1),(l&o.childLanes)===0)return null}else return null;if(a!==null&&o.child!==a.child)throw Error(r(153));if(o.child!==null){for(a=o.child,l=Or(a,a.pendingProps),o.child=l,l.return=o;a.sibling!==null;)a=a.sibling,l=l.sibling=Or(a,a.pendingProps),l.return=o;l.sibling=null}return o.child}function Ip(a,o){return(a.lanes&o)!==0?!0:(a=a.dependencies,!!(a!==null&&pc(a)))}function pq(a,o,l){switch(o.tag){case 3:Pe(o,o.stateNode.containerInfo),ra(o,Ct,a.memoizedState.cache),Ru();break;case 27:case 5:Oe(o);break;case 4:Pe(o,o.stateNode.containerInfo);break;case 10:ra(o,o.type,o.memoizedProps.value);break;case 13:var f=o.memoizedState;if(f!==null)return f.dehydrated!==null?(la(o),o.flags|=128,null):(l&o.child.childLanes)!==0?XS(a,o,l):(la(o),a=Pr(a,o,l),a!==null?a.sibling:null);la(o);break;case 19:var p=(a.flags&128)!==0;if(f=(l&o.childLanes)!==0,f||(Du(a,o,l,!1),f=(l&o.childLanes)!==0),p){if(f)return WS(a,o,l);o.flags|=128}if(p=o.memoizedState,p!==null&&(p.rendering=null,p.tail=null,p.lastEffect=null),te(Pt,Pt.current),f)break;return null;case 22:case 23:return o.lanes=0,HS(a,o,l);case 24:ra(o,Ct,a.memoizedState.cache)}return Pr(a,o,l)}function FS(a,o,l){if(a!==null)if(a.memoizedProps!==o.pendingProps)$t=!0;else{if(!Ip(a,l)&&(o.flags&128)===0)return $t=!1,pq(a,o,l);$t=(a.flags&131072)!==0}else $t=!1,Fe&&(o.flags&1048576)!==0&&T1(o,hc,o.index);switch(o.lanes=0,o.tag){case 16:e:{a=o.pendingProps;var f=o.elementType,p=f._init;if(f=p(f._payload),o.type=f,typeof f=="function")Wh(f)?(a=ni(f,a),o.tag=1,o=ZS(null,o,f,a,l)):(o.tag=0,o=Dp(null,o,f,a,l));else{if(f!=null){if(p=f.$$typeof,p===C){o.tag=11,o=LS(null,o,f,a,l);break e}else if(p===P){o.tag=14,o=IS(null,o,f,a,l);break e}}throw o=ne(f)||f,Error(r(306,o,""))}}return o;case 0:return Dp(a,o,o.type,o.pendingProps,l);case 1:return f=o.type,p=ni(f,o.pendingProps),ZS(a,o,f,p,l);case 3:e:{if(Pe(o,o.stateNode.containerInfo),a===null)throw Error(r(387));f=o.pendingProps;var m=o.memoizedState;p=m.element,cp(a,o),Iu(o,f,null,l);var O=o.memoizedState;if(f=O.cache,ra(o,Ct,f),f!==m.cache&&ap(o,[Ct],l,!0),Lu(),f=O.element,m.isDehydrated)if(m={element:f,isDehydrated:!1,cache:O.cache},o.updateQueue.baseState=m,o.memoizedState=m,o.flags&256){o=YS(a,o,f,l);break e}else if(f!==p){p=jn(Error(r(424)),o),Nu(p),o=YS(a,o,f,l);break e}else{switch(a=o.stateNode.containerInfo,a.nodeType){case 9:a=a.body;break;default:a=a.nodeName==="HTML"?a.ownerDocument.body:a}for(bt=Vn(a.firstChild),Jt=o,Fe=!0,Wa=null,ar=!0,l=jS(o,null,f,l),o.child=l;l;)l.flags=l.flags&-3|4096,l=l.sibling}else{if(Ru(),f===p){o=Pr(a,o,l);break e}Ut(a,o,f,l)}o=o.child}return o;case 26:return Nc(a,o),a===null?(l=tO(o.type,null,o.pendingProps,null))?o.memoizedState=l:Fe||(l=o.type,a=o.pendingProps,f=Xc(be.current).createElement(l),f[Yt]=o,f[an]=a,Gt(f,l,a),qt(f),o.stateNode=f):o.memoizedState=tO(o.type,a.memoizedProps,o.pendingProps,a.memoizedState),null;case 27:return Oe(o),a===null&&Fe&&(f=o.stateNode=Qw(o.type,o.pendingProps,be.current),Jt=o,ar=!0,p=bt,ga(o.type)?(_v=p,bt=Vn(f.firstChild)):bt=p),Ut(a,o,o.pendingProps.children,l),Nc(a,o),a===null&&(o.flags|=4194304),o.child;case 5:return a===null&&Fe&&((p=f=bt)&&(f=Uq(f,o.type,o.pendingProps,ar),f!==null?(o.stateNode=f,Jt=o,bt=Vn(f.firstChild),ar=!1,p=!0):p=!1),p||Fa(o)),Oe(o),p=o.type,m=o.pendingProps,O=a!==null?a.memoizedProps:null,f=m.children,mv(p,m)?f=null:O!==null&&mv(p,O)&&(o.flags|=32),o.memoizedState!==null&&(p=yp(a,o,oq,null,null,l),sl._currentValue=p),Nc(a,o),Ut(a,o,f,l),o.child;case 6:return a===null&&Fe&&((a=l=bt)&&(l=Hq(l,o.pendingProps,ar),l!==null?(o.stateNode=l,Jt=o,bt=null,a=!0):a=!1),a||Fa(o)),null;case 13:return XS(a,o,l);case 4:return Pe(o,o.stateNode.containerInfo),f=o.pendingProps,a===null?o.child=ro(o,null,f,l):Ut(a,o,f,l),o.child;case 11:return LS(a,o,o.type,o.pendingProps,l);case 7:return Ut(a,o,o.pendingProps,l),o.child;case 8:return Ut(a,o,o.pendingProps.children,l),o.child;case 12:return Ut(a,o,o.pendingProps.children,l),o.child;case 10:return f=o.pendingProps,ra(o,o.type,f.value),Ut(a,o,f.children,l),o.child;case 9:return p=o.type._context,f=o.pendingProps.children,Ja(o),p=Xt(p),f=f(p),o.flags|=1,Ut(a,o,f,l),o.child;case 14:return IS(a,o,o.type,o.pendingProps,l);case 15:return US(a,o,o.type,o.pendingProps,l);case 19:return WS(a,o,l);case 31:return f=o.pendingProps,l=o.mode,f={mode:f.mode,children:f.children},a===null?(l=Dc(f,l),l.ref=o.ref,o.child=l,l.return=o,o=l):(l=Or(a.child,f),l.ref=o.ref,o.child=l,l.return=o,o=l),o;case 22:return HS(a,o,l);case 24:return Ja(o),f=Xt(Ct),a===null?(p=up(),p===null&&(p=ct,m=ip(),p.pooledCache=m,m.refCount++,m!==null&&(p.pooledCacheLanes|=l),p=m),o.memoizedState={parent:f,cache:p},sp(o),ra(o,Ct,p)):((a.lanes&l)!==0&&(cp(a,o),Iu(o,null,null,l),Lu()),p=a.memoizedState,m=o.memoizedState,p.parent!==f?(p={parent:f,cache:f},o.memoizedState=p,o.lanes===0&&(o.memoizedState=o.updateQueue.baseState=p),ra(o,Ct,f)):(f=m.cache,ra(o,Ct,f),f!==p.cache&&ap(o,[Ct],l,!0))),Ut(a,o,o.pendingProps.children,l),o.child;case 29:throw o.pendingProps}throw Error(r(156,o.tag))}function Rr(a){a.flags|=4}function QS(a,o){if(o.type!=="stylesheet"||(o.state.loading&4)!==0)a.flags&=-16777217;else if(a.flags|=16777216,!oO(o)){if(o=Nn.current,o!==null&&((Ve&4194048)===Ve?ir!==null:(Ve&62914560)!==Ve&&(Ve&536870912)===0||o!==ir))throw Bu=lp,k1;a.flags|=8192}}function kc(a,o){o!==null&&(a.flags|=4),a.flags&16384&&(o=a.tag!==22?C_():536870912,a.lanes|=o,uo|=o)}function Xu(a,o){if(!Fe)switch(a.tailMode){case"hidden":o=a.tail;for(var l=null;o!==null;)o.alternate!==null&&(l=o),o=o.sibling;l===null?a.tail=null:l.sibling=null;break;case"collapsed":l=a.tail;for(var f=null;l!==null;)l.alternate!==null&&(f=l),l=l.sibling;f===null?o||a.tail===null?a.tail=null:a.tail.sibling=null:f.sibling=null}}function yt(a){var o=a.alternate!==null&&a.alternate.child===a.child,l=0,f=0;if(o)for(var p=a.child;p!==null;)l|=p.lanes|p.childLanes,f|=p.subtreeFlags&65011712,f|=p.flags&65011712,p.return=a,p=p.sibling;else for(p=a.child;p!==null;)l|=p.lanes|p.childLanes,f|=p.subtreeFlags,f|=p.flags,p.return=a,p=p.sibling;return a.subtreeFlags|=f,a.childLanes=l,o}function vq(a,o,l){var f=o.pendingProps;switch(ep(o),o.tag){case 31:case 16:case 15:case 0:case 11:case 7:case 8:case 12:case 9:case 14:return yt(o),null;case 1:return yt(o),null;case 3:return l=o.stateNode,f=null,a!==null&&(f=a.memoizedState.cache),o.memoizedState.cache!==f&&(o.flags|=2048),Mr(Ct),fe(),l.pendingContext&&(l.context=l.pendingContext,l.pendingContext=null),(a===null||a.child===null)&&(Pu(o)?Rr(o):a===null||a.memoizedState.isDehydrated&&(o.flags&256)===0||(o.flags|=1024,C1())),yt(o),null;case 26:return l=o.memoizedState,a===null?(Rr(o),l!==null?(yt(o),QS(o,l)):(yt(o),o.flags&=-16777217)):l?l!==a.memoizedState?(Rr(o),yt(o),QS(o,l)):(yt(o),o.flags&=-16777217):(a.memoizedProps!==f&&Rr(o),yt(o),o.flags&=-16777217),null;case 27:Me(o),l=be.current;var p=o.type;if(a!==null&&o.stateNode!=null)a.memoizedProps!==f&&Rr(o);else{if(!f){if(o.stateNode===null)throw Error(r(166));return yt(o),null}a=ce.current,Pu(o)?M1(o):(a=Qw(p,f,l),o.stateNode=a,Rr(o))}return yt(o),null;case 5:if(Me(o),l=o.type,a!==null&&o.stateNode!=null)a.memoizedProps!==f&&Rr(o);else{if(!f){if(o.stateNode===null)throw Error(r(166));return yt(o),null}if(a=ce.current,Pu(o))M1(o);else{switch(p=Xc(be.current),a){case 1:a=p.createElementNS("http://www.w3.org/2000/svg",l);break;case 2:a=p.createElementNS("http://www.w3.org/1998/Math/MathML",l);break;default:switch(l){case"svg":a=p.createElementNS("http://www.w3.org/2000/svg",l);break;case"math":a=p.createElementNS("http://www.w3.org/1998/Math/MathML",l);break;case"script":a=p.createElement("div"),a.innerHTML="<script><\/script>",a=a.removeChild(a.firstChild);break;case"select":a=typeof f.is=="string"?p.createElement("select",{is:f.is}):p.createElement("select"),f.multiple?a.multiple=!0:f.size&&(a.size=f.size);break;default:a=typeof f.is=="string"?p.createElement(l,{is:f.is}):p.createElement(l)}}a[Yt]=o,a[an]=f;e:for(p=o.child;p!==null;){if(p.tag===5||p.tag===6)a.appendChild(p.stateNode);else if(p.tag!==4&&p.tag!==27&&p.child!==null){p.child.return=p,p=p.child;continue}if(p===o)break e;for(;p.sibling===null;){if(p.return===null||p.return===o)break e;p=p.return}p.sibling.return=p.return,p=p.sibling}o.stateNode=a;e:switch(Gt(a,l,f),l){case"button":case"input":case"select":case"textarea":a=!!f.autoFocus;break e;case"img":a=!0;break e;default:a=!1}a&&Rr(o)}}return yt(o),o.flags&=-16777217,null;case 6:if(a&&o.stateNode!=null)a.memoizedProps!==f&&Rr(o);else{if(typeof f!="string"&&o.stateNode===null)throw Error(r(166));if(a=be.current,Pu(o)){if(a=o.stateNode,l=o.memoizedProps,f=null,p=Jt,p!==null)switch(p.tag){case 27:case 5:f=p.memoizedProps}a[Yt]=o,a=!!(a.nodeValue===l||f!==null&&f.suppressHydrationWarning===!0||Vw(a.nodeValue,l)),a||Fa(o)}else a=Xc(a).createTextNode(f),a[Yt]=o,o.stateNode=a}return yt(o),null;case 13:if(f=o.memoizedState,a===null||a.memoizedState!==null&&a.memoizedState.dehydrated!==null){if(p=Pu(o),f!==null&&f.dehydrated!==null){if(a===null){if(!p)throw Error(r(318));if(p=o.memoizedState,p=p!==null?p.dehydrated:null,!p)throw Error(r(317));p[Yt]=o}else Ru(),(o.flags&128)===0&&(o.memoizedState=null),o.flags|=4;yt(o),p=!1}else p=C1(),a!==null&&a.memoizedState!==null&&(a.memoizedState.hydrationErrors=p),p=!0;if(!p)return o.flags&256?(Cr(o),o):(Cr(o),null)}if(Cr(o),(o.flags&128)!==0)return o.lanes=l,o;if(l=f!==null,a=a!==null&&a.memoizedState!==null,l){f=o.child,p=null,f.alternate!==null&&f.alternate.memoizedState!==null&&f.alternate.memoizedState.cachePool!==null&&(p=f.alternate.memoizedState.cachePool.pool);var m=null;f.memoizedState!==null&&f.memoizedState.cachePool!==null&&(m=f.memoizedState.cachePool.pool),m!==p&&(f.flags|=2048)}return l!==a&&l&&(o.child.flags|=8192),kc(o,o.updateQueue),yt(o),null;case 4:return fe(),a===null&&dv(o.stateNode.containerInfo),yt(o),null;case 10:return Mr(o.type),yt(o),null;case 19:if(ee(Pt),p=o.memoizedState,p===null)return yt(o),null;if(f=(o.flags&128)!==0,m=p.rendering,m===null)if(f)Xu(p,!1);else{if(xt!==0||a!==null&&(a.flags&128)!==0)for(a=o.child;a!==null;){if(m=Cc(a),m!==null){for(o.flags|=128,Xu(p,!1),a=m.updateQueue,o.updateQueue=a,kc(o,a),o.subtreeFlags=0,a=l,l=o.child;l!==null;)E1(l,a),l=l.sibling;return te(Pt,Pt.current&1|2),o.child}a=a.sibling}p.tail!==null&&dt()>Bc&&(o.flags|=128,f=!0,Xu(p,!1),o.lanes=4194304)}else{if(!f)if(a=Cc(m),a!==null){if(o.flags|=128,f=!0,a=a.updateQueue,o.updateQueue=a,kc(o,a),Xu(p,!0),p.tail===null&&p.tailMode==="hidden"&&!m.alternate&&!Fe)return yt(o),null}else 2*dt()-p.renderingStartTime>Bc&&l!==536870912&&(o.flags|=128,f=!0,Xu(p,!1),o.lanes=4194304);p.isBackwards?(m.sibling=o.child,o.child=m):(a=p.last,a!==null?a.sibling=m:o.child=m,p.last=m)}return p.tail!==null?(o=p.tail,p.rendering=o,p.tail=o.sibling,p.renderingStartTime=dt(),o.sibling=null,a=Pt.current,te(Pt,f?a&1|2:a&1),o):(yt(o),null);case 22:case 23:return Cr(o),pp(),f=o.memoizedState!==null,a!==null?a.memoizedState!==null!==f&&(o.flags|=8192):f&&(o.flags|=8192),f?(l&536870912)!==0&&(o.flags&128)===0&&(yt(o),o.subtreeFlags&6&&(o.flags|=8192)):yt(o),l=o.updateQueue,l!==null&&kc(o,l.retryQueue),l=null,a!==null&&a.memoizedState!==null&&a.memoizedState.cachePool!==null&&(l=a.memoizedState.cachePool.pool),f=null,o.memoizedState!==null&&o.memoizedState.cachePool!==null&&(f=o.memoizedState.cachePool.pool),f!==l&&(o.flags|=2048),a!==null&&ee(ei),null;case 24:return l=null,a!==null&&(l=a.memoizedState.cache),o.memoizedState.cache!==l&&(o.flags|=2048),Mr(Ct),yt(o),null;case 25:return null;case 30:return null}throw Error(r(156,o.tag))}function yq(a,o){switch(ep(o),o.tag){case 1:return a=o.flags,a&65536?(o.flags=a&-65537|128,o):null;case 3:return Mr(Ct),fe(),a=o.flags,(a&65536)!==0&&(a&128)===0?(o.flags=a&-65537|128,o):null;case 26:case 27:case 5:return Me(o),null;case 13:if(Cr(o),a=o.memoizedState,a!==null&&a.dehydrated!==null){if(o.alternate===null)throw Error(r(340));Ru()}return a=o.flags,a&65536?(o.flags=a&-65537|128,o):null;case 19:return ee(Pt),null;case 4:return fe(),null;case 10:return Mr(o.type),null;case 22:case 23:return Cr(o),pp(),a!==null&&ee(ei),a=o.flags,a&65536?(o.flags=a&-65537|128,o):null;case 24:return Mr(Ct),null;case 25:return null;default:return null}}function JS(a,o){switch(ep(o),o.tag){case 3:Mr(Ct),fe();break;case 26:case 27:case 5:Me(o);break;case 4:fe();break;case 13:Cr(o);break;case 19:ee(Pt);break;case 10:Mr(o.type);break;case 22:case 23:Cr(o),pp(),a!==null&&ee(ei);break;case 24:Mr(Ct)}}function Ku(a,o){try{var l=o.updateQueue,f=l!==null?l.lastEffect:null;if(f!==null){var p=f.next;l=p;do{if((l.tag&a)===a){f=void 0;var m=l.create,O=l.inst;f=m(),O.destroy=f}l=l.next}while(l!==p)}}catch(j){ot(o,o.return,j)}}function ca(a,o,l){try{var f=o.updateQueue,p=f!==null?f.lastEffect:null;if(p!==null){var m=p.next;f=m;do{if((f.tag&a)===a){var O=f.inst,j=O.destroy;if(j!==void 0){O.destroy=void 0,p=o;var k=l,Y=j;try{Y()}catch(ae){ot(p,k,ae)}}}f=f.next}while(f!==m)}}catch(ae){ot(o,o.return,ae)}}function ew(a){var o=a.updateQueue;if(o!==null){var l=a.stateNode;try{I1(o,l)}catch(f){ot(a,a.return,f)}}}function tw(a,o,l){l.props=ni(a.type,a.memoizedProps),l.state=a.memoizedState;try{l.componentWillUnmount()}catch(f){ot(a,o,f)}}function Wu(a,o){try{var l=a.ref;if(l!==null){switch(a.tag){case 26:case 27:case 5:var f=a.stateNode;break;case 30:f=a.stateNode;break;default:f=a.stateNode}typeof l=="function"?a.refCleanup=l(f):l.current=f}}catch(p){ot(a,o,p)}}function or(a,o){var l=a.ref,f=a.refCleanup;if(l!==null)if(typeof f=="function")try{f()}catch(p){ot(a,o,p)}finally{a.refCleanup=null,a=a.alternate,a!=null&&(a.refCleanup=null)}else if(typeof l=="function")try{l(null)}catch(p){ot(a,o,p)}else l.current=null}function nw(a){var o=a.type,l=a.memoizedProps,f=a.stateNode;try{e:switch(o){case"button":case"input":case"select":case"textarea":l.autoFocus&&f.focus();break e;case"img":l.src?f.src=l.src:l.srcSet&&(f.srcset=l.srcSet)}}catch(p){ot(a,a.return,p)}}function Up(a,o,l){try{var f=a.stateNode;$q(f,a.type,l,o),f[an]=o}catch(p){ot(a,a.return,p)}}function rw(a){return a.tag===5||a.tag===3||a.tag===26||a.tag===27&&ga(a.type)||a.tag===4}function Hp(a){e:for(;;){for(;a.sibling===null;){if(a.return===null||rw(a.return))return null;a=a.return}for(a.sibling.return=a.return,a=a.sibling;a.tag!==5&&a.tag!==6&&a.tag!==18;){if(a.tag===27&&ga(a.type)||a.flags&2||a.child===null||a.tag===4)continue e;a.child.return=a,a=a.child}if(!(a.flags&2))return a.stateNode}}function Gp(a,o,l){var f=a.tag;if(f===5||f===6)a=a.stateNode,o?(l.nodeType===9?l.body:l.nodeName==="HTML"?l.ownerDocument.body:l).insertBefore(a,o):(o=l.nodeType===9?l.body:l.nodeName==="HTML"?l.ownerDocument.body:l,o.appendChild(a),l=l._reactRootContainer,l!=null||o.onclick!==null||(o.onclick=Yc));else if(f!==4&&(f===27&&ga(a.type)&&(l=a.stateNode,o=null),a=a.child,a!==null))for(Gp(a,o,l),a=a.sibling;a!==null;)Gp(a,o,l),a=a.sibling}function qc(a,o,l){var f=a.tag;if(f===5||f===6)a=a.stateNode,o?l.insertBefore(a,o):l.appendChild(a);else if(f!==4&&(f===27&&ga(a.type)&&(l=a.stateNode),a=a.child,a!==null))for(qc(a,o,l),a=a.sibling;a!==null;)qc(a,o,l),a=a.sibling}function aw(a){var o=a.stateNode,l=a.memoizedProps;try{for(var f=a.type,p=o.attributes;p.length;)o.removeAttributeNode(p[0]);Gt(o,f,l),o[Yt]=a,o[an]=l}catch(m){ot(a,a.return,m)}}var Nr=!1,At=!1,Vp=!1,iw=typeof WeakSet=="function"?WeakSet:Set,Bt=null;function mq(a,o){if(a=a.containerInfo,vv=ef,a=y1(a),Hh(a)){if("selectionStart"in a)var l={start:a.selectionStart,end:a.selectionEnd};else e:{l=(l=a.ownerDocument)&&l.defaultView||window;var f=l.getSelection&&l.getSelection();if(f&&f.rangeCount!==0){l=f.anchorNode;var p=f.anchorOffset,m=f.focusNode;f=f.focusOffset;try{l.nodeType,m.nodeType}catch{l=null;break e}var O=0,j=-1,k=-1,Y=0,ae=0,oe=a,K=null;t:for(;;){for(var F;oe!==l||p!==0&&oe.nodeType!==3||(j=O+p),oe!==m||f!==0&&oe.nodeType!==3||(k=O+f),oe.nodeType===3&&(O+=oe.nodeValue.length),(F=oe.firstChild)!==null;)K=oe,oe=F;for(;;){if(oe===a)break t;if(K===l&&++Y===p&&(j=O),K===m&&++ae===f&&(k=O),(F=oe.nextSibling)!==null)break;oe=K,K=oe.parentNode}oe=F}l=j===-1||k===-1?null:{start:j,end:k}}else l=null}l=l||{start:0,end:0}}else l=null;for(yv={focusedElem:a,selectionRange:l},ef=!1,Bt=o;Bt!==null;)if(o=Bt,a=o.child,(o.subtreeFlags&1024)!==0&&a!==null)a.return=o,Bt=a;else for(;Bt!==null;){switch(o=Bt,m=o.alternate,a=o.flags,o.tag){case 0:break;case 11:case 15:break;case 1:if((a&1024)!==0&&m!==null){a=void 0,l=o,p=m.memoizedProps,m=m.memoizedState,f=l.stateNode;try{var je=ni(l.type,p,l.elementType===l.type);a=f.getSnapshotBeforeUpdate(je,m),f.__reactInternalSnapshotBeforeUpdate=a}catch(we){ot(l,l.return,we)}}break;case 3:if((a&1024)!==0){if(a=o.stateNode.containerInfo,l=a.nodeType,l===9)bv(a);else if(l===1)switch(a.nodeName){case"HEAD":case"HTML":case"BODY":bv(a);break;default:a.textContent=""}}break;case 5:case 26:case 27:case 6:case 4:case 17:break;default:if((a&1024)!==0)throw Error(r(163))}if(a=o.sibling,a!==null){a.return=o.return,Bt=a;break}Bt=o.return}}function ow(a,o,l){var f=l.flags;switch(l.tag){case 0:case 11:case 15:fa(a,l),f&4&&Ku(5,l);break;case 1:if(fa(a,l),f&4)if(a=l.stateNode,o===null)try{a.componentDidMount()}catch(O){ot(l,l.return,O)}else{var p=ni(l.type,o.memoizedProps);o=o.memoizedState;try{a.componentDidUpdate(p,o,a.__reactInternalSnapshotBeforeUpdate)}catch(O){ot(l,l.return,O)}}f&64&&ew(l),f&512&&Wu(l,l.return);break;case 3:if(fa(a,l),f&64&&(a=l.updateQueue,a!==null)){if(o=null,l.child!==null)switch(l.child.tag){case 27:case 5:o=l.child.stateNode;break;case 1:o=l.child.stateNode}try{I1(a,o)}catch(O){ot(l,l.return,O)}}break;case 27:o===null&&f&4&&aw(l);case 26:case 5:fa(a,l),o===null&&f&4&&nw(l),f&512&&Wu(l,l.return);break;case 12:fa(a,l);break;case 13:fa(a,l),f&4&&sw(a,l),f&64&&(a=l.memoizedState,a!==null&&(a=a.dehydrated,a!==null&&(l=Eq.bind(null,l),Gq(a,l))));break;case 22:if(f=l.memoizedState!==null||Nr,!f){o=o!==null&&o.memoizedState!==null||At,p=Nr;var m=At;Nr=f,(At=o)&&!m?da(a,l,(l.subtreeFlags&8772)!==0):fa(a,l),Nr=p,At=m}break;case 30:break;default:fa(a,l)}}function uw(a){var o=a.alternate;o!==null&&(a.alternate=null,uw(o)),a.child=null,a.deletions=null,a.sibling=null,a.tag===5&&(o=a.stateNode,o!==null&&Oh(o)),a.stateNode=null,a.return=null,a.dependencies=null,a.memoizedProps=null,a.memoizedState=null,a.pendingProps=null,a.stateNode=null,a.updateQueue=null}var pt=null,ln=!1;function Dr(a,o,l){for(l=l.child;l!==null;)lw(a,o,l),l=l.sibling}function lw(a,o,l){if(dn&&typeof dn.onCommitFiberUnmount=="function")try{dn.onCommitFiberUnmount(mu,l)}catch{}switch(l.tag){case 26:At||or(l,o),Dr(a,o,l),l.memoizedState?l.memoizedState.count--:l.stateNode&&(l=l.stateNode,l.parentNode.removeChild(l));break;case 27:At||or(l,o);var f=pt,p=ln;ga(l.type)&&(pt=l.stateNode,ln=!1),Dr(a,o,l),il(l.stateNode),pt=f,ln=p;break;case 5:At||or(l,o);case 6:if(f=pt,p=ln,pt=null,Dr(a,o,l),pt=f,ln=p,pt!==null)if(ln)try{(pt.nodeType===9?pt.body:pt.nodeName==="HTML"?pt.ownerDocument.body:pt).removeChild(l.stateNode)}catch(m){ot(l,o,m)}else try{pt.removeChild(l.stateNode)}catch(m){ot(l,o,m)}break;case 18:pt!==null&&(ln?(a=pt,Ww(a.nodeType===9?a.body:a.nodeName==="HTML"?a.ownerDocument.body:a,l.stateNode),hl(a)):Ww(pt,l.stateNode));break;case 4:f=pt,p=ln,pt=l.stateNode.containerInfo,ln=!0,Dr(a,o,l),pt=f,ln=p;break;case 0:case 11:case 14:case 15:At||ca(2,l,o),At||ca(4,l,o),Dr(a,o,l);break;case 1:At||(or(l,o),f=l.stateNode,typeof f.componentWillUnmount=="function"&&tw(l,o,f)),Dr(a,o,l);break;case 21:Dr(a,o,l);break;case 22:At=(f=At)||l.memoizedState!==null,Dr(a,o,l),At=f;break;default:Dr(a,o,l)}}function sw(a,o){if(o.memoizedState===null&&(a=o.alternate,a!==null&&(a=a.memoizedState,a!==null&&(a=a.dehydrated,a!==null))))try{hl(a)}catch(l){ot(o,o.return,l)}}function gq(a){switch(a.tag){case 13:case 19:var o=a.stateNode;return o===null&&(o=a.stateNode=new iw),o;case 22:return a=a.stateNode,o=a._retryCache,o===null&&(o=a._retryCache=new iw),o;default:throw Error(r(435,a.tag))}}function Zp(a,o){var l=gq(a);o.forEach(function(f){var p=Tq.bind(null,a,f);l.has(f)||(l.add(f),f.then(p,p))})}function yn(a,o){var l=o.deletions;if(l!==null)for(var f=0;f<l.length;f++){var p=l[f],m=a,O=o,j=O;e:for(;j!==null;){switch(j.tag){case 27:if(ga(j.type)){pt=j.stateNode,ln=!1;break e}break;case 5:pt=j.stateNode,ln=!1;break e;case 3:case 4:pt=j.stateNode.containerInfo,ln=!0;break e}j=j.return}if(pt===null)throw Error(r(160));lw(m,O,p),pt=null,ln=!1,m=p.alternate,m!==null&&(m.return=null),p.return=null}if(o.subtreeFlags&13878)for(o=o.child;o!==null;)cw(o,a),o=o.sibling}var Gn=null;function cw(a,o){var l=a.alternate,f=a.flags;switch(a.tag){case 0:case 11:case 14:case 15:yn(o,a),mn(a),f&4&&(ca(3,a,a.return),Ku(3,a),ca(5,a,a.return));break;case 1:yn(o,a),mn(a),f&512&&(At||l===null||or(l,l.return)),f&64&&Nr&&(a=a.updateQueue,a!==null&&(f=a.callbacks,f!==null&&(l=a.shared.hiddenCallbacks,a.shared.hiddenCallbacks=l===null?f:l.concat(f))));break;case 26:var p=Gn;if(yn(o,a),mn(a),f&512&&(At||l===null||or(l,l.return)),f&4){var m=l!==null?l.memoizedState:null;if(f=a.memoizedState,l===null)if(f===null)if(a.stateNode===null){e:{f=a.type,l=a.memoizedProps,p=p.ownerDocument||p;t:switch(f){case"title":m=p.getElementsByTagName("title")[0],(!m||m[xu]||m[Yt]||m.namespaceURI==="http://www.w3.org/2000/svg"||m.hasAttribute("itemprop"))&&(m=p.createElement(f),p.head.insertBefore(m,p.querySelector("head > title"))),Gt(m,f,l),m[Yt]=a,qt(m),f=m;break e;case"link":var O=aO("link","href",p).get(f+(l.href||""));if(O){for(var j=0;j<O.length;j++)if(m=O[j],m.getAttribute("href")===(l.href==null||l.href===""?null:l.href)&&m.getAttribute("rel")===(l.rel==null?null:l.rel)&&m.getAttribute("title")===(l.title==null?null:l.title)&&m.getAttribute("crossorigin")===(l.crossOrigin==null?null:l.crossOrigin)){O.splice(j,1);break t}}m=p.createElement(f),Gt(m,f,l),p.head.appendChild(m);break;case"meta":if(O=aO("meta","content",p).get(f+(l.content||""))){for(j=0;j<O.length;j++)if(m=O[j],m.getAttribute("content")===(l.content==null?null:""+l.content)&&m.getAttribute("name")===(l.name==null?null:l.name)&&m.getAttribute("property")===(l.property==null?null:l.property)&&m.getAttribute("http-equiv")===(l.httpEquiv==null?null:l.httpEquiv)&&m.getAttribute("charset")===(l.charSet==null?null:l.charSet)){O.splice(j,1);break t}}m=p.createElement(f),Gt(m,f,l),p.head.appendChild(m);break;default:throw Error(r(468,f))}m[Yt]=a,qt(m),f=m}a.stateNode=f}else iO(p,a.type,a.stateNode);else a.stateNode=rO(p,f,a.memoizedProps);else m!==f?(m===null?l.stateNode!==null&&(l=l.stateNode,l.parentNode.removeChild(l)):m.count--,f===null?iO(p,a.type,a.stateNode):rO(p,f,a.memoizedProps)):f===null&&a.stateNode!==null&&Up(a,a.memoizedProps,l.memoizedProps)}break;case 27:yn(o,a),mn(a),f&512&&(At||l===null||or(l,l.return)),l!==null&&f&4&&Up(a,a.memoizedProps,l.memoizedProps);break;case 5:if(yn(o,a),mn(a),f&512&&(At||l===null||or(l,l.return)),a.flags&32){p=a.stateNode;try{zi(p,"")}catch(F){ot(a,a.return,F)}}f&4&&a.stateNode!=null&&(p=a.memoizedProps,Up(a,p,l!==null?l.memoizedProps:p)),f&1024&&(Vp=!0);break;case 6:if(yn(o,a),mn(a),f&4){if(a.stateNode===null)throw Error(r(162));f=a.memoizedProps,l=a.stateNode;try{l.nodeValue=f}catch(F){ot(a,a.return,F)}}break;case 3:if(Fc=null,p=Gn,Gn=Kc(o.containerInfo),yn(o,a),Gn=p,mn(a),f&4&&l!==null&&l.memoizedState.isDehydrated)try{hl(o.containerInfo)}catch(F){ot(a,a.return,F)}Vp&&(Vp=!1,fw(a));break;case 4:f=Gn,Gn=Kc(a.stateNode.containerInfo),yn(o,a),mn(a),Gn=f;break;case 12:yn(o,a),mn(a);break;case 13:yn(o,a),mn(a),a.child.flags&8192&&a.memoizedState!==null!=(l!==null&&l.memoizedState!==null)&&(Qp=dt()),f&4&&(f=a.updateQueue,f!==null&&(a.updateQueue=null,Zp(a,f)));break;case 22:p=a.memoizedState!==null;var k=l!==null&&l.memoizedState!==null,Y=Nr,ae=At;if(Nr=Y||p,At=ae||k,yn(o,a),At=ae,Nr=Y,mn(a),f&8192)e:for(o=a.stateNode,o._visibility=p?o._visibility&-2:o._visibility|1,p&&(l===null||k||Nr||At||ri(a)),l=null,o=a;;){if(o.tag===5||o.tag===26){if(l===null){k=l=o;try{if(m=k.stateNode,p)O=m.style,typeof O.setProperty=="function"?O.setProperty("display","none","important"):O.display="none";else{j=k.stateNode;var oe=k.memoizedProps.style,K=oe!=null&&oe.hasOwnProperty("display")?oe.display:null;j.style.display=K==null||typeof K=="boolean"?"":(""+K).trim()}}catch(F){ot(k,k.return,F)}}}else if(o.tag===6){if(l===null){k=o;try{k.stateNode.nodeValue=p?"":k.memoizedProps}catch(F){ot(k,k.return,F)}}}else if((o.tag!==22&&o.tag!==23||o.memoizedState===null||o===a)&&o.child!==null){o.child.return=o,o=o.child;continue}if(o===a)break e;for(;o.sibling===null;){if(o.return===null||o.return===a)break e;l===o&&(l=null),o=o.return}l===o&&(l=null),o.sibling.return=o.return,o=o.sibling}f&4&&(f=a.updateQueue,f!==null&&(l=f.retryQueue,l!==null&&(f.retryQueue=null,Zp(a,l))));break;case 19:yn(o,a),mn(a),f&4&&(f=a.updateQueue,f!==null&&(a.updateQueue=null,Zp(a,f)));break;case 30:break;case 21:break;default:yn(o,a),mn(a)}}function mn(a){var o=a.flags;if(o&2){try{for(var l,f=a.return;f!==null;){if(rw(f)){l=f;break}f=f.return}if(l==null)throw Error(r(160));switch(l.tag){case 27:var p=l.stateNode,m=Hp(a);qc(a,m,p);break;case 5:var O=l.stateNode;l.flags&32&&(zi(O,""),l.flags&=-33);var j=Hp(a);qc(a,j,O);break;case 3:case 4:var k=l.stateNode.containerInfo,Y=Hp(a);Gp(a,Y,k);break;default:throw Error(r(161))}}catch(ae){ot(a,a.return,ae)}a.flags&=-3}o&4096&&(a.flags&=-4097)}function fw(a){if(a.subtreeFlags&1024)for(a=a.child;a!==null;){var o=a;fw(o),o.tag===5&&o.flags&1024&&o.stateNode.reset(),a=a.sibling}}function fa(a,o){if(o.subtreeFlags&8772)for(o=o.child;o!==null;)ow(a,o.alternate,o),o=o.sibling}function ri(a){for(a=a.child;a!==null;){var o=a;switch(o.tag){case 0:case 11:case 14:case 15:ca(4,o,o.return),ri(o);break;case 1:or(o,o.return);var l=o.stateNode;typeof l.componentWillUnmount=="function"&&tw(o,o.return,l),ri(o);break;case 27:il(o.stateNode);case 26:case 5:or(o,o.return),ri(o);break;case 22:o.memoizedState===null&&ri(o);break;case 30:ri(o);break;default:ri(o)}a=a.sibling}}function da(a,o,l){for(l=l&&(o.subtreeFlags&8772)!==0,o=o.child;o!==null;){var f=o.alternate,p=a,m=o,O=m.flags;switch(m.tag){case 0:case 11:case 15:da(p,m,l),Ku(4,m);break;case 1:if(da(p,m,l),f=m,p=f.stateNode,typeof p.componentDidMount=="function")try{p.componentDidMount()}catch(Y){ot(f,f.return,Y)}if(f=m,p=f.updateQueue,p!==null){var j=f.stateNode;try{var k=p.shared.hiddenCallbacks;if(k!==null)for(p.shared.hiddenCallbacks=null,p=0;p<k.length;p++)L1(k[p],j)}catch(Y){ot(f,f.return,Y)}}l&&O&64&&ew(m),Wu(m,m.return);break;case 27:aw(m);case 26:case 5:da(p,m,l),l&&f===null&&O&4&&nw(m),Wu(m,m.return);break;case 12:da(p,m,l);break;case 13:da(p,m,l),l&&O&4&&sw(p,m);break;case 22:m.memoizedState===null&&da(p,m,l),Wu(m,m.return);break;case 30:break;default:da(p,m,l)}o=o.sibling}}function Yp(a,o){var l=null;a!==null&&a.memoizedState!==null&&a.memoizedState.cachePool!==null&&(l=a.memoizedState.cachePool.pool),a=null,o.memoizedState!==null&&o.memoizedState.cachePool!==null&&(a=o.memoizedState.cachePool.pool),a!==l&&(a!=null&&a.refCount++,l!=null&&ku(l))}function Xp(a,o){a=null,o.alternate!==null&&(a=o.alternate.memoizedState.cache),o=o.memoizedState.cache,o!==a&&(o.refCount++,a!=null&&ku(a))}function ur(a,o,l,f){if(o.subtreeFlags&10256)for(o=o.child;o!==null;)dw(a,o,l,f),o=o.sibling}function dw(a,o,l,f){var p=o.flags;switch(o.tag){case 0:case 11:case 15:ur(a,o,l,f),p&2048&&Ku(9,o);break;case 1:ur(a,o,l,f);break;case 3:ur(a,o,l,f),p&2048&&(a=null,o.alternate!==null&&(a=o.alternate.memoizedState.cache),o=o.memoizedState.cache,o!==a&&(o.refCount++,a!=null&&ku(a)));break;case 12:if(p&2048){ur(a,o,l,f),a=o.stateNode;try{var m=o.memoizedProps,O=m.id,j=m.onPostCommit;typeof j=="function"&&j(O,o.alternate===null?"mount":"update",a.passiveEffectDuration,-0)}catch(k){ot(o,o.return,k)}}else ur(a,o,l,f);break;case 13:ur(a,o,l,f);break;case 23:break;case 22:m=o.stateNode,O=o.alternate,o.memoizedState!==null?m._visibility&2?ur(a,o,l,f):Fu(a,o):m._visibility&2?ur(a,o,l,f):(m._visibility|=2,ao(a,o,l,f,(o.subtreeFlags&10256)!==0)),p&2048&&Yp(O,o);break;case 24:ur(a,o,l,f),p&2048&&Xp(o.alternate,o);break;default:ur(a,o,l,f)}}function ao(a,o,l,f,p){for(p=p&&(o.subtreeFlags&10256)!==0,o=o.child;o!==null;){var m=a,O=o,j=l,k=f,Y=O.flags;switch(O.tag){case 0:case 11:case 15:ao(m,O,j,k,p),Ku(8,O);break;case 23:break;case 22:var ae=O.stateNode;O.memoizedState!==null?ae._visibility&2?ao(m,O,j,k,p):Fu(m,O):(ae._visibility|=2,ao(m,O,j,k,p)),p&&Y&2048&&Yp(O.alternate,O);break;case 24:ao(m,O,j,k,p),p&&Y&2048&&Xp(O.alternate,O);break;default:ao(m,O,j,k,p)}o=o.sibling}}function Fu(a,o){if(o.subtreeFlags&10256)for(o=o.child;o!==null;){var l=a,f=o,p=f.flags;switch(f.tag){case 22:Fu(l,f),p&2048&&Yp(f.alternate,f);break;case 24:Fu(l,f),p&2048&&Xp(f.alternate,f);break;default:Fu(l,f)}o=o.sibling}}var Qu=8192;function io(a){if(a.subtreeFlags&Qu)for(a=a.child;a!==null;)hw(a),a=a.sibling}function hw(a){switch(a.tag){case 26:io(a),a.flags&Qu&&a.memoizedState!==null&&r$(Gn,a.memoizedState,a.memoizedProps);break;case 5:io(a);break;case 3:case 4:var o=Gn;Gn=Kc(a.stateNode.containerInfo),io(a),Gn=o;break;case 22:a.memoizedState===null&&(o=a.alternate,o!==null&&o.memoizedState!==null?(o=Qu,Qu=16777216,io(a),Qu=o):io(a));break;default:io(a)}}function pw(a){var o=a.alternate;if(o!==null&&(a=o.child,a!==null)){o.child=null;do o=a.sibling,a.sibling=null,a=o;while(a!==null)}}function Ju(a){var o=a.deletions;if((a.flags&16)!==0){if(o!==null)for(var l=0;l<o.length;l++){var f=o[l];Bt=f,yw(f,a)}pw(a)}if(a.subtreeFlags&10256)for(a=a.child;a!==null;)vw(a),a=a.sibling}function vw(a){switch(a.tag){case 0:case 11:case 15:Ju(a),a.flags&2048&&ca(9,a,a.return);break;case 3:Ju(a);break;case 12:Ju(a);break;case 22:var o=a.stateNode;a.memoizedState!==null&&o._visibility&2&&(a.return===null||a.return.tag!==13)?(o._visibility&=-3,$c(a)):Ju(a);break;default:Ju(a)}}function $c(a){var o=a.deletions;if((a.flags&16)!==0){if(o!==null)for(var l=0;l<o.length;l++){var f=o[l];Bt=f,yw(f,a)}pw(a)}for(a=a.child;a!==null;){switch(o=a,o.tag){case 0:case 11:case 15:ca(8,o,o.return),$c(o);break;case 22:l=o.stateNode,l._visibility&2&&(l._visibility&=-3,$c(o));break;default:$c(o)}a=a.sibling}}function yw(a,o){for(;Bt!==null;){var l=Bt;switch(l.tag){case 0:case 11:case 15:ca(8,l,o);break;case 23:case 22:if(l.memoizedState!==null&&l.memoizedState.cachePool!==null){var f=l.memoizedState.cachePool.pool;f!=null&&f.refCount++}break;case 24:ku(l.memoizedState.cache)}if(f=l.child,f!==null)f.return=l,Bt=f;else e:for(l=a;Bt!==null;){f=Bt;var p=f.sibling,m=f.return;if(uw(f),f===l){Bt=null;break e}if(p!==null){p.return=m,Bt=p;break e}Bt=m}}}var bq={getCacheForType:function(a){var o=Xt(Ct),l=o.data.get(a);return l===void 0&&(l=a(),o.data.set(a,l)),l}},xq=typeof WeakMap=="function"?WeakMap:Map,et=0,ct=null,Ue=null,Ve=0,tt=0,gn=null,ha=!1,oo=!1,Kp=!1,kr=0,xt=0,pa=0,ai=0,Wp=0,Dn=0,uo=0,el=null,sn=null,Fp=!1,Qp=0,Bc=1/0,zc=null,va=null,Ht=0,ya=null,lo=null,so=0,Jp=0,ev=null,mw=null,tl=0,tv=null;function bn(){if((et&2)!==0&&Ve!==0)return Ve&-Ve;if(D.T!==null){var a=Wi;return a!==0?a:lv()}return N_()}function gw(){Dn===0&&(Dn=(Ve&536870912)===0||Fe?j_():536870912);var a=Nn.current;return a!==null&&(a.flags|=32),Dn}function xn(a,o,l){(a===ct&&(tt===2||tt===9)||a.cancelPendingCommit!==null)&&(co(a,0),ma(a,Ve,Dn,!1)),bu(a,l),((et&2)===0||a!==ct)&&(a===ct&&((et&2)===0&&(ai|=l),xt===4&&ma(a,Ve,Dn,!1)),lr(a))}function bw(a,o,l){if((et&6)!==0)throw Error(r(327));var f=!l&&(o&124)===0&&(o&a.expiredLanes)===0||gu(a,o),p=f?wq(a,o):av(a,o,!0),m=f;do{if(p===0){oo&&!f&&ma(a,o,0,!1);break}else{if(l=a.current.alternate,m&&!_q(l)){p=av(a,o,!1),m=!1;continue}if(p===2){if(m=o,a.errorRecoveryDisabledLanes&m)var O=0;else O=a.pendingLanes&-536870913,O=O!==0?O:O&536870912?536870912:0;if(O!==0){o=O;e:{var j=a;p=el;var k=j.current.memoizedState.isDehydrated;if(k&&(co(j,O).flags|=256),O=av(j,O,!1),O!==2){if(Kp&&!k){j.errorRecoveryDisabledLanes|=m,ai|=m,p=4;break e}m=sn,sn=p,m!==null&&(sn===null?sn=m:sn.push.apply(sn,m))}p=O}if(m=!1,p!==2)continue}}if(p===1){co(a,0),ma(a,o,0,!0);break}e:{switch(f=a,m=p,m){case 0:case 1:throw Error(r(345));case 4:if((o&4194048)!==o)break;case 6:ma(f,o,Dn,!ha);break e;case 2:sn=null;break;case 3:case 5:break;default:throw Error(r(329))}if((o&62914560)===o&&(p=Qp+300-dt(),10<p)){if(ma(f,o,Dn,!ha),Ws(f,0,!0)!==0)break e;f.timeoutHandle=Xw(xw.bind(null,f,l,sn,zc,Fp,o,Dn,ai,uo,ha,m,2,-0,0),p);break e}xw(f,l,sn,zc,Fp,o,Dn,ai,uo,ha,m,0,-0,0)}}break}while(!0);lr(a)}function xw(a,o,l,f,p,m,O,j,k,Y,ae,oe,K,F){if(a.timeoutHandle=-1,oe=o.subtreeFlags,(oe&8192||(oe&16785408)===16785408)&&(ll={stylesheets:null,count:0,unsuspend:n$},hw(o),oe=a$(),oe!==null)){a.cancelPendingCommit=oe(Tw.bind(null,a,o,m,l,f,p,O,j,k,ae,1,K,F)),ma(a,m,O,!Y);return}Tw(a,o,m,l,f,p,O,j,k)}function _q(a){for(var o=a;;){var l=o.tag;if((l===0||l===11||l===15)&&o.flags&16384&&(l=o.updateQueue,l!==null&&(l=l.stores,l!==null)))for(var f=0;f<l.length;f++){var p=l[f],m=p.getSnapshot;p=p.value;try{if(!pn(m(),p))return!1}catch{return!1}}if(l=o.child,o.subtreeFlags&16384&&l!==null)l.return=o,o=l;else{if(o===a)break;for(;o.sibling===null;){if(o.return===null||o.return===a)return!0;o=o.return}o.sibling.return=o.return,o=o.sibling}}return!0}function ma(a,o,l,f){o&=~Wp,o&=~ai,a.suspendedLanes|=o,a.pingedLanes&=~o,f&&(a.warmLanes|=o),f=a.expirationTimes;for(var p=o;0<p;){var m=31-hn(p),O=1<<m;f[m]=-1,p&=~O}l!==0&&P_(a,l,o)}function Lc(){return(et&6)===0?(nl(0),!1):!0}function nv(){if(Ue!==null){if(tt===0)var a=Ue.return;else a=Ue,Tr=Qa=null,bp(a),no=null,Zu=0,a=Ue;for(;a!==null;)JS(a.alternate,a),a=a.return;Ue=null}}function co(a,o){var l=a.timeoutHandle;l!==-1&&(a.timeoutHandle=-1,zq(l)),l=a.cancelPendingCommit,l!==null&&(a.cancelPendingCommit=null,l()),nv(),ct=a,Ue=l=Or(a.current,null),Ve=o,tt=0,gn=null,ha=!1,oo=gu(a,o),Kp=!1,uo=Dn=Wp=ai=pa=xt=0,sn=el=null,Fp=!1,(o&8)!==0&&(o|=o&32);var f=a.entangledLanes;if(f!==0)for(a=a.entanglements,f&=o;0<f;){var p=31-hn(f),m=1<<p;o|=a[p],f&=~m}return kr=o,lc(),l}function _w(a,o){$e=null,D.H=Tc,o===$u||o===mc?(o=B1(),tt=3):o===k1?(o=B1(),tt=4):tt=o===zS?8:o!==null&&typeof o=="object"&&typeof o.then=="function"?6:1,gn=o,Ue===null&&(xt=1,Rc(a,jn(o,a.current)))}function Sw(){var a=D.H;return D.H=Tc,a===null?Tc:a}function ww(){var a=D.A;return D.A=bq,a}function rv(){xt=4,ha||(Ve&4194048)!==Ve&&Nn.current!==null||(oo=!0),(pa&134217727)===0&&(ai&134217727)===0||ct===null||ma(ct,Ve,Dn,!1)}function av(a,o,l){var f=et;et|=2;var p=Sw(),m=ww();(ct!==a||Ve!==o)&&(zc=null,co(a,o)),o=!1;var O=xt;e:do try{if(tt!==0&&Ue!==null){var j=Ue,k=gn;switch(tt){case 8:nv(),O=6;break e;case 3:case 2:case 9:case 6:Nn.current===null&&(o=!0);var Y=tt;if(tt=0,gn=null,fo(a,j,k,Y),l&&oo){O=0;break e}break;default:Y=tt,tt=0,gn=null,fo(a,j,k,Y)}}Sq(),O=xt;break}catch(ae){_w(a,ae)}while(!0);return o&&a.shellSuspendCounter++,Tr=Qa=null,et=f,D.H=p,D.A=m,Ue===null&&(ct=null,Ve=0,lc()),O}function Sq(){for(;Ue!==null;)Ow(Ue)}function wq(a,o){var l=et;et|=2;var f=Sw(),p=ww();ct!==a||Ve!==o?(zc=null,Bc=dt()+500,co(a,o)):oo=gu(a,o);e:do try{if(tt!==0&&Ue!==null){o=Ue;var m=gn;t:switch(tt){case 1:tt=0,gn=null,fo(a,o,m,1);break;case 2:case 9:if(q1(m)){tt=0,gn=null,Aw(o);break}o=function(){tt!==2&&tt!==9||ct!==a||(tt=7),lr(a)},m.then(o,o);break e;case 3:tt=7;break e;case 4:tt=5;break e;case 7:q1(m)?(tt=0,gn=null,Aw(o)):(tt=0,gn=null,fo(a,o,m,7));break;case 5:var O=null;switch(Ue.tag){case 26:O=Ue.memoizedState;case 5:case 27:var j=Ue;if(!O||oO(O)){tt=0,gn=null;var k=j.sibling;if(k!==null)Ue=k;else{var Y=j.return;Y!==null?(Ue=Y,Ic(Y)):Ue=null}break t}}tt=0,gn=null,fo(a,o,m,5);break;case 6:tt=0,gn=null,fo(a,o,m,6);break;case 8:nv(),xt=6;break e;default:throw Error(r(462))}}Oq();break}catch(ae){_w(a,ae)}while(!0);return Tr=Qa=null,D.H=f,D.A=p,et=l,Ue!==null?0:(ct=null,Ve=0,lc(),xt)}function Oq(){for(;Ue!==null&&!St();)Ow(Ue)}function Ow(a){var o=FS(a.alternate,a,kr);a.memoizedProps=a.pendingProps,o===null?Ic(a):Ue=o}function Aw(a){var o=a,l=o.alternate;switch(o.tag){case 15:case 0:o=VS(l,o,o.pendingProps,o.type,void 0,Ve);break;case 11:o=VS(l,o,o.pendingProps,o.type.render,o.ref,Ve);break;case 5:bp(o);default:JS(l,o),o=Ue=E1(o,kr),o=FS(l,o,kr)}a.memoizedProps=a.pendingProps,o===null?Ic(a):Ue=o}function fo(a,o,l,f){Tr=Qa=null,bp(o),no=null,Zu=0;var p=o.return;try{if(hq(a,p,o,l,Ve)){xt=1,Rc(a,jn(l,a.current)),Ue=null;return}}catch(m){if(p!==null)throw Ue=p,m;xt=1,Rc(a,jn(l,a.current)),Ue=null;return}o.flags&32768?(Fe||f===1?a=!0:oo||(Ve&536870912)!==0?a=!1:(ha=a=!0,(f===2||f===9||f===3||f===6)&&(f=Nn.current,f!==null&&f.tag===13&&(f.flags|=16384))),Ew(o,a)):Ic(o)}function Ic(a){var o=a;do{if((o.flags&32768)!==0){Ew(o,ha);return}a=o.return;var l=vq(o.alternate,o,kr);if(l!==null){Ue=l;return}if(o=o.sibling,o!==null){Ue=o;return}Ue=o=a}while(o!==null);xt===0&&(xt=5)}function Ew(a,o){do{var l=yq(a.alternate,a);if(l!==null){l.flags&=32767,Ue=l;return}if(l=a.return,l!==null&&(l.flags|=32768,l.subtreeFlags=0,l.deletions=null),!o&&(a=a.sibling,a!==null)){Ue=a;return}Ue=a=l}while(a!==null);xt=6,Ue=null}function Tw(a,o,l,f,p,m,O,j,k){a.cancelPendingCommit=null;do Uc();while(Ht!==0);if((et&6)!==0)throw Error(r(327));if(o!==null){if(o===a.current)throw Error(r(177));if(m=o.lanes|o.childLanes,m|=Xh,nk(a,l,m,O,j,k),a===ct&&(Ue=ct=null,Ve=0),lo=o,ya=a,so=l,Jp=m,ev=p,mw=f,(o.subtreeFlags&10256)!==0||(o.flags&10256)!==0?(a.callbackNode=null,a.callbackPriority=0,Mq(En,function(){return Rw(),null})):(a.callbackNode=null,a.callbackPriority=0),f=(o.flags&13878)!==0,(o.subtreeFlags&13878)!==0||f){f=D.T,D.T=null,p=H.p,H.p=2,O=et,et|=4;try{mq(a,o,l)}finally{et=O,H.p=p,D.T=f}}Ht=1,Mw(),jw(),Cw()}}function Mw(){if(Ht===1){Ht=0;var a=ya,o=lo,l=(o.flags&13878)!==0;if((o.subtreeFlags&13878)!==0||l){l=D.T,D.T=null;var f=H.p;H.p=2;var p=et;et|=4;try{cw(o,a);var m=yv,O=y1(a.containerInfo),j=m.focusedElem,k=m.selectionRange;if(O!==j&&j&&j.ownerDocument&&v1(j.ownerDocument.documentElement,j)){if(k!==null&&Hh(j)){var Y=k.start,ae=k.end;if(ae===void 0&&(ae=Y),"selectionStart"in j)j.selectionStart=Y,j.selectionEnd=Math.min(ae,j.value.length);else{var oe=j.ownerDocument||document,K=oe&&oe.defaultView||window;if(K.getSelection){var F=K.getSelection(),je=j.textContent.length,we=Math.min(k.start,je),at=k.end===void 0?we:Math.min(k.end,je);!F.extend&&we>at&&(O=at,at=we,we=O);var I=p1(j,we),z=p1(j,at);if(I&&z&&(F.rangeCount!==1||F.anchorNode!==I.node||F.anchorOffset!==I.offset||F.focusNode!==z.node||F.focusOffset!==z.offset)){var Z=oe.createRange();Z.setStart(I.node,I.offset),F.removeAllRanges(),we>at?(F.addRange(Z),F.extend(z.node,z.offset)):(Z.setEnd(z.node,z.offset),F.addRange(Z))}}}}for(oe=[],F=j;F=F.parentNode;)F.nodeType===1&&oe.push({element:F,left:F.scrollLeft,top:F.scrollTop});for(typeof j.focus=="function"&&j.focus(),j=0;j<oe.length;j++){var ie=oe[j];ie.element.scrollLeft=ie.left,ie.element.scrollTop=ie.top}}ef=!!vv,yv=vv=null}finally{et=p,H.p=f,D.T=l}}a.current=o,Ht=2}}function jw(){if(Ht===2){Ht=0;var a=ya,o=lo,l=(o.flags&8772)!==0;if((o.subtreeFlags&8772)!==0||l){l=D.T,D.T=null;var f=H.p;H.p=2;var p=et;et|=4;try{ow(a,o.alternate,o)}finally{et=p,H.p=f,D.T=l}}Ht=3}}function Cw(){if(Ht===4||Ht===3){Ht=0,wt();var a=ya,o=lo,l=so,f=mw;(o.subtreeFlags&10256)!==0||(o.flags&10256)!==0?Ht=5:(Ht=0,lo=ya=null,Pw(a,a.pendingLanes));var p=a.pendingLanes;if(p===0&&(va=null),Sh(l),o=o.stateNode,dn&&typeof dn.onCommitFiberRoot=="function")try{dn.onCommitFiberRoot(mu,o,void 0,(o.current.flags&128)===128)}catch{}if(f!==null){o=D.T,p=H.p,H.p=2,D.T=null;try{for(var m=a.onRecoverableError,O=0;O<f.length;O++){var j=f[O];m(j.value,{componentStack:j.stack})}}finally{D.T=o,H.p=p}}(so&3)!==0&&Uc(),lr(a),p=a.pendingLanes,(l&4194090)!==0&&(p&42)!==0?a===tv?tl++:(tl=0,tv=a):tl=0,nl(0)}}function Pw(a,o){(a.pooledCacheLanes&=o)===0&&(o=a.pooledCache,o!=null&&(a.pooledCache=null,ku(o)))}function Uc(a){return Mw(),jw(),Cw(),Rw()}function Rw(){if(Ht!==5)return!1;var a=ya,o=Jp;Jp=0;var l=Sh(so),f=D.T,p=H.p;try{H.p=32>l?32:l,D.T=null,l=ev,ev=null;var m=ya,O=so;if(Ht=0,lo=ya=null,so=0,(et&6)!==0)throw Error(r(331));var j=et;if(et|=4,vw(m.current),dw(m,m.current,O,l),et=j,nl(0,!1),dn&&typeof dn.onPostCommitFiberRoot=="function")try{dn.onPostCommitFiberRoot(mu,m)}catch{}return!0}finally{H.p=p,D.T=f,Pw(a,o)}}function Nw(a,o,l){o=jn(l,o),o=Np(a.stateNode,o,2),a=oa(a,o,2),a!==null&&(bu(a,2),lr(a))}function ot(a,o,l){if(a.tag===3)Nw(a,a,l);else for(;o!==null;){if(o.tag===3){Nw(o,a,l);break}else if(o.tag===1){var f=o.stateNode;if(typeof o.type.getDerivedStateFromError=="function"||typeof f.componentDidCatch=="function"&&(va===null||!va.has(f))){a=jn(l,a),l=$S(2),f=oa(o,l,2),f!==null&&(BS(l,f,o,a),bu(f,2),lr(f));break}}o=o.return}}function iv(a,o,l){var f=a.pingCache;if(f===null){f=a.pingCache=new xq;var p=new Set;f.set(o,p)}else p=f.get(o),p===void 0&&(p=new Set,f.set(o,p));p.has(l)||(Kp=!0,p.add(l),a=Aq.bind(null,a,o,l),o.then(a,a))}function Aq(a,o,l){var f=a.pingCache;f!==null&&f.delete(o),a.pingedLanes|=a.suspendedLanes&l,a.warmLanes&=~l,ct===a&&(Ve&l)===l&&(xt===4||xt===3&&(Ve&62914560)===Ve&&300>dt()-Qp?(et&2)===0&&co(a,0):Wp|=l,uo===Ve&&(uo=0)),lr(a)}function Dw(a,o){o===0&&(o=C_()),a=Zi(a,o),a!==null&&(bu(a,o),lr(a))}function Eq(a){var o=a.memoizedState,l=0;o!==null&&(l=o.retryLane),Dw(a,l)}function Tq(a,o){var l=0;switch(a.tag){case 13:var f=a.stateNode,p=a.memoizedState;p!==null&&(l=p.retryLane);break;case 19:f=a.stateNode;break;case 22:f=a.stateNode._retryCache;break;default:throw Error(r(314))}f!==null&&f.delete(o),Dw(a,l)}function Mq(a,o){return st(a,o)}var Hc=null,ho=null,ov=!1,Gc=!1,uv=!1,ii=0;function lr(a){a!==ho&&a.next===null&&(ho===null?Hc=ho=a:ho=ho.next=a),Gc=!0,ov||(ov=!0,Cq())}function nl(a,o){if(!uv&&Gc){uv=!0;do for(var l=!1,f=Hc;f!==null;){if(a!==0){var p=f.pendingLanes;if(p===0)var m=0;else{var O=f.suspendedLanes,j=f.pingedLanes;m=(1<<31-hn(42|a)+1)-1,m&=p&~(O&~j),m=m&201326741?m&201326741|1:m?m|2:0}m!==0&&(l=!0,Bw(f,m))}else m=Ve,m=Ws(f,f===ct?m:0,f.cancelPendingCommit!==null||f.timeoutHandle!==-1),(m&3)===0||gu(f,m)||(l=!0,Bw(f,m));f=f.next}while(l);uv=!1}}function jq(){kw()}function kw(){Gc=ov=!1;var a=0;ii!==0&&(Bq()&&(a=ii),ii=0);for(var o=dt(),l=null,f=Hc;f!==null;){var p=f.next,m=qw(f,o);m===0?(f.next=null,l===null?Hc=p:l.next=p,p===null&&(ho=l)):(l=f,(a!==0||(m&3)!==0)&&(Gc=!0)),f=p}nl(a)}function qw(a,o){for(var l=a.suspendedLanes,f=a.pingedLanes,p=a.expirationTimes,m=a.pendingLanes&-62914561;0<m;){var O=31-hn(m),j=1<<O,k=p[O];k===-1?((j&l)===0||(j&f)!==0)&&(p[O]=tk(j,o)):k<=o&&(a.expiredLanes|=j),m&=~j}if(o=ct,l=Ve,l=Ws(a,a===o?l:0,a.cancelPendingCommit!==null||a.timeoutHandle!==-1),f=a.callbackNode,l===0||a===o&&(tt===2||tt===9)||a.cancelPendingCommit!==null)return f!==null&&f!==null&&Ie(f),a.callbackNode=null,a.callbackPriority=0;if((l&3)===0||gu(a,l)){if(o=l&-l,o===a.callbackPriority)return o;switch(f!==null&&Ie(f),Sh(l)){case 2:case 8:l=Ia;break;case 32:l=En;break;case 268435456:l=Pi;break;default:l=En}return f=$w.bind(null,a),l=st(l,f),a.callbackPriority=o,a.callbackNode=l,o}return f!==null&&f!==null&&Ie(f),a.callbackPriority=2,a.callbackNode=null,2}function $w(a,o){if(Ht!==0&&Ht!==5)return a.callbackNode=null,a.callbackPriority=0,null;var l=a.callbackNode;if(Uc()&&a.callbackNode!==l)return null;var f=Ve;return f=Ws(a,a===ct?f:0,a.cancelPendingCommit!==null||a.timeoutHandle!==-1),f===0?null:(bw(a,f,o),qw(a,dt()),a.callbackNode!=null&&a.callbackNode===l?$w.bind(null,a):null)}function Bw(a,o){if(Uc())return null;bw(a,o,!0)}function Cq(){Lq(function(){(et&6)!==0?st(xr,jq):kw()})}function lv(){return ii===0&&(ii=j_()),ii}function zw(a){return a==null||typeof a=="symbol"||typeof a=="boolean"?null:typeof a=="function"?a:tc(""+a)}function Lw(a,o){var l=o.ownerDocument.createElement("input");return l.name=o.name,l.value=o.value,a.id&&l.setAttribute("form",a.id),o.parentNode.insertBefore(l,o),a=new FormData(a),l.parentNode.removeChild(l),a}function Pq(a,o,l,f,p){if(o==="submit"&&l&&l.stateNode===p){var m=zw((p[an]||null).action),O=f.submitter;O&&(o=(o=O[an]||null)?zw(o.formAction):O.getAttribute("formAction"),o!==null&&(m=o,O=null));var j=new ic("action","action",null,f,p);a.push({event:j,listeners:[{instance:null,listener:function(){if(f.defaultPrevented){if(ii!==0){var k=O?Lw(p,O):new FormData(p);Mp(l,{pending:!0,data:k,method:p.method,action:m},null,k)}}else typeof m=="function"&&(j.preventDefault(),k=O?Lw(p,O):new FormData(p),Mp(l,{pending:!0,data:k,method:p.method,action:m},m,k))},currentTarget:p}]})}}for(var sv=0;sv<Yh.length;sv++){var cv=Yh[sv],Rq=cv.toLowerCase(),Nq=cv[0].toUpperCase()+cv.slice(1);Hn(Rq,"on"+Nq)}Hn(b1,"onAnimationEnd"),Hn(x1,"onAnimationIteration"),Hn(_1,"onAnimationStart"),Hn("dblclick","onDoubleClick"),Hn("focusin","onFocus"),Hn("focusout","onBlur"),Hn(Wk,"onTransitionRun"),Hn(Fk,"onTransitionStart"),Hn(Qk,"onTransitionCancel"),Hn(S1,"onTransitionEnd"),qi("onMouseEnter",["mouseout","mouseover"]),qi("onMouseLeave",["mouseout","mouseover"]),qi("onPointerEnter",["pointerout","pointerover"]),qi("onPointerLeave",["pointerout","pointerover"]),Ha("onChange","change click focusin focusout input keydown keyup selectionchange".split(" ")),Ha("onSelect","focusout contextmenu dragend focusin keydown keyup mousedown mouseup selectionchange".split(" ")),Ha("onBeforeInput",["compositionend","keypress","textInput","paste"]),Ha("onCompositionEnd","compositionend focusout keydown keypress keyup mousedown".split(" ")),Ha("onCompositionStart","compositionstart focusout keydown keypress keyup mousedown".split(" ")),Ha("onCompositionUpdate","compositionupdate focusout keydown keypress keyup mousedown".split(" "));var rl="abort canplay canplaythrough durationchange emptied encrypted ended error loadeddata loadedmetadata loadstart pause play playing progress ratechange resize seeked seeking stalled suspend timeupdate volumechange waiting".split(" "),Dq=new Set("beforetoggle cancel close invalid load scroll scrollend toggle".split(" ").concat(rl));function Iw(a,o){o=(o&4)!==0;for(var l=0;l<a.length;l++){var f=a[l],p=f.event;f=f.listeners;e:{var m=void 0;if(o)for(var O=f.length-1;0<=O;O--){var j=f[O],k=j.instance,Y=j.currentTarget;if(j=j.listener,k!==m&&p.isPropagationStopped())break e;m=j,p.currentTarget=Y;try{m(p)}catch(ae){Pc(ae)}p.currentTarget=null,m=k}else for(O=0;O<f.length;O++){if(j=f[O],k=j.instance,Y=j.currentTarget,j=j.listener,k!==m&&p.isPropagationStopped())break e;m=j,p.currentTarget=Y;try{m(p)}catch(ae){Pc(ae)}p.currentTarget=null,m=k}}}}function He(a,o){var l=o[wh];l===void 0&&(l=o[wh]=new Set);var f=a+"__bubble";l.has(f)||(Uw(o,a,2,!1),l.add(f))}function fv(a,o,l){var f=0;o&&(f|=4),Uw(l,a,f,o)}var Vc="_reactListening"+Math.random().toString(36).slice(2);function dv(a){if(!a[Vc]){a[Vc]=!0,k_.forEach(function(l){l!=="selectionchange"&&(Dq.has(l)||fv(l,!1,a),fv(l,!0,a))});var o=a.nodeType===9?a:a.ownerDocument;o===null||o[Vc]||(o[Vc]=!0,fv("selectionchange",!1,o))}}function Uw(a,o,l,f){switch(dO(o)){case 2:var p=u$;break;case 8:p=l$;break;default:p=Ev}l=p.bind(null,o,l,a),p=void 0,!Dh||o!=="touchstart"&&o!=="touchmove"&&o!=="wheel"||(p=!0),f?p!==void 0?a.addEventListener(o,l,{capture:!0,passive:p}):a.addEventListener(o,l,!0):p!==void 0?a.addEventListener(o,l,{passive:p}):a.addEventListener(o,l,!1)}function hv(a,o,l,f,p){var m=f;if((o&1)===0&&(o&2)===0&&f!==null)e:for(;;){if(f===null)return;var O=f.tag;if(O===3||O===4){var j=f.stateNode.containerInfo;if(j===p)break;if(O===4)for(O=f.return;O!==null;){var k=O.tag;if((k===3||k===4)&&O.stateNode.containerInfo===p)return;O=O.return}for(;j!==null;){if(O=Ni(j),O===null)return;if(k=O.tag,k===5||k===6||k===26||k===27){f=m=O;continue e}j=j.parentNode}}f=f.return}K_(function(){var Y=m,ae=Rh(l),oe=[];e:{var K=w1.get(a);if(K!==void 0){var F=ic,je=a;switch(a){case"keypress":if(rc(l)===0)break e;case"keydown":case"keyup":F=Mk;break;case"focusin":je="focus",F=Bh;break;case"focusout":je="blur",F=Bh;break;case"beforeblur":case"afterblur":F=Bh;break;case"click":if(l.button===2)break e;case"auxclick":case"dblclick":case"mousedown":case"mousemove":case"mouseup":case"mouseout":case"mouseover":case"contextmenu":F=Q_;break;case"drag":case"dragend":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"dragstart":case"drop":F=yk;break;case"touchcancel":case"touchend":case"touchmove":case"touchstart":F=Pk;break;case b1:case x1:case _1:F=bk;break;case S1:F=Nk;break;case"scroll":case"scrollend":F=pk;break;case"wheel":F=kk;break;case"copy":case"cut":case"paste":F=_k;break;case"gotpointercapture":case"lostpointercapture":case"pointercancel":case"pointerdown":case"pointermove":case"pointerout":case"pointerover":case"pointerup":F=e1;break;case"toggle":case"beforetoggle":F=$k}var we=(o&4)!==0,at=!we&&(a==="scroll"||a==="scrollend"),I=we?K!==null?K+"Capture":null:K;we=[];for(var z=Y,Z;z!==null;){var ie=z;if(Z=ie.stateNode,ie=ie.tag,ie!==5&&ie!==26&&ie!==27||Z===null||I===null||(ie=Su(z,I),ie!=null&&we.push(al(z,ie,Z))),at)break;z=z.return}0<we.length&&(K=new F(K,je,null,l,ae),oe.push({event:K,listeners:we}))}}if((o&7)===0){e:{if(K=a==="mouseover"||a==="pointerover",F=a==="mouseout"||a==="pointerout",K&&l!==Ph&&(je=l.relatedTarget||l.fromElement)&&(Ni(je)||je[Ri]))break e;if((F||K)&&(K=ae.window===ae?ae:(K=ae.ownerDocument)?K.defaultView||K.parentWindow:window,F?(je=l.relatedTarget||l.toElement,F=Y,je=je?Ni(je):null,je!==null&&(at=u(je),we=je.tag,je!==at||we!==5&&we!==27&&we!==6)&&(je=null)):(F=null,je=Y),F!==je)){if(we=Q_,ie="onMouseLeave",I="onMouseEnter",z="mouse",(a==="pointerout"||a==="pointerover")&&(we=e1,ie="onPointerLeave",I="onPointerEnter",z="pointer"),at=F==null?K:_u(F),Z=je==null?K:_u(je),K=new we(ie,z+"leave",F,l,ae),K.target=at,K.relatedTarget=Z,ie=null,Ni(ae)===Y&&(we=new we(I,z+"enter",je,l,ae),we.target=Z,we.relatedTarget=at,ie=we),at=ie,F&&je)t:{for(we=F,I=je,z=0,Z=we;Z;Z=po(Z))z++;for(Z=0,ie=I;ie;ie=po(ie))Z++;for(;0<z-Z;)we=po(we),z--;for(;0<Z-z;)I=po(I),Z--;for(;z--;){if(we===I||I!==null&&we===I.alternate)break t;we=po(we),I=po(I)}we=null}else we=null;F!==null&&Hw(oe,K,F,we,!1),je!==null&&at!==null&&Hw(oe,at,je,we,!0)}}e:{if(K=Y?_u(Y):window,F=K.nodeName&&K.nodeName.toLowerCase(),F==="select"||F==="input"&&K.type==="file")var ye=l1;else if(o1(K))if(s1)ye=Yk;else{ye=Vk;var Le=Gk}else F=K.nodeName,!F||F.toLowerCase()!=="input"||K.type!=="checkbox"&&K.type!=="radio"?Y&&Ch(Y.elementType)&&(ye=l1):ye=Zk;if(ye&&(ye=ye(a,Y))){u1(oe,ye,l,ae);break e}Le&&Le(a,K,Y),a==="focusout"&&Y&&K.type==="number"&&Y.memoizedProps.value!=null&&jh(K,"number",K.value)}switch(Le=Y?_u(Y):window,a){case"focusin":(o1(Le)||Le.contentEditable==="true")&&(Hi=Le,Gh=Y,Cu=null);break;case"focusout":Cu=Gh=Hi=null;break;case"mousedown":Vh=!0;break;case"contextmenu":case"mouseup":case"dragend":Vh=!1,m1(oe,l,ae);break;case"selectionchange":if(Kk)break;case"keydown":case"keyup":m1(oe,l,ae)}var Se;if(Lh)e:{switch(a){case"compositionstart":var Ae="onCompositionStart";break e;case"compositionend":Ae="onCompositionEnd";break e;case"compositionupdate":Ae="onCompositionUpdate";break e}Ae=void 0}else Ui?a1(a,l)&&(Ae="onCompositionEnd"):a==="keydown"&&l.keyCode===229&&(Ae="onCompositionStart");Ae&&(t1&&l.locale!=="ko"&&(Ui||Ae!=="onCompositionStart"?Ae==="onCompositionEnd"&&Ui&&(Se=W_()):(na=ae,kh="value"in na?na.value:na.textContent,Ui=!0)),Le=Zc(Y,Ae),0<Le.length&&(Ae=new J_(Ae,a,null,l,ae),oe.push({event:Ae,listeners:Le}),Se?Ae.data=Se:(Se=i1(l),Se!==null&&(Ae.data=Se)))),(Se=zk?Lk(a,l):Ik(a,l))&&(Ae=Zc(Y,"onBeforeInput"),0<Ae.length&&(Le=new J_("onBeforeInput","beforeinput",null,l,ae),oe.push({event:Le,listeners:Ae}),Le.data=Se)),Pq(oe,a,Y,l,ae)}Iw(oe,o)})}function al(a,o,l){return{instance:a,listener:o,currentTarget:l}}function Zc(a,o){for(var l=o+"Capture",f=[];a!==null;){var p=a,m=p.stateNode;if(p=p.tag,p!==5&&p!==26&&p!==27||m===null||(p=Su(a,l),p!=null&&f.unshift(al(a,p,m)),p=Su(a,o),p!=null&&f.push(al(a,p,m))),a.tag===3)return f;a=a.return}return[]}function po(a){if(a===null)return null;do a=a.return;while(a&&a.tag!==5&&a.tag!==27);return a||null}function Hw(a,o,l,f,p){for(var m=o._reactName,O=[];l!==null&&l!==f;){var j=l,k=j.alternate,Y=j.stateNode;if(j=j.tag,k!==null&&k===f)break;j!==5&&j!==26&&j!==27||Y===null||(k=Y,p?(Y=Su(l,m),Y!=null&&O.unshift(al(l,Y,k))):p||(Y=Su(l,m),Y!=null&&O.push(al(l,Y,k)))),l=l.return}O.length!==0&&a.push({event:o,listeners:O})}var kq=/\r\n?/g,qq=/\u0000|\uFFFD/g;function Gw(a){return(typeof a=="string"?a:""+a).replace(kq,`
`).replace(qq,"")}function Vw(a,o){return o=Gw(o),Gw(a)===o}function Yc(){}function rt(a,o,l,f,p,m){switch(l){case"children":typeof f=="string"?o==="body"||o==="textarea"&&f===""||zi(a,f):(typeof f=="number"||typeof f=="bigint")&&o!=="body"&&zi(a,""+f);break;case"className":Qs(a,"class",f);break;case"tabIndex":Qs(a,"tabindex",f);break;case"dir":case"role":case"viewBox":case"width":case"height":Qs(a,l,f);break;case"style":Y_(a,f,m);break;case"data":if(o!=="object"){Qs(a,"data",f);break}case"src":case"href":if(f===""&&(o!=="a"||l!=="href")){a.removeAttribute(l);break}if(f==null||typeof f=="function"||typeof f=="symbol"||typeof f=="boolean"){a.removeAttribute(l);break}f=tc(""+f),a.setAttribute(l,f);break;case"action":case"formAction":if(typeof f=="function"){a.setAttribute(l,"javascript:throw new Error('A React form was unexpectedly submitted. If you called form.submit() manually, consider using form.requestSubmit() instead. If you\\'re trying to use event.stopPropagation() in a submit event handler, consider also calling event.preventDefault().')");break}else typeof m=="function"&&(l==="formAction"?(o!=="input"&&rt(a,o,"name",p.name,p,null),rt(a,o,"formEncType",p.formEncType,p,null),rt(a,o,"formMethod",p.formMethod,p,null),rt(a,o,"formTarget",p.formTarget,p,null)):(rt(a,o,"encType",p.encType,p,null),rt(a,o,"method",p.method,p,null),rt(a,o,"target",p.target,p,null)));if(f==null||typeof f=="symbol"||typeof f=="boolean"){a.removeAttribute(l);break}f=tc(""+f),a.setAttribute(l,f);break;case"onClick":f!=null&&(a.onclick=Yc);break;case"onScroll":f!=null&&He("scroll",a);break;case"onScrollEnd":f!=null&&He("scrollend",a);break;case"dangerouslySetInnerHTML":if(f!=null){if(typeof f!="object"||!("__html"in f))throw Error(r(61));if(l=f.__html,l!=null){if(p.children!=null)throw Error(r(60));a.innerHTML=l}}break;case"multiple":a.multiple=f&&typeof f!="function"&&typeof f!="symbol";break;case"muted":a.muted=f&&typeof f!="function"&&typeof f!="symbol";break;case"suppressContentEditableWarning":case"suppressHydrationWarning":case"defaultValue":case"defaultChecked":case"innerHTML":case"ref":break;case"autoFocus":break;case"xlinkHref":if(f==null||typeof f=="function"||typeof f=="boolean"||typeof f=="symbol"){a.removeAttribute("xlink:href");break}l=tc(""+f),a.setAttributeNS("http://www.w3.org/1999/xlink","xlink:href",l);break;case"contentEditable":case"spellCheck":case"draggable":case"value":case"autoReverse":case"externalResourcesRequired":case"focusable":case"preserveAlpha":f!=null&&typeof f!="function"&&typeof f!="symbol"?a.setAttribute(l,""+f):a.removeAttribute(l);break;case"inert":case"allowFullScreen":case"async":case"autoPlay":case"controls":case"default":case"defer":case"disabled":case"disablePictureInPicture":case"disableRemotePlayback":case"formNoValidate":case"hidden":case"loop":case"noModule":case"noValidate":case"open":case"playsInline":case"readOnly":case"required":case"reversed":case"scoped":case"seamless":case"itemScope":f&&typeof f!="function"&&typeof f!="symbol"?a.setAttribute(l,""):a.removeAttribute(l);break;case"capture":case"download":f===!0?a.setAttribute(l,""):f!==!1&&f!=null&&typeof f!="function"&&typeof f!="symbol"?a.setAttribute(l,f):a.removeAttribute(l);break;case"cols":case"rows":case"size":case"span":f!=null&&typeof f!="function"&&typeof f!="symbol"&&!isNaN(f)&&1<=f?a.setAttribute(l,f):a.removeAttribute(l);break;case"rowSpan":case"start":f==null||typeof f=="function"||typeof f=="symbol"||isNaN(f)?a.removeAttribute(l):a.setAttribute(l,f);break;case"popover":He("beforetoggle",a),He("toggle",a),Fs(a,"popover",f);break;case"xlinkActuate":Sr(a,"http://www.w3.org/1999/xlink","xlink:actuate",f);break;case"xlinkArcrole":Sr(a,"http://www.w3.org/1999/xlink","xlink:arcrole",f);break;case"xlinkRole":Sr(a,"http://www.w3.org/1999/xlink","xlink:role",f);break;case"xlinkShow":Sr(a,"http://www.w3.org/1999/xlink","xlink:show",f);break;case"xlinkTitle":Sr(a,"http://www.w3.org/1999/xlink","xlink:title",f);break;case"xlinkType":Sr(a,"http://www.w3.org/1999/xlink","xlink:type",f);break;case"xmlBase":Sr(a,"http://www.w3.org/XML/1998/namespace","xml:base",f);break;case"xmlLang":Sr(a,"http://www.w3.org/XML/1998/namespace","xml:lang",f);break;case"xmlSpace":Sr(a,"http://www.w3.org/XML/1998/namespace","xml:space",f);break;case"is":Fs(a,"is",f);break;case"innerText":case"textContent":break;default:(!(2<l.length)||l[0]!=="o"&&l[0]!=="O"||l[1]!=="n"&&l[1]!=="N")&&(l=dk.get(l)||l,Fs(a,l,f))}}function pv(a,o,l,f,p,m){switch(l){case"style":Y_(a,f,m);break;case"dangerouslySetInnerHTML":if(f!=null){if(typeof f!="object"||!("__html"in f))throw Error(r(61));if(l=f.__html,l!=null){if(p.children!=null)throw Error(r(60));a.innerHTML=l}}break;case"children":typeof f=="string"?zi(a,f):(typeof f=="number"||typeof f=="bigint")&&zi(a,""+f);break;case"onScroll":f!=null&&He("scroll",a);break;case"onScrollEnd":f!=null&&He("scrollend",a);break;case"onClick":f!=null&&(a.onclick=Yc);break;case"suppressContentEditableWarning":case"suppressHydrationWarning":case"innerHTML":case"ref":break;case"innerText":case"textContent":break;default:if(!q_.hasOwnProperty(l))e:{if(l[0]==="o"&&l[1]==="n"&&(p=l.endsWith("Capture"),o=l.slice(2,p?l.length-7:void 0),m=a[an]||null,m=m!=null?m[l]:null,typeof m=="function"&&a.removeEventListener(o,m,p),typeof f=="function")){typeof m!="function"&&m!==null&&(l in a?a[l]=null:a.hasAttribute(l)&&a.removeAttribute(l)),a.addEventListener(o,f,p);break e}l in a?a[l]=f:f===!0?a.setAttribute(l,""):Fs(a,l,f)}}}function Gt(a,o,l){switch(o){case"div":case"span":case"svg":case"path":case"a":case"g":case"p":case"li":break;case"img":He("error",a),He("load",a);var f=!1,p=!1,m;for(m in l)if(l.hasOwnProperty(m)){var O=l[m];if(O!=null)switch(m){case"src":f=!0;break;case"srcSet":p=!0;break;case"children":case"dangerouslySetInnerHTML":throw Error(r(137,o));default:rt(a,o,m,O,l,null)}}p&&rt(a,o,"srcSet",l.srcSet,l,null),f&&rt(a,o,"src",l.src,l,null);return;case"input":He("invalid",a);var j=m=O=p=null,k=null,Y=null;for(f in l)if(l.hasOwnProperty(f)){var ae=l[f];if(ae!=null)switch(f){case"name":p=ae;break;case"type":O=ae;break;case"checked":k=ae;break;case"defaultChecked":Y=ae;break;case"value":m=ae;break;case"defaultValue":j=ae;break;case"children":case"dangerouslySetInnerHTML":if(ae!=null)throw Error(r(137,o));break;default:rt(a,o,f,ae,l,null)}}H_(a,m,j,k,Y,O,p,!1),Js(a);return;case"select":He("invalid",a),f=O=m=null;for(p in l)if(l.hasOwnProperty(p)&&(j=l[p],j!=null))switch(p){case"value":m=j;break;case"defaultValue":O=j;break;case"multiple":f=j;default:rt(a,o,p,j,l,null)}o=m,l=O,a.multiple=!!f,o!=null?Bi(a,!!f,o,!1):l!=null&&Bi(a,!!f,l,!0);return;case"textarea":He("invalid",a),m=p=f=null;for(O in l)if(l.hasOwnProperty(O)&&(j=l[O],j!=null))switch(O){case"value":f=j;break;case"defaultValue":p=j;break;case"children":m=j;break;case"dangerouslySetInnerHTML":if(j!=null)throw Error(r(91));break;default:rt(a,o,O,j,l,null)}V_(a,f,p,m),Js(a);return;case"option":for(k in l)if(l.hasOwnProperty(k)&&(f=l[k],f!=null))switch(k){case"selected":a.selected=f&&typeof f!="function"&&typeof f!="symbol";break;default:rt(a,o,k,f,l,null)}return;case"dialog":He("beforetoggle",a),He("toggle",a),He("cancel",a),He("close",a);break;case"iframe":case"object":He("load",a);break;case"video":case"audio":for(f=0;f<rl.length;f++)He(rl[f],a);break;case"image":He("error",a),He("load",a);break;case"details":He("toggle",a);break;case"embed":case"source":case"link":He("error",a),He("load",a);case"area":case"base":case"br":case"col":case"hr":case"keygen":case"meta":case"param":case"track":case"wbr":case"menuitem":for(Y in l)if(l.hasOwnProperty(Y)&&(f=l[Y],f!=null))switch(Y){case"children":case"dangerouslySetInnerHTML":throw Error(r(137,o));default:rt(a,o,Y,f,l,null)}return;default:if(Ch(o)){for(ae in l)l.hasOwnProperty(ae)&&(f=l[ae],f!==void 0&&pv(a,o,ae,f,l,void 0));return}}for(j in l)l.hasOwnProperty(j)&&(f=l[j],f!=null&&rt(a,o,j,f,l,null))}function $q(a,o,l,f){switch(o){case"div":case"span":case"svg":case"path":case"a":case"g":case"p":case"li":break;case"input":var p=null,m=null,O=null,j=null,k=null,Y=null,ae=null;for(F in l){var oe=l[F];if(l.hasOwnProperty(F)&&oe!=null)switch(F){case"checked":break;case"value":break;case"defaultValue":k=oe;default:f.hasOwnProperty(F)||rt(a,o,F,null,f,oe)}}for(var K in f){var F=f[K];if(oe=l[K],f.hasOwnProperty(K)&&(F!=null||oe!=null))switch(K){case"type":m=F;break;case"name":p=F;break;case"checked":Y=F;break;case"defaultChecked":ae=F;break;case"value":O=F;break;case"defaultValue":j=F;break;case"children":case"dangerouslySetInnerHTML":if(F!=null)throw Error(r(137,o));break;default:F!==oe&&rt(a,o,K,F,f,oe)}}Mh(a,O,j,k,Y,ae,m,p);return;case"select":F=O=j=K=null;for(m in l)if(k=l[m],l.hasOwnProperty(m)&&k!=null)switch(m){case"value":break;case"multiple":F=k;default:f.hasOwnProperty(m)||rt(a,o,m,null,f,k)}for(p in f)if(m=f[p],k=l[p],f.hasOwnProperty(p)&&(m!=null||k!=null))switch(p){case"value":K=m;break;case"defaultValue":j=m;break;case"multiple":O=m;default:m!==k&&rt(a,o,p,m,f,k)}o=j,l=O,f=F,K!=null?Bi(a,!!l,K,!1):!!f!=!!l&&(o!=null?Bi(a,!!l,o,!0):Bi(a,!!l,l?[]:"",!1));return;case"textarea":F=K=null;for(j in l)if(p=l[j],l.hasOwnProperty(j)&&p!=null&&!f.hasOwnProperty(j))switch(j){case"value":break;case"children":break;default:rt(a,o,j,null,f,p)}for(O in f)if(p=f[O],m=l[O],f.hasOwnProperty(O)&&(p!=null||m!=null))switch(O){case"value":K=p;break;case"defaultValue":F=p;break;case"children":break;case"dangerouslySetInnerHTML":if(p!=null)throw Error(r(91));break;default:p!==m&&rt(a,o,O,p,f,m)}G_(a,K,F);return;case"option":for(var je in l)if(K=l[je],l.hasOwnProperty(je)&&K!=null&&!f.hasOwnProperty(je))switch(je){case"selected":a.selected=!1;break;default:rt(a,o,je,null,f,K)}for(k in f)if(K=f[k],F=l[k],f.hasOwnProperty(k)&&K!==F&&(K!=null||F!=null))switch(k){case"selected":a.selected=K&&typeof K!="function"&&typeof K!="symbol";break;default:rt(a,o,k,K,f,F)}return;case"img":case"link":case"area":case"base":case"br":case"col":case"embed":case"hr":case"keygen":case"meta":case"param":case"source":case"track":case"wbr":case"menuitem":for(var we in l)K=l[we],l.hasOwnProperty(we)&&K!=null&&!f.hasOwnProperty(we)&&rt(a,o,we,null,f,K);for(Y in f)if(K=f[Y],F=l[Y],f.hasOwnProperty(Y)&&K!==F&&(K!=null||F!=null))switch(Y){case"children":case"dangerouslySetInnerHTML":if(K!=null)throw Error(r(137,o));break;default:rt(a,o,Y,K,f,F)}return;default:if(Ch(o)){for(var at in l)K=l[at],l.hasOwnProperty(at)&&K!==void 0&&!f.hasOwnProperty(at)&&pv(a,o,at,void 0,f,K);for(ae in f)K=f[ae],F=l[ae],!f.hasOwnProperty(ae)||K===F||K===void 0&&F===void 0||pv(a,o,ae,K,f,F);return}}for(var I in l)K=l[I],l.hasOwnProperty(I)&&K!=null&&!f.hasOwnProperty(I)&&rt(a,o,I,null,f,K);for(oe in f)K=f[oe],F=l[oe],!f.hasOwnProperty(oe)||K===F||K==null&&F==null||rt(a,o,oe,K,f,F)}var vv=null,yv=null;function Xc(a){return a.nodeType===9?a:a.ownerDocument}function Zw(a){switch(a){case"http://www.w3.org/2000/svg":return 1;case"http://www.w3.org/1998/Math/MathML":return 2;default:return 0}}function Yw(a,o){if(a===0)switch(o){case"svg":return 1;case"math":return 2;default:return 0}return a===1&&o==="foreignObject"?0:a}function mv(a,o){return a==="textarea"||a==="noscript"||typeof o.children=="string"||typeof o.children=="number"||typeof o.children=="bigint"||typeof o.dangerouslySetInnerHTML=="object"&&o.dangerouslySetInnerHTML!==null&&o.dangerouslySetInnerHTML.__html!=null}var gv=null;function Bq(){var a=window.event;return a&&a.type==="popstate"?a===gv?!1:(gv=a,!0):(gv=null,!1)}var Xw=typeof setTimeout=="function"?setTimeout:void 0,zq=typeof clearTimeout=="function"?clearTimeout:void 0,Kw=typeof Promise=="function"?Promise:void 0,Lq=typeof queueMicrotask=="function"?queueMicrotask:typeof Kw<"u"?function(a){return Kw.resolve(null).then(a).catch(Iq)}:Xw;function Iq(a){setTimeout(function(){throw a})}function ga(a){return a==="head"}function Ww(a,o){var l=o,f=0,p=0;do{var m=l.nextSibling;if(a.removeChild(l),m&&m.nodeType===8)if(l=m.data,l==="/$"){if(0<f&&8>f){l=f;var O=a.ownerDocument;if(l&1&&il(O.documentElement),l&2&&il(O.body),l&4)for(l=O.head,il(l),O=l.firstChild;O;){var j=O.nextSibling,k=O.nodeName;O[xu]||k==="SCRIPT"||k==="STYLE"||k==="LINK"&&O.rel.toLowerCase()==="stylesheet"||l.removeChild(O),O=j}}if(p===0){a.removeChild(m),hl(o);return}p--}else l==="$"||l==="$?"||l==="$!"?p++:f=l.charCodeAt(0)-48;else f=0;l=m}while(l);hl(o)}function bv(a){var o=a.firstChild;for(o&&o.nodeType===10&&(o=o.nextSibling);o;){var l=o;switch(o=o.nextSibling,l.nodeName){case"HTML":case"HEAD":case"BODY":bv(l),Oh(l);continue;case"SCRIPT":case"STYLE":continue;case"LINK":if(l.rel.toLowerCase()==="stylesheet")continue}a.removeChild(l)}}function Uq(a,o,l,f){for(;a.nodeType===1;){var p=l;if(a.nodeName.toLowerCase()!==o.toLowerCase()){if(!f&&(a.nodeName!=="INPUT"||a.type!=="hidden"))break}else if(f){if(!a[xu])switch(o){case"meta":if(!a.hasAttribute("itemprop"))break;return a;case"link":if(m=a.getAttribute("rel"),m==="stylesheet"&&a.hasAttribute("data-precedence"))break;if(m!==p.rel||a.getAttribute("href")!==(p.href==null||p.href===""?null:p.href)||a.getAttribute("crossorigin")!==(p.crossOrigin==null?null:p.crossOrigin)||a.getAttribute("title")!==(p.title==null?null:p.title))break;return a;case"style":if(a.hasAttribute("data-precedence"))break;return a;case"script":if(m=a.getAttribute("src"),(m!==(p.src==null?null:p.src)||a.getAttribute("type")!==(p.type==null?null:p.type)||a.getAttribute("crossorigin")!==(p.crossOrigin==null?null:p.crossOrigin))&&m&&a.hasAttribute("async")&&!a.hasAttribute("itemprop"))break;return a;default:return a}}else if(o==="input"&&a.type==="hidden"){var m=p.name==null?null:""+p.name;if(p.type==="hidden"&&a.getAttribute("name")===m)return a}else return a;if(a=Vn(a.nextSibling),a===null)break}return null}function Hq(a,o,l){if(o==="")return null;for(;a.nodeType!==3;)if((a.nodeType!==1||a.nodeName!=="INPUT"||a.type!=="hidden")&&!l||(a=Vn(a.nextSibling),a===null))return null;return a}function xv(a){return a.data==="$!"||a.data==="$?"&&a.ownerDocument.readyState==="complete"}function Gq(a,o){var l=a.ownerDocument;if(a.data!=="$?"||l.readyState==="complete")o();else{var f=function(){o(),l.removeEventListener("DOMContentLoaded",f)};l.addEventListener("DOMContentLoaded",f),a._reactRetry=f}}function Vn(a){for(;a!=null;a=a.nextSibling){var o=a.nodeType;if(o===1||o===3)break;if(o===8){if(o=a.data,o==="$"||o==="$!"||o==="$?"||o==="F!"||o==="F")break;if(o==="/$")return null}}return a}var _v=null;function Fw(a){a=a.previousSibling;for(var o=0;a;){if(a.nodeType===8){var l=a.data;if(l==="$"||l==="$!"||l==="$?"){if(o===0)return a;o--}else l==="/$"&&o++}a=a.previousSibling}return null}function Qw(a,o,l){switch(o=Xc(l),a){case"html":if(a=o.documentElement,!a)throw Error(r(452));return a;case"head":if(a=o.head,!a)throw Error(r(453));return a;case"body":if(a=o.body,!a)throw Error(r(454));return a;default:throw Error(r(451))}}function il(a){for(var o=a.attributes;o.length;)a.removeAttributeNode(o[0]);Oh(a)}var kn=new Map,Jw=new Set;function Kc(a){return typeof a.getRootNode=="function"?a.getRootNode():a.nodeType===9?a:a.ownerDocument}var qr=H.d;H.d={f:Vq,r:Zq,D:Yq,C:Xq,L:Kq,m:Wq,X:Qq,S:Fq,M:Jq};function Vq(){var a=qr.f(),o=Lc();return a||o}function Zq(a){var o=Di(a);o!==null&&o.tag===5&&o.type==="form"?bS(o):qr.r(a)}var vo=typeof document>"u"?null:document;function eO(a,o,l){var f=vo;if(f&&typeof o=="string"&&o){var p=Mn(o);p='link[rel="'+a+'"][href="'+p+'"]',typeof l=="string"&&(p+='[crossorigin="'+l+'"]'),Jw.has(p)||(Jw.add(p),a={rel:a,crossOrigin:l,href:o},f.querySelector(p)===null&&(o=f.createElement("link"),Gt(o,"link",a),qt(o),f.head.appendChild(o)))}}function Yq(a){qr.D(a),eO("dns-prefetch",a,null)}function Xq(a,o){qr.C(a,o),eO("preconnect",a,o)}function Kq(a,o,l){qr.L(a,o,l);var f=vo;if(f&&a&&o){var p='link[rel="preload"][as="'+Mn(o)+'"]';o==="image"&&l&&l.imageSrcSet?(p+='[imagesrcset="'+Mn(l.imageSrcSet)+'"]',typeof l.imageSizes=="string"&&(p+='[imagesizes="'+Mn(l.imageSizes)+'"]')):p+='[href="'+Mn(a)+'"]';var m=p;switch(o){case"style":m=yo(a);break;case"script":m=mo(a)}kn.has(m)||(a=v({rel:"preload",href:o==="image"&&l&&l.imageSrcSet?void 0:a,as:o},l),kn.set(m,a),f.querySelector(p)!==null||o==="style"&&f.querySelector(ol(m))||o==="script"&&f.querySelector(ul(m))||(o=f.createElement("link"),Gt(o,"link",a),qt(o),f.head.appendChild(o)))}}function Wq(a,o){qr.m(a,o);var l=vo;if(l&&a){var f=o&&typeof o.as=="string"?o.as:"script",p='link[rel="modulepreload"][as="'+Mn(f)+'"][href="'+Mn(a)+'"]',m=p;switch(f){case"audioworklet":case"paintworklet":case"serviceworker":case"sharedworker":case"worker":case"script":m=mo(a)}if(!kn.has(m)&&(a=v({rel:"modulepreload",href:a},o),kn.set(m,a),l.querySelector(p)===null)){switch(f){case"audioworklet":case"paintworklet":case"serviceworker":case"sharedworker":case"worker":case"script":if(l.querySelector(ul(m)))return}f=l.createElement("link"),Gt(f,"link",a),qt(f),l.head.appendChild(f)}}}function Fq(a,o,l){qr.S(a,o,l);var f=vo;if(f&&a){var p=ki(f).hoistableStyles,m=yo(a);o=o||"default";var O=p.get(m);if(!O){var j={loading:0,preload:null};if(O=f.querySelector(ol(m)))j.loading=5;else{a=v({rel:"stylesheet",href:a,"data-precedence":o},l),(l=kn.get(m))&&Sv(a,l);var k=O=f.createElement("link");qt(k),Gt(k,"link",a),k._p=new Promise(function(Y,ae){k.onload=Y,k.onerror=ae}),k.addEventListener("load",function(){j.loading|=1}),k.addEventListener("error",function(){j.loading|=2}),j.loading|=4,Wc(O,o,f)}O={type:"stylesheet",instance:O,count:1,state:j},p.set(m,O)}}}function Qq(a,o){qr.X(a,o);var l=vo;if(l&&a){var f=ki(l).hoistableScripts,p=mo(a),m=f.get(p);m||(m=l.querySelector(ul(p)),m||(a=v({src:a,async:!0},o),(o=kn.get(p))&&wv(a,o),m=l.createElement("script"),qt(m),Gt(m,"link",a),l.head.appendChild(m)),m={type:"script",instance:m,count:1,state:null},f.set(p,m))}}function Jq(a,o){qr.M(a,o);var l=vo;if(l&&a){var f=ki(l).hoistableScripts,p=mo(a),m=f.get(p);m||(m=l.querySelector(ul(p)),m||(a=v({src:a,async:!0,type:"module"},o),(o=kn.get(p))&&wv(a,o),m=l.createElement("script"),qt(m),Gt(m,"link",a),l.head.appendChild(m)),m={type:"script",instance:m,count:1,state:null},f.set(p,m))}}function tO(a,o,l,f){var p=(p=be.current)?Kc(p):null;if(!p)throw Error(r(446));switch(a){case"meta":case"title":return null;case"style":return typeof l.precedence=="string"&&typeof l.href=="string"?(o=yo(l.href),l=ki(p).hoistableStyles,f=l.get(o),f||(f={type:"style",instance:null,count:0,state:null},l.set(o,f)),f):{type:"void",instance:null,count:0,state:null};case"link":if(l.rel==="stylesheet"&&typeof l.href=="string"&&typeof l.precedence=="string"){a=yo(l.href);var m=ki(p).hoistableStyles,O=m.get(a);if(O||(p=p.ownerDocument||p,O={type:"stylesheet",instance:null,count:0,state:{loading:0,preload:null}},m.set(a,O),(m=p.querySelector(ol(a)))&&!m._p&&(O.instance=m,O.state.loading=5),kn.has(a)||(l={rel:"preload",as:"style",href:l.href,crossOrigin:l.crossOrigin,integrity:l.integrity,media:l.media,hrefLang:l.hrefLang,referrerPolicy:l.referrerPolicy},kn.set(a,l),m||e$(p,a,l,O.state))),o&&f===null)throw Error(r(528,""));return O}if(o&&f!==null)throw Error(r(529,""));return null;case"script":return o=l.async,l=l.src,typeof l=="string"&&o&&typeof o!="function"&&typeof o!="symbol"?(o=mo(l),l=ki(p).hoistableScripts,f=l.get(o),f||(f={type:"script",instance:null,count:0,state:null},l.set(o,f)),f):{type:"void",instance:null,count:0,state:null};default:throw Error(r(444,a))}}function yo(a){return'href="'+Mn(a)+'"'}function ol(a){return'link[rel="stylesheet"]['+a+"]"}function nO(a){return v({},a,{"data-precedence":a.precedence,precedence:null})}function e$(a,o,l,f){a.querySelector('link[rel="preload"][as="style"]['+o+"]")?f.loading=1:(o=a.createElement("link"),f.preload=o,o.addEventListener("load",function(){return f.loading|=1}),o.addEventListener("error",function(){return f.loading|=2}),Gt(o,"link",l),qt(o),a.head.appendChild(o))}function mo(a){return'[src="'+Mn(a)+'"]'}function ul(a){return"script[async]"+a}function rO(a,o,l){if(o.count++,o.instance===null)switch(o.type){case"style":var f=a.querySelector('style[data-href~="'+Mn(l.href)+'"]');if(f)return o.instance=f,qt(f),f;var p=v({},l,{"data-href":l.href,"data-precedence":l.precedence,href:null,precedence:null});return f=(a.ownerDocument||a).createElement("style"),qt(f),Gt(f,"style",p),Wc(f,l.precedence,a),o.instance=f;case"stylesheet":p=yo(l.href);var m=a.querySelector(ol(p));if(m)return o.state.loading|=4,o.instance=m,qt(m),m;f=nO(l),(p=kn.get(p))&&Sv(f,p),m=(a.ownerDocument||a).createElement("link"),qt(m);var O=m;return O._p=new Promise(function(j,k){O.onload=j,O.onerror=k}),Gt(m,"link",f),o.state.loading|=4,Wc(m,l.precedence,a),o.instance=m;case"script":return m=mo(l.src),(p=a.querySelector(ul(m)))?(o.instance=p,qt(p),p):(f=l,(p=kn.get(m))&&(f=v({},l),wv(f,p)),a=a.ownerDocument||a,p=a.createElement("script"),qt(p),Gt(p,"link",f),a.head.appendChild(p),o.instance=p);case"void":return null;default:throw Error(r(443,o.type))}else o.type==="stylesheet"&&(o.state.loading&4)===0&&(f=o.instance,o.state.loading|=4,Wc(f,l.precedence,a));return o.instance}function Wc(a,o,l){for(var f=l.querySelectorAll('link[rel="stylesheet"][data-precedence],style[data-precedence]'),p=f.length?f[f.length-1]:null,m=p,O=0;O<f.length;O++){var j=f[O];if(j.dataset.precedence===o)m=j;else if(m!==p)break}m?m.parentNode.insertBefore(a,m.nextSibling):(o=l.nodeType===9?l.head:l,o.insertBefore(a,o.firstChild))}function Sv(a,o){a.crossOrigin==null&&(a.crossOrigin=o.crossOrigin),a.referrerPolicy==null&&(a.referrerPolicy=o.referrerPolicy),a.title==null&&(a.title=o.title)}function wv(a,o){a.crossOrigin==null&&(a.crossOrigin=o.crossOrigin),a.referrerPolicy==null&&(a.referrerPolicy=o.referrerPolicy),a.integrity==null&&(a.integrity=o.integrity)}var Fc=null;function aO(a,o,l){if(Fc===null){var f=new Map,p=Fc=new Map;p.set(l,f)}else p=Fc,f=p.get(l),f||(f=new Map,p.set(l,f));if(f.has(a))return f;for(f.set(a,null),l=l.getElementsByTagName(a),p=0;p<l.length;p++){var m=l[p];if(!(m[xu]||m[Yt]||a==="link"&&m.getAttribute("rel")==="stylesheet")&&m.namespaceURI!=="http://www.w3.org/2000/svg"){var O=m.getAttribute(o)||"";O=a+O;var j=f.get(O);j?j.push(m):f.set(O,[m])}}return f}function iO(a,o,l){a=a.ownerDocument||a,a.head.insertBefore(l,o==="title"?a.querySelector("head > title"):null)}function t$(a,o,l){if(l===1||o.itemProp!=null)return!1;switch(a){case"meta":case"title":return!0;case"style":if(typeof o.precedence!="string"||typeof o.href!="string"||o.href==="")break;return!0;case"link":if(typeof o.rel!="string"||typeof o.href!="string"||o.href===""||o.onLoad||o.onError)break;switch(o.rel){case"stylesheet":return a=o.disabled,typeof o.precedence=="string"&&a==null;default:return!0}case"script":if(o.async&&typeof o.async!="function"&&typeof o.async!="symbol"&&!o.onLoad&&!o.onError&&o.src&&typeof o.src=="string")return!0}return!1}function oO(a){return!(a.type==="stylesheet"&&(a.state.loading&3)===0)}var ll=null;function n$(){}function r$(a,o,l){if(ll===null)throw Error(r(475));var f=ll;if(o.type==="stylesheet"&&(typeof l.media!="string"||matchMedia(l.media).matches!==!1)&&(o.state.loading&4)===0){if(o.instance===null){var p=yo(l.href),m=a.querySelector(ol(p));if(m){a=m._p,a!==null&&typeof a=="object"&&typeof a.then=="function"&&(f.count++,f=Qc.bind(f),a.then(f,f)),o.state.loading|=4,o.instance=m,qt(m);return}m=a.ownerDocument||a,l=nO(l),(p=kn.get(p))&&Sv(l,p),m=m.createElement("link"),qt(m);var O=m;O._p=new Promise(function(j,k){O.onload=j,O.onerror=k}),Gt(m,"link",l),o.instance=m}f.stylesheets===null&&(f.stylesheets=new Map),f.stylesheets.set(o,a),(a=o.state.preload)&&(o.state.loading&3)===0&&(f.count++,o=Qc.bind(f),a.addEventListener("load",o),a.addEventListener("error",o))}}function a$(){if(ll===null)throw Error(r(475));var a=ll;return a.stylesheets&&a.count===0&&Ov(a,a.stylesheets),0<a.count?function(o){var l=setTimeout(function(){if(a.stylesheets&&Ov(a,a.stylesheets),a.unsuspend){var f=a.unsuspend;a.unsuspend=null,f()}},6e4);return a.unsuspend=o,function(){a.unsuspend=null,clearTimeout(l)}}:null}function Qc(){if(this.count--,this.count===0){if(this.stylesheets)Ov(this,this.stylesheets);else if(this.unsuspend){var a=this.unsuspend;this.unsuspend=null,a()}}}var Jc=null;function Ov(a,o){a.stylesheets=null,a.unsuspend!==null&&(a.count++,Jc=new Map,o.forEach(i$,a),Jc=null,Qc.call(a))}function i$(a,o){if(!(o.state.loading&4)){var l=Jc.get(a);if(l)var f=l.get(null);else{l=new Map,Jc.set(a,l);for(var p=a.querySelectorAll("link[data-precedence],style[data-precedence]"),m=0;m<p.length;m++){var O=p[m];(O.nodeName==="LINK"||O.getAttribute("media")!=="not all")&&(l.set(O.dataset.precedence,O),f=O)}f&&l.set(null,f)}p=o.instance,O=p.getAttribute("data-precedence"),m=l.get(O)||f,m===f&&l.set(null,p),l.set(O,p),this.count++,f=Qc.bind(this),p.addEventListener("load",f),p.addEventListener("error",f),m?m.parentNode.insertBefore(p,m.nextSibling):(a=a.nodeType===9?a.head:a,a.insertBefore(p,a.firstChild)),o.state.loading|=4}}var sl={$$typeof:A,Provider:null,Consumer:null,_currentValue:U,_currentValue2:U,_threadCount:0};function o$(a,o,l,f,p,m,O,j){this.tag=1,this.containerInfo=a,this.pingCache=this.current=this.pendingChildren=null,this.timeoutHandle=-1,this.callbackNode=this.next=this.pendingContext=this.context=this.cancelPendingCommit=null,this.callbackPriority=0,this.expirationTimes=xh(-1),this.entangledLanes=this.shellSuspendCounter=this.errorRecoveryDisabledLanes=this.expiredLanes=this.warmLanes=this.pingedLanes=this.suspendedLanes=this.pendingLanes=0,this.entanglements=xh(0),this.hiddenUpdates=xh(null),this.identifierPrefix=f,this.onUncaughtError=p,this.onCaughtError=m,this.onRecoverableError=O,this.pooledCache=null,this.pooledCacheLanes=0,this.formState=j,this.incompleteTransitions=new Map}function uO(a,o,l,f,p,m,O,j,k,Y,ae,oe){return a=new o$(a,o,l,O,j,k,Y,oe),o=1,m===!0&&(o|=24),m=vn(3,null,null,o),a.current=m,m.stateNode=a,o=ip(),o.refCount++,a.pooledCache=o,o.refCount++,m.memoizedState={element:f,isDehydrated:l,cache:o},sp(m),a}function lO(a){return a?(a=Yi,a):Yi}function sO(a,o,l,f,p,m){p=lO(p),f.context===null?f.context=p:f.pendingContext=p,f=ia(o),f.payload={element:l},m=m===void 0?null:m,m!==null&&(f.callback=m),l=oa(a,f,o),l!==null&&(xn(l,a,o),zu(l,a,o))}function cO(a,o){if(a=a.memoizedState,a!==null&&a.dehydrated!==null){var l=a.retryLane;a.retryLane=l!==0&&l<o?l:o}}function Av(a,o){cO(a,o),(a=a.alternate)&&cO(a,o)}function fO(a){if(a.tag===13){var o=Zi(a,67108864);o!==null&&xn(o,a,67108864),Av(a,67108864)}}var ef=!0;function u$(a,o,l,f){var p=D.T;D.T=null;var m=H.p;try{H.p=2,Ev(a,o,l,f)}finally{H.p=m,D.T=p}}function l$(a,o,l,f){var p=D.T;D.T=null;var m=H.p;try{H.p=8,Ev(a,o,l,f)}finally{H.p=m,D.T=p}}function Ev(a,o,l,f){if(ef){var p=Tv(f);if(p===null)hv(a,o,f,tf,l),hO(a,f);else if(c$(p,a,o,l,f))f.stopPropagation();else if(hO(a,f),o&4&&-1<s$.indexOf(a)){for(;p!==null;){var m=Di(p);if(m!==null)switch(m.tag){case 3:if(m=m.stateNode,m.current.memoizedState.isDehydrated){var O=Ua(m.pendingLanes);if(O!==0){var j=m;for(j.pendingLanes|=2,j.entangledLanes|=2;O;){var k=1<<31-hn(O);j.entanglements[1]|=k,O&=~k}lr(m),(et&6)===0&&(Bc=dt()+500,nl(0))}}break;case 13:j=Zi(m,2),j!==null&&xn(j,m,2),Lc(),Av(m,2)}if(m=Tv(f),m===null&&hv(a,o,f,tf,l),m===p)break;p=m}p!==null&&f.stopPropagation()}else hv(a,o,f,null,l)}}function Tv(a){return a=Rh(a),Mv(a)}var tf=null;function Mv(a){if(tf=null,a=Ni(a),a!==null){var o=u(a);if(o===null)a=null;else{var l=o.tag;if(l===13){if(a=s(o),a!==null)return a;a=null}else if(l===3){if(o.stateNode.current.memoizedState.isDehydrated)return o.tag===3?o.stateNode.containerInfo:null;a=null}else o!==a&&(a=null)}}return tf=a,null}function dO(a){switch(a){case"beforetoggle":case"cancel":case"click":case"close":case"contextmenu":case"copy":case"cut":case"auxclick":case"dblclick":case"dragend":case"dragstart":case"drop":case"focusin":case"focusout":case"input":case"invalid":case"keydown":case"keypress":case"keyup":case"mousedown":case"mouseup":case"paste":case"pause":case"play":case"pointercancel":case"pointerdown":case"pointerup":case"ratechange":case"reset":case"resize":case"seeked":case"submit":case"toggle":case"touchcancel":case"touchend":case"touchstart":case"volumechange":case"change":case"selectionchange":case"textInput":case"compositionstart":case"compositionend":case"compositionupdate":case"beforeblur":case"afterblur":case"beforeinput":case"blur":case"fullscreenchange":case"focus":case"hashchange":case"popstate":case"select":case"selectstart":return 2;case"drag":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"mousemove":case"mouseout":case"mouseover":case"pointermove":case"pointerout":case"pointerover":case"scroll":case"touchmove":case"wheel":case"mouseenter":case"mouseleave":case"pointerenter":case"pointerleave":return 8;case"message":switch(rr()){case xr:return 2;case Ia:return 8;case En:case _r:return 32;case Pi:return 268435456;default:return 32}default:return 32}}var jv=!1,ba=null,xa=null,_a=null,cl=new Map,fl=new Map,Sa=[],s$="mousedown mouseup touchcancel touchend touchstart auxclick dblclick pointercancel pointerdown pointerup dragend dragstart drop compositionend compositionstart keydown keypress keyup input textInput copy cut paste click change contextmenu reset".split(" ");function hO(a,o){switch(a){case"focusin":case"focusout":ba=null;break;case"dragenter":case"dragleave":xa=null;break;case"mouseover":case"mouseout":_a=null;break;case"pointerover":case"pointerout":cl.delete(o.pointerId);break;case"gotpointercapture":case"lostpointercapture":fl.delete(o.pointerId)}}function dl(a,o,l,f,p,m){return a===null||a.nativeEvent!==m?(a={blockedOn:o,domEventName:l,eventSystemFlags:f,nativeEvent:m,targetContainers:[p]},o!==null&&(o=Di(o),o!==null&&fO(o)),a):(a.eventSystemFlags|=f,o=a.targetContainers,p!==null&&o.indexOf(p)===-1&&o.push(p),a)}function c$(a,o,l,f,p){switch(o){case"focusin":return ba=dl(ba,a,o,l,f,p),!0;case"dragenter":return xa=dl(xa,a,o,l,f,p),!0;case"mouseover":return _a=dl(_a,a,o,l,f,p),!0;case"pointerover":var m=p.pointerId;return cl.set(m,dl(cl.get(m)||null,a,o,l,f,p)),!0;case"gotpointercapture":return m=p.pointerId,fl.set(m,dl(fl.get(m)||null,a,o,l,f,p)),!0}return!1}function pO(a){var o=Ni(a.target);if(o!==null){var l=u(o);if(l!==null){if(o=l.tag,o===13){if(o=s(l),o!==null){a.blockedOn=o,rk(a.priority,function(){if(l.tag===13){var f=bn();f=_h(f);var p=Zi(l,f);p!==null&&xn(p,l,f),Av(l,f)}});return}}else if(o===3&&l.stateNode.current.memoizedState.isDehydrated){a.blockedOn=l.tag===3?l.stateNode.containerInfo:null;return}}}a.blockedOn=null}function nf(a){if(a.blockedOn!==null)return!1;for(var o=a.targetContainers;0<o.length;){var l=Tv(a.nativeEvent);if(l===null){l=a.nativeEvent;var f=new l.constructor(l.type,l);Ph=f,l.target.dispatchEvent(f),Ph=null}else return o=Di(l),o!==null&&fO(o),a.blockedOn=l,!1;o.shift()}return!0}function vO(a,o,l){nf(a)&&l.delete(o)}function f$(){jv=!1,ba!==null&&nf(ba)&&(ba=null),xa!==null&&nf(xa)&&(xa=null),_a!==null&&nf(_a)&&(_a=null),cl.forEach(vO),fl.forEach(vO)}function rf(a,o){a.blockedOn===o&&(a.blockedOn=null,jv||(jv=!0,e.unstable_scheduleCallback(e.unstable_NormalPriority,f$)))}var af=null;function yO(a){af!==a&&(af=a,e.unstable_scheduleCallback(e.unstable_NormalPriority,function(){af===a&&(af=null);for(var o=0;o<a.length;o+=3){var l=a[o],f=a[o+1],p=a[o+2];if(typeof f!="function"){if(Mv(f||l)===null)continue;break}var m=Di(l);m!==null&&(a.splice(o,3),o-=3,Mp(m,{pending:!0,data:p,method:l.method,action:f},f,p))}}))}function hl(a){function o(k){return rf(k,a)}ba!==null&&rf(ba,a),xa!==null&&rf(xa,a),_a!==null&&rf(_a,a),cl.forEach(o),fl.forEach(o);for(var l=0;l<Sa.length;l++){var f=Sa[l];f.blockedOn===a&&(f.blockedOn=null)}for(;0<Sa.length&&(l=Sa[0],l.blockedOn===null);)pO(l),l.blockedOn===null&&Sa.shift();if(l=(a.ownerDocument||a).$$reactFormReplay,l!=null)for(f=0;f<l.length;f+=3){var p=l[f],m=l[f+1],O=p[an]||null;if(typeof m=="function")O||yO(l);else if(O){var j=null;if(m&&m.hasAttribute("formAction")){if(p=m,O=m[an]||null)j=O.formAction;else if(Mv(p)!==null)continue}else j=O.action;typeof j=="function"?l[f+1]=j:(l.splice(f,3),f-=3),yO(l)}}}function Cv(a){this._internalRoot=a}of.prototype.render=Cv.prototype.render=function(a){var o=this._internalRoot;if(o===null)throw Error(r(409));var l=o.current,f=bn();sO(l,f,a,o,null,null)},of.prototype.unmount=Cv.prototype.unmount=function(){var a=this._internalRoot;if(a!==null){this._internalRoot=null;var o=a.containerInfo;sO(a.current,2,null,a,null,null),Lc(),o[Ri]=null}};function of(a){this._internalRoot=a}of.prototype.unstable_scheduleHydration=function(a){if(a){var o=N_();a={blockedOn:null,target:a,priority:o};for(var l=0;l<Sa.length&&o!==0&&o<Sa[l].priority;l++);Sa.splice(l,0,a),l===0&&pO(a)}};var mO=t.version;if(mO!=="19.1.0")throw Error(r(527,mO,"19.1.0"));H.findDOMNode=function(a){var o=a._reactInternals;if(o===void 0)throw typeof a.render=="function"?Error(r(188)):(a=Object.keys(a).join(","),Error(r(268,a)));return a=d(o),a=a!==null?h(a):null,a=a===null?null:a.stateNode,a};var d$={bundleType:0,version:"19.1.0",rendererPackageName:"react-dom",currentDispatcherRef:D,reconcilerVersion:"19.1.0"};if(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__<"u"){var uf=__REACT_DEVTOOLS_GLOBAL_HOOK__;if(!uf.isDisabled&&uf.supportsFiber)try{mu=uf.inject(d$),dn=uf}catch{}}return vl.createRoot=function(a,o){if(!i(a))throw Error(r(299));var l=!1,f="",p=NS,m=DS,O=kS,j=null;return o!=null&&(o.unstable_strictMode===!0&&(l=!0),o.identifierPrefix!==void 0&&(f=o.identifierPrefix),o.onUncaughtError!==void 0&&(p=o.onUncaughtError),o.onCaughtError!==void 0&&(m=o.onCaughtError),o.onRecoverableError!==void 0&&(O=o.onRecoverableError),o.unstable_transitionCallbacks!==void 0&&(j=o.unstable_transitionCallbacks)),o=uO(a,1,!1,null,null,l,f,p,m,O,j,null),a[Ri]=o.current,dv(a),new Cv(o)},vl.hydrateRoot=function(a,o,l){if(!i(a))throw Error(r(299));var f=!1,p="",m=NS,O=DS,j=kS,k=null,Y=null;return l!=null&&(l.unstable_strictMode===!0&&(f=!0),l.identifierPrefix!==void 0&&(p=l.identifierPrefix),l.onUncaughtError!==void 0&&(m=l.onUncaughtError),l.onCaughtError!==void 0&&(O=l.onCaughtError),l.onRecoverableError!==void 0&&(j=l.onRecoverableError),l.unstable_transitionCallbacks!==void 0&&(k=l.unstable_transitionCallbacks),l.formState!==void 0&&(Y=l.formState)),o=uO(a,1,!0,o,l??null,f,p,m,O,j,k,Y),o.context=lO(null),l=o.current,f=bn(),f=_h(f),p=ia(f),p.callback=null,oa(l,p,f),l=f,o.current.lanes=l,bu(o,l),lr(o),a[Ri]=o.current,dv(a),new of(o)},vl.version="19.1.0",vl}var TO;function w$(){if(TO)return Nv.exports;TO=1;function e(){if(!(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__>"u"||typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE!="function"))try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(e)}catch(t){console.error(t)}}return e(),Nv.exports=S$(),Nv.exports}var O$=w$();function k2(e){var t,n,r="";if(typeof e=="string"||typeof e=="number")r+=e;else if(typeof e=="object")if(Array.isArray(e)){var i=e.length;for(t=0;t<i;t++)e[t]&&(n=k2(e[t]))&&(r&&(r+=" "),r+=n)}else for(n in e)e[n]&&(r&&(r+=" "),r+=n);return r}function Xe(){for(var e,t,n=0,r="",i=arguments.length;n<i;n++)(e=arguments[n])&&(t=k2(e))&&(r&&(r+=" "),r+=t);return r}const wx="-",A$=e=>{const t=T$(e),{conflictingClassGroups:n,conflictingClassGroupModifiers:r}=e;return{getClassGroupId:s=>{const c=s.split(wx);return c[0]===""&&c.length!==1&&c.shift(),q2(c,t)||E$(s)},getConflictingClassGroupIds:(s,c)=>{const d=n[s]||[];return c&&r[s]?[...d,...r[s]]:d}}},q2=(e,t)=>{var s;if(e.length===0)return t.classGroupId;const n=e[0],r=t.nextPart.get(n),i=r?q2(e.slice(1),r):void 0;if(i)return i;if(t.validators.length===0)return;const u=e.join(wx);return(s=t.validators.find(({validator:c})=>c(u)))==null?void 0:s.classGroupId},MO=/^\[(.+)\]$/,E$=e=>{if(MO.test(e)){const t=MO.exec(e)[1],n=t==null?void 0:t.substring(0,t.indexOf(":"));if(n)return"arbitrary.."+n}},T$=e=>{const{theme:t,classGroups:n}=e,r={nextPart:new Map,validators:[]};for(const i in n)Xb(n[i],r,i,t);return r},Xb=(e,t,n,r)=>{e.forEach(i=>{if(typeof i=="string"){const u=i===""?t:jO(t,i);u.classGroupId=n;return}if(typeof i=="function"){if(M$(i)){Xb(i(r),t,n,r);return}t.validators.push({validator:i,classGroupId:n});return}Object.entries(i).forEach(([u,s])=>{Xb(s,jO(t,u),n,r)})})},jO=(e,t)=>{let n=e;return t.split(wx).forEach(r=>{n.nextPart.has(r)||n.nextPart.set(r,{nextPart:new Map,validators:[]}),n=n.nextPart.get(r)}),n},M$=e=>e.isThemeGetter,j$=e=>{if(e<1)return{get:()=>{},set:()=>{}};let t=0,n=new Map,r=new Map;const i=(u,s)=>{n.set(u,s),t++,t>e&&(t=0,r=n,n=new Map)};return{get(u){let s=n.get(u);if(s!==void 0)return s;if((s=r.get(u))!==void 0)return i(u,s),s},set(u,s){n.has(u)?n.set(u,s):i(u,s)}}},Kb="!",Wb=":",C$=Wb.length,P$=e=>{const{prefix:t,experimentalParseClassName:n}=e;let r=i=>{const u=[];let s=0,c=0,d=0,h;for(let S=0;S<i.length;S++){let _=i[S];if(s===0&&c===0){if(_===Wb){u.push(i.slice(d,S)),d=S+C$;continue}if(_==="/"){h=S;continue}}_==="["?s++:_==="]"?s--:_==="("?c++:_===")"&&c--}const v=u.length===0?i:i.substring(d),y=R$(v),g=y!==v,b=h&&h>d?h-d:void 0;return{modifiers:u,hasImportantModifier:g,baseClassName:y,maybePostfixModifierPosition:b}};if(t){const i=t+Wb,u=r;r=s=>s.startsWith(i)?u(s.substring(i.length)):{isExternal:!0,modifiers:[],hasImportantModifier:!1,baseClassName:s,maybePostfixModifierPosition:void 0}}if(n){const i=r;r=u=>n({className:u,parseClassName:i})}return r},R$=e=>e.endsWith(Kb)?e.substring(0,e.length-1):e.startsWith(Kb)?e.substring(1):e,N$=e=>{const t=Object.fromEntries(e.orderSensitiveModifiers.map(r=>[r,!0]));return r=>{if(r.length<=1)return r;const i=[];let u=[];return r.forEach(s=>{s[0]==="["||t[s]?(i.push(...u.sort(),s),u=[]):u.push(s)}),i.push(...u.sort()),i}},D$=e=>({cache:j$(e.cacheSize),parseClassName:P$(e),sortModifiers:N$(e),...A$(e)}),k$=/\s+/,q$=(e,t)=>{const{parseClassName:n,getClassGroupId:r,getConflictingClassGroupIds:i,sortModifiers:u}=t,s=[],c=e.trim().split(k$);let d="";for(let h=c.length-1;h>=0;h-=1){const v=c[h],{isExternal:y,modifiers:g,hasImportantModifier:b,baseClassName:S,maybePostfixModifierPosition:_}=n(v);if(y){d=v+(d.length>0?" "+d:d);continue}let x=!!_,E=r(x?S.substring(0,_):S);if(!E){if(!x){d=v+(d.length>0?" "+d:d);continue}if(E=r(S),!E){d=v+(d.length>0?" "+d:d);continue}x=!1}const M=u(g).join(":"),A=b?M+Kb:M,C=A+E;if(s.includes(C))continue;s.push(C);const w=i(E,x);for(let T=0;T<w.length;++T){const P=w[T];s.push(A+P)}d=v+(d.length>0?" "+d:d)}return d};function $$(){let e=0,t,n,r="";for(;e<arguments.length;)(t=arguments[e++])&&(n=$2(t))&&(r&&(r+=" "),r+=n);return r}const $2=e=>{if(typeof e=="string")return e;let t,n="";for(let r=0;r<e.length;r++)e[r]&&(t=$2(e[r]))&&(n&&(n+=" "),n+=t);return n};function B$(e,...t){let n,r,i,u=s;function s(d){const h=t.reduce((v,y)=>y(v),e());return n=D$(h),r=n.cache.get,i=n.cache.set,u=c,c(d)}function c(d){const h=r(d);if(h)return h;const v=q$(d,n);return i(d,v),v}return function(){return u($$.apply(null,arguments))}}const Rt=e=>{const t=n=>n[e]||[];return t.isThemeGetter=!0,t},B2=/^\[(?:(\w[\w-]*):)?(.+)\]$/i,z2=/^\((?:(\w[\w-]*):)?(.+)\)$/i,z$=/^\d+\/\d+$/,L$=/^(\d+(\.\d+)?)?(xs|sm|md|lg|xl)$/,I$=/\d+(%|px|r?em|[sdl]?v([hwib]|min|max)|pt|pc|in|cm|mm|cap|ch|ex|r?lh|cq(w|h|i|b|min|max))|\b(calc|min|max|clamp)\(.+\)|^0$/,U$=/^(rgba?|hsla?|hwb|(ok)?(lab|lch))\(.+\)$/,H$=/^(inset_)?-?((\d+)?\.?(\d+)[a-z]+|0)_-?((\d+)?\.?(\d+)[a-z]+|0)/,G$=/^(url|image|image-set|cross-fade|element|(repeating-)?(linear|radial|conic)-gradient)\(.+\)$/,go=e=>z$.test(e),Be=e=>!!e&&!Number.isNaN(Number(e)),Oa=e=>!!e&&Number.isInteger(Number(e)),$v=e=>e.endsWith("%")&&Be(e.slice(0,-1)),$r=e=>L$.test(e),V$=()=>!0,Z$=e=>I$.test(e)&&!U$.test(e),L2=()=>!1,Y$=e=>H$.test(e),X$=e=>G$.test(e),K$=e=>!me(e)&&!ge(e),W$=e=>lu(e,H2,L2),me=e=>B2.test(e),oi=e=>lu(e,G2,Z$),Bv=e=>lu(e,t3,Be),CO=e=>lu(e,I2,L2),F$=e=>lu(e,U2,X$),sf=e=>lu(e,V2,Y$),ge=e=>z2.test(e),yl=e=>su(e,G2),Q$=e=>su(e,n3),PO=e=>su(e,I2),J$=e=>su(e,H2),e3=e=>su(e,U2),cf=e=>su(e,V2,!0),lu=(e,t,n)=>{const r=B2.exec(e);return r?r[1]?t(r[1]):n(r[2]):!1},su=(e,t,n=!1)=>{const r=z2.exec(e);return r?r[1]?t(r[1]):n:!1},I2=e=>e==="position"||e==="percentage",U2=e=>e==="image"||e==="url",H2=e=>e==="length"||e==="size"||e==="bg-size",G2=e=>e==="length",t3=e=>e==="number",n3=e=>e==="family-name",V2=e=>e==="shadow",r3=()=>{const e=Rt("color"),t=Rt("font"),n=Rt("text"),r=Rt("font-weight"),i=Rt("tracking"),u=Rt("leading"),s=Rt("breakpoint"),c=Rt("container"),d=Rt("spacing"),h=Rt("radius"),v=Rt("shadow"),y=Rt("inset-shadow"),g=Rt("text-shadow"),b=Rt("drop-shadow"),S=Rt("blur"),_=Rt("perspective"),x=Rt("aspect"),E=Rt("ease"),M=Rt("animate"),A=()=>["auto","avoid","all","avoid-page","page","left","right","column"],C=()=>["center","top","bottom","left","right","top-left","left-top","top-right","right-top","bottom-right","right-bottom","bottom-left","left-bottom"],w=()=>[...C(),ge,me],T=()=>["auto","hidden","clip","visible","scroll"],P=()=>["auto","contain","none"],R=()=>[ge,me,d],$=()=>[go,"full","auto",...R()],J=()=>[Oa,"none","subgrid",ge,me],B=()=>["auto",{span:["full",Oa,ge,me]},Oa,ge,me],G=()=>[Oa,"auto",ge,me],W=()=>["auto","min","max","fr",ge,me],ne=()=>["start","end","center","between","around","evenly","stretch","baseline","center-safe","end-safe"],Q=()=>["start","end","center","stretch","center-safe","end-safe"],D=()=>["auto",...R()],H=()=>[go,"auto","full","dvw","dvh","lvw","lvh","svw","svh","min","max","fit",...R()],U=()=>[e,ge,me],ue=()=>[...C(),PO,CO,{position:[ge,me]}],N=()=>["no-repeat",{repeat:["","x","y","space","round"]}],X=()=>["auto","cover","contain",J$,W$,{size:[ge,me]}],ee=()=>[$v,yl,oi],te=()=>["","none","full",h,ge,me],ce=()=>["",Be,yl,oi],_e=()=>["solid","dashed","dotted","double"],be=()=>["normal","multiply","screen","overlay","darken","lighten","color-dodge","color-burn","hard-light","soft-light","difference","exclusion","hue","saturation","color","luminosity"],ke=()=>[Be,$v,PO,CO],Pe=()=>["","none",S,ge,me],fe=()=>["none",Be,ge,me],Oe=()=>["none",Be,ge,me],Me=()=>[Be,ge,me],le=()=>[go,"full",...R()];return{cacheSize:500,theme:{animate:["spin","ping","pulse","bounce"],aspect:["video"],blur:[$r],breakpoint:[$r],color:[V$],container:[$r],"drop-shadow":[$r],ease:["in","out","in-out"],font:[K$],"font-weight":["thin","extralight","light","normal","medium","semibold","bold","extrabold","black"],"inset-shadow":[$r],leading:["none","tight","snug","normal","relaxed","loose"],perspective:["dramatic","near","normal","midrange","distant","none"],radius:[$r],shadow:[$r],spacing:["px",Be],text:[$r],"text-shadow":[$r],tracking:["tighter","tight","normal","wide","wider","widest"]},classGroups:{aspect:[{aspect:["auto","square",go,me,ge,x]}],container:["container"],columns:[{columns:[Be,me,ge,c]}],"break-after":[{"break-after":A()}],"break-before":[{"break-before":A()}],"break-inside":[{"break-inside":["auto","avoid","avoid-page","avoid-column"]}],"box-decoration":[{"box-decoration":["slice","clone"]}],box:[{box:["border","content"]}],display:["block","inline-block","inline","flex","inline-flex","table","inline-table","table-caption","table-cell","table-column","table-column-group","table-footer-group","table-header-group","table-row-group","table-row","flow-root","grid","inline-grid","contents","list-item","hidden"],sr:["sr-only","not-sr-only"],float:[{float:["right","left","none","start","end"]}],clear:[{clear:["left","right","both","none","start","end"]}],isolation:["isolate","isolation-auto"],"object-fit":[{object:["contain","cover","fill","none","scale-down"]}],"object-position":[{object:w()}],overflow:[{overflow:T()}],"overflow-x":[{"overflow-x":T()}],"overflow-y":[{"overflow-y":T()}],overscroll:[{overscroll:P()}],"overscroll-x":[{"overscroll-x":P()}],"overscroll-y":[{"overscroll-y":P()}],position:["static","fixed","absolute","relative","sticky"],inset:[{inset:$()}],"inset-x":[{"inset-x":$()}],"inset-y":[{"inset-y":$()}],start:[{start:$()}],end:[{end:$()}],top:[{top:$()}],right:[{right:$()}],bottom:[{bottom:$()}],left:[{left:$()}],visibility:["visible","invisible","collapse"],z:[{z:[Oa,"auto",ge,me]}],basis:[{basis:[go,"full","auto",c,...R()]}],"flex-direction":[{flex:["row","row-reverse","col","col-reverse"]}],"flex-wrap":[{flex:["nowrap","wrap","wrap-reverse"]}],flex:[{flex:[Be,go,"auto","initial","none",me]}],grow:[{grow:["",Be,ge,me]}],shrink:[{shrink:["",Be,ge,me]}],order:[{order:[Oa,"first","last","none",ge,me]}],"grid-cols":[{"grid-cols":J()}],"col-start-end":[{col:B()}],"col-start":[{"col-start":G()}],"col-end":[{"col-end":G()}],"grid-rows":[{"grid-rows":J()}],"row-start-end":[{row:B()}],"row-start":[{"row-start":G()}],"row-end":[{"row-end":G()}],"grid-flow":[{"grid-flow":["row","col","dense","row-dense","col-dense"]}],"auto-cols":[{"auto-cols":W()}],"auto-rows":[{"auto-rows":W()}],gap:[{gap:R()}],"gap-x":[{"gap-x":R()}],"gap-y":[{"gap-y":R()}],"justify-content":[{justify:[...ne(),"normal"]}],"justify-items":[{"justify-items":[...Q(),"normal"]}],"justify-self":[{"justify-self":["auto",...Q()]}],"align-content":[{content:["normal",...ne()]}],"align-items":[{items:[...Q(),{baseline:["","last"]}]}],"align-self":[{self:["auto",...Q(),{baseline:["","last"]}]}],"place-content":[{"place-content":ne()}],"place-items":[{"place-items":[...Q(),"baseline"]}],"place-self":[{"place-self":["auto",...Q()]}],p:[{p:R()}],px:[{px:R()}],py:[{py:R()}],ps:[{ps:R()}],pe:[{pe:R()}],pt:[{pt:R()}],pr:[{pr:R()}],pb:[{pb:R()}],pl:[{pl:R()}],m:[{m:D()}],mx:[{mx:D()}],my:[{my:D()}],ms:[{ms:D()}],me:[{me:D()}],mt:[{mt:D()}],mr:[{mr:D()}],mb:[{mb:D()}],ml:[{ml:D()}],"space-x":[{"space-x":R()}],"space-x-reverse":["space-x-reverse"],"space-y":[{"space-y":R()}],"space-y-reverse":["space-y-reverse"],size:[{size:H()}],w:[{w:[c,"screen",...H()]}],"min-w":[{"min-w":[c,"screen","none",...H()]}],"max-w":[{"max-w":[c,"screen","none","prose",{screen:[s]},...H()]}],h:[{h:["screen",...H()]}],"min-h":[{"min-h":["screen","none",...H()]}],"max-h":[{"max-h":["screen",...H()]}],"font-size":[{text:["base",n,yl,oi]}],"font-smoothing":["antialiased","subpixel-antialiased"],"font-style":["italic","not-italic"],"font-weight":[{font:[r,ge,Bv]}],"font-stretch":[{"font-stretch":["ultra-condensed","extra-condensed","condensed","semi-condensed","normal","semi-expanded","expanded","extra-expanded","ultra-expanded",$v,me]}],"font-family":[{font:[Q$,me,t]}],"fvn-normal":["normal-nums"],"fvn-ordinal":["ordinal"],"fvn-slashed-zero":["slashed-zero"],"fvn-figure":["lining-nums","oldstyle-nums"],"fvn-spacing":["proportional-nums","tabular-nums"],"fvn-fraction":["diagonal-fractions","stacked-fractions"],tracking:[{tracking:[i,ge,me]}],"line-clamp":[{"line-clamp":[Be,"none",ge,Bv]}],leading:[{leading:[u,...R()]}],"list-image":[{"list-image":["none",ge,me]}],"list-style-position":[{list:["inside","outside"]}],"list-style-type":[{list:["disc","decimal","none",ge,me]}],"text-alignment":[{text:["left","center","right","justify","start","end"]}],"placeholder-color":[{placeholder:U()}],"text-color":[{text:U()}],"text-decoration":["underline","overline","line-through","no-underline"],"text-decoration-style":[{decoration:[..._e(),"wavy"]}],"text-decoration-thickness":[{decoration:[Be,"from-font","auto",ge,oi]}],"text-decoration-color":[{decoration:U()}],"underline-offset":[{"underline-offset":[Be,"auto",ge,me]}],"text-transform":["uppercase","lowercase","capitalize","normal-case"],"text-overflow":["truncate","text-ellipsis","text-clip"],"text-wrap":[{text:["wrap","nowrap","balance","pretty"]}],indent:[{indent:R()}],"vertical-align":[{align:["baseline","top","middle","bottom","text-top","text-bottom","sub","super",ge,me]}],whitespace:[{whitespace:["normal","nowrap","pre","pre-line","pre-wrap","break-spaces"]}],break:[{break:["normal","words","all","keep"]}],wrap:[{wrap:["break-word","anywhere","normal"]}],hyphens:[{hyphens:["none","manual","auto"]}],content:[{content:["none",ge,me]}],"bg-attachment":[{bg:["fixed","local","scroll"]}],"bg-clip":[{"bg-clip":["border","padding","content","text"]}],"bg-origin":[{"bg-origin":["border","padding","content"]}],"bg-position":[{bg:ue()}],"bg-repeat":[{bg:N()}],"bg-size":[{bg:X()}],"bg-image":[{bg:["none",{linear:[{to:["t","tr","r","br","b","bl","l","tl"]},Oa,ge,me],radial:["",ge,me],conic:[Oa,ge,me]},e3,F$]}],"bg-color":[{bg:U()}],"gradient-from-pos":[{from:ee()}],"gradient-via-pos":[{via:ee()}],"gradient-to-pos":[{to:ee()}],"gradient-from":[{from:U()}],"gradient-via":[{via:U()}],"gradient-to":[{to:U()}],rounded:[{rounded:te()}],"rounded-s":[{"rounded-s":te()}],"rounded-e":[{"rounded-e":te()}],"rounded-t":[{"rounded-t":te()}],"rounded-r":[{"rounded-r":te()}],"rounded-b":[{"rounded-b":te()}],"rounded-l":[{"rounded-l":te()}],"rounded-ss":[{"rounded-ss":te()}],"rounded-se":[{"rounded-se":te()}],"rounded-ee":[{"rounded-ee":te()}],"rounded-es":[{"rounded-es":te()}],"rounded-tl":[{"rounded-tl":te()}],"rounded-tr":[{"rounded-tr":te()}],"rounded-br":[{"rounded-br":te()}],"rounded-bl":[{"rounded-bl":te()}],"border-w":[{border:ce()}],"border-w-x":[{"border-x":ce()}],"border-w-y":[{"border-y":ce()}],"border-w-s":[{"border-s":ce()}],"border-w-e":[{"border-e":ce()}],"border-w-t":[{"border-t":ce()}],"border-w-r":[{"border-r":ce()}],"border-w-b":[{"border-b":ce()}],"border-w-l":[{"border-l":ce()}],"divide-x":[{"divide-x":ce()}],"divide-x-reverse":["divide-x-reverse"],"divide-y":[{"divide-y":ce()}],"divide-y-reverse":["divide-y-reverse"],"border-style":[{border:[..._e(),"hidden","none"]}],"divide-style":[{divide:[..._e(),"hidden","none"]}],"border-color":[{border:U()}],"border-color-x":[{"border-x":U()}],"border-color-y":[{"border-y":U()}],"border-color-s":[{"border-s":U()}],"border-color-e":[{"border-e":U()}],"border-color-t":[{"border-t":U()}],"border-color-r":[{"border-r":U()}],"border-color-b":[{"border-b":U()}],"border-color-l":[{"border-l":U()}],"divide-color":[{divide:U()}],"outline-style":[{outline:[..._e(),"none","hidden"]}],"outline-offset":[{"outline-offset":[Be,ge,me]}],"outline-w":[{outline:["",Be,yl,oi]}],"outline-color":[{outline:U()}],shadow:[{shadow:["","none",v,cf,sf]}],"shadow-color":[{shadow:U()}],"inset-shadow":[{"inset-shadow":["none",y,cf,sf]}],"inset-shadow-color":[{"inset-shadow":U()}],"ring-w":[{ring:ce()}],"ring-w-inset":["ring-inset"],"ring-color":[{ring:U()}],"ring-offset-w":[{"ring-offset":[Be,oi]}],"ring-offset-color":[{"ring-offset":U()}],"inset-ring-w":[{"inset-ring":ce()}],"inset-ring-color":[{"inset-ring":U()}],"text-shadow":[{"text-shadow":["none",g,cf,sf]}],"text-shadow-color":[{"text-shadow":U()}],opacity:[{opacity:[Be,ge,me]}],"mix-blend":[{"mix-blend":[...be(),"plus-darker","plus-lighter"]}],"bg-blend":[{"bg-blend":be()}],"mask-clip":[{"mask-clip":["border","padding","content","fill","stroke","view"]},"mask-no-clip"],"mask-composite":[{mask:["add","subtract","intersect","exclude"]}],"mask-image-linear-pos":[{"mask-linear":[Be]}],"mask-image-linear-from-pos":[{"mask-linear-from":ke()}],"mask-image-linear-to-pos":[{"mask-linear-to":ke()}],"mask-image-linear-from-color":[{"mask-linear-from":U()}],"mask-image-linear-to-color":[{"mask-linear-to":U()}],"mask-image-t-from-pos":[{"mask-t-from":ke()}],"mask-image-t-to-pos":[{"mask-t-to":ke()}],"mask-image-t-from-color":[{"mask-t-from":U()}],"mask-image-t-to-color":[{"mask-t-to":U()}],"mask-image-r-from-pos":[{"mask-r-from":ke()}],"mask-image-r-to-pos":[{"mask-r-to":ke()}],"mask-image-r-from-color":[{"mask-r-from":U()}],"mask-image-r-to-color":[{"mask-r-to":U()}],"mask-image-b-from-pos":[{"mask-b-from":ke()}],"mask-image-b-to-pos":[{"mask-b-to":ke()}],"mask-image-b-from-color":[{"mask-b-from":U()}],"mask-image-b-to-color":[{"mask-b-to":U()}],"mask-image-l-from-pos":[{"mask-l-from":ke()}],"mask-image-l-to-pos":[{"mask-l-to":ke()}],"mask-image-l-from-color":[{"mask-l-from":U()}],"mask-image-l-to-color":[{"mask-l-to":U()}],"mask-image-x-from-pos":[{"mask-x-from":ke()}],"mask-image-x-to-pos":[{"mask-x-to":ke()}],"mask-image-x-from-color":[{"mask-x-from":U()}],"mask-image-x-to-color":[{"mask-x-to":U()}],"mask-image-y-from-pos":[{"mask-y-from":ke()}],"mask-image-y-to-pos":[{"mask-y-to":ke()}],"mask-image-y-from-color":[{"mask-y-from":U()}],"mask-image-y-to-color":[{"mask-y-to":U()}],"mask-image-radial":[{"mask-radial":[ge,me]}],"mask-image-radial-from-pos":[{"mask-radial-from":ke()}],"mask-image-radial-to-pos":[{"mask-radial-to":ke()}],"mask-image-radial-from-color":[{"mask-radial-from":U()}],"mask-image-radial-to-color":[{"mask-radial-to":U()}],"mask-image-radial-shape":[{"mask-radial":["circle","ellipse"]}],"mask-image-radial-size":[{"mask-radial":[{closest:["side","corner"],farthest:["side","corner"]}]}],"mask-image-radial-pos":[{"mask-radial-at":C()}],"mask-image-conic-pos":[{"mask-conic":[Be]}],"mask-image-conic-from-pos":[{"mask-conic-from":ke()}],"mask-image-conic-to-pos":[{"mask-conic-to":ke()}],"mask-image-conic-from-color":[{"mask-conic-from":U()}],"mask-image-conic-to-color":[{"mask-conic-to":U()}],"mask-mode":[{mask:["alpha","luminance","match"]}],"mask-origin":[{"mask-origin":["border","padding","content","fill","stroke","view"]}],"mask-position":[{mask:ue()}],"mask-repeat":[{mask:N()}],"mask-size":[{mask:X()}],"mask-type":[{"mask-type":["alpha","luminance"]}],"mask-image":[{mask:["none",ge,me]}],filter:[{filter:["","none",ge,me]}],blur:[{blur:Pe()}],brightness:[{brightness:[Be,ge,me]}],contrast:[{contrast:[Be,ge,me]}],"drop-shadow":[{"drop-shadow":["","none",b,cf,sf]}],"drop-shadow-color":[{"drop-shadow":U()}],grayscale:[{grayscale:["",Be,ge,me]}],"hue-rotate":[{"hue-rotate":[Be,ge,me]}],invert:[{invert:["",Be,ge,me]}],saturate:[{saturate:[Be,ge,me]}],sepia:[{sepia:["",Be,ge,me]}],"backdrop-filter":[{"backdrop-filter":["","none",ge,me]}],"backdrop-blur":[{"backdrop-blur":Pe()}],"backdrop-brightness":[{"backdrop-brightness":[Be,ge,me]}],"backdrop-contrast":[{"backdrop-contrast":[Be,ge,me]}],"backdrop-grayscale":[{"backdrop-grayscale":["",Be,ge,me]}],"backdrop-hue-rotate":[{"backdrop-hue-rotate":[Be,ge,me]}],"backdrop-invert":[{"backdrop-invert":["",Be,ge,me]}],"backdrop-opacity":[{"backdrop-opacity":[Be,ge,me]}],"backdrop-saturate":[{"backdrop-saturate":[Be,ge,me]}],"backdrop-sepia":[{"backdrop-sepia":["",Be,ge,me]}],"border-collapse":[{border:["collapse","separate"]}],"border-spacing":[{"border-spacing":R()}],"border-spacing-x":[{"border-spacing-x":R()}],"border-spacing-y":[{"border-spacing-y":R()}],"table-layout":[{table:["auto","fixed"]}],caption:[{caption:["top","bottom"]}],transition:[{transition:["","all","colors","opacity","shadow","transform","none",ge,me]}],"transition-behavior":[{transition:["normal","discrete"]}],duration:[{duration:[Be,"initial",ge,me]}],ease:[{ease:["linear","initial",E,ge,me]}],delay:[{delay:[Be,ge,me]}],animate:[{animate:["none",M,ge,me]}],backface:[{backface:["hidden","visible"]}],perspective:[{perspective:[_,ge,me]}],"perspective-origin":[{"perspective-origin":w()}],rotate:[{rotate:fe()}],"rotate-x":[{"rotate-x":fe()}],"rotate-y":[{"rotate-y":fe()}],"rotate-z":[{"rotate-z":fe()}],scale:[{scale:Oe()}],"scale-x":[{"scale-x":Oe()}],"scale-y":[{"scale-y":Oe()}],"scale-z":[{"scale-z":Oe()}],"scale-3d":["scale-3d"],skew:[{skew:Me()}],"skew-x":[{"skew-x":Me()}],"skew-y":[{"skew-y":Me()}],transform:[{transform:[ge,me,"","none","gpu","cpu"]}],"transform-origin":[{origin:w()}],"transform-style":[{transform:["3d","flat"]}],translate:[{translate:le()}],"translate-x":[{"translate-x":le()}],"translate-y":[{"translate-y":le()}],"translate-z":[{"translate-z":le()}],"translate-none":["translate-none"],accent:[{accent:U()}],appearance:[{appearance:["none","auto"]}],"caret-color":[{caret:U()}],"color-scheme":[{scheme:["normal","dark","light","light-dark","only-dark","only-light"]}],cursor:[{cursor:["auto","default","pointer","wait","text","move","help","not-allowed","none","context-menu","progress","cell","crosshair","vertical-text","alias","copy","no-drop","grab","grabbing","all-scroll","col-resize","row-resize","n-resize","e-resize","s-resize","w-resize","ne-resize","nw-resize","se-resize","sw-resize","ew-resize","ns-resize","nesw-resize","nwse-resize","zoom-in","zoom-out",ge,me]}],"field-sizing":[{"field-sizing":["fixed","content"]}],"pointer-events":[{"pointer-events":["auto","none"]}],resize:[{resize:["none","","y","x"]}],"scroll-behavior":[{scroll:["auto","smooth"]}],"scroll-m":[{"scroll-m":R()}],"scroll-mx":[{"scroll-mx":R()}],"scroll-my":[{"scroll-my":R()}],"scroll-ms":[{"scroll-ms":R()}],"scroll-me":[{"scroll-me":R()}],"scroll-mt":[{"scroll-mt":R()}],"scroll-mr":[{"scroll-mr":R()}],"scroll-mb":[{"scroll-mb":R()}],"scroll-ml":[{"scroll-ml":R()}],"scroll-p":[{"scroll-p":R()}],"scroll-px":[{"scroll-px":R()}],"scroll-py":[{"scroll-py":R()}],"scroll-ps":[{"scroll-ps":R()}],"scroll-pe":[{"scroll-pe":R()}],"scroll-pt":[{"scroll-pt":R()}],"scroll-pr":[{"scroll-pr":R()}],"scroll-pb":[{"scroll-pb":R()}],"scroll-pl":[{"scroll-pl":R()}],"snap-align":[{snap:["start","end","center","align-none"]}],"snap-stop":[{snap:["normal","always"]}],"snap-type":[{snap:["none","x","y","both"]}],"snap-strictness":[{snap:["mandatory","proximity"]}],touch:[{touch:["auto","none","manipulation"]}],"touch-x":[{"touch-pan":["x","left","right"]}],"touch-y":[{"touch-pan":["y","up","down"]}],"touch-pz":["touch-pinch-zoom"],select:[{select:["none","text","all","auto"]}],"will-change":[{"will-change":["auto","scroll","contents","transform",ge,me]}],fill:[{fill:["none",...U()]}],"stroke-w":[{stroke:[Be,yl,oi,Bv]}],stroke:[{stroke:["none",...U()]}],"forced-color-adjust":[{"forced-color-adjust":["auto","none"]}]},conflictingClassGroups:{overflow:["overflow-x","overflow-y"],overscroll:["overscroll-x","overscroll-y"],inset:["inset-x","inset-y","start","end","top","right","bottom","left"],"inset-x":["right","left"],"inset-y":["top","bottom"],flex:["basis","grow","shrink"],gap:["gap-x","gap-y"],p:["px","py","ps","pe","pt","pr","pb","pl"],px:["pr","pl"],py:["pt","pb"],m:["mx","my","ms","me","mt","mr","mb","ml"],mx:["mr","ml"],my:["mt","mb"],size:["w","h"],"font-size":["leading"],"fvn-normal":["fvn-ordinal","fvn-slashed-zero","fvn-figure","fvn-spacing","fvn-fraction"],"fvn-ordinal":["fvn-normal"],"fvn-slashed-zero":["fvn-normal"],"fvn-figure":["fvn-normal"],"fvn-spacing":["fvn-normal"],"fvn-fraction":["fvn-normal"],"line-clamp":["display","overflow"],rounded:["rounded-s","rounded-e","rounded-t","rounded-r","rounded-b","rounded-l","rounded-ss","rounded-se","rounded-ee","rounded-es","rounded-tl","rounded-tr","rounded-br","rounded-bl"],"rounded-s":["rounded-ss","rounded-es"],"rounded-e":["rounded-se","rounded-ee"],"rounded-t":["rounded-tl","rounded-tr"],"rounded-r":["rounded-tr","rounded-br"],"rounded-b":["rounded-br","rounded-bl"],"rounded-l":["rounded-tl","rounded-bl"],"border-spacing":["border-spacing-x","border-spacing-y"],"border-w":["border-w-x","border-w-y","border-w-s","border-w-e","border-w-t","border-w-r","border-w-b","border-w-l"],"border-w-x":["border-w-r","border-w-l"],"border-w-y":["border-w-t","border-w-b"],"border-color":["border-color-x","border-color-y","border-color-s","border-color-e","border-color-t","border-color-r","border-color-b","border-color-l"],"border-color-x":["border-color-r","border-color-l"],"border-color-y":["border-color-t","border-color-b"],translate:["translate-x","translate-y","translate-none"],"translate-none":["translate","translate-x","translate-y","translate-z"],"scroll-m":["scroll-mx","scroll-my","scroll-ms","scroll-me","scroll-mt","scroll-mr","scroll-mb","scroll-ml"],"scroll-mx":["scroll-mr","scroll-ml"],"scroll-my":["scroll-mt","scroll-mb"],"scroll-p":["scroll-px","scroll-py","scroll-ps","scroll-pe","scroll-pt","scroll-pr","scroll-pb","scroll-pl"],"scroll-px":["scroll-pr","scroll-pl"],"scroll-py":["scroll-pt","scroll-pb"],touch:["touch-x","touch-y","touch-pz"],"touch-x":["touch"],"touch-y":["touch"],"touch-pz":["touch"]},conflictingClassGroupModifiers:{"font-size":["leading"]},orderSensitiveModifiers:["*","**","after","backdrop","before","details-content","file","first-letter","first-line","marker","placeholder","selection"]}},a3=B$(r3);function er(...e){return a3(Xe(e))}function i3({className:e,...t}){return V.jsx("div",{"data-slot":"card",className:er("bg-card text-card-foreground flex flex-col gap-6 rounded-xl border py-6 shadow-sm",e),...t})}function o3({className:e,...t}){return V.jsx("div",{"data-slot":"card-header",className:er("@container/card-header grid auto-rows-min grid-rows-[auto_auto] items-start gap-1.5 px-6 has-data-[slot=card-action]:grid-cols-[1fr_auto] [.border-b]:pb-6",e),...t})}function u3({className:e,...t}){return V.jsx("div",{"data-slot":"card-title",className:er("leading-none font-semibold",e),...t})}function l3({className:e,...t}){return V.jsx("div",{"data-slot":"card-description",className:er("text-muted-foreground text-sm",e),...t})}function s3({className:e,...t}){return V.jsx("div",{"data-slot":"card-content",className:er("px-6",e),...t})}function zv({className:e,type:t,...n}){return V.jsx("input",{type:t,"data-slot":"input",className:er("file:text-foreground placeholder:text-muted-foreground selection:bg-primary selection:text-primary-foreground dark:bg-input/30 border-input flex h-9 w-full min-w-0 rounded-md border bg-transparent px-3 py-1 text-base shadow-xs transition-[color,box-shadow] outline-none file:inline-flex file:h-7 file:border-0 file:bg-transparent file:text-sm file:font-medium disabled:pointer-events-none disabled:cursor-not-allowed disabled:opacity-50 md:text-sm","focus-visible:border-ring focus-visible:ring-ring/50 focus-visible:ring-[3px]","aria-invalid:ring-destructive/20 dark:aria-invalid:ring-destructive/40 aria-invalid:border-destructive",e),...n})}var Ze;(function(e){e.assertEqual=i=>i;function t(i){}e.assertIs=t;function n(i){throw new Error}e.assertNever=n,e.arrayToEnum=i=>{const u={};for(const s of i)u[s]=s;return u},e.getValidEnumValues=i=>{const u=e.objectKeys(i).filter(c=>typeof i[i[c]]!="number"),s={};for(const c of u)s[c]=i[c];return e.objectValues(s)},e.objectValues=i=>e.objectKeys(i).map(function(u){return i[u]}),e.objectKeys=typeof Object.keys=="function"?i=>Object.keys(i):i=>{const u=[];for(const s in i)Object.prototype.hasOwnProperty.call(i,s)&&u.push(s);return u},e.find=(i,u)=>{for(const s of i)if(u(s))return s},e.isInteger=typeof Number.isInteger=="function"?i=>Number.isInteger(i):i=>typeof i=="number"&&isFinite(i)&&Math.floor(i)===i;function r(i,u=" | "){return i.map(s=>typeof s=="string"?`'${s}'`:s).join(u)}e.joinValues=r,e.jsonStringifyReplacer=(i,u)=>typeof u=="bigint"?u.toString():u})(Ze||(Ze={}));var Fb;(function(e){e.mergeShapes=(t,n)=>({...t,...n})})(Fb||(Fb={}));const pe=Ze.arrayToEnum(["string","nan","number","integer","float","boolean","date","bigint","symbol","function","undefined","null","array","object","unknown","promise","void","never","map","set"]),Lr=e=>{switch(typeof e){case"undefined":return pe.undefined;case"string":return pe.string;case"number":return isNaN(e)?pe.nan:pe.number;case"boolean":return pe.boolean;case"function":return pe.function;case"bigint":return pe.bigint;case"symbol":return pe.symbol;case"object":return Array.isArray(e)?pe.array:e===null?pe.null:e.then&&typeof e.then=="function"&&e.catch&&typeof e.catch=="function"?pe.promise:typeof Map<"u"&&e instanceof Map?pe.map:typeof Set<"u"&&e instanceof Set?pe.set:typeof Date<"u"&&e instanceof Date?pe.date:pe.object;default:return pe.unknown}},se=Ze.arrayToEnum(["invalid_type","invalid_literal","custom","invalid_union","invalid_union_discriminator","invalid_enum_value","unrecognized_keys","invalid_arguments","invalid_return_type","invalid_date","invalid_string","too_small","too_big","invalid_intersection_types","not_multiple_of","not_finite"]),c3=e=>JSON.stringify(e,null,2).replace(/"([^"]+)":/g,"$1:");class On extends Error{get errors(){return this.issues}constructor(t){super(),this.issues=[],this.addIssue=r=>{this.issues=[...this.issues,r]},this.addIssues=(r=[])=>{this.issues=[...this.issues,...r]};const n=new.target.prototype;Object.setPrototypeOf?Object.setPrototypeOf(this,n):this.__proto__=n,this.name="ZodError",this.issues=t}format(t){const n=t||function(u){return u.message},r={_errors:[]},i=u=>{for(const s of u.issues)if(s.code==="invalid_union")s.unionErrors.map(i);else if(s.code==="invalid_return_type")i(s.returnTypeError);else if(s.code==="invalid_arguments")i(s.argumentsError);else if(s.path.length===0)r._errors.push(n(s));else{let c=r,d=0;for(;d<s.path.length;){const h=s.path[d];d===s.path.length-1?(c[h]=c[h]||{_errors:[]},c[h]._errors.push(n(s))):c[h]=c[h]||{_errors:[]},c=c[h],d++}}};return i(this),r}static assert(t){if(!(t instanceof On))throw new Error(`Not a ZodError: ${t}`)}toString(){return this.message}get message(){return JSON.stringify(this.issues,Ze.jsonStringifyReplacer,2)}get isEmpty(){return this.issues.length===0}flatten(t=n=>n.message){const n={},r=[];for(const i of this.issues)i.path.length>0?(n[i.path[0]]=n[i.path[0]]||[],n[i.path[0]].push(t(i))):r.push(t(i));return{formErrors:r,fieldErrors:n}}get formErrors(){return this.flatten()}}On.create=e=>new On(e);const qo=(e,t)=>{let n;switch(e.code){case se.invalid_type:e.received===pe.undefined?n="Required":n=`Expected ${e.expected}, received ${e.received}`;break;case se.invalid_literal:n=`Invalid literal value, expected ${JSON.stringify(e.expected,Ze.jsonStringifyReplacer)}`;break;case se.unrecognized_keys:n=`Unrecognized key(s) in object: ${Ze.joinValues(e.keys,", ")}`;break;case se.invalid_union:n="Invalid input";break;case se.invalid_union_discriminator:n=`Invalid discriminator value. Expected ${Ze.joinValues(e.options)}`;break;case se.invalid_enum_value:n=`Invalid enum value. Expected ${Ze.joinValues(e.options)}, received '${e.received}'`;break;case se.invalid_arguments:n="Invalid function arguments";break;case se.invalid_return_type:n="Invalid function return type";break;case se.invalid_date:n="Invalid date";break;case se.invalid_string:typeof e.validation=="object"?"includes"in e.validation?(n=`Invalid input: must include "${e.validation.includes}"`,typeof e.validation.position=="number"&&(n=`${n} at one or more positions greater than or equal to ${e.validation.position}`)):"startsWith"in e.validation?n=`Invalid input: must start with "${e.validation.startsWith}"`:"endsWith"in e.validation?n=`Invalid input: must end with "${e.validation.endsWith}"`:Ze.assertNever(e.validation):e.validation!=="regex"?n=`Invalid ${e.validation}`:n="Invalid";break;case se.too_small:e.type==="array"?n=`Array must contain ${e.exact?"exactly":e.inclusive?"at least":"more than"} ${e.minimum} element(s)`:e.type==="string"?n=`String must contain ${e.exact?"exactly":e.inclusive?"at least":"over"} ${e.minimum} character(s)`:e.type==="number"?n=`Number must be ${e.exact?"exactly equal to ":e.inclusive?"greater than or equal to ":"greater than "}${e.minimum}`:e.type==="date"?n=`Date must be ${e.exact?"exactly equal to ":e.inclusive?"greater than or equal to ":"greater than "}${new Date(Number(e.minimum))}`:n="Invalid input";break;case se.too_big:e.type==="array"?n=`Array must contain ${e.exact?"exactly":e.inclusive?"at most":"less than"} ${e.maximum} element(s)`:e.type==="string"?n=`String must contain ${e.exact?"exactly":e.inclusive?"at most":"under"} ${e.maximum} character(s)`:e.type==="number"?n=`Number must be ${e.exact?"exactly":e.inclusive?"less than or equal to":"less than"} ${e.maximum}`:e.type==="bigint"?n=`BigInt must be ${e.exact?"exactly":e.inclusive?"less than or equal to":"less than"} ${e.maximum}`:e.type==="date"?n=`Date must be ${e.exact?"exactly":e.inclusive?"smaller than or equal to":"smaller than"} ${new Date(Number(e.maximum))}`:n="Invalid input";break;case se.custom:n="Invalid input";break;case se.invalid_intersection_types:n="Intersection results could not be merged";break;case se.not_multiple_of:n=`Number must be a multiple of ${e.multipleOf}`;break;case se.not_finite:n="Number must be finite";break;default:n=t.defaultError,Ze.assertNever(e)}return{message:n}};let Z2=qo;function f3(e){Z2=e}function Pf(){return Z2}const Rf=e=>{const{data:t,path:n,errorMaps:r,issueData:i}=e,u=[...n,...i.path||[]],s={...i,path:u};if(i.message!==void 0)return{...i,path:u,message:i.message};let c="";const d=r.filter(h=>!!h).slice().reverse();for(const h of d)c=h(s,{data:t,defaultError:c}).message;return{...i,path:u,message:c}},d3=[];function de(e,t){const n=Pf(),r=Rf({issueData:t,data:e.data,path:e.path,errorMaps:[e.common.contextualErrorMap,e.schemaErrorMap,n,n===qo?void 0:qo].filter(i=>!!i)});e.common.issues.push(r)}class Qt{constructor(){this.value="valid"}dirty(){this.value==="valid"&&(this.value="dirty")}abort(){this.value!=="aborted"&&(this.value="aborted")}static mergeArray(t,n){const r=[];for(const i of n){if(i.status==="aborted")return Ce;i.status==="dirty"&&t.dirty(),r.push(i.value)}return{status:t.value,value:r}}static async mergeObjectAsync(t,n){const r=[];for(const i of n){const u=await i.key,s=await i.value;r.push({key:u,value:s})}return Qt.mergeObjectSync(t,r)}static mergeObjectSync(t,n){const r={};for(const i of n){const{key:u,value:s}=i;if(u.status==="aborted"||s.status==="aborted")return Ce;u.status==="dirty"&&t.dirty(),s.status==="dirty"&&t.dirty(),u.value!=="__proto__"&&(typeof s.value<"u"||i.alwaysSet)&&(r[u.value]=s.value)}return{status:t.value,value:r}}}const Ce=Object.freeze({status:"aborted"}),Eo=e=>({status:"dirty",value:e}),rn=e=>({status:"valid",value:e}),Qb=e=>e.status==="aborted",Jb=e=>e.status==="dirty",gi=e=>e.status==="valid",$l=e=>typeof Promise<"u"&&e instanceof Promise;function Nf(e,t,n,r){if(typeof t=="function"?e!==t||!0:!t.has(e))throw new TypeError("Cannot read private member from an object whose class did not declare it");return t.get(e)}function Y2(e,t,n,r,i){if(typeof t=="function"?e!==t||!0:!t.has(e))throw new TypeError("Cannot write private member to an object whose class did not declare it");return t.set(e,n),n}var xe;(function(e){e.errToObj=t=>typeof t=="string"?{message:t}:t||{},e.toString=t=>typeof t=="string"?t:t==null?void 0:t.message})(xe||(xe={}));var El,Tl;class yr{constructor(t,n,r,i){this._cachedPath=[],this.parent=t,this.data=n,this._path=r,this._key=i}get path(){return this._cachedPath.length||(this._key instanceof Array?this._cachedPath.push(...this._path,...this._key):this._cachedPath.push(...this._path,this._key)),this._cachedPath}}const RO=(e,t)=>{if(gi(t))return{success:!0,data:t.value};if(!e.common.issues.length)throw new Error("Validation failed but no issues detected.");return{success:!1,get error(){if(this._error)return this._error;const n=new On(e.common.issues);return this._error=n,this._error}}};function Ne(e){if(!e)return{};const{errorMap:t,invalid_type_error:n,required_error:r,description:i}=e;if(t&&(n||r))throw new Error(`Can't use "invalid_type_error" or "required_error" in conjunction with custom error map.`);return t?{errorMap:t,description:i}:{errorMap:(s,c)=>{var d,h;const{message:v}=e;return s.code==="invalid_enum_value"?{message:v??c.defaultError}:typeof c.data>"u"?{message:(d=v??r)!==null&&d!==void 0?d:c.defaultError}:s.code!=="invalid_type"?{message:c.defaultError}:{message:(h=v??n)!==null&&h!==void 0?h:c.defaultError}},description:i}}class qe{get description(){return this._def.description}_getType(t){return Lr(t.data)}_getOrReturnCtx(t,n){return n||{common:t.parent.common,data:t.data,parsedType:Lr(t.data),schemaErrorMap:this._def.errorMap,path:t.path,parent:t.parent}}_processInputParams(t){return{status:new Qt,ctx:{common:t.parent.common,data:t.data,parsedType:Lr(t.data),schemaErrorMap:this._def.errorMap,path:t.path,parent:t.parent}}}_parseSync(t){const n=this._parse(t);if($l(n))throw new Error("Synchronous parse encountered promise.");return n}_parseAsync(t){const n=this._parse(t);return Promise.resolve(n)}parse(t,n){const r=this.safeParse(t,n);if(r.success)return r.data;throw r.error}safeParse(t,n){var r;const i={common:{issues:[],async:(r=n==null?void 0:n.async)!==null&&r!==void 0?r:!1,contextualErrorMap:n==null?void 0:n.errorMap},path:(n==null?void 0:n.path)||[],schemaErrorMap:this._def.errorMap,parent:null,data:t,parsedType:Lr(t)},u=this._parseSync({data:t,path:i.path,parent:i});return RO(i,u)}"~validate"(t){var n,r;const i={common:{issues:[],async:!!this["~standard"].async},path:[],schemaErrorMap:this._def.errorMap,parent:null,data:t,parsedType:Lr(t)};if(!this["~standard"].async)try{const u=this._parseSync({data:t,path:[],parent:i});return gi(u)?{value:u.value}:{issues:i.common.issues}}catch(u){!((r=(n=u==null?void 0:u.message)===null||n===void 0?void 0:n.toLowerCase())===null||r===void 0)&&r.includes("encountered")&&(this["~standard"].async=!0),i.common={issues:[],async:!0}}return this._parseAsync({data:t,path:[],parent:i}).then(u=>gi(u)?{value:u.value}:{issues:i.common.issues})}async parseAsync(t,n){const r=await this.safeParseAsync(t,n);if(r.success)return r.data;throw r.error}async safeParseAsync(t,n){const r={common:{issues:[],contextualErrorMap:n==null?void 0:n.errorMap,async:!0},path:(n==null?void 0:n.path)||[],schemaErrorMap:this._def.errorMap,parent:null,data:t,parsedType:Lr(t)},i=this._parse({data:t,path:r.path,parent:r}),u=await($l(i)?i:Promise.resolve(i));return RO(r,u)}refine(t,n){const r=i=>typeof n=="string"||typeof n>"u"?{message:n}:typeof n=="function"?n(i):n;return this._refinement((i,u)=>{const s=t(i),c=()=>u.addIssue({code:se.custom,...r(i)});return typeof Promise<"u"&&s instanceof Promise?s.then(d=>d?!0:(c(),!1)):s?!0:(c(),!1)})}refinement(t,n){return this._refinement((r,i)=>t(r)?!0:(i.addIssue(typeof n=="function"?n(r,i):n),!1))}_refinement(t){return new Jn({schema:this,typeName:Te.ZodEffects,effect:{type:"refinement",refinement:t}})}superRefine(t){return this._refinement(t)}constructor(t){this.spa=this.safeParseAsync,this._def=t,this.parse=this.parse.bind(this),this.safeParse=this.safeParse.bind(this),this.parseAsync=this.parseAsync.bind(this),this.safeParseAsync=this.safeParseAsync.bind(this),this.spa=this.spa.bind(this),this.refine=this.refine.bind(this),this.refinement=this.refinement.bind(this),this.superRefine=this.superRefine.bind(this),this.optional=this.optional.bind(this),this.nullable=this.nullable.bind(this),this.nullish=this.nullish.bind(this),this.array=this.array.bind(this),this.promise=this.promise.bind(this),this.or=this.or.bind(this),this.and=this.and.bind(this),this.transform=this.transform.bind(this),this.brand=this.brand.bind(this),this.default=this.default.bind(this),this.catch=this.catch.bind(this),this.describe=this.describe.bind(this),this.pipe=this.pipe.bind(this),this.readonly=this.readonly.bind(this),this.isNullable=this.isNullable.bind(this),this.isOptional=this.isOptional.bind(this),this["~standard"]={version:1,vendor:"zod",validate:n=>this["~validate"](n)}}optional(){return hr.create(this,this._def)}nullable(){return ka.create(this,this._def)}nullish(){return this.nullable().optional()}array(){return Qn.create(this)}promise(){return Bo.create(this,this._def)}or(t){return Il.create([this,t],this._def)}and(t){return Ul.create(this,t,this._def)}transform(t){return new Jn({...Ne(this._def),schema:this,typeName:Te.ZodEffects,effect:{type:"transform",transform:t}})}default(t){const n=typeof t=="function"?t:()=>t;return new Yl({...Ne(this._def),innerType:this,defaultValue:n,typeName:Te.ZodDefault})}brand(){return new Ox({typeName:Te.ZodBranded,type:this,...Ne(this._def)})}catch(t){const n=typeof t=="function"?t:()=>t;return new Xl({...Ne(this._def),innerType:this,catchValue:n,typeName:Te.ZodCatch})}describe(t){const n=this.constructor;return new n({...this._def,description:t})}pipe(t){return Rs.create(this,t)}readonly(){return Kl.create(this)}isOptional(){return this.safeParse(void 0).success}isNullable(){return this.safeParse(null).success}}const h3=/^c[^\s-]{8,}$/i,p3=/^[0-9a-z]+$/,v3=/^[0-9A-HJKMNP-TV-Z]{26}$/i,y3=/^[0-9a-fA-F]{8}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{12}$/i,m3=/^[a-z0-9_-]{21}$/i,g3=/^[A-Za-z0-9-_]+\.[A-Za-z0-9-_]+\.[A-Za-z0-9-_]*$/,b3=/^[-+]?P(?!$)(?:(?:[-+]?\d+Y)|(?:[-+]?\d+[.,]\d+Y$))?(?:(?:[-+]?\d+M)|(?:[-+]?\d+[.,]\d+M$))?(?:(?:[-+]?\d+W)|(?:[-+]?\d+[.,]\d+W$))?(?:(?:[-+]?\d+D)|(?:[-+]?\d+[.,]\d+D$))?(?:T(?=[\d+-])(?:(?:[-+]?\d+H)|(?:[-+]?\d+[.,]\d+H$))?(?:(?:[-+]?\d+M)|(?:[-+]?\d+[.,]\d+M$))?(?:[-+]?\d+(?:[.,]\d+)?S)?)??$/,x3=/^(?!\.)(?!.*\.\.)([A-Z0-9_'+\-\.]*)[A-Z0-9_+-]@([A-Z0-9][A-Z0-9\-]*\.)+[A-Z]{2,}$/i,_3="^(\\p{Extended_Pictographic}|\\p{Emoji_Component})+$";let Lv;const S3=/^(?:(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9][0-9]|[0-9])\.){3}(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9][0-9]|[0-9])$/,w3=/^(?:(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9][0-9]|[0-9])\.){3}(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9][0-9]|[0-9])\/(3[0-2]|[12]?[0-9])$/,O3=/^(([0-9a-fA-F]{1,4}:){7,7}[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,7}:|([0-9a-fA-F]{1,4}:){1,6}:[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,5}(:[0-9a-fA-F]{1,4}){1,2}|([0-9a-fA-F]{1,4}:){1,4}(:[0-9a-fA-F]{1,4}){1,3}|([0-9a-fA-F]{1,4}:){1,3}(:[0-9a-fA-F]{1,4}){1,4}|([0-9a-fA-F]{1,4}:){1,2}(:[0-9a-fA-F]{1,4}){1,5}|[0-9a-fA-F]{1,4}:((:[0-9a-fA-F]{1,4}){1,6})|:((:[0-9a-fA-F]{1,4}){1,7}|:)|fe80:(:[0-9a-fA-F]{0,4}){0,4}%[0-9a-zA-Z]{1,}|::(ffff(:0{1,4}){0,1}:){0,1}((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])|([0-9a-fA-F]{1,4}:){1,4}:((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9]))$/,A3=/^(([0-9a-fA-F]{1,4}:){7,7}[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,7}:|([0-9a-fA-F]{1,4}:){1,6}:[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,5}(:[0-9a-fA-F]{1,4}){1,2}|([0-9a-fA-F]{1,4}:){1,4}(:[0-9a-fA-F]{1,4}){1,3}|([0-9a-fA-F]{1,4}:){1,3}(:[0-9a-fA-F]{1,4}){1,4}|([0-9a-fA-F]{1,4}:){1,2}(:[0-9a-fA-F]{1,4}){1,5}|[0-9a-fA-F]{1,4}:((:[0-9a-fA-F]{1,4}){1,6})|:((:[0-9a-fA-F]{1,4}){1,7}|:)|fe80:(:[0-9a-fA-F]{0,4}){0,4}%[0-9a-zA-Z]{1,}|::(ffff(:0{1,4}){0,1}:){0,1}((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])|([0-9a-fA-F]{1,4}:){1,4}:((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9]))\/(12[0-8]|1[01][0-9]|[1-9]?[0-9])$/,E3=/^([0-9a-zA-Z+/]{4})*(([0-9a-zA-Z+/]{2}==)|([0-9a-zA-Z+/]{3}=))?$/,T3=/^([0-9a-zA-Z-_]{4})*(([0-9a-zA-Z-_]{2}(==)?)|([0-9a-zA-Z-_]{3}(=)?))?$/,X2="((\\d\\d[2468][048]|\\d\\d[13579][26]|\\d\\d0[48]|[02468][048]00|[13579][26]00)-02-29|\\d{4}-((0[13578]|1[02])-(0[1-9]|[12]\\d|3[01])|(0[469]|11)-(0[1-9]|[12]\\d|30)|(02)-(0[1-9]|1\\d|2[0-8])))",M3=new RegExp(`^${X2}$`);function K2(e){let t="([01]\\d|2[0-3]):[0-5]\\d:[0-5]\\d";return e.precision?t=`${t}\\.\\d{${e.precision}}`:e.precision==null&&(t=`${t}(\\.\\d+)?`),t}function j3(e){return new RegExp(`^${K2(e)}$`)}function W2(e){let t=`${X2}T${K2(e)}`;const n=[];return n.push(e.local?"Z?":"Z"),e.offset&&n.push("([+-]\\d{2}:?\\d{2})"),t=`${t}(${n.join("|")})`,new RegExp(`^${t}$`)}function C3(e,t){return!!((t==="v4"||!t)&&S3.test(e)||(t==="v6"||!t)&&O3.test(e))}function P3(e,t){if(!g3.test(e))return!1;try{const[n]=e.split("."),r=n.replace(/-/g,"+").replace(/_/g,"/").padEnd(n.length+(4-n.length%4)%4,"="),i=JSON.parse(atob(r));return!(typeof i!="object"||i===null||!i.typ||!i.alg||t&&i.alg!==t)}catch{return!1}}function R3(e,t){return!!((t==="v4"||!t)&&w3.test(e)||(t==="v6"||!t)&&A3.test(e))}class Wn extends qe{_parse(t){if(this._def.coerce&&(t.data=String(t.data)),this._getType(t)!==pe.string){const u=this._getOrReturnCtx(t);return de(u,{code:se.invalid_type,expected:pe.string,received:u.parsedType}),Ce}const r=new Qt;let i;for(const u of this._def.checks)if(u.kind==="min")t.data.length<u.value&&(i=this._getOrReturnCtx(t,i),de(i,{code:se.too_small,minimum:u.value,type:"string",inclusive:!0,exact:!1,message:u.message}),r.dirty());else if(u.kind==="max")t.data.length>u.value&&(i=this._getOrReturnCtx(t,i),de(i,{code:se.too_big,maximum:u.value,type:"string",inclusive:!0,exact:!1,message:u.message}),r.dirty());else if(u.kind==="length"){const s=t.data.length>u.value,c=t.data.length<u.value;(s||c)&&(i=this._getOrReturnCtx(t,i),s?de(i,{code:se.too_big,maximum:u.value,type:"string",inclusive:!0,exact:!0,message:u.message}):c&&de(i,{code:se.too_small,minimum:u.value,type:"string",inclusive:!0,exact:!0,message:u.message}),r.dirty())}else if(u.kind==="email")x3.test(t.data)||(i=this._getOrReturnCtx(t,i),de(i,{validation:"email",code:se.invalid_string,message:u.message}),r.dirty());else if(u.kind==="emoji")Lv||(Lv=new RegExp(_3,"u")),Lv.test(t.data)||(i=this._getOrReturnCtx(t,i),de(i,{validation:"emoji",code:se.invalid_string,message:u.message}),r.dirty());else if(u.kind==="uuid")y3.test(t.data)||(i=this._getOrReturnCtx(t,i),de(i,{validation:"uuid",code:se.invalid_string,message:u.message}),r.dirty());else if(u.kind==="nanoid")m3.test(t.data)||(i=this._getOrReturnCtx(t,i),de(i,{validation:"nanoid",code:se.invalid_string,message:u.message}),r.dirty());else if(u.kind==="cuid")h3.test(t.data)||(i=this._getOrReturnCtx(t,i),de(i,{validation:"cuid",code:se.invalid_string,message:u.message}),r.dirty());else if(u.kind==="cuid2")p3.test(t.data)||(i=this._getOrReturnCtx(t,i),de(i,{validation:"cuid2",code:se.invalid_string,message:u.message}),r.dirty());else if(u.kind==="ulid")v3.test(t.data)||(i=this._getOrReturnCtx(t,i),de(i,{validation:"ulid",code:se.invalid_string,message:u.message}),r.dirty());else if(u.kind==="url")try{new URL(t.data)}catch{i=this._getOrReturnCtx(t,i),de(i,{validation:"url",code:se.invalid_string,message:u.message}),r.dirty()}else u.kind==="regex"?(u.regex.lastIndex=0,u.regex.test(t.data)||(i=this._getOrReturnCtx(t,i),de(i,{validation:"regex",code:se.invalid_string,message:u.message}),r.dirty())):u.kind==="trim"?t.data=t.data.trim():u.kind==="includes"?t.data.includes(u.value,u.position)||(i=this._getOrReturnCtx(t,i),de(i,{code:se.invalid_string,validation:{includes:u.value,position:u.position},message:u.message}),r.dirty()):u.kind==="toLowerCase"?t.data=t.data.toLowerCase():u.kind==="toUpperCase"?t.data=t.data.toUpperCase():u.kind==="startsWith"?t.data.startsWith(u.value)||(i=this._getOrReturnCtx(t,i),de(i,{code:se.invalid_string,validation:{startsWith:u.value},message:u.message}),r.dirty()):u.kind==="endsWith"?t.data.endsWith(u.value)||(i=this._getOrReturnCtx(t,i),de(i,{code:se.invalid_string,validation:{endsWith:u.value},message:u.message}),r.dirty()):u.kind==="datetime"?W2(u).test(t.data)||(i=this._getOrReturnCtx(t,i),de(i,{code:se.invalid_string,validation:"datetime",message:u.message}),r.dirty()):u.kind==="date"?M3.test(t.data)||(i=this._getOrReturnCtx(t,i),de(i,{code:se.invalid_string,validation:"date",message:u.message}),r.dirty()):u.kind==="time"?j3(u).test(t.data)||(i=this._getOrReturnCtx(t,i),de(i,{code:se.invalid_string,validation:"time",message:u.message}),r.dirty()):u.kind==="duration"?b3.test(t.data)||(i=this._getOrReturnCtx(t,i),de(i,{validation:"duration",code:se.invalid_string,message:u.message}),r.dirty()):u.kind==="ip"?C3(t.data,u.version)||(i=this._getOrReturnCtx(t,i),de(i,{validation:"ip",code:se.invalid_string,message:u.message}),r.dirty()):u.kind==="jwt"?P3(t.data,u.alg)||(i=this._getOrReturnCtx(t,i),de(i,{validation:"jwt",code:se.invalid_string,message:u.message}),r.dirty()):u.kind==="cidr"?R3(t.data,u.version)||(i=this._getOrReturnCtx(t,i),de(i,{validation:"cidr",code:se.invalid_string,message:u.message}),r.dirty()):u.kind==="base64"?E3.test(t.data)||(i=this._getOrReturnCtx(t,i),de(i,{validation:"base64",code:se.invalid_string,message:u.message}),r.dirty()):u.kind==="base64url"?T3.test(t.data)||(i=this._getOrReturnCtx(t,i),de(i,{validation:"base64url",code:se.invalid_string,message:u.message}),r.dirty()):Ze.assertNever(u);return{status:r.value,value:t.data}}_regex(t,n,r){return this.refinement(i=>t.test(i),{validation:n,code:se.invalid_string,...xe.errToObj(r)})}_addCheck(t){return new Wn({...this._def,checks:[...this._def.checks,t]})}email(t){return this._addCheck({kind:"email",...xe.errToObj(t)})}url(t){return this._addCheck({kind:"url",...xe.errToObj(t)})}emoji(t){return this._addCheck({kind:"emoji",...xe.errToObj(t)})}uuid(t){return this._addCheck({kind:"uuid",...xe.errToObj(t)})}nanoid(t){return this._addCheck({kind:"nanoid",...xe.errToObj(t)})}cuid(t){return this._addCheck({kind:"cuid",...xe.errToObj(t)})}cuid2(t){return this._addCheck({kind:"cuid2",...xe.errToObj(t)})}ulid(t){return this._addCheck({kind:"ulid",...xe.errToObj(t)})}base64(t){return this._addCheck({kind:"base64",...xe.errToObj(t)})}base64url(t){return this._addCheck({kind:"base64url",...xe.errToObj(t)})}jwt(t){return this._addCheck({kind:"jwt",...xe.errToObj(t)})}ip(t){return this._addCheck({kind:"ip",...xe.errToObj(t)})}cidr(t){return this._addCheck({kind:"cidr",...xe.errToObj(t)})}datetime(t){var n,r;return typeof t=="string"?this._addCheck({kind:"datetime",precision:null,offset:!1,local:!1,message:t}):this._addCheck({kind:"datetime",precision:typeof(t==null?void 0:t.precision)>"u"?null:t==null?void 0:t.precision,offset:(n=t==null?void 0:t.offset)!==null&&n!==void 0?n:!1,local:(r=t==null?void 0:t.local)!==null&&r!==void 0?r:!1,...xe.errToObj(t==null?void 0:t.message)})}date(t){return this._addCheck({kind:"date",message:t})}time(t){return typeof t=="string"?this._addCheck({kind:"time",precision:null,message:t}):this._addCheck({kind:"time",precision:typeof(t==null?void 0:t.precision)>"u"?null:t==null?void 0:t.precision,...xe.errToObj(t==null?void 0:t.message)})}duration(t){return this._addCheck({kind:"duration",...xe.errToObj(t)})}regex(t,n){return this._addCheck({kind:"regex",regex:t,...xe.errToObj(n)})}includes(t,n){return this._addCheck({kind:"includes",value:t,position:n==null?void 0:n.position,...xe.errToObj(n==null?void 0:n.message)})}startsWith(t,n){return this._addCheck({kind:"startsWith",value:t,...xe.errToObj(n)})}endsWith(t,n){return this._addCheck({kind:"endsWith",value:t,...xe.errToObj(n)})}min(t,n){return this._addCheck({kind:"min",value:t,...xe.errToObj(n)})}max(t,n){return this._addCheck({kind:"max",value:t,...xe.errToObj(n)})}length(t,n){return this._addCheck({kind:"length",value:t,...xe.errToObj(n)})}nonempty(t){return this.min(1,xe.errToObj(t))}trim(){return new Wn({...this._def,checks:[...this._def.checks,{kind:"trim"}]})}toLowerCase(){return new Wn({...this._def,checks:[...this._def.checks,{kind:"toLowerCase"}]})}toUpperCase(){return new Wn({...this._def,checks:[...this._def.checks,{kind:"toUpperCase"}]})}get isDatetime(){return!!this._def.checks.find(t=>t.kind==="datetime")}get isDate(){return!!this._def.checks.find(t=>t.kind==="date")}get isTime(){return!!this._def.checks.find(t=>t.kind==="time")}get isDuration(){return!!this._def.checks.find(t=>t.kind==="duration")}get isEmail(){return!!this._def.checks.find(t=>t.kind==="email")}get isURL(){return!!this._def.checks.find(t=>t.kind==="url")}get isEmoji(){return!!this._def.checks.find(t=>t.kind==="emoji")}get isUUID(){return!!this._def.checks.find(t=>t.kind==="uuid")}get isNANOID(){return!!this._def.checks.find(t=>t.kind==="nanoid")}get isCUID(){return!!this._def.checks.find(t=>t.kind==="cuid")}get isCUID2(){return!!this._def.checks.find(t=>t.kind==="cuid2")}get isULID(){return!!this._def.checks.find(t=>t.kind==="ulid")}get isIP(){return!!this._def.checks.find(t=>t.kind==="ip")}get isCIDR(){return!!this._def.checks.find(t=>t.kind==="cidr")}get isBase64(){return!!this._def.checks.find(t=>t.kind==="base64")}get isBase64url(){return!!this._def.checks.find(t=>t.kind==="base64url")}get minLength(){let t=null;for(const n of this._def.checks)n.kind==="min"&&(t===null||n.value>t)&&(t=n.value);return t}get maxLength(){let t=null;for(const n of this._def.checks)n.kind==="max"&&(t===null||n.value<t)&&(t=n.value);return t}}Wn.create=e=>{var t;return new Wn({checks:[],typeName:Te.ZodString,coerce:(t=e==null?void 0:e.coerce)!==null&&t!==void 0?t:!1,...Ne(e)})};function N3(e,t){const n=(e.toString().split(".")[1]||"").length,r=(t.toString().split(".")[1]||"").length,i=n>r?n:r,u=parseInt(e.toFixed(i).replace(".","")),s=parseInt(t.toFixed(i).replace(".",""));return u%s/Math.pow(10,i)}class Ra extends qe{constructor(){super(...arguments),this.min=this.gte,this.max=this.lte,this.step=this.multipleOf}_parse(t){if(this._def.coerce&&(t.data=Number(t.data)),this._getType(t)!==pe.number){const u=this._getOrReturnCtx(t);return de(u,{code:se.invalid_type,expected:pe.number,received:u.parsedType}),Ce}let r;const i=new Qt;for(const u of this._def.checks)u.kind==="int"?Ze.isInteger(t.data)||(r=this._getOrReturnCtx(t,r),de(r,{code:se.invalid_type,expected:"integer",received:"float",message:u.message}),i.dirty()):u.kind==="min"?(u.inclusive?t.data<u.value:t.data<=u.value)&&(r=this._getOrReturnCtx(t,r),de(r,{code:se.too_small,minimum:u.value,type:"number",inclusive:u.inclusive,exact:!1,message:u.message}),i.dirty()):u.kind==="max"?(u.inclusive?t.data>u.value:t.data>=u.value)&&(r=this._getOrReturnCtx(t,r),de(r,{code:se.too_big,maximum:u.value,type:"number",inclusive:u.inclusive,exact:!1,message:u.message}),i.dirty()):u.kind==="multipleOf"?N3(t.data,u.value)!==0&&(r=this._getOrReturnCtx(t,r),de(r,{code:se.not_multiple_of,multipleOf:u.value,message:u.message}),i.dirty()):u.kind==="finite"?Number.isFinite(t.data)||(r=this._getOrReturnCtx(t,r),de(r,{code:se.not_finite,message:u.message}),i.dirty()):Ze.assertNever(u);return{status:i.value,value:t.data}}gte(t,n){return this.setLimit("min",t,!0,xe.toString(n))}gt(t,n){return this.setLimit("min",t,!1,xe.toString(n))}lte(t,n){return this.setLimit("max",t,!0,xe.toString(n))}lt(t,n){return this.setLimit("max",t,!1,xe.toString(n))}setLimit(t,n,r,i){return new Ra({...this._def,checks:[...this._def.checks,{kind:t,value:n,inclusive:r,message:xe.toString(i)}]})}_addCheck(t){return new Ra({...this._def,checks:[...this._def.checks,t]})}int(t){return this._addCheck({kind:"int",message:xe.toString(t)})}positive(t){return this._addCheck({kind:"min",value:0,inclusive:!1,message:xe.toString(t)})}negative(t){return this._addCheck({kind:"max",value:0,inclusive:!1,message:xe.toString(t)})}nonpositive(t){return this._addCheck({kind:"max",value:0,inclusive:!0,message:xe.toString(t)})}nonnegative(t){return this._addCheck({kind:"min",value:0,inclusive:!0,message:xe.toString(t)})}multipleOf(t,n){return this._addCheck({kind:"multipleOf",value:t,message:xe.toString(n)})}finite(t){return this._addCheck({kind:"finite",message:xe.toString(t)})}safe(t){return this._addCheck({kind:"min",inclusive:!0,value:Number.MIN_SAFE_INTEGER,message:xe.toString(t)})._addCheck({kind:"max",inclusive:!0,value:Number.MAX_SAFE_INTEGER,message:xe.toString(t)})}get minValue(){let t=null;for(const n of this._def.checks)n.kind==="min"&&(t===null||n.value>t)&&(t=n.value);return t}get maxValue(){let t=null;for(const n of this._def.checks)n.kind==="max"&&(t===null||n.value<t)&&(t=n.value);return t}get isInt(){return!!this._def.checks.find(t=>t.kind==="int"||t.kind==="multipleOf"&&Ze.isInteger(t.value))}get isFinite(){let t=null,n=null;for(const r of this._def.checks){if(r.kind==="finite"||r.kind==="int"||r.kind==="multipleOf")return!0;r.kind==="min"?(n===null||r.value>n)&&(n=r.value):r.kind==="max"&&(t===null||r.value<t)&&(t=r.value)}return Number.isFinite(n)&&Number.isFinite(t)}}Ra.create=e=>new Ra({checks:[],typeName:Te.ZodNumber,coerce:(e==null?void 0:e.coerce)||!1,...Ne(e)});class Na extends qe{constructor(){super(...arguments),this.min=this.gte,this.max=this.lte}_parse(t){if(this._def.coerce)try{t.data=BigInt(t.data)}catch{return this._getInvalidInput(t)}if(this._getType(t)!==pe.bigint)return this._getInvalidInput(t);let r;const i=new Qt;for(const u of this._def.checks)u.kind==="min"?(u.inclusive?t.data<u.value:t.data<=u.value)&&(r=this._getOrReturnCtx(t,r),de(r,{code:se.too_small,type:"bigint",minimum:u.value,inclusive:u.inclusive,message:u.message}),i.dirty()):u.kind==="max"?(u.inclusive?t.data>u.value:t.data>=u.value)&&(r=this._getOrReturnCtx(t,r),de(r,{code:se.too_big,type:"bigint",maximum:u.value,inclusive:u.inclusive,message:u.message}),i.dirty()):u.kind==="multipleOf"?t.data%u.value!==BigInt(0)&&(r=this._getOrReturnCtx(t,r),de(r,{code:se.not_multiple_of,multipleOf:u.value,message:u.message}),i.dirty()):Ze.assertNever(u);return{status:i.value,value:t.data}}_getInvalidInput(t){const n=this._getOrReturnCtx(t);return de(n,{code:se.invalid_type,expected:pe.bigint,received:n.parsedType}),Ce}gte(t,n){return this.setLimit("min",t,!0,xe.toString(n))}gt(t,n){return this.setLimit("min",t,!1,xe.toString(n))}lte(t,n){return this.setLimit("max",t,!0,xe.toString(n))}lt(t,n){return this.setLimit("max",t,!1,xe.toString(n))}setLimit(t,n,r,i){return new Na({...this._def,checks:[...this._def.checks,{kind:t,value:n,inclusive:r,message:xe.toString(i)}]})}_addCheck(t){return new Na({...this._def,checks:[...this._def.checks,t]})}positive(t){return this._addCheck({kind:"min",value:BigInt(0),inclusive:!1,message:xe.toString(t)})}negative(t){return this._addCheck({kind:"max",value:BigInt(0),inclusive:!1,message:xe.toString(t)})}nonpositive(t){return this._addCheck({kind:"max",value:BigInt(0),inclusive:!0,message:xe.toString(t)})}nonnegative(t){return this._addCheck({kind:"min",value:BigInt(0),inclusive:!0,message:xe.toString(t)})}multipleOf(t,n){return this._addCheck({kind:"multipleOf",value:t,message:xe.toString(n)})}get minValue(){let t=null;for(const n of this._def.checks)n.kind==="min"&&(t===null||n.value>t)&&(t=n.value);return t}get maxValue(){let t=null;for(const n of this._def.checks)n.kind==="max"&&(t===null||n.value<t)&&(t=n.value);return t}}Na.create=e=>{var t;return new Na({checks:[],typeName:Te.ZodBigInt,coerce:(t=e==null?void 0:e.coerce)!==null&&t!==void 0?t:!1,...Ne(e)})};class Bl extends qe{_parse(t){if(this._def.coerce&&(t.data=!!t.data),this._getType(t)!==pe.boolean){const r=this._getOrReturnCtx(t);return de(r,{code:se.invalid_type,expected:pe.boolean,received:r.parsedType}),Ce}return rn(t.data)}}Bl.create=e=>new Bl({typeName:Te.ZodBoolean,coerce:(e==null?void 0:e.coerce)||!1,...Ne(e)});class bi extends qe{_parse(t){if(this._def.coerce&&(t.data=new Date(t.data)),this._getType(t)!==pe.date){const u=this._getOrReturnCtx(t);return de(u,{code:se.invalid_type,expected:pe.date,received:u.parsedType}),Ce}if(isNaN(t.data.getTime())){const u=this._getOrReturnCtx(t);return de(u,{code:se.invalid_date}),Ce}const r=new Qt;let i;for(const u of this._def.checks)u.kind==="min"?t.data.getTime()<u.value&&(i=this._getOrReturnCtx(t,i),de(i,{code:se.too_small,message:u.message,inclusive:!0,exact:!1,minimum:u.value,type:"date"}),r.dirty()):u.kind==="max"?t.data.getTime()>u.value&&(i=this._getOrReturnCtx(t,i),de(i,{code:se.too_big,message:u.message,inclusive:!0,exact:!1,maximum:u.value,type:"date"}),r.dirty()):Ze.assertNever(u);return{status:r.value,value:new Date(t.data.getTime())}}_addCheck(t){return new bi({...this._def,checks:[...this._def.checks,t]})}min(t,n){return this._addCheck({kind:"min",value:t.getTime(),message:xe.toString(n)})}max(t,n){return this._addCheck({kind:"max",value:t.getTime(),message:xe.toString(n)})}get minDate(){let t=null;for(const n of this._def.checks)n.kind==="min"&&(t===null||n.value>t)&&(t=n.value);return t!=null?new Date(t):null}get maxDate(){let t=null;for(const n of this._def.checks)n.kind==="max"&&(t===null||n.value<t)&&(t=n.value);return t!=null?new Date(t):null}}bi.create=e=>new bi({checks:[],coerce:(e==null?void 0:e.coerce)||!1,typeName:Te.ZodDate,...Ne(e)});class Df extends qe{_parse(t){if(this._getType(t)!==pe.symbol){const r=this._getOrReturnCtx(t);return de(r,{code:se.invalid_type,expected:pe.symbol,received:r.parsedType}),Ce}return rn(t.data)}}Df.create=e=>new Df({typeName:Te.ZodSymbol,...Ne(e)});class zl extends qe{_parse(t){if(this._getType(t)!==pe.undefined){const r=this._getOrReturnCtx(t);return de(r,{code:se.invalid_type,expected:pe.undefined,received:r.parsedType}),Ce}return rn(t.data)}}zl.create=e=>new zl({typeName:Te.ZodUndefined,...Ne(e)});class Ll extends qe{_parse(t){if(this._getType(t)!==pe.null){const r=this._getOrReturnCtx(t);return de(r,{code:se.invalid_type,expected:pe.null,received:r.parsedType}),Ce}return rn(t.data)}}Ll.create=e=>new Ll({typeName:Te.ZodNull,...Ne(e)});class $o extends qe{constructor(){super(...arguments),this._any=!0}_parse(t){return rn(t.data)}}$o.create=e=>new $o({typeName:Te.ZodAny,...Ne(e)});class pi extends qe{constructor(){super(...arguments),this._unknown=!0}_parse(t){return rn(t.data)}}pi.create=e=>new pi({typeName:Te.ZodUnknown,...Ne(e)});class Vr extends qe{_parse(t){const n=this._getOrReturnCtx(t);return de(n,{code:se.invalid_type,expected:pe.never,received:n.parsedType}),Ce}}Vr.create=e=>new Vr({typeName:Te.ZodNever,...Ne(e)});class kf extends qe{_parse(t){if(this._getType(t)!==pe.undefined){const r=this._getOrReturnCtx(t);return de(r,{code:se.invalid_type,expected:pe.void,received:r.parsedType}),Ce}return rn(t.data)}}kf.create=e=>new kf({typeName:Te.ZodVoid,...Ne(e)});class Qn extends qe{_parse(t){const{ctx:n,status:r}=this._processInputParams(t),i=this._def;if(n.parsedType!==pe.array)return de(n,{code:se.invalid_type,expected:pe.array,received:n.parsedType}),Ce;if(i.exactLength!==null){const s=n.data.length>i.exactLength.value,c=n.data.length<i.exactLength.value;(s||c)&&(de(n,{code:s?se.too_big:se.too_small,minimum:c?i.exactLength.value:void 0,maximum:s?i.exactLength.value:void 0,type:"array",inclusive:!0,exact:!0,message:i.exactLength.message}),r.dirty())}if(i.minLength!==null&&n.data.length<i.minLength.value&&(de(n,{code:se.too_small,minimum:i.minLength.value,type:"array",inclusive:!0,exact:!1,message:i.minLength.message}),r.dirty()),i.maxLength!==null&&n.data.length>i.maxLength.value&&(de(n,{code:se.too_big,maximum:i.maxLength.value,type:"array",inclusive:!0,exact:!1,message:i.maxLength.message}),r.dirty()),n.common.async)return Promise.all([...n.data].map((s,c)=>i.type._parseAsync(new yr(n,s,n.path,c)))).then(s=>Qt.mergeArray(r,s));const u=[...n.data].map((s,c)=>i.type._parseSync(new yr(n,s,n.path,c)));return Qt.mergeArray(r,u)}get element(){return this._def.type}min(t,n){return new Qn({...this._def,minLength:{value:t,message:xe.toString(n)}})}max(t,n){return new Qn({...this._def,maxLength:{value:t,message:xe.toString(n)}})}length(t,n){return new Qn({...this._def,exactLength:{value:t,message:xe.toString(n)}})}nonempty(t){return this.min(1,t)}}Qn.create=(e,t)=>new Qn({type:e,minLength:null,maxLength:null,exactLength:null,typeName:Te.ZodArray,...Ne(t)});function Oo(e){if(e instanceof mt){const t={};for(const n in e.shape){const r=e.shape[n];t[n]=hr.create(Oo(r))}return new mt({...e._def,shape:()=>t})}else return e instanceof Qn?new Qn({...e._def,type:Oo(e.element)}):e instanceof hr?hr.create(Oo(e.unwrap())):e instanceof ka?ka.create(Oo(e.unwrap())):e instanceof mr?mr.create(e.items.map(t=>Oo(t))):e}class mt extends qe{constructor(){super(...arguments),this._cached=null,this.nonstrict=this.passthrough,this.augment=this.extend}_getCached(){if(this._cached!==null)return this._cached;const t=this._def.shape(),n=Ze.objectKeys(t);return this._cached={shape:t,keys:n}}_parse(t){if(this._getType(t)!==pe.object){const h=this._getOrReturnCtx(t);return de(h,{code:se.invalid_type,expected:pe.object,received:h.parsedType}),Ce}const{status:r,ctx:i}=this._processInputParams(t),{shape:u,keys:s}=this._getCached(),c=[];if(!(this._def.catchall instanceof Vr&&this._def.unknownKeys==="strip"))for(const h in i.data)s.includes(h)||c.push(h);const d=[];for(const h of s){const v=u[h],y=i.data[h];d.push({key:{status:"valid",value:h},value:v._parse(new yr(i,y,i.path,h)),alwaysSet:h in i.data})}if(this._def.catchall instanceof Vr){const h=this._def.unknownKeys;if(h==="passthrough")for(const v of c)d.push({key:{status:"valid",value:v},value:{status:"valid",value:i.data[v]}});else if(h==="strict")c.length>0&&(de(i,{code:se.unrecognized_keys,keys:c}),r.dirty());else if(h!=="strip")throw new Error("Internal ZodObject error: invalid unknownKeys value.")}else{const h=this._def.catchall;for(const v of c){const y=i.data[v];d.push({key:{status:"valid",value:v},value:h._parse(new yr(i,y,i.path,v)),alwaysSet:v in i.data})}}return i.common.async?Promise.resolve().then(async()=>{const h=[];for(const v of d){const y=await v.key,g=await v.value;h.push({key:y,value:g,alwaysSet:v.alwaysSet})}return h}).then(h=>Qt.mergeObjectSync(r,h)):Qt.mergeObjectSync(r,d)}get shape(){return this._def.shape()}strict(t){return xe.errToObj,new mt({...this._def,unknownKeys:"strict",...t!==void 0?{errorMap:(n,r)=>{var i,u,s,c;const d=(s=(u=(i=this._def).errorMap)===null||u===void 0?void 0:u.call(i,n,r).message)!==null&&s!==void 0?s:r.defaultError;return n.code==="unrecognized_keys"?{message:(c=xe.errToObj(t).message)!==null&&c!==void 0?c:d}:{message:d}}}:{}})}strip(){return new mt({...this._def,unknownKeys:"strip"})}passthrough(){return new mt({...this._def,unknownKeys:"passthrough"})}extend(t){return new mt({...this._def,shape:()=>({...this._def.shape(),...t})})}merge(t){return new mt({unknownKeys:t._def.unknownKeys,catchall:t._def.catchall,shape:()=>({...this._def.shape(),...t._def.shape()}),typeName:Te.ZodObject})}setKey(t,n){return this.augment({[t]:n})}catchall(t){return new mt({...this._def,catchall:t})}pick(t){const n={};return Ze.objectKeys(t).forEach(r=>{t[r]&&this.shape[r]&&(n[r]=this.shape[r])}),new mt({...this._def,shape:()=>n})}omit(t){const n={};return Ze.objectKeys(this.shape).forEach(r=>{t[r]||(n[r]=this.shape[r])}),new mt({...this._def,shape:()=>n})}deepPartial(){return Oo(this)}partial(t){const n={};return Ze.objectKeys(this.shape).forEach(r=>{const i=this.shape[r];t&&!t[r]?n[r]=i:n[r]=i.optional()}),new mt({...this._def,shape:()=>n})}required(t){const n={};return Ze.objectKeys(this.shape).forEach(r=>{if(t&&!t[r])n[r]=this.shape[r];else{let u=this.shape[r];for(;u instanceof hr;)u=u._def.innerType;n[r]=u}}),new mt({...this._def,shape:()=>n})}keyof(){return F2(Ze.objectKeys(this.shape))}}mt.create=(e,t)=>new mt({shape:()=>e,unknownKeys:"strip",catchall:Vr.create(),typeName:Te.ZodObject,...Ne(t)});mt.strictCreate=(e,t)=>new mt({shape:()=>e,unknownKeys:"strict",catchall:Vr.create(),typeName:Te.ZodObject,...Ne(t)});mt.lazycreate=(e,t)=>new mt({shape:e,unknownKeys:"strip",catchall:Vr.create(),typeName:Te.ZodObject,...Ne(t)});class Il extends qe{_parse(t){const{ctx:n}=this._processInputParams(t),r=this._def.options;function i(u){for(const c of u)if(c.result.status==="valid")return c.result;for(const c of u)if(c.result.status==="dirty")return n.common.issues.push(...c.ctx.common.issues),c.result;const s=u.map(c=>new On(c.ctx.common.issues));return de(n,{code:se.invalid_union,unionErrors:s}),Ce}if(n.common.async)return Promise.all(r.map(async u=>{const s={...n,common:{...n.common,issues:[]},parent:null};return{result:await u._parseAsync({data:n.data,path:n.path,parent:s}),ctx:s}})).then(i);{let u;const s=[];for(const d of r){const h={...n,common:{...n.common,issues:[]},parent:null},v=d._parseSync({data:n.data,path:n.path,parent:h});if(v.status==="valid")return v;v.status==="dirty"&&!u&&(u={result:v,ctx:h}),h.common.issues.length&&s.push(h.common.issues)}if(u)return n.common.issues.push(...u.ctx.common.issues),u.result;const c=s.map(d=>new On(d));return de(n,{code:se.invalid_union,unionErrors:c}),Ce}}get options(){return this._def.options}}Il.create=(e,t)=>new Il({options:e,typeName:Te.ZodUnion,...Ne(t)});const zr=e=>e instanceof Gl?zr(e.schema):e instanceof Jn?zr(e.innerType()):e instanceof Vl?[e.value]:e instanceof Da?e.options:e instanceof Zl?Ze.objectValues(e.enum):e instanceof Yl?zr(e._def.innerType):e instanceof zl?[void 0]:e instanceof Ll?[null]:e instanceof hr?[void 0,...zr(e.unwrap())]:e instanceof ka?[null,...zr(e.unwrap())]:e instanceof Ox||e instanceof Kl?zr(e.unwrap()):e instanceof Xl?zr(e._def.innerType):[];class qd extends qe{_parse(t){const{ctx:n}=this._processInputParams(t);if(n.parsedType!==pe.object)return de(n,{code:se.invalid_type,expected:pe.object,received:n.parsedType}),Ce;const r=this.discriminator,i=n.data[r],u=this.optionsMap.get(i);return u?n.common.async?u._parseAsync({data:n.data,path:n.path,parent:n}):u._parseSync({data:n.data,path:n.path,parent:n}):(de(n,{code:se.invalid_union_discriminator,options:Array.from(this.optionsMap.keys()),path:[r]}),Ce)}get discriminator(){return this._def.discriminator}get options(){return this._def.options}get optionsMap(){return this._def.optionsMap}static create(t,n,r){const i=new Map;for(const u of n){const s=zr(u.shape[t]);if(!s.length)throw new Error(`A discriminator value for key \`${t}\` could not be extracted from all schema options`);for(const c of s){if(i.has(c))throw new Error(`Discriminator property ${String(t)} has duplicate value ${String(c)}`);i.set(c,u)}}return new qd({typeName:Te.ZodDiscriminatedUnion,discriminator:t,options:n,optionsMap:i,...Ne(r)})}}function e0(e,t){const n=Lr(e),r=Lr(t);if(e===t)return{valid:!0,data:e};if(n===pe.object&&r===pe.object){const i=Ze.objectKeys(t),u=Ze.objectKeys(e).filter(c=>i.indexOf(c)!==-1),s={...e,...t};for(const c of u){const d=e0(e[c],t[c]);if(!d.valid)return{valid:!1};s[c]=d.data}return{valid:!0,data:s}}else if(n===pe.array&&r===pe.array){if(e.length!==t.length)return{valid:!1};const i=[];for(let u=0;u<e.length;u++){const s=e[u],c=t[u],d=e0(s,c);if(!d.valid)return{valid:!1};i.push(d.data)}return{valid:!0,data:i}}else return n===pe.date&&r===pe.date&&+e==+t?{valid:!0,data:e}:{valid:!1}}class Ul extends qe{_parse(t){const{status:n,ctx:r}=this._processInputParams(t),i=(u,s)=>{if(Qb(u)||Qb(s))return Ce;const c=e0(u.value,s.value);return c.valid?((Jb(u)||Jb(s))&&n.dirty(),{status:n.value,value:c.data}):(de(r,{code:se.invalid_intersection_types}),Ce)};return r.common.async?Promise.all([this._def.left._parseAsync({data:r.data,path:r.path,parent:r}),this._def.right._parseAsync({data:r.data,path:r.path,parent:r})]).then(([u,s])=>i(u,s)):i(this._def.left._parseSync({data:r.data,path:r.path,parent:r}),this._def.right._parseSync({data:r.data,path:r.path,parent:r}))}}Ul.create=(e,t,n)=>new Ul({left:e,right:t,typeName:Te.ZodIntersection,...Ne(n)});class mr extends qe{_parse(t){const{status:n,ctx:r}=this._processInputParams(t);if(r.parsedType!==pe.array)return de(r,{code:se.invalid_type,expected:pe.array,received:r.parsedType}),Ce;if(r.data.length<this._def.items.length)return de(r,{code:se.too_small,minimum:this._def.items.length,inclusive:!0,exact:!1,type:"array"}),Ce;!this._def.rest&&r.data.length>this._def.items.length&&(de(r,{code:se.too_big,maximum:this._def.items.length,inclusive:!0,exact:!1,type:"array"}),n.dirty());const u=[...r.data].map((s,c)=>{const d=this._def.items[c]||this._def.rest;return d?d._parse(new yr(r,s,r.path,c)):null}).filter(s=>!!s);return r.common.async?Promise.all(u).then(s=>Qt.mergeArray(n,s)):Qt.mergeArray(n,u)}get items(){return this._def.items}rest(t){return new mr({...this._def,rest:t})}}mr.create=(e,t)=>{if(!Array.isArray(e))throw new Error("You must pass an array of schemas to z.tuple([ ... ])");return new mr({items:e,typeName:Te.ZodTuple,rest:null,...Ne(t)})};class Hl extends qe{get keySchema(){return this._def.keyType}get valueSchema(){return this._def.valueType}_parse(t){const{status:n,ctx:r}=this._processInputParams(t);if(r.parsedType!==pe.object)return de(r,{code:se.invalid_type,expected:pe.object,received:r.parsedType}),Ce;const i=[],u=this._def.keyType,s=this._def.valueType;for(const c in r.data)i.push({key:u._parse(new yr(r,c,r.path,c)),value:s._parse(new yr(r,r.data[c],r.path,c)),alwaysSet:c in r.data});return r.common.async?Qt.mergeObjectAsync(n,i):Qt.mergeObjectSync(n,i)}get element(){return this._def.valueType}static create(t,n,r){return n instanceof qe?new Hl({keyType:t,valueType:n,typeName:Te.ZodRecord,...Ne(r)}):new Hl({keyType:Wn.create(),valueType:t,typeName:Te.ZodRecord,...Ne(n)})}}class qf extends qe{get keySchema(){return this._def.keyType}get valueSchema(){return this._def.valueType}_parse(t){const{status:n,ctx:r}=this._processInputParams(t);if(r.parsedType!==pe.map)return de(r,{code:se.invalid_type,expected:pe.map,received:r.parsedType}),Ce;const i=this._def.keyType,u=this._def.valueType,s=[...r.data.entries()].map(([c,d],h)=>({key:i._parse(new yr(r,c,r.path,[h,"key"])),value:u._parse(new yr(r,d,r.path,[h,"value"]))}));if(r.common.async){const c=new Map;return Promise.resolve().then(async()=>{for(const d of s){const h=await d.key,v=await d.value;if(h.status==="aborted"||v.status==="aborted")return Ce;(h.status==="dirty"||v.status==="dirty")&&n.dirty(),c.set(h.value,v.value)}return{status:n.value,value:c}})}else{const c=new Map;for(const d of s){const h=d.key,v=d.value;if(h.status==="aborted"||v.status==="aborted")return Ce;(h.status==="dirty"||v.status==="dirty")&&n.dirty(),c.set(h.value,v.value)}return{status:n.value,value:c}}}}qf.create=(e,t,n)=>new qf({valueType:t,keyType:e,typeName:Te.ZodMap,...Ne(n)});class xi extends qe{_parse(t){const{status:n,ctx:r}=this._processInputParams(t);if(r.parsedType!==pe.set)return de(r,{code:se.invalid_type,expected:pe.set,received:r.parsedType}),Ce;const i=this._def;i.minSize!==null&&r.data.size<i.minSize.value&&(de(r,{code:se.too_small,minimum:i.minSize.value,type:"set",inclusive:!0,exact:!1,message:i.minSize.message}),n.dirty()),i.maxSize!==null&&r.data.size>i.maxSize.value&&(de(r,{code:se.too_big,maximum:i.maxSize.value,type:"set",inclusive:!0,exact:!1,message:i.maxSize.message}),n.dirty());const u=this._def.valueType;function s(d){const h=new Set;for(const v of d){if(v.status==="aborted")return Ce;v.status==="dirty"&&n.dirty(),h.add(v.value)}return{status:n.value,value:h}}const c=[...r.data.values()].map((d,h)=>u._parse(new yr(r,d,r.path,h)));return r.common.async?Promise.all(c).then(d=>s(d)):s(c)}min(t,n){return new xi({...this._def,minSize:{value:t,message:xe.toString(n)}})}max(t,n){return new xi({...this._def,maxSize:{value:t,message:xe.toString(n)}})}size(t,n){return this.min(t,n).max(t,n)}nonempty(t){return this.min(1,t)}}xi.create=(e,t)=>new xi({valueType:e,minSize:null,maxSize:null,typeName:Te.ZodSet,...Ne(t)});class Co extends qe{constructor(){super(...arguments),this.validate=this.implement}_parse(t){const{ctx:n}=this._processInputParams(t);if(n.parsedType!==pe.function)return de(n,{code:se.invalid_type,expected:pe.function,received:n.parsedType}),Ce;function r(c,d){return Rf({data:c,path:n.path,errorMaps:[n.common.contextualErrorMap,n.schemaErrorMap,Pf(),qo].filter(h=>!!h),issueData:{code:se.invalid_arguments,argumentsError:d}})}function i(c,d){return Rf({data:c,path:n.path,errorMaps:[n.common.contextualErrorMap,n.schemaErrorMap,Pf(),qo].filter(h=>!!h),issueData:{code:se.invalid_return_type,returnTypeError:d}})}const u={errorMap:n.common.contextualErrorMap},s=n.data;if(this._def.returns instanceof Bo){const c=this;return rn(async function(...d){const h=new On([]),v=await c._def.args.parseAsync(d,u).catch(b=>{throw h.addIssue(r(d,b)),h}),y=await Reflect.apply(s,this,v);return await c._def.returns._def.type.parseAsync(y,u).catch(b=>{throw h.addIssue(i(y,b)),h})})}else{const c=this;return rn(function(...d){const h=c._def.args.safeParse(d,u);if(!h.success)throw new On([r(d,h.error)]);const v=Reflect.apply(s,this,h.data),y=c._def.returns.safeParse(v,u);if(!y.success)throw new On([i(v,y.error)]);return y.data})}}parameters(){return this._def.args}returnType(){return this._def.returns}args(...t){return new Co({...this._def,args:mr.create(t).rest(pi.create())})}returns(t){return new Co({...this._def,returns:t})}implement(t){return this.parse(t)}strictImplement(t){return this.parse(t)}static create(t,n,r){return new Co({args:t||mr.create([]).rest(pi.create()),returns:n||pi.create(),typeName:Te.ZodFunction,...Ne(r)})}}class Gl extends qe{get schema(){return this._def.getter()}_parse(t){const{ctx:n}=this._processInputParams(t);return this._def.getter()._parse({data:n.data,path:n.path,parent:n})}}Gl.create=(e,t)=>new Gl({getter:e,typeName:Te.ZodLazy,...Ne(t)});class Vl extends qe{_parse(t){if(t.data!==this._def.value){const n=this._getOrReturnCtx(t);return de(n,{received:n.data,code:se.invalid_literal,expected:this._def.value}),Ce}return{status:"valid",value:t.data}}get value(){return this._def.value}}Vl.create=(e,t)=>new Vl({value:e,typeName:Te.ZodLiteral,...Ne(t)});function F2(e,t){return new Da({values:e,typeName:Te.ZodEnum,...Ne(t)})}class Da extends qe{constructor(){super(...arguments),El.set(this,void 0)}_parse(t){if(typeof t.data!="string"){const n=this._getOrReturnCtx(t),r=this._def.values;return de(n,{expected:Ze.joinValues(r),received:n.parsedType,code:se.invalid_type}),Ce}if(Nf(this,El)||Y2(this,El,new Set(this._def.values)),!Nf(this,El).has(t.data)){const n=this._getOrReturnCtx(t),r=this._def.values;return de(n,{received:n.data,code:se.invalid_enum_value,options:r}),Ce}return rn(t.data)}get options(){return this._def.values}get enum(){const t={};for(const n of this._def.values)t[n]=n;return t}get Values(){const t={};for(const n of this._def.values)t[n]=n;return t}get Enum(){const t={};for(const n of this._def.values)t[n]=n;return t}extract(t,n=this._def){return Da.create(t,{...this._def,...n})}exclude(t,n=this._def){return Da.create(this.options.filter(r=>!t.includes(r)),{...this._def,...n})}}El=new WeakMap;Da.create=F2;class Zl extends qe{constructor(){super(...arguments),Tl.set(this,void 0)}_parse(t){const n=Ze.getValidEnumValues(this._def.values),r=this._getOrReturnCtx(t);if(r.parsedType!==pe.string&&r.parsedType!==pe.number){const i=Ze.objectValues(n);return de(r,{expected:Ze.joinValues(i),received:r.parsedType,code:se.invalid_type}),Ce}if(Nf(this,Tl)||Y2(this,Tl,new Set(Ze.getValidEnumValues(this._def.values))),!Nf(this,Tl).has(t.data)){const i=Ze.objectValues(n);return de(r,{received:r.data,code:se.invalid_enum_value,options:i}),Ce}return rn(t.data)}get enum(){return this._def.values}}Tl=new WeakMap;Zl.create=(e,t)=>new Zl({values:e,typeName:Te.ZodNativeEnum,...Ne(t)});class Bo extends qe{unwrap(){return this._def.type}_parse(t){const{ctx:n}=this._processInputParams(t);if(n.parsedType!==pe.promise&&n.common.async===!1)return de(n,{code:se.invalid_type,expected:pe.promise,received:n.parsedType}),Ce;const r=n.parsedType===pe.promise?n.data:Promise.resolve(n.data);return rn(r.then(i=>this._def.type.parseAsync(i,{path:n.path,errorMap:n.common.contextualErrorMap})))}}Bo.create=(e,t)=>new Bo({type:e,typeName:Te.ZodPromise,...Ne(t)});class Jn extends qe{innerType(){return this._def.schema}sourceType(){return this._def.schema._def.typeName===Te.ZodEffects?this._def.schema.sourceType():this._def.schema}_parse(t){const{status:n,ctx:r}=this._processInputParams(t),i=this._def.effect||null,u={addIssue:s=>{de(r,s),s.fatal?n.abort():n.dirty()},get path(){return r.path}};if(u.addIssue=u.addIssue.bind(u),i.type==="preprocess"){const s=i.transform(r.data,u);if(r.common.async)return Promise.resolve(s).then(async c=>{if(n.value==="aborted")return Ce;const d=await this._def.schema._parseAsync({data:c,path:r.path,parent:r});return d.status==="aborted"?Ce:d.status==="dirty"||n.value==="dirty"?Eo(d.value):d});{if(n.value==="aborted")return Ce;const c=this._def.schema._parseSync({data:s,path:r.path,parent:r});return c.status==="aborted"?Ce:c.status==="dirty"||n.value==="dirty"?Eo(c.value):c}}if(i.type==="refinement"){const s=c=>{const d=i.refinement(c,u);if(r.common.async)return Promise.resolve(d);if(d instanceof Promise)throw new Error("Async refinement encountered during synchronous parse operation. Use .parseAsync instead.");return c};if(r.common.async===!1){const c=this._def.schema._parseSync({data:r.data,path:r.path,parent:r});return c.status==="aborted"?Ce:(c.status==="dirty"&&n.dirty(),s(c.value),{status:n.value,value:c.value})}else return this._def.schema._parseAsync({data:r.data,path:r.path,parent:r}).then(c=>c.status==="aborted"?Ce:(c.status==="dirty"&&n.dirty(),s(c.value).then(()=>({status:n.value,value:c.value}))))}if(i.type==="transform")if(r.common.async===!1){const s=this._def.schema._parseSync({data:r.data,path:r.path,parent:r});if(!gi(s))return s;const c=i.transform(s.value,u);if(c instanceof Promise)throw new Error("Asynchronous transform encountered during synchronous parse operation. Use .parseAsync instead.");return{status:n.value,value:c}}else return this._def.schema._parseAsync({data:r.data,path:r.path,parent:r}).then(s=>gi(s)?Promise.resolve(i.transform(s.value,u)).then(c=>({status:n.value,value:c})):s);Ze.assertNever(i)}}Jn.create=(e,t,n)=>new Jn({schema:e,typeName:Te.ZodEffects,effect:t,...Ne(n)});Jn.createWithPreprocess=(e,t,n)=>new Jn({schema:t,effect:{type:"preprocess",transform:e},typeName:Te.ZodEffects,...Ne(n)});class hr extends qe{_parse(t){return this._getType(t)===pe.undefined?rn(void 0):this._def.innerType._parse(t)}unwrap(){return this._def.innerType}}hr.create=(e,t)=>new hr({innerType:e,typeName:Te.ZodOptional,...Ne(t)});class ka extends qe{_parse(t){return this._getType(t)===pe.null?rn(null):this._def.innerType._parse(t)}unwrap(){return this._def.innerType}}ka.create=(e,t)=>new ka({innerType:e,typeName:Te.ZodNullable,...Ne(t)});class Yl extends qe{_parse(t){const{ctx:n}=this._processInputParams(t);let r=n.data;return n.parsedType===pe.undefined&&(r=this._def.defaultValue()),this._def.innerType._parse({data:r,path:n.path,parent:n})}removeDefault(){return this._def.innerType}}Yl.create=(e,t)=>new Yl({innerType:e,typeName:Te.ZodDefault,defaultValue:typeof t.default=="function"?t.default:()=>t.default,...Ne(t)});class Xl extends qe{_parse(t){const{ctx:n}=this._processInputParams(t),r={...n,common:{...n.common,issues:[]}},i=this._def.innerType._parse({data:r.data,path:r.path,parent:{...r}});return $l(i)?i.then(u=>({status:"valid",value:u.status==="valid"?u.value:this._def.catchValue({get error(){return new On(r.common.issues)},input:r.data})})):{status:"valid",value:i.status==="valid"?i.value:this._def.catchValue({get error(){return new On(r.common.issues)},input:r.data})}}removeCatch(){return this._def.innerType}}Xl.create=(e,t)=>new Xl({innerType:e,typeName:Te.ZodCatch,catchValue:typeof t.catch=="function"?t.catch:()=>t.catch,...Ne(t)});class $f extends qe{_parse(t){if(this._getType(t)!==pe.nan){const r=this._getOrReturnCtx(t);return de(r,{code:se.invalid_type,expected:pe.nan,received:r.parsedType}),Ce}return{status:"valid",value:t.data}}}$f.create=e=>new $f({typeName:Te.ZodNaN,...Ne(e)});const D3=Symbol("zod_brand");class Ox extends qe{_parse(t){const{ctx:n}=this._processInputParams(t),r=n.data;return this._def.type._parse({data:r,path:n.path,parent:n})}unwrap(){return this._def.type}}class Rs extends qe{_parse(t){const{status:n,ctx:r}=this._processInputParams(t);if(r.common.async)return(async()=>{const u=await this._def.in._parseAsync({data:r.data,path:r.path,parent:r});return u.status==="aborted"?Ce:u.status==="dirty"?(n.dirty(),Eo(u.value)):this._def.out._parseAsync({data:u.value,path:r.path,parent:r})})();{const i=this._def.in._parseSync({data:r.data,path:r.path,parent:r});return i.status==="aborted"?Ce:i.status==="dirty"?(n.dirty(),{status:"dirty",value:i.value}):this._def.out._parseSync({data:i.value,path:r.path,parent:r})}}static create(t,n){return new Rs({in:t,out:n,typeName:Te.ZodPipeline})}}class Kl extends qe{_parse(t){const n=this._def.innerType._parse(t),r=i=>(gi(i)&&(i.value=Object.freeze(i.value)),i);return $l(n)?n.then(i=>r(i)):r(n)}unwrap(){return this._def.innerType}}Kl.create=(e,t)=>new Kl({innerType:e,typeName:Te.ZodReadonly,...Ne(t)});function NO(e,t){const n=typeof e=="function"?e(t):typeof e=="string"?{message:e}:e;return typeof n=="string"?{message:n}:n}function Q2(e,t={},n){return e?$o.create().superRefine((r,i)=>{var u,s;const c=e(r);if(c instanceof Promise)return c.then(d=>{var h,v;if(!d){const y=NO(t,r),g=(v=(h=y.fatal)!==null&&h!==void 0?h:n)!==null&&v!==void 0?v:!0;i.addIssue({code:"custom",...y,fatal:g})}});if(!c){const d=NO(t,r),h=(s=(u=d.fatal)!==null&&u!==void 0?u:n)!==null&&s!==void 0?s:!0;i.addIssue({code:"custom",...d,fatal:h})}}):$o.create()}const k3={object:mt.lazycreate};var Te;(function(e){e.ZodString="ZodString",e.ZodNumber="ZodNumber",e.ZodNaN="ZodNaN",e.ZodBigInt="ZodBigInt",e.ZodBoolean="ZodBoolean",e.ZodDate="ZodDate",e.ZodSymbol="ZodSymbol",e.ZodUndefined="ZodUndefined",e.ZodNull="ZodNull",e.ZodAny="ZodAny",e.ZodUnknown="ZodUnknown",e.ZodNever="ZodNever",e.ZodVoid="ZodVoid",e.ZodArray="ZodArray",e.ZodObject="ZodObject",e.ZodUnion="ZodUnion",e.ZodDiscriminatedUnion="ZodDiscriminatedUnion",e.ZodIntersection="ZodIntersection",e.ZodTuple="ZodTuple",e.ZodRecord="ZodRecord",e.ZodMap="ZodMap",e.ZodSet="ZodSet",e.ZodFunction="ZodFunction",e.ZodLazy="ZodLazy",e.ZodLiteral="ZodLiteral",e.ZodEnum="ZodEnum",e.ZodEffects="ZodEffects",e.ZodNativeEnum="ZodNativeEnum",e.ZodOptional="ZodOptional",e.ZodNullable="ZodNullable",e.ZodDefault="ZodDefault",e.ZodCatch="ZodCatch",e.ZodPromise="ZodPromise",e.ZodBranded="ZodBranded",e.ZodPipeline="ZodPipeline",e.ZodReadonly="ZodReadonly"})(Te||(Te={}));const q3=(e,t={message:`Input not instance of ${e.name}`})=>Q2(n=>n instanceof e,t),J2=Wn.create,eP=Ra.create,$3=$f.create,B3=Na.create,tP=Bl.create,z3=bi.create,L3=Df.create,I3=zl.create,U3=Ll.create,H3=$o.create,G3=pi.create,V3=Vr.create,Z3=kf.create,Y3=Qn.create,X3=mt.create,K3=mt.strictCreate,W3=Il.create,F3=qd.create,Q3=Ul.create,J3=mr.create,eB=Hl.create,tB=qf.create,nB=xi.create,rB=Co.create,aB=Gl.create,iB=Vl.create,oB=Da.create,uB=Zl.create,lB=Bo.create,DO=Jn.create,sB=hr.create,cB=ka.create,fB=Jn.createWithPreprocess,dB=Rs.create,hB=()=>J2().optional(),pB=()=>eP().optional(),vB=()=>tP().optional(),yB={string:e=>Wn.create({...e,coerce:!0}),number:e=>Ra.create({...e,coerce:!0}),boolean:e=>Bl.create({...e,coerce:!0}),bigint:e=>Na.create({...e,coerce:!0}),date:e=>bi.create({...e,coerce:!0})},mB=Ce;var Re=Object.freeze({__proto__:null,defaultErrorMap:qo,setErrorMap:f3,getErrorMap:Pf,makeIssue:Rf,EMPTY_PATH:d3,addIssueToContext:de,ParseStatus:Qt,INVALID:Ce,DIRTY:Eo,OK:rn,isAborted:Qb,isDirty:Jb,isValid:gi,isAsync:$l,get util(){return Ze},get objectUtil(){return Fb},ZodParsedType:pe,getParsedType:Lr,ZodType:qe,datetimeRegex:W2,ZodString:Wn,ZodNumber:Ra,ZodBigInt:Na,ZodBoolean:Bl,ZodDate:bi,ZodSymbol:Df,ZodUndefined:zl,ZodNull:Ll,ZodAny:$o,ZodUnknown:pi,ZodNever:Vr,ZodVoid:kf,ZodArray:Qn,ZodObject:mt,ZodUnion:Il,ZodDiscriminatedUnion:qd,ZodIntersection:Ul,ZodTuple:mr,ZodRecord:Hl,ZodMap:qf,ZodSet:xi,ZodFunction:Co,ZodLazy:Gl,ZodLiteral:Vl,ZodEnum:Da,ZodNativeEnum:Zl,ZodPromise:Bo,ZodEffects:Jn,ZodTransformer:Jn,ZodOptional:hr,ZodNullable:ka,ZodDefault:Yl,ZodCatch:Xl,ZodNaN:$f,BRAND:D3,ZodBranded:Ox,ZodPipeline:Rs,ZodReadonly:Kl,custom:Q2,Schema:qe,ZodSchema:qe,late:k3,get ZodFirstPartyTypeKind(){return Te},coerce:yB,any:H3,array:Y3,bigint:B3,boolean:tP,date:z3,discriminatedUnion:F3,effect:DO,enum:oB,function:rB,instanceof:q3,intersection:Q3,lazy:aB,literal:iB,map:tB,nan:$3,nativeEnum:uB,never:V3,null:U3,nullable:cB,number:eP,object:X3,oboolean:vB,onumber:pB,optional:sB,ostring:hB,pipeline:dB,preprocess:fB,promise:lB,record:eB,set:nB,strictObject:K3,string:J2,symbol:L3,transformer:DO,tuple:J3,undefined:I3,union:W3,unknown:G3,void:Z3,NEVER:mB,ZodIssueCode:se,quotelessJson:c3,ZodError:On});const nP=Re.record(Re.string(),Re.union([Re.string(),Re.number(),Re.boolean(),Re.array(Re.string()),Re.array(Re.number()),Re.array(Re.boolean())])),gB=Re.object({created_at:Re.string(),role:Re.enum(["user","assistant"]),content:Re.string()}),rP=Re.object({chat_id:Re.string(),created_at:Re.string(),messages:Re.array(gB),metadata:nP}),bB=Re.array(rP),xB=Re.object({chat_id:Re.string(),summary:Re.string(),metadata:nP}),_B=Re.array(xB),aP=Re.object({id:Re.string(),name:Re.string(),description:Re.string(),chat_ids:Re.array(Re.string()),parent_id:Re.string().nullable(),x_coord:Re.number(),y_coord:Re.number(),level:Re.number(),count:Re.number()}),SB=Re.array(aP),wB=async e=>{try{const t=await e.text();let n;e.name.endsWith(".json")?n=JSON.parse(t):n=t.split(`
`).filter(u=>u.trim()!=="").map(u=>JSON.parse(u));const r=bB.safeParse(n);return r.success?r.data:(console.error("Error parsing conversation file",r.error),null)}catch(t){return console.error("Error parsing conversation file",t),null}},OB=async e=>{try{const t=await e.text();let n;e.name.endsWith(".json")?n=JSON.parse(t):n=t.split(`
`).filter(u=>u.trim()!=="").map(u=>JSON.parse(u));const r=_B.safeParse(n);return r.success?r.data:(console.error("Error parsing conversation summary file",r.error),null)}catch(t){return console.error("Error parsing conversation summary file",t),null}},AB=async e=>{try{const r=(await e.text()).split(`
`).filter(u=>u.trim()!=="").map(u=>JSON.parse(u)),i=SB.safeParse(r);return i.success?i.data:(console.error("Error parsing conversation cluster file",i.error),null)}catch(t){return console.error("Error parsing conversation cluster file",t),null}};function kO(e,t){if(typeof e=="function")return e(t);e!=null&&(e.current=t)}function iP(...e){return t=>{let n=!1;const r=e.map(i=>{const u=kO(i,t);return!n&&typeof u=="function"&&(n=!0),u});if(n)return()=>{for(let i=0;i<r.length;i++){const u=r[i];typeof u=="function"?u():kO(e[i],null)}}}}function Ei(...e){return q.useCallback(iP(...e),e)}function Ax(e){const t=TB(e),n=q.forwardRef((r,i)=>{const{children:u,...s}=r,c=q.Children.toArray(u),d=c.find(jB);if(d){const h=d.props.children,v=c.map(y=>y===d?q.Children.count(h)>1?q.Children.only(null):q.isValidElement(h)?h.props.children:null:y);return V.jsx(t,{...s,ref:i,children:q.isValidElement(h)?q.cloneElement(h,void 0,v):null})}return V.jsx(t,{...s,ref:i,children:u})});return n.displayName=`${e}.Slot`,n}var EB=Ax("Slot");function TB(e){const t=q.forwardRef((n,r)=>{const{children:i,...u}=n;if(q.isValidElement(i)){const s=PB(i),c=CB(u,i.props);return i.type!==q.Fragment&&(c.ref=r?iP(r,s):s),q.cloneElement(i,c)}return q.Children.count(i)>1?q.Children.only(null):null});return t.displayName=`${e}.SlotClone`,t}var MB=Symbol("radix.slottable");function jB(e){return q.isValidElement(e)&&typeof e.type=="function"&&"__radixId"in e.type&&e.type.__radixId===MB}function CB(e,t){const n={...t};for(const r in t){const i=e[r],u=t[r];/^on[A-Z]/.test(r)?i&&u?n[r]=(...c)=>{u(...c),i(...c)}:i&&(n[r]=i):r==="style"?n[r]={...i,...u}:r==="className"&&(n[r]=[i,u].filter(Boolean).join(" "))}return{...e,...n}}function PB(e){var r,i;let t=(r=Object.getOwnPropertyDescriptor(e.props,"ref"))==null?void 0:r.get,n=t&&"isReactWarning"in t&&t.isReactWarning;return n?e.ref:(t=(i=Object.getOwnPropertyDescriptor(e,"ref"))==null?void 0:i.get,n=t&&"isReactWarning"in t&&t.isReactWarning,n?e.props.ref:e.props.ref||e.ref)}const qO=e=>typeof e=="boolean"?`${e}`:e===0?"0":e,$O=Xe,RB=(e,t)=>n=>{var r;if((t==null?void 0:t.variants)==null)return $O(e,n==null?void 0:n.class,n==null?void 0:n.className);const{variants:i,defaultVariants:u}=t,s=Object.keys(i).map(h=>{const v=n==null?void 0:n[h],y=u==null?void 0:u[h];if(v===null)return null;const g=qO(v)||qO(y);return i[h][g]}),c=n&&Object.entries(n).reduce((h,v)=>{let[y,g]=v;return g===void 0||(h[y]=g),h},{}),d=t==null||(r=t.compoundVariants)===null||r===void 0?void 0:r.reduce((h,v)=>{let{class:y,className:g,...b}=v;return Object.entries(b).every(S=>{let[_,x]=S;return Array.isArray(x)?x.includes({...u,...c}[_]):{...u,...c}[_]===x})?[...h,y,g]:h},[]);return $O(e,s,d,n==null?void 0:n.class,n==null?void 0:n.className)},NB=RB("inline-flex items-center justify-center gap-2 whitespace-nowrap rounded-md text-sm font-medium transition-all disabled:pointer-events-none disabled:opacity-50 [&_svg]:pointer-events-none [&_svg:not([class*='size-'])]:size-4 shrink-0 [&_svg]:shrink-0 outline-none focus-visible:border-ring focus-visible:ring-ring/50 focus-visible:ring-[3px] aria-invalid:ring-destructive/20 dark:aria-invalid:ring-destructive/40 aria-invalid:border-destructive",{variants:{variant:{default:"bg-primary text-primary-foreground shadow-xs hover:bg-primary/90",destructive:"bg-destructive text-white shadow-xs hover:bg-destructive/90 focus-visible:ring-destructive/20 dark:focus-visible:ring-destructive/40 dark:bg-destructive/60",outline:"border bg-background shadow-xs hover:bg-accent hover:text-accent-foreground dark:bg-input/30 dark:border-input dark:hover:bg-input/50",secondary:"bg-secondary text-secondary-foreground shadow-xs hover:bg-secondary/80",ghost:"hover:bg-accent hover:text-accent-foreground dark:hover:bg-accent/50",link:"text-primary underline-offset-4 hover:underline"},size:{default:"h-9 px-4 py-2 has-[>svg]:px-3",sm:"h-8 rounded-md gap-1.5 px-3 has-[>svg]:px-2.5",lg:"h-10 rounded-md px-6 has-[>svg]:px-4",icon:"size-9"}},defaultVariants:{variant:"default",size:"default"}});function oP({className:e,variant:t,size:n,asChild:r=!1,...i}){const u=r?EB:"button";return V.jsx(u,{"data-slot":"button",className:er(NB({variant:t,size:n,className:e})),...i})}const DB=({setConversations:e,conversations:t,setSummaries:n,summaries:r,setClusters:i,clusters:u,handleVisualiseClusters:s})=>{const c=async v=>{var b;console.log("handleConversationsChange");const y=(b=v.target.files)==null?void 0:b[0];if(!y)return;console.log("Parsing conversation file");const g=await wB(y);g&&e(g)},d=async v=>{var b;const y=(b=v.target.files)==null?void 0:b[0];if(!y)return;console.log("Parsing conversation summary file");const g=await OB(y);g&&n(g)},h=async v=>{var b;const y=(b=v.target.files)==null?void 0:b[0];if(!y)return;console.log("Parsing conversation cluster file");const g=await AB(y);g&&i(g)};return V.jsxs(i3,{className:"max-w-2xl mx-auto mt-10",children:[V.jsxs(o3,{children:[V.jsx(u3,{children:"Load Checkpoint"}),V.jsx(l3,{children:"Upload individual files created by Kura"})]}),V.jsxs(s3,{children:[V.jsxs("div",{className:"space-y-4",children:[V.jsxs("div",{children:[V.jsx("label",{className:"text-sm font-medium",children:"Conversations File"}),V.jsx(zv,{type:"file",className:"cursor-pointer mt-1",accept:".json,.jsonl,application/json,application/jsonl",onChange:c}),V.jsx("p",{className:"text-xs text-muted-foreground mt-1",children:"(accepts .json or .jsonl format)"})]}),V.jsxs("div",{children:[V.jsx("label",{className:"text-sm font-medium",children:"Summaries File"}),V.jsx(zv,{type:"file",className:"cursor-pointer mt-1",accept:".jsonl",onChange:d}),V.jsx("p",{className:"text-xs text-muted-foreground mt-1",children:"(by default this is summaries.jsonl)"})]}),V.jsxs("div",{children:[V.jsx("label",{className:"text-sm font-medium",children:"Clusters File"}),V.jsx(zv,{type:"file",className:"cursor-pointer mt-1",accept:".jsonl",onChange:h}),V.jsx("p",{className:"text-xs text-muted-foreground mt-1",children:"(by default this is dimensionality.jsonl)"})]})]}),V.jsxs("div",{className:"mt-4 text-left text-muted-foreground text-sm",children:[V.jsx(oP,{className:"w-full mt-4",onClick:s,children:"Visualise Clusters"}),t&&r&&u&&V.jsx("div",{children:V.jsxs("p",{children:["Loaded in ",t.length," conversations,"," ",r==null?void 0:r.length," summaries, ",u==null?void 0:u.length," clusters"]})})]})]})]})},uP=rP.extend({summary:Re.string()}),lP=aP.extend({children:Re.lazy(()=>Re.array(lP)),depth:Re.number().optional()});Re.array(uP);const sP=(e,t)=>(t.push(e),e.children.forEach(n=>sP(n,t)),t),cP=(e,t,n)=>{const r=e.filter(s=>s.parent_id===t),i=e.find(s=>s.id===t)??{name:"Root",id:"root",description:"Root",chat_ids:[],x_coord:0,y_coord:0,count:0,level:n,parent_id:null},u=lP.safeParse({...i,children:r.map(s=>cP(e,s.id,n+1))});if(!u.success)throw console.error(u.error),new Error("Failed to build cluster tree");return{...u.data,level:n}};/**
 * @license lucide-react v0.487.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const kB=e=>e.replace(/([a-z0-9])([A-Z])/g,"$1-$2").toLowerCase(),qB=e=>e.replace(/^([A-Z])|[\s-_]+(\w)/g,(t,n,r)=>r?r.toUpperCase():n.toLowerCase()),BO=e=>{const t=qB(e);return t.charAt(0).toUpperCase()+t.slice(1)},fP=(...e)=>e.filter((t,n,r)=>!!t&&t.trim()!==""&&r.indexOf(t)===n).join(" ").trim();/**
 * @license lucide-react v0.487.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */var $B={xmlns:"http://www.w3.org/2000/svg",width:24,height:24,viewBox:"0 0 24 24",fill:"none",stroke:"currentColor",strokeWidth:2,strokeLinecap:"round",strokeLinejoin:"round"};/**
 * @license lucide-react v0.487.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const BB=q.forwardRef(({color:e="currentColor",size:t=24,strokeWidth:n=2,absoluteStrokeWidth:r,className:i="",children:u,iconNode:s,...c},d)=>q.createElement("svg",{ref:d,...$B,width:t,height:t,stroke:e,strokeWidth:r?Number(n)*24/Number(t):n,className:fP("lucide",i),...c},[...s.map(([h,v])=>q.createElement(h,v)),...Array.isArray(u)?u:[u]]));/**
 * @license lucide-react v0.487.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const Ex=(e,t)=>{const n=q.forwardRef(({className:r,...i},u)=>q.createElement(BB,{ref:u,iconNode:t,className:fP(`lucide-${kB(BO(e))}`,`lucide-${e}`,r),...i}));return n.displayName=BO(e),n};/**
 * @license lucide-react v0.487.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const zB=[["path",{d:"m6 9 6 6 6-6",key:"qrunsl"}]],LB=Ex("chevron-down",zB);/**
 * @license lucide-react v0.487.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const IB=[["path",{d:"m9 18 6-6-6-6",key:"mthhwq"}]],UB=Ex("chevron-right",IB);/**
 * @license lucide-react v0.487.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const HB=[["path",{d:"M18 6 6 18",key:"1bl5f8"}],["path",{d:"m6 6 12 12",key:"d8bk6v"}]],dP=Ex("x",HB),t0=({clusterTree:e,indent:t=0,onSelectCluster:n})=>{var c;const[r,i]=q.useState(!1),u=()=>i(!r),s=d=>{d.preventDefault(),u(),n&&n(e)};return e.name==="Root"?V.jsx("div",{className:"text-left",children:(c=e.children)==null?void 0:c.map((d,h)=>V.jsx(t0,{clusterTree:d,indent:0,onSelectCluster:n},d.id||h))}):V.jsxs("div",{className:"text-left",children:[V.jsxs("div",{className:"flex items-center hover:bg-slate-100 rounded py-2 cursor-pointer",style:{paddingLeft:`${t}px`},onClick:s,children:[e.children&&e.children.length>0?V.jsx("div",{className:"flex-shrink-0 mr-1",children:r?V.jsx(LB,{className:"h-4 w-4 text-slate-500"}):V.jsx(UB,{className:"h-4 w-4 text-slate-500"})}):V.jsx("div",{className:"w-5"}),V.jsxs("div",{className:"font-medium text-wrap",children:[e.name,e.count>0&&V.jsxs("span",{className:"ml-2 text-xs text-slate-500",children:["(",e.count,")"]})]})]}),r&&e.children&&e.children.length>0&&V.jsx("div",{className:"pl-2 border-l border-slate-200 ml-2",children:e.children.map((d,h)=>V.jsx(t0,{clusterTree:d,indent:t+5,onSelectCluster:n},d.id||h))})]})};function ja(e,t,{checkForDefaultPrevented:n=!0}={}){return function(i){if(e==null||e(i),n===!1||!i.defaultPrevented)return t==null?void 0:t(i)}}function GB(e,t){const n=q.createContext(t),r=u=>{const{children:s,...c}=u,d=q.useMemo(()=>c,Object.values(c));return V.jsx(n.Provider,{value:d,children:s})};r.displayName=e+"Provider";function i(u){const s=q.useContext(n);if(s)return s;if(t!==void 0)return t;throw new Error(`\`${u}\` must be used within \`${e}\``)}return[r,i]}function VB(e,t=[]){let n=[];function r(u,s){const c=q.createContext(s),d=n.length;n=[...n,s];const h=y=>{var E;const{scope:g,children:b,...S}=y,_=((E=g==null?void 0:g[e])==null?void 0:E[d])||c,x=q.useMemo(()=>S,Object.values(S));return V.jsx(_.Provider,{value:x,children:b})};h.displayName=u+"Provider";function v(y,g){var _;const b=((_=g==null?void 0:g[e])==null?void 0:_[d])||c,S=q.useContext(b);if(S)return S;if(s!==void 0)return s;throw new Error(`\`${y}\` must be used within \`${u}\``)}return[h,v]}const i=()=>{const u=n.map(s=>q.createContext(s));return function(c){const d=(c==null?void 0:c[e])||u;return q.useMemo(()=>({[`__scope${e}`]:{...c,[e]:d}}),[c,d])}};return i.scopeName=e,[r,ZB(i,...t)]}function ZB(...e){const t=e[0];if(e.length===1)return t;const n=()=>{const r=e.map(i=>({useScope:i(),scopeName:i.scopeName}));return function(u){const s=r.reduce((c,{useScope:d,scopeName:h})=>{const y=d(u)[`__scope${h}`];return{...c,...y}},{});return q.useMemo(()=>({[`__scope${t.scopeName}`]:s}),[s])}};return n.scopeName=t.scopeName,n}var Bf=globalThis!=null&&globalThis.document?q.useLayoutEffect:()=>{},YB=g$[" useId ".trim().toString()]||(()=>{}),XB=0;function Iv(e){const[t,n]=q.useState(YB());return Bf(()=>{n(r=>r??String(XB++))},[e]),e||(t?`radix-${t}`:"")}function _i(e){const t=q.useRef(e);return q.useEffect(()=>{t.current=e}),q.useMemo(()=>(...n)=>{var r;return(r=t.current)==null?void 0:r.call(t,...n)},[])}function KB({prop:e,defaultProp:t,onChange:n=()=>{}}){const[r,i]=WB({defaultProp:t,onChange:n}),u=e!==void 0,s=u?e:r,c=_i(n),d=q.useCallback(h=>{if(u){const y=typeof h=="function"?h(e):h;y!==e&&c(y)}else i(h)},[u,e,i,c]);return[s,d]}function WB({defaultProp:e,onChange:t}){const n=q.useState(e),[r]=n,i=q.useRef(r),u=_i(t);return q.useEffect(()=>{i.current!==r&&(u(r),i.current=r)},[r,i,u]),n}var hP=D2();const FB=lt(hP);var QB=["a","button","div","form","h2","h3","img","input","label","li","nav","ol","p","span","svg","ul"],Wr=QB.reduce((e,t)=>{const n=Ax(`Primitive.${t}`),r=q.forwardRef((i,u)=>{const{asChild:s,...c}=i,d=s?n:t;return typeof window<"u"&&(window[Symbol.for("radix-ui")]=!0),V.jsx(d,{...c,ref:u})});return r.displayName=`Primitive.${t}`,{...e,[t]:r}},{});function JB(e,t){e&&hP.flushSync(()=>e.dispatchEvent(t))}function ez(e,t=globalThis==null?void 0:globalThis.document){const n=_i(e);q.useEffect(()=>{const r=i=>{i.key==="Escape"&&n(i)};return t.addEventListener("keydown",r,{capture:!0}),()=>t.removeEventListener("keydown",r,{capture:!0})},[n,t])}var tz="DismissableLayer",n0="dismissableLayer.update",nz="dismissableLayer.pointerDownOutside",rz="dismissableLayer.focusOutside",zO,pP=q.createContext({layers:new Set,layersWithOutsidePointerEventsDisabled:new Set,branches:new Set}),vP=q.forwardRef((e,t)=>{const{disableOutsidePointerEvents:n=!1,onEscapeKeyDown:r,onPointerDownOutside:i,onFocusOutside:u,onInteractOutside:s,onDismiss:c,...d}=e,h=q.useContext(pP),[v,y]=q.useState(null),g=(v==null?void 0:v.ownerDocument)??(globalThis==null?void 0:globalThis.document),[,b]=q.useState({}),S=Ei(t,P=>y(P)),_=Array.from(h.layers),[x]=[...h.layersWithOutsidePointerEventsDisabled].slice(-1),E=_.indexOf(x),M=v?_.indexOf(v):-1,A=h.layersWithOutsidePointerEventsDisabled.size>0,C=M>=E,w=oz(P=>{const R=P.target,$=[...h.branches].some(J=>J.contains(R));!C||$||(i==null||i(P),s==null||s(P),P.defaultPrevented||c==null||c())},g),T=uz(P=>{const R=P.target;[...h.branches].some(J=>J.contains(R))||(u==null||u(P),s==null||s(P),P.defaultPrevented||c==null||c())},g);return ez(P=>{M===h.layers.size-1&&(r==null||r(P),!P.defaultPrevented&&c&&(P.preventDefault(),c()))},g),q.useEffect(()=>{if(v)return n&&(h.layersWithOutsidePointerEventsDisabled.size===0&&(zO=g.body.style.pointerEvents,g.body.style.pointerEvents="none"),h.layersWithOutsidePointerEventsDisabled.add(v)),h.layers.add(v),LO(),()=>{n&&h.layersWithOutsidePointerEventsDisabled.size===1&&(g.body.style.pointerEvents=zO)}},[v,g,n,h]),q.useEffect(()=>()=>{v&&(h.layers.delete(v),h.layersWithOutsidePointerEventsDisabled.delete(v),LO())},[v,h]),q.useEffect(()=>{const P=()=>b({});return document.addEventListener(n0,P),()=>document.removeEventListener(n0,P)},[]),V.jsx(Wr.div,{...d,ref:S,style:{pointerEvents:A?C?"auto":"none":void 0,...e.style},onFocusCapture:ja(e.onFocusCapture,T.onFocusCapture),onBlurCapture:ja(e.onBlurCapture,T.onBlurCapture),onPointerDownCapture:ja(e.onPointerDownCapture,w.onPointerDownCapture)})});vP.displayName=tz;var az="DismissableLayerBranch",iz=q.forwardRef((e,t)=>{const n=q.useContext(pP),r=q.useRef(null),i=Ei(t,r);return q.useEffect(()=>{const u=r.current;if(u)return n.branches.add(u),()=>{n.branches.delete(u)}},[n.branches]),V.jsx(Wr.div,{...e,ref:i})});iz.displayName=az;function oz(e,t=globalThis==null?void 0:globalThis.document){const n=_i(e),r=q.useRef(!1),i=q.useRef(()=>{});return q.useEffect(()=>{const u=c=>{if(c.target&&!r.current){let d=function(){yP(nz,n,h,{discrete:!0})};const h={originalEvent:c};c.pointerType==="touch"?(t.removeEventListener("click",i.current),i.current=d,t.addEventListener("click",i.current,{once:!0})):d()}else t.removeEventListener("click",i.current);r.current=!1},s=window.setTimeout(()=>{t.addEventListener("pointerdown",u)},0);return()=>{window.clearTimeout(s),t.removeEventListener("pointerdown",u),t.removeEventListener("click",i.current)}},[t,n]),{onPointerDownCapture:()=>r.current=!0}}function uz(e,t=globalThis==null?void 0:globalThis.document){const n=_i(e),r=q.useRef(!1);return q.useEffect(()=>{const i=u=>{u.target&&!r.current&&yP(rz,n,{originalEvent:u},{discrete:!1})};return t.addEventListener("focusin",i),()=>t.removeEventListener("focusin",i)},[t,n]),{onFocusCapture:()=>r.current=!0,onBlurCapture:()=>r.current=!1}}function LO(){const e=new CustomEvent(n0);document.dispatchEvent(e)}function yP(e,t,n,{discrete:r}){const i=n.originalEvent.target,u=new CustomEvent(e,{bubbles:!1,cancelable:!0,detail:n});t&&i.addEventListener(e,t,{once:!0}),r?JB(i,u):i.dispatchEvent(u)}var Uv="focusScope.autoFocusOnMount",Hv="focusScope.autoFocusOnUnmount",IO={bubbles:!1,cancelable:!0},lz="FocusScope",mP=q.forwardRef((e,t)=>{const{loop:n=!1,trapped:r=!1,onMountAutoFocus:i,onUnmountAutoFocus:u,...s}=e,[c,d]=q.useState(null),h=_i(i),v=_i(u),y=q.useRef(null),g=Ei(t,_=>d(_)),b=q.useRef({paused:!1,pause(){this.paused=!0},resume(){this.paused=!1}}).current;q.useEffect(()=>{if(r){let _=function(A){if(b.paused||!c)return;const C=A.target;c.contains(C)?y.current=C:Aa(y.current,{select:!0})},x=function(A){if(b.paused||!c)return;const C=A.relatedTarget;C!==null&&(c.contains(C)||Aa(y.current,{select:!0}))},E=function(A){if(document.activeElement===document.body)for(const w of A)w.removedNodes.length>0&&Aa(c)};document.addEventListener("focusin",_),document.addEventListener("focusout",x);const M=new MutationObserver(E);return c&&M.observe(c,{childList:!0,subtree:!0}),()=>{document.removeEventListener("focusin",_),document.removeEventListener("focusout",x),M.disconnect()}}},[r,c,b.paused]),q.useEffect(()=>{if(c){HO.add(b);const _=document.activeElement;if(!c.contains(_)){const E=new CustomEvent(Uv,IO);c.addEventListener(Uv,h),c.dispatchEvent(E),E.defaultPrevented||(sz(pz(gP(c)),{select:!0}),document.activeElement===_&&Aa(c))}return()=>{c.removeEventListener(Uv,h),setTimeout(()=>{const E=new CustomEvent(Hv,IO);c.addEventListener(Hv,v),c.dispatchEvent(E),E.defaultPrevented||Aa(_??document.body,{select:!0}),c.removeEventListener(Hv,v),HO.remove(b)},0)}}},[c,h,v,b]);const S=q.useCallback(_=>{if(!n&&!r||b.paused)return;const x=_.key==="Tab"&&!_.altKey&&!_.ctrlKey&&!_.metaKey,E=document.activeElement;if(x&&E){const M=_.currentTarget,[A,C]=cz(M);A&&C?!_.shiftKey&&E===C?(_.preventDefault(),n&&Aa(A,{select:!0})):_.shiftKey&&E===A&&(_.preventDefault(),n&&Aa(C,{select:!0})):E===M&&_.preventDefault()}},[n,r,b.paused]);return V.jsx(Wr.div,{tabIndex:-1,...s,ref:g,onKeyDown:S})});mP.displayName=lz;function sz(e,{select:t=!1}={}){const n=document.activeElement;for(const r of e)if(Aa(r,{select:t}),document.activeElement!==n)return}function cz(e){const t=gP(e),n=UO(t,e),r=UO(t.reverse(),e);return[n,r]}function gP(e){const t=[],n=document.createTreeWalker(e,NodeFilter.SHOW_ELEMENT,{acceptNode:r=>{const i=r.tagName==="INPUT"&&r.type==="hidden";return r.disabled||r.hidden||i?NodeFilter.FILTER_SKIP:r.tabIndex>=0?NodeFilter.FILTER_ACCEPT:NodeFilter.FILTER_SKIP}});for(;n.nextNode();)t.push(n.currentNode);return t}function UO(e,t){for(const n of e)if(!fz(n,{upTo:t}))return n}function fz(e,{upTo:t}){if(getComputedStyle(e).visibility==="hidden")return!0;for(;e;){if(t!==void 0&&e===t)return!1;if(getComputedStyle(e).display==="none")return!0;e=e.parentElement}return!1}function dz(e){return e instanceof HTMLInputElement&&"select"in e}function Aa(e,{select:t=!1}={}){if(e&&e.focus){const n=document.activeElement;e.focus({preventScroll:!0}),e!==n&&dz(e)&&t&&e.select()}}var HO=hz();function hz(){let e=[];return{add(t){const n=e[0];t!==n&&(n==null||n.pause()),e=GO(e,t),e.unshift(t)},remove(t){var n;e=GO(e,t),(n=e[0])==null||n.resume()}}}function GO(e,t){const n=[...e],r=n.indexOf(t);return r!==-1&&n.splice(r,1),n}function pz(e){return e.filter(t=>t.tagName!=="A")}var vz="Portal",bP=q.forwardRef((e,t)=>{var c;const{container:n,...r}=e,[i,u]=q.useState(!1);Bf(()=>u(!0),[]);const s=n||i&&((c=globalThis==null?void 0:globalThis.document)==null?void 0:c.body);return s?FB.createPortal(V.jsx(Wr.div,{...r,ref:t}),s):null});bP.displayName=vz;function yz(e,t){return q.useReducer((n,r)=>t[n][r]??n,e)}var $d=e=>{const{present:t,children:n}=e,r=mz(t),i=typeof n=="function"?n({present:r.isPresent}):q.Children.only(n),u=Ei(r.ref,gz(i));return typeof n=="function"||r.isPresent?q.cloneElement(i,{ref:u}):null};$d.displayName="Presence";function mz(e){const[t,n]=q.useState(),r=q.useRef({}),i=q.useRef(e),u=q.useRef("none"),s=e?"mounted":"unmounted",[c,d]=yz(s,{mounted:{UNMOUNT:"unmounted",ANIMATION_OUT:"unmountSuspended"},unmountSuspended:{MOUNT:"mounted",ANIMATION_END:"unmounted"},unmounted:{MOUNT:"mounted"}});return q.useEffect(()=>{const h=ff(r.current);u.current=c==="mounted"?h:"none"},[c]),Bf(()=>{const h=r.current,v=i.current;if(v!==e){const g=u.current,b=ff(h);e?d("MOUNT"):b==="none"||(h==null?void 0:h.display)==="none"?d("UNMOUNT"):d(v&&g!==b?"ANIMATION_OUT":"UNMOUNT"),i.current=e}},[e,d]),Bf(()=>{if(t){let h;const v=t.ownerDocument.defaultView??window,y=b=>{const _=ff(r.current).includes(b.animationName);if(b.target===t&&_&&(d("ANIMATION_END"),!i.current)){const x=t.style.animationFillMode;t.style.animationFillMode="forwards",h=v.setTimeout(()=>{t.style.animationFillMode==="forwards"&&(t.style.animationFillMode=x)})}},g=b=>{b.target===t&&(u.current=ff(r.current))};return t.addEventListener("animationstart",g),t.addEventListener("animationcancel",y),t.addEventListener("animationend",y),()=>{v.clearTimeout(h),t.removeEventListener("animationstart",g),t.removeEventListener("animationcancel",y),t.removeEventListener("animationend",y)}}else d("ANIMATION_END")},[t,d]),{isPresent:["mounted","unmountSuspended"].includes(c),ref:q.useCallback(h=>{h&&(r.current=getComputedStyle(h)),n(h)},[])}}function ff(e){return(e==null?void 0:e.animationName)||"none"}function gz(e){var r,i;let t=(r=Object.getOwnPropertyDescriptor(e.props,"ref"))==null?void 0:r.get,n=t&&"isReactWarning"in t&&t.isReactWarning;return n?e.ref:(t=(i=Object.getOwnPropertyDescriptor(e,"ref"))==null?void 0:i.get,n=t&&"isReactWarning"in t&&t.isReactWarning,n?e.props.ref:e.props.ref||e.ref)}var Gv=0;function bz(){q.useEffect(()=>{const e=document.querySelectorAll("[data-radix-focus-guard]");return document.body.insertAdjacentElement("afterbegin",e[0]??VO()),document.body.insertAdjacentElement("beforeend",e[1]??VO()),Gv++,()=>{Gv===1&&document.querySelectorAll("[data-radix-focus-guard]").forEach(t=>t.remove()),Gv--}},[])}function VO(){const e=document.createElement("span");return e.setAttribute("data-radix-focus-guard",""),e.tabIndex=0,e.style.outline="none",e.style.opacity="0",e.style.position="fixed",e.style.pointerEvents="none",e}var fr=function(){return fr=Object.assign||function(t){for(var n,r=1,i=arguments.length;r<i;r++){n=arguments[r];for(var u in n)Object.prototype.hasOwnProperty.call(n,u)&&(t[u]=n[u])}return t},fr.apply(this,arguments)};function xP(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var i=0,r=Object.getOwnPropertySymbols(e);i<r.length;i++)t.indexOf(r[i])<0&&Object.prototype.propertyIsEnumerable.call(e,r[i])&&(n[r[i]]=e[r[i]]);return n}function xz(e,t,n){if(n||arguments.length===2)for(var r=0,i=t.length,u;r<i;r++)(u||!(r in t))&&(u||(u=Array.prototype.slice.call(t,0,r)),u[r]=t[r]);return e.concat(u||Array.prototype.slice.call(t))}var jf="right-scroll-bar-position",Cf="width-before-scroll-bar",_z="with-scroll-bars-hidden",Sz="--removed-body-scroll-bar-size";function Vv(e,t){return typeof e=="function"?e(t):e&&(e.current=t),e}function wz(e,t){var n=q.useState(function(){return{value:e,callback:t,facade:{get current(){return n.value},set current(r){var i=n.value;i!==r&&(n.value=r,n.callback(r,i))}}}})[0];return n.callback=t,n.facade}var Oz=typeof window<"u"?q.useLayoutEffect:q.useEffect,ZO=new WeakMap;function Az(e,t){var n=wz(null,function(r){return e.forEach(function(i){return Vv(i,r)})});return Oz(function(){var r=ZO.get(n);if(r){var i=new Set(r),u=new Set(e),s=n.current;i.forEach(function(c){u.has(c)||Vv(c,null)}),u.forEach(function(c){i.has(c)||Vv(c,s)})}ZO.set(n,e)},[e]),n}function Ez(e){return e}function Tz(e,t){t===void 0&&(t=Ez);var n=[],r=!1,i={read:function(){if(r)throw new Error("Sidecar: could not `read` from an `assigned` medium. `read` could be used only with `useMedium`.");return n.length?n[n.length-1]:e},useMedium:function(u){var s=t(u,r);return n.push(s),function(){n=n.filter(function(c){return c!==s})}},assignSyncMedium:function(u){for(r=!0;n.length;){var s=n;n=[],s.forEach(u)}n={push:function(c){return u(c)},filter:function(){return n}}},assignMedium:function(u){r=!0;var s=[];if(n.length){var c=n;n=[],c.forEach(u),s=n}var d=function(){var v=s;s=[],v.forEach(u)},h=function(){return Promise.resolve().then(d)};h(),n={push:function(v){s.push(v),h()},filter:function(v){return s=s.filter(v),n}}}};return i}function Mz(e){e===void 0&&(e={});var t=Tz(null);return t.options=fr({async:!0,ssr:!1},e),t}var _P=function(e){var t=e.sideCar,n=xP(e,["sideCar"]);if(!t)throw new Error("Sidecar: please provide `sideCar` property to import the right car");var r=t.read();if(!r)throw new Error("Sidecar medium not found");return q.createElement(r,fr({},n))};_P.isSideCarExport=!0;function jz(e,t){return e.useMedium(t),_P}var SP=Mz(),Zv=function(){},Bd=q.forwardRef(function(e,t){var n=q.useRef(null),r=q.useState({onScrollCapture:Zv,onWheelCapture:Zv,onTouchMoveCapture:Zv}),i=r[0],u=r[1],s=e.forwardProps,c=e.children,d=e.className,h=e.removeScrollBar,v=e.enabled,y=e.shards,g=e.sideCar,b=e.noIsolation,S=e.inert,_=e.allowPinchZoom,x=e.as,E=x===void 0?"div":x,M=e.gapMode,A=xP(e,["forwardProps","children","className","removeScrollBar","enabled","shards","sideCar","noIsolation","inert","allowPinchZoom","as","gapMode"]),C=g,w=Az([n,t]),T=fr(fr({},A),i);return q.createElement(q.Fragment,null,v&&q.createElement(C,{sideCar:SP,removeScrollBar:h,shards:y,noIsolation:b,inert:S,setCallbacks:u,allowPinchZoom:!!_,lockRef:n,gapMode:M}),s?q.cloneElement(q.Children.only(c),fr(fr({},T),{ref:w})):q.createElement(E,fr({},T,{className:d,ref:w}),c))});Bd.defaultProps={enabled:!0,removeScrollBar:!0,inert:!1};Bd.classNames={fullWidth:Cf,zeroRight:jf};var Cz=function(){if(typeof __webpack_nonce__<"u")return __webpack_nonce__};function Pz(){if(!document)return null;var e=document.createElement("style");e.type="text/css";var t=Cz();return t&&e.setAttribute("nonce",t),e}function Rz(e,t){e.styleSheet?e.styleSheet.cssText=t:e.appendChild(document.createTextNode(t))}function Nz(e){var t=document.head||document.getElementsByTagName("head")[0];t.appendChild(e)}var Dz=function(){var e=0,t=null;return{add:function(n){e==0&&(t=Pz())&&(Rz(t,n),Nz(t)),e++},remove:function(){e--,!e&&t&&(t.parentNode&&t.parentNode.removeChild(t),t=null)}}},kz=function(){var e=Dz();return function(t,n){q.useEffect(function(){return e.add(t),function(){e.remove()}},[t&&n])}},wP=function(){var e=kz(),t=function(n){var r=n.styles,i=n.dynamic;return e(r,i),null};return t},qz={left:0,top:0,right:0,gap:0},Yv=function(e){return parseInt(e||"",10)||0},$z=function(e){var t=window.getComputedStyle(document.body),n=t[e==="padding"?"paddingLeft":"marginLeft"],r=t[e==="padding"?"paddingTop":"marginTop"],i=t[e==="padding"?"paddingRight":"marginRight"];return[Yv(n),Yv(r),Yv(i)]},Bz=function(e){if(e===void 0&&(e="margin"),typeof window>"u")return qz;var t=$z(e),n=document.documentElement.clientWidth,r=window.innerWidth;return{left:t[0],top:t[1],right:t[2],gap:Math.max(0,r-n+t[2]-t[0])}},zz=wP(),Po="data-scroll-locked",Lz=function(e,t,n,r){var i=e.left,u=e.top,s=e.right,c=e.gap;return n===void 0&&(n="margin"),`
  .`.concat(_z,` {
   overflow: hidden `).concat(r,`;
   padding-right: `).concat(c,"px ").concat(r,`;
  }
  body[`).concat(Po,`] {
    overflow: hidden `).concat(r,`;
    overscroll-behavior: contain;
    `).concat([t&&"position: relative ".concat(r,";"),n==="margin"&&`
    padding-left: `.concat(i,`px;
    padding-top: `).concat(u,`px;
    padding-right: `).concat(s,`px;
    margin-left:0;
    margin-top:0;
    margin-right: `).concat(c,"px ").concat(r,`;
    `),n==="padding"&&"padding-right: ".concat(c,"px ").concat(r,";")].filter(Boolean).join(""),`
  }
  
  .`).concat(jf,` {
    right: `).concat(c,"px ").concat(r,`;
  }
  
  .`).concat(Cf,` {
    margin-right: `).concat(c,"px ").concat(r,`;
  }
  
  .`).concat(jf," .").concat(jf,` {
    right: 0 `).concat(r,`;
  }
  
  .`).concat(Cf," .").concat(Cf,` {
    margin-right: 0 `).concat(r,`;
  }
  
  body[`).concat(Po,`] {
    `).concat(Sz,": ").concat(c,`px;
  }
`)},YO=function(){var e=parseInt(document.body.getAttribute(Po)||"0",10);return isFinite(e)?e:0},Iz=function(){q.useEffect(function(){return document.body.setAttribute(Po,(YO()+1).toString()),function(){var e=YO()-1;e<=0?document.body.removeAttribute(Po):document.body.setAttribute(Po,e.toString())}},[])},Uz=function(e){var t=e.noRelative,n=e.noImportant,r=e.gapMode,i=r===void 0?"margin":r;Iz();var u=q.useMemo(function(){return Bz(i)},[i]);return q.createElement(zz,{styles:Lz(u,!t,i,n?"":"!important")})},r0=!1;if(typeof window<"u")try{var df=Object.defineProperty({},"passive",{get:function(){return r0=!0,!0}});window.addEventListener("test",df,df),window.removeEventListener("test",df,df)}catch{r0=!1}var bo=r0?{passive:!1}:!1,Hz=function(e){return e.tagName==="TEXTAREA"},OP=function(e,t){if(!(e instanceof Element))return!1;var n=window.getComputedStyle(e);return n[t]!=="hidden"&&!(n.overflowY===n.overflowX&&!Hz(e)&&n[t]==="visible")},Gz=function(e){return OP(e,"overflowY")},Vz=function(e){return OP(e,"overflowX")},XO=function(e,t){var n=t.ownerDocument,r=t;do{typeof ShadowRoot<"u"&&r instanceof ShadowRoot&&(r=r.host);var i=AP(e,r);if(i){var u=EP(e,r),s=u[1],c=u[2];if(s>c)return!0}r=r.parentNode}while(r&&r!==n.body);return!1},Zz=function(e){var t=e.scrollTop,n=e.scrollHeight,r=e.clientHeight;return[t,n,r]},Yz=function(e){var t=e.scrollLeft,n=e.scrollWidth,r=e.clientWidth;return[t,n,r]},AP=function(e,t){return e==="v"?Gz(t):Vz(t)},EP=function(e,t){return e==="v"?Zz(t):Yz(t)},Xz=function(e,t){return e==="h"&&t==="rtl"?-1:1},Kz=function(e,t,n,r,i){var u=Xz(e,window.getComputedStyle(t).direction),s=u*r,c=n.target,d=t.contains(c),h=!1,v=s>0,y=0,g=0;do{var b=EP(e,c),S=b[0],_=b[1],x=b[2],E=_-x-u*S;(S||E)&&AP(e,c)&&(y+=E,g+=S),c instanceof ShadowRoot?c=c.host:c=c.parentNode}while(!d&&c!==document.body||d&&(t.contains(c)||t===c));return(v&&Math.abs(y)<1||!v&&Math.abs(g)<1)&&(h=!0),h},hf=function(e){return"changedTouches"in e?[e.changedTouches[0].clientX,e.changedTouches[0].clientY]:[0,0]},KO=function(e){return[e.deltaX,e.deltaY]},WO=function(e){return e&&"current"in e?e.current:e},Wz=function(e,t){return e[0]===t[0]&&e[1]===t[1]},Fz=function(e){return`
  .block-interactivity-`.concat(e,` {pointer-events: none;}
  .allow-interactivity-`).concat(e,` {pointer-events: all;}
`)},Qz=0,xo=[];function Jz(e){var t=q.useRef([]),n=q.useRef([0,0]),r=q.useRef(),i=q.useState(Qz++)[0],u=q.useState(wP)[0],s=q.useRef(e);q.useEffect(function(){s.current=e},[e]),q.useEffect(function(){if(e.inert){document.body.classList.add("block-interactivity-".concat(i));var _=xz([e.lockRef.current],(e.shards||[]).map(WO),!0).filter(Boolean);return _.forEach(function(x){return x.classList.add("allow-interactivity-".concat(i))}),function(){document.body.classList.remove("block-interactivity-".concat(i)),_.forEach(function(x){return x.classList.remove("allow-interactivity-".concat(i))})}}},[e.inert,e.lockRef.current,e.shards]);var c=q.useCallback(function(_,x){if("touches"in _&&_.touches.length===2||_.type==="wheel"&&_.ctrlKey)return!s.current.allowPinchZoom;var E=hf(_),M=n.current,A="deltaX"in _?_.deltaX:M[0]-E[0],C="deltaY"in _?_.deltaY:M[1]-E[1],w,T=_.target,P=Math.abs(A)>Math.abs(C)?"h":"v";if("touches"in _&&P==="h"&&T.type==="range")return!1;var R=XO(P,T);if(!R)return!0;if(R?w=P:(w=P==="v"?"h":"v",R=XO(P,T)),!R)return!1;if(!r.current&&"changedTouches"in _&&(A||C)&&(r.current=w),!w)return!0;var $=r.current||w;return Kz($,x,_,$==="h"?A:C)},[]),d=q.useCallback(function(_){var x=_;if(!(!xo.length||xo[xo.length-1]!==u)){var E="deltaY"in x?KO(x):hf(x),M=t.current.filter(function(w){return w.name===x.type&&(w.target===x.target||x.target===w.shadowParent)&&Wz(w.delta,E)})[0];if(M&&M.should){x.cancelable&&x.preventDefault();return}if(!M){var A=(s.current.shards||[]).map(WO).filter(Boolean).filter(function(w){return w.contains(x.target)}),C=A.length>0?c(x,A[0]):!s.current.noIsolation;C&&x.cancelable&&x.preventDefault()}}},[]),h=q.useCallback(function(_,x,E,M){var A={name:_,delta:x,target:E,should:M,shadowParent:e4(E)};t.current.push(A),setTimeout(function(){t.current=t.current.filter(function(C){return C!==A})},1)},[]),v=q.useCallback(function(_){n.current=hf(_),r.current=void 0},[]),y=q.useCallback(function(_){h(_.type,KO(_),_.target,c(_,e.lockRef.current))},[]),g=q.useCallback(function(_){h(_.type,hf(_),_.target,c(_,e.lockRef.current))},[]);q.useEffect(function(){return xo.push(u),e.setCallbacks({onScrollCapture:y,onWheelCapture:y,onTouchMoveCapture:g}),document.addEventListener("wheel",d,bo),document.addEventListener("touchmove",d,bo),document.addEventListener("touchstart",v,bo),function(){xo=xo.filter(function(_){return _!==u}),document.removeEventListener("wheel",d,bo),document.removeEventListener("touchmove",d,bo),document.removeEventListener("touchstart",v,bo)}},[]);var b=e.removeScrollBar,S=e.inert;return q.createElement(q.Fragment,null,S?q.createElement(u,{styles:Fz(i)}):null,b?q.createElement(Uz,{gapMode:e.gapMode}):null)}function e4(e){for(var t=null;e!==null;)e instanceof ShadowRoot&&(t=e.host,e=e.host),e=e.parentNode;return t}const t4=jz(SP,Jz);var TP=q.forwardRef(function(e,t){return q.createElement(Bd,fr({},e,{ref:t,sideCar:t4}))});TP.classNames=Bd.classNames;var n4=function(e){if(typeof document>"u")return null;var t=Array.isArray(e)?e[0]:e;return t.ownerDocument.body},_o=new WeakMap,pf=new WeakMap,vf={},Xv=0,MP=function(e){return e&&(e.host||MP(e.parentNode))},r4=function(e,t){return t.map(function(n){if(e.contains(n))return n;var r=MP(n);return r&&e.contains(r)?r:(console.error("aria-hidden",n,"in not contained inside",e,". Doing nothing"),null)}).filter(function(n){return!!n})},a4=function(e,t,n,r){var i=r4(t,Array.isArray(e)?e:[e]);vf[n]||(vf[n]=new WeakMap);var u=vf[n],s=[],c=new Set,d=new Set(i),h=function(y){!y||c.has(y)||(c.add(y),h(y.parentNode))};i.forEach(h);var v=function(y){!y||d.has(y)||Array.prototype.forEach.call(y.children,function(g){if(c.has(g))v(g);else try{var b=g.getAttribute(r),S=b!==null&&b!=="false",_=(_o.get(g)||0)+1,x=(u.get(g)||0)+1;_o.set(g,_),u.set(g,x),s.push(g),_===1&&S&&pf.set(g,!0),x===1&&g.setAttribute(n,"true"),S||g.setAttribute(r,"true")}catch(E){console.error("aria-hidden: cannot operate on ",g,E)}})};return v(t),c.clear(),Xv++,function(){s.forEach(function(y){var g=_o.get(y)-1,b=u.get(y)-1;_o.set(y,g),u.set(y,b),g||(pf.has(y)||y.removeAttribute(r),pf.delete(y)),b||y.removeAttribute(n)}),Xv--,Xv||(_o=new WeakMap,_o=new WeakMap,pf=new WeakMap,vf={})}},i4=function(e,t,n){n===void 0&&(n="data-aria-hidden");var r=Array.from(Array.isArray(e)?e:[e]),i=n4(e);return i?(r.push.apply(r,Array.from(i.querySelectorAll("[aria-live]"))),a4(r,i,n,"aria-hidden")):function(){return null}},Tx="Dialog",[jP,yF]=VB(Tx),[o4,tr]=jP(Tx),CP=e=>{const{__scopeDialog:t,children:n,open:r,defaultOpen:i,onOpenChange:u,modal:s=!0}=e,c=q.useRef(null),d=q.useRef(null),[h=!1,v]=KB({prop:r,defaultProp:i,onChange:u});return V.jsx(o4,{scope:t,triggerRef:c,contentRef:d,contentId:Iv(),titleId:Iv(),descriptionId:Iv(),open:h,onOpenChange:v,onOpenToggle:q.useCallback(()=>v(y=>!y),[v]),modal:s,children:n})};CP.displayName=Tx;var PP="DialogTrigger",u4=q.forwardRef((e,t)=>{const{__scopeDialog:n,...r}=e,i=tr(PP,n),u=Ei(t,i.triggerRef);return V.jsx(Wr.button,{type:"button","aria-haspopup":"dialog","aria-expanded":i.open,"aria-controls":i.contentId,"data-state":Cx(i.open),...r,ref:u,onClick:ja(e.onClick,i.onOpenToggle)})});u4.displayName=PP;var Mx="DialogPortal",[l4,RP]=jP(Mx,{forceMount:void 0}),NP=e=>{const{__scopeDialog:t,forceMount:n,children:r,container:i}=e,u=tr(Mx,t);return V.jsx(l4,{scope:t,forceMount:n,children:q.Children.map(r,s=>V.jsx($d,{present:n||u.open,children:V.jsx(bP,{asChild:!0,container:i,children:s})}))})};NP.displayName=Mx;var zf="DialogOverlay",DP=q.forwardRef((e,t)=>{const n=RP(zf,e.__scopeDialog),{forceMount:r=n.forceMount,...i}=e,u=tr(zf,e.__scopeDialog);return u.modal?V.jsx($d,{present:r||u.open,children:V.jsx(c4,{...i,ref:t})}):null});DP.displayName=zf;var s4=Ax("DialogOverlay.RemoveScroll"),c4=q.forwardRef((e,t)=>{const{__scopeDialog:n,...r}=e,i=tr(zf,n);return V.jsx(TP,{as:s4,allowPinchZoom:!0,shards:[i.contentRef],children:V.jsx(Wr.div,{"data-state":Cx(i.open),...r,ref:t,style:{pointerEvents:"auto",...r.style}})})}),Si="DialogContent",kP=q.forwardRef((e,t)=>{const n=RP(Si,e.__scopeDialog),{forceMount:r=n.forceMount,...i}=e,u=tr(Si,e.__scopeDialog);return V.jsx($d,{present:r||u.open,children:u.modal?V.jsx(f4,{...i,ref:t}):V.jsx(d4,{...i,ref:t})})});kP.displayName=Si;var f4=q.forwardRef((e,t)=>{const n=tr(Si,e.__scopeDialog),r=q.useRef(null),i=Ei(t,n.contentRef,r);return q.useEffect(()=>{const u=r.current;if(u)return i4(u)},[]),V.jsx(qP,{...e,ref:i,trapFocus:n.open,disableOutsidePointerEvents:!0,onCloseAutoFocus:ja(e.onCloseAutoFocus,u=>{var s;u.preventDefault(),(s=n.triggerRef.current)==null||s.focus()}),onPointerDownOutside:ja(e.onPointerDownOutside,u=>{const s=u.detail.originalEvent,c=s.button===0&&s.ctrlKey===!0;(s.button===2||c)&&u.preventDefault()}),onFocusOutside:ja(e.onFocusOutside,u=>u.preventDefault())})}),d4=q.forwardRef((e,t)=>{const n=tr(Si,e.__scopeDialog),r=q.useRef(!1),i=q.useRef(!1);return V.jsx(qP,{...e,ref:t,trapFocus:!1,disableOutsidePointerEvents:!1,onCloseAutoFocus:u=>{var s,c;(s=e.onCloseAutoFocus)==null||s.call(e,u),u.defaultPrevented||(r.current||(c=n.triggerRef.current)==null||c.focus(),u.preventDefault()),r.current=!1,i.current=!1},onInteractOutside:u=>{var d,h;(d=e.onInteractOutside)==null||d.call(e,u),u.defaultPrevented||(r.current=!0,u.detail.originalEvent.type==="pointerdown"&&(i.current=!0));const s=u.target;((h=n.triggerRef.current)==null?void 0:h.contains(s))&&u.preventDefault(),u.detail.originalEvent.type==="focusin"&&i.current&&u.preventDefault()}})}),qP=q.forwardRef((e,t)=>{const{__scopeDialog:n,trapFocus:r,onOpenAutoFocus:i,onCloseAutoFocus:u,...s}=e,c=tr(Si,n),d=q.useRef(null),h=Ei(t,d);return bz(),V.jsxs(V.Fragment,{children:[V.jsx(mP,{asChild:!0,loop:!0,trapped:r,onMountAutoFocus:i,onUnmountAutoFocus:u,children:V.jsx(vP,{role:"dialog",id:c.contentId,"aria-describedby":c.descriptionId,"aria-labelledby":c.titleId,"data-state":Cx(c.open),...s,ref:h,onDismiss:()=>c.onOpenChange(!1)})}),V.jsxs(V.Fragment,{children:[V.jsx(p4,{titleId:c.titleId}),V.jsx(y4,{contentRef:d,descriptionId:c.descriptionId})]})]})}),jx="DialogTitle",$P=q.forwardRef((e,t)=>{const{__scopeDialog:n,...r}=e,i=tr(jx,n);return V.jsx(Wr.h2,{id:i.titleId,...r,ref:t})});$P.displayName=jx;var BP="DialogDescription",h4=q.forwardRef((e,t)=>{const{__scopeDialog:n,...r}=e,i=tr(BP,n);return V.jsx(Wr.p,{id:i.descriptionId,...r,ref:t})});h4.displayName=BP;var zP="DialogClose",LP=q.forwardRef((e,t)=>{const{__scopeDialog:n,...r}=e,i=tr(zP,n);return V.jsx(Wr.button,{type:"button",...r,ref:t,onClick:ja(e.onClick,()=>i.onOpenChange(!1))})});LP.displayName=zP;function Cx(e){return e?"open":"closed"}var IP="DialogTitleWarning",[mF,UP]=GB(IP,{contentName:Si,titleName:jx,docsSlug:"dialog"}),p4=({titleId:e})=>{const t=UP(IP),n=`\`${t.contentName}\` requires a \`${t.titleName}\` for the component to be accessible for screen reader users.

If you want to hide the \`${t.titleName}\`, you can wrap it with our VisuallyHidden component.

For more information, see https://radix-ui.com/primitives/docs/components/${t.docsSlug}`;return q.useEffect(()=>{e&&(document.getElementById(e)||console.error(n))},[n,e]),null},v4="DialogDescriptionWarning",y4=({contentRef:e,descriptionId:t})=>{const r=`Warning: Missing \`Description\` or \`aria-describedby={undefined}\` for {${UP(v4).contentName}}.`;return q.useEffect(()=>{var u;const i=(u=e.current)==null?void 0:u.getAttribute("aria-describedby");t&&i&&(document.getElementById(t)||console.warn(r))},[r,e,t]),null},m4=CP,g4=NP,b4=DP,x4=kP,_4=$P,S4=LP;function w4({...e}){return V.jsx(m4,{"data-slot":"dialog",...e})}function O4({...e}){return V.jsx(g4,{"data-slot":"dialog-portal",...e})}function A4({className:e,...t}){return V.jsx(b4,{"data-slot":"dialog-overlay",className:er("data-[state=open]:animate-in data-[state=closed]:animate-out data-[state=closed]:fade-out-0 data-[state=open]:fade-in-0 fixed inset-0 z-50 bg-black/50",e),...t})}function E4({className:e,children:t,...n}){return V.jsxs(O4,{"data-slot":"dialog-portal",children:[V.jsx(A4,{}),V.jsxs(x4,{"data-slot":"dialog-content",className:er("bg-background data-[state=open]:animate-in data-[state=closed]:animate-out data-[state=closed]:fade-out-0 data-[state=open]:fade-in-0 data-[state=closed]:zoom-out-95 data-[state=open]:zoom-in-95 fixed top-[50%] left-[50%] z-50 grid w-full max-w-[calc(100%-2rem)] translate-x-[-50%] translate-y-[-50%] gap-4 rounded-lg border p-6 shadow-lg duration-200 sm:max-w-lg",e),...n,children:[t,V.jsxs(S4,{className:"ring-offset-background focus:ring-ring data-[state=open]:bg-accent data-[state=open]:text-muted-foreground absolute top-4 right-4 rounded-xs opacity-70 transition-opacity hover:opacity-100 focus:ring-2 focus:ring-offset-2 focus:outline-hidden disabled:pointer-events-none [&_svg]:pointer-events-none [&_svg]:shrink-0 [&_svg:not([class*='size-'])]:size-4",children:[V.jsx(dP,{}),V.jsx("span",{className:"sr-only",children:"Close"})]})]})]})}function T4({className:e,...t}){return V.jsx("div",{"data-slot":"dialog-header",className:er("flex flex-col gap-2 text-center sm:text-left",e),...t})}function M4({className:e,...t}){return V.jsx(_4,{"data-slot":"dialog-title",className:er("text-lg leading-none font-semibold",e),...t})}function j4({conversation:e,isOpen:t,onOpenChange:n}){var r;return e?V.jsx(w4,{open:t,onOpenChange:n,children:V.jsxs(E4,{className:"max-h-[90vh] overflow-hidden flex flex-col max-w-[80vw] min-w-[80vw]",children:[V.jsx(T4,{children:V.jsxs(M4,{className:"flex flex-col gap-1",children:[V.jsx("span",{children:"Conversation Details"}),V.jsxs("span",{className:"text-xs text-slate-500 font-normal",children:["ID: ",e.chat_id]})]})}),V.jsxs("div",{className:"border-b pb-3",children:[V.jsx("h3",{className:"text-sm font-medium mb-2",children:"Metadata"}),V.jsxs("div",{className:"grid grid-cols-2 gap-2",children:[e.metadata&&Object.entries(e.metadata).map(([i,u])=>V.jsxs("div",{className:"text-xs",children:[V.jsxs("span",{className:"font-medium",children:[i,":"]})," ",V.jsx("span",{className:"text-slate-600",children:Array.isArray(u)?u.join(", "):String(u)})]},i)),V.jsxs("div",{className:"text-xs",children:[V.jsx("span",{className:"font-medium",children:"Created:"})," ",V.jsx("span",{className:"text-slate-600",children:new Date(e.created_at).toLocaleString()})]}),V.jsxs("div",{className:"text-xs col-span-2",children:[V.jsx("span",{className:"font-medium",children:"Summary:"})," ",V.jsx("span",{className:"text-slate-600",children:e.summary})]})]})]}),V.jsxs("div",{className:"overflow-y-auto flex-1 p-2",children:[V.jsx("h3",{className:"text-sm font-medium mb-2",children:"Messages"}),V.jsx("div",{className:"space-y-4",children:(r=e.messages)==null?void 0:r.map((i,u)=>V.jsxs("div",{className:`flex flex-col ${i.role==="user"?"items-end":"items-start"}`,children:[V.jsx("div",{className:`max-w-[80%] rounded-xl px-4 py-2 ${i.role==="user"?"bg-blue-100 text-blue-900 rounded-br-none":"bg-slate-100 text-slate-800 rounded-bl-none"}`,children:V.jsx("p",{className:"whitespace-pre-wrap break-words",children:i.content})}),V.jsxs("div",{className:"text-xs text-slate-500 mt-1 px-2 flex gap-2",children:[V.jsx("span",{children:i.role==="user"?"User":"Assistant"}),V.jsx("span",{children:"·"}),V.jsx("span",{children:new Date(i.created_at).toLocaleTimeString()})]})]},u))})]})]})}):null}function C4({aggregatedMetadata:e}){const[t,n]=q.useState("individual");return Object.keys(e).length===0?null:V.jsxs("div",{className:"mt-3 border-t pt-2",children:[V.jsxs("div",{className:"flex justify-between items-center mb-2",children:[V.jsx("h4",{className:"text-xs font-semibold",children:"Metadata Summary"}),V.jsxs("div",{className:"flex items-center gap-2",children:[V.jsx("span",{className:"text-xs text-slate-600",children:"Aggregation:"}),V.jsx("button",{onClick:()=>n("individual"),className:`text-xs px-2 py-0.5 rounded ${t==="individual"?"bg-blue-100 text-blue-800":"bg-slate-100 text-slate-600"}`,children:"Individual"}),V.jsx("button",{onClick:()=>n("list"),className:`text-xs px-2 py-0.5 rounded ${t==="list"?"bg-blue-100 text-blue-800":"bg-slate-100 text-slate-600"}`,children:"List"})]})]}),V.jsx("div",{className:"grid grid-cols-2 gap-2",children:Object.entries(e).map(([r,i])=>{const u=i.reduce((s,c)=>{if(Array.isArray(c)&&t==="individual")c.forEach(d=>{const h=String(d);s[h]=(s[h]||0)+1});else{const d=Array.isArray(c)?c.join(", "):String(c);s[d]=(s[d]||0)+1}return s},{});return V.jsxs("div",{className:"text-xs",children:[V.jsxs("span",{className:"font-medium",children:[r,":"]})," ",V.jsx("div",{className:"flex flex-wrap gap-1 mt-1",children:Object.entries(u).map(([s,c])=>V.jsxs("div",{className:"inline-flex items-center border rounded-full px-2 py-0.5 bg-slate-100 text-slate-800 text-[10px]",children:[s,V.jsxs("span",{className:"ml-1 text-slate-500",children:["(",c,")"]})]},`${r}-${s}`))})]},r)})})]})}function P4({selectedCluster:e,conversationMetadataMap:t}){var h,v;const[n,r]=q.useState(null),[i,u]=q.useState(!1);if(!e)return null;const s=(h=e.chat_ids)==null?void 0:h.map(y=>t.get(y)),c=s.reduce((y,g)=>(!g||!g.metadata||Object.entries(g.metadata).forEach(([b,S])=>{y[b]||(y[b]=[]),Array.isArray(S),y[b].push(S)}),y),{}),d=y=>{r(y),u(!0)};return V.jsxs("div",{className:"flex-1 p-4 overflow-y-auto h-[50vh]",children:[V.jsx("h3",{className:"text-sm font-semibold mb-2",children:"Cluster Details"}),V.jsxs("div",{className:"rounded-md",children:[V.jsx("p",{className:"font-medium",children:e.name}),e.description&&V.jsx("p",{className:"text-xs text-slate-600 mt-1",children:e.description}),V.jsx("div",{className:"flex space-x-4 mt-2",children:V.jsxs("p",{className:"text-xs text-slate-600",children:[V.jsx("span",{className:"font-medium",children:"Level:"})," ",e.level]})}),e.id&&V.jsxs("p",{className:"text-xs text-slate-500 mt-1",children:["ID: ",e.id]}),V.jsxs("p",{className:"text-xs text-slate-500 mt-1",children:[(v=e.chat_ids)==null?void 0:v.length," chats"]}),V.jsx(C4,{aggregatedMetadata:c}),V.jsx("div",{className:"mt-3 space-y-2 ",children:s.map(y=>V.jsxs("div",{className:"p-2 border rounded-md cursor-pointer hover:bg-slate-50 transition-colors",onClick:()=>d(y),children:[V.jsxs("p",{className:"text-xs mb-3 text-slate-500 mt-1",children:["ID: ",y.chat_id]}),V.jsx("p",{className:"text-xs font-medium text-wrap",children:y.summary})]},y.chat_id))})]}),n&&V.jsx(j4,{conversation:n,isOpen:i,onOpenChange:u})]})}var Kv,FO;function fn(){if(FO)return Kv;FO=1;var e=Array.isArray;return Kv=e,Kv}var Wv,QO;function HP(){if(QO)return Wv;QO=1;var e=typeof lf=="object"&&lf&&lf.Object===Object&&lf;return Wv=e,Wv}var Fv,JO;function br(){if(JO)return Fv;JO=1;var e=HP(),t=typeof self=="object"&&self&&self.Object===Object&&self,n=e||t||Function("return this")();return Fv=n,Fv}var Qv,eA;function Ns(){if(eA)return Qv;eA=1;var e=br(),t=e.Symbol;return Qv=t,Qv}var Jv,tA;function R4(){if(tA)return Jv;tA=1;var e=Ns(),t=Object.prototype,n=t.hasOwnProperty,r=t.toString,i=e?e.toStringTag:void 0;function u(s){var c=n.call(s,i),d=s[i];try{s[i]=void 0;var h=!0}catch{}var v=r.call(s);return h&&(c?s[i]=d:delete s[i]),v}return Jv=u,Jv}var ey,nA;function N4(){if(nA)return ey;nA=1;var e=Object.prototype,t=e.toString;function n(r){return t.call(r)}return ey=n,ey}var ty,rA;function Fr(){if(rA)return ty;rA=1;var e=Ns(),t=R4(),n=N4(),r="[object Null]",i="[object Undefined]",u=e?e.toStringTag:void 0;function s(c){return c==null?c===void 0?i:r:u&&u in Object(c)?t(c):n(c)}return ty=s,ty}var ny,aA;function Qr(){if(aA)return ny;aA=1;function e(t){return t!=null&&typeof t=="object"}return ny=e,ny}var ry,iA;function cu(){if(iA)return ry;iA=1;var e=Fr(),t=Qr(),n="[object Symbol]";function r(i){return typeof i=="symbol"||t(i)&&e(i)==n}return ry=r,ry}var ay,oA;function Px(){if(oA)return ay;oA=1;var e=fn(),t=cu(),n=/\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/,r=/^\w*$/;function i(u,s){if(e(u))return!1;var c=typeof u;return c=="number"||c=="symbol"||c=="boolean"||u==null||t(u)?!0:r.test(u)||!n.test(u)||s!=null&&u in Object(s)}return ay=i,ay}var iy,uA;function qa(){if(uA)return iy;uA=1;function e(t){var n=typeof t;return t!=null&&(n=="object"||n=="function")}return iy=e,iy}var oy,lA;function Rx(){if(lA)return oy;lA=1;var e=Fr(),t=qa(),n="[object AsyncFunction]",r="[object Function]",i="[object GeneratorFunction]",u="[object Proxy]";function s(c){if(!t(c))return!1;var d=e(c);return d==r||d==i||d==n||d==u}return oy=s,oy}var uy,sA;function D4(){if(sA)return uy;sA=1;var e=br(),t=e["__core-js_shared__"];return uy=t,uy}var ly,cA;function k4(){if(cA)return ly;cA=1;var e=D4(),t=function(){var r=/[^.]+$/.exec(e&&e.keys&&e.keys.IE_PROTO||"");return r?"Symbol(src)_1."+r:""}();function n(r){return!!t&&t in r}return ly=n,ly}var sy,fA;function GP(){if(fA)return sy;fA=1;var e=Function.prototype,t=e.toString;function n(r){if(r!=null){try{return t.call(r)}catch{}try{return r+""}catch{}}return""}return sy=n,sy}var cy,dA;function q4(){if(dA)return cy;dA=1;var e=Rx(),t=k4(),n=qa(),r=GP(),i=/[\\^$.*+?()[\]{}|]/g,u=/^\[object .+?Constructor\]$/,s=Function.prototype,c=Object.prototype,d=s.toString,h=c.hasOwnProperty,v=RegExp("^"+d.call(h).replace(i,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$");function y(g){if(!n(g)||t(g))return!1;var b=e(g)?v:u;return b.test(r(g))}return cy=y,cy}var fy,hA;function $4(){if(hA)return fy;hA=1;function e(t,n){return t==null?void 0:t[n]}return fy=e,fy}var dy,pA;function Ti(){if(pA)return dy;pA=1;var e=q4(),t=$4();function n(r,i){var u=t(r,i);return e(u)?u:void 0}return dy=n,dy}var hy,vA;function zd(){if(vA)return hy;vA=1;var e=Ti(),t=e(Object,"create");return hy=t,hy}var py,yA;function B4(){if(yA)return py;yA=1;var e=zd();function t(){this.__data__=e?e(null):{},this.size=0}return py=t,py}var vy,mA;function z4(){if(mA)return vy;mA=1;function e(t){var n=this.has(t)&&delete this.__data__[t];return this.size-=n?1:0,n}return vy=e,vy}var yy,gA;function L4(){if(gA)return yy;gA=1;var e=zd(),t="__lodash_hash_undefined__",n=Object.prototype,r=n.hasOwnProperty;function i(u){var s=this.__data__;if(e){var c=s[u];return c===t?void 0:c}return r.call(s,u)?s[u]:void 0}return yy=i,yy}var my,bA;function I4(){if(bA)return my;bA=1;var e=zd(),t=Object.prototype,n=t.hasOwnProperty;function r(i){var u=this.__data__;return e?u[i]!==void 0:n.call(u,i)}return my=r,my}var gy,xA;function U4(){if(xA)return gy;xA=1;var e=zd(),t="__lodash_hash_undefined__";function n(r,i){var u=this.__data__;return this.size+=this.has(r)?0:1,u[r]=e&&i===void 0?t:i,this}return gy=n,gy}var by,_A;function H4(){if(_A)return by;_A=1;var e=B4(),t=z4(),n=L4(),r=I4(),i=U4();function u(s){var c=-1,d=s==null?0:s.length;for(this.clear();++c<d;){var h=s[c];this.set(h[0],h[1])}}return u.prototype.clear=e,u.prototype.delete=t,u.prototype.get=n,u.prototype.has=r,u.prototype.set=i,by=u,by}var xy,SA;function G4(){if(SA)return xy;SA=1;function e(){this.__data__=[],this.size=0}return xy=e,xy}var _y,wA;function Nx(){if(wA)return _y;wA=1;function e(t,n){return t===n||t!==t&&n!==n}return _y=e,_y}var Sy,OA;function Ld(){if(OA)return Sy;OA=1;var e=Nx();function t(n,r){for(var i=n.length;i--;)if(e(n[i][0],r))return i;return-1}return Sy=t,Sy}var wy,AA;function V4(){if(AA)return wy;AA=1;var e=Ld(),t=Array.prototype,n=t.splice;function r(i){var u=this.__data__,s=e(u,i);if(s<0)return!1;var c=u.length-1;return s==c?u.pop():n.call(u,s,1),--this.size,!0}return wy=r,wy}var Oy,EA;function Z4(){if(EA)return Oy;EA=1;var e=Ld();function t(n){var r=this.__data__,i=e(r,n);return i<0?void 0:r[i][1]}return Oy=t,Oy}var Ay,TA;function Y4(){if(TA)return Ay;TA=1;var e=Ld();function t(n){return e(this.__data__,n)>-1}return Ay=t,Ay}var Ey,MA;function X4(){if(MA)return Ey;MA=1;var e=Ld();function t(n,r){var i=this.__data__,u=e(i,n);return u<0?(++this.size,i.push([n,r])):i[u][1]=r,this}return Ey=t,Ey}var Ty,jA;function Id(){if(jA)return Ty;jA=1;var e=G4(),t=V4(),n=Z4(),r=Y4(),i=X4();function u(s){var c=-1,d=s==null?0:s.length;for(this.clear();++c<d;){var h=s[c];this.set(h[0],h[1])}}return u.prototype.clear=e,u.prototype.delete=t,u.prototype.get=n,u.prototype.has=r,u.prototype.set=i,Ty=u,Ty}var My,CA;function Dx(){if(CA)return My;CA=1;var e=Ti(),t=br(),n=e(t,"Map");return My=n,My}var jy,PA;function K4(){if(PA)return jy;PA=1;var e=H4(),t=Id(),n=Dx();function r(){this.size=0,this.__data__={hash:new e,map:new(n||t),string:new e}}return jy=r,jy}var Cy,RA;function W4(){if(RA)return Cy;RA=1;function e(t){var n=typeof t;return n=="string"||n=="number"||n=="symbol"||n=="boolean"?t!=="__proto__":t===null}return Cy=e,Cy}var Py,NA;function Ud(){if(NA)return Py;NA=1;var e=W4();function t(n,r){var i=n.__data__;return e(r)?i[typeof r=="string"?"string":"hash"]:i.map}return Py=t,Py}var Ry,DA;function F4(){if(DA)return Ry;DA=1;var e=Ud();function t(n){var r=e(this,n).delete(n);return this.size-=r?1:0,r}return Ry=t,Ry}var Ny,kA;function Q4(){if(kA)return Ny;kA=1;var e=Ud();function t(n){return e(this,n).get(n)}return Ny=t,Ny}var Dy,qA;function J4(){if(qA)return Dy;qA=1;var e=Ud();function t(n){return e(this,n).has(n)}return Dy=t,Dy}var ky,$A;function eL(){if($A)return ky;$A=1;var e=Ud();function t(n,r){var i=e(this,n),u=i.size;return i.set(n,r),this.size+=i.size==u?0:1,this}return ky=t,ky}var qy,BA;function kx(){if(BA)return qy;BA=1;var e=K4(),t=F4(),n=Q4(),r=J4(),i=eL();function u(s){var c=-1,d=s==null?0:s.length;for(this.clear();++c<d;){var h=s[c];this.set(h[0],h[1])}}return u.prototype.clear=e,u.prototype.delete=t,u.prototype.get=n,u.prototype.has=r,u.prototype.set=i,qy=u,qy}var $y,zA;function VP(){if(zA)return $y;zA=1;var e=kx(),t="Expected a function";function n(r,i){if(typeof r!="function"||i!=null&&typeof i!="function")throw new TypeError(t);var u=function(){var s=arguments,c=i?i.apply(this,s):s[0],d=u.cache;if(d.has(c))return d.get(c);var h=r.apply(this,s);return u.cache=d.set(c,h)||d,h};return u.cache=new(n.Cache||e),u}return n.Cache=e,$y=n,$y}var By,LA;function tL(){if(LA)return By;LA=1;var e=VP(),t=500;function n(r){var i=e(r,function(s){return u.size===t&&u.clear(),s}),u=i.cache;return i}return By=n,By}var zy,IA;function nL(){if(IA)return zy;IA=1;var e=tL(),t=/[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g,n=/\\(\\)?/g,r=e(function(i){var u=[];return i.charCodeAt(0)===46&&u.push(""),i.replace(t,function(s,c,d,h){u.push(d?h.replace(n,"$1"):c||s)}),u});return zy=r,zy}var Ly,UA;function qx(){if(UA)return Ly;UA=1;function e(t,n){for(var r=-1,i=t==null?0:t.length,u=Array(i);++r<i;)u[r]=n(t[r],r,t);return u}return Ly=e,Ly}var Iy,HA;function rL(){if(HA)return Iy;HA=1;var e=Ns(),t=qx(),n=fn(),r=cu(),i=e?e.prototype:void 0,u=i?i.toString:void 0;function s(c){if(typeof c=="string")return c;if(n(c))return t(c,s)+"";if(r(c))return u?u.call(c):"";var d=c+"";return d=="0"&&1/c==-1/0?"-0":d}return Iy=s,Iy}var Uy,GA;function ZP(){if(GA)return Uy;GA=1;var e=rL();function t(n){return n==null?"":e(n)}return Uy=t,Uy}var Hy,VA;function YP(){if(VA)return Hy;VA=1;var e=fn(),t=Px(),n=nL(),r=ZP();function i(u,s){return e(u)?u:t(u,s)?[u]:n(r(u))}return Hy=i,Hy}var Gy,ZA;function Hd(){if(ZA)return Gy;ZA=1;var e=cu();function t(n){if(typeof n=="string"||e(n))return n;var r=n+"";return r=="0"&&1/n==-1/0?"-0":r}return Gy=t,Gy}var Vy,YA;function $x(){if(YA)return Vy;YA=1;var e=YP(),t=Hd();function n(r,i){i=e(i,r);for(var u=0,s=i.length;r!=null&&u<s;)r=r[t(i[u++])];return u&&u==s?r:void 0}return Vy=n,Vy}var Zy,XA;function XP(){if(XA)return Zy;XA=1;var e=$x();function t(n,r,i){var u=n==null?void 0:e(n,r);return u===void 0?i:u}return Zy=t,Zy}var aL=XP();const Ln=lt(aL);var Yy,KA;function iL(){if(KA)return Yy;KA=1;function e(t){return t==null}return Yy=e,Yy}var oL=iL();const ze=lt(oL);var Xy,WA;function uL(){if(WA)return Xy;WA=1;var e=Fr(),t=fn(),n=Qr(),r="[object String]";function i(u){return typeof u=="string"||!t(u)&&n(u)&&e(u)==r}return Xy=i,Xy}var lL=uL();const Ds=lt(lL);var sL=Rx();const Ge=lt(sL);var cL=qa();const fu=lt(cL);var Ky={exports:{}},Qe={};/**
 * @license React
 * react-is.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var FA;function fL(){if(FA)return Qe;FA=1;var e=Symbol.for("react.element"),t=Symbol.for("react.portal"),n=Symbol.for("react.fragment"),r=Symbol.for("react.strict_mode"),i=Symbol.for("react.profiler"),u=Symbol.for("react.provider"),s=Symbol.for("react.context"),c=Symbol.for("react.server_context"),d=Symbol.for("react.forward_ref"),h=Symbol.for("react.suspense"),v=Symbol.for("react.suspense_list"),y=Symbol.for("react.memo"),g=Symbol.for("react.lazy"),b=Symbol.for("react.offscreen"),S;S=Symbol.for("react.module.reference");function _(x){if(typeof x=="object"&&x!==null){var E=x.$$typeof;switch(E){case e:switch(x=x.type,x){case n:case i:case r:case h:case v:return x;default:switch(x=x&&x.$$typeof,x){case c:case s:case d:case g:case y:case u:return x;default:return E}}case t:return E}}}return Qe.ContextConsumer=s,Qe.ContextProvider=u,Qe.Element=e,Qe.ForwardRef=d,Qe.Fragment=n,Qe.Lazy=g,Qe.Memo=y,Qe.Portal=t,Qe.Profiler=i,Qe.StrictMode=r,Qe.Suspense=h,Qe.SuspenseList=v,Qe.isAsyncMode=function(){return!1},Qe.isConcurrentMode=function(){return!1},Qe.isContextConsumer=function(x){return _(x)===s},Qe.isContextProvider=function(x){return _(x)===u},Qe.isElement=function(x){return typeof x=="object"&&x!==null&&x.$$typeof===e},Qe.isForwardRef=function(x){return _(x)===d},Qe.isFragment=function(x){return _(x)===n},Qe.isLazy=function(x){return _(x)===g},Qe.isMemo=function(x){return _(x)===y},Qe.isPortal=function(x){return _(x)===t},Qe.isProfiler=function(x){return _(x)===i},Qe.isStrictMode=function(x){return _(x)===r},Qe.isSuspense=function(x){return _(x)===h},Qe.isSuspenseList=function(x){return _(x)===v},Qe.isValidElementType=function(x){return typeof x=="string"||typeof x=="function"||x===n||x===i||x===r||x===h||x===v||x===b||typeof x=="object"&&x!==null&&(x.$$typeof===g||x.$$typeof===y||x.$$typeof===u||x.$$typeof===s||x.$$typeof===d||x.$$typeof===S||x.getModuleId!==void 0)},Qe.typeOf=_,Qe}var QA;function dL(){return QA||(QA=1,Ky.exports=fL()),Ky.exports}var hL=dL(),Wy,JA;function KP(){if(JA)return Wy;JA=1;var e=Fr(),t=Qr(),n="[object Number]";function r(i){return typeof i=="number"||t(i)&&e(i)==n}return Wy=r,Wy}var Fy,eE;function pL(){if(eE)return Fy;eE=1;var e=KP();function t(n){return e(n)&&n!=+n}return Fy=t,Fy}var vL=pL();const ks=lt(vL);var yL=KP();const mL=lt(yL);var Fn=function(t){return t===0?0:t>0?1:-1},ci=function(t){return Ds(t)&&t.indexOf("%")===t.length-1},he=function(t){return mL(t)&&!ks(t)},Dt=function(t){return he(t)||Ds(t)},gL=0,qs=function(t){var n=++gL;return"".concat(t||"").concat(n)},wi=function(t,n){var r=arguments.length>2&&arguments[2]!==void 0?arguments[2]:0,i=arguments.length>3&&arguments[3]!==void 0?arguments[3]:!1;if(!he(t)&&!Ds(t))return r;var u;if(ci(t)){var s=t.indexOf("%");u=n*parseFloat(t.slice(0,s))/100}else u=+t;return ks(u)&&(u=r),i&&u>n&&(u=n),u},Ao=function(t){if(!t)return null;var n=Object.keys(t);return n&&n.length?t[n[0]]:null},bL=function(t){if(!Array.isArray(t))return!1;for(var n=t.length,r={},i=0;i<n;i++)if(!r[t[i]])r[t[i]]=!0;else return!0;return!1},cr=function(t,n){return he(t)&&he(n)?function(r){return t+r*(n-t)}:function(){return n}};function Lf(e,t,n){return!e||!e.length?null:e.find(function(r){return r&&(typeof t=="function"?t(r):Ln(r,t))===n})}var xL=function(t){if(!t||!t.length)return null;for(var n=t.length,r=0,i=0,u=0,s=0,c=1/0,d=-1/0,h=0,v=0,y=0;y<n;y++)h=t[y].cx||0,v=t[y].cy||0,r+=h,i+=v,u+=h*v,s+=h*h,c=Math.min(c,h),d=Math.max(d,h);var g=n*s!==r*r?(n*u-r*i)/(n*s-r*r):0;return{xmin:c,xmax:d,a:g,b:(i-g*r)/n}};function Ro(e,t){for(var n in e)if({}.hasOwnProperty.call(e,n)&&(!{}.hasOwnProperty.call(t,n)||e[n]!==t[n]))return!1;for(var r in t)if({}.hasOwnProperty.call(t,r)&&!{}.hasOwnProperty.call(e,r))return!1;return!0}function a0(e){"@babel/helpers - typeof";return a0=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},a0(e)}var _L=["viewBox","children"],SL=["aria-activedescendant","aria-atomic","aria-autocomplete","aria-busy","aria-checked","aria-colcount","aria-colindex","aria-colspan","aria-controls","aria-current","aria-describedby","aria-details","aria-disabled","aria-errormessage","aria-expanded","aria-flowto","aria-haspopup","aria-hidden","aria-invalid","aria-keyshortcuts","aria-label","aria-labelledby","aria-level","aria-live","aria-modal","aria-multiline","aria-multiselectable","aria-orientation","aria-owns","aria-placeholder","aria-posinset","aria-pressed","aria-readonly","aria-relevant","aria-required","aria-roledescription","aria-rowcount","aria-rowindex","aria-rowspan","aria-selected","aria-setsize","aria-sort","aria-valuemax","aria-valuemin","aria-valuenow","aria-valuetext","className","color","height","id","lang","max","media","method","min","name","style","target","width","role","tabIndex","accentHeight","accumulate","additive","alignmentBaseline","allowReorder","alphabetic","amplitude","arabicForm","ascent","attributeName","attributeType","autoReverse","azimuth","baseFrequency","baselineShift","baseProfile","bbox","begin","bias","by","calcMode","capHeight","clip","clipPath","clipPathUnits","clipRule","colorInterpolation","colorInterpolationFilters","colorProfile","colorRendering","contentScriptType","contentStyleType","cursor","cx","cy","d","decelerate","descent","diffuseConstant","direction","display","divisor","dominantBaseline","dur","dx","dy","edgeMode","elevation","enableBackground","end","exponent","externalResourcesRequired","fill","fillOpacity","fillRule","filter","filterRes","filterUnits","floodColor","floodOpacity","focusable","fontFamily","fontSize","fontSizeAdjust","fontStretch","fontStyle","fontVariant","fontWeight","format","from","fx","fy","g1","g2","glyphName","glyphOrientationHorizontal","glyphOrientationVertical","glyphRef","gradientTransform","gradientUnits","hanging","horizAdvX","horizOriginX","href","ideographic","imageRendering","in2","in","intercept","k1","k2","k3","k4","k","kernelMatrix","kernelUnitLength","kerning","keyPoints","keySplines","keyTimes","lengthAdjust","letterSpacing","lightingColor","limitingConeAngle","local","markerEnd","markerHeight","markerMid","markerStart","markerUnits","markerWidth","mask","maskContentUnits","maskUnits","mathematical","mode","numOctaves","offset","opacity","operator","order","orient","orientation","origin","overflow","overlinePosition","overlineThickness","paintOrder","panose1","pathLength","patternContentUnits","patternTransform","patternUnits","pointerEvents","pointsAtX","pointsAtY","pointsAtZ","preserveAlpha","preserveAspectRatio","primitiveUnits","r","radius","refX","refY","renderingIntent","repeatCount","repeatDur","requiredExtensions","requiredFeatures","restart","result","rotate","rx","ry","seed","shapeRendering","slope","spacing","specularConstant","specularExponent","speed","spreadMethod","startOffset","stdDeviation","stemh","stemv","stitchTiles","stopColor","stopOpacity","strikethroughPosition","strikethroughThickness","string","stroke","strokeDasharray","strokeDashoffset","strokeLinecap","strokeLinejoin","strokeMiterlimit","strokeOpacity","strokeWidth","surfaceScale","systemLanguage","tableValues","targetX","targetY","textAnchor","textDecoration","textLength","textRendering","to","transform","u1","u2","underlinePosition","underlineThickness","unicode","unicodeBidi","unicodeRange","unitsPerEm","vAlphabetic","values","vectorEffect","version","vertAdvY","vertOriginX","vertOriginY","vHanging","vIdeographic","viewTarget","visibility","vMathematical","widths","wordSpacing","writingMode","x1","x2","x","xChannelSelector","xHeight","xlinkActuate","xlinkArcrole","xlinkHref","xlinkRole","xlinkShow","xlinkTitle","xlinkType","xmlBase","xmlLang","xmlns","xmlnsXlink","xmlSpace","y1","y2","y","yChannelSelector","z","zoomAndPan","ref","key","angle"],tE=["points","pathLength"],Qy={svg:_L,polygon:tE,polyline:tE},Bx=["dangerouslySetInnerHTML","onCopy","onCopyCapture","onCut","onCutCapture","onPaste","onPasteCapture","onCompositionEnd","onCompositionEndCapture","onCompositionStart","onCompositionStartCapture","onCompositionUpdate","onCompositionUpdateCapture","onFocus","onFocusCapture","onBlur","onBlurCapture","onChange","onChangeCapture","onBeforeInput","onBeforeInputCapture","onInput","onInputCapture","onReset","onResetCapture","onSubmit","onSubmitCapture","onInvalid","onInvalidCapture","onLoad","onLoadCapture","onError","onErrorCapture","onKeyDown","onKeyDownCapture","onKeyPress","onKeyPressCapture","onKeyUp","onKeyUpCapture","onAbort","onAbortCapture","onCanPlay","onCanPlayCapture","onCanPlayThrough","onCanPlayThroughCapture","onDurationChange","onDurationChangeCapture","onEmptied","onEmptiedCapture","onEncrypted","onEncryptedCapture","onEnded","onEndedCapture","onLoadedData","onLoadedDataCapture","onLoadedMetadata","onLoadedMetadataCapture","onLoadStart","onLoadStartCapture","onPause","onPauseCapture","onPlay","onPlayCapture","onPlaying","onPlayingCapture","onProgress","onProgressCapture","onRateChange","onRateChangeCapture","onSeeked","onSeekedCapture","onSeeking","onSeekingCapture","onStalled","onStalledCapture","onSuspend","onSuspendCapture","onTimeUpdate","onTimeUpdateCapture","onVolumeChange","onVolumeChangeCapture","onWaiting","onWaitingCapture","onAuxClick","onAuxClickCapture","onClick","onClickCapture","onContextMenu","onContextMenuCapture","onDoubleClick","onDoubleClickCapture","onDrag","onDragCapture","onDragEnd","onDragEndCapture","onDragEnter","onDragEnterCapture","onDragExit","onDragExitCapture","onDragLeave","onDragLeaveCapture","onDragOver","onDragOverCapture","onDragStart","onDragStartCapture","onDrop","onDropCapture","onMouseDown","onMouseDownCapture","onMouseEnter","onMouseLeave","onMouseMove","onMouseMoveCapture","onMouseOut","onMouseOutCapture","onMouseOver","onMouseOverCapture","onMouseUp","onMouseUpCapture","onSelect","onSelectCapture","onTouchCancel","onTouchCancelCapture","onTouchEnd","onTouchEndCapture","onTouchMove","onTouchMoveCapture","onTouchStart","onTouchStartCapture","onPointerDown","onPointerDownCapture","onPointerMove","onPointerMoveCapture","onPointerUp","onPointerUpCapture","onPointerCancel","onPointerCancelCapture","onPointerEnter","onPointerEnterCapture","onPointerLeave","onPointerLeaveCapture","onPointerOver","onPointerOverCapture","onPointerOut","onPointerOutCapture","onGotPointerCapture","onGotPointerCaptureCapture","onLostPointerCapture","onLostPointerCaptureCapture","onScroll","onScrollCapture","onWheel","onWheelCapture","onAnimationStart","onAnimationStartCapture","onAnimationEnd","onAnimationEndCapture","onAnimationIteration","onAnimationIterationCapture","onTransitionEnd","onTransitionEndCapture"],If=function(t,n){if(!t||typeof t=="function"||typeof t=="boolean")return null;var r=t;if(q.isValidElement(t)&&(r=t.props),!fu(r))return null;var i={};return Object.keys(r).forEach(function(u){Bx.includes(u)&&(i[u]=n||function(s){return r[u](r,s)})}),i},wL=function(t,n,r){return function(i){return t(n,r,i),null}},Wl=function(t,n,r){if(!fu(t)||a0(t)!=="object")return null;var i=null;return Object.keys(t).forEach(function(u){var s=t[u];Bx.includes(u)&&typeof s=="function"&&(i||(i={}),i[u]=wL(s,n,r))}),i},OL=["children"],AL=["children"];function nE(e,t){if(e==null)return{};var n=EL(e,t),r,i;if(Object.getOwnPropertySymbols){var u=Object.getOwnPropertySymbols(e);for(i=0;i<u.length;i++)r=u[i],!(t.indexOf(r)>=0)&&Object.prototype.propertyIsEnumerable.call(e,r)&&(n[r]=e[r])}return n}function EL(e,t){if(e==null)return{};var n={};for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){if(t.indexOf(r)>=0)continue;n[r]=e[r]}return n}var rE={click:"onClick",mousedown:"onMouseDown",mouseup:"onMouseUp",mouseover:"onMouseOver",mousemove:"onMouseMove",mouseout:"onMouseOut",mouseenter:"onMouseEnter",mouseleave:"onMouseLeave",touchcancel:"onTouchCancel",touchend:"onTouchEnd",touchmove:"onTouchMove",touchstart:"onTouchStart",contextmenu:"onContextMenu",dblclick:"onDoubleClick"},Hr=function(t){return typeof t=="string"?t:t?t.displayName||t.name||"Component":""},aE=null,Jy=null,zx=function e(t){if(t===aE&&Array.isArray(Jy))return Jy;var n=[];return q.Children.forEach(t,function(r){ze(r)||(hL.isFragment(r)?n=n.concat(e(r.props.children)):n.push(r))}),Jy=n,aE=t,n};function An(e,t){var n=[],r=[];return Array.isArray(t)?r=t.map(function(i){return Hr(i)}):r=[Hr(t)],zx(e).forEach(function(i){var u=Ln(i,"type.displayName")||Ln(i,"type.name");r.indexOf(u)!==-1&&n.push(i)}),n}function Sn(e,t){var n=An(e,t);return n&&n[0]}var iE=function(t){if(!t||!t.props)return!1;var n=t.props,r=n.width,i=n.height;return!(!he(r)||r<=0||!he(i)||i<=0)},TL=["a","altGlyph","altGlyphDef","altGlyphItem","animate","animateColor","animateMotion","animateTransform","circle","clipPath","color-profile","cursor","defs","desc","ellipse","feBlend","feColormatrix","feComponentTransfer","feComposite","feConvolveMatrix","feDiffuseLighting","feDisplacementMap","feDistantLight","feFlood","feFuncA","feFuncB","feFuncG","feFuncR","feGaussianBlur","feImage","feMerge","feMergeNode","feMorphology","feOffset","fePointLight","feSpecularLighting","feSpotLight","feTile","feTurbulence","filter","font","font-face","font-face-format","font-face-name","font-face-url","foreignObject","g","glyph","glyphRef","hkern","image","line","lineGradient","marker","mask","metadata","missing-glyph","mpath","path","pattern","polygon","polyline","radialGradient","rect","script","set","stop","style","svg","switch","symbol","text","textPath","title","tref","tspan","use","view","vkern"],ML=function(t){return t&&t.type&&Ds(t.type)&&TL.indexOf(t.type)>=0},jL=function(t,n,r,i){var u,s=(u=Qy==null?void 0:Qy[i])!==null&&u!==void 0?u:[];return n.startsWith("data-")||!Ge(t)&&(i&&s.includes(n)||SL.includes(n))||r&&Bx.includes(n)},Ye=function(t,n,r){if(!t||typeof t=="function"||typeof t=="boolean")return null;var i=t;if(q.isValidElement(t)&&(i=t.props),!fu(i))return null;var u={};return Object.keys(i).forEach(function(s){var c;jL((c=i)===null||c===void 0?void 0:c[s],s,n,r)&&(u[s]=i[s])}),u},i0=function e(t,n){if(t===n)return!0;var r=q.Children.count(t);if(r!==q.Children.count(n))return!1;if(r===0)return!0;if(r===1)return oE(Array.isArray(t)?t[0]:t,Array.isArray(n)?n[0]:n);for(var i=0;i<r;i++){var u=t[i],s=n[i];if(Array.isArray(u)||Array.isArray(s)){if(!e(u,s))return!1}else if(!oE(u,s))return!1}return!0},oE=function(t,n){if(ze(t)&&ze(n))return!0;if(!ze(t)&&!ze(n)){var r=t.props||{},i=r.children,u=nE(r,OL),s=n.props||{},c=s.children,d=nE(s,AL);return i&&c?Ro(u,d)&&i0(i,c):!i&&!c?Ro(u,d):!1}return!1},uE=function(t,n){var r=[],i={};return zx(t).forEach(function(u,s){if(ML(u))r.push(u);else if(u){var c=Hr(u.type),d=n[c]||{},h=d.handler,v=d.once;if(h&&(!v||!i[c])){var y=h(u,c,s);r.push(y),i[c]=!0}}}),r},CL=function(t){var n=t&&t.type;return n&&rE[n]?rE[n]:null},PL=function(t,n){return zx(n).indexOf(t)},RL=["children","width","height","viewBox","className","style","title","desc"];function o0(){return o0=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},o0.apply(this,arguments)}function NL(e,t){if(e==null)return{};var n=DL(e,t),r,i;if(Object.getOwnPropertySymbols){var u=Object.getOwnPropertySymbols(e);for(i=0;i<u.length;i++)r=u[i],!(t.indexOf(r)>=0)&&Object.prototype.propertyIsEnumerable.call(e,r)&&(n[r]=e[r])}return n}function DL(e,t){if(e==null)return{};var n={};for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){if(t.indexOf(r)>=0)continue;n[r]=e[r]}return n}function u0(e){var t=e.children,n=e.width,r=e.height,i=e.viewBox,u=e.className,s=e.style,c=e.title,d=e.desc,h=NL(e,RL),v=i||{width:n,height:r,x:0,y:0},y=Xe("recharts-surface",u);return L.createElement("svg",o0({},Ye(h,!0,"svg"),{className:y,width:n,height:r,style:s,viewBox:"".concat(v.x," ").concat(v.y," ").concat(v.width," ").concat(v.height)}),L.createElement("title",null,c),L.createElement("desc",null,d),t)}var kL=["children","className"];function l0(){return l0=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},l0.apply(this,arguments)}function qL(e,t){if(e==null)return{};var n=$L(e,t),r,i;if(Object.getOwnPropertySymbols){var u=Object.getOwnPropertySymbols(e);for(i=0;i<u.length;i++)r=u[i],!(t.indexOf(r)>=0)&&Object.prototype.propertyIsEnumerable.call(e,r)&&(n[r]=e[r])}return n}function $L(e,t){if(e==null)return{};var n={};for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){if(t.indexOf(r)>=0)continue;n[r]=e[r]}return n}var gt=L.forwardRef(function(e,t){var n=e.children,r=e.className,i=qL(e,kL),u=Xe("recharts-layer",r);return L.createElement("g",l0({className:u},Ye(i,!0),{ref:t}),n)}),vi=function(t,n){for(var r=arguments.length,i=new Array(r>2?r-2:0),u=2;u<r;u++)i[u-2]=arguments[u]},em,lE;function BL(){if(lE)return em;lE=1;function e(t,n,r){var i=-1,u=t.length;n<0&&(n=-n>u?0:u+n),r=r>u?u:r,r<0&&(r+=u),u=n>r?0:r-n>>>0,n>>>=0;for(var s=Array(u);++i<u;)s[i]=t[i+n];return s}return em=e,em}var tm,sE;function zL(){if(sE)return tm;sE=1;var e=BL();function t(n,r,i){var u=n.length;return i=i===void 0?u:i,!r&&i>=u?n:e(n,r,i)}return tm=t,tm}var nm,cE;function WP(){if(cE)return nm;cE=1;var e="\\ud800-\\udfff",t="\\u0300-\\u036f",n="\\ufe20-\\ufe2f",r="\\u20d0-\\u20ff",i=t+n+r,u="\\ufe0e\\ufe0f",s="\\u200d",c=RegExp("["+s+e+i+u+"]");function d(h){return c.test(h)}return nm=d,nm}var rm,fE;function LL(){if(fE)return rm;fE=1;function e(t){return t.split("")}return rm=e,rm}var am,dE;function IL(){if(dE)return am;dE=1;var e="\\ud800-\\udfff",t="\\u0300-\\u036f",n="\\ufe20-\\ufe2f",r="\\u20d0-\\u20ff",i=t+n+r,u="\\ufe0e\\ufe0f",s="["+e+"]",c="["+i+"]",d="\\ud83c[\\udffb-\\udfff]",h="(?:"+c+"|"+d+")",v="[^"+e+"]",y="(?:\\ud83c[\\udde6-\\uddff]){2}",g="[\\ud800-\\udbff][\\udc00-\\udfff]",b="\\u200d",S=h+"?",_="["+u+"]?",x="(?:"+b+"(?:"+[v,y,g].join("|")+")"+_+S+")*",E=_+S+x,M="(?:"+[v+c+"?",c,y,g,s].join("|")+")",A=RegExp(d+"(?="+d+")|"+M+E,"g");function C(w){return w.match(A)||[]}return am=C,am}var im,hE;function UL(){if(hE)return im;hE=1;var e=LL(),t=WP(),n=IL();function r(i){return t(i)?n(i):e(i)}return im=r,im}var om,pE;function HL(){if(pE)return om;pE=1;var e=zL(),t=WP(),n=UL(),r=ZP();function i(u){return function(s){s=r(s);var c=t(s)?n(s):void 0,d=c?c[0]:s.charAt(0),h=c?e(c,1).join(""):s.slice(1);return d[u]()+h}}return om=i,om}var um,vE;function GL(){if(vE)return um;vE=1;var e=HL(),t=e("toUpperCase");return um=t,um}var VL=GL();const Gd=lt(VL);function ft(e){return function(){return e}}const FP=Math.cos,Uf=Math.sin,nr=Math.sqrt,Hf=Math.PI,Vd=2*Hf,s0=Math.PI,c0=2*s0,li=1e-6,ZL=c0-li;function QP(e){this._+=e[0];for(let t=1,n=e.length;t<n;++t)this._+=arguments[t]+e[t]}function YL(e){let t=Math.floor(e);if(!(t>=0))throw new Error(`invalid digits: ${e}`);if(t>15)return QP;const n=10**t;return function(r){this._+=r[0];for(let i=1,u=r.length;i<u;++i)this._+=Math.round(arguments[i]*n)/n+r[i]}}class XL{constructor(t){this._x0=this._y0=this._x1=this._y1=null,this._="",this._append=t==null?QP:YL(t)}moveTo(t,n){this._append`M${this._x0=this._x1=+t},${this._y0=this._y1=+n}`}closePath(){this._x1!==null&&(this._x1=this._x0,this._y1=this._y0,this._append`Z`)}lineTo(t,n){this._append`L${this._x1=+t},${this._y1=+n}`}quadraticCurveTo(t,n,r,i){this._append`Q${+t},${+n},${this._x1=+r},${this._y1=+i}`}bezierCurveTo(t,n,r,i,u,s){this._append`C${+t},${+n},${+r},${+i},${this._x1=+u},${this._y1=+s}`}arcTo(t,n,r,i,u){if(t=+t,n=+n,r=+r,i=+i,u=+u,u<0)throw new Error(`negative radius: ${u}`);let s=this._x1,c=this._y1,d=r-t,h=i-n,v=s-t,y=c-n,g=v*v+y*y;if(this._x1===null)this._append`M${this._x1=t},${this._y1=n}`;else if(g>li)if(!(Math.abs(y*d-h*v)>li)||!u)this._append`L${this._x1=t},${this._y1=n}`;else{let b=r-s,S=i-c,_=d*d+h*h,x=b*b+S*S,E=Math.sqrt(_),M=Math.sqrt(g),A=u*Math.tan((s0-Math.acos((_+g-x)/(2*E*M)))/2),C=A/M,w=A/E;Math.abs(C-1)>li&&this._append`L${t+C*v},${n+C*y}`,this._append`A${u},${u},0,0,${+(y*b>v*S)},${this._x1=t+w*d},${this._y1=n+w*h}`}}arc(t,n,r,i,u,s){if(t=+t,n=+n,r=+r,s=!!s,r<0)throw new Error(`negative radius: ${r}`);let c=r*Math.cos(i),d=r*Math.sin(i),h=t+c,v=n+d,y=1^s,g=s?i-u:u-i;this._x1===null?this._append`M${h},${v}`:(Math.abs(this._x1-h)>li||Math.abs(this._y1-v)>li)&&this._append`L${h},${v}`,r&&(g<0&&(g=g%c0+c0),g>ZL?this._append`A${r},${r},0,1,${y},${t-c},${n-d}A${r},${r},0,1,${y},${this._x1=h},${this._y1=v}`:g>li&&this._append`A${r},${r},0,${+(g>=s0)},${y},${this._x1=t+r*Math.cos(u)},${this._y1=n+r*Math.sin(u)}`)}rect(t,n,r,i){this._append`M${this._x0=this._x1=+t},${this._y0=this._y1=+n}h${r=+r}v${+i}h${-r}Z`}toString(){return this._}}function Lx(e){let t=3;return e.digits=function(n){if(!arguments.length)return t;if(n==null)t=null;else{const r=Math.floor(n);if(!(r>=0))throw new RangeError(`invalid digits: ${n}`);t=r}return e},()=>new XL(t)}function Ix(e){return typeof e=="object"&&"length"in e?e:Array.from(e)}function JP(e){this._context=e}JP.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._point=0},lineEnd:function(){(this._line||this._line!==0&&this._point===1)&&this._context.closePath(),this._line=1-this._line},point:function(e,t){switch(e=+e,t=+t,this._point){case 0:this._point=1,this._line?this._context.lineTo(e,t):this._context.moveTo(e,t);break;case 1:this._point=2;default:this._context.lineTo(e,t);break}}};function Zd(e){return new JP(e)}function eR(e){return e[0]}function tR(e){return e[1]}function nR(e,t){var n=ft(!0),r=null,i=Zd,u=null,s=Lx(c);e=typeof e=="function"?e:e===void 0?eR:ft(e),t=typeof t=="function"?t:t===void 0?tR:ft(t);function c(d){var h,v=(d=Ix(d)).length,y,g=!1,b;for(r==null&&(u=i(b=s())),h=0;h<=v;++h)!(h<v&&n(y=d[h],h,d))===g&&((g=!g)?u.lineStart():u.lineEnd()),g&&u.point(+e(y,h,d),+t(y,h,d));if(b)return u=null,b+""||null}return c.x=function(d){return arguments.length?(e=typeof d=="function"?d:ft(+d),c):e},c.y=function(d){return arguments.length?(t=typeof d=="function"?d:ft(+d),c):t},c.defined=function(d){return arguments.length?(n=typeof d=="function"?d:ft(!!d),c):n},c.curve=function(d){return arguments.length?(i=d,r!=null&&(u=i(r)),c):i},c.context=function(d){return arguments.length?(d==null?r=u=null:u=i(r=d),c):r},c}function yf(e,t,n){var r=null,i=ft(!0),u=null,s=Zd,c=null,d=Lx(h);e=typeof e=="function"?e:e===void 0?eR:ft(+e),t=typeof t=="function"?t:ft(t===void 0?0:+t),n=typeof n=="function"?n:n===void 0?tR:ft(+n);function h(y){var g,b,S,_=(y=Ix(y)).length,x,E=!1,M,A=new Array(_),C=new Array(_);for(u==null&&(c=s(M=d())),g=0;g<=_;++g){if(!(g<_&&i(x=y[g],g,y))===E)if(E=!E)b=g,c.areaStart(),c.lineStart();else{for(c.lineEnd(),c.lineStart(),S=g-1;S>=b;--S)c.point(A[S],C[S]);c.lineEnd(),c.areaEnd()}E&&(A[g]=+e(x,g,y),C[g]=+t(x,g,y),c.point(r?+r(x,g,y):A[g],n?+n(x,g,y):C[g]))}if(M)return c=null,M+""||null}function v(){return nR().defined(i).curve(s).context(u)}return h.x=function(y){return arguments.length?(e=typeof y=="function"?y:ft(+y),r=null,h):e},h.x0=function(y){return arguments.length?(e=typeof y=="function"?y:ft(+y),h):e},h.x1=function(y){return arguments.length?(r=y==null?null:typeof y=="function"?y:ft(+y),h):r},h.y=function(y){return arguments.length?(t=typeof y=="function"?y:ft(+y),n=null,h):t},h.y0=function(y){return arguments.length?(t=typeof y=="function"?y:ft(+y),h):t},h.y1=function(y){return arguments.length?(n=y==null?null:typeof y=="function"?y:ft(+y),h):n},h.lineX0=h.lineY0=function(){return v().x(e).y(t)},h.lineY1=function(){return v().x(e).y(n)},h.lineX1=function(){return v().x(r).y(t)},h.defined=function(y){return arguments.length?(i=typeof y=="function"?y:ft(!!y),h):i},h.curve=function(y){return arguments.length?(s=y,u!=null&&(c=s(u)),h):s},h.context=function(y){return arguments.length?(y==null?u=c=null:c=s(u=y),h):u},h}class rR{constructor(t,n){this._context=t,this._x=n}areaStart(){this._line=0}areaEnd(){this._line=NaN}lineStart(){this._point=0}lineEnd(){(this._line||this._line!==0&&this._point===1)&&this._context.closePath(),this._line=1-this._line}point(t,n){switch(t=+t,n=+n,this._point){case 0:{this._point=1,this._line?this._context.lineTo(t,n):this._context.moveTo(t,n);break}case 1:this._point=2;default:{this._x?this._context.bezierCurveTo(this._x0=(this._x0+t)/2,this._y0,this._x0,n,t,n):this._context.bezierCurveTo(this._x0,this._y0=(this._y0+n)/2,t,this._y0,t,n);break}}this._x0=t,this._y0=n}}function KL(e){return new rR(e,!0)}function WL(e){return new rR(e,!1)}const Ux={draw(e,t){const n=nr(t/Hf);e.moveTo(n,0),e.arc(0,0,n,0,Vd)}},FL={draw(e,t){const n=nr(t/5)/2;e.moveTo(-3*n,-n),e.lineTo(-n,-n),e.lineTo(-n,-3*n),e.lineTo(n,-3*n),e.lineTo(n,-n),e.lineTo(3*n,-n),e.lineTo(3*n,n),e.lineTo(n,n),e.lineTo(n,3*n),e.lineTo(-n,3*n),e.lineTo(-n,n),e.lineTo(-3*n,n),e.closePath()}},aR=nr(1/3),QL=aR*2,JL={draw(e,t){const n=nr(t/QL),r=n*aR;e.moveTo(0,-n),e.lineTo(r,0),e.lineTo(0,n),e.lineTo(-r,0),e.closePath()}},eI={draw(e,t){const n=nr(t),r=-n/2;e.rect(r,r,n,n)}},tI=.8908130915292852,iR=Uf(Hf/10)/Uf(7*Hf/10),nI=Uf(Vd/10)*iR,rI=-FP(Vd/10)*iR,aI={draw(e,t){const n=nr(t*tI),r=nI*n,i=rI*n;e.moveTo(0,-n),e.lineTo(r,i);for(let u=1;u<5;++u){const s=Vd*u/5,c=FP(s),d=Uf(s);e.lineTo(d*n,-c*n),e.lineTo(c*r-d*i,d*r+c*i)}e.closePath()}},lm=nr(3),iI={draw(e,t){const n=-nr(t/(lm*3));e.moveTo(0,n*2),e.lineTo(-lm*n,-n),e.lineTo(lm*n,-n),e.closePath()}},qn=-.5,$n=nr(3)/2,f0=1/nr(12),oI=(f0/2+1)*3,uI={draw(e,t){const n=nr(t/oI),r=n/2,i=n*f0,u=r,s=n*f0+n,c=-u,d=s;e.moveTo(r,i),e.lineTo(u,s),e.lineTo(c,d),e.lineTo(qn*r-$n*i,$n*r+qn*i),e.lineTo(qn*u-$n*s,$n*u+qn*s),e.lineTo(qn*c-$n*d,$n*c+qn*d),e.lineTo(qn*r+$n*i,qn*i-$n*r),e.lineTo(qn*u+$n*s,qn*s-$n*u),e.lineTo(qn*c+$n*d,qn*d-$n*c),e.closePath()}};function lI(e,t){let n=null,r=Lx(i);e=typeof e=="function"?e:ft(e||Ux),t=typeof t=="function"?t:ft(t===void 0?64:+t);function i(){let u;if(n||(n=u=r()),e.apply(this,arguments).draw(n,+t.apply(this,arguments)),u)return n=null,u+""||null}return i.type=function(u){return arguments.length?(e=typeof u=="function"?u:ft(u),i):e},i.size=function(u){return arguments.length?(t=typeof u=="function"?u:ft(+u),i):t},i.context=function(u){return arguments.length?(n=u??null,i):n},i}function Gf(){}function Vf(e,t,n){e._context.bezierCurveTo((2*e._x0+e._x1)/3,(2*e._y0+e._y1)/3,(e._x0+2*e._x1)/3,(e._y0+2*e._y1)/3,(e._x0+4*e._x1+t)/6,(e._y0+4*e._y1+n)/6)}function oR(e){this._context=e}oR.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._y0=this._y1=NaN,this._point=0},lineEnd:function(){switch(this._point){case 3:Vf(this,this._x1,this._y1);case 2:this._context.lineTo(this._x1,this._y1);break}(this._line||this._line!==0&&this._point===1)&&this._context.closePath(),this._line=1-this._line},point:function(e,t){switch(e=+e,t=+t,this._point){case 0:this._point=1,this._line?this._context.lineTo(e,t):this._context.moveTo(e,t);break;case 1:this._point=2;break;case 2:this._point=3,this._context.lineTo((5*this._x0+this._x1)/6,(5*this._y0+this._y1)/6);default:Vf(this,e,t);break}this._x0=this._x1,this._x1=e,this._y0=this._y1,this._y1=t}};function sI(e){return new oR(e)}function uR(e){this._context=e}uR.prototype={areaStart:Gf,areaEnd:Gf,lineStart:function(){this._x0=this._x1=this._x2=this._x3=this._x4=this._y0=this._y1=this._y2=this._y3=this._y4=NaN,this._point=0},lineEnd:function(){switch(this._point){case 1:{this._context.moveTo(this._x2,this._y2),this._context.closePath();break}case 2:{this._context.moveTo((this._x2+2*this._x3)/3,(this._y2+2*this._y3)/3),this._context.lineTo((this._x3+2*this._x2)/3,(this._y3+2*this._y2)/3),this._context.closePath();break}case 3:{this.point(this._x2,this._y2),this.point(this._x3,this._y3),this.point(this._x4,this._y4);break}}},point:function(e,t){switch(e=+e,t=+t,this._point){case 0:this._point=1,this._x2=e,this._y2=t;break;case 1:this._point=2,this._x3=e,this._y3=t;break;case 2:this._point=3,this._x4=e,this._y4=t,this._context.moveTo((this._x0+4*this._x1+e)/6,(this._y0+4*this._y1+t)/6);break;default:Vf(this,e,t);break}this._x0=this._x1,this._x1=e,this._y0=this._y1,this._y1=t}};function cI(e){return new uR(e)}function lR(e){this._context=e}lR.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._y0=this._y1=NaN,this._point=0},lineEnd:function(){(this._line||this._line!==0&&this._point===3)&&this._context.closePath(),this._line=1-this._line},point:function(e,t){switch(e=+e,t=+t,this._point){case 0:this._point=1;break;case 1:this._point=2;break;case 2:this._point=3;var n=(this._x0+4*this._x1+e)/6,r=(this._y0+4*this._y1+t)/6;this._line?this._context.lineTo(n,r):this._context.moveTo(n,r);break;case 3:this._point=4;default:Vf(this,e,t);break}this._x0=this._x1,this._x1=e,this._y0=this._y1,this._y1=t}};function fI(e){return new lR(e)}function sR(e){this._context=e}sR.prototype={areaStart:Gf,areaEnd:Gf,lineStart:function(){this._point=0},lineEnd:function(){this._point&&this._context.closePath()},point:function(e,t){e=+e,t=+t,this._point?this._context.lineTo(e,t):(this._point=1,this._context.moveTo(e,t))}};function dI(e){return new sR(e)}function yE(e){return e<0?-1:1}function mE(e,t,n){var r=e._x1-e._x0,i=t-e._x1,u=(e._y1-e._y0)/(r||i<0&&-0),s=(n-e._y1)/(i||r<0&&-0),c=(u*i+s*r)/(r+i);return(yE(u)+yE(s))*Math.min(Math.abs(u),Math.abs(s),.5*Math.abs(c))||0}function gE(e,t){var n=e._x1-e._x0;return n?(3*(e._y1-e._y0)/n-t)/2:t}function sm(e,t,n){var r=e._x0,i=e._y0,u=e._x1,s=e._y1,c=(u-r)/3;e._context.bezierCurveTo(r+c,i+c*t,u-c,s-c*n,u,s)}function Zf(e){this._context=e}Zf.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x0=this._x1=this._y0=this._y1=this._t0=NaN,this._point=0},lineEnd:function(){switch(this._point){case 2:this._context.lineTo(this._x1,this._y1);break;case 3:sm(this,this._t0,gE(this,this._t0));break}(this._line||this._line!==0&&this._point===1)&&this._context.closePath(),this._line=1-this._line},point:function(e,t){var n=NaN;if(e=+e,t=+t,!(e===this._x1&&t===this._y1)){switch(this._point){case 0:this._point=1,this._line?this._context.lineTo(e,t):this._context.moveTo(e,t);break;case 1:this._point=2;break;case 2:this._point=3,sm(this,gE(this,n=mE(this,e,t)),n);break;default:sm(this,this._t0,n=mE(this,e,t));break}this._x0=this._x1,this._x1=e,this._y0=this._y1,this._y1=t,this._t0=n}}};function cR(e){this._context=new fR(e)}(cR.prototype=Object.create(Zf.prototype)).point=function(e,t){Zf.prototype.point.call(this,t,e)};function fR(e){this._context=e}fR.prototype={moveTo:function(e,t){this._context.moveTo(t,e)},closePath:function(){this._context.closePath()},lineTo:function(e,t){this._context.lineTo(t,e)},bezierCurveTo:function(e,t,n,r,i,u){this._context.bezierCurveTo(t,e,r,n,u,i)}};function hI(e){return new Zf(e)}function pI(e){return new cR(e)}function dR(e){this._context=e}dR.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x=[],this._y=[]},lineEnd:function(){var e=this._x,t=this._y,n=e.length;if(n)if(this._line?this._context.lineTo(e[0],t[0]):this._context.moveTo(e[0],t[0]),n===2)this._context.lineTo(e[1],t[1]);else for(var r=bE(e),i=bE(t),u=0,s=1;s<n;++u,++s)this._context.bezierCurveTo(r[0][u],i[0][u],r[1][u],i[1][u],e[s],t[s]);(this._line||this._line!==0&&n===1)&&this._context.closePath(),this._line=1-this._line,this._x=this._y=null},point:function(e,t){this._x.push(+e),this._y.push(+t)}};function bE(e){var t,n=e.length-1,r,i=new Array(n),u=new Array(n),s=new Array(n);for(i[0]=0,u[0]=2,s[0]=e[0]+2*e[1],t=1;t<n-1;++t)i[t]=1,u[t]=4,s[t]=4*e[t]+2*e[t+1];for(i[n-1]=2,u[n-1]=7,s[n-1]=8*e[n-1]+e[n],t=1;t<n;++t)r=i[t]/u[t-1],u[t]-=r,s[t]-=r*s[t-1];for(i[n-1]=s[n-1]/u[n-1],t=n-2;t>=0;--t)i[t]=(s[t]-i[t+1])/u[t];for(u[n-1]=(e[n]+i[n-1])/2,t=0;t<n-1;++t)u[t]=2*e[t+1]-i[t+1];return[i,u]}function vI(e){return new dR(e)}function Yd(e,t){this._context=e,this._t=t}Yd.prototype={areaStart:function(){this._line=0},areaEnd:function(){this._line=NaN},lineStart:function(){this._x=this._y=NaN,this._point=0},lineEnd:function(){0<this._t&&this._t<1&&this._point===2&&this._context.lineTo(this._x,this._y),(this._line||this._line!==0&&this._point===1)&&this._context.closePath(),this._line>=0&&(this._t=1-this._t,this._line=1-this._line)},point:function(e,t){switch(e=+e,t=+t,this._point){case 0:this._point=1,this._line?this._context.lineTo(e,t):this._context.moveTo(e,t);break;case 1:this._point=2;default:{if(this._t<=0)this._context.lineTo(this._x,t),this._context.lineTo(e,t);else{var n=this._x*(1-this._t)+e*this._t;this._context.lineTo(n,this._y),this._context.lineTo(n,t)}break}}this._x=e,this._y=t}};function yI(e){return new Yd(e,.5)}function mI(e){return new Yd(e,0)}function gI(e){return new Yd(e,1)}function zo(e,t){if((s=e.length)>1)for(var n=1,r,i,u=e[t[0]],s,c=u.length;n<s;++n)for(i=u,u=e[t[n]],r=0;r<c;++r)u[r][1]+=u[r][0]=isNaN(i[r][1])?i[r][0]:i[r][1]}function d0(e){for(var t=e.length,n=new Array(t);--t>=0;)n[t]=t;return n}function bI(e,t){return e[t]}function xI(e){const t=[];return t.key=e,t}function _I(){var e=ft([]),t=d0,n=zo,r=bI;function i(u){var s=Array.from(e.apply(this,arguments),xI),c,d=s.length,h=-1,v;for(const y of u)for(c=0,++h;c<d;++c)(s[c][h]=[0,+r(y,s[c].key,h,u)]).data=y;for(c=0,v=Ix(t(s));c<d;++c)s[v[c]].index=c;return n(s,v),s}return i.keys=function(u){return arguments.length?(e=typeof u=="function"?u:ft(Array.from(u)),i):e},i.value=function(u){return arguments.length?(r=typeof u=="function"?u:ft(+u),i):r},i.order=function(u){return arguments.length?(t=u==null?d0:typeof u=="function"?u:ft(Array.from(u)),i):t},i.offset=function(u){return arguments.length?(n=u??zo,i):n},i}function SI(e,t){if((r=e.length)>0){for(var n,r,i=0,u=e[0].length,s;i<u;++i){for(s=n=0;n<r;++n)s+=e[n][i][1]||0;if(s)for(n=0;n<r;++n)e[n][i][1]/=s}zo(e,t)}}function wI(e,t){if((i=e.length)>0){for(var n=0,r=e[t[0]],i,u=r.length;n<u;++n){for(var s=0,c=0;s<i;++s)c+=e[s][n][1]||0;r[n][1]+=r[n][0]=-c/2}zo(e,t)}}function OI(e,t){if(!(!((s=e.length)>0)||!((u=(i=e[t[0]]).length)>0))){for(var n=0,r=1,i,u,s;r<u;++r){for(var c=0,d=0,h=0;c<s;++c){for(var v=e[t[c]],y=v[r][1]||0,g=v[r-1][1]||0,b=(y-g)/2,S=0;S<c;++S){var _=e[t[S]],x=_[r][1]||0,E=_[r-1][1]||0;b+=x-E}d+=y,h+=b*y}i[r-1][1]+=i[r-1][0]=n,d&&(n-=h/d)}i[r-1][1]+=i[r-1][0]=n,zo(e,t)}}function Fl(e){"@babel/helpers - typeof";return Fl=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},Fl(e)}var AI=["type","size","sizeType"];function h0(){return h0=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},h0.apply(this,arguments)}function xE(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function _E(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?xE(Object(n),!0).forEach(function(r){EI(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):xE(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function EI(e,t,n){return t=TI(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function TI(e){var t=MI(e,"string");return Fl(t)=="symbol"?t:t+""}function MI(e,t){if(Fl(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(Fl(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return(t==="string"?String:Number)(e)}function jI(e,t){if(e==null)return{};var n=CI(e,t),r,i;if(Object.getOwnPropertySymbols){var u=Object.getOwnPropertySymbols(e);for(i=0;i<u.length;i++)r=u[i],!(t.indexOf(r)>=0)&&Object.prototype.propertyIsEnumerable.call(e,r)&&(n[r]=e[r])}return n}function CI(e,t){if(e==null)return{};var n={};for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){if(t.indexOf(r)>=0)continue;n[r]=e[r]}return n}var hR={symbolCircle:Ux,symbolCross:FL,symbolDiamond:JL,symbolSquare:eI,symbolStar:aI,symbolTriangle:iI,symbolWye:uI},PI=Math.PI/180,RI=function(t){var n="symbol".concat(Gd(t));return hR[n]||Ux},NI=function(t,n,r){if(n==="area")return t;switch(r){case"cross":return 5*t*t/9;case"diamond":return .5*t*t/Math.sqrt(3);case"square":return t*t;case"star":{var i=18*PI;return 1.25*t*t*(Math.tan(i)-Math.tan(i*2)*Math.pow(Math.tan(i),2))}case"triangle":return Math.sqrt(3)*t*t/4;case"wye":return(21-10*Math.sqrt(3))*t*t/8;default:return Math.PI*t*t/4}},DI=function(t,n){hR["symbol".concat(Gd(t))]=n},Xd=function(t){var n=t.type,r=n===void 0?"circle":n,i=t.size,u=i===void 0?64:i,s=t.sizeType,c=s===void 0?"area":s,d=jI(t,AI),h=_E(_E({},d),{},{type:r,size:u,sizeType:c}),v=function(){var x=RI(r),E=lI().type(x).size(NI(u,c,r));return E()},y=h.className,g=h.cx,b=h.cy,S=Ye(h,!0);return g===+g&&b===+b&&u===+u?L.createElement("path",h0({},S,{className:Xe("recharts-symbols",y),transform:"translate(".concat(g,", ").concat(b,")"),d:v()})):null};Xd.registerSymbol=DI;function Lo(e){"@babel/helpers - typeof";return Lo=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},Lo(e)}function p0(){return p0=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},p0.apply(this,arguments)}function SE(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function kI(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?SE(Object(n),!0).forEach(function(r){Ql(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):SE(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function qI(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function $I(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,vR(r.key),r)}}function BI(e,t,n){return t&&$I(e.prototype,t),Object.defineProperty(e,"prototype",{writable:!1}),e}function zI(e,t,n){return t=Yf(t),LI(e,pR()?Reflect.construct(t,n||[],Yf(e).constructor):t.apply(e,n))}function LI(e,t){if(t&&(Lo(t)==="object"||typeof t=="function"))return t;if(t!==void 0)throw new TypeError("Derived constructors may only return object or undefined");return II(e)}function II(e){if(e===void 0)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function pR(){try{var e=!Boolean.prototype.valueOf.call(Reflect.construct(Boolean,[],function(){}))}catch{}return(pR=function(){return!!e})()}function Yf(e){return Yf=Object.setPrototypeOf?Object.getPrototypeOf.bind():function(n){return n.__proto__||Object.getPrototypeOf(n)},Yf(e)}function UI(e,t){if(typeof t!="function"&&t!==null)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),Object.defineProperty(e,"prototype",{writable:!1}),t&&v0(e,t)}function v0(e,t){return v0=Object.setPrototypeOf?Object.setPrototypeOf.bind():function(r,i){return r.__proto__=i,r},v0(e,t)}function Ql(e,t,n){return t=vR(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function vR(e){var t=HI(e,"string");return Lo(t)=="symbol"?t:t+""}function HI(e,t){if(Lo(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(Lo(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return String(e)}var Zn=32,Hx=function(e){function t(){return qI(this,t),zI(this,t,arguments)}return UI(t,e),BI(t,[{key:"renderIcon",value:function(r){var i=this.props.inactiveColor,u=Zn/2,s=Zn/6,c=Zn/3,d=r.inactive?i:r.color;if(r.type==="plainline")return L.createElement("line",{strokeWidth:4,fill:"none",stroke:d,strokeDasharray:r.payload.strokeDasharray,x1:0,y1:u,x2:Zn,y2:u,className:"recharts-legend-icon"});if(r.type==="line")return L.createElement("path",{strokeWidth:4,fill:"none",stroke:d,d:"M0,".concat(u,"h").concat(c,`
            A`).concat(s,",").concat(s,",0,1,1,").concat(2*c,",").concat(u,`
            H`).concat(Zn,"M").concat(2*c,",").concat(u,`
            A`).concat(s,",").concat(s,",0,1,1,").concat(c,",").concat(u),className:"recharts-legend-icon"});if(r.type==="rect")return L.createElement("path",{stroke:"none",fill:d,d:"M0,".concat(Zn/8,"h").concat(Zn,"v").concat(Zn*3/4,"h").concat(-32,"z"),className:"recharts-legend-icon"});if(L.isValidElement(r.legendIcon)){var h=kI({},r);return delete h.legendIcon,L.cloneElement(r.legendIcon,h)}return L.createElement(Xd,{fill:d,cx:u,cy:u,size:Zn,sizeType:"diameter",type:r.type})}},{key:"renderItems",value:function(){var r=this,i=this.props,u=i.payload,s=i.iconSize,c=i.layout,d=i.formatter,h=i.inactiveColor,v={x:0,y:0,width:Zn,height:Zn},y={display:c==="horizontal"?"inline-block":"block",marginRight:10},g={display:"inline-block",verticalAlign:"middle",marginRight:4};return u.map(function(b,S){var _=b.formatter||d,x=Xe(Ql(Ql({"recharts-legend-item":!0},"legend-item-".concat(S),!0),"inactive",b.inactive));if(b.type==="none")return null;var E=Ge(b.value)?null:b.value;vi(!Ge(b.value),`The name property is also required when using a function for the dataKey of a chart's cartesian components. Ex: <Bar name="Name of my Data"/>`);var M=b.inactive?h:b.color;return L.createElement("li",p0({className:x,style:y,key:"legend-item-".concat(S)},Wl(r.props,b,S)),L.createElement(u0,{width:s,height:s,viewBox:v,style:g},r.renderIcon(b)),L.createElement("span",{className:"recharts-legend-item-text",style:{color:M}},_?_(E,b,S):E))})}},{key:"render",value:function(){var r=this.props,i=r.payload,u=r.layout,s=r.align;if(!i||!i.length)return null;var c={padding:0,margin:0,textAlign:u==="horizontal"?s:"left"};return L.createElement("ul",{className:"recharts-default-legend",style:c},this.renderItems())}}])}(q.PureComponent);Ql(Hx,"displayName","Legend");Ql(Hx,"defaultProps",{iconSize:14,layout:"horizontal",align:"center",verticalAlign:"middle",inactiveColor:"#ccc"});var cm,wE;function GI(){if(wE)return cm;wE=1;var e=Id();function t(){this.__data__=new e,this.size=0}return cm=t,cm}var fm,OE;function VI(){if(OE)return fm;OE=1;function e(t){var n=this.__data__,r=n.delete(t);return this.size=n.size,r}return fm=e,fm}var dm,AE;function ZI(){if(AE)return dm;AE=1;function e(t){return this.__data__.get(t)}return dm=e,dm}var hm,EE;function YI(){if(EE)return hm;EE=1;function e(t){return this.__data__.has(t)}return hm=e,hm}var pm,TE;function XI(){if(TE)return pm;TE=1;var e=Id(),t=Dx(),n=kx(),r=200;function i(u,s){var c=this.__data__;if(c instanceof e){var d=c.__data__;if(!t||d.length<r-1)return d.push([u,s]),this.size=++c.size,this;c=this.__data__=new n(d)}return c.set(u,s),this.size=c.size,this}return pm=i,pm}var vm,ME;function yR(){if(ME)return vm;ME=1;var e=Id(),t=GI(),n=VI(),r=ZI(),i=YI(),u=XI();function s(c){var d=this.__data__=new e(c);this.size=d.size}return s.prototype.clear=t,s.prototype.delete=n,s.prototype.get=r,s.prototype.has=i,s.prototype.set=u,vm=s,vm}var ym,jE;function KI(){if(jE)return ym;jE=1;var e="__lodash_hash_undefined__";function t(n){return this.__data__.set(n,e),this}return ym=t,ym}var mm,CE;function WI(){if(CE)return mm;CE=1;function e(t){return this.__data__.has(t)}return mm=e,mm}var gm,PE;function mR(){if(PE)return gm;PE=1;var e=kx(),t=KI(),n=WI();function r(i){var u=-1,s=i==null?0:i.length;for(this.__data__=new e;++u<s;)this.add(i[u])}return r.prototype.add=r.prototype.push=t,r.prototype.has=n,gm=r,gm}var bm,RE;function gR(){if(RE)return bm;RE=1;function e(t,n){for(var r=-1,i=t==null?0:t.length;++r<i;)if(n(t[r],r,t))return!0;return!1}return bm=e,bm}var xm,NE;function bR(){if(NE)return xm;NE=1;function e(t,n){return t.has(n)}return xm=e,xm}var _m,DE;function xR(){if(DE)return _m;DE=1;var e=mR(),t=gR(),n=bR(),r=1,i=2;function u(s,c,d,h,v,y){var g=d&r,b=s.length,S=c.length;if(b!=S&&!(g&&S>b))return!1;var _=y.get(s),x=y.get(c);if(_&&x)return _==c&&x==s;var E=-1,M=!0,A=d&i?new e:void 0;for(y.set(s,c),y.set(c,s);++E<b;){var C=s[E],w=c[E];if(h)var T=g?h(w,C,E,c,s,y):h(C,w,E,s,c,y);if(T!==void 0){if(T)continue;M=!1;break}if(A){if(!t(c,function(P,R){if(!n(A,R)&&(C===P||v(C,P,d,h,y)))return A.push(R)})){M=!1;break}}else if(!(C===w||v(C,w,d,h,y))){M=!1;break}}return y.delete(s),y.delete(c),M}return _m=u,_m}var Sm,kE;function FI(){if(kE)return Sm;kE=1;var e=br(),t=e.Uint8Array;return Sm=t,Sm}var wm,qE;function QI(){if(qE)return wm;qE=1;function e(t){var n=-1,r=Array(t.size);return t.forEach(function(i,u){r[++n]=[u,i]}),r}return wm=e,wm}var Om,$E;function Gx(){if($E)return Om;$E=1;function e(t){var n=-1,r=Array(t.size);return t.forEach(function(i){r[++n]=i}),r}return Om=e,Om}var Am,BE;function JI(){if(BE)return Am;BE=1;var e=Ns(),t=FI(),n=Nx(),r=xR(),i=QI(),u=Gx(),s=1,c=2,d="[object Boolean]",h="[object Date]",v="[object Error]",y="[object Map]",g="[object Number]",b="[object RegExp]",S="[object Set]",_="[object String]",x="[object Symbol]",E="[object ArrayBuffer]",M="[object DataView]",A=e?e.prototype:void 0,C=A?A.valueOf:void 0;function w(T,P,R,$,J,B,G){switch(R){case M:if(T.byteLength!=P.byteLength||T.byteOffset!=P.byteOffset)return!1;T=T.buffer,P=P.buffer;case E:return!(T.byteLength!=P.byteLength||!B(new t(T),new t(P)));case d:case h:case g:return n(+T,+P);case v:return T.name==P.name&&T.message==P.message;case b:case _:return T==P+"";case y:var W=i;case S:var ne=$&s;if(W||(W=u),T.size!=P.size&&!ne)return!1;var Q=G.get(T);if(Q)return Q==P;$|=c,G.set(T,P);var D=r(W(T),W(P),$,J,B,G);return G.delete(T),D;case x:if(C)return C.call(T)==C.call(P)}return!1}return Am=w,Am}var Em,zE;function _R(){if(zE)return Em;zE=1;function e(t,n){for(var r=-1,i=n.length,u=t.length;++r<i;)t[u+r]=n[r];return t}return Em=e,Em}var Tm,LE;function e6(){if(LE)return Tm;LE=1;var e=_R(),t=fn();function n(r,i,u){var s=i(r);return t(r)?s:e(s,u(r))}return Tm=n,Tm}var Mm,IE;function t6(){if(IE)return Mm;IE=1;function e(t,n){for(var r=-1,i=t==null?0:t.length,u=0,s=[];++r<i;){var c=t[r];n(c,r,t)&&(s[u++]=c)}return s}return Mm=e,Mm}var jm,UE;function n6(){if(UE)return jm;UE=1;function e(){return[]}return jm=e,jm}var Cm,HE;function r6(){if(HE)return Cm;HE=1;var e=t6(),t=n6(),n=Object.prototype,r=n.propertyIsEnumerable,i=Object.getOwnPropertySymbols,u=i?function(s){return s==null?[]:(s=Object(s),e(i(s),function(c){return r.call(s,c)}))}:t;return Cm=u,Cm}var Pm,GE;function a6(){if(GE)return Pm;GE=1;function e(t,n){for(var r=-1,i=Array(t);++r<t;)i[r]=n(r);return i}return Pm=e,Pm}var Rm,VE;function i6(){if(VE)return Rm;VE=1;var e=Fr(),t=Qr(),n="[object Arguments]";function r(i){return t(i)&&e(i)==n}return Rm=r,Rm}var Nm,ZE;function Vx(){if(ZE)return Nm;ZE=1;var e=i6(),t=Qr(),n=Object.prototype,r=n.hasOwnProperty,i=n.propertyIsEnumerable,u=e(function(){return arguments}())?e:function(s){return t(s)&&r.call(s,"callee")&&!i.call(s,"callee")};return Nm=u,Nm}var Ml={exports:{}},Dm,YE;function o6(){if(YE)return Dm;YE=1;function e(){return!1}return Dm=e,Dm}Ml.exports;var XE;function SR(){return XE||(XE=1,function(e,t){var n=br(),r=o6(),i=t&&!t.nodeType&&t,u=i&&!0&&e&&!e.nodeType&&e,s=u&&u.exports===i,c=s?n.Buffer:void 0,d=c?c.isBuffer:void 0,h=d||r;e.exports=h}(Ml,Ml.exports)),Ml.exports}var km,KE;function Zx(){if(KE)return km;KE=1;var e=9007199254740991,t=/^(?:0|[1-9]\d*)$/;function n(r,i){var u=typeof r;return i=i??e,!!i&&(u=="number"||u!="symbol"&&t.test(r))&&r>-1&&r%1==0&&r<i}return km=n,km}var qm,WE;function Yx(){if(WE)return qm;WE=1;var e=9007199254740991;function t(n){return typeof n=="number"&&n>-1&&n%1==0&&n<=e}return qm=t,qm}var $m,FE;function u6(){if(FE)return $m;FE=1;var e=Fr(),t=Yx(),n=Qr(),r="[object Arguments]",i="[object Array]",u="[object Boolean]",s="[object Date]",c="[object Error]",d="[object Function]",h="[object Map]",v="[object Number]",y="[object Object]",g="[object RegExp]",b="[object Set]",S="[object String]",_="[object WeakMap]",x="[object ArrayBuffer]",E="[object DataView]",M="[object Float32Array]",A="[object Float64Array]",C="[object Int8Array]",w="[object Int16Array]",T="[object Int32Array]",P="[object Uint8Array]",R="[object Uint8ClampedArray]",$="[object Uint16Array]",J="[object Uint32Array]",B={};B[M]=B[A]=B[C]=B[w]=B[T]=B[P]=B[R]=B[$]=B[J]=!0,B[r]=B[i]=B[x]=B[u]=B[E]=B[s]=B[c]=B[d]=B[h]=B[v]=B[y]=B[g]=B[b]=B[S]=B[_]=!1;function G(W){return n(W)&&t(W.length)&&!!B[e(W)]}return $m=G,$m}var Bm,QE;function wR(){if(QE)return Bm;QE=1;function e(t){return function(n){return t(n)}}return Bm=e,Bm}var jl={exports:{}};jl.exports;var JE;function l6(){return JE||(JE=1,function(e,t){var n=HP(),r=t&&!t.nodeType&&t,i=r&&!0&&e&&!e.nodeType&&e,u=i&&i.exports===r,s=u&&n.process,c=function(){try{var d=i&&i.require&&i.require("util").types;return d||s&&s.binding&&s.binding("util")}catch{}}();e.exports=c}(jl,jl.exports)),jl.exports}var zm,eT;function OR(){if(eT)return zm;eT=1;var e=u6(),t=wR(),n=l6(),r=n&&n.isTypedArray,i=r?t(r):e;return zm=i,zm}var Lm,tT;function s6(){if(tT)return Lm;tT=1;var e=a6(),t=Vx(),n=fn(),r=SR(),i=Zx(),u=OR(),s=Object.prototype,c=s.hasOwnProperty;function d(h,v){var y=n(h),g=!y&&t(h),b=!y&&!g&&r(h),S=!y&&!g&&!b&&u(h),_=y||g||b||S,x=_?e(h.length,String):[],E=x.length;for(var M in h)(v||c.call(h,M))&&!(_&&(M=="length"||b&&(M=="offset"||M=="parent")||S&&(M=="buffer"||M=="byteLength"||M=="byteOffset")||i(M,E)))&&x.push(M);return x}return Lm=d,Lm}var Im,nT;function c6(){if(nT)return Im;nT=1;var e=Object.prototype;function t(n){var r=n&&n.constructor,i=typeof r=="function"&&r.prototype||e;return n===i}return Im=t,Im}var Um,rT;function AR(){if(rT)return Um;rT=1;function e(t,n){return function(r){return t(n(r))}}return Um=e,Um}var Hm,aT;function f6(){if(aT)return Hm;aT=1;var e=AR(),t=e(Object.keys,Object);return Hm=t,Hm}var Gm,iT;function d6(){if(iT)return Gm;iT=1;var e=c6(),t=f6(),n=Object.prototype,r=n.hasOwnProperty;function i(u){if(!e(u))return t(u);var s=[];for(var c in Object(u))r.call(u,c)&&c!="constructor"&&s.push(c);return s}return Gm=i,Gm}var Vm,oT;function $s(){if(oT)return Vm;oT=1;var e=Rx(),t=Yx();function n(r){return r!=null&&t(r.length)&&!e(r)}return Vm=n,Vm}var Zm,uT;function Kd(){if(uT)return Zm;uT=1;var e=s6(),t=d6(),n=$s();function r(i){return n(i)?e(i):t(i)}return Zm=r,Zm}var Ym,lT;function h6(){if(lT)return Ym;lT=1;var e=e6(),t=r6(),n=Kd();function r(i){return e(i,n,t)}return Ym=r,Ym}var Xm,sT;function p6(){if(sT)return Xm;sT=1;var e=h6(),t=1,n=Object.prototype,r=n.hasOwnProperty;function i(u,s,c,d,h,v){var y=c&t,g=e(u),b=g.length,S=e(s),_=S.length;if(b!=_&&!y)return!1;for(var x=b;x--;){var E=g[x];if(!(y?E in s:r.call(s,E)))return!1}var M=v.get(u),A=v.get(s);if(M&&A)return M==s&&A==u;var C=!0;v.set(u,s),v.set(s,u);for(var w=y;++x<b;){E=g[x];var T=u[E],P=s[E];if(d)var R=y?d(P,T,E,s,u,v):d(T,P,E,u,s,v);if(!(R===void 0?T===P||h(T,P,c,d,v):R)){C=!1;break}w||(w=E=="constructor")}if(C&&!w){var $=u.constructor,J=s.constructor;$!=J&&"constructor"in u&&"constructor"in s&&!(typeof $=="function"&&$ instanceof $&&typeof J=="function"&&J instanceof J)&&(C=!1)}return v.delete(u),v.delete(s),C}return Xm=i,Xm}var Km,cT;function v6(){if(cT)return Km;cT=1;var e=Ti(),t=br(),n=e(t,"DataView");return Km=n,Km}var Wm,fT;function y6(){if(fT)return Wm;fT=1;var e=Ti(),t=br(),n=e(t,"Promise");return Wm=n,Wm}var Fm,dT;function ER(){if(dT)return Fm;dT=1;var e=Ti(),t=br(),n=e(t,"Set");return Fm=n,Fm}var Qm,hT;function m6(){if(hT)return Qm;hT=1;var e=Ti(),t=br(),n=e(t,"WeakMap");return Qm=n,Qm}var Jm,pT;function g6(){if(pT)return Jm;pT=1;var e=v6(),t=Dx(),n=y6(),r=ER(),i=m6(),u=Fr(),s=GP(),c="[object Map]",d="[object Object]",h="[object Promise]",v="[object Set]",y="[object WeakMap]",g="[object DataView]",b=s(e),S=s(t),_=s(n),x=s(r),E=s(i),M=u;return(e&&M(new e(new ArrayBuffer(1)))!=g||t&&M(new t)!=c||n&&M(n.resolve())!=h||r&&M(new r)!=v||i&&M(new i)!=y)&&(M=function(A){var C=u(A),w=C==d?A.constructor:void 0,T=w?s(w):"";if(T)switch(T){case b:return g;case S:return c;case _:return h;case x:return v;case E:return y}return C}),Jm=M,Jm}var eg,vT;function b6(){if(vT)return eg;vT=1;var e=yR(),t=xR(),n=JI(),r=p6(),i=g6(),u=fn(),s=SR(),c=OR(),d=1,h="[object Arguments]",v="[object Array]",y="[object Object]",g=Object.prototype,b=g.hasOwnProperty;function S(_,x,E,M,A,C){var w=u(_),T=u(x),P=w?v:i(_),R=T?v:i(x);P=P==h?y:P,R=R==h?y:R;var $=P==y,J=R==y,B=P==R;if(B&&s(_)){if(!s(x))return!1;w=!0,$=!1}if(B&&!$)return C||(C=new e),w||c(_)?t(_,x,E,M,A,C):n(_,x,P,E,M,A,C);if(!(E&d)){var G=$&&b.call(_,"__wrapped__"),W=J&&b.call(x,"__wrapped__");if(G||W){var ne=G?_.value():_,Q=W?x.value():x;return C||(C=new e),A(ne,Q,E,M,C)}}return B?(C||(C=new e),r(_,x,E,M,A,C)):!1}return eg=S,eg}var tg,yT;function Xx(){if(yT)return tg;yT=1;var e=b6(),t=Qr();function n(r,i,u,s,c){return r===i?!0:r==null||i==null||!t(r)&&!t(i)?r!==r&&i!==i:e(r,i,u,s,n,c)}return tg=n,tg}var ng,mT;function x6(){if(mT)return ng;mT=1;var e=yR(),t=Xx(),n=1,r=2;function i(u,s,c,d){var h=c.length,v=h,y=!d;if(u==null)return!v;for(u=Object(u);h--;){var g=c[h];if(y&&g[2]?g[1]!==u[g[0]]:!(g[0]in u))return!1}for(;++h<v;){g=c[h];var b=g[0],S=u[b],_=g[1];if(y&&g[2]){if(S===void 0&&!(b in u))return!1}else{var x=new e;if(d)var E=d(S,_,b,u,s,x);if(!(E===void 0?t(_,S,n|r,d,x):E))return!1}}return!0}return ng=i,ng}var rg,gT;function TR(){if(gT)return rg;gT=1;var e=qa();function t(n){return n===n&&!e(n)}return rg=t,rg}var ag,bT;function _6(){if(bT)return ag;bT=1;var e=TR(),t=Kd();function n(r){for(var i=t(r),u=i.length;u--;){var s=i[u],c=r[s];i[u]=[s,c,e(c)]}return i}return ag=n,ag}var ig,xT;function MR(){if(xT)return ig;xT=1;function e(t,n){return function(r){return r==null?!1:r[t]===n&&(n!==void 0||t in Object(r))}}return ig=e,ig}var og,_T;function S6(){if(_T)return og;_T=1;var e=x6(),t=_6(),n=MR();function r(i){var u=t(i);return u.length==1&&u[0][2]?n(u[0][0],u[0][1]):function(s){return s===i||e(s,i,u)}}return og=r,og}var ug,ST;function w6(){if(ST)return ug;ST=1;function e(t,n){return t!=null&&n in Object(t)}return ug=e,ug}var lg,wT;function O6(){if(wT)return lg;wT=1;var e=YP(),t=Vx(),n=fn(),r=Zx(),i=Yx(),u=Hd();function s(c,d,h){d=e(d,c);for(var v=-1,y=d.length,g=!1;++v<y;){var b=u(d[v]);if(!(g=c!=null&&h(c,b)))break;c=c[b]}return g||++v!=y?g:(y=c==null?0:c.length,!!y&&i(y)&&r(b,y)&&(n(c)||t(c)))}return lg=s,lg}var sg,OT;function A6(){if(OT)return sg;OT=1;var e=w6(),t=O6();function n(r,i){return r!=null&&t(r,i,e)}return sg=n,sg}var cg,AT;function E6(){if(AT)return cg;AT=1;var e=Xx(),t=XP(),n=A6(),r=Px(),i=TR(),u=MR(),s=Hd(),c=1,d=2;function h(v,y){return r(v)&&i(y)?u(s(v),y):function(g){var b=t(g,v);return b===void 0&&b===y?n(g,v):e(y,b,c|d)}}return cg=h,cg}var fg,ET;function du(){if(ET)return fg;ET=1;function e(t){return t}return fg=e,fg}var dg,TT;function T6(){if(TT)return dg;TT=1;function e(t){return function(n){return n==null?void 0:n[t]}}return dg=e,dg}var hg,MT;function M6(){if(MT)return hg;MT=1;var e=$x();function t(n){return function(r){return e(r,n)}}return hg=t,hg}var pg,jT;function j6(){if(jT)return pg;jT=1;var e=T6(),t=M6(),n=Px(),r=Hd();function i(u){return n(u)?e(r(u)):t(u)}return pg=i,pg}var vg,CT;function $a(){if(CT)return vg;CT=1;var e=S6(),t=E6(),n=du(),r=fn(),i=j6();function u(s){return typeof s=="function"?s:s==null?n:typeof s=="object"?r(s)?t(s[0],s[1]):e(s):i(s)}return vg=u,vg}var yg,PT;function jR(){if(PT)return yg;PT=1;function e(t,n,r,i){for(var u=t.length,s=r+(i?1:-1);i?s--:++s<u;)if(n(t[s],s,t))return s;return-1}return yg=e,yg}var mg,RT;function C6(){if(RT)return mg;RT=1;function e(t){return t!==t}return mg=e,mg}var gg,NT;function P6(){if(NT)return gg;NT=1;function e(t,n,r){for(var i=r-1,u=t.length;++i<u;)if(t[i]===n)return i;return-1}return gg=e,gg}var bg,DT;function R6(){if(DT)return bg;DT=1;var e=jR(),t=C6(),n=P6();function r(i,u,s){return u===u?n(i,u,s):e(i,t,s)}return bg=r,bg}var xg,kT;function N6(){if(kT)return xg;kT=1;var e=R6();function t(n,r){var i=n==null?0:n.length;return!!i&&e(n,r,0)>-1}return xg=t,xg}var _g,qT;function D6(){if(qT)return _g;qT=1;function e(t,n,r){for(var i=-1,u=t==null?0:t.length;++i<u;)if(r(n,t[i]))return!0;return!1}return _g=e,_g}var Sg,$T;function k6(){if($T)return Sg;$T=1;function e(){}return Sg=e,Sg}var wg,BT;function q6(){if(BT)return wg;BT=1;var e=ER(),t=k6(),n=Gx(),r=1/0,i=e&&1/n(new e([,-0]))[1]==r?function(u){return new e(u)}:t;return wg=i,wg}var Og,zT;function $6(){if(zT)return Og;zT=1;var e=mR(),t=N6(),n=D6(),r=bR(),i=q6(),u=Gx(),s=200;function c(d,h,v){var y=-1,g=t,b=d.length,S=!0,_=[],x=_;if(v)S=!1,g=n;else if(b>=s){var E=h?null:i(d);if(E)return u(E);S=!1,g=r,x=new e}else x=h?[]:_;e:for(;++y<b;){var M=d[y],A=h?h(M):M;if(M=v||M!==0?M:0,S&&A===A){for(var C=x.length;C--;)if(x[C]===A)continue e;h&&x.push(A),_.push(M)}else g(x,A,v)||(x!==_&&x.push(A),_.push(M))}return _}return Og=c,Og}var Ag,LT;function B6(){if(LT)return Ag;LT=1;var e=$a(),t=$6();function n(r,i){return r&&r.length?t(r,e(i,2)):[]}return Ag=n,Ag}var z6=B6();const IT=lt(z6);function CR(e,t,n){return t===!0?IT(e,n):Ge(t)?IT(e,t):e}function Io(e){"@babel/helpers - typeof";return Io=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},Io(e)}var L6=["ref"];function UT(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function Br(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?UT(Object(n),!0).forEach(function(r){Wd(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):UT(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function I6(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function HT(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,RR(r.key),r)}}function U6(e,t,n){return t&&HT(e.prototype,t),n&&HT(e,n),Object.defineProperty(e,"prototype",{writable:!1}),e}function H6(e,t,n){return t=Xf(t),G6(e,PR()?Reflect.construct(t,n||[],Xf(e).constructor):t.apply(e,n))}function G6(e,t){if(t&&(Io(t)==="object"||typeof t=="function"))return t;if(t!==void 0)throw new TypeError("Derived constructors may only return object or undefined");return V6(e)}function V6(e){if(e===void 0)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function PR(){try{var e=!Boolean.prototype.valueOf.call(Reflect.construct(Boolean,[],function(){}))}catch{}return(PR=function(){return!!e})()}function Xf(e){return Xf=Object.setPrototypeOf?Object.getPrototypeOf.bind():function(n){return n.__proto__||Object.getPrototypeOf(n)},Xf(e)}function Z6(e,t){if(typeof t!="function"&&t!==null)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),Object.defineProperty(e,"prototype",{writable:!1}),t&&y0(e,t)}function y0(e,t){return y0=Object.setPrototypeOf?Object.setPrototypeOf.bind():function(r,i){return r.__proto__=i,r},y0(e,t)}function Wd(e,t,n){return t=RR(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function RR(e){var t=Y6(e,"string");return Io(t)=="symbol"?t:t+""}function Y6(e,t){if(Io(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(Io(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return String(e)}function X6(e,t){if(e==null)return{};var n=K6(e,t),r,i;if(Object.getOwnPropertySymbols){var u=Object.getOwnPropertySymbols(e);for(i=0;i<u.length;i++)r=u[i],!(t.indexOf(r)>=0)&&Object.prototype.propertyIsEnumerable.call(e,r)&&(n[r]=e[r])}return n}function K6(e,t){if(e==null)return{};var n={};for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){if(t.indexOf(r)>=0)continue;n[r]=e[r]}return n}function W6(e){return e.value}function F6(e,t){if(L.isValidElement(e))return L.cloneElement(e,t);if(typeof e=="function")return L.createElement(e,t);t.ref;var n=X6(t,L6);return L.createElement(Hx,n)}var GT=1,No=function(e){function t(){var n;I6(this,t);for(var r=arguments.length,i=new Array(r),u=0;u<r;u++)i[u]=arguments[u];return n=H6(this,t,[].concat(i)),Wd(n,"lastBoundingBox",{width:-1,height:-1}),n}return Z6(t,e),U6(t,[{key:"componentDidMount",value:function(){this.updateBBox()}},{key:"componentDidUpdate",value:function(){this.updateBBox()}},{key:"getBBox",value:function(){if(this.wrapperNode&&this.wrapperNode.getBoundingClientRect){var r=this.wrapperNode.getBoundingClientRect();return r.height=this.wrapperNode.offsetHeight,r.width=this.wrapperNode.offsetWidth,r}return null}},{key:"updateBBox",value:function(){var r=this.props.onBBoxUpdate,i=this.getBBox();i?(Math.abs(i.width-this.lastBoundingBox.width)>GT||Math.abs(i.height-this.lastBoundingBox.height)>GT)&&(this.lastBoundingBox.width=i.width,this.lastBoundingBox.height=i.height,r&&r(i)):(this.lastBoundingBox.width!==-1||this.lastBoundingBox.height!==-1)&&(this.lastBoundingBox.width=-1,this.lastBoundingBox.height=-1,r&&r(null))}},{key:"getBBoxSnapshot",value:function(){return this.lastBoundingBox.width>=0&&this.lastBoundingBox.height>=0?Br({},this.lastBoundingBox):{width:0,height:0}}},{key:"getDefaultPosition",value:function(r){var i=this.props,u=i.layout,s=i.align,c=i.verticalAlign,d=i.margin,h=i.chartWidth,v=i.chartHeight,y,g;if(!r||(r.left===void 0||r.left===null)&&(r.right===void 0||r.right===null))if(s==="center"&&u==="vertical"){var b=this.getBBoxSnapshot();y={left:((h||0)-b.width)/2}}else y=s==="right"?{right:d&&d.right||0}:{left:d&&d.left||0};if(!r||(r.top===void 0||r.top===null)&&(r.bottom===void 0||r.bottom===null))if(c==="middle"){var S=this.getBBoxSnapshot();g={top:((v||0)-S.height)/2}}else g=c==="bottom"?{bottom:d&&d.bottom||0}:{top:d&&d.top||0};return Br(Br({},y),g)}},{key:"render",value:function(){var r=this,i=this.props,u=i.content,s=i.width,c=i.height,d=i.wrapperStyle,h=i.payloadUniqBy,v=i.payload,y=Br(Br({position:"absolute",width:s||"auto",height:c||"auto"},this.getDefaultPosition(d)),d);return L.createElement("div",{className:"recharts-legend-wrapper",style:y,ref:function(b){r.wrapperNode=b}},F6(u,Br(Br({},this.props),{},{payload:CR(v,h,W6)})))}}],[{key:"getWithHeight",value:function(r,i){var u=Br(Br({},this.defaultProps),r.props),s=u.layout;return s==="vertical"&&he(r.props.height)?{height:r.props.height}:s==="horizontal"?{width:r.props.width||i}:null}}])}(q.PureComponent);Wd(No,"displayName","Legend");Wd(No,"defaultProps",{iconSize:14,layout:"horizontal",align:"center",verticalAlign:"bottom"});var Eg,VT;function Q6(){if(VT)return Eg;VT=1;var e=Ns(),t=Vx(),n=fn(),r=e?e.isConcatSpreadable:void 0;function i(u){return n(u)||t(u)||!!(r&&u&&u[r])}return Eg=i,Eg}var Tg,ZT;function NR(){if(ZT)return Tg;ZT=1;var e=_R(),t=Q6();function n(r,i,u,s,c){var d=-1,h=r.length;for(u||(u=t),c||(c=[]);++d<h;){var v=r[d];i>0&&u(v)?i>1?n(v,i-1,u,s,c):e(c,v):s||(c[c.length]=v)}return c}return Tg=n,Tg}var Mg,YT;function J6(){if(YT)return Mg;YT=1;function e(t){return function(n,r,i){for(var u=-1,s=Object(n),c=i(n),d=c.length;d--;){var h=c[t?d:++u];if(r(s[h],h,s)===!1)break}return n}}return Mg=e,Mg}var jg,XT;function eU(){if(XT)return jg;XT=1;var e=J6(),t=e();return jg=t,jg}var Cg,KT;function DR(){if(KT)return Cg;KT=1;var e=eU(),t=Kd();function n(r,i){return r&&e(r,i,t)}return Cg=n,Cg}var Pg,WT;function tU(){if(WT)return Pg;WT=1;var e=$s();function t(n,r){return function(i,u){if(i==null)return i;if(!e(i))return n(i,u);for(var s=i.length,c=r?s:-1,d=Object(i);(r?c--:++c<s)&&u(d[c],c,d)!==!1;);return i}}return Pg=t,Pg}var Rg,FT;function Kx(){if(FT)return Rg;FT=1;var e=DR(),t=tU(),n=t(e);return Rg=n,Rg}var Ng,QT;function kR(){if(QT)return Ng;QT=1;var e=Kx(),t=$s();function n(r,i){var u=-1,s=t(r)?Array(r.length):[];return e(r,function(c,d,h){s[++u]=i(c,d,h)}),s}return Ng=n,Ng}var Dg,JT;function nU(){if(JT)return Dg;JT=1;function e(t,n){var r=t.length;for(t.sort(n);r--;)t[r]=t[r].value;return t}return Dg=e,Dg}var kg,eM;function rU(){if(eM)return kg;eM=1;var e=cu();function t(n,r){if(n!==r){var i=n!==void 0,u=n===null,s=n===n,c=e(n),d=r!==void 0,h=r===null,v=r===r,y=e(r);if(!h&&!y&&!c&&n>r||c&&d&&v&&!h&&!y||u&&d&&v||!i&&v||!s)return 1;if(!u&&!c&&!y&&n<r||y&&i&&s&&!u&&!c||h&&i&&s||!d&&s||!v)return-1}return 0}return kg=t,kg}var qg,tM;function aU(){if(tM)return qg;tM=1;var e=rU();function t(n,r,i){for(var u=-1,s=n.criteria,c=r.criteria,d=s.length,h=i.length;++u<d;){var v=e(s[u],c[u]);if(v){if(u>=h)return v;var y=i[u];return v*(y=="desc"?-1:1)}}return n.index-r.index}return qg=t,qg}var $g,nM;function iU(){if(nM)return $g;nM=1;var e=qx(),t=$x(),n=$a(),r=kR(),i=nU(),u=wR(),s=aU(),c=du(),d=fn();function h(v,y,g){y.length?y=e(y,function(_){return d(_)?function(x){return t(x,_.length===1?_[0]:_)}:_}):y=[c];var b=-1;y=e(y,u(n));var S=r(v,function(_,x,E){var M=e(y,function(A){return A(_)});return{criteria:M,index:++b,value:_}});return i(S,function(_,x){return s(_,x,g)})}return $g=h,$g}var Bg,rM;function oU(){if(rM)return Bg;rM=1;function e(t,n,r){switch(r.length){case 0:return t.call(n);case 1:return t.call(n,r[0]);case 2:return t.call(n,r[0],r[1]);case 3:return t.call(n,r[0],r[1],r[2])}return t.apply(n,r)}return Bg=e,Bg}var zg,aM;function uU(){if(aM)return zg;aM=1;var e=oU(),t=Math.max;function n(r,i,u){return i=t(i===void 0?r.length-1:i,0),function(){for(var s=arguments,c=-1,d=t(s.length-i,0),h=Array(d);++c<d;)h[c]=s[i+c];c=-1;for(var v=Array(i+1);++c<i;)v[c]=s[c];return v[i]=u(h),e(r,this,v)}}return zg=n,zg}var Lg,iM;function lU(){if(iM)return Lg;iM=1;function e(t){return function(){return t}}return Lg=e,Lg}var Ig,oM;function qR(){if(oM)return Ig;oM=1;var e=Ti(),t=function(){try{var n=e(Object,"defineProperty");return n({},"",{}),n}catch{}}();return Ig=t,Ig}var Ug,uM;function sU(){if(uM)return Ug;uM=1;var e=lU(),t=qR(),n=du(),r=t?function(i,u){return t(i,"toString",{configurable:!0,enumerable:!1,value:e(u),writable:!0})}:n;return Ug=r,Ug}var Hg,lM;function cU(){if(lM)return Hg;lM=1;var e=800,t=16,n=Date.now;function r(i){var u=0,s=0;return function(){var c=n(),d=t-(c-s);if(s=c,d>0){if(++u>=e)return arguments[0]}else u=0;return i.apply(void 0,arguments)}}return Hg=r,Hg}var Gg,sM;function fU(){if(sM)return Gg;sM=1;var e=sU(),t=cU(),n=t(e);return Gg=n,Gg}var Vg,cM;function dU(){if(cM)return Vg;cM=1;var e=du(),t=uU(),n=fU();function r(i,u){return n(t(i,u,e),i+"")}return Vg=r,Vg}var Zg,fM;function Fd(){if(fM)return Zg;fM=1;var e=Nx(),t=$s(),n=Zx(),r=qa();function i(u,s,c){if(!r(c))return!1;var d=typeof s;return(d=="number"?t(c)&&n(s,c.length):d=="string"&&s in c)?e(c[s],u):!1}return Zg=i,Zg}var Yg,dM;function hU(){if(dM)return Yg;dM=1;var e=NR(),t=iU(),n=dU(),r=Fd(),i=n(function(u,s){if(u==null)return[];var c=s.length;return c>1&&r(u,s[0],s[1])?s=[]:c>2&&r(s[0],s[1],s[2])&&(s=[s[0]]),t(u,e(s,1),[])});return Yg=i,Yg}var pU=hU();const Wx=lt(pU);function Jl(e){"@babel/helpers - typeof";return Jl=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},Jl(e)}function m0(){return m0=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},m0.apply(this,arguments)}function vU(e,t){return bU(e)||gU(e,t)||mU(e,t)||yU()}function yU(){throw new TypeError(`Invalid attempt to destructure non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function mU(e,t){if(e){if(typeof e=="string")return hM(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);if(n==="Object"&&e.constructor&&(n=e.constructor.name),n==="Map"||n==="Set")return Array.from(e);if(n==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return hM(e,t)}}function hM(e,t){(t==null||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}function gU(e,t){var n=e==null?null:typeof Symbol<"u"&&e[Symbol.iterator]||e["@@iterator"];if(n!=null){var r,i,u,s,c=[],d=!0,h=!1;try{if(u=(n=n.call(e)).next,t!==0)for(;!(d=(r=u.call(n)).done)&&(c.push(r.value),c.length!==t);d=!0);}catch(v){h=!0,i=v}finally{try{if(!d&&n.return!=null&&(s=n.return(),Object(s)!==s))return}finally{if(h)throw i}}return c}}function bU(e){if(Array.isArray(e))return e}function pM(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function Xg(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?pM(Object(n),!0).forEach(function(r){xU(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):pM(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function xU(e,t,n){return t=_U(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function _U(e){var t=SU(e,"string");return Jl(t)=="symbol"?t:t+""}function SU(e,t){if(Jl(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(Jl(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return(t==="string"?String:Number)(e)}function wU(e){return Array.isArray(e)&&Dt(e[0])&&Dt(e[1])?e.join(" ~ "):e}var OU=function(t){var n=t.separator,r=n===void 0?" : ":n,i=t.contentStyle,u=i===void 0?{}:i,s=t.itemStyle,c=s===void 0?{}:s,d=t.labelStyle,h=d===void 0?{}:d,v=t.payload,y=t.formatter,g=t.itemSorter,b=t.wrapperClassName,S=t.labelClassName,_=t.label,x=t.labelFormatter,E=t.accessibilityLayer,M=E===void 0?!1:E,A=function(){if(v&&v.length){var G={padding:0,margin:0},W=(g?Wx(v,g):v).map(function(ne,Q){if(ne.type==="none")return null;var D=Xg({display:"block",paddingTop:4,paddingBottom:4,color:ne.color||"#000"},c),H=ne.formatter||y||wU,U=ne.value,ue=ne.name,N=U,X=ue;if(H&&N!=null&&X!=null){var ee=H(U,ue,ne,Q,v);if(Array.isArray(ee)){var te=vU(ee,2);N=te[0],X=te[1]}else N=ee}return L.createElement("li",{className:"recharts-tooltip-item",key:"tooltip-item-".concat(Q),style:D},Dt(X)?L.createElement("span",{className:"recharts-tooltip-item-name"},X):null,Dt(X)?L.createElement("span",{className:"recharts-tooltip-item-separator"},r):null,L.createElement("span",{className:"recharts-tooltip-item-value"},N),L.createElement("span",{className:"recharts-tooltip-item-unit"},ne.unit||""))});return L.createElement("ul",{className:"recharts-tooltip-item-list",style:G},W)}return null},C=Xg({margin:0,padding:10,backgroundColor:"#fff",border:"1px solid #ccc",whiteSpace:"nowrap"},u),w=Xg({margin:0},h),T=!ze(_),P=T?_:"",R=Xe("recharts-default-tooltip",b),$=Xe("recharts-tooltip-label",S);T&&x&&v!==void 0&&v!==null&&(P=x(_,v));var J=M?{role:"status","aria-live":"assertive"}:{};return L.createElement("div",m0({className:R,style:C},J),L.createElement("p",{className:$,style:w},L.isValidElement(P)?P:"".concat(P)),A())};function es(e){"@babel/helpers - typeof";return es=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},es(e)}function mf(e,t,n){return t=AU(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function AU(e){var t=EU(e,"string");return es(t)=="symbol"?t:t+""}function EU(e,t){if(es(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(es(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return(t==="string"?String:Number)(e)}var ml="recharts-tooltip-wrapper",TU={visibility:"hidden"};function MU(e){var t=e.coordinate,n=e.translateX,r=e.translateY;return Xe(ml,mf(mf(mf(mf({},"".concat(ml,"-right"),he(n)&&t&&he(t.x)&&n>=t.x),"".concat(ml,"-left"),he(n)&&t&&he(t.x)&&n<t.x),"".concat(ml,"-bottom"),he(r)&&t&&he(t.y)&&r>=t.y),"".concat(ml,"-top"),he(r)&&t&&he(t.y)&&r<t.y))}function vM(e){var t=e.allowEscapeViewBox,n=e.coordinate,r=e.key,i=e.offsetTopLeft,u=e.position,s=e.reverseDirection,c=e.tooltipDimension,d=e.viewBox,h=e.viewBoxDimension;if(u&&he(u[r]))return u[r];var v=n[r]-c-i,y=n[r]+i;if(t[r])return s[r]?v:y;if(s[r]){var g=v,b=d[r];return g<b?Math.max(y,d[r]):Math.max(v,d[r])}var S=y+c,_=d[r]+h;return S>_?Math.max(v,d[r]):Math.max(y,d[r])}function jU(e){var t=e.translateX,n=e.translateY,r=e.useTranslate3d;return{transform:r?"translate3d(".concat(t,"px, ").concat(n,"px, 0)"):"translate(".concat(t,"px, ").concat(n,"px)")}}function CU(e){var t=e.allowEscapeViewBox,n=e.coordinate,r=e.offsetTopLeft,i=e.position,u=e.reverseDirection,s=e.tooltipBox,c=e.useTranslate3d,d=e.viewBox,h,v,y;return s.height>0&&s.width>0&&n?(v=vM({allowEscapeViewBox:t,coordinate:n,key:"x",offsetTopLeft:r,position:i,reverseDirection:u,tooltipDimension:s.width,viewBox:d,viewBoxDimension:d.width}),y=vM({allowEscapeViewBox:t,coordinate:n,key:"y",offsetTopLeft:r,position:i,reverseDirection:u,tooltipDimension:s.height,viewBox:d,viewBoxDimension:d.height}),h=jU({translateX:v,translateY:y,useTranslate3d:c})):h=TU,{cssProperties:h,cssClasses:MU({translateX:v,translateY:y,coordinate:n})}}function Uo(e){"@babel/helpers - typeof";return Uo=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},Uo(e)}function yM(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function mM(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?yM(Object(n),!0).forEach(function(r){b0(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):yM(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function PU(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function RU(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,BR(r.key),r)}}function NU(e,t,n){return t&&RU(e.prototype,t),Object.defineProperty(e,"prototype",{writable:!1}),e}function DU(e,t,n){return t=Kf(t),kU(e,$R()?Reflect.construct(t,n||[],Kf(e).constructor):t.apply(e,n))}function kU(e,t){if(t&&(Uo(t)==="object"||typeof t=="function"))return t;if(t!==void 0)throw new TypeError("Derived constructors may only return object or undefined");return qU(e)}function qU(e){if(e===void 0)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function $R(){try{var e=!Boolean.prototype.valueOf.call(Reflect.construct(Boolean,[],function(){}))}catch{}return($R=function(){return!!e})()}function Kf(e){return Kf=Object.setPrototypeOf?Object.getPrototypeOf.bind():function(n){return n.__proto__||Object.getPrototypeOf(n)},Kf(e)}function $U(e,t){if(typeof t!="function"&&t!==null)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),Object.defineProperty(e,"prototype",{writable:!1}),t&&g0(e,t)}function g0(e,t){return g0=Object.setPrototypeOf?Object.setPrototypeOf.bind():function(r,i){return r.__proto__=i,r},g0(e,t)}function b0(e,t,n){return t=BR(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function BR(e){var t=BU(e,"string");return Uo(t)=="symbol"?t:t+""}function BU(e,t){if(Uo(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(Uo(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return String(e)}var gM=1,zU=function(e){function t(){var n;PU(this,t);for(var r=arguments.length,i=new Array(r),u=0;u<r;u++)i[u]=arguments[u];return n=DU(this,t,[].concat(i)),b0(n,"state",{dismissed:!1,dismissedAtCoordinate:{x:0,y:0},lastBoundingBox:{width:-1,height:-1}}),b0(n,"handleKeyDown",function(s){if(s.key==="Escape"){var c,d,h,v;n.setState({dismissed:!0,dismissedAtCoordinate:{x:(c=(d=n.props.coordinate)===null||d===void 0?void 0:d.x)!==null&&c!==void 0?c:0,y:(h=(v=n.props.coordinate)===null||v===void 0?void 0:v.y)!==null&&h!==void 0?h:0}})}}),n}return $U(t,e),NU(t,[{key:"updateBBox",value:function(){if(this.wrapperNode&&this.wrapperNode.getBoundingClientRect){var r=this.wrapperNode.getBoundingClientRect();(Math.abs(r.width-this.state.lastBoundingBox.width)>gM||Math.abs(r.height-this.state.lastBoundingBox.height)>gM)&&this.setState({lastBoundingBox:{width:r.width,height:r.height}})}else(this.state.lastBoundingBox.width!==-1||this.state.lastBoundingBox.height!==-1)&&this.setState({lastBoundingBox:{width:-1,height:-1}})}},{key:"componentDidMount",value:function(){document.addEventListener("keydown",this.handleKeyDown),this.updateBBox()}},{key:"componentWillUnmount",value:function(){document.removeEventListener("keydown",this.handleKeyDown)}},{key:"componentDidUpdate",value:function(){var r,i;this.props.active&&this.updateBBox(),this.state.dismissed&&(((r=this.props.coordinate)===null||r===void 0?void 0:r.x)!==this.state.dismissedAtCoordinate.x||((i=this.props.coordinate)===null||i===void 0?void 0:i.y)!==this.state.dismissedAtCoordinate.y)&&(this.state.dismissed=!1)}},{key:"render",value:function(){var r=this,i=this.props,u=i.active,s=i.allowEscapeViewBox,c=i.animationDuration,d=i.animationEasing,h=i.children,v=i.coordinate,y=i.hasPayload,g=i.isAnimationActive,b=i.offset,S=i.position,_=i.reverseDirection,x=i.useTranslate3d,E=i.viewBox,M=i.wrapperStyle,A=CU({allowEscapeViewBox:s,coordinate:v,offsetTopLeft:b,position:S,reverseDirection:_,tooltipBox:this.state.lastBoundingBox,useTranslate3d:x,viewBox:E}),C=A.cssClasses,w=A.cssProperties,T=mM(mM({transition:g&&u?"transform ".concat(c,"ms ").concat(d):void 0},w),{},{pointerEvents:"none",visibility:!this.state.dismissed&&u&&y?"visible":"hidden",position:"absolute",top:0,left:0},M);return L.createElement("div",{tabIndex:-1,className:C,style:T,ref:function(R){r.wrapperNode=R}},h)}}])}(q.PureComponent),LU=function(){return!(typeof window<"u"&&window.document&&window.document.createElement&&window.setTimeout)},hu={isSsr:LU()};function Ho(e){"@babel/helpers - typeof";return Ho=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},Ho(e)}function bM(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function xM(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?bM(Object(n),!0).forEach(function(r){Fx(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):bM(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function IU(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function UU(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,LR(r.key),r)}}function HU(e,t,n){return t&&UU(e.prototype,t),Object.defineProperty(e,"prototype",{writable:!1}),e}function GU(e,t,n){return t=Wf(t),VU(e,zR()?Reflect.construct(t,n||[],Wf(e).constructor):t.apply(e,n))}function VU(e,t){if(t&&(Ho(t)==="object"||typeof t=="function"))return t;if(t!==void 0)throw new TypeError("Derived constructors may only return object or undefined");return ZU(e)}function ZU(e){if(e===void 0)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function zR(){try{var e=!Boolean.prototype.valueOf.call(Reflect.construct(Boolean,[],function(){}))}catch{}return(zR=function(){return!!e})()}function Wf(e){return Wf=Object.setPrototypeOf?Object.getPrototypeOf.bind():function(n){return n.__proto__||Object.getPrototypeOf(n)},Wf(e)}function YU(e,t){if(typeof t!="function"&&t!==null)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),Object.defineProperty(e,"prototype",{writable:!1}),t&&x0(e,t)}function x0(e,t){return x0=Object.setPrototypeOf?Object.setPrototypeOf.bind():function(r,i){return r.__proto__=i,r},x0(e,t)}function Fx(e,t,n){return t=LR(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function LR(e){var t=XU(e,"string");return Ho(t)=="symbol"?t:t+""}function XU(e,t){if(Ho(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(Ho(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return String(e)}function KU(e){return e.dataKey}function WU(e,t){return L.isValidElement(e)?L.cloneElement(e,t):typeof e=="function"?L.createElement(e,t):L.createElement(OU,t)}var sr=function(e){function t(){return IU(this,t),GU(this,t,arguments)}return YU(t,e),HU(t,[{key:"render",value:function(){var r=this,i=this.props,u=i.active,s=i.allowEscapeViewBox,c=i.animationDuration,d=i.animationEasing,h=i.content,v=i.coordinate,y=i.filterNull,g=i.isAnimationActive,b=i.offset,S=i.payload,_=i.payloadUniqBy,x=i.position,E=i.reverseDirection,M=i.useTranslate3d,A=i.viewBox,C=i.wrapperStyle,w=S??[];y&&w.length&&(w=CR(S.filter(function(P){return P.value!=null&&(P.hide!==!0||r.props.includeHidden)}),_,KU));var T=w.length>0;return L.createElement(zU,{allowEscapeViewBox:s,animationDuration:c,animationEasing:d,isAnimationActive:g,active:u,coordinate:v,hasPayload:T,offset:b,position:x,reverseDirection:E,useTranslate3d:M,viewBox:A,wrapperStyle:C},WU(h,xM(xM({},this.props),{},{payload:w})))}}])}(q.PureComponent);Fx(sr,"displayName","Tooltip");Fx(sr,"defaultProps",{accessibilityLayer:!1,allowEscapeViewBox:{x:!1,y:!1},animationDuration:400,animationEasing:"ease",contentStyle:{},coordinate:{x:0,y:0},cursor:!0,cursorStyle:{},filterNull:!0,isAnimationActive:!hu.isSsr,itemStyle:{},labelStyle:{},offset:10,reverseDirection:{x:!1,y:!1},separator:" : ",trigger:"hover",useTranslate3d:!1,viewBox:{x:0,y:0,height:0,width:0},wrapperStyle:{}});var Kg,_M;function FU(){if(_M)return Kg;_M=1;var e=br(),t=function(){return e.Date.now()};return Kg=t,Kg}var Wg,SM;function QU(){if(SM)return Wg;SM=1;var e=/\s/;function t(n){for(var r=n.length;r--&&e.test(n.charAt(r)););return r}return Wg=t,Wg}var Fg,wM;function JU(){if(wM)return Fg;wM=1;var e=QU(),t=/^\s+/;function n(r){return r&&r.slice(0,e(r)+1).replace(t,"")}return Fg=n,Fg}var Qg,OM;function IR(){if(OM)return Qg;OM=1;var e=JU(),t=qa(),n=cu(),r=NaN,i=/^[-+]0x[0-9a-f]+$/i,u=/^0b[01]+$/i,s=/^0o[0-7]+$/i,c=parseInt;function d(h){if(typeof h=="number")return h;if(n(h))return r;if(t(h)){var v=typeof h.valueOf=="function"?h.valueOf():h;h=t(v)?v+"":v}if(typeof h!="string")return h===0?h:+h;h=e(h);var y=u.test(h);return y||s.test(h)?c(h.slice(2),y?2:8):i.test(h)?r:+h}return Qg=d,Qg}var Jg,AM;function e8(){if(AM)return Jg;AM=1;var e=qa(),t=FU(),n=IR(),r="Expected a function",i=Math.max,u=Math.min;function s(c,d,h){var v,y,g,b,S,_,x=0,E=!1,M=!1,A=!0;if(typeof c!="function")throw new TypeError(r);d=n(d)||0,e(h)&&(E=!!h.leading,M="maxWait"in h,g=M?i(n(h.maxWait)||0,d):g,A="trailing"in h?!!h.trailing:A);function C(W){var ne=v,Q=y;return v=y=void 0,x=W,b=c.apply(Q,ne),b}function w(W){return x=W,S=setTimeout(R,d),E?C(W):b}function T(W){var ne=W-_,Q=W-x,D=d-ne;return M?u(D,g-Q):D}function P(W){var ne=W-_,Q=W-x;return _===void 0||ne>=d||ne<0||M&&Q>=g}function R(){var W=t();if(P(W))return $(W);S=setTimeout(R,T(W))}function $(W){return S=void 0,A&&v?C(W):(v=y=void 0,b)}function J(){S!==void 0&&clearTimeout(S),x=0,v=_=y=S=void 0}function B(){return S===void 0?b:$(t())}function G(){var W=t(),ne=P(W);if(v=arguments,y=this,_=W,ne){if(S===void 0)return w(_);if(M)return clearTimeout(S),S=setTimeout(R,d),C(_)}return S===void 0&&(S=setTimeout(R,d)),b}return G.cancel=J,G.flush=B,G}return Jg=s,Jg}var eb,EM;function t8(){if(EM)return eb;EM=1;var e=e8(),t=qa(),n="Expected a function";function r(i,u,s){var c=!0,d=!0;if(typeof i!="function")throw new TypeError(n);return t(s)&&(c="leading"in s?!!s.leading:c,d="trailing"in s?!!s.trailing:d),e(i,u,{leading:c,maxWait:u,trailing:d})}return eb=r,eb}var n8=t8();const UR=lt(n8);function ts(e){"@babel/helpers - typeof";return ts=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},ts(e)}function TM(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function gf(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?TM(Object(n),!0).forEach(function(r){r8(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):TM(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function r8(e,t,n){return t=a8(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function a8(e){var t=i8(e,"string");return ts(t)=="symbol"?t:t+""}function i8(e,t){if(ts(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(ts(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return(t==="string"?String:Number)(e)}function o8(e,t){return c8(e)||s8(e,t)||l8(e,t)||u8()}function u8(){throw new TypeError(`Invalid attempt to destructure non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function l8(e,t){if(e){if(typeof e=="string")return MM(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);if(n==="Object"&&e.constructor&&(n=e.constructor.name),n==="Map"||n==="Set")return Array.from(e);if(n==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return MM(e,t)}}function MM(e,t){(t==null||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}function s8(e,t){var n=e==null?null:typeof Symbol<"u"&&e[Symbol.iterator]||e["@@iterator"];if(n!=null){var r,i,u,s,c=[],d=!0,h=!1;try{if(u=(n=n.call(e)).next,t!==0)for(;!(d=(r=u.call(n)).done)&&(c.push(r.value),c.length!==t);d=!0);}catch(v){h=!0,i=v}finally{try{if(!d&&n.return!=null&&(s=n.return(),Object(s)!==s))return}finally{if(h)throw i}}return c}}function c8(e){if(Array.isArray(e))return e}var f8=q.forwardRef(function(e,t){var n=e.aspect,r=e.initialDimension,i=r===void 0?{width:-1,height:-1}:r,u=e.width,s=u===void 0?"100%":u,c=e.height,d=c===void 0?"100%":c,h=e.minWidth,v=h===void 0?0:h,y=e.minHeight,g=e.maxHeight,b=e.children,S=e.debounce,_=S===void 0?0:S,x=e.id,E=e.className,M=e.onResize,A=e.style,C=A===void 0?{}:A,w=q.useRef(null),T=q.useRef();T.current=M,q.useImperativeHandle(t,function(){return Object.defineProperty(w.current,"current",{get:function(){return console.warn("The usage of ref.current.current is deprecated and will no longer be supported."),w.current},configurable:!0})});var P=q.useState({containerWidth:i.width,containerHeight:i.height}),R=o8(P,2),$=R[0],J=R[1],B=q.useCallback(function(W,ne){J(function(Q){var D=Math.round(W),H=Math.round(ne);return Q.containerWidth===D&&Q.containerHeight===H?Q:{containerWidth:D,containerHeight:H}})},[]);q.useEffect(function(){var W=function(ue){var N,X=ue[0].contentRect,ee=X.width,te=X.height;B(ee,te),(N=T.current)===null||N===void 0||N.call(T,ee,te)};_>0&&(W=UR(W,_,{trailing:!0,leading:!1}));var ne=new ResizeObserver(W),Q=w.current.getBoundingClientRect(),D=Q.width,H=Q.height;return B(D,H),ne.observe(w.current),function(){ne.disconnect()}},[B,_]);var G=q.useMemo(function(){var W=$.containerWidth,ne=$.containerHeight;if(W<0||ne<0)return null;vi(ci(s)||ci(d),`The width(%s) and height(%s) are both fixed numbers,
       maybe you don't need to use a ResponsiveContainer.`,s,d),vi(!n||n>0,"The aspect(%s) must be greater than zero.",n);var Q=ci(s)?W:s,D=ci(d)?ne:d;n&&n>0&&(Q?D=Q/n:D&&(Q=D*n),g&&D>g&&(D=g)),vi(Q>0||D>0,`The width(%s) and height(%s) of chart should be greater than 0,
       please check the style of container, or the props width(%s) and height(%s),
       or add a minWidth(%s) or minHeight(%s) or use aspect(%s) to control the
       height and width.`,Q,D,s,d,v,y,n);var H=!Array.isArray(b)&&Hr(b.type).endsWith("Chart");return L.Children.map(b,function(U){return L.isValidElement(U)?q.cloneElement(U,gf({width:Q,height:D},H?{style:gf({height:"100%",width:"100%",maxHeight:D,maxWidth:Q},U.props.style)}:{})):U})},[n,b,d,g,y,v,$,s]);return L.createElement("div",{id:x?"".concat(x):void 0,className:Xe("recharts-responsive-container",E),style:gf(gf({},C),{},{width:s,height:d,minWidth:v,minHeight:y,maxHeight:g}),ref:w},G)}),Qx=function(t){return null};Qx.displayName="Cell";function ns(e){"@babel/helpers - typeof";return ns=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},ns(e)}function jM(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function _0(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?jM(Object(n),!0).forEach(function(r){d8(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):jM(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function d8(e,t,n){return t=h8(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function h8(e){var t=p8(e,"string");return ns(t)=="symbol"?t:t+""}function p8(e,t){if(ns(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(ns(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return(t==="string"?String:Number)(e)}var So={widthCache:{},cacheCount:0},v8=2e3,y8={position:"absolute",top:"-20000px",left:0,padding:0,margin:0,border:"none",whiteSpace:"pre"},CM="recharts_measurement_span";function m8(e){var t=_0({},e);return Object.keys(t).forEach(function(n){t[n]||delete t[n]}),t}var Pl=function(t){var n=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{};if(t==null||hu.isSsr)return{width:0,height:0};var r=m8(n),i=JSON.stringify({text:t,copyStyle:r});if(So.widthCache[i])return So.widthCache[i];try{var u=document.getElementById(CM);u||(u=document.createElement("span"),u.setAttribute("id",CM),u.setAttribute("aria-hidden","true"),document.body.appendChild(u));var s=_0(_0({},y8),r);Object.assign(u.style,s),u.textContent="".concat(t);var c=u.getBoundingClientRect(),d={width:c.width,height:c.height};return So.widthCache[i]=d,++So.cacheCount>v8&&(So.cacheCount=0,So.widthCache={}),d}catch{return{width:0,height:0}}},g8=function(t){return{top:t.top+window.scrollY-document.documentElement.clientTop,left:t.left+window.scrollX-document.documentElement.clientLeft}};function rs(e){"@babel/helpers - typeof";return rs=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},rs(e)}function Ff(e,t){return S8(e)||_8(e,t)||x8(e,t)||b8()}function b8(){throw new TypeError(`Invalid attempt to destructure non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function x8(e,t){if(e){if(typeof e=="string")return PM(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);if(n==="Object"&&e.constructor&&(n=e.constructor.name),n==="Map"||n==="Set")return Array.from(e);if(n==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return PM(e,t)}}function PM(e,t){(t==null||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}function _8(e,t){var n=e==null?null:typeof Symbol<"u"&&e[Symbol.iterator]||e["@@iterator"];if(n!=null){var r,i,u,s,c=[],d=!0,h=!1;try{if(u=(n=n.call(e)).next,t===0){if(Object(n)!==n)return;d=!1}else for(;!(d=(r=u.call(n)).done)&&(c.push(r.value),c.length!==t);d=!0);}catch(v){h=!0,i=v}finally{try{if(!d&&n.return!=null&&(s=n.return(),Object(s)!==s))return}finally{if(h)throw i}}return c}}function S8(e){if(Array.isArray(e))return e}function w8(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function RM(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,A8(r.key),r)}}function O8(e,t,n){return t&&RM(e.prototype,t),n&&RM(e,n),Object.defineProperty(e,"prototype",{writable:!1}),e}function A8(e){var t=E8(e,"string");return rs(t)=="symbol"?t:t+""}function E8(e,t){if(rs(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(rs(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return String(e)}var NM=/(-?\d+(?:\.\d+)?[a-zA-Z%]*)([*/])(-?\d+(?:\.\d+)?[a-zA-Z%]*)/,DM=/(-?\d+(?:\.\d+)?[a-zA-Z%]*)([+-])(-?\d+(?:\.\d+)?[a-zA-Z%]*)/,T8=/^px|cm|vh|vw|em|rem|%|mm|in|pt|pc|ex|ch|vmin|vmax|Q$/,M8=/(-?\d+(?:\.\d+)?)([a-zA-Z%]+)?/,HR={cm:96/2.54,mm:96/25.4,pt:96/72,pc:96/6,in:96,Q:96/(2.54*40),px:1},j8=Object.keys(HR),To="NaN";function C8(e,t){return e*HR[t]}var bf=function(){function e(t,n){w8(this,e),this.num=t,this.unit=n,this.num=t,this.unit=n,Number.isNaN(t)&&(this.unit=""),n!==""&&!T8.test(n)&&(this.num=NaN,this.unit=""),j8.includes(n)&&(this.num=C8(t,n),this.unit="px")}return O8(e,[{key:"add",value:function(n){return this.unit!==n.unit?new e(NaN,""):new e(this.num+n.num,this.unit)}},{key:"subtract",value:function(n){return this.unit!==n.unit?new e(NaN,""):new e(this.num-n.num,this.unit)}},{key:"multiply",value:function(n){return this.unit!==""&&n.unit!==""&&this.unit!==n.unit?new e(NaN,""):new e(this.num*n.num,this.unit||n.unit)}},{key:"divide",value:function(n){return this.unit!==""&&n.unit!==""&&this.unit!==n.unit?new e(NaN,""):new e(this.num/n.num,this.unit||n.unit)}},{key:"toString",value:function(){return"".concat(this.num).concat(this.unit)}},{key:"isNaN",value:function(){return Number.isNaN(this.num)}}],[{key:"parse",value:function(n){var r,i=(r=M8.exec(n))!==null&&r!==void 0?r:[],u=Ff(i,3),s=u[1],c=u[2];return new e(parseFloat(s),c??"")}}])}();function GR(e){if(e.includes(To))return To;for(var t=e;t.includes("*")||t.includes("/");){var n,r=(n=NM.exec(t))!==null&&n!==void 0?n:[],i=Ff(r,4),u=i[1],s=i[2],c=i[3],d=bf.parse(u??""),h=bf.parse(c??""),v=s==="*"?d.multiply(h):d.divide(h);if(v.isNaN())return To;t=t.replace(NM,v.toString())}for(;t.includes("+")||/.-\d+(?:\.\d+)?/.test(t);){var y,g=(y=DM.exec(t))!==null&&y!==void 0?y:[],b=Ff(g,4),S=b[1],_=b[2],x=b[3],E=bf.parse(S??""),M=bf.parse(x??""),A=_==="+"?E.add(M):E.subtract(M);if(A.isNaN())return To;t=t.replace(DM,A.toString())}return t}var kM=/\(([^()]*)\)/;function P8(e){for(var t=e;t.includes("(");){var n=kM.exec(t),r=Ff(n,2),i=r[1];t=t.replace(kM,GR(i))}return t}function R8(e){var t=e.replace(/\s+/g,"");return t=P8(t),t=GR(t),t}function N8(e){try{return R8(e)}catch{return To}}function tb(e){var t=N8(e.slice(5,-1));return t===To?"":t}var D8=["x","y","lineHeight","capHeight","scaleToFit","textAnchor","verticalAnchor","fill"],k8=["dx","dy","angle","className","breakAll"];function S0(){return S0=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},S0.apply(this,arguments)}function qM(e,t){if(e==null)return{};var n=q8(e,t),r,i;if(Object.getOwnPropertySymbols){var u=Object.getOwnPropertySymbols(e);for(i=0;i<u.length;i++)r=u[i],!(t.indexOf(r)>=0)&&Object.prototype.propertyIsEnumerable.call(e,r)&&(n[r]=e[r])}return n}function q8(e,t){if(e==null)return{};var n={};for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){if(t.indexOf(r)>=0)continue;n[r]=e[r]}return n}function $M(e,t){return L8(e)||z8(e,t)||B8(e,t)||$8()}function $8(){throw new TypeError(`Invalid attempt to destructure non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function B8(e,t){if(e){if(typeof e=="string")return BM(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);if(n==="Object"&&e.constructor&&(n=e.constructor.name),n==="Map"||n==="Set")return Array.from(e);if(n==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return BM(e,t)}}function BM(e,t){(t==null||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}function z8(e,t){var n=e==null?null:typeof Symbol<"u"&&e[Symbol.iterator]||e["@@iterator"];if(n!=null){var r,i,u,s,c=[],d=!0,h=!1;try{if(u=(n=n.call(e)).next,t===0){if(Object(n)!==n)return;d=!1}else for(;!(d=(r=u.call(n)).done)&&(c.push(r.value),c.length!==t);d=!0);}catch(v){h=!0,i=v}finally{try{if(!d&&n.return!=null&&(s=n.return(),Object(s)!==s))return}finally{if(h)throw i}}return c}}function L8(e){if(Array.isArray(e))return e}var VR=/[ \f\n\r\t\v\u2028\u2029]+/,ZR=function(t){var n=t.children,r=t.breakAll,i=t.style;try{var u=[];ze(n)||(r?u=n.toString().split(""):u=n.toString().split(VR));var s=u.map(function(d){return{word:d,width:Pl(d,i).width}}),c=r?0:Pl(" ",i).width;return{wordsWithComputedWidth:s,spaceWidth:c}}catch{return null}},I8=function(t,n,r,i,u){var s=t.maxLines,c=t.children,d=t.style,h=t.breakAll,v=he(s),y=c,g=function(){var Q=arguments.length>0&&arguments[0]!==void 0?arguments[0]:[];return Q.reduce(function(D,H){var U=H.word,ue=H.width,N=D[D.length-1];if(N&&(i==null||u||N.width+ue+r<Number(i)))N.words.push(U),N.width+=ue+r;else{var X={words:[U],width:ue};D.push(X)}return D},[])},b=g(n),S=function(Q){return Q.reduce(function(D,H){return D.width>H.width?D:H})};if(!v)return b;for(var _="…",x=function(Q){var D=y.slice(0,Q),H=ZR({breakAll:h,style:d,children:D+_}).wordsWithComputedWidth,U=g(H),ue=U.length>s||S(U).width>Number(i);return[ue,U]},E=0,M=y.length-1,A=0,C;E<=M&&A<=y.length-1;){var w=Math.floor((E+M)/2),T=w-1,P=x(T),R=$M(P,2),$=R[0],J=R[1],B=x(w),G=$M(B,1),W=G[0];if(!$&&!W&&(E=w+1),$&&W&&(M=w-1),!$&&W){C=J;break}A++}return C||b},zM=function(t){var n=ze(t)?[]:t.toString().split(VR);return[{words:n}]},U8=function(t){var n=t.width,r=t.scaleToFit,i=t.children,u=t.style,s=t.breakAll,c=t.maxLines;if((n||r)&&!hu.isSsr){var d,h,v=ZR({breakAll:s,children:i,style:u});if(v){var y=v.wordsWithComputedWidth,g=v.spaceWidth;d=y,h=g}else return zM(i);return I8({breakAll:s,children:i,maxLines:c,style:u},d,h,n,r)}return zM(i)},LM="#808080",Qf=function(t){var n=t.x,r=n===void 0?0:n,i=t.y,u=i===void 0?0:i,s=t.lineHeight,c=s===void 0?"1em":s,d=t.capHeight,h=d===void 0?"0.71em":d,v=t.scaleToFit,y=v===void 0?!1:v,g=t.textAnchor,b=g===void 0?"start":g,S=t.verticalAnchor,_=S===void 0?"end":S,x=t.fill,E=x===void 0?LM:x,M=qM(t,D8),A=q.useMemo(function(){return U8({breakAll:M.breakAll,children:M.children,maxLines:M.maxLines,scaleToFit:y,style:M.style,width:M.width})},[M.breakAll,M.children,M.maxLines,y,M.style,M.width]),C=M.dx,w=M.dy,T=M.angle,P=M.className,R=M.breakAll,$=qM(M,k8);if(!Dt(r)||!Dt(u))return null;var J=r+(he(C)?C:0),B=u+(he(w)?w:0),G;switch(_){case"start":G=tb("calc(".concat(h,")"));break;case"middle":G=tb("calc(".concat((A.length-1)/2," * -").concat(c," + (").concat(h," / 2))"));break;default:G=tb("calc(".concat(A.length-1," * -").concat(c,")"));break}var W=[];if(y){var ne=A[0].width,Q=M.width;W.push("scale(".concat((he(Q)?Q/ne:1)/ne,")"))}return T&&W.push("rotate(".concat(T,", ").concat(J,", ").concat(B,")")),W.length&&($.transform=W.join(" ")),L.createElement("text",S0({},Ye($,!0),{x:J,y:B,className:Xe("recharts-text",P),textAnchor:b,fill:E.includes("url")?LM:E}),A.map(function(D,H){var U=D.words.join(R?"":" ");return L.createElement("tspan",{x:J,dy:H===0?G:c,key:"".concat(U,"-").concat(H)},U)}))};function Ca(e,t){return e==null||t==null?NaN:e<t?-1:e>t?1:e>=t?0:NaN}function H8(e,t){return e==null||t==null?NaN:t<e?-1:t>e?1:t>=e?0:NaN}function Jx(e){let t,n,r;e.length!==2?(t=Ca,n=(c,d)=>Ca(e(c),d),r=(c,d)=>e(c)-d):(t=e===Ca||e===H8?e:G8,n=e,r=e);function i(c,d,h=0,v=c.length){if(h<v){if(t(d,d)!==0)return v;do{const y=h+v>>>1;n(c[y],d)<0?h=y+1:v=y}while(h<v)}return h}function u(c,d,h=0,v=c.length){if(h<v){if(t(d,d)!==0)return v;do{const y=h+v>>>1;n(c[y],d)<=0?h=y+1:v=y}while(h<v)}return h}function s(c,d,h=0,v=c.length){const y=i(c,d,h,v-1);return y>h&&r(c[y-1],d)>-r(c[y],d)?y-1:y}return{left:i,center:s,right:u}}function G8(){return 0}function YR(e){return e===null?NaN:+e}function*V8(e,t){for(let n of e)n!=null&&(n=+n)>=n&&(yield n)}const Z8=Jx(Ca),Bs=Z8.right;Jx(YR).center;class IM extends Map{constructor(t,n=K8){if(super(),Object.defineProperties(this,{_intern:{value:new Map},_key:{value:n}}),t!=null)for(const[r,i]of t)this.set(r,i)}get(t){return super.get(UM(this,t))}has(t){return super.has(UM(this,t))}set(t,n){return super.set(Y8(this,t),n)}delete(t){return super.delete(X8(this,t))}}function UM({_intern:e,_key:t},n){const r=t(n);return e.has(r)?e.get(r):n}function Y8({_intern:e,_key:t},n){const r=t(n);return e.has(r)?e.get(r):(e.set(r,n),n)}function X8({_intern:e,_key:t},n){const r=t(n);return e.has(r)&&(n=e.get(r),e.delete(r)),n}function K8(e){return e!==null&&typeof e=="object"?e.valueOf():e}function W8(e=Ca){if(e===Ca)return XR;if(typeof e!="function")throw new TypeError("compare is not a function");return(t,n)=>{const r=e(t,n);return r||r===0?r:(e(n,n)===0)-(e(t,t)===0)}}function XR(e,t){return(e==null||!(e>=e))-(t==null||!(t>=t))||(e<t?-1:e>t?1:0)}const F8=Math.sqrt(50),Q8=Math.sqrt(10),J8=Math.sqrt(2);function Jf(e,t,n){const r=(t-e)/Math.max(0,n),i=Math.floor(Math.log10(r)),u=r/Math.pow(10,i),s=u>=F8?10:u>=Q8?5:u>=J8?2:1;let c,d,h;return i<0?(h=Math.pow(10,-i)/s,c=Math.round(e*h),d=Math.round(t*h),c/h<e&&++c,d/h>t&&--d,h=-h):(h=Math.pow(10,i)*s,c=Math.round(e/h),d=Math.round(t/h),c*h<e&&++c,d*h>t&&--d),d<c&&.5<=n&&n<2?Jf(e,t,n*2):[c,d,h]}function w0(e,t,n){if(t=+t,e=+e,n=+n,!(n>0))return[];if(e===t)return[e];const r=t<e,[i,u,s]=r?Jf(t,e,n):Jf(e,t,n);if(!(u>=i))return[];const c=u-i+1,d=new Array(c);if(r)if(s<0)for(let h=0;h<c;++h)d[h]=(u-h)/-s;else for(let h=0;h<c;++h)d[h]=(u-h)*s;else if(s<0)for(let h=0;h<c;++h)d[h]=(i+h)/-s;else for(let h=0;h<c;++h)d[h]=(i+h)*s;return d}function O0(e,t,n){return t=+t,e=+e,n=+n,Jf(e,t,n)[2]}function A0(e,t,n){t=+t,e=+e,n=+n;const r=t<e,i=r?O0(t,e,n):O0(e,t,n);return(r?-1:1)*(i<0?1/-i:i)}function HM(e,t){let n;for(const r of e)r!=null&&(n<r||n===void 0&&r>=r)&&(n=r);return n}function GM(e,t){let n;for(const r of e)r!=null&&(n>r||n===void 0&&r>=r)&&(n=r);return n}function KR(e,t,n=0,r=1/0,i){if(t=Math.floor(t),n=Math.floor(Math.max(0,n)),r=Math.floor(Math.min(e.length-1,r)),!(n<=t&&t<=r))return e;for(i=i===void 0?XR:W8(i);r>n;){if(r-n>600){const d=r-n+1,h=t-n+1,v=Math.log(d),y=.5*Math.exp(2*v/3),g=.5*Math.sqrt(v*y*(d-y)/d)*(h-d/2<0?-1:1),b=Math.max(n,Math.floor(t-h*y/d+g)),S=Math.min(r,Math.floor(t+(d-h)*y/d+g));KR(e,t,b,S,i)}const u=e[t];let s=n,c=r;for(gl(e,n,t),i(e[r],u)>0&&gl(e,n,r);s<c;){for(gl(e,s,c),++s,--c;i(e[s],u)<0;)++s;for(;i(e[c],u)>0;)--c}i(e[n],u)===0?gl(e,n,c):(++c,gl(e,c,r)),c<=t&&(n=c+1),t<=c&&(r=c-1)}return e}function gl(e,t,n){const r=e[t];e[t]=e[n],e[n]=r}function e5(e,t,n){if(e=Float64Array.from(V8(e)),!(!(r=e.length)||isNaN(t=+t))){if(t<=0||r<2)return GM(e);if(t>=1)return HM(e);var r,i=(r-1)*t,u=Math.floor(i),s=HM(KR(e,u).subarray(0,u+1)),c=GM(e.subarray(u+1));return s+(c-s)*(i-u)}}function t5(e,t,n=YR){if(!(!(r=e.length)||isNaN(t=+t))){if(t<=0||r<2)return+n(e[0],0,e);if(t>=1)return+n(e[r-1],r-1,e);var r,i=(r-1)*t,u=Math.floor(i),s=+n(e[u],u,e),c=+n(e[u+1],u+1,e);return s+(c-s)*(i-u)}}function n5(e,t,n){e=+e,t=+t,n=(i=arguments.length)<2?(t=e,e=0,1):i<3?1:+n;for(var r=-1,i=Math.max(0,Math.ceil((t-e)/n))|0,u=new Array(i);++r<i;)u[r]=e+r*n;return u}function Un(e,t){switch(arguments.length){case 0:break;case 1:this.range(e);break;default:this.range(t).domain(e);break}return this}function Jr(e,t){switch(arguments.length){case 0:break;case 1:{typeof e=="function"?this.interpolator(e):this.range(e);break}default:{this.domain(e),typeof t=="function"?this.interpolator(t):this.range(t);break}}return this}const E0=Symbol("implicit");function e_(){var e=new IM,t=[],n=[],r=E0;function i(u){let s=e.get(u);if(s===void 0){if(r!==E0)return r;e.set(u,s=t.push(u)-1)}return n[s%n.length]}return i.domain=function(u){if(!arguments.length)return t.slice();t=[],e=new IM;for(const s of u)e.has(s)||e.set(s,t.push(s)-1);return i},i.range=function(u){return arguments.length?(n=Array.from(u),i):n.slice()},i.unknown=function(u){return arguments.length?(r=u,i):r},i.copy=function(){return e_(t,n).unknown(r)},Un.apply(i,arguments),i}function as(){var e=e_().unknown(void 0),t=e.domain,n=e.range,r=0,i=1,u,s,c=!1,d=0,h=0,v=.5;delete e.unknown;function y(){var g=t().length,b=i<r,S=b?i:r,_=b?r:i;u=(_-S)/Math.max(1,g-d+h*2),c&&(u=Math.floor(u)),S+=(_-S-u*(g-d))*v,s=u*(1-d),c&&(S=Math.round(S),s=Math.round(s));var x=n5(g).map(function(E){return S+u*E});return n(b?x.reverse():x)}return e.domain=function(g){return arguments.length?(t(g),y()):t()},e.range=function(g){return arguments.length?([r,i]=g,r=+r,i=+i,y()):[r,i]},e.rangeRound=function(g){return[r,i]=g,r=+r,i=+i,c=!0,y()},e.bandwidth=function(){return s},e.step=function(){return u},e.round=function(g){return arguments.length?(c=!!g,y()):c},e.padding=function(g){return arguments.length?(d=Math.min(1,h=+g),y()):d},e.paddingInner=function(g){return arguments.length?(d=Math.min(1,g),y()):d},e.paddingOuter=function(g){return arguments.length?(h=+g,y()):h},e.align=function(g){return arguments.length?(v=Math.max(0,Math.min(1,g)),y()):v},e.copy=function(){return as(t(),[r,i]).round(c).paddingInner(d).paddingOuter(h).align(v)},Un.apply(y(),arguments)}function WR(e){var t=e.copy;return e.padding=e.paddingOuter,delete e.paddingInner,delete e.paddingOuter,e.copy=function(){return WR(t())},e}function Rl(){return WR(as.apply(null,arguments).paddingInner(1))}function t_(e,t,n){e.prototype=t.prototype=n,n.constructor=e}function FR(e,t){var n=Object.create(e.prototype);for(var r in t)n[r]=t[r];return n}function zs(){}var is=.7,ed=1/is,Do="\\s*([+-]?\\d+)\\s*",os="\\s*([+-]?(?:\\d*\\.)?\\d+(?:[eE][+-]?\\d+)?)\\s*",pr="\\s*([+-]?(?:\\d*\\.)?\\d+(?:[eE][+-]?\\d+)?)%\\s*",r5=/^#([0-9a-f]{3,8})$/,a5=new RegExp(`^rgb\\(${Do},${Do},${Do}\\)$`),i5=new RegExp(`^rgb\\(${pr},${pr},${pr}\\)$`),o5=new RegExp(`^rgba\\(${Do},${Do},${Do},${os}\\)$`),u5=new RegExp(`^rgba\\(${pr},${pr},${pr},${os}\\)$`),l5=new RegExp(`^hsl\\(${os},${pr},${pr}\\)$`),s5=new RegExp(`^hsla\\(${os},${pr},${pr},${os}\\)$`),VM={aliceblue:15792383,antiquewhite:16444375,aqua:65535,aquamarine:8388564,azure:15794175,beige:16119260,bisque:16770244,black:0,blanchedalmond:16772045,blue:255,blueviolet:9055202,brown:10824234,burlywood:14596231,cadetblue:6266528,chartreuse:8388352,chocolate:13789470,coral:16744272,cornflowerblue:6591981,cornsilk:16775388,crimson:14423100,cyan:65535,darkblue:139,darkcyan:35723,darkgoldenrod:12092939,darkgray:11119017,darkgreen:25600,darkgrey:11119017,darkkhaki:12433259,darkmagenta:9109643,darkolivegreen:5597999,darkorange:16747520,darkorchid:10040012,darkred:9109504,darksalmon:15308410,darkseagreen:9419919,darkslateblue:4734347,darkslategray:3100495,darkslategrey:3100495,darkturquoise:52945,darkviolet:9699539,deeppink:16716947,deepskyblue:49151,dimgray:6908265,dimgrey:6908265,dodgerblue:2003199,firebrick:11674146,floralwhite:16775920,forestgreen:2263842,fuchsia:16711935,gainsboro:14474460,ghostwhite:16316671,gold:16766720,goldenrod:14329120,gray:8421504,green:32768,greenyellow:11403055,grey:8421504,honeydew:15794160,hotpink:16738740,indianred:13458524,indigo:4915330,ivory:16777200,khaki:15787660,lavender:15132410,lavenderblush:16773365,lawngreen:8190976,lemonchiffon:16775885,lightblue:11393254,lightcoral:15761536,lightcyan:14745599,lightgoldenrodyellow:16448210,lightgray:13882323,lightgreen:9498256,lightgrey:13882323,lightpink:16758465,lightsalmon:16752762,lightseagreen:2142890,lightskyblue:8900346,lightslategray:7833753,lightslategrey:7833753,lightsteelblue:11584734,lightyellow:16777184,lime:65280,limegreen:3329330,linen:16445670,magenta:16711935,maroon:8388608,mediumaquamarine:6737322,mediumblue:205,mediumorchid:12211667,mediumpurple:9662683,mediumseagreen:3978097,mediumslateblue:8087790,mediumspringgreen:64154,mediumturquoise:4772300,mediumvioletred:13047173,midnightblue:1644912,mintcream:16121850,mistyrose:16770273,moccasin:16770229,navajowhite:16768685,navy:128,oldlace:16643558,olive:8421376,olivedrab:7048739,orange:16753920,orangered:16729344,orchid:14315734,palegoldenrod:15657130,palegreen:10025880,paleturquoise:11529966,palevioletred:14381203,papayawhip:16773077,peachpuff:16767673,peru:13468991,pink:16761035,plum:14524637,powderblue:11591910,purple:8388736,rebeccapurple:6697881,red:16711680,rosybrown:12357519,royalblue:4286945,saddlebrown:9127187,salmon:16416882,sandybrown:16032864,seagreen:3050327,seashell:16774638,sienna:10506797,silver:12632256,skyblue:8900331,slateblue:6970061,slategray:7372944,slategrey:7372944,snow:16775930,springgreen:65407,steelblue:4620980,tan:13808780,teal:32896,thistle:14204888,tomato:16737095,turquoise:4251856,violet:15631086,wheat:16113331,white:16777215,whitesmoke:16119285,yellow:16776960,yellowgreen:10145074};t_(zs,us,{copy(e){return Object.assign(new this.constructor,this,e)},displayable(){return this.rgb().displayable()},hex:ZM,formatHex:ZM,formatHex8:c5,formatHsl:f5,formatRgb:YM,toString:YM});function ZM(){return this.rgb().formatHex()}function c5(){return this.rgb().formatHex8()}function f5(){return QR(this).formatHsl()}function YM(){return this.rgb().formatRgb()}function us(e){var t,n;return e=(e+"").trim().toLowerCase(),(t=r5.exec(e))?(n=t[1].length,t=parseInt(t[1],16),n===6?XM(t):n===3?new cn(t>>8&15|t>>4&240,t>>4&15|t&240,(t&15)<<4|t&15,1):n===8?xf(t>>24&255,t>>16&255,t>>8&255,(t&255)/255):n===4?xf(t>>12&15|t>>8&240,t>>8&15|t>>4&240,t>>4&15|t&240,((t&15)<<4|t&15)/255):null):(t=a5.exec(e))?new cn(t[1],t[2],t[3],1):(t=i5.exec(e))?new cn(t[1]*255/100,t[2]*255/100,t[3]*255/100,1):(t=o5.exec(e))?xf(t[1],t[2],t[3],t[4]):(t=u5.exec(e))?xf(t[1]*255/100,t[2]*255/100,t[3]*255/100,t[4]):(t=l5.exec(e))?FM(t[1],t[2]/100,t[3]/100,1):(t=s5.exec(e))?FM(t[1],t[2]/100,t[3]/100,t[4]):VM.hasOwnProperty(e)?XM(VM[e]):e==="transparent"?new cn(NaN,NaN,NaN,0):null}function XM(e){return new cn(e>>16&255,e>>8&255,e&255,1)}function xf(e,t,n,r){return r<=0&&(e=t=n=NaN),new cn(e,t,n,r)}function d5(e){return e instanceof zs||(e=us(e)),e?(e=e.rgb(),new cn(e.r,e.g,e.b,e.opacity)):new cn}function T0(e,t,n,r){return arguments.length===1?d5(e):new cn(e,t,n,r??1)}function cn(e,t,n,r){this.r=+e,this.g=+t,this.b=+n,this.opacity=+r}t_(cn,T0,FR(zs,{brighter(e){return e=e==null?ed:Math.pow(ed,e),new cn(this.r*e,this.g*e,this.b*e,this.opacity)},darker(e){return e=e==null?is:Math.pow(is,e),new cn(this.r*e,this.g*e,this.b*e,this.opacity)},rgb(){return this},clamp(){return new cn(yi(this.r),yi(this.g),yi(this.b),td(this.opacity))},displayable(){return-.5<=this.r&&this.r<255.5&&-.5<=this.g&&this.g<255.5&&-.5<=this.b&&this.b<255.5&&0<=this.opacity&&this.opacity<=1},hex:KM,formatHex:KM,formatHex8:h5,formatRgb:WM,toString:WM}));function KM(){return`#${fi(this.r)}${fi(this.g)}${fi(this.b)}`}function h5(){return`#${fi(this.r)}${fi(this.g)}${fi(this.b)}${fi((isNaN(this.opacity)?1:this.opacity)*255)}`}function WM(){const e=td(this.opacity);return`${e===1?"rgb(":"rgba("}${yi(this.r)}, ${yi(this.g)}, ${yi(this.b)}${e===1?")":`, ${e})`}`}function td(e){return isNaN(e)?1:Math.max(0,Math.min(1,e))}function yi(e){return Math.max(0,Math.min(255,Math.round(e)||0))}function fi(e){return e=yi(e),(e<16?"0":"")+e.toString(16)}function FM(e,t,n,r){return r<=0?e=t=n=NaN:n<=0||n>=1?e=t=NaN:t<=0&&(e=NaN),new Kn(e,t,n,r)}function QR(e){if(e instanceof Kn)return new Kn(e.h,e.s,e.l,e.opacity);if(e instanceof zs||(e=us(e)),!e)return new Kn;if(e instanceof Kn)return e;e=e.rgb();var t=e.r/255,n=e.g/255,r=e.b/255,i=Math.min(t,n,r),u=Math.max(t,n,r),s=NaN,c=u-i,d=(u+i)/2;return c?(t===u?s=(n-r)/c+(n<r)*6:n===u?s=(r-t)/c+2:s=(t-n)/c+4,c/=d<.5?u+i:2-u-i,s*=60):c=d>0&&d<1?0:s,new Kn(s,c,d,e.opacity)}function p5(e,t,n,r){return arguments.length===1?QR(e):new Kn(e,t,n,r??1)}function Kn(e,t,n,r){this.h=+e,this.s=+t,this.l=+n,this.opacity=+r}t_(Kn,p5,FR(zs,{brighter(e){return e=e==null?ed:Math.pow(ed,e),new Kn(this.h,this.s,this.l*e,this.opacity)},darker(e){return e=e==null?is:Math.pow(is,e),new Kn(this.h,this.s,this.l*e,this.opacity)},rgb(){var e=this.h%360+(this.h<0)*360,t=isNaN(e)||isNaN(this.s)?0:this.s,n=this.l,r=n+(n<.5?n:1-n)*t,i=2*n-r;return new cn(nb(e>=240?e-240:e+120,i,r),nb(e,i,r),nb(e<120?e+240:e-120,i,r),this.opacity)},clamp(){return new Kn(QM(this.h),_f(this.s),_f(this.l),td(this.opacity))},displayable(){return(0<=this.s&&this.s<=1||isNaN(this.s))&&0<=this.l&&this.l<=1&&0<=this.opacity&&this.opacity<=1},formatHsl(){const e=td(this.opacity);return`${e===1?"hsl(":"hsla("}${QM(this.h)}, ${_f(this.s)*100}%, ${_f(this.l)*100}%${e===1?")":`, ${e})`}`}}));function QM(e){return e=(e||0)%360,e<0?e+360:e}function _f(e){return Math.max(0,Math.min(1,e||0))}function nb(e,t,n){return(e<60?t+(n-t)*e/60:e<180?n:e<240?t+(n-t)*(240-e)/60:t)*255}const n_=e=>()=>e;function v5(e,t){return function(n){return e+n*t}}function y5(e,t,n){return e=Math.pow(e,n),t=Math.pow(t,n)-e,n=1/n,function(r){return Math.pow(e+r*t,n)}}function m5(e){return(e=+e)==1?JR:function(t,n){return n-t?y5(t,n,e):n_(isNaN(t)?n:t)}}function JR(e,t){var n=t-e;return n?v5(e,n):n_(isNaN(e)?t:e)}const JM=function e(t){var n=m5(t);function r(i,u){var s=n((i=T0(i)).r,(u=T0(u)).r),c=n(i.g,u.g),d=n(i.b,u.b),h=JR(i.opacity,u.opacity);return function(v){return i.r=s(v),i.g=c(v),i.b=d(v),i.opacity=h(v),i+""}}return r.gamma=e,r}(1);function g5(e,t){t||(t=[]);var n=e?Math.min(t.length,e.length):0,r=t.slice(),i;return function(u){for(i=0;i<n;++i)r[i]=e[i]*(1-u)+t[i]*u;return r}}function b5(e){return ArrayBuffer.isView(e)&&!(e instanceof DataView)}function x5(e,t){var n=t?t.length:0,r=e?Math.min(n,e.length):0,i=new Array(r),u=new Array(n),s;for(s=0;s<r;++s)i[s]=pu(e[s],t[s]);for(;s<n;++s)u[s]=t[s];return function(c){for(s=0;s<r;++s)u[s]=i[s](c);return u}}function _5(e,t){var n=new Date;return e=+e,t=+t,function(r){return n.setTime(e*(1-r)+t*r),n}}function nd(e,t){return e=+e,t=+t,function(n){return e*(1-n)+t*n}}function S5(e,t){var n={},r={},i;(e===null||typeof e!="object")&&(e={}),(t===null||typeof t!="object")&&(t={});for(i in t)i in e?n[i]=pu(e[i],t[i]):r[i]=t[i];return function(u){for(i in n)r[i]=n[i](u);return r}}var M0=/[-+]?(?:\d+\.?\d*|\.?\d+)(?:[eE][-+]?\d+)?/g,rb=new RegExp(M0.source,"g");function w5(e){return function(){return e}}function O5(e){return function(t){return e(t)+""}}function A5(e,t){var n=M0.lastIndex=rb.lastIndex=0,r,i,u,s=-1,c=[],d=[];for(e=e+"",t=t+"";(r=M0.exec(e))&&(i=rb.exec(t));)(u=i.index)>n&&(u=t.slice(n,u),c[s]?c[s]+=u:c[++s]=u),(r=r[0])===(i=i[0])?c[s]?c[s]+=i:c[++s]=i:(c[++s]=null,d.push({i:s,x:nd(r,i)})),n=rb.lastIndex;return n<t.length&&(u=t.slice(n),c[s]?c[s]+=u:c[++s]=u),c.length<2?d[0]?O5(d[0].x):w5(t):(t=d.length,function(h){for(var v=0,y;v<t;++v)c[(y=d[v]).i]=y.x(h);return c.join("")})}function pu(e,t){var n=typeof t,r;return t==null||n==="boolean"?n_(t):(n==="number"?nd:n==="string"?(r=us(t))?(t=r,JM):A5:t instanceof us?JM:t instanceof Date?_5:b5(t)?g5:Array.isArray(t)?x5:typeof t.valueOf!="function"&&typeof t.toString!="function"||isNaN(t)?S5:nd)(e,t)}function r_(e,t){return e=+e,t=+t,function(n){return Math.round(e*(1-n)+t*n)}}function E5(e,t){t===void 0&&(t=e,e=pu);for(var n=0,r=t.length-1,i=t[0],u=new Array(r<0?0:r);n<r;)u[n]=e(i,i=t[++n]);return function(s){var c=Math.max(0,Math.min(r-1,Math.floor(s*=r)));return u[c](s-c)}}function T5(e){return function(){return e}}function rd(e){return+e}var ej=[0,1];function tn(e){return e}function j0(e,t){return(t-=e=+e)?function(n){return(n-e)/t}:T5(isNaN(t)?NaN:.5)}function M5(e,t){var n;return e>t&&(n=e,e=t,t=n),function(r){return Math.max(e,Math.min(t,r))}}function j5(e,t,n){var r=e[0],i=e[1],u=t[0],s=t[1];return i<r?(r=j0(i,r),u=n(s,u)):(r=j0(r,i),u=n(u,s)),function(c){return u(r(c))}}function C5(e,t,n){var r=Math.min(e.length,t.length)-1,i=new Array(r),u=new Array(r),s=-1;for(e[r]<e[0]&&(e=e.slice().reverse(),t=t.slice().reverse());++s<r;)i[s]=j0(e[s],e[s+1]),u[s]=n(t[s],t[s+1]);return function(c){var d=Bs(e,c,1,r)-1;return u[d](i[d](c))}}function Ls(e,t){return t.domain(e.domain()).range(e.range()).interpolate(e.interpolate()).clamp(e.clamp()).unknown(e.unknown())}function Qd(){var e=ej,t=ej,n=pu,r,i,u,s=tn,c,d,h;function v(){var g=Math.min(e.length,t.length);return s!==tn&&(s=M5(e[0],e[g-1])),c=g>2?C5:j5,d=h=null,y}function y(g){return g==null||isNaN(g=+g)?u:(d||(d=c(e.map(r),t,n)))(r(s(g)))}return y.invert=function(g){return s(i((h||(h=c(t,e.map(r),nd)))(g)))},y.domain=function(g){return arguments.length?(e=Array.from(g,rd),v()):e.slice()},y.range=function(g){return arguments.length?(t=Array.from(g),v()):t.slice()},y.rangeRound=function(g){return t=Array.from(g),n=r_,v()},y.clamp=function(g){return arguments.length?(s=g?!0:tn,v()):s!==tn},y.interpolate=function(g){return arguments.length?(n=g,v()):n},y.unknown=function(g){return arguments.length?(u=g,y):u},function(g,b){return r=g,i=b,v()}}function a_(){return Qd()(tn,tn)}function P5(e){return Math.abs(e=Math.round(e))>=1e21?e.toLocaleString("en").replace(/,/g,""):e.toString(10)}function ad(e,t){if((n=(e=t?e.toExponential(t-1):e.toExponential()).indexOf("e"))<0)return null;var n,r=e.slice(0,n);return[r.length>1?r[0]+r.slice(2):r,+e.slice(n+1)]}function Go(e){return e=ad(Math.abs(e)),e?e[1]:NaN}function R5(e,t){return function(n,r){for(var i=n.length,u=[],s=0,c=e[0],d=0;i>0&&c>0&&(d+c+1>r&&(c=Math.max(1,r-d)),u.push(n.substring(i-=c,i+c)),!((d+=c+1)>r));)c=e[s=(s+1)%e.length];return u.reverse().join(t)}}function N5(e){return function(t){return t.replace(/[0-9]/g,function(n){return e[+n]})}}var D5=/^(?:(.)?([<>=^]))?([+\-( ])?([$#])?(0)?(\d+)?(,)?(\.\d+)?(~)?([a-z%])?$/i;function ls(e){if(!(t=D5.exec(e)))throw new Error("invalid format: "+e);var t;return new i_({fill:t[1],align:t[2],sign:t[3],symbol:t[4],zero:t[5],width:t[6],comma:t[7],precision:t[8]&&t[8].slice(1),trim:t[9],type:t[10]})}ls.prototype=i_.prototype;function i_(e){this.fill=e.fill===void 0?" ":e.fill+"",this.align=e.align===void 0?">":e.align+"",this.sign=e.sign===void 0?"-":e.sign+"",this.symbol=e.symbol===void 0?"":e.symbol+"",this.zero=!!e.zero,this.width=e.width===void 0?void 0:+e.width,this.comma=!!e.comma,this.precision=e.precision===void 0?void 0:+e.precision,this.trim=!!e.trim,this.type=e.type===void 0?"":e.type+""}i_.prototype.toString=function(){return this.fill+this.align+this.sign+this.symbol+(this.zero?"0":"")+(this.width===void 0?"":Math.max(1,this.width|0))+(this.comma?",":"")+(this.precision===void 0?"":"."+Math.max(0,this.precision|0))+(this.trim?"~":"")+this.type};function k5(e){e:for(var t=e.length,n=1,r=-1,i;n<t;++n)switch(e[n]){case".":r=i=n;break;case"0":r===0&&(r=n),i=n;break;default:if(!+e[n])break e;r>0&&(r=0);break}return r>0?e.slice(0,r)+e.slice(i+1):e}var eN;function q5(e,t){var n=ad(e,t);if(!n)return e+"";var r=n[0],i=n[1],u=i-(eN=Math.max(-8,Math.min(8,Math.floor(i/3)))*3)+1,s=r.length;return u===s?r:u>s?r+new Array(u-s+1).join("0"):u>0?r.slice(0,u)+"."+r.slice(u):"0."+new Array(1-u).join("0")+ad(e,Math.max(0,t+u-1))[0]}function tj(e,t){var n=ad(e,t);if(!n)return e+"";var r=n[0],i=n[1];return i<0?"0."+new Array(-i).join("0")+r:r.length>i+1?r.slice(0,i+1)+"."+r.slice(i+1):r+new Array(i-r.length+2).join("0")}const nj={"%":(e,t)=>(e*100).toFixed(t),b:e=>Math.round(e).toString(2),c:e=>e+"",d:P5,e:(e,t)=>e.toExponential(t),f:(e,t)=>e.toFixed(t),g:(e,t)=>e.toPrecision(t),o:e=>Math.round(e).toString(8),p:(e,t)=>tj(e*100,t),r:tj,s:q5,X:e=>Math.round(e).toString(16).toUpperCase(),x:e=>Math.round(e).toString(16)};function rj(e){return e}var aj=Array.prototype.map,ij=["y","z","a","f","p","n","µ","m","","k","M","G","T","P","E","Z","Y"];function $5(e){var t=e.grouping===void 0||e.thousands===void 0?rj:R5(aj.call(e.grouping,Number),e.thousands+""),n=e.currency===void 0?"":e.currency[0]+"",r=e.currency===void 0?"":e.currency[1]+"",i=e.decimal===void 0?".":e.decimal+"",u=e.numerals===void 0?rj:N5(aj.call(e.numerals,String)),s=e.percent===void 0?"%":e.percent+"",c=e.minus===void 0?"−":e.minus+"",d=e.nan===void 0?"NaN":e.nan+"";function h(y){y=ls(y);var g=y.fill,b=y.align,S=y.sign,_=y.symbol,x=y.zero,E=y.width,M=y.comma,A=y.precision,C=y.trim,w=y.type;w==="n"?(M=!0,w="g"):nj[w]||(A===void 0&&(A=12),C=!0,w="g"),(x||g==="0"&&b==="=")&&(x=!0,g="0",b="=");var T=_==="$"?n:_==="#"&&/[boxX]/.test(w)?"0"+w.toLowerCase():"",P=_==="$"?r:/[%p]/.test(w)?s:"",R=nj[w],$=/[defgprs%]/.test(w);A=A===void 0?6:/[gprs]/.test(w)?Math.max(1,Math.min(21,A)):Math.max(0,Math.min(20,A));function J(B){var G=T,W=P,ne,Q,D;if(w==="c")W=R(B)+W,B="";else{B=+B;var H=B<0||1/B<0;if(B=isNaN(B)?d:R(Math.abs(B),A),C&&(B=k5(B)),H&&+B==0&&S!=="+"&&(H=!1),G=(H?S==="("?S:c:S==="-"||S==="("?"":S)+G,W=(w==="s"?ij[8+eN/3]:"")+W+(H&&S==="("?")":""),$){for(ne=-1,Q=B.length;++ne<Q;)if(D=B.charCodeAt(ne),48>D||D>57){W=(D===46?i+B.slice(ne+1):B.slice(ne))+W,B=B.slice(0,ne);break}}}M&&!x&&(B=t(B,1/0));var U=G.length+B.length+W.length,ue=U<E?new Array(E-U+1).join(g):"";switch(M&&x&&(B=t(ue+B,ue.length?E-W.length:1/0),ue=""),b){case"<":B=G+B+W+ue;break;case"=":B=G+ue+B+W;break;case"^":B=ue.slice(0,U=ue.length>>1)+G+B+W+ue.slice(U);break;default:B=ue+G+B+W;break}return u(B)}return J.toString=function(){return y+""},J}function v(y,g){var b=h((y=ls(y),y.type="f",y)),S=Math.max(-8,Math.min(8,Math.floor(Go(g)/3)))*3,_=Math.pow(10,-S),x=ij[8+S/3];return function(E){return b(_*E)+x}}return{format:h,formatPrefix:v}}var Sf,o_,tN;B5({thousands:",",grouping:[3],currency:["$",""]});function B5(e){return Sf=$5(e),o_=Sf.format,tN=Sf.formatPrefix,Sf}function z5(e){return Math.max(0,-Go(Math.abs(e)))}function L5(e,t){return Math.max(0,Math.max(-8,Math.min(8,Math.floor(Go(t)/3)))*3-Go(Math.abs(e)))}function I5(e,t){return e=Math.abs(e),t=Math.abs(t)-e,Math.max(0,Go(t)-Go(e))+1}function nN(e,t,n,r){var i=A0(e,t,n),u;switch(r=ls(r??",f"),r.type){case"s":{var s=Math.max(Math.abs(e),Math.abs(t));return r.precision==null&&!isNaN(u=L5(i,s))&&(r.precision=u),tN(r,s)}case"":case"e":case"g":case"p":case"r":{r.precision==null&&!isNaN(u=I5(i,Math.max(Math.abs(e),Math.abs(t))))&&(r.precision=u-(r.type==="e"));break}case"f":case"%":{r.precision==null&&!isNaN(u=z5(i))&&(r.precision=u-(r.type==="%")*2);break}}return o_(r)}function Ba(e){var t=e.domain;return e.ticks=function(n){var r=t();return w0(r[0],r[r.length-1],n??10)},e.tickFormat=function(n,r){var i=t();return nN(i[0],i[i.length-1],n??10,r)},e.nice=function(n){n==null&&(n=10);var r=t(),i=0,u=r.length-1,s=r[i],c=r[u],d,h,v=10;for(c<s&&(h=s,s=c,c=h,h=i,i=u,u=h);v-- >0;){if(h=O0(s,c,n),h===d)return r[i]=s,r[u]=c,t(r);if(h>0)s=Math.floor(s/h)*h,c=Math.ceil(c/h)*h;else if(h<0)s=Math.ceil(s*h)/h,c=Math.floor(c*h)/h;else break;d=h}return e},e}function id(){var e=a_();return e.copy=function(){return Ls(e,id())},Un.apply(e,arguments),Ba(e)}function rN(e){var t;function n(r){return r==null||isNaN(r=+r)?t:r}return n.invert=n,n.domain=n.range=function(r){return arguments.length?(e=Array.from(r,rd),n):e.slice()},n.unknown=function(r){return arguments.length?(t=r,n):t},n.copy=function(){return rN(e).unknown(t)},e=arguments.length?Array.from(e,rd):[0,1],Ba(n)}function aN(e,t){e=e.slice();var n=0,r=e.length-1,i=e[n],u=e[r],s;return u<i&&(s=n,n=r,r=s,s=i,i=u,u=s),e[n]=t.floor(i),e[r]=t.ceil(u),e}function oj(e){return Math.log(e)}function uj(e){return Math.exp(e)}function U5(e){return-Math.log(-e)}function H5(e){return-Math.exp(-e)}function G5(e){return isFinite(e)?+("1e"+e):e<0?0:e}function V5(e){return e===10?G5:e===Math.E?Math.exp:t=>Math.pow(e,t)}function Z5(e){return e===Math.E?Math.log:e===10&&Math.log10||e===2&&Math.log2||(e=Math.log(e),t=>Math.log(t)/e)}function lj(e){return(t,n)=>-e(-t,n)}function u_(e){const t=e(oj,uj),n=t.domain;let r=10,i,u;function s(){return i=Z5(r),u=V5(r),n()[0]<0?(i=lj(i),u=lj(u),e(U5,H5)):e(oj,uj),t}return t.base=function(c){return arguments.length?(r=+c,s()):r},t.domain=function(c){return arguments.length?(n(c),s()):n()},t.ticks=c=>{const d=n();let h=d[0],v=d[d.length-1];const y=v<h;y&&([h,v]=[v,h]);let g=i(h),b=i(v),S,_;const x=c==null?10:+c;let E=[];if(!(r%1)&&b-g<x){if(g=Math.floor(g),b=Math.ceil(b),h>0){for(;g<=b;++g)for(S=1;S<r;++S)if(_=g<0?S/u(-g):S*u(g),!(_<h)){if(_>v)break;E.push(_)}}else for(;g<=b;++g)for(S=r-1;S>=1;--S)if(_=g>0?S/u(-g):S*u(g),!(_<h)){if(_>v)break;E.push(_)}E.length*2<x&&(E=w0(h,v,x))}else E=w0(g,b,Math.min(b-g,x)).map(u);return y?E.reverse():E},t.tickFormat=(c,d)=>{if(c==null&&(c=10),d==null&&(d=r===10?"s":","),typeof d!="function"&&(!(r%1)&&(d=ls(d)).precision==null&&(d.trim=!0),d=o_(d)),c===1/0)return d;const h=Math.max(1,r*c/t.ticks().length);return v=>{let y=v/u(Math.round(i(v)));return y*r<r-.5&&(y*=r),y<=h?d(v):""}},t.nice=()=>n(aN(n(),{floor:c=>u(Math.floor(i(c))),ceil:c=>u(Math.ceil(i(c)))})),t}function iN(){const e=u_(Qd()).domain([1,10]);return e.copy=()=>Ls(e,iN()).base(e.base()),Un.apply(e,arguments),e}function sj(e){return function(t){return Math.sign(t)*Math.log1p(Math.abs(t/e))}}function cj(e){return function(t){return Math.sign(t)*Math.expm1(Math.abs(t))*e}}function l_(e){var t=1,n=e(sj(t),cj(t));return n.constant=function(r){return arguments.length?e(sj(t=+r),cj(t)):t},Ba(n)}function oN(){var e=l_(Qd());return e.copy=function(){return Ls(e,oN()).constant(e.constant())},Un.apply(e,arguments)}function fj(e){return function(t){return t<0?-Math.pow(-t,e):Math.pow(t,e)}}function Y5(e){return e<0?-Math.sqrt(-e):Math.sqrt(e)}function X5(e){return e<0?-e*e:e*e}function s_(e){var t=e(tn,tn),n=1;function r(){return n===1?e(tn,tn):n===.5?e(Y5,X5):e(fj(n),fj(1/n))}return t.exponent=function(i){return arguments.length?(n=+i,r()):n},Ba(t)}function c_(){var e=s_(Qd());return e.copy=function(){return Ls(e,c_()).exponent(e.exponent())},Un.apply(e,arguments),e}function K5(){return c_.apply(null,arguments).exponent(.5)}function dj(e){return Math.sign(e)*e*e}function W5(e){return Math.sign(e)*Math.sqrt(Math.abs(e))}function uN(){var e=a_(),t=[0,1],n=!1,r;function i(u){var s=W5(e(u));return isNaN(s)?r:n?Math.round(s):s}return i.invert=function(u){return e.invert(dj(u))},i.domain=function(u){return arguments.length?(e.domain(u),i):e.domain()},i.range=function(u){return arguments.length?(e.range((t=Array.from(u,rd)).map(dj)),i):t.slice()},i.rangeRound=function(u){return i.range(u).round(!0)},i.round=function(u){return arguments.length?(n=!!u,i):n},i.clamp=function(u){return arguments.length?(e.clamp(u),i):e.clamp()},i.unknown=function(u){return arguments.length?(r=u,i):r},i.copy=function(){return uN(e.domain(),t).round(n).clamp(e.clamp()).unknown(r)},Un.apply(i,arguments),Ba(i)}function lN(){var e=[],t=[],n=[],r;function i(){var s=0,c=Math.max(1,t.length);for(n=new Array(c-1);++s<c;)n[s-1]=t5(e,s/c);return u}function u(s){return s==null||isNaN(s=+s)?r:t[Bs(n,s)]}return u.invertExtent=function(s){var c=t.indexOf(s);return c<0?[NaN,NaN]:[c>0?n[c-1]:e[0],c<n.length?n[c]:e[e.length-1]]},u.domain=function(s){if(!arguments.length)return e.slice();e=[];for(let c of s)c!=null&&!isNaN(c=+c)&&e.push(c);return e.sort(Ca),i()},u.range=function(s){return arguments.length?(t=Array.from(s),i()):t.slice()},u.unknown=function(s){return arguments.length?(r=s,u):r},u.quantiles=function(){return n.slice()},u.copy=function(){return lN().domain(e).range(t).unknown(r)},Un.apply(u,arguments)}function sN(){var e=0,t=1,n=1,r=[.5],i=[0,1],u;function s(d){return d!=null&&d<=d?i[Bs(r,d,0,n)]:u}function c(){var d=-1;for(r=new Array(n);++d<n;)r[d]=((d+1)*t-(d-n)*e)/(n+1);return s}return s.domain=function(d){return arguments.length?([e,t]=d,e=+e,t=+t,c()):[e,t]},s.range=function(d){return arguments.length?(n=(i=Array.from(d)).length-1,c()):i.slice()},s.invertExtent=function(d){var h=i.indexOf(d);return h<0?[NaN,NaN]:h<1?[e,r[0]]:h>=n?[r[n-1],t]:[r[h-1],r[h]]},s.unknown=function(d){return arguments.length&&(u=d),s},s.thresholds=function(){return r.slice()},s.copy=function(){return sN().domain([e,t]).range(i).unknown(u)},Un.apply(Ba(s),arguments)}function cN(){var e=[.5],t=[0,1],n,r=1;function i(u){return u!=null&&u<=u?t[Bs(e,u,0,r)]:n}return i.domain=function(u){return arguments.length?(e=Array.from(u),r=Math.min(e.length,t.length-1),i):e.slice()},i.range=function(u){return arguments.length?(t=Array.from(u),r=Math.min(e.length,t.length-1),i):t.slice()},i.invertExtent=function(u){var s=t.indexOf(u);return[e[s-1],e[s]]},i.unknown=function(u){return arguments.length?(n=u,i):n},i.copy=function(){return cN().domain(e).range(t).unknown(n)},Un.apply(i,arguments)}const ab=new Date,ib=new Date;function kt(e,t,n,r){function i(u){return e(u=arguments.length===0?new Date:new Date(+u)),u}return i.floor=u=>(e(u=new Date(+u)),u),i.ceil=u=>(e(u=new Date(u-1)),t(u,1),e(u),u),i.round=u=>{const s=i(u),c=i.ceil(u);return u-s<c-u?s:c},i.offset=(u,s)=>(t(u=new Date(+u),s==null?1:Math.floor(s)),u),i.range=(u,s,c)=>{const d=[];if(u=i.ceil(u),c=c==null?1:Math.floor(c),!(u<s)||!(c>0))return d;let h;do d.push(h=new Date(+u)),t(u,c),e(u);while(h<u&&u<s);return d},i.filter=u=>kt(s=>{if(s>=s)for(;e(s),!u(s);)s.setTime(s-1)},(s,c)=>{if(s>=s)if(c<0)for(;++c<=0;)for(;t(s,-1),!u(s););else for(;--c>=0;)for(;t(s,1),!u(s););}),n&&(i.count=(u,s)=>(ab.setTime(+u),ib.setTime(+s),e(ab),e(ib),Math.floor(n(ab,ib))),i.every=u=>(u=Math.floor(u),!isFinite(u)||!(u>0)?null:u>1?i.filter(r?s=>r(s)%u===0:s=>i.count(0,s)%u===0):i)),i}const od=kt(()=>{},(e,t)=>{e.setTime(+e+t)},(e,t)=>t-e);od.every=e=>(e=Math.floor(e),!isFinite(e)||!(e>0)?null:e>1?kt(t=>{t.setTime(Math.floor(t/e)*e)},(t,n)=>{t.setTime(+t+n*e)},(t,n)=>(n-t)/e):od);od.range;const Ir=1e3,zn=Ir*60,Ur=zn*60,Zr=Ur*24,f_=Zr*7,hj=Zr*30,ob=Zr*365,di=kt(e=>{e.setTime(e-e.getMilliseconds())},(e,t)=>{e.setTime(+e+t*Ir)},(e,t)=>(t-e)/Ir,e=>e.getUTCSeconds());di.range;const d_=kt(e=>{e.setTime(e-e.getMilliseconds()-e.getSeconds()*Ir)},(e,t)=>{e.setTime(+e+t*zn)},(e,t)=>(t-e)/zn,e=>e.getMinutes());d_.range;const h_=kt(e=>{e.setUTCSeconds(0,0)},(e,t)=>{e.setTime(+e+t*zn)},(e,t)=>(t-e)/zn,e=>e.getUTCMinutes());h_.range;const p_=kt(e=>{e.setTime(e-e.getMilliseconds()-e.getSeconds()*Ir-e.getMinutes()*zn)},(e,t)=>{e.setTime(+e+t*Ur)},(e,t)=>(t-e)/Ur,e=>e.getHours());p_.range;const v_=kt(e=>{e.setUTCMinutes(0,0,0)},(e,t)=>{e.setTime(+e+t*Ur)},(e,t)=>(t-e)/Ur,e=>e.getUTCHours());v_.range;const Is=kt(e=>e.setHours(0,0,0,0),(e,t)=>e.setDate(e.getDate()+t),(e,t)=>(t-e-(t.getTimezoneOffset()-e.getTimezoneOffset())*zn)/Zr,e=>e.getDate()-1);Is.range;const Jd=kt(e=>{e.setUTCHours(0,0,0,0)},(e,t)=>{e.setUTCDate(e.getUTCDate()+t)},(e,t)=>(t-e)/Zr,e=>e.getUTCDate()-1);Jd.range;const fN=kt(e=>{e.setUTCHours(0,0,0,0)},(e,t)=>{e.setUTCDate(e.getUTCDate()+t)},(e,t)=>(t-e)/Zr,e=>Math.floor(e/Zr));fN.range;function Mi(e){return kt(t=>{t.setDate(t.getDate()-(t.getDay()+7-e)%7),t.setHours(0,0,0,0)},(t,n)=>{t.setDate(t.getDate()+n*7)},(t,n)=>(n-t-(n.getTimezoneOffset()-t.getTimezoneOffset())*zn)/f_)}const eh=Mi(0),ud=Mi(1),F5=Mi(2),Q5=Mi(3),Vo=Mi(4),J5=Mi(5),e9=Mi(6);eh.range;ud.range;F5.range;Q5.range;Vo.range;J5.range;e9.range;function ji(e){return kt(t=>{t.setUTCDate(t.getUTCDate()-(t.getUTCDay()+7-e)%7),t.setUTCHours(0,0,0,0)},(t,n)=>{t.setUTCDate(t.getUTCDate()+n*7)},(t,n)=>(n-t)/f_)}const th=ji(0),ld=ji(1),t9=ji(2),n9=ji(3),Zo=ji(4),r9=ji(5),a9=ji(6);th.range;ld.range;t9.range;n9.range;Zo.range;r9.range;a9.range;const y_=kt(e=>{e.setDate(1),e.setHours(0,0,0,0)},(e,t)=>{e.setMonth(e.getMonth()+t)},(e,t)=>t.getMonth()-e.getMonth()+(t.getFullYear()-e.getFullYear())*12,e=>e.getMonth());y_.range;const m_=kt(e=>{e.setUTCDate(1),e.setUTCHours(0,0,0,0)},(e,t)=>{e.setUTCMonth(e.getUTCMonth()+t)},(e,t)=>t.getUTCMonth()-e.getUTCMonth()+(t.getUTCFullYear()-e.getUTCFullYear())*12,e=>e.getUTCMonth());m_.range;const Yr=kt(e=>{e.setMonth(0,1),e.setHours(0,0,0,0)},(e,t)=>{e.setFullYear(e.getFullYear()+t)},(e,t)=>t.getFullYear()-e.getFullYear(),e=>e.getFullYear());Yr.every=e=>!isFinite(e=Math.floor(e))||!(e>0)?null:kt(t=>{t.setFullYear(Math.floor(t.getFullYear()/e)*e),t.setMonth(0,1),t.setHours(0,0,0,0)},(t,n)=>{t.setFullYear(t.getFullYear()+n*e)});Yr.range;const Xr=kt(e=>{e.setUTCMonth(0,1),e.setUTCHours(0,0,0,0)},(e,t)=>{e.setUTCFullYear(e.getUTCFullYear()+t)},(e,t)=>t.getUTCFullYear()-e.getUTCFullYear(),e=>e.getUTCFullYear());Xr.every=e=>!isFinite(e=Math.floor(e))||!(e>0)?null:kt(t=>{t.setUTCFullYear(Math.floor(t.getUTCFullYear()/e)*e),t.setUTCMonth(0,1),t.setUTCHours(0,0,0,0)},(t,n)=>{t.setUTCFullYear(t.getUTCFullYear()+n*e)});Xr.range;function dN(e,t,n,r,i,u){const s=[[di,1,Ir],[di,5,5*Ir],[di,15,15*Ir],[di,30,30*Ir],[u,1,zn],[u,5,5*zn],[u,15,15*zn],[u,30,30*zn],[i,1,Ur],[i,3,3*Ur],[i,6,6*Ur],[i,12,12*Ur],[r,1,Zr],[r,2,2*Zr],[n,1,f_],[t,1,hj],[t,3,3*hj],[e,1,ob]];function c(h,v,y){const g=v<h;g&&([h,v]=[v,h]);const b=y&&typeof y.range=="function"?y:d(h,v,y),S=b?b.range(h,+v+1):[];return g?S.reverse():S}function d(h,v,y){const g=Math.abs(v-h)/y,b=Jx(([,,x])=>x).right(s,g);if(b===s.length)return e.every(A0(h/ob,v/ob,y));if(b===0)return od.every(Math.max(A0(h,v,y),1));const[S,_]=s[g/s[b-1][2]<s[b][2]/g?b-1:b];return S.every(_)}return[c,d]}const[i9,o9]=dN(Xr,m_,th,fN,v_,h_),[u9,l9]=dN(Yr,y_,eh,Is,p_,d_);function ub(e){if(0<=e.y&&e.y<100){var t=new Date(-1,e.m,e.d,e.H,e.M,e.S,e.L);return t.setFullYear(e.y),t}return new Date(e.y,e.m,e.d,e.H,e.M,e.S,e.L)}function lb(e){if(0<=e.y&&e.y<100){var t=new Date(Date.UTC(-1,e.m,e.d,e.H,e.M,e.S,e.L));return t.setUTCFullYear(e.y),t}return new Date(Date.UTC(e.y,e.m,e.d,e.H,e.M,e.S,e.L))}function bl(e,t,n){return{y:e,m:t,d:n,H:0,M:0,S:0,L:0}}function s9(e){var t=e.dateTime,n=e.date,r=e.time,i=e.periods,u=e.days,s=e.shortDays,c=e.months,d=e.shortMonths,h=xl(i),v=_l(i),y=xl(u),g=_l(u),b=xl(s),S=_l(s),_=xl(c),x=_l(c),E=xl(d),M=_l(d),A={a:H,A:U,b:ue,B:N,c:null,d:bj,e:bj,f:R9,g:U9,G:G9,H:j9,I:C9,j:P9,L:hN,m:N9,M:D9,p:X,q:ee,Q:Sj,s:wj,S:k9,u:q9,U:$9,V:B9,w:z9,W:L9,x:null,X:null,y:I9,Y:H9,Z:V9,"%":_j},C={a:te,A:ce,b:_e,B:be,c:null,d:xj,e:xj,f:K9,g:iH,G:uH,H:Z9,I:Y9,j:X9,L:vN,m:W9,M:F9,p:ke,q:Pe,Q:Sj,s:wj,S:Q9,u:J9,U:eH,V:tH,w:nH,W:rH,x:null,X:null,y:aH,Y:oH,Z:lH,"%":_j},w={a:J,A:B,b:G,B:W,c:ne,d:mj,e:mj,f:A9,g:yj,G:vj,H:gj,I:gj,j:_9,L:O9,m:x9,M:S9,p:$,q:b9,Q:T9,s:M9,S:w9,u:p9,U:v9,V:y9,w:h9,W:m9,x:Q,X:D,y:yj,Y:vj,Z:g9,"%":E9};A.x=T(n,A),A.X=T(r,A),A.c=T(t,A),C.x=T(n,C),C.X=T(r,C),C.c=T(t,C);function T(fe,Oe){return function(Me){var le=[],st=-1,Ie=0,St=fe.length,wt,dt,rr;for(Me instanceof Date||(Me=new Date(+Me));++st<St;)fe.charCodeAt(st)===37&&(le.push(fe.slice(Ie,st)),(dt=pj[wt=fe.charAt(++st)])!=null?wt=fe.charAt(++st):dt=wt==="e"?" ":"0",(rr=Oe[wt])&&(wt=rr(Me,dt)),le.push(wt),Ie=st+1);return le.push(fe.slice(Ie,st)),le.join("")}}function P(fe,Oe){return function(Me){var le=bl(1900,void 0,1),st=R(le,fe,Me+="",0),Ie,St;if(st!=Me.length)return null;if("Q"in le)return new Date(le.Q);if("s"in le)return new Date(le.s*1e3+("L"in le?le.L:0));if(Oe&&!("Z"in le)&&(le.Z=0),"p"in le&&(le.H=le.H%12+le.p*12),le.m===void 0&&(le.m="q"in le?le.q:0),"V"in le){if(le.V<1||le.V>53)return null;"w"in le||(le.w=1),"Z"in le?(Ie=lb(bl(le.y,0,1)),St=Ie.getUTCDay(),Ie=St>4||St===0?ld.ceil(Ie):ld(Ie),Ie=Jd.offset(Ie,(le.V-1)*7),le.y=Ie.getUTCFullYear(),le.m=Ie.getUTCMonth(),le.d=Ie.getUTCDate()+(le.w+6)%7):(Ie=ub(bl(le.y,0,1)),St=Ie.getDay(),Ie=St>4||St===0?ud.ceil(Ie):ud(Ie),Ie=Is.offset(Ie,(le.V-1)*7),le.y=Ie.getFullYear(),le.m=Ie.getMonth(),le.d=Ie.getDate()+(le.w+6)%7)}else("W"in le||"U"in le)&&("w"in le||(le.w="u"in le?le.u%7:"W"in le?1:0),St="Z"in le?lb(bl(le.y,0,1)).getUTCDay():ub(bl(le.y,0,1)).getDay(),le.m=0,le.d="W"in le?(le.w+6)%7+le.W*7-(St+5)%7:le.w+le.U*7-(St+6)%7);return"Z"in le?(le.H+=le.Z/100|0,le.M+=le.Z%100,lb(le)):ub(le)}}function R(fe,Oe,Me,le){for(var st=0,Ie=Oe.length,St=Me.length,wt,dt;st<Ie;){if(le>=St)return-1;if(wt=Oe.charCodeAt(st++),wt===37){if(wt=Oe.charAt(st++),dt=w[wt in pj?Oe.charAt(st++):wt],!dt||(le=dt(fe,Me,le))<0)return-1}else if(wt!=Me.charCodeAt(le++))return-1}return le}function $(fe,Oe,Me){var le=h.exec(Oe.slice(Me));return le?(fe.p=v.get(le[0].toLowerCase()),Me+le[0].length):-1}function J(fe,Oe,Me){var le=b.exec(Oe.slice(Me));return le?(fe.w=S.get(le[0].toLowerCase()),Me+le[0].length):-1}function B(fe,Oe,Me){var le=y.exec(Oe.slice(Me));return le?(fe.w=g.get(le[0].toLowerCase()),Me+le[0].length):-1}function G(fe,Oe,Me){var le=E.exec(Oe.slice(Me));return le?(fe.m=M.get(le[0].toLowerCase()),Me+le[0].length):-1}function W(fe,Oe,Me){var le=_.exec(Oe.slice(Me));return le?(fe.m=x.get(le[0].toLowerCase()),Me+le[0].length):-1}function ne(fe,Oe,Me){return R(fe,t,Oe,Me)}function Q(fe,Oe,Me){return R(fe,n,Oe,Me)}function D(fe,Oe,Me){return R(fe,r,Oe,Me)}function H(fe){return s[fe.getDay()]}function U(fe){return u[fe.getDay()]}function ue(fe){return d[fe.getMonth()]}function N(fe){return c[fe.getMonth()]}function X(fe){return i[+(fe.getHours()>=12)]}function ee(fe){return 1+~~(fe.getMonth()/3)}function te(fe){return s[fe.getUTCDay()]}function ce(fe){return u[fe.getUTCDay()]}function _e(fe){return d[fe.getUTCMonth()]}function be(fe){return c[fe.getUTCMonth()]}function ke(fe){return i[+(fe.getUTCHours()>=12)]}function Pe(fe){return 1+~~(fe.getUTCMonth()/3)}return{format:function(fe){var Oe=T(fe+="",A);return Oe.toString=function(){return fe},Oe},parse:function(fe){var Oe=P(fe+="",!1);return Oe.toString=function(){return fe},Oe},utcFormat:function(fe){var Oe=T(fe+="",C);return Oe.toString=function(){return fe},Oe},utcParse:function(fe){var Oe=P(fe+="",!0);return Oe.toString=function(){return fe},Oe}}}var pj={"-":"",_:" ",0:"0"},It=/^\s*\d+/,c9=/^%/,f9=/[\\^$*+?|[\]().{}]/g;function We(e,t,n){var r=e<0?"-":"",i=(r?-e:e)+"",u=i.length;return r+(u<n?new Array(n-u+1).join(t)+i:i)}function d9(e){return e.replace(f9,"\\$&")}function xl(e){return new RegExp("^(?:"+e.map(d9).join("|")+")","i")}function _l(e){return new Map(e.map((t,n)=>[t.toLowerCase(),n]))}function h9(e,t,n){var r=It.exec(t.slice(n,n+1));return r?(e.w=+r[0],n+r[0].length):-1}function p9(e,t,n){var r=It.exec(t.slice(n,n+1));return r?(e.u=+r[0],n+r[0].length):-1}function v9(e,t,n){var r=It.exec(t.slice(n,n+2));return r?(e.U=+r[0],n+r[0].length):-1}function y9(e,t,n){var r=It.exec(t.slice(n,n+2));return r?(e.V=+r[0],n+r[0].length):-1}function m9(e,t,n){var r=It.exec(t.slice(n,n+2));return r?(e.W=+r[0],n+r[0].length):-1}function vj(e,t,n){var r=It.exec(t.slice(n,n+4));return r?(e.y=+r[0],n+r[0].length):-1}function yj(e,t,n){var r=It.exec(t.slice(n,n+2));return r?(e.y=+r[0]+(+r[0]>68?1900:2e3),n+r[0].length):-1}function g9(e,t,n){var r=/^(Z)|([+-]\d\d)(?::?(\d\d))?/.exec(t.slice(n,n+6));return r?(e.Z=r[1]?0:-(r[2]+(r[3]||"00")),n+r[0].length):-1}function b9(e,t,n){var r=It.exec(t.slice(n,n+1));return r?(e.q=r[0]*3-3,n+r[0].length):-1}function x9(e,t,n){var r=It.exec(t.slice(n,n+2));return r?(e.m=r[0]-1,n+r[0].length):-1}function mj(e,t,n){var r=It.exec(t.slice(n,n+2));return r?(e.d=+r[0],n+r[0].length):-1}function _9(e,t,n){var r=It.exec(t.slice(n,n+3));return r?(e.m=0,e.d=+r[0],n+r[0].length):-1}function gj(e,t,n){var r=It.exec(t.slice(n,n+2));return r?(e.H=+r[0],n+r[0].length):-1}function S9(e,t,n){var r=It.exec(t.slice(n,n+2));return r?(e.M=+r[0],n+r[0].length):-1}function w9(e,t,n){var r=It.exec(t.slice(n,n+2));return r?(e.S=+r[0],n+r[0].length):-1}function O9(e,t,n){var r=It.exec(t.slice(n,n+3));return r?(e.L=+r[0],n+r[0].length):-1}function A9(e,t,n){var r=It.exec(t.slice(n,n+6));return r?(e.L=Math.floor(r[0]/1e3),n+r[0].length):-1}function E9(e,t,n){var r=c9.exec(t.slice(n,n+1));return r?n+r[0].length:-1}function T9(e,t,n){var r=It.exec(t.slice(n));return r?(e.Q=+r[0],n+r[0].length):-1}function M9(e,t,n){var r=It.exec(t.slice(n));return r?(e.s=+r[0],n+r[0].length):-1}function bj(e,t){return We(e.getDate(),t,2)}function j9(e,t){return We(e.getHours(),t,2)}function C9(e,t){return We(e.getHours()%12||12,t,2)}function P9(e,t){return We(1+Is.count(Yr(e),e),t,3)}function hN(e,t){return We(e.getMilliseconds(),t,3)}function R9(e,t){return hN(e,t)+"000"}function N9(e,t){return We(e.getMonth()+1,t,2)}function D9(e,t){return We(e.getMinutes(),t,2)}function k9(e,t){return We(e.getSeconds(),t,2)}function q9(e){var t=e.getDay();return t===0?7:t}function $9(e,t){return We(eh.count(Yr(e)-1,e),t,2)}function pN(e){var t=e.getDay();return t>=4||t===0?Vo(e):Vo.ceil(e)}function B9(e,t){return e=pN(e),We(Vo.count(Yr(e),e)+(Yr(e).getDay()===4),t,2)}function z9(e){return e.getDay()}function L9(e,t){return We(ud.count(Yr(e)-1,e),t,2)}function I9(e,t){return We(e.getFullYear()%100,t,2)}function U9(e,t){return e=pN(e),We(e.getFullYear()%100,t,2)}function H9(e,t){return We(e.getFullYear()%1e4,t,4)}function G9(e,t){var n=e.getDay();return e=n>=4||n===0?Vo(e):Vo.ceil(e),We(e.getFullYear()%1e4,t,4)}function V9(e){var t=e.getTimezoneOffset();return(t>0?"-":(t*=-1,"+"))+We(t/60|0,"0",2)+We(t%60,"0",2)}function xj(e,t){return We(e.getUTCDate(),t,2)}function Z9(e,t){return We(e.getUTCHours(),t,2)}function Y9(e,t){return We(e.getUTCHours()%12||12,t,2)}function X9(e,t){return We(1+Jd.count(Xr(e),e),t,3)}function vN(e,t){return We(e.getUTCMilliseconds(),t,3)}function K9(e,t){return vN(e,t)+"000"}function W9(e,t){return We(e.getUTCMonth()+1,t,2)}function F9(e,t){return We(e.getUTCMinutes(),t,2)}function Q9(e,t){return We(e.getUTCSeconds(),t,2)}function J9(e){var t=e.getUTCDay();return t===0?7:t}function eH(e,t){return We(th.count(Xr(e)-1,e),t,2)}function yN(e){var t=e.getUTCDay();return t>=4||t===0?Zo(e):Zo.ceil(e)}function tH(e,t){return e=yN(e),We(Zo.count(Xr(e),e)+(Xr(e).getUTCDay()===4),t,2)}function nH(e){return e.getUTCDay()}function rH(e,t){return We(ld.count(Xr(e)-1,e),t,2)}function aH(e,t){return We(e.getUTCFullYear()%100,t,2)}function iH(e,t){return e=yN(e),We(e.getUTCFullYear()%100,t,2)}function oH(e,t){return We(e.getUTCFullYear()%1e4,t,4)}function uH(e,t){var n=e.getUTCDay();return e=n>=4||n===0?Zo(e):Zo.ceil(e),We(e.getUTCFullYear()%1e4,t,4)}function lH(){return"+0000"}function _j(){return"%"}function Sj(e){return+e}function wj(e){return Math.floor(+e/1e3)}var wo,mN,gN;sH({dateTime:"%x, %X",date:"%-m/%-d/%Y",time:"%-I:%M:%S %p",periods:["AM","PM"],days:["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],shortDays:["Sun","Mon","Tue","Wed","Thu","Fri","Sat"],months:["January","February","March","April","May","June","July","August","September","October","November","December"],shortMonths:["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"]});function sH(e){return wo=s9(e),mN=wo.format,wo.parse,gN=wo.utcFormat,wo.utcParse,wo}function cH(e){return new Date(e)}function fH(e){return e instanceof Date?+e:+new Date(+e)}function g_(e,t,n,r,i,u,s,c,d,h){var v=a_(),y=v.invert,g=v.domain,b=h(".%L"),S=h(":%S"),_=h("%I:%M"),x=h("%I %p"),E=h("%a %d"),M=h("%b %d"),A=h("%B"),C=h("%Y");function w(T){return(d(T)<T?b:c(T)<T?S:s(T)<T?_:u(T)<T?x:r(T)<T?i(T)<T?E:M:n(T)<T?A:C)(T)}return v.invert=function(T){return new Date(y(T))},v.domain=function(T){return arguments.length?g(Array.from(T,fH)):g().map(cH)},v.ticks=function(T){var P=g();return e(P[0],P[P.length-1],T??10)},v.tickFormat=function(T,P){return P==null?w:h(P)},v.nice=function(T){var P=g();return(!T||typeof T.range!="function")&&(T=t(P[0],P[P.length-1],T??10)),T?g(aN(P,T)):v},v.copy=function(){return Ls(v,g_(e,t,n,r,i,u,s,c,d,h))},v}function dH(){return Un.apply(g_(u9,l9,Yr,y_,eh,Is,p_,d_,di,mN).domain([new Date(2e3,0,1),new Date(2e3,0,2)]),arguments)}function hH(){return Un.apply(g_(i9,o9,Xr,m_,th,Jd,v_,h_,di,gN).domain([Date.UTC(2e3,0,1),Date.UTC(2e3,0,2)]),arguments)}function nh(){var e=0,t=1,n,r,i,u,s=tn,c=!1,d;function h(y){return y==null||isNaN(y=+y)?d:s(i===0?.5:(y=(u(y)-n)*i,c?Math.max(0,Math.min(1,y)):y))}h.domain=function(y){return arguments.length?([e,t]=y,n=u(e=+e),r=u(t=+t),i=n===r?0:1/(r-n),h):[e,t]},h.clamp=function(y){return arguments.length?(c=!!y,h):c},h.interpolator=function(y){return arguments.length?(s=y,h):s};function v(y){return function(g){var b,S;return arguments.length?([b,S]=g,s=y(b,S),h):[s(0),s(1)]}}return h.range=v(pu),h.rangeRound=v(r_),h.unknown=function(y){return arguments.length?(d=y,h):d},function(y){return u=y,n=y(e),r=y(t),i=n===r?0:1/(r-n),h}}function za(e,t){return t.domain(e.domain()).interpolator(e.interpolator()).clamp(e.clamp()).unknown(e.unknown())}function bN(){var e=Ba(nh()(tn));return e.copy=function(){return za(e,bN())},Jr.apply(e,arguments)}function xN(){var e=u_(nh()).domain([1,10]);return e.copy=function(){return za(e,xN()).base(e.base())},Jr.apply(e,arguments)}function _N(){var e=l_(nh());return e.copy=function(){return za(e,_N()).constant(e.constant())},Jr.apply(e,arguments)}function b_(){var e=s_(nh());return e.copy=function(){return za(e,b_()).exponent(e.exponent())},Jr.apply(e,arguments)}function pH(){return b_.apply(null,arguments).exponent(.5)}function SN(){var e=[],t=tn;function n(r){if(r!=null&&!isNaN(r=+r))return t((Bs(e,r,1)-1)/(e.length-1))}return n.domain=function(r){if(!arguments.length)return e.slice();e=[];for(let i of r)i!=null&&!isNaN(i=+i)&&e.push(i);return e.sort(Ca),n},n.interpolator=function(r){return arguments.length?(t=r,n):t},n.range=function(){return e.map((r,i)=>t(i/(e.length-1)))},n.quantiles=function(r){return Array.from({length:r+1},(i,u)=>e5(e,u/r))},n.copy=function(){return SN(t).domain(e)},Jr.apply(n,arguments)}function rh(){var e=0,t=.5,n=1,r=1,i,u,s,c,d,h=tn,v,y=!1,g;function b(_){return isNaN(_=+_)?g:(_=.5+((_=+v(_))-u)*(r*_<r*u?c:d),h(y?Math.max(0,Math.min(1,_)):_))}b.domain=function(_){return arguments.length?([e,t,n]=_,i=v(e=+e),u=v(t=+t),s=v(n=+n),c=i===u?0:.5/(u-i),d=u===s?0:.5/(s-u),r=u<i?-1:1,b):[e,t,n]},b.clamp=function(_){return arguments.length?(y=!!_,b):y},b.interpolator=function(_){return arguments.length?(h=_,b):h};function S(_){return function(x){var E,M,A;return arguments.length?([E,M,A]=x,h=E5(_,[E,M,A]),b):[h(0),h(.5),h(1)]}}return b.range=S(pu),b.rangeRound=S(r_),b.unknown=function(_){return arguments.length?(g=_,b):g},function(_){return v=_,i=_(e),u=_(t),s=_(n),c=i===u?0:.5/(u-i),d=u===s?0:.5/(s-u),r=u<i?-1:1,b}}function wN(){var e=Ba(rh()(tn));return e.copy=function(){return za(e,wN())},Jr.apply(e,arguments)}function ON(){var e=u_(rh()).domain([.1,1,10]);return e.copy=function(){return za(e,ON()).base(e.base())},Jr.apply(e,arguments)}function AN(){var e=l_(rh());return e.copy=function(){return za(e,AN()).constant(e.constant())},Jr.apply(e,arguments)}function x_(){var e=s_(rh());return e.copy=function(){return za(e,x_()).exponent(e.exponent())},Jr.apply(e,arguments)}function vH(){return x_.apply(null,arguments).exponent(.5)}const Oj=Object.freeze(Object.defineProperty({__proto__:null,scaleBand:as,scaleDiverging:wN,scaleDivergingLog:ON,scaleDivergingPow:x_,scaleDivergingSqrt:vH,scaleDivergingSymlog:AN,scaleIdentity:rN,scaleImplicit:E0,scaleLinear:id,scaleLog:iN,scaleOrdinal:e_,scalePoint:Rl,scalePow:c_,scaleQuantile:lN,scaleQuantize:sN,scaleRadial:uN,scaleSequential:bN,scaleSequentialLog:xN,scaleSequentialPow:b_,scaleSequentialQuantile:SN,scaleSequentialSqrt:pH,scaleSequentialSymlog:_N,scaleSqrt:K5,scaleSymlog:oN,scaleThreshold:cN,scaleTime:dH,scaleUtc:hH,tickFormat:nN},Symbol.toStringTag,{value:"Module"}));var sb,Aj;function EN(){if(Aj)return sb;Aj=1;var e=cu();function t(n,r,i){for(var u=-1,s=n.length;++u<s;){var c=n[u],d=r(c);if(d!=null&&(h===void 0?d===d&&!e(d):i(d,h)))var h=d,v=c}return v}return sb=t,sb}var cb,Ej;function yH(){if(Ej)return cb;Ej=1;function e(t,n){return t>n}return cb=e,cb}var fb,Tj;function mH(){if(Tj)return fb;Tj=1;var e=EN(),t=yH(),n=du();function r(i){return i&&i.length?e(i,n,t):void 0}return fb=r,fb}var gH=mH();const ah=lt(gH);var db,Mj;function bH(){if(Mj)return db;Mj=1;function e(t,n){return t<n}return db=e,db}var hb,jj;function xH(){if(jj)return hb;jj=1;var e=EN(),t=bH(),n=du();function r(i){return i&&i.length?e(i,n,t):void 0}return hb=r,hb}var _H=xH();const ih=lt(_H);var pb,Cj;function SH(){if(Cj)return pb;Cj=1;var e=qx(),t=$a(),n=kR(),r=fn();function i(u,s){var c=r(u)?e:n;return c(u,t(s,3))}return pb=i,pb}var vb,Pj;function wH(){if(Pj)return vb;Pj=1;var e=NR(),t=SH();function n(r,i){return e(t(r,i),1)}return vb=n,vb}var OH=wH();const AH=lt(OH);var yb,Rj;function EH(){if(Rj)return yb;Rj=1;var e=Xx();function t(n,r){return e(n,r)}return yb=t,yb}var TH=EH();const oh=lt(TH);var vu=1e9,MH={precision:20,rounding:4,toExpNeg:-7,toExpPos:21,LN10:"2.302585092994045684017991454684364207601101488628772976033327900967572609677352480235997205089598298341967784042286"},S_,vt=!0,In="[DecimalError] ",mi=In+"Invalid argument: ",__=In+"Exponent out of range: ",yu=Math.floor,si=Math.pow,jH=/^(\d+(\.\d*)?|\.\d+)(e[+-]?\d+)?$/i,wn,Lt=1e7,ht=7,TN=9007199254740991,sd=yu(TN/ht),ve={};ve.absoluteValue=ve.abs=function(){var e=new this.constructor(this);return e.s&&(e.s=1),e};ve.comparedTo=ve.cmp=function(e){var t,n,r,i,u=this;if(e=new u.constructor(e),u.s!==e.s)return u.s||-e.s;if(u.e!==e.e)return u.e>e.e^u.s<0?1:-1;for(r=u.d.length,i=e.d.length,t=0,n=r<i?r:i;t<n;++t)if(u.d[t]!==e.d[t])return u.d[t]>e.d[t]^u.s<0?1:-1;return r===i?0:r>i^u.s<0?1:-1};ve.decimalPlaces=ve.dp=function(){var e=this,t=e.d.length-1,n=(t-e.e)*ht;if(t=e.d[t],t)for(;t%10==0;t/=10)n--;return n<0?0:n};ve.dividedBy=ve.div=function(e){return Gr(this,new this.constructor(e))};ve.dividedToIntegerBy=ve.idiv=function(e){var t=this,n=t.constructor;return ut(Gr(t,new n(e),0,1),n.precision)};ve.equals=ve.eq=function(e){return!this.cmp(e)};ve.exponent=function(){return jt(this)};ve.greaterThan=ve.gt=function(e){return this.cmp(e)>0};ve.greaterThanOrEqualTo=ve.gte=function(e){return this.cmp(e)>=0};ve.isInteger=ve.isint=function(){return this.e>this.d.length-2};ve.isNegative=ve.isneg=function(){return this.s<0};ve.isPositive=ve.ispos=function(){return this.s>0};ve.isZero=function(){return this.s===0};ve.lessThan=ve.lt=function(e){return this.cmp(e)<0};ve.lessThanOrEqualTo=ve.lte=function(e){return this.cmp(e)<1};ve.logarithm=ve.log=function(e){var t,n=this,r=n.constructor,i=r.precision,u=i+5;if(e===void 0)e=new r(10);else if(e=new r(e),e.s<1||e.eq(wn))throw Error(In+"NaN");if(n.s<1)throw Error(In+(n.s?"NaN":"-Infinity"));return n.eq(wn)?new r(0):(vt=!1,t=Gr(ss(n,u),ss(e,u),u),vt=!0,ut(t,i))};ve.minus=ve.sub=function(e){var t=this;return e=new t.constructor(e),t.s==e.s?CN(t,e):MN(t,(e.s=-e.s,e))};ve.modulo=ve.mod=function(e){var t,n=this,r=n.constructor,i=r.precision;if(e=new r(e),!e.s)throw Error(In+"NaN");return n.s?(vt=!1,t=Gr(n,e,0,1).times(e),vt=!0,n.minus(t)):ut(new r(n),i)};ve.naturalExponential=ve.exp=function(){return jN(this)};ve.naturalLogarithm=ve.ln=function(){return ss(this)};ve.negated=ve.neg=function(){var e=new this.constructor(this);return e.s=-e.s||0,e};ve.plus=ve.add=function(e){var t=this;return e=new t.constructor(e),t.s==e.s?MN(t,e):CN(t,(e.s=-e.s,e))};ve.precision=ve.sd=function(e){var t,n,r,i=this;if(e!==void 0&&e!==!!e&&e!==1&&e!==0)throw Error(mi+e);if(t=jt(i)+1,r=i.d.length-1,n=r*ht+1,r=i.d[r],r){for(;r%10==0;r/=10)n--;for(r=i.d[0];r>=10;r/=10)n++}return e&&t>n?t:n};ve.squareRoot=ve.sqrt=function(){var e,t,n,r,i,u,s,c=this,d=c.constructor;if(c.s<1){if(!c.s)return new d(0);throw Error(In+"NaN")}for(e=jt(c),vt=!1,i=Math.sqrt(+c),i==0||i==1/0?(t=dr(c.d),(t.length+e)%2==0&&(t+="0"),i=Math.sqrt(t),e=yu((e+1)/2)-(e<0||e%2),i==1/0?t="5e"+e:(t=i.toExponential(),t=t.slice(0,t.indexOf("e")+1)+e),r=new d(t)):r=new d(i.toString()),n=d.precision,i=s=n+3;;)if(u=r,r=u.plus(Gr(c,u,s+2)).times(.5),dr(u.d).slice(0,s)===(t=dr(r.d)).slice(0,s)){if(t=t.slice(s-3,s+1),i==s&&t=="4999"){if(ut(u,n+1,0),u.times(u).eq(c)){r=u;break}}else if(t!="9999")break;s+=4}return vt=!0,ut(r,n)};ve.times=ve.mul=function(e){var t,n,r,i,u,s,c,d,h,v=this,y=v.constructor,g=v.d,b=(e=new y(e)).d;if(!v.s||!e.s)return new y(0);for(e.s*=v.s,n=v.e+e.e,d=g.length,h=b.length,d<h&&(u=g,g=b,b=u,s=d,d=h,h=s),u=[],s=d+h,r=s;r--;)u.push(0);for(r=h;--r>=0;){for(t=0,i=d+r;i>r;)c=u[i]+b[r]*g[i-r-1]+t,u[i--]=c%Lt|0,t=c/Lt|0;u[i]=(u[i]+t)%Lt|0}for(;!u[--s];)u.pop();return t?++n:u.shift(),e.d=u,e.e=n,vt?ut(e,y.precision):e};ve.toDecimalPlaces=ve.todp=function(e,t){var n=this,r=n.constructor;return n=new r(n),e===void 0?n:(gr(e,0,vu),t===void 0?t=r.rounding:gr(t,0,8),ut(n,e+jt(n)+1,t))};ve.toExponential=function(e,t){var n,r=this,i=r.constructor;return e===void 0?n=Oi(r,!0):(gr(e,0,vu),t===void 0?t=i.rounding:gr(t,0,8),r=ut(new i(r),e+1,t),n=Oi(r,!0,e+1)),n};ve.toFixed=function(e,t){var n,r,i=this,u=i.constructor;return e===void 0?Oi(i):(gr(e,0,vu),t===void 0?t=u.rounding:gr(t,0,8),r=ut(new u(i),e+jt(i)+1,t),n=Oi(r.abs(),!1,e+jt(r)+1),i.isneg()&&!i.isZero()?"-"+n:n)};ve.toInteger=ve.toint=function(){var e=this,t=e.constructor;return ut(new t(e),jt(e)+1,t.rounding)};ve.toNumber=function(){return+this};ve.toPower=ve.pow=function(e){var t,n,r,i,u,s,c=this,d=c.constructor,h=12,v=+(e=new d(e));if(!e.s)return new d(wn);if(c=new d(c),!c.s){if(e.s<1)throw Error(In+"Infinity");return c}if(c.eq(wn))return c;if(r=d.precision,e.eq(wn))return ut(c,r);if(t=e.e,n=e.d.length-1,s=t>=n,u=c.s,s){if((n=v<0?-v:v)<=TN){for(i=new d(wn),t=Math.ceil(r/ht+4),vt=!1;n%2&&(i=i.times(c),Dj(i.d,t)),n=yu(n/2),n!==0;)c=c.times(c),Dj(c.d,t);return vt=!0,e.s<0?new d(wn).div(i):ut(i,r)}}else if(u<0)throw Error(In+"NaN");return u=u<0&&e.d[Math.max(t,n)]&1?-1:1,c.s=1,vt=!1,i=e.times(ss(c,r+h)),vt=!0,i=jN(i),i.s=u,i};ve.toPrecision=function(e,t){var n,r,i=this,u=i.constructor;return e===void 0?(n=jt(i),r=Oi(i,n<=u.toExpNeg||n>=u.toExpPos)):(gr(e,1,vu),t===void 0?t=u.rounding:gr(t,0,8),i=ut(new u(i),e,t),n=jt(i),r=Oi(i,e<=n||n<=u.toExpNeg,e)),r};ve.toSignificantDigits=ve.tosd=function(e,t){var n=this,r=n.constructor;return e===void 0?(e=r.precision,t=r.rounding):(gr(e,1,vu),t===void 0?t=r.rounding:gr(t,0,8)),ut(new r(n),e,t)};ve.toString=ve.valueOf=ve.val=ve.toJSON=ve[Symbol.for("nodejs.util.inspect.custom")]=function(){var e=this,t=jt(e),n=e.constructor;return Oi(e,t<=n.toExpNeg||t>=n.toExpPos)};function MN(e,t){var n,r,i,u,s,c,d,h,v=e.constructor,y=v.precision;if(!e.s||!t.s)return t.s||(t=new v(e)),vt?ut(t,y):t;if(d=e.d,h=t.d,s=e.e,i=t.e,d=d.slice(),u=s-i,u){for(u<0?(r=d,u=-u,c=h.length):(r=h,i=s,c=d.length),s=Math.ceil(y/ht),c=s>c?s+1:c+1,u>c&&(u=c,r.length=1),r.reverse();u--;)r.push(0);r.reverse()}for(c=d.length,u=h.length,c-u<0&&(u=c,r=h,h=d,d=r),n=0;u;)n=(d[--u]=d[u]+h[u]+n)/Lt|0,d[u]%=Lt;for(n&&(d.unshift(n),++i),c=d.length;d[--c]==0;)d.pop();return t.d=d,t.e=i,vt?ut(t,y):t}function gr(e,t,n){if(e!==~~e||e<t||e>n)throw Error(mi+e)}function dr(e){var t,n,r,i=e.length-1,u="",s=e[0];if(i>0){for(u+=s,t=1;t<i;t++)r=e[t]+"",n=ht-r.length,n&&(u+=Ea(n)),u+=r;s=e[t],r=s+"",n=ht-r.length,n&&(u+=Ea(n))}else if(s===0)return"0";for(;s%10===0;)s/=10;return u+s}var Gr=function(){function e(r,i){var u,s=0,c=r.length;for(r=r.slice();c--;)u=r[c]*i+s,r[c]=u%Lt|0,s=u/Lt|0;return s&&r.unshift(s),r}function t(r,i,u,s){var c,d;if(u!=s)d=u>s?1:-1;else for(c=d=0;c<u;c++)if(r[c]!=i[c]){d=r[c]>i[c]?1:-1;break}return d}function n(r,i,u){for(var s=0;u--;)r[u]-=s,s=r[u]<i[u]?1:0,r[u]=s*Lt+r[u]-i[u];for(;!r[0]&&r.length>1;)r.shift()}return function(r,i,u,s){var c,d,h,v,y,g,b,S,_,x,E,M,A,C,w,T,P,R,$=r.constructor,J=r.s==i.s?1:-1,B=r.d,G=i.d;if(!r.s)return new $(r);if(!i.s)throw Error(In+"Division by zero");for(d=r.e-i.e,P=G.length,w=B.length,b=new $(J),S=b.d=[],h=0;G[h]==(B[h]||0);)++h;if(G[h]>(B[h]||0)&&--d,u==null?M=u=$.precision:s?M=u+(jt(r)-jt(i))+1:M=u,M<0)return new $(0);if(M=M/ht+2|0,h=0,P==1)for(v=0,G=G[0],M++;(h<w||v)&&M--;h++)A=v*Lt+(B[h]||0),S[h]=A/G|0,v=A%G|0;else{for(v=Lt/(G[0]+1)|0,v>1&&(G=e(G,v),B=e(B,v),P=G.length,w=B.length),C=P,_=B.slice(0,P),x=_.length;x<P;)_[x++]=0;R=G.slice(),R.unshift(0),T=G[0],G[1]>=Lt/2&&++T;do v=0,c=t(G,_,P,x),c<0?(E=_[0],P!=x&&(E=E*Lt+(_[1]||0)),v=E/T|0,v>1?(v>=Lt&&(v=Lt-1),y=e(G,v),g=y.length,x=_.length,c=t(y,_,g,x),c==1&&(v--,n(y,P<g?R:G,g))):(v==0&&(c=v=1),y=G.slice()),g=y.length,g<x&&y.unshift(0),n(_,y,x),c==-1&&(x=_.length,c=t(G,_,P,x),c<1&&(v++,n(_,P<x?R:G,x))),x=_.length):c===0&&(v++,_=[0]),S[h++]=v,c&&_[0]?_[x++]=B[C]||0:(_=[B[C]],x=1);while((C++<w||_[0]!==void 0)&&M--)}return S[0]||S.shift(),b.e=d,ut(b,s?u+jt(b)+1:u)}}();function jN(e,t){var n,r,i,u,s,c,d=0,h=0,v=e.constructor,y=v.precision;if(jt(e)>16)throw Error(__+jt(e));if(!e.s)return new v(wn);for(vt=!1,c=y,s=new v(.03125);e.abs().gte(.1);)e=e.times(s),h+=5;for(r=Math.log(si(2,h))/Math.LN10*2+5|0,c+=r,n=i=u=new v(wn),v.precision=c;;){if(i=ut(i.times(e),c),n=n.times(++d),s=u.plus(Gr(i,n,c)),dr(s.d).slice(0,c)===dr(u.d).slice(0,c)){for(;h--;)u=ut(u.times(u),c);return v.precision=y,t==null?(vt=!0,ut(u,y)):u}u=s}}function jt(e){for(var t=e.e*ht,n=e.d[0];n>=10;n/=10)t++;return t}function mb(e,t,n){if(t>e.LN10.sd())throw vt=!0,n&&(e.precision=n),Error(In+"LN10 precision limit exceeded");return ut(new e(e.LN10),t)}function Ea(e){for(var t="";e--;)t+="0";return t}function ss(e,t){var n,r,i,u,s,c,d,h,v,y=1,g=10,b=e,S=b.d,_=b.constructor,x=_.precision;if(b.s<1)throw Error(In+(b.s?"NaN":"-Infinity"));if(b.eq(wn))return new _(0);if(t==null?(vt=!1,h=x):h=t,b.eq(10))return t==null&&(vt=!0),mb(_,h);if(h+=g,_.precision=h,n=dr(S),r=n.charAt(0),u=jt(b),Math.abs(u)<15e14){for(;r<7&&r!=1||r==1&&n.charAt(1)>3;)b=b.times(e),n=dr(b.d),r=n.charAt(0),y++;u=jt(b),r>1?(b=new _("0."+n),u++):b=new _(r+"."+n.slice(1))}else return d=mb(_,h+2,x).times(u+""),b=ss(new _(r+"."+n.slice(1)),h-g).plus(d),_.precision=x,t==null?(vt=!0,ut(b,x)):b;for(c=s=b=Gr(b.minus(wn),b.plus(wn),h),v=ut(b.times(b),h),i=3;;){if(s=ut(s.times(v),h),d=c.plus(Gr(s,new _(i),h)),dr(d.d).slice(0,h)===dr(c.d).slice(0,h))return c=c.times(2),u!==0&&(c=c.plus(mb(_,h+2,x).times(u+""))),c=Gr(c,new _(y),h),_.precision=x,t==null?(vt=!0,ut(c,x)):c;c=d,i+=2}}function Nj(e,t){var n,r,i;for((n=t.indexOf("."))>-1&&(t=t.replace(".","")),(r=t.search(/e/i))>0?(n<0&&(n=r),n+=+t.slice(r+1),t=t.substring(0,r)):n<0&&(n=t.length),r=0;t.charCodeAt(r)===48;)++r;for(i=t.length;t.charCodeAt(i-1)===48;)--i;if(t=t.slice(r,i),t){if(i-=r,n=n-r-1,e.e=yu(n/ht),e.d=[],r=(n+1)%ht,n<0&&(r+=ht),r<i){for(r&&e.d.push(+t.slice(0,r)),i-=ht;r<i;)e.d.push(+t.slice(r,r+=ht));t=t.slice(r),r=ht-t.length}else r-=i;for(;r--;)t+="0";if(e.d.push(+t),vt&&(e.e>sd||e.e<-sd))throw Error(__+n)}else e.s=0,e.e=0,e.d=[0];return e}function ut(e,t,n){var r,i,u,s,c,d,h,v,y=e.d;for(s=1,u=y[0];u>=10;u/=10)s++;if(r=t-s,r<0)r+=ht,i=t,h=y[v=0];else{if(v=Math.ceil((r+1)/ht),u=y.length,v>=u)return e;for(h=u=y[v],s=1;u>=10;u/=10)s++;r%=ht,i=r-ht+s}if(n!==void 0&&(u=si(10,s-i-1),c=h/u%10|0,d=t<0||y[v+1]!==void 0||h%u,d=n<4?(c||d)&&(n==0||n==(e.s<0?3:2)):c>5||c==5&&(n==4||d||n==6&&(r>0?i>0?h/si(10,s-i):0:y[v-1])%10&1||n==(e.s<0?8:7))),t<1||!y[0])return d?(u=jt(e),y.length=1,t=t-u-1,y[0]=si(10,(ht-t%ht)%ht),e.e=yu(-t/ht)||0):(y.length=1,y[0]=e.e=e.s=0),e;if(r==0?(y.length=v,u=1,v--):(y.length=v+1,u=si(10,ht-r),y[v]=i>0?(h/si(10,s-i)%si(10,i)|0)*u:0),d)for(;;)if(v==0){(y[0]+=u)==Lt&&(y[0]=1,++e.e);break}else{if(y[v]+=u,y[v]!=Lt)break;y[v--]=0,u=1}for(r=y.length;y[--r]===0;)y.pop();if(vt&&(e.e>sd||e.e<-sd))throw Error(__+jt(e));return e}function CN(e,t){var n,r,i,u,s,c,d,h,v,y,g=e.constructor,b=g.precision;if(!e.s||!t.s)return t.s?t.s=-t.s:t=new g(e),vt?ut(t,b):t;if(d=e.d,y=t.d,r=t.e,h=e.e,d=d.slice(),s=h-r,s){for(v=s<0,v?(n=d,s=-s,c=y.length):(n=y,r=h,c=d.length),i=Math.max(Math.ceil(b/ht),c)+2,s>i&&(s=i,n.length=1),n.reverse(),i=s;i--;)n.push(0);n.reverse()}else{for(i=d.length,c=y.length,v=i<c,v&&(c=i),i=0;i<c;i++)if(d[i]!=y[i]){v=d[i]<y[i];break}s=0}for(v&&(n=d,d=y,y=n,t.s=-t.s),c=d.length,i=y.length-c;i>0;--i)d[c++]=0;for(i=y.length;i>s;){if(d[--i]<y[i]){for(u=i;u&&d[--u]===0;)d[u]=Lt-1;--d[u],d[i]+=Lt}d[i]-=y[i]}for(;d[--c]===0;)d.pop();for(;d[0]===0;d.shift())--r;return d[0]?(t.d=d,t.e=r,vt?ut(t,b):t):new g(0)}function Oi(e,t,n){var r,i=jt(e),u=dr(e.d),s=u.length;return t?(n&&(r=n-s)>0?u=u.charAt(0)+"."+u.slice(1)+Ea(r):s>1&&(u=u.charAt(0)+"."+u.slice(1)),u=u+(i<0?"e":"e+")+i):i<0?(u="0."+Ea(-i-1)+u,n&&(r=n-s)>0&&(u+=Ea(r))):i>=s?(u+=Ea(i+1-s),n&&(r=n-i-1)>0&&(u=u+"."+Ea(r))):((r=i+1)<s&&(u=u.slice(0,r)+"."+u.slice(r)),n&&(r=n-s)>0&&(i+1===s&&(u+="."),u+=Ea(r))),e.s<0?"-"+u:u}function Dj(e,t){if(e.length>t)return e.length=t,!0}function PN(e){var t,n,r;function i(u){var s=this;if(!(s instanceof i))return new i(u);if(s.constructor=i,u instanceof i){s.s=u.s,s.e=u.e,s.d=(u=u.d)?u.slice():u;return}if(typeof u=="number"){if(u*0!==0)throw Error(mi+u);if(u>0)s.s=1;else if(u<0)u=-u,s.s=-1;else{s.s=0,s.e=0,s.d=[0];return}if(u===~~u&&u<1e7){s.e=0,s.d=[u];return}return Nj(s,u.toString())}else if(typeof u!="string")throw Error(mi+u);if(u.charCodeAt(0)===45?(u=u.slice(1),s.s=-1):s.s=1,jH.test(u))Nj(s,u);else throw Error(mi+u)}if(i.prototype=ve,i.ROUND_UP=0,i.ROUND_DOWN=1,i.ROUND_CEIL=2,i.ROUND_FLOOR=3,i.ROUND_HALF_UP=4,i.ROUND_HALF_DOWN=5,i.ROUND_HALF_EVEN=6,i.ROUND_HALF_CEIL=7,i.ROUND_HALF_FLOOR=8,i.clone=PN,i.config=i.set=CH,e===void 0&&(e={}),e)for(r=["precision","rounding","toExpNeg","toExpPos","LN10"],t=0;t<r.length;)e.hasOwnProperty(n=r[t++])||(e[n]=this[n]);return i.config(e),i}function CH(e){if(!e||typeof e!="object")throw Error(In+"Object expected");var t,n,r,i=["precision",1,vu,"rounding",0,8,"toExpNeg",-1/0,0,"toExpPos",0,1/0];for(t=0;t<i.length;t+=3)if((r=e[n=i[t]])!==void 0)if(yu(r)===r&&r>=i[t+1]&&r<=i[t+2])this[n]=r;else throw Error(mi+n+": "+r);if((r=e[n="LN10"])!==void 0)if(r==Math.LN10)this[n]=new this(r);else throw Error(mi+n+": "+r);return this}var S_=PN(MH);wn=new S_(1);const it=S_;function PH(e){return kH(e)||DH(e)||NH(e)||RH()}function RH(){throw new TypeError(`Invalid attempt to spread non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function NH(e,t){if(e){if(typeof e=="string")return C0(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);if(n==="Object"&&e.constructor&&(n=e.constructor.name),n==="Map"||n==="Set")return Array.from(e);if(n==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return C0(e,t)}}function DH(e){if(typeof Symbol<"u"&&Symbol.iterator in Object(e))return Array.from(e)}function kH(e){if(Array.isArray(e))return C0(e)}function C0(e,t){(t==null||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}var qH=function(t){return t},RN={},NN=function(t){return t===RN},kj=function(t){return function n(){return arguments.length===0||arguments.length===1&&NN(arguments.length<=0?void 0:arguments[0])?n:t.apply(void 0,arguments)}},$H=function e(t,n){return t===1?n:kj(function(){for(var r=arguments.length,i=new Array(r),u=0;u<r;u++)i[u]=arguments[u];var s=i.filter(function(c){return c!==RN}).length;return s>=t?n.apply(void 0,i):e(t-s,kj(function(){for(var c=arguments.length,d=new Array(c),h=0;h<c;h++)d[h]=arguments[h];var v=i.map(function(y){return NN(y)?d.shift():y});return n.apply(void 0,PH(v).concat(d))}))})},uh=function(t){return $H(t.length,t)},P0=function(t,n){for(var r=[],i=t;i<n;++i)r[i-t]=i;return r},BH=uh(function(e,t){return Array.isArray(t)?t.map(e):Object.keys(t).map(function(n){return t[n]}).map(e)}),zH=function(){for(var t=arguments.length,n=new Array(t),r=0;r<t;r++)n[r]=arguments[r];if(!n.length)return qH;var i=n.reverse(),u=i[0],s=i.slice(1);return function(){return s.reduce(function(c,d){return d(c)},u.apply(void 0,arguments))}},R0=function(t){return Array.isArray(t)?t.reverse():t.split("").reverse.join("")},DN=function(t){var n=null,r=null;return function(){for(var i=arguments.length,u=new Array(i),s=0;s<i;s++)u[s]=arguments[s];return n&&u.every(function(c,d){return c===n[d]})||(n=u,r=t.apply(void 0,u)),r}};function LH(e){var t;return e===0?t=1:t=Math.floor(new it(e).abs().log(10).toNumber())+1,t}function IH(e,t,n){for(var r=new it(e),i=0,u=[];r.lt(t)&&i<1e5;)u.push(r.toNumber()),r=r.add(n),i++;return u}var UH=uh(function(e,t,n){var r=+e,i=+t;return r+n*(i-r)}),HH=uh(function(e,t,n){var r=t-+e;return r=r||1/0,(n-e)/r}),GH=uh(function(e,t,n){var r=t-+e;return r=r||1/0,Math.max(0,Math.min(1,(n-e)/r))});const lh={rangeStep:IH,getDigitCount:LH,interpolateNumber:UH,uninterpolateNumber:HH,uninterpolateTruncation:GH};function N0(e){return YH(e)||ZH(e)||kN(e)||VH()}function VH(){throw new TypeError(`Invalid attempt to spread non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function ZH(e){if(typeof Symbol<"u"&&Symbol.iterator in Object(e))return Array.from(e)}function YH(e){if(Array.isArray(e))return D0(e)}function cs(e,t){return WH(e)||KH(e,t)||kN(e,t)||XH()}function XH(){throw new TypeError(`Invalid attempt to destructure non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function kN(e,t){if(e){if(typeof e=="string")return D0(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);if(n==="Object"&&e.constructor&&(n=e.constructor.name),n==="Map"||n==="Set")return Array.from(e);if(n==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return D0(e,t)}}function D0(e,t){(t==null||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}function KH(e,t){if(!(typeof Symbol>"u"||!(Symbol.iterator in Object(e)))){var n=[],r=!0,i=!1,u=void 0;try{for(var s=e[Symbol.iterator](),c;!(r=(c=s.next()).done)&&(n.push(c.value),!(t&&n.length===t));r=!0);}catch(d){i=!0,u=d}finally{try{!r&&s.return!=null&&s.return()}finally{if(i)throw u}}return n}}function WH(e){if(Array.isArray(e))return e}function qN(e){var t=cs(e,2),n=t[0],r=t[1],i=n,u=r;return n>r&&(i=r,u=n),[i,u]}function $N(e,t,n){if(e.lte(0))return new it(0);var r=lh.getDigitCount(e.toNumber()),i=new it(10).pow(r),u=e.div(i),s=r!==1?.05:.1,c=new it(Math.ceil(u.div(s).toNumber())).add(n).mul(s),d=c.mul(i);return t?d:new it(Math.ceil(d))}function FH(e,t,n){var r=1,i=new it(e);if(!i.isint()&&n){var u=Math.abs(e);u<1?(r=new it(10).pow(lh.getDigitCount(e)-1),i=new it(Math.floor(i.div(r).toNumber())).mul(r)):u>1&&(i=new it(Math.floor(e)))}else e===0?i=new it(Math.floor((t-1)/2)):n||(i=new it(Math.floor(e)));var s=Math.floor((t-1)/2),c=zH(BH(function(d){return i.add(new it(d-s).mul(r)).toNumber()}),P0);return c(0,t)}function BN(e,t,n,r){var i=arguments.length>4&&arguments[4]!==void 0?arguments[4]:0;if(!Number.isFinite((t-e)/(n-1)))return{step:new it(0),tickMin:new it(0),tickMax:new it(0)};var u=$N(new it(t).sub(e).div(n-1),r,i),s;e<=0&&t>=0?s=new it(0):(s=new it(e).add(t).div(2),s=s.sub(new it(s).mod(u)));var c=Math.ceil(s.sub(e).div(u).toNumber()),d=Math.ceil(new it(t).sub(s).div(u).toNumber()),h=c+d+1;return h>n?BN(e,t,n,r,i+1):(h<n&&(d=t>0?d+(n-h):d,c=t>0?c:c+(n-h)),{step:u,tickMin:s.sub(new it(c).mul(u)),tickMax:s.add(new it(d).mul(u))})}function QH(e){var t=cs(e,2),n=t[0],r=t[1],i=arguments.length>1&&arguments[1]!==void 0?arguments[1]:6,u=arguments.length>2&&arguments[2]!==void 0?arguments[2]:!0,s=Math.max(i,2),c=qN([n,r]),d=cs(c,2),h=d[0],v=d[1];if(h===-1/0||v===1/0){var y=v===1/0?[h].concat(N0(P0(0,i-1).map(function(){return 1/0}))):[].concat(N0(P0(0,i-1).map(function(){return-1/0})),[v]);return n>r?R0(y):y}if(h===v)return FH(h,i,u);var g=BN(h,v,s,u),b=g.step,S=g.tickMin,_=g.tickMax,x=lh.rangeStep(S,_.add(new it(.1).mul(b)),b);return n>r?R0(x):x}function JH(e,t){var n=cs(e,2),r=n[0],i=n[1],u=arguments.length>2&&arguments[2]!==void 0?arguments[2]:!0,s=qN([r,i]),c=cs(s,2),d=c[0],h=c[1];if(d===-1/0||h===1/0)return[r,i];if(d===h)return[d];var v=Math.max(t,2),y=$N(new it(h).sub(d).div(v-1),u,0),g=[].concat(N0(lh.rangeStep(new it(d),new it(h).sub(new it(.99).mul(y)),y)),[h]);return r>i?R0(g):g}var e7=DN(QH),t7=DN(JH),n7="Invariant failed";function Ai(e,t){throw new Error(n7)}var r7=["offset","layout","width","dataKey","data","dataPointFormatter","xAxis","yAxis"];function Yo(e){"@babel/helpers - typeof";return Yo=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},Yo(e)}function cd(){return cd=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},cd.apply(this,arguments)}function a7(e,t){return l7(e)||u7(e,t)||o7(e,t)||i7()}function i7(){throw new TypeError(`Invalid attempt to destructure non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function o7(e,t){if(e){if(typeof e=="string")return qj(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);if(n==="Object"&&e.constructor&&(n=e.constructor.name),n==="Map"||n==="Set")return Array.from(e);if(n==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return qj(e,t)}}function qj(e,t){(t==null||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}function u7(e,t){var n=e==null?null:typeof Symbol<"u"&&e[Symbol.iterator]||e["@@iterator"];if(n!=null){var r,i,u,s,c=[],d=!0,h=!1;try{if(u=(n=n.call(e)).next,t!==0)for(;!(d=(r=u.call(n)).done)&&(c.push(r.value),c.length!==t);d=!0);}catch(v){h=!0,i=v}finally{try{if(!d&&n.return!=null&&(s=n.return(),Object(s)!==s))return}finally{if(h)throw i}}return c}}function l7(e){if(Array.isArray(e))return e}function s7(e,t){if(e==null)return{};var n=c7(e,t),r,i;if(Object.getOwnPropertySymbols){var u=Object.getOwnPropertySymbols(e);for(i=0;i<u.length;i++)r=u[i],!(t.indexOf(r)>=0)&&Object.prototype.propertyIsEnumerable.call(e,r)&&(n[r]=e[r])}return n}function c7(e,t){if(e==null)return{};var n={};for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){if(t.indexOf(r)>=0)continue;n[r]=e[r]}return n}function f7(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function d7(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,IN(r.key),r)}}function h7(e,t,n){return t&&d7(e.prototype,t),Object.defineProperty(e,"prototype",{writable:!1}),e}function p7(e,t,n){return t=fd(t),v7(e,zN()?Reflect.construct(t,n||[],fd(e).constructor):t.apply(e,n))}function v7(e,t){if(t&&(Yo(t)==="object"||typeof t=="function"))return t;if(t!==void 0)throw new TypeError("Derived constructors may only return object or undefined");return y7(e)}function y7(e){if(e===void 0)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function zN(){try{var e=!Boolean.prototype.valueOf.call(Reflect.construct(Boolean,[],function(){}))}catch{}return(zN=function(){return!!e})()}function fd(e){return fd=Object.setPrototypeOf?Object.getPrototypeOf.bind():function(n){return n.__proto__||Object.getPrototypeOf(n)},fd(e)}function m7(e,t){if(typeof t!="function"&&t!==null)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),Object.defineProperty(e,"prototype",{writable:!1}),t&&k0(e,t)}function k0(e,t){return k0=Object.setPrototypeOf?Object.setPrototypeOf.bind():function(r,i){return r.__proto__=i,r},k0(e,t)}function LN(e,t,n){return t=IN(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function IN(e){var t=g7(e,"string");return Yo(t)=="symbol"?t:t+""}function g7(e,t){if(Yo(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(Yo(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return String(e)}var Us=function(e){function t(){return f7(this,t),p7(this,t,arguments)}return m7(t,e),h7(t,[{key:"render",value:function(){var r=this.props,i=r.offset,u=r.layout,s=r.width,c=r.dataKey,d=r.data,h=r.dataPointFormatter,v=r.xAxis,y=r.yAxis,g=s7(r,r7),b=Ye(g,!1);this.props.direction==="x"&&v.type!=="number"&&Ai();var S=d.map(function(_){var x=h(_,c),E=x.x,M=x.y,A=x.value,C=x.errorVal;if(!C)return null;var w=[],T,P;if(Array.isArray(C)){var R=a7(C,2);T=R[0],P=R[1]}else T=P=C;if(u==="vertical"){var $=v.scale,J=M+i,B=J+s,G=J-s,W=$(A-T),ne=$(A+P);w.push({x1:ne,y1:B,x2:ne,y2:G}),w.push({x1:W,y1:J,x2:ne,y2:J}),w.push({x1:W,y1:B,x2:W,y2:G})}else if(u==="horizontal"){var Q=y.scale,D=E+i,H=D-s,U=D+s,ue=Q(A-T),N=Q(A+P);w.push({x1:H,y1:N,x2:U,y2:N}),w.push({x1:D,y1:ue,x2:D,y2:N}),w.push({x1:H,y1:ue,x2:U,y2:ue})}return L.createElement(gt,cd({className:"recharts-errorBar",key:"bar-".concat(w.map(function(X){return"".concat(X.x1,"-").concat(X.x2,"-").concat(X.y1,"-").concat(X.y2)}))},b),w.map(function(X){return L.createElement("line",cd({},X,{key:"line-".concat(X.x1,"-").concat(X.x2,"-").concat(X.y1,"-").concat(X.y2)}))}))});return L.createElement(gt,{className:"recharts-errorBars"},S)}}])}(L.Component);LN(Us,"defaultProps",{stroke:"black",strokeWidth:1.5,width:5,offset:0,layout:"horizontal"});LN(Us,"displayName","ErrorBar");function fs(e){"@babel/helpers - typeof";return fs=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},fs(e)}function $j(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function ui(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?$j(Object(n),!0).forEach(function(r){b7(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):$j(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function b7(e,t,n){return t=x7(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function x7(e){var t=_7(e,"string");return fs(t)=="symbol"?t:t+""}function _7(e,t){if(fs(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(fs(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return(t==="string"?String:Number)(e)}var UN=function(t){var n=t.children,r=t.formattedGraphicalItems,i=t.legendWidth,u=t.legendContent,s=Sn(n,No);if(!s)return null;var c=No.defaultProps,d=c!==void 0?ui(ui({},c),s.props):{},h;return s.props&&s.props.payload?h=s.props&&s.props.payload:u==="children"?h=(r||[]).reduce(function(v,y){var g=y.item,b=y.props,S=b.sectors||b.data||[];return v.concat(S.map(function(_){return{type:s.props.iconType||g.props.legendType,value:_.name,color:_.fill,payload:_}}))},[]):h=(r||[]).map(function(v){var y=v.item,g=y.type.defaultProps,b=g!==void 0?ui(ui({},g),y.props):{},S=b.dataKey,_=b.name,x=b.legendType,E=b.hide;return{inactive:E,dataKey:S,type:d.iconType||x||"square",color:w_(y),value:_||S,payload:b}}),ui(ui(ui({},d),No.getWithHeight(s,i)),{},{payload:h,item:s})};function ds(e){"@babel/helpers - typeof";return ds=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},ds(e)}function Bj(e){return A7(e)||O7(e)||w7(e)||S7()}function S7(){throw new TypeError(`Invalid attempt to spread non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function w7(e,t){if(e){if(typeof e=="string")return q0(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);if(n==="Object"&&e.constructor&&(n=e.constructor.name),n==="Map"||n==="Set")return Array.from(e);if(n==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return q0(e,t)}}function O7(e){if(typeof Symbol<"u"&&e[Symbol.iterator]!=null||e["@@iterator"]!=null)return Array.from(e)}function A7(e){if(Array.isArray(e))return q0(e)}function q0(e,t){(t==null||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}function zj(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function _t(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?zj(Object(n),!0).forEach(function(r){ko(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):zj(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function ko(e,t,n){return t=E7(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function E7(e){var t=T7(e,"string");return ds(t)=="symbol"?t:t+""}function T7(e,t){if(ds(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(ds(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return(t==="string"?String:Number)(e)}function nn(e,t,n){return ze(e)||ze(t)?n:Dt(t)?Ln(e,t,n):Ge(t)?t(e):n}function Nl(e,t,n,r){var i=AH(e,function(c){return nn(c,t)});if(n==="number"){var u=i.filter(function(c){return he(c)||parseFloat(c)});return u.length?[ih(u),ah(u)]:[1/0,-1/0]}var s=r?i.filter(function(c){return!ze(c)}):i;return s.map(function(c){return Dt(c)||c instanceof Date?c:""})}var M7=function(t){var n,r=arguments.length>1&&arguments[1]!==void 0?arguments[1]:[],i=arguments.length>2?arguments[2]:void 0,u=arguments.length>3?arguments[3]:void 0,s=-1,c=(n=r==null?void 0:r.length)!==null&&n!==void 0?n:0;if(c<=1)return 0;if(u&&u.axisType==="angleAxis"&&Math.abs(Math.abs(u.range[1]-u.range[0])-360)<=1e-6)for(var d=u.range,h=0;h<c;h++){var v=h>0?i[h-1].coordinate:i[c-1].coordinate,y=i[h].coordinate,g=h>=c-1?i[0].coordinate:i[h+1].coordinate,b=void 0;if(Fn(y-v)!==Fn(g-y)){var S=[];if(Fn(g-y)===Fn(d[1]-d[0])){b=g;var _=y+d[1]-d[0];S[0]=Math.min(_,(_+v)/2),S[1]=Math.max(_,(_+v)/2)}else{b=v;var x=g+d[1]-d[0];S[0]=Math.min(y,(x+y)/2),S[1]=Math.max(y,(x+y)/2)}var E=[Math.min(y,(b+y)/2),Math.max(y,(b+y)/2)];if(t>E[0]&&t<=E[1]||t>=S[0]&&t<=S[1]){s=i[h].index;break}}else{var M=Math.min(v,g),A=Math.max(v,g);if(t>(M+y)/2&&t<=(A+y)/2){s=i[h].index;break}}}else for(var C=0;C<c;C++)if(C===0&&t<=(r[C].coordinate+r[C+1].coordinate)/2||C>0&&C<c-1&&t>(r[C].coordinate+r[C-1].coordinate)/2&&t<=(r[C].coordinate+r[C+1].coordinate)/2||C===c-1&&t>(r[C].coordinate+r[C-1].coordinate)/2){s=r[C].index;break}return s},w_=function(t){var n,r=t,i=r.type.displayName,u=(n=t.type)!==null&&n!==void 0&&n.defaultProps?_t(_t({},t.type.defaultProps),t.props):t.props,s=u.stroke,c=u.fill,d;switch(i){case"Line":d=s;break;case"Area":case"Radar":d=s&&s!=="none"?s:c;break;default:d=c;break}return d},j7=function(t){var n=t.barSize,r=t.totalSize,i=t.stackGroups,u=i===void 0?{}:i;if(!u)return{};for(var s={},c=Object.keys(u),d=0,h=c.length;d<h;d++)for(var v=u[c[d]].stackGroups,y=Object.keys(v),g=0,b=y.length;g<b;g++){var S=v[y[g]],_=S.items,x=S.cateAxisId,E=_.filter(function(P){return Hr(P.type).indexOf("Bar")>=0});if(E&&E.length){var M=E[0].type.defaultProps,A=M!==void 0?_t(_t({},M),E[0].props):E[0].props,C=A.barSize,w=A[x];s[w]||(s[w]=[]);var T=ze(C)?n:C;s[w].push({item:E[0],stackList:E.slice(1),barSize:ze(T)?void 0:wi(T,r,0)})}}return s},C7=function(t){var n=t.barGap,r=t.barCategoryGap,i=t.bandSize,u=t.sizeList,s=u===void 0?[]:u,c=t.maxBarSize,d=s.length;if(d<1)return null;var h=wi(n,i,0,!0),v,y=[];if(s[0].barSize===+s[0].barSize){var g=!1,b=i/d,S=s.reduce(function(C,w){return C+w.barSize||0},0);S+=(d-1)*h,S>=i&&(S-=(d-1)*h,h=0),S>=i&&b>0&&(g=!0,b*=.9,S=d*b);var _=(i-S)/2>>0,x={offset:_-h,size:0};v=s.reduce(function(C,w){var T={item:w.item,position:{offset:x.offset+x.size+h,size:g?b:w.barSize}},P=[].concat(Bj(C),[T]);return x=P[P.length-1].position,w.stackList&&w.stackList.length&&w.stackList.forEach(function(R){P.push({item:R,position:x})}),P},y)}else{var E=wi(r,i,0,!0);i-2*E-(d-1)*h<=0&&(h=0);var M=(i-2*E-(d-1)*h)/d;M>1&&(M>>=0);var A=c===+c?Math.min(M,c):M;v=s.reduce(function(C,w,T){var P=[].concat(Bj(C),[{item:w.item,position:{offset:E+(M+h)*T+(M-A)/2,size:A}}]);return w.stackList&&w.stackList.length&&w.stackList.forEach(function(R){P.push({item:R,position:P[P.length-1].position})}),P},y)}return v},P7=function(t,n,r,i){var u=r.children,s=r.width,c=r.margin,d=s-(c.left||0)-(c.right||0),h=UN({children:u,legendWidth:d});if(h){var v=i||{},y=v.width,g=v.height,b=h.align,S=h.verticalAlign,_=h.layout;if((_==="vertical"||_==="horizontal"&&S==="middle")&&b!=="center"&&he(t[b]))return _t(_t({},t),{},ko({},b,t[b]+(y||0)));if((_==="horizontal"||_==="vertical"&&b==="center")&&S!=="middle"&&he(t[S]))return _t(_t({},t),{},ko({},S,t[S]+(g||0)))}return t},R7=function(t,n,r){return ze(n)?!0:t==="horizontal"?n==="yAxis":t==="vertical"||r==="x"?n==="xAxis":r==="y"?n==="yAxis":!0},HN=function(t,n,r,i,u){var s=n.props.children,c=An(s,Us).filter(function(h){return R7(i,u,h.props.direction)});if(c&&c.length){var d=c.map(function(h){return h.props.dataKey});return t.reduce(function(h,v){var y=nn(v,r);if(ze(y))return h;var g=Array.isArray(y)?[ih(y),ah(y)]:[y,y],b=d.reduce(function(S,_){var x=nn(v,_,0),E=g[0]-Math.abs(Array.isArray(x)?x[0]:x),M=g[1]+Math.abs(Array.isArray(x)?x[1]:x);return[Math.min(E,S[0]),Math.max(M,S[1])]},[1/0,-1/0]);return[Math.min(b[0],h[0]),Math.max(b[1],h[1])]},[1/0,-1/0])}return null},N7=function(t,n,r,i,u){var s=n.map(function(c){return HN(t,c,r,u,i)}).filter(function(c){return!ze(c)});return s&&s.length?s.reduce(function(c,d){return[Math.min(c[0],d[0]),Math.max(c[1],d[1])]},[1/0,-1/0]):null},GN=function(t,n,r,i,u){var s=n.map(function(d){var h=d.props.dataKey;return r==="number"&&h&&HN(t,d,h,i)||Nl(t,h,r,u)});if(r==="number")return s.reduce(function(d,h){return[Math.min(d[0],h[0]),Math.max(d[1],h[1])]},[1/0,-1/0]);var c={};return s.reduce(function(d,h){for(var v=0,y=h.length;v<y;v++)c[h[v]]||(c[h[v]]=!0,d.push(h[v]));return d},[])},VN=function(t,n){return t==="horizontal"&&n==="xAxis"||t==="vertical"&&n==="yAxis"||t==="centric"&&n==="angleAxis"||t==="radial"&&n==="radiusAxis"},hi=function(t,n,r){if(!t)return null;var i=t.scale,u=t.duplicateDomain,s=t.type,c=t.range,d=t.realScaleType==="scaleBand"?i.bandwidth()/2:2,h=(n||r)&&s==="category"&&i.bandwidth?i.bandwidth()/d:0;if(h=t.axisType==="angleAxis"&&(c==null?void 0:c.length)>=2?Fn(c[0]-c[1])*2*h:h,n&&(t.ticks||t.niceTicks)){var v=(t.ticks||t.niceTicks).map(function(y){var g=u?u.indexOf(y):y;return{coordinate:i(g)+h,value:y,offset:h}});return v.filter(function(y){return!ks(y.coordinate)})}return t.isCategorical&&t.categoricalDomain?t.categoricalDomain.map(function(y,g){return{coordinate:i(y)+h,value:y,index:g,offset:h}}):i.ticks&&!r?i.ticks(t.tickCount).map(function(y){return{coordinate:i(y)+h,value:y,offset:h}}):i.domain().map(function(y,g){return{coordinate:i(y)+h,value:u?u[y]:y,index:g,offset:h}})},gb=new WeakMap,wf=function(t,n){if(typeof n!="function")return t;gb.has(t)||gb.set(t,new WeakMap);var r=gb.get(t);if(r.has(n))return r.get(n);var i=function(){t.apply(void 0,arguments),n.apply(void 0,arguments)};return r.set(n,i),i},D7=function(t,n,r){var i=t.scale,u=t.type,s=t.layout,c=t.axisType;if(i==="auto")return s==="radial"&&c==="radiusAxis"?{scale:as(),realScaleType:"band"}:s==="radial"&&c==="angleAxis"?{scale:id(),realScaleType:"linear"}:u==="category"&&n&&(n.indexOf("LineChart")>=0||n.indexOf("AreaChart")>=0||n.indexOf("ComposedChart")>=0&&!r)?{scale:Rl(),realScaleType:"point"}:u==="category"?{scale:as(),realScaleType:"band"}:{scale:id(),realScaleType:"linear"};if(Ds(i)){var d="scale".concat(Gd(i));return{scale:(Oj[d]||Rl)(),realScaleType:Oj[d]?d:"point"}}return Ge(i)?{scale:i}:{scale:Rl(),realScaleType:"point"}},Lj=1e-4,k7=function(t){var n=t.domain();if(!(!n||n.length<=2)){var r=n.length,i=t.range(),u=Math.min(i[0],i[1])-Lj,s=Math.max(i[0],i[1])+Lj,c=t(n[0]),d=t(n[r-1]);(c<u||c>s||d<u||d>s)&&t.domain([n[0],n[r-1]])}},q7=function(t,n){if(!t)return null;for(var r=0,i=t.length;r<i;r++)if(t[r].item===n)return t[r].position;return null},$7=function(t,n){if(!n||n.length!==2||!he(n[0])||!he(n[1]))return t;var r=Math.min(n[0],n[1]),i=Math.max(n[0],n[1]),u=[t[0],t[1]];return(!he(t[0])||t[0]<r)&&(u[0]=r),(!he(t[1])||t[1]>i)&&(u[1]=i),u[0]>i&&(u[0]=i),u[1]<r&&(u[1]=r),u},B7=function(t){var n=t.length;if(!(n<=0))for(var r=0,i=t[0].length;r<i;++r)for(var u=0,s=0,c=0;c<n;++c){var d=ks(t[c][r][1])?t[c][r][0]:t[c][r][1];d>=0?(t[c][r][0]=u,t[c][r][1]=u+d,u=t[c][r][1]):(t[c][r][0]=s,t[c][r][1]=s+d,s=t[c][r][1])}},z7=function(t){var n=t.length;if(!(n<=0))for(var r=0,i=t[0].length;r<i;++r)for(var u=0,s=0;s<n;++s){var c=ks(t[s][r][1])?t[s][r][0]:t[s][r][1];c>=0?(t[s][r][0]=u,t[s][r][1]=u+c,u=t[s][r][1]):(t[s][r][0]=0,t[s][r][1]=0)}},L7={sign:B7,expand:SI,none:zo,silhouette:wI,wiggle:OI,positive:z7},I7=function(t,n,r){var i=n.map(function(c){return c.props.dataKey}),u=L7[r],s=_I().keys(i).value(function(c,d){return+nn(c,d,0)}).order(d0).offset(u);return s(t)},U7=function(t,n,r,i,u,s){if(!t)return null;var c=s?n.reverse():n,d={},h=c.reduce(function(y,g){var b,S=(b=g.type)!==null&&b!==void 0&&b.defaultProps?_t(_t({},g.type.defaultProps),g.props):g.props,_=S.stackId,x=S.hide;if(x)return y;var E=S[r],M=y[E]||{hasStack:!1,stackGroups:{}};if(Dt(_)){var A=M.stackGroups[_]||{numericAxisId:r,cateAxisId:i,items:[]};A.items.push(g),M.hasStack=!0,M.stackGroups[_]=A}else M.stackGroups[qs("_stackId_")]={numericAxisId:r,cateAxisId:i,items:[g]};return _t(_t({},y),{},ko({},E,M))},d),v={};return Object.keys(h).reduce(function(y,g){var b=h[g];if(b.hasStack){var S={};b.stackGroups=Object.keys(b.stackGroups).reduce(function(_,x){var E=b.stackGroups[x];return _t(_t({},_),{},ko({},x,{numericAxisId:r,cateAxisId:i,items:E.items,stackedData:I7(t,E.items,u)}))},S)}return _t(_t({},y),{},ko({},g,b))},v)},H7=function(t,n){var r=n.realScaleType,i=n.type,u=n.tickCount,s=n.originalDomain,c=n.allowDecimals,d=r||n.scale;if(d!=="auto"&&d!=="linear")return null;if(u&&i==="number"&&s&&(s[0]==="auto"||s[1]==="auto")){var h=t.domain();if(!h.length)return null;var v=e7(h,u,c);return t.domain([ih(v),ah(v)]),{niceTicks:v}}if(u&&i==="number"){var y=t.domain(),g=t7(y,u,c);return{niceTicks:g}}return null};function Ij(e){var t=e.axis,n=e.ticks,r=e.bandSize,i=e.entry,u=e.index,s=e.dataKey;if(t.type==="category"){if(!t.allowDuplicatedCategory&&t.dataKey&&!ze(i[t.dataKey])){var c=Lf(n,"value",i[t.dataKey]);if(c)return c.coordinate+r/2}return n[u]?n[u].coordinate+r/2:null}var d=nn(i,ze(s)?t.dataKey:s);return ze(d)?null:t.scale(d)}var Uj=function(t){var n=t.axis,r=t.ticks,i=t.offset,u=t.bandSize,s=t.entry,c=t.index;if(n.type==="category")return r[c]?r[c].coordinate+i:null;var d=nn(s,n.dataKey,n.domain[c]);return ze(d)?null:n.scale(d)-u/2+i},G7=function(t){var n=t.numericAxis,r=n.scale.domain();if(n.type==="number"){var i=Math.min(r[0],r[1]),u=Math.max(r[0],r[1]);return i<=0&&u>=0?0:u<0?u:i}return r[0]},V7=function(t,n){var r,i=(r=t.type)!==null&&r!==void 0&&r.defaultProps?_t(_t({},t.type.defaultProps),t.props):t.props,u=i.stackId;if(Dt(u)){var s=n[u];if(s){var c=s.items.indexOf(t);return c>=0?s.stackedData[c]:null}}return null},Z7=function(t){return t.reduce(function(n,r){return[ih(r.concat([n[0]]).filter(he)),ah(r.concat([n[1]]).filter(he))]},[1/0,-1/0])},ZN=function(t,n,r){return Object.keys(t).reduce(function(i,u){var s=t[u],c=s.stackedData,d=c.reduce(function(h,v){var y=Z7(v.slice(n,r+1));return[Math.min(h[0],y[0]),Math.max(h[1],y[1])]},[1/0,-1/0]);return[Math.min(d[0],i[0]),Math.max(d[1],i[1])]},[1/0,-1/0]).map(function(i){return i===1/0||i===-1/0?0:i})},Hj=/^dataMin[\s]*-[\s]*([0-9]+([.]{1}[0-9]+){0,1})$/,Gj=/^dataMax[\s]*\+[\s]*([0-9]+([.]{1}[0-9]+){0,1})$/,$0=function(t,n,r){if(Ge(t))return t(n,r);if(!Array.isArray(t))return n;var i=[];if(he(t[0]))i[0]=r?t[0]:Math.min(t[0],n[0]);else if(Hj.test(t[0])){var u=+Hj.exec(t[0])[1];i[0]=n[0]-u}else Ge(t[0])?i[0]=t[0](n[0]):i[0]=n[0];if(he(t[1]))i[1]=r?t[1]:Math.max(t[1],n[1]);else if(Gj.test(t[1])){var s=+Gj.exec(t[1])[1];i[1]=n[1]+s}else Ge(t[1])?i[1]=t[1](n[1]):i[1]=n[1];return i},dd=function(t,n,r){if(t&&t.scale&&t.scale.bandwidth){var i=t.scale.bandwidth();if(!r||i>0)return i}if(t&&n&&n.length>=2){for(var u=Wx(n,function(y){return y.coordinate}),s=1/0,c=1,d=u.length;c<d;c++){var h=u[c],v=u[c-1];s=Math.min((h.coordinate||0)-(v.coordinate||0),s)}return s===1/0?0:s}return r?void 0:0},Vj=function(t,n,r){return!t||!t.length||oh(t,Ln(r,"type.defaultProps.domain"))?n:t},YN=function(t,n){var r=t.type.defaultProps?_t(_t({},t.type.defaultProps),t.props):t.props,i=r.dataKey,u=r.name,s=r.unit,c=r.formatter,d=r.tooltipType,h=r.chartType,v=r.hide;return _t(_t({},Ye(t,!1)),{},{dataKey:i,unit:s,formatter:c,name:u||i,color:w_(t),value:nn(n,i),type:d,payload:n,chartType:h,hide:v})};function hs(e){"@babel/helpers - typeof";return hs=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},hs(e)}function Zj(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function Yj(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?Zj(Object(n),!0).forEach(function(r){Y7(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):Zj(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function Y7(e,t,n){return t=X7(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function X7(e){var t=K7(e,"string");return hs(t)=="symbol"?t:t+""}function K7(e,t){if(hs(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(hs(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return(t==="string"?String:Number)(e)}var hd=Math.PI/180,W7=function(t){return t*180/Math.PI},Zt=function(t,n,r,i){return{x:t+Math.cos(-hd*i)*r,y:n+Math.sin(-hd*i)*r}},F7=function(t,n){var r=t.x,i=t.y,u=n.x,s=n.y;return Math.sqrt(Math.pow(r-u,2)+Math.pow(i-s,2))},Q7=function(t,n){var r=t.x,i=t.y,u=n.cx,s=n.cy,c=F7({x:r,y:i},{x:u,y:s});if(c<=0)return{radius:c};var d=(r-u)/c,h=Math.acos(d);return i>s&&(h=2*Math.PI-h),{radius:c,angle:W7(h),angleInRadian:h}},J7=function(t){var n=t.startAngle,r=t.endAngle,i=Math.floor(n/360),u=Math.floor(r/360),s=Math.min(i,u);return{startAngle:n-s*360,endAngle:r-s*360}},eG=function(t,n){var r=n.startAngle,i=n.endAngle,u=Math.floor(r/360),s=Math.floor(i/360),c=Math.min(u,s);return t+c*360},Xj=function(t,n){var r=t.x,i=t.y,u=Q7({x:r,y:i},n),s=u.radius,c=u.angle,d=n.innerRadius,h=n.outerRadius;if(s<d||s>h)return!1;if(s===0)return!0;var v=J7(n),y=v.startAngle,g=v.endAngle,b=c,S;if(y<=g){for(;b>g;)b-=360;for(;b<y;)b+=360;S=b>=y&&b<=g}else{for(;b>y;)b-=360;for(;b<g;)b+=360;S=b>=g&&b<=y}return S?Yj(Yj({},n),{},{radius:s,angle:eG(b,n)}):null};function ps(e){"@babel/helpers - typeof";return ps=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},ps(e)}var tG=["offset"];function nG(e){return oG(e)||iG(e)||aG(e)||rG()}function rG(){throw new TypeError(`Invalid attempt to spread non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function aG(e,t){if(e){if(typeof e=="string")return B0(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);if(n==="Object"&&e.constructor&&(n=e.constructor.name),n==="Map"||n==="Set")return Array.from(e);if(n==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return B0(e,t)}}function iG(e){if(typeof Symbol<"u"&&e[Symbol.iterator]!=null||e["@@iterator"]!=null)return Array.from(e)}function oG(e){if(Array.isArray(e))return B0(e)}function B0(e,t){(t==null||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}function uG(e,t){if(e==null)return{};var n=lG(e,t),r,i;if(Object.getOwnPropertySymbols){var u=Object.getOwnPropertySymbols(e);for(i=0;i<u.length;i++)r=u[i],!(t.indexOf(r)>=0)&&Object.prototype.propertyIsEnumerable.call(e,r)&&(n[r]=e[r])}return n}function lG(e,t){if(e==null)return{};var n={};for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){if(t.indexOf(r)>=0)continue;n[r]=e[r]}return n}function Kj(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function Nt(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?Kj(Object(n),!0).forEach(function(r){sG(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):Kj(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function sG(e,t,n){return t=cG(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function cG(e){var t=fG(e,"string");return ps(t)=="symbol"?t:t+""}function fG(e,t){if(ps(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(ps(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return(t==="string"?String:Number)(e)}function vs(){return vs=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},vs.apply(this,arguments)}var dG=function(t){var n=t.value,r=t.formatter,i=ze(t.children)?n:t.children;return Ge(r)?r(i):i},hG=function(t,n){var r=Fn(n-t),i=Math.min(Math.abs(n-t),360);return r*i},pG=function(t,n,r){var i=t.position,u=t.viewBox,s=t.offset,c=t.className,d=u,h=d.cx,v=d.cy,y=d.innerRadius,g=d.outerRadius,b=d.startAngle,S=d.endAngle,_=d.clockWise,x=(y+g)/2,E=hG(b,S),M=E>=0?1:-1,A,C;i==="insideStart"?(A=b+M*s,C=_):i==="insideEnd"?(A=S-M*s,C=!_):i==="end"&&(A=S+M*s,C=_),C=E<=0?C:!C;var w=Zt(h,v,x,A),T=Zt(h,v,x,A+(C?1:-1)*359),P="M".concat(w.x,",").concat(w.y,`
    A`).concat(x,",").concat(x,",0,1,").concat(C?0:1,`,
    `).concat(T.x,",").concat(T.y),R=ze(t.id)?qs("recharts-radial-line-"):t.id;return L.createElement("text",vs({},r,{dominantBaseline:"central",className:Xe("recharts-radial-bar-label",c)}),L.createElement("defs",null,L.createElement("path",{id:R,d:P})),L.createElement("textPath",{xlinkHref:"#".concat(R)},n))},vG=function(t){var n=t.viewBox,r=t.offset,i=t.position,u=n,s=u.cx,c=u.cy,d=u.innerRadius,h=u.outerRadius,v=u.startAngle,y=u.endAngle,g=(v+y)/2;if(i==="outside"){var b=Zt(s,c,h+r,g),S=b.x,_=b.y;return{x:S,y:_,textAnchor:S>=s?"start":"end",verticalAnchor:"middle"}}if(i==="center")return{x:s,y:c,textAnchor:"middle",verticalAnchor:"middle"};if(i==="centerTop")return{x:s,y:c,textAnchor:"middle",verticalAnchor:"start"};if(i==="centerBottom")return{x:s,y:c,textAnchor:"middle",verticalAnchor:"end"};var x=(d+h)/2,E=Zt(s,c,x,g),M=E.x,A=E.y;return{x:M,y:A,textAnchor:"middle",verticalAnchor:"middle"}},yG=function(t){var n=t.viewBox,r=t.parentViewBox,i=t.offset,u=t.position,s=n,c=s.x,d=s.y,h=s.width,v=s.height,y=v>=0?1:-1,g=y*i,b=y>0?"end":"start",S=y>0?"start":"end",_=h>=0?1:-1,x=_*i,E=_>0?"end":"start",M=_>0?"start":"end";if(u==="top"){var A={x:c+h/2,y:d-y*i,textAnchor:"middle",verticalAnchor:b};return Nt(Nt({},A),r?{height:Math.max(d-r.y,0),width:h}:{})}if(u==="bottom"){var C={x:c+h/2,y:d+v+g,textAnchor:"middle",verticalAnchor:S};return Nt(Nt({},C),r?{height:Math.max(r.y+r.height-(d+v),0),width:h}:{})}if(u==="left"){var w={x:c-x,y:d+v/2,textAnchor:E,verticalAnchor:"middle"};return Nt(Nt({},w),r?{width:Math.max(w.x-r.x,0),height:v}:{})}if(u==="right"){var T={x:c+h+x,y:d+v/2,textAnchor:M,verticalAnchor:"middle"};return Nt(Nt({},T),r?{width:Math.max(r.x+r.width-T.x,0),height:v}:{})}var P=r?{width:h,height:v}:{};return u==="insideLeft"?Nt({x:c+x,y:d+v/2,textAnchor:M,verticalAnchor:"middle"},P):u==="insideRight"?Nt({x:c+h-x,y:d+v/2,textAnchor:E,verticalAnchor:"middle"},P):u==="insideTop"?Nt({x:c+h/2,y:d+g,textAnchor:"middle",verticalAnchor:S},P):u==="insideBottom"?Nt({x:c+h/2,y:d+v-g,textAnchor:"middle",verticalAnchor:b},P):u==="insideTopLeft"?Nt({x:c+x,y:d+g,textAnchor:M,verticalAnchor:S},P):u==="insideTopRight"?Nt({x:c+h-x,y:d+g,textAnchor:E,verticalAnchor:S},P):u==="insideBottomLeft"?Nt({x:c+x,y:d+v-g,textAnchor:M,verticalAnchor:b},P):u==="insideBottomRight"?Nt({x:c+h-x,y:d+v-g,textAnchor:E,verticalAnchor:b},P):fu(u)&&(he(u.x)||ci(u.x))&&(he(u.y)||ci(u.y))?Nt({x:c+wi(u.x,h),y:d+wi(u.y,v),textAnchor:"end",verticalAnchor:"end"},P):Nt({x:c+h/2,y:d+v/2,textAnchor:"middle",verticalAnchor:"middle"},P)},mG=function(t){return"cx"in t&&he(t.cx)};function Ft(e){var t=e.offset,n=t===void 0?5:t,r=uG(e,tG),i=Nt({offset:n},r),u=i.viewBox,s=i.position,c=i.value,d=i.children,h=i.content,v=i.className,y=v===void 0?"":v,g=i.textBreakAll;if(!u||ze(c)&&ze(d)&&!q.isValidElement(h)&&!Ge(h))return null;if(q.isValidElement(h))return q.cloneElement(h,i);var b;if(Ge(h)){if(b=q.createElement(h,i),q.isValidElement(b))return b}else b=dG(i);var S=mG(u),_=Ye(i,!0);if(S&&(s==="insideStart"||s==="insideEnd"||s==="end"))return pG(i,b,_);var x=S?vG(i):yG(i);return L.createElement(Qf,vs({className:Xe("recharts-label",y)},_,x,{breakAll:g}),b)}Ft.displayName="Label";var XN=function(t){var n=t.cx,r=t.cy,i=t.angle,u=t.startAngle,s=t.endAngle,c=t.r,d=t.radius,h=t.innerRadius,v=t.outerRadius,y=t.x,g=t.y,b=t.top,S=t.left,_=t.width,x=t.height,E=t.clockWise,M=t.labelViewBox;if(M)return M;if(he(_)&&he(x)){if(he(y)&&he(g))return{x:y,y:g,width:_,height:x};if(he(b)&&he(S))return{x:b,y:S,width:_,height:x}}return he(y)&&he(g)?{x:y,y:g,width:0,height:0}:he(n)&&he(r)?{cx:n,cy:r,startAngle:u||i||0,endAngle:s||i||0,innerRadius:h||0,outerRadius:v||d||c||0,clockWise:E}:t.viewBox?t.viewBox:{}},gG=function(t,n){return t?t===!0?L.createElement(Ft,{key:"label-implicit",viewBox:n}):Dt(t)?L.createElement(Ft,{key:"label-implicit",viewBox:n,value:t}):q.isValidElement(t)?t.type===Ft?q.cloneElement(t,{key:"label-implicit",viewBox:n}):L.createElement(Ft,{key:"label-implicit",content:t,viewBox:n}):Ge(t)?L.createElement(Ft,{key:"label-implicit",content:t,viewBox:n}):fu(t)?L.createElement(Ft,vs({viewBox:n},t,{key:"label-implicit"})):null:null},bG=function(t,n){var r=arguments.length>2&&arguments[2]!==void 0?arguments[2]:!0;if(!t||!t.children&&r&&!t.label)return null;var i=t.children,u=XN(t),s=An(i,Ft).map(function(d,h){return q.cloneElement(d,{viewBox:n||u,key:"label-".concat(h)})});if(!r)return s;var c=gG(t.label,n||u);return[c].concat(nG(s))};Ft.parseViewBox=XN;Ft.renderCallByParent=bG;var bb,Wj;function xG(){if(Wj)return bb;Wj=1;function e(t){var n=t==null?0:t.length;return n?t[n-1]:void 0}return bb=e,bb}var _G=xG();const SG=lt(_G);function ys(e){"@babel/helpers - typeof";return ys=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},ys(e)}var wG=["valueAccessor"],OG=["data","dataKey","clockWise","id","textBreakAll"];function AG(e){return jG(e)||MG(e)||TG(e)||EG()}function EG(){throw new TypeError(`Invalid attempt to spread non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function TG(e,t){if(e){if(typeof e=="string")return z0(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);if(n==="Object"&&e.constructor&&(n=e.constructor.name),n==="Map"||n==="Set")return Array.from(e);if(n==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return z0(e,t)}}function MG(e){if(typeof Symbol<"u"&&e[Symbol.iterator]!=null||e["@@iterator"]!=null)return Array.from(e)}function jG(e){if(Array.isArray(e))return z0(e)}function z0(e,t){(t==null||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}function pd(){return pd=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},pd.apply(this,arguments)}function Fj(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function Qj(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?Fj(Object(n),!0).forEach(function(r){CG(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):Fj(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function CG(e,t,n){return t=PG(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function PG(e){var t=RG(e,"string");return ys(t)=="symbol"?t:t+""}function RG(e,t){if(ys(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(ys(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return(t==="string"?String:Number)(e)}function Jj(e,t){if(e==null)return{};var n=NG(e,t),r,i;if(Object.getOwnPropertySymbols){var u=Object.getOwnPropertySymbols(e);for(i=0;i<u.length;i++)r=u[i],!(t.indexOf(r)>=0)&&Object.prototype.propertyIsEnumerable.call(e,r)&&(n[r]=e[r])}return n}function NG(e,t){if(e==null)return{};var n={};for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){if(t.indexOf(r)>=0)continue;n[r]=e[r]}return n}var DG=function(t){return Array.isArray(t.value)?SG(t.value):t.value};function Pa(e){var t=e.valueAccessor,n=t===void 0?DG:t,r=Jj(e,wG),i=r.data,u=r.dataKey,s=r.clockWise,c=r.id,d=r.textBreakAll,h=Jj(r,OG);return!i||!i.length?null:L.createElement(gt,{className:"recharts-label-list"},i.map(function(v,y){var g=ze(u)?n(v,y):nn(v&&v.payload,u),b=ze(c)?{}:{id:"".concat(c,"-").concat(y)};return L.createElement(Ft,pd({},Ye(v,!0),h,b,{parentViewBox:v.parentViewBox,value:g,textBreakAll:d,viewBox:Ft.parseViewBox(ze(s)?v:Qj(Qj({},v),{},{clockWise:s})),key:"label-".concat(y),index:y}))}))}Pa.displayName="LabelList";function kG(e,t){return e?e===!0?L.createElement(Pa,{key:"labelList-implicit",data:t}):L.isValidElement(e)||Ge(e)?L.createElement(Pa,{key:"labelList-implicit",data:t,content:e}):fu(e)?L.createElement(Pa,pd({data:t},e,{key:"labelList-implicit"})):null:null}function qG(e,t){var n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:!0;if(!e||!e.children&&n&&!e.label)return null;var r=e.children,i=An(r,Pa).map(function(s,c){return q.cloneElement(s,{data:t,key:"labelList-".concat(c)})});if(!n)return i;var u=kG(e.label,t);return[u].concat(AG(i))}Pa.renderCallByParent=qG;function ms(e){"@babel/helpers - typeof";return ms=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},ms(e)}function L0(){return L0=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},L0.apply(this,arguments)}function eC(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function tC(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?eC(Object(n),!0).forEach(function(r){$G(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):eC(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function $G(e,t,n){return t=BG(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function BG(e){var t=zG(e,"string");return ms(t)=="symbol"?t:t+""}function zG(e,t){if(ms(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(ms(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return(t==="string"?String:Number)(e)}var LG=function(t,n){var r=Fn(n-t),i=Math.min(Math.abs(n-t),359.999);return r*i},Of=function(t){var n=t.cx,r=t.cy,i=t.radius,u=t.angle,s=t.sign,c=t.isExternal,d=t.cornerRadius,h=t.cornerIsExternal,v=d*(c?1:-1)+i,y=Math.asin(d/v)/hd,g=h?u:u+s*y,b=Zt(n,r,v,g),S=Zt(n,r,i,g),_=h?u-s*y:u,x=Zt(n,r,v*Math.cos(y*hd),_);return{center:b,circleTangency:S,lineTangency:x,theta:y}},KN=function(t){var n=t.cx,r=t.cy,i=t.innerRadius,u=t.outerRadius,s=t.startAngle,c=t.endAngle,d=LG(s,c),h=s+d,v=Zt(n,r,u,s),y=Zt(n,r,u,h),g="M ".concat(v.x,",").concat(v.y,`
    A `).concat(u,",").concat(u,`,0,
    `).concat(+(Math.abs(d)>180),",").concat(+(s>h),`,
    `).concat(y.x,",").concat(y.y,`
  `);if(i>0){var b=Zt(n,r,i,s),S=Zt(n,r,i,h);g+="L ".concat(S.x,",").concat(S.y,`
            A `).concat(i,",").concat(i,`,0,
            `).concat(+(Math.abs(d)>180),",").concat(+(s<=h),`,
            `).concat(b.x,",").concat(b.y," Z")}else g+="L ".concat(n,",").concat(r," Z");return g},IG=function(t){var n=t.cx,r=t.cy,i=t.innerRadius,u=t.outerRadius,s=t.cornerRadius,c=t.forceCornerRadius,d=t.cornerIsExternal,h=t.startAngle,v=t.endAngle,y=Fn(v-h),g=Of({cx:n,cy:r,radius:u,angle:h,sign:y,cornerRadius:s,cornerIsExternal:d}),b=g.circleTangency,S=g.lineTangency,_=g.theta,x=Of({cx:n,cy:r,radius:u,angle:v,sign:-y,cornerRadius:s,cornerIsExternal:d}),E=x.circleTangency,M=x.lineTangency,A=x.theta,C=d?Math.abs(h-v):Math.abs(h-v)-_-A;if(C<0)return c?"M ".concat(S.x,",").concat(S.y,`
        a`).concat(s,",").concat(s,",0,0,1,").concat(s*2,`,0
        a`).concat(s,",").concat(s,",0,0,1,").concat(-s*2,`,0
      `):KN({cx:n,cy:r,innerRadius:i,outerRadius:u,startAngle:h,endAngle:v});var w="M ".concat(S.x,",").concat(S.y,`
    A`).concat(s,",").concat(s,",0,0,").concat(+(y<0),",").concat(b.x,",").concat(b.y,`
    A`).concat(u,",").concat(u,",0,").concat(+(C>180),",").concat(+(y<0),",").concat(E.x,",").concat(E.y,`
    A`).concat(s,",").concat(s,",0,0,").concat(+(y<0),",").concat(M.x,",").concat(M.y,`
  `);if(i>0){var T=Of({cx:n,cy:r,radius:i,angle:h,sign:y,isExternal:!0,cornerRadius:s,cornerIsExternal:d}),P=T.circleTangency,R=T.lineTangency,$=T.theta,J=Of({cx:n,cy:r,radius:i,angle:v,sign:-y,isExternal:!0,cornerRadius:s,cornerIsExternal:d}),B=J.circleTangency,G=J.lineTangency,W=J.theta,ne=d?Math.abs(h-v):Math.abs(h-v)-$-W;if(ne<0&&s===0)return"".concat(w,"L").concat(n,",").concat(r,"Z");w+="L".concat(G.x,",").concat(G.y,`
      A`).concat(s,",").concat(s,",0,0,").concat(+(y<0),",").concat(B.x,",").concat(B.y,`
      A`).concat(i,",").concat(i,",0,").concat(+(ne>180),",").concat(+(y>0),",").concat(P.x,",").concat(P.y,`
      A`).concat(s,",").concat(s,",0,0,").concat(+(y<0),",").concat(R.x,",").concat(R.y,"Z")}else w+="L".concat(n,",").concat(r,"Z");return w},UG={cx:0,cy:0,innerRadius:0,outerRadius:0,startAngle:0,endAngle:0,cornerRadius:0,forceCornerRadius:!1,cornerIsExternal:!1},WN=function(t){var n=tC(tC({},UG),t),r=n.cx,i=n.cy,u=n.innerRadius,s=n.outerRadius,c=n.cornerRadius,d=n.forceCornerRadius,h=n.cornerIsExternal,v=n.startAngle,y=n.endAngle,g=n.className;if(s<u||v===y)return null;var b=Xe("recharts-sector",g),S=s-u,_=wi(c,S,0,!0),x;return _>0&&Math.abs(v-y)<360?x=IG({cx:r,cy:i,innerRadius:u,outerRadius:s,cornerRadius:Math.min(_,S/2),forceCornerRadius:d,cornerIsExternal:h,startAngle:v,endAngle:y}):x=KN({cx:r,cy:i,innerRadius:u,outerRadius:s,startAngle:v,endAngle:y}),L.createElement("path",L0({},Ye(n,!0),{className:b,d:x,role:"img"}))};function gs(e){"@babel/helpers - typeof";return gs=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},gs(e)}function I0(){return I0=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},I0.apply(this,arguments)}function nC(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function rC(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?nC(Object(n),!0).forEach(function(r){HG(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):nC(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function HG(e,t,n){return t=GG(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function GG(e){var t=VG(e,"string");return gs(t)=="symbol"?t:t+""}function VG(e,t){if(gs(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(gs(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return(t==="string"?String:Number)(e)}var aC={curveBasisClosed:cI,curveBasisOpen:fI,curveBasis:sI,curveBumpX:KL,curveBumpY:WL,curveLinearClosed:dI,curveLinear:Zd,curveMonotoneX:hI,curveMonotoneY:pI,curveNatural:vI,curveStep:yI,curveStepAfter:gI,curveStepBefore:mI},Af=function(t){return t.x===+t.x&&t.y===+t.y},Sl=function(t){return t.x},wl=function(t){return t.y},ZG=function(t,n){if(Ge(t))return t;var r="curve".concat(Gd(t));return(r==="curveMonotone"||r==="curveBump")&&n?aC["".concat(r).concat(n==="vertical"?"Y":"X")]:aC[r]||Zd},YG=function(t){var n=t.type,r=n===void 0?"linear":n,i=t.points,u=i===void 0?[]:i,s=t.baseLine,c=t.layout,d=t.connectNulls,h=d===void 0?!1:d,v=ZG(r,c),y=h?u.filter(function(_){return Af(_)}):u,g;if(Array.isArray(s)){var b=h?s.filter(function(_){return Af(_)}):s,S=y.map(function(_,x){return rC(rC({},_),{},{base:b[x]})});return c==="vertical"?g=yf().y(wl).x1(Sl).x0(function(_){return _.base.x}):g=yf().x(Sl).y1(wl).y0(function(_){return _.base.y}),g.defined(Af).curve(v),g(S)}return c==="vertical"&&he(s)?g=yf().y(wl).x1(Sl).x0(s):he(s)?g=yf().x(Sl).y1(wl).y0(s):g=nR().x(Sl).y(wl),g.defined(Af).curve(v),g(y)},U0=function(t){var n=t.className,r=t.points,i=t.path,u=t.pathRef;if((!r||!r.length)&&!i)return null;var s=r&&r.length?YG(t):i;return L.createElement("path",I0({},Ye(t,!1),If(t),{className:Xe("recharts-curve",n),d:s,ref:u}))},xb={exports:{}},_b,iC;function XG(){if(iC)return _b;iC=1;var e="SECRET_DO_NOT_PASS_THIS_OR_YOU_WILL_BE_FIRED";return _b=e,_b}var Sb,oC;function KG(){if(oC)return Sb;oC=1;var e=XG();function t(){}function n(){}return n.resetWarningCache=t,Sb=function(){function r(s,c,d,h,v,y){if(y!==e){var g=new Error("Calling PropTypes validators directly is not supported by the `prop-types` package. Use PropTypes.checkPropTypes() to call them. Read more at http://fb.me/use-check-prop-types");throw g.name="Invariant Violation",g}}r.isRequired=r;function i(){return r}var u={array:r,bigint:r,bool:r,func:r,number:r,object:r,string:r,symbol:r,any:r,arrayOf:i,element:r,elementType:r,instanceOf:i,node:r,objectOf:i,oneOf:i,oneOfType:i,shape:i,exact:i,checkPropTypes:n,resetWarningCache:t};return u.PropTypes=u,u},Sb}var uC;function WG(){return uC||(uC=1,xb.exports=KG()()),xb.exports}var FG=WG();const Je=lt(FG);var QG=Object.getOwnPropertyNames,JG=Object.getOwnPropertySymbols,eV=Object.prototype.hasOwnProperty;function lC(e,t){return function(r,i,u){return e(r,i,u)&&t(r,i,u)}}function Ef(e){return function(n,r,i){if(!n||!r||typeof n!="object"||typeof r!="object")return e(n,r,i);var u=i.cache,s=u.get(n),c=u.get(r);if(s&&c)return s===r&&c===n;u.set(n,r),u.set(r,n);var d=e(n,r,i);return u.delete(n),u.delete(r),d}}function sC(e){return QG(e).concat(JG(e))}var tV=Object.hasOwn||function(e,t){return eV.call(e,t)};function Ci(e,t){return e===t||!e&&!t&&e!==e&&t!==t}var nV="__v",rV="__o",aV="_owner",cC=Object.getOwnPropertyDescriptor,fC=Object.keys;function iV(e,t,n){var r=e.length;if(t.length!==r)return!1;for(;r-- >0;)if(!n.equals(e[r],t[r],r,r,e,t,n))return!1;return!0}function oV(e,t){return Ci(e.getTime(),t.getTime())}function uV(e,t){return e.name===t.name&&e.message===t.message&&e.cause===t.cause&&e.stack===t.stack}function lV(e,t){return e===t}function dC(e,t,n){var r=e.size;if(r!==t.size)return!1;if(!r)return!0;for(var i=new Array(r),u=e.entries(),s,c,d=0;(s=u.next())&&!s.done;){for(var h=t.entries(),v=!1,y=0;(c=h.next())&&!c.done;){if(i[y]){y++;continue}var g=s.value,b=c.value;if(n.equals(g[0],b[0],d,y,e,t,n)&&n.equals(g[1],b[1],g[0],b[0],e,t,n)){v=i[y]=!0;break}y++}if(!v)return!1;d++}return!0}var sV=Ci;function cV(e,t,n){var r=fC(e),i=r.length;if(fC(t).length!==i)return!1;for(;i-- >0;)if(!FN(e,t,n,r[i]))return!1;return!0}function Ol(e,t,n){var r=sC(e),i=r.length;if(sC(t).length!==i)return!1;for(var u,s,c;i-- >0;)if(u=r[i],!FN(e,t,n,u)||(s=cC(e,u),c=cC(t,u),(s||c)&&(!s||!c||s.configurable!==c.configurable||s.enumerable!==c.enumerable||s.writable!==c.writable)))return!1;return!0}function fV(e,t){return Ci(e.valueOf(),t.valueOf())}function dV(e,t){return e.source===t.source&&e.flags===t.flags}function hC(e,t,n){var r=e.size;if(r!==t.size)return!1;if(!r)return!0;for(var i=new Array(r),u=e.values(),s,c;(s=u.next())&&!s.done;){for(var d=t.values(),h=!1,v=0;(c=d.next())&&!c.done;){if(!i[v]&&n.equals(s.value,c.value,s.value,c.value,e,t,n)){h=i[v]=!0;break}v++}if(!h)return!1}return!0}function hV(e,t){var n=e.length;if(t.length!==n)return!1;for(;n-- >0;)if(e[n]!==t[n])return!1;return!0}function pV(e,t){return e.hostname===t.hostname&&e.pathname===t.pathname&&e.protocol===t.protocol&&e.port===t.port&&e.hash===t.hash&&e.username===t.username&&e.password===t.password}function FN(e,t,n,r){return(r===aV||r===rV||r===nV)&&(e.$$typeof||t.$$typeof)?!0:tV(t,r)&&n.equals(e[r],t[r],r,r,e,t,n)}var vV="[object Arguments]",yV="[object Boolean]",mV="[object Date]",gV="[object Error]",bV="[object Map]",xV="[object Number]",_V="[object Object]",SV="[object RegExp]",wV="[object Set]",OV="[object String]",AV="[object URL]",EV=Array.isArray,pC=typeof ArrayBuffer=="function"&&ArrayBuffer.isView?ArrayBuffer.isView:null,vC=Object.assign,TV=Object.prototype.toString.call.bind(Object.prototype.toString);function MV(e){var t=e.areArraysEqual,n=e.areDatesEqual,r=e.areErrorsEqual,i=e.areFunctionsEqual,u=e.areMapsEqual,s=e.areNumbersEqual,c=e.areObjectsEqual,d=e.arePrimitiveWrappersEqual,h=e.areRegExpsEqual,v=e.areSetsEqual,y=e.areTypedArraysEqual,g=e.areUrlsEqual;return function(S,_,x){if(S===_)return!0;if(S==null||_==null)return!1;var E=typeof S;if(E!==typeof _)return!1;if(E!=="object")return E==="number"?s(S,_,x):E==="function"?i(S,_,x):!1;var M=S.constructor;if(M!==_.constructor)return!1;if(M===Object)return c(S,_,x);if(EV(S))return t(S,_,x);if(pC!=null&&pC(S))return y(S,_,x);if(M===Date)return n(S,_,x);if(M===RegExp)return h(S,_,x);if(M===Map)return u(S,_,x);if(M===Set)return v(S,_,x);var A=TV(S);return A===mV?n(S,_,x):A===SV?h(S,_,x):A===bV?u(S,_,x):A===wV?v(S,_,x):A===_V?typeof S.then!="function"&&typeof _.then!="function"&&c(S,_,x):A===AV?g(S,_,x):A===gV?r(S,_,x):A===vV?c(S,_,x):A===yV||A===xV||A===OV?d(S,_,x):!1}}function jV(e){var t=e.circular,n=e.createCustomConfig,r=e.strict,i={areArraysEqual:r?Ol:iV,areDatesEqual:oV,areErrorsEqual:uV,areFunctionsEqual:lV,areMapsEqual:r?lC(dC,Ol):dC,areNumbersEqual:sV,areObjectsEqual:r?Ol:cV,arePrimitiveWrappersEqual:fV,areRegExpsEqual:dV,areSetsEqual:r?lC(hC,Ol):hC,areTypedArraysEqual:r?Ol:hV,areUrlsEqual:pV};if(n&&(i=vC({},i,n(i))),t){var u=Ef(i.areArraysEqual),s=Ef(i.areMapsEqual),c=Ef(i.areObjectsEqual),d=Ef(i.areSetsEqual);i=vC({},i,{areArraysEqual:u,areMapsEqual:s,areObjectsEqual:c,areSetsEqual:d})}return i}function CV(e){return function(t,n,r,i,u,s,c){return e(t,n,c)}}function PV(e){var t=e.circular,n=e.comparator,r=e.createState,i=e.equals,u=e.strict;if(r)return function(d,h){var v=r(),y=v.cache,g=y===void 0?t?new WeakMap:void 0:y,b=v.meta;return n(d,h,{cache:g,equals:i,meta:b,strict:u})};if(t)return function(d,h){return n(d,h,{cache:new WeakMap,equals:i,meta:void 0,strict:u})};var s={cache:void 0,equals:i,meta:void 0,strict:u};return function(d,h){return n(d,h,s)}}var RV=La();La({strict:!0});La({circular:!0});La({circular:!0,strict:!0});La({createInternalComparator:function(){return Ci}});La({strict:!0,createInternalComparator:function(){return Ci}});La({circular:!0,createInternalComparator:function(){return Ci}});La({circular:!0,createInternalComparator:function(){return Ci},strict:!0});function La(e){e===void 0&&(e={});var t=e.circular,n=t===void 0?!1:t,r=e.createInternalComparator,i=e.createState,u=e.strict,s=u===void 0?!1:u,c=jV(e),d=MV(c),h=r?r(d):CV(d);return PV({circular:n,comparator:d,createState:i,equals:h,strict:s})}function NV(e){typeof requestAnimationFrame<"u"&&requestAnimationFrame(e)}function yC(e){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:0,n=-1,r=function i(u){n<0&&(n=u),u-n>t?(e(u),n=-1):NV(i)};requestAnimationFrame(r)}function H0(e){"@babel/helpers - typeof";return H0=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},H0(e)}function DV(e){return BV(e)||$V(e)||qV(e)||kV()}function kV(){throw new TypeError(`Invalid attempt to destructure non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function qV(e,t){if(e){if(typeof e=="string")return mC(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);if(n==="Object"&&e.constructor&&(n=e.constructor.name),n==="Map"||n==="Set")return Array.from(e);if(n==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return mC(e,t)}}function mC(e,t){(t==null||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}function $V(e){if(typeof Symbol<"u"&&e[Symbol.iterator]!=null||e["@@iterator"]!=null)return Array.from(e)}function BV(e){if(Array.isArray(e))return e}function zV(){var e={},t=function(){return null},n=!1,r=function i(u){if(!n){if(Array.isArray(u)){if(!u.length)return;var s=u,c=DV(s),d=c[0],h=c.slice(1);if(typeof d=="number"){yC(i.bind(null,h),d);return}i(d),yC(i.bind(null,h));return}H0(u)==="object"&&(e=u,t(e)),typeof u=="function"&&u()}};return{stop:function(){n=!0},start:function(u){n=!1,r(u)},subscribe:function(u){return t=u,function(){t=function(){return null}}}}}function bs(e){"@babel/helpers - typeof";return bs=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},bs(e)}function gC(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function bC(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?gC(Object(n),!0).forEach(function(r){QN(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):gC(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function QN(e,t,n){return t=LV(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function LV(e){var t=IV(e,"string");return bs(t)==="symbol"?t:String(t)}function IV(e,t){if(bs(e)!=="object"||e===null)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(bs(r)!=="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return(t==="string"?String:Number)(e)}var UV=function(t,n){return[Object.keys(t),Object.keys(n)].reduce(function(r,i){return r.filter(function(u){return i.includes(u)})})},HV=function(t){return t},GV=function(t){return t.replace(/([A-Z])/g,function(n){return"-".concat(n.toLowerCase())})},Dl=function(t,n){return Object.keys(n).reduce(function(r,i){return bC(bC({},r),{},QN({},i,t(i,n[i])))},{})},xC=function(t,n,r){return t.map(function(i){return"".concat(GV(i)," ").concat(n,"ms ").concat(r)}).join(",")};function VV(e,t){return XV(e)||YV(e,t)||JN(e,t)||ZV()}function ZV(){throw new TypeError(`Invalid attempt to destructure non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function YV(e,t){var n=e==null?null:typeof Symbol<"u"&&e[Symbol.iterator]||e["@@iterator"];if(n!=null){var r,i,u,s,c=[],d=!0,h=!1;try{if(u=(n=n.call(e)).next,t!==0)for(;!(d=(r=u.call(n)).done)&&(c.push(r.value),c.length!==t);d=!0);}catch(v){h=!0,i=v}finally{try{if(!d&&n.return!=null&&(s=n.return(),Object(s)!==s))return}finally{if(h)throw i}}return c}}function XV(e){if(Array.isArray(e))return e}function KV(e){return QV(e)||FV(e)||JN(e)||WV()}function WV(){throw new TypeError(`Invalid attempt to spread non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function JN(e,t){if(e){if(typeof e=="string")return G0(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);if(n==="Object"&&e.constructor&&(n=e.constructor.name),n==="Map"||n==="Set")return Array.from(e);if(n==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return G0(e,t)}}function FV(e){if(typeof Symbol<"u"&&e[Symbol.iterator]!=null||e["@@iterator"]!=null)return Array.from(e)}function QV(e){if(Array.isArray(e))return G0(e)}function G0(e,t){(t==null||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}var vd=1e-4,eD=function(t,n){return[0,3*t,3*n-6*t,3*t-3*n+1]},tD=function(t,n){return t.map(function(r,i){return r*Math.pow(n,i)}).reduce(function(r,i){return r+i})},_C=function(t,n){return function(r){var i=eD(t,n);return tD(i,r)}},JV=function(t,n){return function(r){var i=eD(t,n),u=[].concat(KV(i.map(function(s,c){return s*c}).slice(1)),[0]);return tD(u,r)}},SC=function(){for(var t=arguments.length,n=new Array(t),r=0;r<t;r++)n[r]=arguments[r];var i=n[0],u=n[1],s=n[2],c=n[3];if(n.length===1)switch(n[0]){case"linear":i=0,u=0,s=1,c=1;break;case"ease":i=.25,u=.1,s=.25,c=1;break;case"ease-in":i=.42,u=0,s=1,c=1;break;case"ease-out":i=.42,u=0,s=.58,c=1;break;case"ease-in-out":i=0,u=0,s=.58,c=1;break;default:{var d=n[0].split("(");if(d[0]==="cubic-bezier"&&d[1].split(")")[0].split(",").length===4){var h=d[1].split(")")[0].split(",").map(function(x){return parseFloat(x)}),v=VV(h,4);i=v[0],u=v[1],s=v[2],c=v[3]}}}var y=_C(i,s),g=_C(u,c),b=JV(i,s),S=function(E){return E>1?1:E<0?0:E},_=function(E){for(var M=E>1?1:E,A=M,C=0;C<8;++C){var w=y(A)-M,T=b(A);if(Math.abs(w-M)<vd||T<vd)return g(A);A=S(A-w/T)}return g(A)};return _.isStepper=!1,_},eZ=function(){var t=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{},n=t.stiff,r=n===void 0?100:n,i=t.damping,u=i===void 0?8:i,s=t.dt,c=s===void 0?17:s,d=function(v,y,g){var b=-(v-y)*r,S=g*u,_=g+(b-S)*c/1e3,x=g*c/1e3+v;return Math.abs(x-y)<vd&&Math.abs(_)<vd?[y,0]:[x,_]};return d.isStepper=!0,d.dt=c,d},tZ=function(){for(var t=arguments.length,n=new Array(t),r=0;r<t;r++)n[r]=arguments[r];var i=n[0];if(typeof i=="string")switch(i){case"ease":case"ease-in-out":case"ease-out":case"ease-in":case"linear":return SC(i);case"spring":return eZ();default:if(i.split("(")[0]==="cubic-bezier")return SC(i)}return typeof i=="function"?i:null};function xs(e){"@babel/helpers - typeof";return xs=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},xs(e)}function wC(e){return aZ(e)||rZ(e)||nD(e)||nZ()}function nZ(){throw new TypeError(`Invalid attempt to spread non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function rZ(e){if(typeof Symbol<"u"&&e[Symbol.iterator]!=null||e["@@iterator"]!=null)return Array.from(e)}function aZ(e){if(Array.isArray(e))return Z0(e)}function OC(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function Vt(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?OC(Object(n),!0).forEach(function(r){V0(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):OC(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function V0(e,t,n){return t=iZ(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function iZ(e){var t=oZ(e,"string");return xs(t)==="symbol"?t:String(t)}function oZ(e,t){if(xs(e)!=="object"||e===null)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(xs(r)!=="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return(t==="string"?String:Number)(e)}function uZ(e,t){return cZ(e)||sZ(e,t)||nD(e,t)||lZ()}function lZ(){throw new TypeError(`Invalid attempt to destructure non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function nD(e,t){if(e){if(typeof e=="string")return Z0(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);if(n==="Object"&&e.constructor&&(n=e.constructor.name),n==="Map"||n==="Set")return Array.from(e);if(n==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return Z0(e,t)}}function Z0(e,t){(t==null||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}function sZ(e,t){var n=e==null?null:typeof Symbol<"u"&&e[Symbol.iterator]||e["@@iterator"];if(n!=null){var r,i,u,s,c=[],d=!0,h=!1;try{if(u=(n=n.call(e)).next,t!==0)for(;!(d=(r=u.call(n)).done)&&(c.push(r.value),c.length!==t);d=!0);}catch(v){h=!0,i=v}finally{try{if(!d&&n.return!=null&&(s=n.return(),Object(s)!==s))return}finally{if(h)throw i}}return c}}function cZ(e){if(Array.isArray(e))return e}var yd=function(t,n,r){return t+(n-t)*r},Y0=function(t){var n=t.from,r=t.to;return n!==r},fZ=function e(t,n,r){var i=Dl(function(u,s){if(Y0(s)){var c=t(s.from,s.to,s.velocity),d=uZ(c,2),h=d[0],v=d[1];return Vt(Vt({},s),{},{from:h,velocity:v})}return s},n);return r<1?Dl(function(u,s){return Y0(s)?Vt(Vt({},s),{},{velocity:yd(s.velocity,i[u].velocity,r),from:yd(s.from,i[u].from,r)}):s},n):e(t,i,r-1)};const dZ=function(e,t,n,r,i){var u=UV(e,t),s=u.reduce(function(x,E){return Vt(Vt({},x),{},V0({},E,[e[E],t[E]]))},{}),c=u.reduce(function(x,E){return Vt(Vt({},x),{},V0({},E,{from:e[E],velocity:0,to:t[E]}))},{}),d=-1,h,v,y=function(){return null},g=function(){return Dl(function(E,M){return M.from},c)},b=function(){return!Object.values(c).filter(Y0).length},S=function(E){h||(h=E);var M=E-h,A=M/n.dt;c=fZ(n,c,A),i(Vt(Vt(Vt({},e),t),g())),h=E,b()||(d=requestAnimationFrame(y))},_=function(E){v||(v=E);var M=(E-v)/r,A=Dl(function(w,T){return yd.apply(void 0,wC(T).concat([n(M)]))},s);if(i(Vt(Vt(Vt({},e),t),A)),M<1)d=requestAnimationFrame(y);else{var C=Dl(function(w,T){return yd.apply(void 0,wC(T).concat([n(1)]))},s);i(Vt(Vt(Vt({},e),t),C))}};return y=n.isStepper?S:_,function(){return requestAnimationFrame(y),function(){cancelAnimationFrame(d)}}};function Xo(e){"@babel/helpers - typeof";return Xo=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},Xo(e)}var hZ=["children","begin","duration","attributeName","easing","isActive","steps","from","to","canBegin","onAnimationEnd","shouldReAnimate","onAnimationReStart"];function pZ(e,t){if(e==null)return{};var n=vZ(e,t),r,i;if(Object.getOwnPropertySymbols){var u=Object.getOwnPropertySymbols(e);for(i=0;i<u.length;i++)r=u[i],!(t.indexOf(r)>=0)&&Object.prototype.propertyIsEnumerable.call(e,r)&&(n[r]=e[r])}return n}function vZ(e,t){if(e==null)return{};var n={},r=Object.keys(e),i,u;for(u=0;u<r.length;u++)i=r[u],!(t.indexOf(i)>=0)&&(n[i]=e[i]);return n}function wb(e){return bZ(e)||gZ(e)||mZ(e)||yZ()}function yZ(){throw new TypeError(`Invalid attempt to spread non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function mZ(e,t){if(e){if(typeof e=="string")return X0(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);if(n==="Object"&&e.constructor&&(n=e.constructor.name),n==="Map"||n==="Set")return Array.from(e);if(n==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return X0(e,t)}}function gZ(e){if(typeof Symbol<"u"&&e[Symbol.iterator]!=null||e["@@iterator"]!=null)return Array.from(e)}function bZ(e){if(Array.isArray(e))return X0(e)}function X0(e,t){(t==null||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}function AC(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function Yn(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?AC(Object(n),!0).forEach(function(r){Cl(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):AC(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function Cl(e,t,n){return t=rD(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function xZ(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function _Z(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,rD(r.key),r)}}function SZ(e,t,n){return t&&_Z(e.prototype,t),Object.defineProperty(e,"prototype",{writable:!1}),e}function rD(e){var t=wZ(e,"string");return Xo(t)==="symbol"?t:String(t)}function wZ(e,t){if(Xo(e)!=="object"||e===null)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(Xo(r)!=="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return(t==="string"?String:Number)(e)}function OZ(e,t){if(typeof t!="function"&&t!==null)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),Object.defineProperty(e,"prototype",{writable:!1}),t&&K0(e,t)}function K0(e,t){return K0=Object.setPrototypeOf?Object.setPrototypeOf.bind():function(r,i){return r.__proto__=i,r},K0(e,t)}function AZ(e){var t=EZ();return function(){var r=md(e),i;if(t){var u=md(this).constructor;i=Reflect.construct(r,arguments,u)}else i=r.apply(this,arguments);return W0(this,i)}}function W0(e,t){if(t&&(Xo(t)==="object"||typeof t=="function"))return t;if(t!==void 0)throw new TypeError("Derived constructors may only return object or undefined");return F0(e)}function F0(e){if(e===void 0)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function EZ(){if(typeof Reflect>"u"||!Reflect.construct||Reflect.construct.sham)return!1;if(typeof Proxy=="function")return!0;try{return Boolean.prototype.valueOf.call(Reflect.construct(Boolean,[],function(){})),!0}catch{return!1}}function md(e){return md=Object.setPrototypeOf?Object.getPrototypeOf.bind():function(n){return n.__proto__||Object.getPrototypeOf(n)},md(e)}var Kr=function(e){OZ(n,e);var t=AZ(n);function n(r,i){var u;xZ(this,n),u=t.call(this,r,i);var s=u.props,c=s.isActive,d=s.attributeName,h=s.from,v=s.to,y=s.steps,g=s.children,b=s.duration;if(u.handleStyleChange=u.handleStyleChange.bind(F0(u)),u.changeStyle=u.changeStyle.bind(F0(u)),!c||b<=0)return u.state={style:{}},typeof g=="function"&&(u.state={style:v}),W0(u);if(y&&y.length)u.state={style:y[0].style};else if(h){if(typeof g=="function")return u.state={style:h},W0(u);u.state={style:d?Cl({},d,h):h}}else u.state={style:{}};return u}return SZ(n,[{key:"componentDidMount",value:function(){var i=this.props,u=i.isActive,s=i.canBegin;this.mounted=!0,!(!u||!s)&&this.runAnimation(this.props)}},{key:"componentDidUpdate",value:function(i){var u=this.props,s=u.isActive,c=u.canBegin,d=u.attributeName,h=u.shouldReAnimate,v=u.to,y=u.from,g=this.state.style;if(c){if(!s){var b={style:d?Cl({},d,v):v};this.state&&g&&(d&&g[d]!==v||!d&&g!==v)&&this.setState(b);return}if(!(RV(i.to,v)&&i.canBegin&&i.isActive)){var S=!i.canBegin||!i.isActive;this.manager&&this.manager.stop(),this.stopJSAnimation&&this.stopJSAnimation();var _=S||h?y:i.to;if(this.state&&g){var x={style:d?Cl({},d,_):_};(d&&g[d]!==_||!d&&g!==_)&&this.setState(x)}this.runAnimation(Yn(Yn({},this.props),{},{from:_,begin:0}))}}}},{key:"componentWillUnmount",value:function(){this.mounted=!1;var i=this.props.onAnimationEnd;this.unSubscribe&&this.unSubscribe(),this.manager&&(this.manager.stop(),this.manager=null),this.stopJSAnimation&&this.stopJSAnimation(),i&&i()}},{key:"handleStyleChange",value:function(i){this.changeStyle(i)}},{key:"changeStyle",value:function(i){this.mounted&&this.setState({style:i})}},{key:"runJSAnimation",value:function(i){var u=this,s=i.from,c=i.to,d=i.duration,h=i.easing,v=i.begin,y=i.onAnimationEnd,g=i.onAnimationStart,b=dZ(s,c,tZ(h),d,this.changeStyle),S=function(){u.stopJSAnimation=b()};this.manager.start([g,v,S,d,y])}},{key:"runStepAnimation",value:function(i){var u=this,s=i.steps,c=i.begin,d=i.onAnimationStart,h=s[0],v=h.style,y=h.duration,g=y===void 0?0:y,b=function(_,x,E){if(E===0)return _;var M=x.duration,A=x.easing,C=A===void 0?"ease":A,w=x.style,T=x.properties,P=x.onAnimationEnd,R=E>0?s[E-1]:x,$=T||Object.keys(w);if(typeof C=="function"||C==="spring")return[].concat(wb(_),[u.runJSAnimation.bind(u,{from:R.style,to:w,duration:M,easing:C}),M]);var J=xC($,M,C),B=Yn(Yn(Yn({},R.style),w),{},{transition:J});return[].concat(wb(_),[B,M,P]).filter(HV)};return this.manager.start([d].concat(wb(s.reduce(b,[v,Math.max(g,c)])),[i.onAnimationEnd]))}},{key:"runAnimation",value:function(i){this.manager||(this.manager=zV());var u=i.begin,s=i.duration,c=i.attributeName,d=i.to,h=i.easing,v=i.onAnimationStart,y=i.onAnimationEnd,g=i.steps,b=i.children,S=this.manager;if(this.unSubscribe=S.subscribe(this.handleStyleChange),typeof h=="function"||typeof b=="function"||h==="spring"){this.runJSAnimation(i);return}if(g.length>1){this.runStepAnimation(i);return}var _=c?Cl({},c,d):d,x=xC(Object.keys(_),s,h);S.start([v,u,Yn(Yn({},_),{},{transition:x}),s,y])}},{key:"render",value:function(){var i=this.props,u=i.children;i.begin;var s=i.duration;i.attributeName,i.easing;var c=i.isActive;i.steps,i.from,i.to,i.canBegin,i.onAnimationEnd,i.shouldReAnimate,i.onAnimationReStart;var d=pZ(i,hZ),h=q.Children.count(u),v=this.state.style;if(typeof u=="function")return u(v);if(!c||h===0||s<=0)return u;var y=function(b){var S=b.props,_=S.style,x=_===void 0?{}:_,E=S.className,M=q.cloneElement(b,Yn(Yn({},d),{},{style:Yn(Yn({},x),v),className:E}));return M};return h===1?y(q.Children.only(u)):L.createElement("div",null,q.Children.map(u,function(g){return y(g)}))}}]),n}(q.PureComponent);Kr.displayName="Animate";Kr.defaultProps={begin:0,duration:1e3,from:"",to:"",attributeName:"",easing:"ease",isActive:!0,canBegin:!0,steps:[],onAnimationEnd:function(){},onAnimationStart:function(){}};Kr.propTypes={from:Je.oneOfType([Je.object,Je.string]),to:Je.oneOfType([Je.object,Je.string]),attributeName:Je.string,duration:Je.number,begin:Je.number,easing:Je.oneOfType([Je.string,Je.func]),steps:Je.arrayOf(Je.shape({duration:Je.number.isRequired,style:Je.object.isRequired,easing:Je.oneOfType([Je.oneOf(["ease","ease-in","ease-out","ease-in-out","linear"]),Je.func]),properties:Je.arrayOf("string"),onAnimationEnd:Je.func})),children:Je.oneOfType([Je.node,Je.func]),isActive:Je.bool,canBegin:Je.bool,onAnimationEnd:Je.func,shouldReAnimate:Je.bool,onAnimationStart:Je.func,onAnimationReStart:Je.func};function _s(e){"@babel/helpers - typeof";return _s=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},_s(e)}function gd(){return gd=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},gd.apply(this,arguments)}function TZ(e,t){return PZ(e)||CZ(e,t)||jZ(e,t)||MZ()}function MZ(){throw new TypeError(`Invalid attempt to destructure non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function jZ(e,t){if(e){if(typeof e=="string")return EC(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);if(n==="Object"&&e.constructor&&(n=e.constructor.name),n==="Map"||n==="Set")return Array.from(e);if(n==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return EC(e,t)}}function EC(e,t){(t==null||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}function CZ(e,t){var n=e==null?null:typeof Symbol<"u"&&e[Symbol.iterator]||e["@@iterator"];if(n!=null){var r,i,u,s,c=[],d=!0,h=!1;try{if(u=(n=n.call(e)).next,t!==0)for(;!(d=(r=u.call(n)).done)&&(c.push(r.value),c.length!==t);d=!0);}catch(v){h=!0,i=v}finally{try{if(!d&&n.return!=null&&(s=n.return(),Object(s)!==s))return}finally{if(h)throw i}}return c}}function PZ(e){if(Array.isArray(e))return e}function TC(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function MC(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?TC(Object(n),!0).forEach(function(r){RZ(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):TC(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function RZ(e,t,n){return t=NZ(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function NZ(e){var t=DZ(e,"string");return _s(t)=="symbol"?t:t+""}function DZ(e,t){if(_s(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(_s(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return(t==="string"?String:Number)(e)}var jC=function(t,n,r,i,u){var s=Math.min(Math.abs(r)/2,Math.abs(i)/2),c=i>=0?1:-1,d=r>=0?1:-1,h=i>=0&&r>=0||i<0&&r<0?1:0,v;if(s>0&&u instanceof Array){for(var y=[0,0,0,0],g=0,b=4;g<b;g++)y[g]=u[g]>s?s:u[g];v="M".concat(t,",").concat(n+c*y[0]),y[0]>0&&(v+="A ".concat(y[0],",").concat(y[0],",0,0,").concat(h,",").concat(t+d*y[0],",").concat(n)),v+="L ".concat(t+r-d*y[1],",").concat(n),y[1]>0&&(v+="A ".concat(y[1],",").concat(y[1],",0,0,").concat(h,`,
        `).concat(t+r,",").concat(n+c*y[1])),v+="L ".concat(t+r,",").concat(n+i-c*y[2]),y[2]>0&&(v+="A ".concat(y[2],",").concat(y[2],",0,0,").concat(h,`,
        `).concat(t+r-d*y[2],",").concat(n+i)),v+="L ".concat(t+d*y[3],",").concat(n+i),y[3]>0&&(v+="A ".concat(y[3],",").concat(y[3],",0,0,").concat(h,`,
        `).concat(t,",").concat(n+i-c*y[3])),v+="Z"}else if(s>0&&u===+u&&u>0){var S=Math.min(s,u);v="M ".concat(t,",").concat(n+c*S,`
            A `).concat(S,",").concat(S,",0,0,").concat(h,",").concat(t+d*S,",").concat(n,`
            L `).concat(t+r-d*S,",").concat(n,`
            A `).concat(S,",").concat(S,",0,0,").concat(h,",").concat(t+r,",").concat(n+c*S,`
            L `).concat(t+r,",").concat(n+i-c*S,`
            A `).concat(S,",").concat(S,",0,0,").concat(h,",").concat(t+r-d*S,",").concat(n+i,`
            L `).concat(t+d*S,",").concat(n+i,`
            A `).concat(S,",").concat(S,",0,0,").concat(h,",").concat(t,",").concat(n+i-c*S," Z")}else v="M ".concat(t,",").concat(n," h ").concat(r," v ").concat(i," h ").concat(-r," Z");return v},kZ=function(t,n){if(!t||!n)return!1;var r=t.x,i=t.y,u=n.x,s=n.y,c=n.width,d=n.height;if(Math.abs(c)>0&&Math.abs(d)>0){var h=Math.min(u,u+c),v=Math.max(u,u+c),y=Math.min(s,s+d),g=Math.max(s,s+d);return r>=h&&r<=v&&i>=y&&i<=g}return!1},qZ={x:0,y:0,width:0,height:0,radius:0,isAnimationActive:!1,isUpdateAnimationActive:!1,animationBegin:0,animationDuration:1500,animationEasing:"ease"},O_=function(t){var n=MC(MC({},qZ),t),r=q.useRef(),i=q.useState(-1),u=TZ(i,2),s=u[0],c=u[1];q.useEffect(function(){if(r.current&&r.current.getTotalLength)try{var C=r.current.getTotalLength();C&&c(C)}catch{}},[]);var d=n.x,h=n.y,v=n.width,y=n.height,g=n.radius,b=n.className,S=n.animationEasing,_=n.animationDuration,x=n.animationBegin,E=n.isAnimationActive,M=n.isUpdateAnimationActive;if(d!==+d||h!==+h||v!==+v||y!==+y||v===0||y===0)return null;var A=Xe("recharts-rectangle",b);return M?L.createElement(Kr,{canBegin:s>0,from:{width:v,height:y,x:d,y:h},to:{width:v,height:y,x:d,y:h},duration:_,animationEasing:S,isActive:M},function(C){var w=C.width,T=C.height,P=C.x,R=C.y;return L.createElement(Kr,{canBegin:s>0,from:"0px ".concat(s===-1?1:s,"px"),to:"".concat(s,"px 0px"),attributeName:"strokeDasharray",begin:x,duration:_,isActive:E,easing:S},L.createElement("path",gd({},Ye(n,!0),{className:A,d:jC(P,R,w,T,g),ref:r})))}):L.createElement("path",gd({},Ye(n,!0),{className:A,d:jC(d,h,v,y,g)}))};function Q0(){return Q0=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},Q0.apply(this,arguments)}var aD=function(t){var n=t.cx,r=t.cy,i=t.r,u=t.className,s=Xe("recharts-dot",u);return n===+n&&r===+r&&i===+i?L.createElement("circle",Q0({},Ye(t,!1),If(t),{className:s,cx:n,cy:r,r:i})):null};function Ss(e){"@babel/helpers - typeof";return Ss=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},Ss(e)}var $Z=["x","y","top","left","width","height","className"];function J0(){return J0=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},J0.apply(this,arguments)}function CC(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function BZ(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?CC(Object(n),!0).forEach(function(r){zZ(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):CC(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function zZ(e,t,n){return t=LZ(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function LZ(e){var t=IZ(e,"string");return Ss(t)=="symbol"?t:t+""}function IZ(e,t){if(Ss(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(Ss(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return(t==="string"?String:Number)(e)}function UZ(e,t){if(e==null)return{};var n=HZ(e,t),r,i;if(Object.getOwnPropertySymbols){var u=Object.getOwnPropertySymbols(e);for(i=0;i<u.length;i++)r=u[i],!(t.indexOf(r)>=0)&&Object.prototype.propertyIsEnumerable.call(e,r)&&(n[r]=e[r])}return n}function HZ(e,t){if(e==null)return{};var n={};for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){if(t.indexOf(r)>=0)continue;n[r]=e[r]}return n}var GZ=function(t,n,r,i,u,s){return"M".concat(t,",").concat(u,"v").concat(i,"M").concat(s,",").concat(n,"h").concat(r)},VZ=function(t){var n=t.x,r=n===void 0?0:n,i=t.y,u=i===void 0?0:i,s=t.top,c=s===void 0?0:s,d=t.left,h=d===void 0?0:d,v=t.width,y=v===void 0?0:v,g=t.height,b=g===void 0?0:g,S=t.className,_=UZ(t,$Z),x=BZ({x:r,y:u,top:c,left:h,width:y,height:b},_);return!he(r)||!he(u)||!he(y)||!he(b)||!he(c)||!he(h)?null:L.createElement("path",J0({},Ye(x,!0),{className:Xe("recharts-cross",S),d:GZ(r,u,y,b,c,h)}))},Ob,PC;function ZZ(){if(PC)return Ob;PC=1;var e=AR(),t=e(Object.getPrototypeOf,Object);return Ob=t,Ob}var Ab,RC;function YZ(){if(RC)return Ab;RC=1;var e=Fr(),t=ZZ(),n=Qr(),r="[object Object]",i=Function.prototype,u=Object.prototype,s=i.toString,c=u.hasOwnProperty,d=s.call(Object);function h(v){if(!n(v)||e(v)!=r)return!1;var y=t(v);if(y===null)return!0;var g=c.call(y,"constructor")&&y.constructor;return typeof g=="function"&&g instanceof g&&s.call(g)==d}return Ab=h,Ab}var XZ=YZ();const KZ=lt(XZ);var Eb,NC;function WZ(){if(NC)return Eb;NC=1;var e=Fr(),t=Qr(),n="[object Boolean]";function r(i){return i===!0||i===!1||t(i)&&e(i)==n}return Eb=r,Eb}var FZ=WZ();const QZ=lt(FZ);function ws(e){"@babel/helpers - typeof";return ws=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},ws(e)}function bd(){return bd=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},bd.apply(this,arguments)}function JZ(e,t){return rY(e)||nY(e,t)||tY(e,t)||eY()}function eY(){throw new TypeError(`Invalid attempt to destructure non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function tY(e,t){if(e){if(typeof e=="string")return DC(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);if(n==="Object"&&e.constructor&&(n=e.constructor.name),n==="Map"||n==="Set")return Array.from(e);if(n==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return DC(e,t)}}function DC(e,t){(t==null||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}function nY(e,t){var n=e==null?null:typeof Symbol<"u"&&e[Symbol.iterator]||e["@@iterator"];if(n!=null){var r,i,u,s,c=[],d=!0,h=!1;try{if(u=(n=n.call(e)).next,t!==0)for(;!(d=(r=u.call(n)).done)&&(c.push(r.value),c.length!==t);d=!0);}catch(v){h=!0,i=v}finally{try{if(!d&&n.return!=null&&(s=n.return(),Object(s)!==s))return}finally{if(h)throw i}}return c}}function rY(e){if(Array.isArray(e))return e}function kC(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function qC(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?kC(Object(n),!0).forEach(function(r){aY(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):kC(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function aY(e,t,n){return t=iY(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function iY(e){var t=oY(e,"string");return ws(t)=="symbol"?t:t+""}function oY(e,t){if(ws(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(ws(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return(t==="string"?String:Number)(e)}var $C=function(t,n,r,i,u){var s=r-i,c;return c="M ".concat(t,",").concat(n),c+="L ".concat(t+r,",").concat(n),c+="L ".concat(t+r-s/2,",").concat(n+u),c+="L ".concat(t+r-s/2-i,",").concat(n+u),c+="L ".concat(t,",").concat(n," Z"),c},uY={x:0,y:0,upperWidth:0,lowerWidth:0,height:0,isUpdateAnimationActive:!1,animationBegin:0,animationDuration:1500,animationEasing:"ease"},lY=function(t){var n=qC(qC({},uY),t),r=q.useRef(),i=q.useState(-1),u=JZ(i,2),s=u[0],c=u[1];q.useEffect(function(){if(r.current&&r.current.getTotalLength)try{var A=r.current.getTotalLength();A&&c(A)}catch{}},[]);var d=n.x,h=n.y,v=n.upperWidth,y=n.lowerWidth,g=n.height,b=n.className,S=n.animationEasing,_=n.animationDuration,x=n.animationBegin,E=n.isUpdateAnimationActive;if(d!==+d||h!==+h||v!==+v||y!==+y||g!==+g||v===0&&y===0||g===0)return null;var M=Xe("recharts-trapezoid",b);return E?L.createElement(Kr,{canBegin:s>0,from:{upperWidth:0,lowerWidth:0,height:g,x:d,y:h},to:{upperWidth:v,lowerWidth:y,height:g,x:d,y:h},duration:_,animationEasing:S,isActive:E},function(A){var C=A.upperWidth,w=A.lowerWidth,T=A.height,P=A.x,R=A.y;return L.createElement(Kr,{canBegin:s>0,from:"0px ".concat(s===-1?1:s,"px"),to:"".concat(s,"px 0px"),attributeName:"strokeDasharray",begin:x,duration:_,easing:S},L.createElement("path",bd({},Ye(n,!0),{className:M,d:$C(P,R,C,w,T),ref:r})))}):L.createElement("g",null,L.createElement("path",bd({},Ye(n,!0),{className:M,d:$C(d,h,v,y,g)})))},sY=["option","shapeType","propTransformer","activeClassName","isActive"];function Os(e){"@babel/helpers - typeof";return Os=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},Os(e)}function cY(e,t){if(e==null)return{};var n=fY(e,t),r,i;if(Object.getOwnPropertySymbols){var u=Object.getOwnPropertySymbols(e);for(i=0;i<u.length;i++)r=u[i],!(t.indexOf(r)>=0)&&Object.prototype.propertyIsEnumerable.call(e,r)&&(n[r]=e[r])}return n}function fY(e,t){if(e==null)return{};var n={};for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){if(t.indexOf(r)>=0)continue;n[r]=e[r]}return n}function BC(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function xd(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?BC(Object(n),!0).forEach(function(r){dY(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):BC(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function dY(e,t,n){return t=hY(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function hY(e){var t=pY(e,"string");return Os(t)=="symbol"?t:t+""}function pY(e,t){if(Os(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(Os(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return(t==="string"?String:Number)(e)}function vY(e,t){return xd(xd({},t),e)}function yY(e,t){return e==="symbols"}function zC(e){var t=e.shapeType,n=e.elementProps;switch(t){case"rectangle":return L.createElement(O_,n);case"trapezoid":return L.createElement(lY,n);case"sector":return L.createElement(WN,n);case"symbols":if(yY(t))return L.createElement(Xd,n);break;default:return null}}function mY(e){return q.isValidElement(e)?e.props:e}function ex(e){var t=e.option,n=e.shapeType,r=e.propTransformer,i=r===void 0?vY:r,u=e.activeClassName,s=u===void 0?"recharts-active-shape":u,c=e.isActive,d=cY(e,sY),h;if(q.isValidElement(t))h=q.cloneElement(t,xd(xd({},d),mY(t)));else if(Ge(t))h=t(d);else if(KZ(t)&&!QZ(t)){var v=i(t,d);h=L.createElement(zC,{shapeType:n,elementProps:v})}else{var y=d;h=L.createElement(zC,{shapeType:n,elementProps:y})}return c?L.createElement(gt,{className:s},h):h}function sh(e,t){return t!=null&&"trapezoids"in e.props}function ch(e,t){return t!=null&&"sectors"in e.props}function As(e,t){return t!=null&&"points"in e.props}function gY(e,t){var n,r,i=e.x===(t==null||(n=t.labelViewBox)===null||n===void 0?void 0:n.x)||e.x===t.x,u=e.y===(t==null||(r=t.labelViewBox)===null||r===void 0?void 0:r.y)||e.y===t.y;return i&&u}function bY(e,t){var n=e.endAngle===t.endAngle,r=e.startAngle===t.startAngle;return n&&r}function xY(e,t){var n=e.x===t.x,r=e.y===t.y,i=e.z===t.z;return n&&r&&i}function _Y(e,t){var n;return sh(e,t)?n=gY:ch(e,t)?n=bY:As(e,t)&&(n=xY),n}function SY(e,t){var n;return sh(e,t)?n="trapezoids":ch(e,t)?n="sectors":As(e,t)&&(n="points"),n}function wY(e,t){if(sh(e,t)){var n;return(n=t.tooltipPayload)===null||n===void 0||(n=n[0])===null||n===void 0||(n=n.payload)===null||n===void 0?void 0:n.payload}if(ch(e,t)){var r;return(r=t.tooltipPayload)===null||r===void 0||(r=r[0])===null||r===void 0||(r=r.payload)===null||r===void 0?void 0:r.payload}return As(e,t)?t.payload:{}}function OY(e){var t=e.activeTooltipItem,n=e.graphicalItem,r=e.itemData,i=SY(n,t),u=wY(n,t),s=r.filter(function(d,h){var v=oh(u,d),y=n.props[i].filter(function(S){var _=_Y(n,t);return _(S,t)}),g=n.props[i].indexOf(y[y.length-1]),b=h===g;return v&&b}),c=r.indexOf(s[s.length-1]);return c}var Tb,LC;function AY(){if(LC)return Tb;LC=1;var e=Math.ceil,t=Math.max;function n(r,i,u,s){for(var c=-1,d=t(e((i-r)/(u||1)),0),h=Array(d);d--;)h[s?d:++c]=r,r+=u;return h}return Tb=n,Tb}var Mb,IC;function iD(){if(IC)return Mb;IC=1;var e=IR(),t=1/0,n=17976931348623157e292;function r(i){if(!i)return i===0?i:0;if(i=e(i),i===t||i===-1/0){var u=i<0?-1:1;return u*n}return i===i?i:0}return Mb=r,Mb}var jb,UC;function EY(){if(UC)return jb;UC=1;var e=AY(),t=Fd(),n=iD();function r(i){return function(u,s,c){return c&&typeof c!="number"&&t(u,s,c)&&(s=c=void 0),u=n(u),s===void 0?(s=u,u=0):s=n(s),c=c===void 0?u<s?1:-1:n(c),e(u,s,c,i)}}return jb=r,jb}var Cb,HC;function TY(){if(HC)return Cb;HC=1;var e=EY(),t=e();return Cb=t,Cb}var MY=TY();const _d=lt(MY);function Es(e){"@babel/helpers - typeof";return Es=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},Es(e)}function GC(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function VC(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?GC(Object(n),!0).forEach(function(r){oD(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):GC(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function oD(e,t,n){return t=jY(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function jY(e){var t=CY(e,"string");return Es(t)=="symbol"?t:t+""}function CY(e,t){if(Es(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(Es(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return(t==="string"?String:Number)(e)}var PY=["Webkit","Moz","O","ms"],RY=function(t,n){var r=t.replace(/(\w)/,function(u){return u.toUpperCase()}),i=PY.reduce(function(u,s){return VC(VC({},u),{},oD({},s+r,n))},{});return i[t]=n,i};function Ko(e){"@babel/helpers - typeof";return Ko=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},Ko(e)}function Sd(){return Sd=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},Sd.apply(this,arguments)}function ZC(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function Pb(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?ZC(Object(n),!0).forEach(function(r){_n(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):ZC(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function NY(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function YC(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,lD(r.key),r)}}function DY(e,t,n){return t&&YC(e.prototype,t),n&&YC(e,n),Object.defineProperty(e,"prototype",{writable:!1}),e}function kY(e,t,n){return t=wd(t),qY(e,uD()?Reflect.construct(t,n||[],wd(e).constructor):t.apply(e,n))}function qY(e,t){if(t&&(Ko(t)==="object"||typeof t=="function"))return t;if(t!==void 0)throw new TypeError("Derived constructors may only return object or undefined");return $Y(e)}function $Y(e){if(e===void 0)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function uD(){try{var e=!Boolean.prototype.valueOf.call(Reflect.construct(Boolean,[],function(){}))}catch{}return(uD=function(){return!!e})()}function wd(e){return wd=Object.setPrototypeOf?Object.getPrototypeOf.bind():function(n){return n.__proto__||Object.getPrototypeOf(n)},wd(e)}function BY(e,t){if(typeof t!="function"&&t!==null)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),Object.defineProperty(e,"prototype",{writable:!1}),t&&tx(e,t)}function tx(e,t){return tx=Object.setPrototypeOf?Object.setPrototypeOf.bind():function(r,i){return r.__proto__=i,r},tx(e,t)}function _n(e,t,n){return t=lD(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function lD(e){var t=zY(e,"string");return Ko(t)=="symbol"?t:t+""}function zY(e,t){if(Ko(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(Ko(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return String(e)}var LY=function(t){var n=t.data,r=t.startIndex,i=t.endIndex,u=t.x,s=t.width,c=t.travellerWidth;if(!n||!n.length)return{};var d=n.length,h=Rl().domain(_d(0,d)).range([u,u+s-c]),v=h.domain().map(function(y){return h(y)});return{isTextActive:!1,isSlideMoving:!1,isTravellerMoving:!1,isTravellerFocused:!1,startX:h(r),endX:h(i),scale:h,scaleValues:v}},XC=function(t){return t.changedTouches&&!!t.changedTouches.length},Wo=function(e){function t(n){var r;return NY(this,t),r=kY(this,t,[n]),_n(r,"handleDrag",function(i){r.leaveTimer&&(clearTimeout(r.leaveTimer),r.leaveTimer=null),r.state.isTravellerMoving?r.handleTravellerMove(i):r.state.isSlideMoving&&r.handleSlideDrag(i)}),_n(r,"handleTouchMove",function(i){i.changedTouches!=null&&i.changedTouches.length>0&&r.handleDrag(i.changedTouches[0])}),_n(r,"handleDragEnd",function(){r.setState({isTravellerMoving:!1,isSlideMoving:!1},function(){var i=r.props,u=i.endIndex,s=i.onDragEnd,c=i.startIndex;s==null||s({endIndex:u,startIndex:c})}),r.detachDragEndListener()}),_n(r,"handleLeaveWrapper",function(){(r.state.isTravellerMoving||r.state.isSlideMoving)&&(r.leaveTimer=window.setTimeout(r.handleDragEnd,r.props.leaveTimeOut))}),_n(r,"handleEnterSlideOrTraveller",function(){r.setState({isTextActive:!0})}),_n(r,"handleLeaveSlideOrTraveller",function(){r.setState({isTextActive:!1})}),_n(r,"handleSlideDragStart",function(i){var u=XC(i)?i.changedTouches[0]:i;r.setState({isTravellerMoving:!1,isSlideMoving:!0,slideMoveStartX:u.pageX}),r.attachDragEndListener()}),r.travellerDragStartHandlers={startX:r.handleTravellerDragStart.bind(r,"startX"),endX:r.handleTravellerDragStart.bind(r,"endX")},r.state={},r}return BY(t,e),DY(t,[{key:"componentWillUnmount",value:function(){this.leaveTimer&&(clearTimeout(this.leaveTimer),this.leaveTimer=null),this.detachDragEndListener()}},{key:"getIndex",value:function(r){var i=r.startX,u=r.endX,s=this.state.scaleValues,c=this.props,d=c.gap,h=c.data,v=h.length-1,y=Math.min(i,u),g=Math.max(i,u),b=t.getIndexInRange(s,y),S=t.getIndexInRange(s,g);return{startIndex:b-b%d,endIndex:S===v?v:S-S%d}}},{key:"getTextOfTick",value:function(r){var i=this.props,u=i.data,s=i.tickFormatter,c=i.dataKey,d=nn(u[r],c,r);return Ge(s)?s(d,r):d}},{key:"attachDragEndListener",value:function(){window.addEventListener("mouseup",this.handleDragEnd,!0),window.addEventListener("touchend",this.handleDragEnd,!0),window.addEventListener("mousemove",this.handleDrag,!0)}},{key:"detachDragEndListener",value:function(){window.removeEventListener("mouseup",this.handleDragEnd,!0),window.removeEventListener("touchend",this.handleDragEnd,!0),window.removeEventListener("mousemove",this.handleDrag,!0)}},{key:"handleSlideDrag",value:function(r){var i=this.state,u=i.slideMoveStartX,s=i.startX,c=i.endX,d=this.props,h=d.x,v=d.width,y=d.travellerWidth,g=d.startIndex,b=d.endIndex,S=d.onChange,_=r.pageX-u;_>0?_=Math.min(_,h+v-y-c,h+v-y-s):_<0&&(_=Math.max(_,h-s,h-c));var x=this.getIndex({startX:s+_,endX:c+_});(x.startIndex!==g||x.endIndex!==b)&&S&&S(x),this.setState({startX:s+_,endX:c+_,slideMoveStartX:r.pageX})}},{key:"handleTravellerDragStart",value:function(r,i){var u=XC(i)?i.changedTouches[0]:i;this.setState({isSlideMoving:!1,isTravellerMoving:!0,movingTravellerId:r,brushMoveStartX:u.pageX}),this.attachDragEndListener()}},{key:"handleTravellerMove",value:function(r){var i=this.state,u=i.brushMoveStartX,s=i.movingTravellerId,c=i.endX,d=i.startX,h=this.state[s],v=this.props,y=v.x,g=v.width,b=v.travellerWidth,S=v.onChange,_=v.gap,x=v.data,E={startX:this.state.startX,endX:this.state.endX},M=r.pageX-u;M>0?M=Math.min(M,y+g-b-h):M<0&&(M=Math.max(M,y-h)),E[s]=h+M;var A=this.getIndex(E),C=A.startIndex,w=A.endIndex,T=function(){var R=x.length-1;return s==="startX"&&(c>d?C%_===0:w%_===0)||c<d&&w===R||s==="endX"&&(c>d?w%_===0:C%_===0)||c>d&&w===R};this.setState(_n(_n({},s,h+M),"brushMoveStartX",r.pageX),function(){S&&T()&&S(A)})}},{key:"handleTravellerMoveKeyboard",value:function(r,i){var u=this,s=this.state,c=s.scaleValues,d=s.startX,h=s.endX,v=this.state[i],y=c.indexOf(v);if(y!==-1){var g=y+r;if(!(g===-1||g>=c.length)){var b=c[g];i==="startX"&&b>=h||i==="endX"&&b<=d||this.setState(_n({},i,b),function(){u.props.onChange(u.getIndex({startX:u.state.startX,endX:u.state.endX}))})}}}},{key:"renderBackground",value:function(){var r=this.props,i=r.x,u=r.y,s=r.width,c=r.height,d=r.fill,h=r.stroke;return L.createElement("rect",{stroke:h,fill:d,x:i,y:u,width:s,height:c})}},{key:"renderPanorama",value:function(){var r=this.props,i=r.x,u=r.y,s=r.width,c=r.height,d=r.data,h=r.children,v=r.padding,y=q.Children.only(h);return y?L.cloneElement(y,{x:i,y:u,width:s,height:c,margin:v,compact:!0,data:d}):null}},{key:"renderTravellerLayer",value:function(r,i){var u,s,c=this,d=this.props,h=d.y,v=d.travellerWidth,y=d.height,g=d.traveller,b=d.ariaLabel,S=d.data,_=d.startIndex,x=d.endIndex,E=Math.max(r,this.props.x),M=Pb(Pb({},Ye(this.props,!1)),{},{x:E,y:h,width:v,height:y}),A=b||"Min value: ".concat((u=S[_])===null||u===void 0?void 0:u.name,", Max value: ").concat((s=S[x])===null||s===void 0?void 0:s.name);return L.createElement(gt,{tabIndex:0,role:"slider","aria-label":A,"aria-valuenow":r,className:"recharts-brush-traveller",onMouseEnter:this.handleEnterSlideOrTraveller,onMouseLeave:this.handleLeaveSlideOrTraveller,onMouseDown:this.travellerDragStartHandlers[i],onTouchStart:this.travellerDragStartHandlers[i],onKeyDown:function(w){["ArrowLeft","ArrowRight"].includes(w.key)&&(w.preventDefault(),w.stopPropagation(),c.handleTravellerMoveKeyboard(w.key==="ArrowRight"?1:-1,i))},onFocus:function(){c.setState({isTravellerFocused:!0})},onBlur:function(){c.setState({isTravellerFocused:!1})},style:{cursor:"col-resize"}},t.renderTraveller(g,M))}},{key:"renderSlide",value:function(r,i){var u=this.props,s=u.y,c=u.height,d=u.stroke,h=u.travellerWidth,v=Math.min(r,i)+h,y=Math.max(Math.abs(i-r)-h,0);return L.createElement("rect",{className:"recharts-brush-slide",onMouseEnter:this.handleEnterSlideOrTraveller,onMouseLeave:this.handleLeaveSlideOrTraveller,onMouseDown:this.handleSlideDragStart,onTouchStart:this.handleSlideDragStart,style:{cursor:"move"},stroke:"none",fill:d,fillOpacity:.2,x:v,y:s,width:y,height:c})}},{key:"renderText",value:function(){var r=this.props,i=r.startIndex,u=r.endIndex,s=r.y,c=r.height,d=r.travellerWidth,h=r.stroke,v=this.state,y=v.startX,g=v.endX,b=5,S={pointerEvents:"none",fill:h};return L.createElement(gt,{className:"recharts-brush-texts"},L.createElement(Qf,Sd({textAnchor:"end",verticalAnchor:"middle",x:Math.min(y,g)-b,y:s+c/2},S),this.getTextOfTick(i)),L.createElement(Qf,Sd({textAnchor:"start",verticalAnchor:"middle",x:Math.max(y,g)+d+b,y:s+c/2},S),this.getTextOfTick(u)))}},{key:"render",value:function(){var r=this.props,i=r.data,u=r.className,s=r.children,c=r.x,d=r.y,h=r.width,v=r.height,y=r.alwaysShowText,g=this.state,b=g.startX,S=g.endX,_=g.isTextActive,x=g.isSlideMoving,E=g.isTravellerMoving,M=g.isTravellerFocused;if(!i||!i.length||!he(c)||!he(d)||!he(h)||!he(v)||h<=0||v<=0)return null;var A=Xe("recharts-brush",u),C=L.Children.count(s)===1,w=RY("userSelect","none");return L.createElement(gt,{className:A,onMouseLeave:this.handleLeaveWrapper,onTouchMove:this.handleTouchMove,style:w},this.renderBackground(),C&&this.renderPanorama(),this.renderSlide(b,S),this.renderTravellerLayer(b,"startX"),this.renderTravellerLayer(S,"endX"),(_||x||E||M||y)&&this.renderText())}}],[{key:"renderDefaultTraveller",value:function(r){var i=r.x,u=r.y,s=r.width,c=r.height,d=r.stroke,h=Math.floor(u+c/2)-1;return L.createElement(L.Fragment,null,L.createElement("rect",{x:i,y:u,width:s,height:c,fill:d,stroke:"none"}),L.createElement("line",{x1:i+1,y1:h,x2:i+s-1,y2:h,fill:"none",stroke:"#fff"}),L.createElement("line",{x1:i+1,y1:h+2,x2:i+s-1,y2:h+2,fill:"none",stroke:"#fff"}))}},{key:"renderTraveller",value:function(r,i){var u;return L.isValidElement(r)?u=L.cloneElement(r,i):Ge(r)?u=r(i):u=t.renderDefaultTraveller(i),u}},{key:"getDerivedStateFromProps",value:function(r,i){var u=r.data,s=r.width,c=r.x,d=r.travellerWidth,h=r.updateId,v=r.startIndex,y=r.endIndex;if(u!==i.prevData||h!==i.prevUpdateId)return Pb({prevData:u,prevTravellerWidth:d,prevUpdateId:h,prevX:c,prevWidth:s},u&&u.length?LY({data:u,width:s,x:c,travellerWidth:d,startIndex:v,endIndex:y}):{scale:null,scaleValues:null});if(i.scale&&(s!==i.prevWidth||c!==i.prevX||d!==i.prevTravellerWidth)){i.scale.range([c,c+s-d]);var g=i.scale.domain().map(function(b){return i.scale(b)});return{prevData:u,prevTravellerWidth:d,prevUpdateId:h,prevX:c,prevWidth:s,startX:i.scale(r.startIndex),endX:i.scale(r.endIndex),scaleValues:g}}return null}},{key:"getIndexInRange",value:function(r,i){for(var u=r.length,s=0,c=u-1;c-s>1;){var d=Math.floor((s+c)/2);r[d]>i?c=d:s=d}return i>=r[c]?c:s}}])}(q.PureComponent);_n(Wo,"displayName","Brush");_n(Wo,"defaultProps",{height:40,travellerWidth:5,gap:1,fill:"#fff",stroke:"#666",padding:{top:1,right:1,bottom:1,left:1},leaveTimeOut:1e3,alwaysShowText:!1});var Rb,KC;function IY(){if(KC)return Rb;KC=1;var e=Kx();function t(n,r){var i;return e(n,function(u,s,c){return i=r(u,s,c),!i}),!!i}return Rb=t,Rb}var Nb,WC;function UY(){if(WC)return Nb;WC=1;var e=gR(),t=$a(),n=IY(),r=fn(),i=Fd();function u(s,c,d){var h=r(s)?e:n;return d&&i(s,c,d)&&(c=void 0),h(s,t(c,3))}return Nb=u,Nb}var HY=UY();const GY=lt(HY);var vr=function(t,n){var r=t.alwaysShow,i=t.ifOverflow;return r&&(i="extendDomain"),i===n},Db,FC;function VY(){if(FC)return Db;FC=1;var e=qR();function t(n,r,i){r=="__proto__"&&e?e(n,r,{configurable:!0,enumerable:!0,value:i,writable:!0}):n[r]=i}return Db=t,Db}var kb,QC;function ZY(){if(QC)return kb;QC=1;var e=VY(),t=DR(),n=$a();function r(i,u){var s={};return u=n(u,3),t(i,function(c,d,h){e(s,d,u(c,d,h))}),s}return kb=r,kb}var YY=ZY();const XY=lt(YY);var qb,JC;function KY(){if(JC)return qb;JC=1;function e(t,n){for(var r=-1,i=t==null?0:t.length;++r<i;)if(!n(t[r],r,t))return!1;return!0}return qb=e,qb}var $b,e2;function WY(){if(e2)return $b;e2=1;var e=Kx();function t(n,r){var i=!0;return e(n,function(u,s,c){return i=!!r(u,s,c),i}),i}return $b=t,$b}var Bb,t2;function FY(){if(t2)return Bb;t2=1;var e=KY(),t=WY(),n=$a(),r=fn(),i=Fd();function u(s,c,d){var h=r(s)?e:t;return d&&i(s,c,d)&&(c=void 0),h(s,n(c,3))}return Bb=u,Bb}var QY=FY();const JY=lt(QY);var eX=["x","y"];function Ts(e){"@babel/helpers - typeof";return Ts=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},Ts(e)}function nx(){return nx=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},nx.apply(this,arguments)}function n2(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function Al(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?n2(Object(n),!0).forEach(function(r){tX(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):n2(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function tX(e,t,n){return t=nX(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function nX(e){var t=rX(e,"string");return Ts(t)=="symbol"?t:t+""}function rX(e,t){if(Ts(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(Ts(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return(t==="string"?String:Number)(e)}function aX(e,t){if(e==null)return{};var n=iX(e,t),r,i;if(Object.getOwnPropertySymbols){var u=Object.getOwnPropertySymbols(e);for(i=0;i<u.length;i++)r=u[i],!(t.indexOf(r)>=0)&&Object.prototype.propertyIsEnumerable.call(e,r)&&(n[r]=e[r])}return n}function iX(e,t){if(e==null)return{};var n={};for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){if(t.indexOf(r)>=0)continue;n[r]=e[r]}return n}function oX(e,t){var n=e.x,r=e.y,i=aX(e,eX),u="".concat(n),s=parseInt(u,10),c="".concat(r),d=parseInt(c,10),h="".concat(t.height||i.height),v=parseInt(h,10),y="".concat(t.width||i.width),g=parseInt(y,10);return Al(Al(Al(Al(Al({},t),i),s?{x:s}:{}),d?{y:d}:{}),{},{height:v,width:g,name:t.name,radius:t.radius})}function r2(e){return L.createElement(ex,nx({shapeType:"rectangle",propTransformer:oX,activeClassName:"recharts-active-bar"},e))}var uX=function(t){var n=arguments.length>1&&arguments[1]!==void 0?arguments[1]:0;return function(r,i){if(typeof t=="number")return t;var u=typeof r=="number";return u?t(r,i):(u||Ai(),n)}},lX=["value","background"],sD;function Fo(e){"@babel/helpers - typeof";return Fo=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},Fo(e)}function sX(e,t){if(e==null)return{};var n=cX(e,t),r,i;if(Object.getOwnPropertySymbols){var u=Object.getOwnPropertySymbols(e);for(i=0;i<u.length;i++)r=u[i],!(t.indexOf(r)>=0)&&Object.prototype.propertyIsEnumerable.call(e,r)&&(n[r]=e[r])}return n}function cX(e,t){if(e==null)return{};var n={};for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){if(t.indexOf(r)>=0)continue;n[r]=e[r]}return n}function Od(){return Od=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},Od.apply(this,arguments)}function a2(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function Mt(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?a2(Object(n),!0).forEach(function(r){Ta(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):a2(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function fX(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function i2(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,fD(r.key),r)}}function dX(e,t,n){return t&&i2(e.prototype,t),n&&i2(e,n),Object.defineProperty(e,"prototype",{writable:!1}),e}function hX(e,t,n){return t=Ad(t),pX(e,cD()?Reflect.construct(t,n||[],Ad(e).constructor):t.apply(e,n))}function pX(e,t){if(t&&(Fo(t)==="object"||typeof t=="function"))return t;if(t!==void 0)throw new TypeError("Derived constructors may only return object or undefined");return vX(e)}function vX(e){if(e===void 0)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function cD(){try{var e=!Boolean.prototype.valueOf.call(Reflect.construct(Boolean,[],function(){}))}catch{}return(cD=function(){return!!e})()}function Ad(e){return Ad=Object.setPrototypeOf?Object.getPrototypeOf.bind():function(n){return n.__proto__||Object.getPrototypeOf(n)},Ad(e)}function yX(e,t){if(typeof t!="function"&&t!==null)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),Object.defineProperty(e,"prototype",{writable:!1}),t&&rx(e,t)}function rx(e,t){return rx=Object.setPrototypeOf?Object.setPrototypeOf.bind():function(r,i){return r.__proto__=i,r},rx(e,t)}function Ta(e,t,n){return t=fD(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function fD(e){var t=mX(e,"string");return Fo(t)=="symbol"?t:t+""}function mX(e,t){if(Fo(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(Fo(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return String(e)}var Hs=function(e){function t(){var n;fX(this,t);for(var r=arguments.length,i=new Array(r),u=0;u<r;u++)i[u]=arguments[u];return n=hX(this,t,[].concat(i)),Ta(n,"state",{isAnimationFinished:!1}),Ta(n,"id",qs("recharts-bar-")),Ta(n,"handleAnimationEnd",function(){var s=n.props.onAnimationEnd;n.setState({isAnimationFinished:!0}),s&&s()}),Ta(n,"handleAnimationStart",function(){var s=n.props.onAnimationStart;n.setState({isAnimationFinished:!1}),s&&s()}),n}return yX(t,e),dX(t,[{key:"renderRectanglesStatically",value:function(r){var i=this,u=this.props,s=u.shape,c=u.dataKey,d=u.activeIndex,h=u.activeBar,v=Ye(this.props,!1);return r&&r.map(function(y,g){var b=g===d,S=b?h:s,_=Mt(Mt(Mt({},v),y),{},{isActive:b,option:S,index:g,dataKey:c,onAnimationStart:i.handleAnimationStart,onAnimationEnd:i.handleAnimationEnd});return L.createElement(gt,Od({className:"recharts-bar-rectangle"},Wl(i.props,y,g),{key:"rectangle-".concat(y==null?void 0:y.x,"-").concat(y==null?void 0:y.y,"-").concat(y==null?void 0:y.value,"-").concat(g)}),L.createElement(r2,_))})}},{key:"renderRectanglesWithAnimation",value:function(){var r=this,i=this.props,u=i.data,s=i.layout,c=i.isAnimationActive,d=i.animationBegin,h=i.animationDuration,v=i.animationEasing,y=i.animationId,g=this.state.prevData;return L.createElement(Kr,{begin:d,duration:h,isActive:c,easing:v,from:{t:0},to:{t:1},key:"bar-".concat(y),onAnimationEnd:this.handleAnimationEnd,onAnimationStart:this.handleAnimationStart},function(b){var S=b.t,_=u.map(function(x,E){var M=g&&g[E];if(M){var A=cr(M.x,x.x),C=cr(M.y,x.y),w=cr(M.width,x.width),T=cr(M.height,x.height);return Mt(Mt({},x),{},{x:A(S),y:C(S),width:w(S),height:T(S)})}if(s==="horizontal"){var P=cr(0,x.height),R=P(S);return Mt(Mt({},x),{},{y:x.y+x.height-R,height:R})}var $=cr(0,x.width),J=$(S);return Mt(Mt({},x),{},{width:J})});return L.createElement(gt,null,r.renderRectanglesStatically(_))})}},{key:"renderRectangles",value:function(){var r=this.props,i=r.data,u=r.isAnimationActive,s=this.state.prevData;return u&&i&&i.length&&(!s||!oh(s,i))?this.renderRectanglesWithAnimation():this.renderRectanglesStatically(i)}},{key:"renderBackground",value:function(){var r=this,i=this.props,u=i.data,s=i.dataKey,c=i.activeIndex,d=Ye(this.props.background,!1);return u.map(function(h,v){h.value;var y=h.background,g=sX(h,lX);if(!y)return null;var b=Mt(Mt(Mt(Mt(Mt({},g),{},{fill:"#eee"},y),d),Wl(r.props,h,v)),{},{onAnimationStart:r.handleAnimationStart,onAnimationEnd:r.handleAnimationEnd,dataKey:s,index:v,className:"recharts-bar-background-rectangle"});return L.createElement(r2,Od({key:"background-bar-".concat(v),option:r.props.background,isActive:v===c},b))})}},{key:"renderErrorBar",value:function(r,i){if(this.props.isAnimationActive&&!this.state.isAnimationFinished)return null;var u=this.props,s=u.data,c=u.xAxis,d=u.yAxis,h=u.layout,v=u.children,y=An(v,Us);if(!y)return null;var g=h==="vertical"?s[0].height/2:s[0].width/2,b=function(x,E){var M=Array.isArray(x.value)?x.value[1]:x.value;return{x:x.x,y:x.y,value:M,errorVal:nn(x,E)}},S={clipPath:r?"url(#clipPath-".concat(i,")"):null};return L.createElement(gt,S,y.map(function(_){return L.cloneElement(_,{key:"error-bar-".concat(i,"-").concat(_.props.dataKey),data:s,xAxis:c,yAxis:d,layout:h,offset:g,dataPointFormatter:b})}))}},{key:"render",value:function(){var r=this.props,i=r.hide,u=r.data,s=r.className,c=r.xAxis,d=r.yAxis,h=r.left,v=r.top,y=r.width,g=r.height,b=r.isAnimationActive,S=r.background,_=r.id;if(i||!u||!u.length)return null;var x=this.state.isAnimationFinished,E=Xe("recharts-bar",s),M=c&&c.allowDataOverflow,A=d&&d.allowDataOverflow,C=M||A,w=ze(_)?this.id:_;return L.createElement(gt,{className:E},M||A?L.createElement("defs",null,L.createElement("clipPath",{id:"clipPath-".concat(w)},L.createElement("rect",{x:M?h:h-y/2,y:A?v:v-g/2,width:M?y:y*2,height:A?g:g*2}))):null,L.createElement(gt,{className:"recharts-bar-rectangles",clipPath:C?"url(#clipPath-".concat(w,")"):null},S?this.renderBackground():null,this.renderRectangles()),this.renderErrorBar(C,w),(!b||x)&&Pa.renderCallByParent(this.props,u))}}],[{key:"getDerivedStateFromProps",value:function(r,i){return r.animationId!==i.prevAnimationId?{prevAnimationId:r.animationId,curData:r.data,prevData:i.curData}:r.data!==i.curData?{curData:r.data}:null}}])}(q.PureComponent);sD=Hs;Ta(Hs,"displayName","Bar");Ta(Hs,"defaultProps",{xAxisId:0,yAxisId:0,legendType:"rect",minPointSize:0,hide:!1,data:[],layout:"vertical",activeBar:!1,isAnimationActive:!hu.isSsr,animationBegin:0,animationDuration:400,animationEasing:"ease"});Ta(Hs,"getComposedData",function(e){var t=e.props,n=e.item,r=e.barPosition,i=e.bandSize,u=e.xAxis,s=e.yAxis,c=e.xAxisTicks,d=e.yAxisTicks,h=e.stackedData,v=e.dataStartIndex,y=e.displayedData,g=e.offset,b=q7(r,n);if(!b)return null;var S=t.layout,_=n.type.defaultProps,x=_!==void 0?Mt(Mt({},_),n.props):n.props,E=x.dataKey,M=x.children,A=x.minPointSize,C=S==="horizontal"?s:u,w=h?C.scale.domain():null,T=G7({numericAxis:C}),P=An(M,Qx),R=y.map(function($,J){var B,G,W,ne,Q,D;h?B=$7(h[v+J],w):(B=nn($,E),Array.isArray(B)||(B=[T,B]));var H=uX(A,sD.defaultProps.minPointSize)(B[1],J);if(S==="horizontal"){var U,ue=[s.scale(B[0]),s.scale(B[1])],N=ue[0],X=ue[1];G=Uj({axis:u,ticks:c,bandSize:i,offset:b.offset,entry:$,index:J}),W=(U=X??N)!==null&&U!==void 0?U:void 0,ne=b.size;var ee=N-X;if(Q=Number.isNaN(ee)?0:ee,D={x:G,y:s.y,width:ne,height:s.height},Math.abs(H)>0&&Math.abs(Q)<Math.abs(H)){var te=Fn(Q||H)*(Math.abs(H)-Math.abs(Q));W-=te,Q+=te}}else{var ce=[u.scale(B[0]),u.scale(B[1])],_e=ce[0],be=ce[1];if(G=_e,W=Uj({axis:s,ticks:d,bandSize:i,offset:b.offset,entry:$,index:J}),ne=be-_e,Q=b.size,D={x:u.x,y:W,width:u.width,height:Q},Math.abs(H)>0&&Math.abs(ne)<Math.abs(H)){var ke=Fn(ne||H)*(Math.abs(H)-Math.abs(ne));ne+=ke}}return Mt(Mt(Mt({},$),{},{x:G,y:W,width:ne,height:Q,value:h?B:B[1],payload:$,background:D},P&&P[J]&&P[J].props),{},{tooltipPayload:[YN(n,$)],tooltipPosition:{x:G+ne/2,y:W+Q/2}})});return Mt({data:R,layout:S},g)});function Ms(e){"@babel/helpers - typeof";return Ms=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},Ms(e)}function gX(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function o2(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,dD(r.key),r)}}function bX(e,t,n){return t&&o2(e.prototype,t),n&&o2(e,n),Object.defineProperty(e,"prototype",{writable:!1}),e}function u2(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function Xn(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?u2(Object(n),!0).forEach(function(r){fh(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):u2(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function fh(e,t,n){return t=dD(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function dD(e){var t=xX(e,"string");return Ms(t)=="symbol"?t:t+""}function xX(e,t){if(Ms(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(Ms(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return(t==="string"?String:Number)(e)}var _X=function(t,n,r,i,u){var s=t.width,c=t.height,d=t.layout,h=t.children,v=Object.keys(n),y={left:r.left,leftMirror:r.left,right:s-r.right,rightMirror:s-r.right,top:r.top,topMirror:r.top,bottom:c-r.bottom,bottomMirror:c-r.bottom},g=!!Sn(h,Hs);return v.reduce(function(b,S){var _=n[S],x=_.orientation,E=_.domain,M=_.padding,A=M===void 0?{}:M,C=_.mirror,w=_.reversed,T="".concat(x).concat(C?"Mirror":""),P,R,$,J,B;if(_.type==="number"&&(_.padding==="gap"||_.padding==="no-gap")){var G=E[1]-E[0],W=1/0,ne=_.categoricalDomain.sort();if(ne.forEach(function(ce,_e){_e>0&&(W=Math.min((ce||0)-(ne[_e-1]||0),W))}),Number.isFinite(W)){var Q=W/G,D=_.layout==="vertical"?r.height:r.width;if(_.padding==="gap"&&(P=Q*D/2),_.padding==="no-gap"){var H=wi(t.barCategoryGap,Q*D),U=Q*D/2;P=U-H-(U-H)/D*H}}}i==="xAxis"?R=[r.left+(A.left||0)+(P||0),r.left+r.width-(A.right||0)-(P||0)]:i==="yAxis"?R=d==="horizontal"?[r.top+r.height-(A.bottom||0),r.top+(A.top||0)]:[r.top+(A.top||0)+(P||0),r.top+r.height-(A.bottom||0)-(P||0)]:R=_.range,w&&(R=[R[1],R[0]]);var ue=D7(_,u,g),N=ue.scale,X=ue.realScaleType;N.domain(E).range(R),k7(N);var ee=H7(N,Xn(Xn({},_),{},{realScaleType:X}));i==="xAxis"?(B=x==="top"&&!C||x==="bottom"&&C,$=r.left,J=y[T]-B*_.height):i==="yAxis"&&(B=x==="left"&&!C||x==="right"&&C,$=y[T]-B*_.width,J=r.top);var te=Xn(Xn(Xn({},_),ee),{},{realScaleType:X,x:$,y:J,scale:N,width:i==="xAxis"?r.width:_.width,height:i==="yAxis"?r.height:_.height});return te.bandSize=dd(te,ee),!_.hide&&i==="xAxis"?y[T]+=(B?-1:1)*te.height:_.hide||(y[T]+=(B?-1:1)*te.width),Xn(Xn({},b),{},fh({},S,te))},{})},hD=function(t,n){var r=t.x,i=t.y,u=n.x,s=n.y;return{x:Math.min(r,u),y:Math.min(i,s),width:Math.abs(u-r),height:Math.abs(s-i)}},SX=function(t){var n=t.x1,r=t.y1,i=t.x2,u=t.y2;return hD({x:n,y:r},{x:i,y:u})},pD=function(){function e(t){gX(this,e),this.scale=t}return bX(e,[{key:"domain",get:function(){return this.scale.domain}},{key:"range",get:function(){return this.scale.range}},{key:"rangeMin",get:function(){return this.range()[0]}},{key:"rangeMax",get:function(){return this.range()[1]}},{key:"bandwidth",get:function(){return this.scale.bandwidth}},{key:"apply",value:function(n){var r=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},i=r.bandAware,u=r.position;if(n!==void 0){if(u)switch(u){case"start":return this.scale(n);case"middle":{var s=this.bandwidth?this.bandwidth()/2:0;return this.scale(n)+s}case"end":{var c=this.bandwidth?this.bandwidth():0;return this.scale(n)+c}default:return this.scale(n)}if(i){var d=this.bandwidth?this.bandwidth()/2:0;return this.scale(n)+d}return this.scale(n)}}},{key:"isInRange",value:function(n){var r=this.range(),i=r[0],u=r[r.length-1];return i<=u?n>=i&&n<=u:n>=u&&n<=i}}],[{key:"create",value:function(n){return new e(n)}}])}();fh(pD,"EPS",1e-4);var A_=function(t){var n=Object.keys(t).reduce(function(r,i){return Xn(Xn({},r),{},fh({},i,pD.create(t[i])))},{});return Xn(Xn({},n),{},{apply:function(i){var u=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},s=u.bandAware,c=u.position;return XY(i,function(d,h){return n[h].apply(d,{bandAware:s,position:c})})},isInRange:function(i){return JY(i,function(u,s){return n[s].isInRange(u)})}})};function wX(e){return(e%180+180)%180}var OX=function(t){var n=t.width,r=t.height,i=arguments.length>1&&arguments[1]!==void 0?arguments[1]:0,u=wX(i),s=u*Math.PI/180,c=Math.atan(r/n),d=s>c&&s<Math.PI-c?r/Math.sin(s):n/Math.cos(s);return Math.abs(d)},zb,l2;function AX(){if(l2)return zb;l2=1;var e=$a(),t=$s(),n=Kd();function r(i){return function(u,s,c){var d=Object(u);if(!t(u)){var h=e(s,3);u=n(u),s=function(y){return h(d[y],y,d)}}var v=i(u,s,c);return v>-1?d[h?u[v]:v]:void 0}}return zb=r,zb}var Lb,s2;function EX(){if(s2)return Lb;s2=1;var e=iD();function t(n){var r=e(n),i=r%1;return r===r?i?r-i:r:0}return Lb=t,Lb}var Ib,c2;function TX(){if(c2)return Ib;c2=1;var e=jR(),t=$a(),n=EX(),r=Math.max;function i(u,s,c){var d=u==null?0:u.length;if(!d)return-1;var h=c==null?0:n(c);return h<0&&(h=r(d+h,0)),e(u,t(s,3),h)}return Ib=i,Ib}var Ub,f2;function MX(){if(f2)return Ub;f2=1;var e=AX(),t=TX(),n=e(t);return Ub=n,Ub}MX();var jX=VP();const CX=lt(jX);var PX=CX(function(e){return{x:e.left,y:e.top,width:e.width,height:e.height}},function(e){return["l",e.left,"t",e.top,"w",e.width,"h",e.height].join("")}),vD=q.createContext(void 0),yD=q.createContext(void 0),mD=q.createContext(void 0),RX=q.createContext({}),gD=q.createContext(void 0),bD=q.createContext(0),xD=q.createContext(0),d2=function(t){var n=t.state,r=n.xAxisMap,i=n.yAxisMap,u=n.offset,s=t.clipPathId,c=t.children,d=t.width,h=t.height,v=PX(u);return L.createElement(vD.Provider,{value:r},L.createElement(yD.Provider,{value:i},L.createElement(RX.Provider,{value:u},L.createElement(mD.Provider,{value:v},L.createElement(gD.Provider,{value:s},L.createElement(bD.Provider,{value:h},L.createElement(xD.Provider,{value:d},c)))))))},NX=function(){return q.useContext(gD)},_D=function(t){var n=q.useContext(vD);n==null&&Ai();var r=n[t];return r==null&&Ai(),r},SD=function(t){var n=q.useContext(yD);n==null&&Ai();var r=n[t];return r==null&&Ai(),r},DX=function(){var t=q.useContext(mD);return t},wD=function(){return q.useContext(xD)},OD=function(){return q.useContext(bD)};function Qo(e){"@babel/helpers - typeof";return Qo=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},Qo(e)}function kX(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function qX(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,ED(r.key),r)}}function $X(e,t,n){return t&&qX(e.prototype,t),Object.defineProperty(e,"prototype",{writable:!1}),e}function BX(e,t,n){return t=Ed(t),zX(e,AD()?Reflect.construct(t,n||[],Ed(e).constructor):t.apply(e,n))}function zX(e,t){if(t&&(Qo(t)==="object"||typeof t=="function"))return t;if(t!==void 0)throw new TypeError("Derived constructors may only return object or undefined");return LX(e)}function LX(e){if(e===void 0)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function AD(){try{var e=!Boolean.prototype.valueOf.call(Reflect.construct(Boolean,[],function(){}))}catch{}return(AD=function(){return!!e})()}function Ed(e){return Ed=Object.setPrototypeOf?Object.getPrototypeOf.bind():function(n){return n.__proto__||Object.getPrototypeOf(n)},Ed(e)}function IX(e,t){if(typeof t!="function"&&t!==null)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),Object.defineProperty(e,"prototype",{writable:!1}),t&&ax(e,t)}function ax(e,t){return ax=Object.setPrototypeOf?Object.setPrototypeOf.bind():function(r,i){return r.__proto__=i,r},ax(e,t)}function h2(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function p2(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?h2(Object(n),!0).forEach(function(r){E_(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):h2(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function E_(e,t,n){return t=ED(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function ED(e){var t=UX(e,"string");return Qo(t)=="symbol"?t:t+""}function UX(e,t){if(Qo(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(Qo(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return String(e)}function HX(e,t){return YX(e)||ZX(e,t)||VX(e,t)||GX()}function GX(){throw new TypeError(`Invalid attempt to destructure non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function VX(e,t){if(e){if(typeof e=="string")return v2(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);if(n==="Object"&&e.constructor&&(n=e.constructor.name),n==="Map"||n==="Set")return Array.from(e);if(n==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return v2(e,t)}}function v2(e,t){(t==null||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}function ZX(e,t){var n=e==null?null:typeof Symbol<"u"&&e[Symbol.iterator]||e["@@iterator"];if(n!=null){var r,i,u,s,c=[],d=!0,h=!1;try{if(u=(n=n.call(e)).next,t!==0)for(;!(d=(r=u.call(n)).done)&&(c.push(r.value),c.length!==t);d=!0);}catch(v){h=!0,i=v}finally{try{if(!d&&n.return!=null&&(s=n.return(),Object(s)!==s))return}finally{if(h)throw i}}return c}}function YX(e){if(Array.isArray(e))return e}function ix(){return ix=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},ix.apply(this,arguments)}var XX=function(t,n){var r;return L.isValidElement(t)?r=L.cloneElement(t,n):Ge(t)?r=t(n):r=L.createElement("line",ix({},n,{className:"recharts-reference-line-line"})),r},KX=function(t,n,r,i,u,s,c,d,h){var v=u.x,y=u.y,g=u.width,b=u.height;if(r){var S=h.y,_=t.y.apply(S,{position:s});if(vr(h,"discard")&&!t.y.isInRange(_))return null;var x=[{x:v+g,y:_},{x:v,y:_}];return d==="left"?x.reverse():x}if(n){var E=h.x,M=t.x.apply(E,{position:s});if(vr(h,"discard")&&!t.x.isInRange(M))return null;var A=[{x:M,y:y+b},{x:M,y}];return c==="top"?A.reverse():A}if(i){var C=h.segment,w=C.map(function(T){return t.apply(T,{position:s})});return vr(h,"discard")&&GY(w,function(T){return!t.isInRange(T)})?null:w}return null};function WX(e){var t=e.x,n=e.y,r=e.segment,i=e.xAxisId,u=e.yAxisId,s=e.shape,c=e.className,d=e.alwaysShow,h=NX(),v=_D(i),y=SD(u),g=DX();if(!h||!g)return null;vi(d===void 0,'The alwaysShow prop is deprecated. Please use ifOverflow="extendDomain" instead.');var b=A_({x:v.scale,y:y.scale}),S=Dt(t),_=Dt(n),x=r&&r.length===2,E=KX(b,S,_,x,g,e.position,v.orientation,y.orientation,e);if(!E)return null;var M=HX(E,2),A=M[0],C=A.x,w=A.y,T=M[1],P=T.x,R=T.y,$=vr(e,"hidden")?"url(#".concat(h,")"):void 0,J=p2(p2({clipPath:$},Ye(e,!0)),{},{x1:C,y1:w,x2:P,y2:R});return L.createElement(gt,{className:Xe("recharts-reference-line",c)},XX(s,J),Ft.renderCallByParent(e,SX({x1:C,y1:w,x2:P,y2:R})))}var T_=function(e){function t(){return kX(this,t),BX(this,t,arguments)}return IX(t,e),$X(t,[{key:"render",value:function(){return L.createElement(WX,this.props)}}])}(L.Component);E_(T_,"displayName","ReferenceLine");E_(T_,"defaultProps",{isFront:!1,ifOverflow:"discard",xAxisId:0,yAxisId:0,fill:"none",stroke:"#ccc",fillOpacity:1,strokeWidth:1,position:"middle"});function ox(){return ox=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},ox.apply(this,arguments)}function Jo(e){"@babel/helpers - typeof";return Jo=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},Jo(e)}function y2(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function m2(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?y2(Object(n),!0).forEach(function(r){dh(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):y2(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function FX(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function QX(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,MD(r.key),r)}}function JX(e,t,n){return t&&QX(e.prototype,t),Object.defineProperty(e,"prototype",{writable:!1}),e}function eK(e,t,n){return t=Td(t),tK(e,TD()?Reflect.construct(t,n||[],Td(e).constructor):t.apply(e,n))}function tK(e,t){if(t&&(Jo(t)==="object"||typeof t=="function"))return t;if(t!==void 0)throw new TypeError("Derived constructors may only return object or undefined");return nK(e)}function nK(e){if(e===void 0)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function TD(){try{var e=!Boolean.prototype.valueOf.call(Reflect.construct(Boolean,[],function(){}))}catch{}return(TD=function(){return!!e})()}function Td(e){return Td=Object.setPrototypeOf?Object.getPrototypeOf.bind():function(n){return n.__proto__||Object.getPrototypeOf(n)},Td(e)}function rK(e,t){if(typeof t!="function"&&t!==null)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),Object.defineProperty(e,"prototype",{writable:!1}),t&&ux(e,t)}function ux(e,t){return ux=Object.setPrototypeOf?Object.setPrototypeOf.bind():function(r,i){return r.__proto__=i,r},ux(e,t)}function dh(e,t,n){return t=MD(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function MD(e){var t=aK(e,"string");return Jo(t)=="symbol"?t:t+""}function aK(e,t){if(Jo(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(Jo(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return String(e)}var iK=function(t){var n=t.x,r=t.y,i=t.xAxis,u=t.yAxis,s=A_({x:i.scale,y:u.scale}),c=s.apply({x:n,y:r},{bandAware:!0});return vr(t,"discard")&&!s.isInRange(c)?null:c},hh=function(e){function t(){return FX(this,t),eK(this,t,arguments)}return rK(t,e),JX(t,[{key:"render",value:function(){var r=this.props,i=r.x,u=r.y,s=r.r,c=r.alwaysShow,d=r.clipPathId,h=Dt(i),v=Dt(u);if(vi(c===void 0,'The alwaysShow prop is deprecated. Please use ifOverflow="extendDomain" instead.'),!h||!v)return null;var y=iK(this.props);if(!y)return null;var g=y.x,b=y.y,S=this.props,_=S.shape,x=S.className,E=vr(this.props,"hidden")?"url(#".concat(d,")"):void 0,M=m2(m2({clipPath:E},Ye(this.props,!0)),{},{cx:g,cy:b});return L.createElement(gt,{className:Xe("recharts-reference-dot",x)},t.renderDot(_,M),Ft.renderCallByParent(this.props,{x:g-s,y:b-s,width:2*s,height:2*s}))}}])}(L.Component);dh(hh,"displayName","ReferenceDot");dh(hh,"defaultProps",{isFront:!1,ifOverflow:"discard",xAxisId:0,yAxisId:0,r:10,fill:"#fff",stroke:"#ccc",fillOpacity:1,strokeWidth:1});dh(hh,"renderDot",function(e,t){var n;return L.isValidElement(e)?n=L.cloneElement(e,t):Ge(e)?n=e(t):n=L.createElement(aD,ox({},t,{cx:t.cx,cy:t.cy,className:"recharts-reference-dot-dot"})),n});function lx(){return lx=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},lx.apply(this,arguments)}function eu(e){"@babel/helpers - typeof";return eu=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},eu(e)}function g2(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function b2(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?g2(Object(n),!0).forEach(function(r){ph(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):g2(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function oK(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function uK(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,CD(r.key),r)}}function lK(e,t,n){return t&&uK(e.prototype,t),Object.defineProperty(e,"prototype",{writable:!1}),e}function sK(e,t,n){return t=Md(t),cK(e,jD()?Reflect.construct(t,n||[],Md(e).constructor):t.apply(e,n))}function cK(e,t){if(t&&(eu(t)==="object"||typeof t=="function"))return t;if(t!==void 0)throw new TypeError("Derived constructors may only return object or undefined");return fK(e)}function fK(e){if(e===void 0)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function jD(){try{var e=!Boolean.prototype.valueOf.call(Reflect.construct(Boolean,[],function(){}))}catch{}return(jD=function(){return!!e})()}function Md(e){return Md=Object.setPrototypeOf?Object.getPrototypeOf.bind():function(n){return n.__proto__||Object.getPrototypeOf(n)},Md(e)}function dK(e,t){if(typeof t!="function"&&t!==null)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),Object.defineProperty(e,"prototype",{writable:!1}),t&&sx(e,t)}function sx(e,t){return sx=Object.setPrototypeOf?Object.setPrototypeOf.bind():function(r,i){return r.__proto__=i,r},sx(e,t)}function ph(e,t,n){return t=CD(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function CD(e){var t=hK(e,"string");return eu(t)=="symbol"?t:t+""}function hK(e,t){if(eu(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(eu(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return String(e)}var pK=function(t,n,r,i,u){var s=u.x1,c=u.x2,d=u.y1,h=u.y2,v=u.xAxis,y=u.yAxis;if(!v||!y)return null;var g=A_({x:v.scale,y:y.scale}),b={x:t?g.x.apply(s,{position:"start"}):g.x.rangeMin,y:r?g.y.apply(d,{position:"start"}):g.y.rangeMin},S={x:n?g.x.apply(c,{position:"end"}):g.x.rangeMax,y:i?g.y.apply(h,{position:"end"}):g.y.rangeMax};return vr(u,"discard")&&(!g.isInRange(b)||!g.isInRange(S))?null:hD(b,S)},vh=function(e){function t(){return oK(this,t),sK(this,t,arguments)}return dK(t,e),lK(t,[{key:"render",value:function(){var r=this.props,i=r.x1,u=r.x2,s=r.y1,c=r.y2,d=r.className,h=r.alwaysShow,v=r.clipPathId;vi(h===void 0,'The alwaysShow prop is deprecated. Please use ifOverflow="extendDomain" instead.');var y=Dt(i),g=Dt(u),b=Dt(s),S=Dt(c),_=this.props.shape;if(!y&&!g&&!b&&!S&&!_)return null;var x=pK(y,g,b,S,this.props);if(!x&&!_)return null;var E=vr(this.props,"hidden")?"url(#".concat(v,")"):void 0;return L.createElement(gt,{className:Xe("recharts-reference-area",d)},t.renderRect(_,b2(b2({clipPath:E},Ye(this.props,!0)),x)),Ft.renderCallByParent(this.props,x))}}])}(L.Component);ph(vh,"displayName","ReferenceArea");ph(vh,"defaultProps",{isFront:!1,ifOverflow:"discard",xAxisId:0,yAxisId:0,r:10,fill:"#ccc",fillOpacity:.5,stroke:"none",strokeWidth:1});ph(vh,"renderRect",function(e,t){var n;return L.isValidElement(e)?n=L.cloneElement(e,t):Ge(e)?n=e(t):n=L.createElement(O_,lx({},t,{className:"recharts-reference-area-rect"})),n});function PD(e,t,n){if(t<1)return[];if(t===1&&n===void 0)return e;for(var r=[],i=0;i<e.length;i+=t)r.push(e[i]);return r}function vK(e,t,n){var r={width:e.width+t.width,height:e.height+t.height};return OX(r,n)}function yK(e,t,n){var r=n==="width",i=e.x,u=e.y,s=e.width,c=e.height;return t===1?{start:r?i:u,end:r?i+s:u+c}:{start:r?i+s:u+c,end:r?i:u}}function jd(e,t,n,r,i){if(e*t<e*r||e*t>e*i)return!1;var u=n();return e*(t-e*u/2-r)>=0&&e*(t+e*u/2-i)<=0}function mK(e,t){return PD(e,t+1)}function gK(e,t,n,r,i){for(var u=(r||[]).slice(),s=t.start,c=t.end,d=0,h=1,v=s,y=function(){var S=r==null?void 0:r[d];if(S===void 0)return{v:PD(r,h)};var _=d,x,E=function(){return x===void 0&&(x=n(S,_)),x},M=S.coordinate,A=d===0||jd(e,M,E,v,c);A||(d=0,v=s,h+=1),A&&(v=M+e*(E()/2+i),d+=h)},g;h<=u.length;)if(g=y(),g)return g.v;return[]}function js(e){"@babel/helpers - typeof";return js=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},js(e)}function x2(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function Wt(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?x2(Object(n),!0).forEach(function(r){bK(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):x2(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function bK(e,t,n){return t=xK(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function xK(e){var t=_K(e,"string");return js(t)=="symbol"?t:t+""}function _K(e,t){if(js(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(js(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return(t==="string"?String:Number)(e)}function SK(e,t,n,r,i){for(var u=(r||[]).slice(),s=u.length,c=t.start,d=t.end,h=function(g){var b=u[g],S,_=function(){return S===void 0&&(S=n(b,g)),S};if(g===s-1){var x=e*(b.coordinate+e*_()/2-d);u[g]=b=Wt(Wt({},b),{},{tickCoord:x>0?b.coordinate-x*e:b.coordinate})}else u[g]=b=Wt(Wt({},b),{},{tickCoord:b.coordinate});var E=jd(e,b.tickCoord,_,c,d);E&&(d=b.tickCoord-e*(_()/2+i),u[g]=Wt(Wt({},b),{},{isShow:!0}))},v=s-1;v>=0;v--)h(v);return u}function wK(e,t,n,r,i,u){var s=(r||[]).slice(),c=s.length,d=t.start,h=t.end;if(u){var v=r[c-1],y=n(v,c-1),g=e*(v.coordinate+e*y/2-h);s[c-1]=v=Wt(Wt({},v),{},{tickCoord:g>0?v.coordinate-g*e:v.coordinate});var b=jd(e,v.tickCoord,function(){return y},d,h);b&&(h=v.tickCoord-e*(y/2+i),s[c-1]=Wt(Wt({},v),{},{isShow:!0}))}for(var S=u?c-1:c,_=function(M){var A=s[M],C,w=function(){return C===void 0&&(C=n(A,M)),C};if(M===0){var T=e*(A.coordinate-e*w()/2-d);s[M]=A=Wt(Wt({},A),{},{tickCoord:T<0?A.coordinate-T*e:A.coordinate})}else s[M]=A=Wt(Wt({},A),{},{tickCoord:A.coordinate});var P=jd(e,A.tickCoord,w,d,h);P&&(d=A.tickCoord+e*(w()/2+i),s[M]=Wt(Wt({},A),{},{isShow:!0}))},x=0;x<S;x++)_(x);return s}function OK(e,t,n){var r=e.tick,i=e.ticks,u=e.viewBox,s=e.minTickGap,c=e.orientation,d=e.interval,h=e.tickFormatter,v=e.unit,y=e.angle;if(!i||!i.length||!r)return[];if(he(d)||hu.isSsr)return mK(i,typeof d=="number"&&he(d)?d:0);var g=[],b=c==="top"||c==="bottom"?"width":"height",S=v&&b==="width"?Pl(v,{fontSize:t,letterSpacing:n}):{width:0,height:0},_=function(A,C){var w=Ge(h)?h(A.value,C):A.value;return b==="width"?vK(Pl(w,{fontSize:t,letterSpacing:n}),S,y):Pl(w,{fontSize:t,letterSpacing:n})[b]},x=i.length>=2?Fn(i[1].coordinate-i[0].coordinate):1,E=yK(u,x,b);return d==="equidistantPreserveStart"?gK(x,E,_,i,s):(d==="preserveStart"||d==="preserveStartEnd"?g=wK(x,E,_,i,s,d==="preserveStartEnd"):g=SK(x,E,_,i,s),g.filter(function(M){return M.isShow}))}var AK=["viewBox"],EK=["viewBox"],TK=["ticks"];function tu(e){"@babel/helpers - typeof";return tu=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},tu(e)}function Mo(){return Mo=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},Mo.apply(this,arguments)}function _2(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function en(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?_2(Object(n),!0).forEach(function(r){M_(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):_2(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function Hb(e,t){if(e==null)return{};var n=MK(e,t),r,i;if(Object.getOwnPropertySymbols){var u=Object.getOwnPropertySymbols(e);for(i=0;i<u.length;i++)r=u[i],!(t.indexOf(r)>=0)&&Object.prototype.propertyIsEnumerable.call(e,r)&&(n[r]=e[r])}return n}function MK(e,t){if(e==null)return{};var n={};for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){if(t.indexOf(r)>=0)continue;n[r]=e[r]}return n}function jK(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function S2(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,ND(r.key),r)}}function CK(e,t,n){return t&&S2(e.prototype,t),n&&S2(e,n),Object.defineProperty(e,"prototype",{writable:!1}),e}function PK(e,t,n){return t=Cd(t),RK(e,RD()?Reflect.construct(t,n||[],Cd(e).constructor):t.apply(e,n))}function RK(e,t){if(t&&(tu(t)==="object"||typeof t=="function"))return t;if(t!==void 0)throw new TypeError("Derived constructors may only return object or undefined");return NK(e)}function NK(e){if(e===void 0)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function RD(){try{var e=!Boolean.prototype.valueOf.call(Reflect.construct(Boolean,[],function(){}))}catch{}return(RD=function(){return!!e})()}function Cd(e){return Cd=Object.setPrototypeOf?Object.getPrototypeOf.bind():function(n){return n.__proto__||Object.getPrototypeOf(n)},Cd(e)}function DK(e,t){if(typeof t!="function"&&t!==null)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),Object.defineProperty(e,"prototype",{writable:!1}),t&&cx(e,t)}function cx(e,t){return cx=Object.setPrototypeOf?Object.setPrototypeOf.bind():function(r,i){return r.__proto__=i,r},cx(e,t)}function M_(e,t,n){return t=ND(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function ND(e){var t=kK(e,"string");return tu(t)=="symbol"?t:t+""}function kK(e,t){if(tu(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(tu(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return String(e)}var yh=function(e){function t(n){var r;return jK(this,t),r=PK(this,t,[n]),r.state={fontSize:"",letterSpacing:""},r}return DK(t,e),CK(t,[{key:"shouldComponentUpdate",value:function(r,i){var u=r.viewBox,s=Hb(r,AK),c=this.props,d=c.viewBox,h=Hb(c,EK);return!Ro(u,d)||!Ro(s,h)||!Ro(i,this.state)}},{key:"componentDidMount",value:function(){var r=this.layerReference;if(r){var i=r.getElementsByClassName("recharts-cartesian-axis-tick-value")[0];i&&this.setState({fontSize:window.getComputedStyle(i).fontSize,letterSpacing:window.getComputedStyle(i).letterSpacing})}}},{key:"getTickLineCoord",value:function(r){var i=this.props,u=i.x,s=i.y,c=i.width,d=i.height,h=i.orientation,v=i.tickSize,y=i.mirror,g=i.tickMargin,b,S,_,x,E,M,A=y?-1:1,C=r.tickSize||v,w=he(r.tickCoord)?r.tickCoord:r.coordinate;switch(h){case"top":b=S=r.coordinate,x=s+ +!y*d,_=x-A*C,M=_-A*g,E=w;break;case"left":_=x=r.coordinate,S=u+ +!y*c,b=S-A*C,E=b-A*g,M=w;break;case"right":_=x=r.coordinate,S=u+ +y*c,b=S+A*C,E=b+A*g,M=w;break;default:b=S=r.coordinate,x=s+ +y*d,_=x+A*C,M=_+A*g,E=w;break}return{line:{x1:b,y1:_,x2:S,y2:x},tick:{x:E,y:M}}}},{key:"getTickTextAnchor",value:function(){var r=this.props,i=r.orientation,u=r.mirror,s;switch(i){case"left":s=u?"start":"end";break;case"right":s=u?"end":"start";break;default:s="middle";break}return s}},{key:"getTickVerticalAnchor",value:function(){var r=this.props,i=r.orientation,u=r.mirror,s="end";switch(i){case"left":case"right":s="middle";break;case"top":s=u?"start":"end";break;default:s=u?"end":"start";break}return s}},{key:"renderAxisLine",value:function(){var r=this.props,i=r.x,u=r.y,s=r.width,c=r.height,d=r.orientation,h=r.mirror,v=r.axisLine,y=en(en(en({},Ye(this.props,!1)),Ye(v,!1)),{},{fill:"none"});if(d==="top"||d==="bottom"){var g=+(d==="top"&&!h||d==="bottom"&&h);y=en(en({},y),{},{x1:i,y1:u+g*c,x2:i+s,y2:u+g*c})}else{var b=+(d==="left"&&!h||d==="right"&&h);y=en(en({},y),{},{x1:i+b*s,y1:u,x2:i+b*s,y2:u+c})}return L.createElement("line",Mo({},y,{className:Xe("recharts-cartesian-axis-line",Ln(v,"className"))}))}},{key:"renderTicks",value:function(r,i,u){var s=this,c=this.props,d=c.tickLine,h=c.stroke,v=c.tick,y=c.tickFormatter,g=c.unit,b=OK(en(en({},this.props),{},{ticks:r}),i,u),S=this.getTickTextAnchor(),_=this.getTickVerticalAnchor(),x=Ye(this.props,!1),E=Ye(v,!1),M=en(en({},x),{},{fill:"none"},Ye(d,!1)),A=b.map(function(C,w){var T=s.getTickLineCoord(C),P=T.line,R=T.tick,$=en(en(en(en({textAnchor:S,verticalAnchor:_},x),{},{stroke:"none",fill:h},E),R),{},{index:w,payload:C,visibleTicksCount:b.length,tickFormatter:y});return L.createElement(gt,Mo({className:"recharts-cartesian-axis-tick",key:"tick-".concat(C.value,"-").concat(C.coordinate,"-").concat(C.tickCoord)},Wl(s.props,C,w)),d&&L.createElement("line",Mo({},M,P,{className:Xe("recharts-cartesian-axis-tick-line",Ln(d,"className"))})),v&&t.renderTickItem(v,$,"".concat(Ge(y)?y(C.value,w):C.value).concat(g||"")))});return L.createElement("g",{className:"recharts-cartesian-axis-ticks"},A)}},{key:"render",value:function(){var r=this,i=this.props,u=i.axisLine,s=i.width,c=i.height,d=i.ticksGenerator,h=i.className,v=i.hide;if(v)return null;var y=this.props,g=y.ticks,b=Hb(y,TK),S=g;return Ge(d)&&(S=g&&g.length>0?d(this.props):d(b)),s<=0||c<=0||!S||!S.length?null:L.createElement(gt,{className:Xe("recharts-cartesian-axis",h),ref:function(x){r.layerReference=x}},u&&this.renderAxisLine(),this.renderTicks(S,this.state.fontSize,this.state.letterSpacing),Ft.renderCallByParent(this.props))}}],[{key:"renderTickItem",value:function(r,i,u){var s;return L.isValidElement(r)?s=L.cloneElement(r,i):Ge(r)?s=r(i):s=L.createElement(Qf,Mo({},i,{className:"recharts-cartesian-axis-tick-value"}),u),s}}])}(q.Component);M_(yh,"displayName","CartesianAxis");M_(yh,"defaultProps",{x:0,y:0,width:0,height:0,viewBox:{x:0,y:0,width:0,height:0},orientation:"bottom",ticks:[],stroke:"#666",tickLine:!0,axisLine:!0,tick:!0,mirror:!1,minTickGap:5,tickSize:6,tickMargin:2,interval:"preserveEnd"});function nu(e){"@babel/helpers - typeof";return nu=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},nu(e)}function qK(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function $K(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,qD(r.key),r)}}function BK(e,t,n){return t&&$K(e.prototype,t),Object.defineProperty(e,"prototype",{writable:!1}),e}function zK(e,t,n){return t=Pd(t),LK(e,DD()?Reflect.construct(t,n||[],Pd(e).constructor):t.apply(e,n))}function LK(e,t){if(t&&(nu(t)==="object"||typeof t=="function"))return t;if(t!==void 0)throw new TypeError("Derived constructors may only return object or undefined");return IK(e)}function IK(e){if(e===void 0)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function DD(){try{var e=!Boolean.prototype.valueOf.call(Reflect.construct(Boolean,[],function(){}))}catch{}return(DD=function(){return!!e})()}function Pd(e){return Pd=Object.setPrototypeOf?Object.getPrototypeOf.bind():function(n){return n.__proto__||Object.getPrototypeOf(n)},Pd(e)}function UK(e,t){if(typeof t!="function"&&t!==null)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),Object.defineProperty(e,"prototype",{writable:!1}),t&&fx(e,t)}function fx(e,t){return fx=Object.setPrototypeOf?Object.setPrototypeOf.bind():function(r,i){return r.__proto__=i,r},fx(e,t)}function kD(e,t,n){return t=qD(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function qD(e){var t=HK(e,"string");return nu(t)=="symbol"?t:t+""}function HK(e,t){if(nu(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(nu(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return String(e)}var Gs=function(e){function t(){return qK(this,t),zK(this,t,arguments)}return UK(t,e),BK(t,[{key:"render",value:function(){return null}}])}(L.Component);kD(Gs,"displayName","ZAxis");kD(Gs,"defaultProps",{zAxisId:0,range:[64,64],scale:"auto",type:"number"});var GK=["option","isActive"];function kl(){return kl=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},kl.apply(this,arguments)}function VK(e,t){if(e==null)return{};var n=ZK(e,t),r,i;if(Object.getOwnPropertySymbols){var u=Object.getOwnPropertySymbols(e);for(i=0;i<u.length;i++)r=u[i],!(t.indexOf(r)>=0)&&Object.prototype.propertyIsEnumerable.call(e,r)&&(n[r]=e[r])}return n}function ZK(e,t){if(e==null)return{};var n={};for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){if(t.indexOf(r)>=0)continue;n[r]=e[r]}return n}function YK(e){var t=e.option,n=e.isActive,r=VK(e,GK);return typeof t=="string"?L.createElement(ex,kl({option:L.createElement(Xd,kl({type:t},r)),isActive:n,shapeType:"symbols"},r)):L.createElement(ex,kl({option:t,isActive:n,shapeType:"symbols"},r))}function ru(e){"@babel/helpers - typeof";return ru=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},ru(e)}function ql(){return ql=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},ql.apply(this,arguments)}function w2(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function Bn(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?w2(Object(n),!0).forEach(function(r){Ma(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):w2(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function XK(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function O2(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,BD(r.key),r)}}function KK(e,t,n){return t&&O2(e.prototype,t),n&&O2(e,n),Object.defineProperty(e,"prototype",{writable:!1}),e}function WK(e,t,n){return t=Rd(t),FK(e,$D()?Reflect.construct(t,n||[],Rd(e).constructor):t.apply(e,n))}function FK(e,t){if(t&&(ru(t)==="object"||typeof t=="function"))return t;if(t!==void 0)throw new TypeError("Derived constructors may only return object or undefined");return QK(e)}function QK(e){if(e===void 0)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function $D(){try{var e=!Boolean.prototype.valueOf.call(Reflect.construct(Boolean,[],function(){}))}catch{}return($D=function(){return!!e})()}function Rd(e){return Rd=Object.setPrototypeOf?Object.getPrototypeOf.bind():function(n){return n.__proto__||Object.getPrototypeOf(n)},Rd(e)}function JK(e,t){if(typeof t!="function"&&t!==null)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),Object.defineProperty(e,"prototype",{writable:!1}),t&&dx(e,t)}function dx(e,t){return dx=Object.setPrototypeOf?Object.setPrototypeOf.bind():function(r,i){return r.__proto__=i,r},dx(e,t)}function Ma(e,t,n){return t=BD(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function BD(e){var t=eW(e,"string");return ru(t)=="symbol"?t:t+""}function eW(e,t){if(ru(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(ru(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return String(e)}var Vs=function(e){function t(){var n;XK(this,t);for(var r=arguments.length,i=new Array(r),u=0;u<r;u++)i[u]=arguments[u];return n=WK(this,t,[].concat(i)),Ma(n,"state",{isAnimationFinished:!1}),Ma(n,"handleAnimationEnd",function(){n.setState({isAnimationFinished:!0})}),Ma(n,"handleAnimationStart",function(){n.setState({isAnimationFinished:!1})}),Ma(n,"id",qs("recharts-scatter-")),n}return JK(t,e),KK(t,[{key:"renderSymbolsStatically",value:function(r){var i=this,u=this.props,s=u.shape,c=u.activeShape,d=u.activeIndex,h=Ye(this.props,!1);return r.map(function(v,y){var g=d===y,b=g?c:s,S=Bn(Bn({},h),v);return L.createElement(gt,ql({className:"recharts-scatter-symbol",key:"symbol-".concat(v==null?void 0:v.cx,"-").concat(v==null?void 0:v.cy,"-").concat(v==null?void 0:v.size,"-").concat(y)},Wl(i.props,v,y),{role:"img"}),L.createElement(YK,ql({option:b,isActive:g,key:"symbol-".concat(y)},S)))})}},{key:"renderSymbolsWithAnimation",value:function(){var r=this,i=this.props,u=i.points,s=i.isAnimationActive,c=i.animationBegin,d=i.animationDuration,h=i.animationEasing,v=i.animationId,y=this.state.prevPoints;return L.createElement(Kr,{begin:c,duration:d,isActive:s,easing:h,from:{t:0},to:{t:1},key:"pie-".concat(v),onAnimationEnd:this.handleAnimationEnd,onAnimationStart:this.handleAnimationStart},function(g){var b=g.t,S=u.map(function(_,x){var E=y&&y[x];if(E){var M=cr(E.cx,_.cx),A=cr(E.cy,_.cy),C=cr(E.size,_.size);return Bn(Bn({},_),{},{cx:M(b),cy:A(b),size:C(b)})}var w=cr(0,_.size);return Bn(Bn({},_),{},{size:w(b)})});return L.createElement(gt,null,r.renderSymbolsStatically(S))})}},{key:"renderSymbols",value:function(){var r=this.props,i=r.points,u=r.isAnimationActive,s=this.state.prevPoints;return u&&i&&i.length&&(!s||!oh(s,i))?this.renderSymbolsWithAnimation():this.renderSymbolsStatically(i)}},{key:"renderErrorBar",value:function(){var r=this.props.isAnimationActive;if(r&&!this.state.isAnimationFinished)return null;var i=this.props,u=i.points,s=i.xAxis,c=i.yAxis,d=i.children,h=An(d,Us);return h?h.map(function(v,y){var g=v.props,b=g.direction,S=g.dataKey;return L.cloneElement(v,{key:"".concat(b,"-").concat(S,"-").concat(u[y]),data:u,xAxis:s,yAxis:c,layout:b==="x"?"vertical":"horizontal",dataPointFormatter:function(x,E){return{x:x.cx,y:x.cy,value:b==="x"?+x.node.x:+x.node.y,errorVal:nn(x,E)}}})}):null}},{key:"renderLine",value:function(){var r=this.props,i=r.points,u=r.line,s=r.lineType,c=r.lineJointType,d=Ye(this.props,!1),h=Ye(u,!1),v,y;if(s==="joint")v=i.map(function(A){return{x:A.cx,y:A.cy}});else if(s==="fitting"){var g=xL(i),b=g.xmin,S=g.xmax,_=g.a,x=g.b,E=function(C){return _*C+x};v=[{x:b,y:E(b)},{x:S,y:E(S)}]}var M=Bn(Bn(Bn({},d),{},{fill:"none",stroke:d&&d.fill},h),{},{points:v});return L.isValidElement(u)?y=L.cloneElement(u,M):Ge(u)?y=u(M):y=L.createElement(U0,ql({},M,{type:c})),L.createElement(gt,{className:"recharts-scatter-line",key:"recharts-scatter-line"},y)}},{key:"render",value:function(){var r=this.props,i=r.hide,u=r.points,s=r.line,c=r.className,d=r.xAxis,h=r.yAxis,v=r.left,y=r.top,g=r.width,b=r.height,S=r.id,_=r.isAnimationActive;if(i||!u||!u.length)return null;var x=this.state.isAnimationFinished,E=Xe("recharts-scatter",c),M=d&&d.allowDataOverflow,A=h&&h.allowDataOverflow,C=M||A,w=ze(S)?this.id:S;return L.createElement(gt,{className:E,clipPath:C?"url(#clipPath-".concat(w,")"):null},M||A?L.createElement("defs",null,L.createElement("clipPath",{id:"clipPath-".concat(w)},L.createElement("rect",{x:M?v:v-g/2,y:A?y:y-b/2,width:M?g:g*2,height:A?b:b*2}))):null,s&&this.renderLine(),this.renderErrorBar(),L.createElement(gt,{key:"recharts-scatter-symbols"},this.renderSymbols()),(!_||x)&&Pa.renderCallByParent(this.props,u))}}],[{key:"getDerivedStateFromProps",value:function(r,i){return r.animationId!==i.prevAnimationId?{prevAnimationId:r.animationId,curPoints:r.points,prevPoints:i.curPoints}:r.points!==i.curPoints?{curPoints:r.points}:null}}])}(q.PureComponent);Ma(Vs,"displayName","Scatter");Ma(Vs,"defaultProps",{xAxisId:0,yAxisId:0,zAxisId:0,legendType:"circle",lineType:"joint",lineJointType:"linear",data:[],shape:"circle",hide:!1,isAnimationActive:!hu.isSsr,animationBegin:0,animationDuration:400,animationEasing:"linear"});Ma(Vs,"getComposedData",function(e){var t=e.xAxis,n=e.yAxis,r=e.zAxis,i=e.item,u=e.displayedData,s=e.xAxisTicks,c=e.yAxisTicks,d=e.offset,h=i.props.tooltipType,v=An(i.props.children,Qx),y=ze(t.dataKey)?i.props.dataKey:t.dataKey,g=ze(n.dataKey)?i.props.dataKey:n.dataKey,b=r&&r.dataKey,S=r?r.range:Gs.defaultProps.range,_=S&&S[0],x=t.scale.bandwidth?t.scale.bandwidth():0,E=n.scale.bandwidth?n.scale.bandwidth():0,M=u.map(function(A,C){var w=nn(A,y),T=nn(A,g),P=!ze(b)&&nn(A,b)||"-",R=[{name:ze(t.dataKey)?i.props.name:t.name||t.dataKey,unit:t.unit||"",value:w,payload:A,dataKey:y,type:h},{name:ze(n.dataKey)?i.props.name:n.name||n.dataKey,unit:n.unit||"",value:T,payload:A,dataKey:g,type:h}];P!=="-"&&R.push({name:r.name||r.dataKey,unit:r.unit||"",value:P,payload:A,dataKey:b,type:h});var $=Ij({axis:t,ticks:s,bandSize:x,entry:A,index:C,dataKey:y}),J=Ij({axis:n,ticks:c,bandSize:E,entry:A,index:C,dataKey:g}),B=P!=="-"?r.scale(P):_,G=Math.sqrt(Math.max(B,0)/Math.PI);return Bn(Bn({},A),{},{cx:$,cy:J,x:$-G,y:J-G,xAxis:t,yAxis:n,zAxis:r,width:2*G,height:2*G,size:B,node:{x:w,y:T,z:P},tooltipPayload:R,tooltipPosition:{x:$,y:J},payload:A},v&&v[C]&&v[C].props)});return Bn({points:M},d)});function au(e){"@babel/helpers - typeof";return au=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},au(e)}function tW(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function nW(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,ID(r.key),r)}}function rW(e,t,n){return t&&nW(e.prototype,t),Object.defineProperty(e,"prototype",{writable:!1}),e}function aW(e,t,n){return t=Nd(t),iW(e,zD()?Reflect.construct(t,n||[],Nd(e).constructor):t.apply(e,n))}function iW(e,t){if(t&&(au(t)==="object"||typeof t=="function"))return t;if(t!==void 0)throw new TypeError("Derived constructors may only return object or undefined");return oW(e)}function oW(e){if(e===void 0)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function zD(){try{var e=!Boolean.prototype.valueOf.call(Reflect.construct(Boolean,[],function(){}))}catch{}return(zD=function(){return!!e})()}function Nd(e){return Nd=Object.setPrototypeOf?Object.getPrototypeOf.bind():function(n){return n.__proto__||Object.getPrototypeOf(n)},Nd(e)}function uW(e,t){if(typeof t!="function"&&t!==null)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),Object.defineProperty(e,"prototype",{writable:!1}),t&&hx(e,t)}function hx(e,t){return hx=Object.setPrototypeOf?Object.setPrototypeOf.bind():function(r,i){return r.__proto__=i,r},hx(e,t)}function LD(e,t,n){return t=ID(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function ID(e){var t=lW(e,"string");return au(t)=="symbol"?t:t+""}function lW(e,t){if(au(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(au(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return String(e)}function px(){return px=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},px.apply(this,arguments)}function sW(e){var t=e.xAxisId,n=wD(),r=OD(),i=_D(t);return i==null?null:L.createElement(yh,px({},i,{className:Xe("recharts-".concat(i.axisType," ").concat(i.axisType),i.className),viewBox:{x:0,y:0,width:n,height:r},ticksGenerator:function(s){return hi(s,!0)}}))}var mh=function(e){function t(){return tW(this,t),aW(this,t,arguments)}return uW(t,e),rW(t,[{key:"render",value:function(){return L.createElement(sW,this.props)}}])}(L.Component);LD(mh,"displayName","XAxis");LD(mh,"defaultProps",{allowDecimals:!0,hide:!1,orientation:"bottom",width:0,height:30,mirror:!1,xAxisId:0,tickCount:5,type:"category",padding:{left:0,right:0},allowDataOverflow:!1,scale:"auto",reversed:!1,allowDuplicatedCategory:!0});function iu(e){"@babel/helpers - typeof";return iu=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},iu(e)}function cW(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function fW(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,GD(r.key),r)}}function dW(e,t,n){return t&&fW(e.prototype,t),Object.defineProperty(e,"prototype",{writable:!1}),e}function hW(e,t,n){return t=Dd(t),pW(e,UD()?Reflect.construct(t,n||[],Dd(e).constructor):t.apply(e,n))}function pW(e,t){if(t&&(iu(t)==="object"||typeof t=="function"))return t;if(t!==void 0)throw new TypeError("Derived constructors may only return object or undefined");return vW(e)}function vW(e){if(e===void 0)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function UD(){try{var e=!Boolean.prototype.valueOf.call(Reflect.construct(Boolean,[],function(){}))}catch{}return(UD=function(){return!!e})()}function Dd(e){return Dd=Object.setPrototypeOf?Object.getPrototypeOf.bind():function(n){return n.__proto__||Object.getPrototypeOf(n)},Dd(e)}function yW(e,t){if(typeof t!="function"&&t!==null)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),Object.defineProperty(e,"prototype",{writable:!1}),t&&vx(e,t)}function vx(e,t){return vx=Object.setPrototypeOf?Object.setPrototypeOf.bind():function(r,i){return r.__proto__=i,r},vx(e,t)}function HD(e,t,n){return t=GD(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function GD(e){var t=mW(e,"string");return iu(t)=="symbol"?t:t+""}function mW(e,t){if(iu(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(iu(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return String(e)}function yx(){return yx=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},yx.apply(this,arguments)}var gW=function(t){var n=t.yAxisId,r=wD(),i=OD(),u=SD(n);return u==null?null:L.createElement(yh,yx({},u,{className:Xe("recharts-".concat(u.axisType," ").concat(u.axisType),u.className),viewBox:{x:0,y:0,width:r,height:i},ticksGenerator:function(c){return hi(c,!0)}}))},gh=function(e){function t(){return cW(this,t),hW(this,t,arguments)}return yW(t,e),dW(t,[{key:"render",value:function(){return L.createElement(gW,this.props)}}])}(L.Component);HD(gh,"displayName","YAxis");HD(gh,"defaultProps",{allowDuplicatedCategory:!0,allowDecimals:!0,hide:!1,orientation:"left",width:60,height:0,mirror:!1,yAxisId:0,tickCount:5,type:"number",padding:{top:0,bottom:0},allowDataOverflow:!1,scale:"auto",reversed:!1});function A2(e){return SW(e)||_W(e)||xW(e)||bW()}function bW(){throw new TypeError(`Invalid attempt to spread non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function xW(e,t){if(e){if(typeof e=="string")return mx(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);if(n==="Object"&&e.constructor&&(n=e.constructor.name),n==="Map"||n==="Set")return Array.from(e);if(n==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return mx(e,t)}}function _W(e){if(typeof Symbol<"u"&&e[Symbol.iterator]!=null||e["@@iterator"]!=null)return Array.from(e)}function SW(e){if(Array.isArray(e))return mx(e)}function mx(e,t){(t==null||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}var gx=function(t,n,r,i,u){var s=An(t,T_),c=An(t,hh),d=[].concat(A2(s),A2(c)),h=An(t,vh),v="".concat(i,"Id"),y=i[0],g=n;if(d.length&&(g=d.reduce(function(_,x){if(x.props[v]===r&&vr(x.props,"extendDomain")&&he(x.props[y])){var E=x.props[y];return[Math.min(_[0],E),Math.max(_[1],E)]}return _},g)),h.length){var b="".concat(y,"1"),S="".concat(y,"2");g=h.reduce(function(_,x){if(x.props[v]===r&&vr(x.props,"extendDomain")&&he(x.props[b])&&he(x.props[S])){var E=x.props[b],M=x.props[S];return[Math.min(_[0],E,M),Math.max(_[1],E,M)]}return _},g)}return u&&u.length&&(g=u.reduce(function(_,x){return he(x)?[Math.min(_[0],x),Math.max(_[1],x)]:_},g)),g},Gb={exports:{}},E2;function wW(){return E2||(E2=1,function(e){var t=Object.prototype.hasOwnProperty,n="~";function r(){}Object.create&&(r.prototype=Object.create(null),new r().__proto__||(n=!1));function i(d,h,v){this.fn=d,this.context=h,this.once=v||!1}function u(d,h,v,y,g){if(typeof v!="function")throw new TypeError("The listener must be a function");var b=new i(v,y||d,g),S=n?n+h:h;return d._events[S]?d._events[S].fn?d._events[S]=[d._events[S],b]:d._events[S].push(b):(d._events[S]=b,d._eventsCount++),d}function s(d,h){--d._eventsCount===0?d._events=new r:delete d._events[h]}function c(){this._events=new r,this._eventsCount=0}c.prototype.eventNames=function(){var h=[],v,y;if(this._eventsCount===0)return h;for(y in v=this._events)t.call(v,y)&&h.push(n?y.slice(1):y);return Object.getOwnPropertySymbols?h.concat(Object.getOwnPropertySymbols(v)):h},c.prototype.listeners=function(h){var v=n?n+h:h,y=this._events[v];if(!y)return[];if(y.fn)return[y.fn];for(var g=0,b=y.length,S=new Array(b);g<b;g++)S[g]=y[g].fn;return S},c.prototype.listenerCount=function(h){var v=n?n+h:h,y=this._events[v];return y?y.fn?1:y.length:0},c.prototype.emit=function(h,v,y,g,b,S){var _=n?n+h:h;if(!this._events[_])return!1;var x=this._events[_],E=arguments.length,M,A;if(x.fn){switch(x.once&&this.removeListener(h,x.fn,void 0,!0),E){case 1:return x.fn.call(x.context),!0;case 2:return x.fn.call(x.context,v),!0;case 3:return x.fn.call(x.context,v,y),!0;case 4:return x.fn.call(x.context,v,y,g),!0;case 5:return x.fn.call(x.context,v,y,g,b),!0;case 6:return x.fn.call(x.context,v,y,g,b,S),!0}for(A=1,M=new Array(E-1);A<E;A++)M[A-1]=arguments[A];x.fn.apply(x.context,M)}else{var C=x.length,w;for(A=0;A<C;A++)switch(x[A].once&&this.removeListener(h,x[A].fn,void 0,!0),E){case 1:x[A].fn.call(x[A].context);break;case 2:x[A].fn.call(x[A].context,v);break;case 3:x[A].fn.call(x[A].context,v,y);break;case 4:x[A].fn.call(x[A].context,v,y,g);break;default:if(!M)for(w=1,M=new Array(E-1);w<E;w++)M[w-1]=arguments[w];x[A].fn.apply(x[A].context,M)}}return!0},c.prototype.on=function(h,v,y){return u(this,h,v,y,!1)},c.prototype.once=function(h,v,y){return u(this,h,v,y,!0)},c.prototype.removeListener=function(h,v,y,g){var b=n?n+h:h;if(!this._events[b])return this;if(!v)return s(this,b),this;var S=this._events[b];if(S.fn)S.fn===v&&(!g||S.once)&&(!y||S.context===y)&&s(this,b);else{for(var _=0,x=[],E=S.length;_<E;_++)(S[_].fn!==v||g&&!S[_].once||y&&S[_].context!==y)&&x.push(S[_]);x.length?this._events[b]=x.length===1?x[0]:x:s(this,b)}return this},c.prototype.removeAllListeners=function(h){var v;return h?(v=n?n+h:h,this._events[v]&&s(this,v)):(this._events=new r,this._eventsCount=0),this},c.prototype.off=c.prototype.removeListener,c.prototype.addListener=c.prototype.on,c.prefixed=n,c.EventEmitter=c,e.exports=c}(Gb)),Gb.exports}var OW=wW();const AW=lt(OW);var Vb=new AW,Zb="recharts.syncMouseEvents";function Cs(e){"@babel/helpers - typeof";return Cs=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},Cs(e)}function EW(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function TW(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,VD(r.key),r)}}function MW(e,t,n){return t&&TW(e.prototype,t),Object.defineProperty(e,"prototype",{writable:!1}),e}function Yb(e,t,n){return t=VD(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function VD(e){var t=jW(e,"string");return Cs(t)=="symbol"?t:t+""}function jW(e,t){if(Cs(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(Cs(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return String(e)}var CW=function(){function e(){EW(this,e),Yb(this,"activeIndex",0),Yb(this,"coordinateList",[]),Yb(this,"layout","horizontal")}return MW(e,[{key:"setDetails",value:function(n){var r,i=n.coordinateList,u=i===void 0?null:i,s=n.container,c=s===void 0?null:s,d=n.layout,h=d===void 0?null:d,v=n.offset,y=v===void 0?null:v,g=n.mouseHandlerCallback,b=g===void 0?null:g;this.coordinateList=(r=u??this.coordinateList)!==null&&r!==void 0?r:[],this.container=c??this.container,this.layout=h??this.layout,this.offset=y??this.offset,this.mouseHandlerCallback=b??this.mouseHandlerCallback,this.activeIndex=Math.min(Math.max(this.activeIndex,0),this.coordinateList.length-1)}},{key:"focus",value:function(){this.spoofMouse()}},{key:"keyboardEvent",value:function(n){if(this.coordinateList.length!==0)switch(n.key){case"ArrowRight":{if(this.layout!=="horizontal")return;this.activeIndex=Math.min(this.activeIndex+1,this.coordinateList.length-1),this.spoofMouse();break}case"ArrowLeft":{if(this.layout!=="horizontal")return;this.activeIndex=Math.max(this.activeIndex-1,0),this.spoofMouse();break}}}},{key:"setIndex",value:function(n){this.activeIndex=n}},{key:"spoofMouse",value:function(){var n,r;if(this.layout==="horizontal"&&this.coordinateList.length!==0){var i=this.container.getBoundingClientRect(),u=i.x,s=i.y,c=i.height,d=this.coordinateList[this.activeIndex].coordinate,h=((n=window)===null||n===void 0?void 0:n.scrollX)||0,v=((r=window)===null||r===void 0?void 0:r.scrollY)||0,y=u+d+h,g=s+this.offset.top+c/2+v;this.mouseHandlerCallback({pageX:y,pageY:g})}}}])}();function PW(e,t,n){if(n==="number"&&t===!0&&Array.isArray(e)){var r=e==null?void 0:e[0],i=e==null?void 0:e[1];if(r&&i&&he(r)&&he(i))return!0}return!1}function RW(e,t,n,r){var i=r/2;return{stroke:"none",fill:"#ccc",x:e==="horizontal"?t.x-i:n.left+.5,y:e==="horizontal"?n.top+.5:t.y-i,width:e==="horizontal"?r:n.width-1,height:e==="horizontal"?n.height-1:r}}function ZD(e){var t=e.cx,n=e.cy,r=e.radius,i=e.startAngle,u=e.endAngle,s=Zt(t,n,r,i),c=Zt(t,n,r,u);return{points:[s,c],cx:t,cy:n,radius:r,startAngle:i,endAngle:u}}function NW(e,t,n){var r,i,u,s;if(e==="horizontal")r=t.x,u=r,i=n.top,s=n.top+n.height;else if(e==="vertical")i=t.y,s=i,r=n.left,u=n.left+n.width;else if(t.cx!=null&&t.cy!=null)if(e==="centric"){var c=t.cx,d=t.cy,h=t.innerRadius,v=t.outerRadius,y=t.angle,g=Zt(c,d,h,y),b=Zt(c,d,v,y);r=g.x,i=g.y,u=b.x,s=b.y}else return ZD(t);return[{x:r,y:i},{x:u,y:s}]}function Ps(e){"@babel/helpers - typeof";return Ps=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},Ps(e)}function T2(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function Tf(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?T2(Object(n),!0).forEach(function(r){DW(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):T2(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function DW(e,t,n){return t=kW(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function kW(e){var t=qW(e,"string");return Ps(t)=="symbol"?t:t+""}function qW(e,t){if(Ps(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(Ps(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return(t==="string"?String:Number)(e)}function $W(e){var t,n,r=e.element,i=e.tooltipEventType,u=e.isActive,s=e.activeCoordinate,c=e.activePayload,d=e.offset,h=e.activeTooltipIndex,v=e.tooltipAxisBandSize,y=e.layout,g=e.chartName,b=(t=r.props.cursor)!==null&&t!==void 0?t:(n=r.type.defaultProps)===null||n===void 0?void 0:n.cursor;if(!r||!b||!u||!s||g!=="ScatterChart"&&i!=="axis")return null;var S,_=U0;if(g==="ScatterChart")S=s,_=VZ;else if(g==="BarChart")S=RW(y,s,d,v),_=O_;else if(y==="radial"){var x=ZD(s),E=x.cx,M=x.cy,A=x.radius,C=x.startAngle,w=x.endAngle;S={cx:E,cy:M,startAngle:C,endAngle:w,innerRadius:A,outerRadius:A},_=WN}else S={points:NW(y,s,d)},_=U0;var T=Tf(Tf(Tf(Tf({stroke:"#ccc",pointerEvents:"none"},d),S),Ye(b,!1)),{},{payload:c,payloadIndex:h,className:Xe("recharts-tooltip-cursor",b.className)});return q.isValidElement(b)?q.cloneElement(b,T):q.createElement(_,T)}var BW=["item"],zW=["children","className","width","height","style","compact","title","desc"];function ou(e){"@babel/helpers - typeof";return ou=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},ou(e)}function jo(){return jo=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},jo.apply(this,arguments)}function M2(e,t){return UW(e)||IW(e,t)||XD(e,t)||LW()}function LW(){throw new TypeError(`Invalid attempt to destructure non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function IW(e,t){var n=e==null?null:typeof Symbol<"u"&&e[Symbol.iterator]||e["@@iterator"];if(n!=null){var r,i,u,s,c=[],d=!0,h=!1;try{if(u=(n=n.call(e)).next,t!==0)for(;!(d=(r=u.call(n)).done)&&(c.push(r.value),c.length!==t);d=!0);}catch(v){h=!0,i=v}finally{try{if(!d&&n.return!=null&&(s=n.return(),Object(s)!==s))return}finally{if(h)throw i}}return c}}function UW(e){if(Array.isArray(e))return e}function j2(e,t){if(e==null)return{};var n=HW(e,t),r,i;if(Object.getOwnPropertySymbols){var u=Object.getOwnPropertySymbols(e);for(i=0;i<u.length;i++)r=u[i],!(t.indexOf(r)>=0)&&Object.prototype.propertyIsEnumerable.call(e,r)&&(n[r]=e[r])}return n}function HW(e,t){if(e==null)return{};var n={};for(var r in e)if(Object.prototype.hasOwnProperty.call(e,r)){if(t.indexOf(r)>=0)continue;n[r]=e[r]}return n}function GW(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function VW(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,KD(r.key),r)}}function ZW(e,t,n){return t&&VW(e.prototype,t),Object.defineProperty(e,"prototype",{writable:!1}),e}function YW(e,t,n){return t=kd(t),XW(e,YD()?Reflect.construct(t,n||[],kd(e).constructor):t.apply(e,n))}function XW(e,t){if(t&&(ou(t)==="object"||typeof t=="function"))return t;if(t!==void 0)throw new TypeError("Derived constructors may only return object or undefined");return KW(e)}function KW(e){if(e===void 0)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function YD(){try{var e=!Boolean.prototype.valueOf.call(Reflect.construct(Boolean,[],function(){}))}catch{}return(YD=function(){return!!e})()}function kd(e){return kd=Object.setPrototypeOf?Object.getPrototypeOf.bind():function(n){return n.__proto__||Object.getPrototypeOf(n)},kd(e)}function WW(e,t){if(typeof t!="function"&&t!==null)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),Object.defineProperty(e,"prototype",{writable:!1}),t&&bx(e,t)}function bx(e,t){return bx=Object.setPrototypeOf?Object.setPrototypeOf.bind():function(r,i){return r.__proto__=i,r},bx(e,t)}function uu(e){return JW(e)||QW(e)||XD(e)||FW()}function FW(){throw new TypeError(`Invalid attempt to spread non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function XD(e,t){if(e){if(typeof e=="string")return xx(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);if(n==="Object"&&e.constructor&&(n=e.constructor.name),n==="Map"||n==="Set")return Array.from(e);if(n==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return xx(e,t)}}function QW(e){if(typeof Symbol<"u"&&e[Symbol.iterator]!=null||e["@@iterator"]!=null)return Array.from(e)}function JW(e){if(Array.isArray(e))return xx(e)}function xx(e,t){(t==null||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}function C2(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(i){return Object.getOwnPropertyDescriptor(e,i).enumerable})),n.push.apply(n,r)}return n}function re(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?C2(Object(n),!0).forEach(function(r){Ee(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):C2(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function Ee(e,t,n){return t=KD(t),t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function KD(e){var t=eF(e,"string");return ou(t)=="symbol"?t:t+""}function eF(e,t){if(ou(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(ou(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return(t==="string"?String:Number)(e)}var tF={xAxis:["bottom","top"],yAxis:["left","right"]},nF={width:"100%",height:"100%"},WD={x:0,y:0};function Mf(e){return e}var rF=function(t,n){return n==="horizontal"?t.x:n==="vertical"?t.y:n==="centric"?t.angle:t.radius},aF=function(t,n,r,i){var u=n.find(function(v){return v&&v.index===r});if(u){if(t==="horizontal")return{x:u.coordinate,y:i.y};if(t==="vertical")return{x:i.x,y:u.coordinate};if(t==="centric"){var s=u.coordinate,c=i.radius;return re(re(re({},i),Zt(i.cx,i.cy,c,s)),{},{angle:s,radius:c})}var d=u.coordinate,h=i.angle;return re(re(re({},i),Zt(i.cx,i.cy,d,h)),{},{angle:h,radius:d})}return WD},bh=function(t,n){var r=n.graphicalItems,i=n.dataStartIndex,u=n.dataEndIndex,s=(r??[]).reduce(function(c,d){var h=d.props.data;return h&&h.length?[].concat(uu(c),uu(h)):c},[]);return s.length>0?s:t&&t.length&&he(i)&&he(u)?t.slice(i,u+1):[]};function FD(e){return e==="number"?[0,"auto"]:void 0}var _x=function(t,n,r,i){var u=t.graphicalItems,s=t.tooltipAxis,c=bh(n,t);return r<0||!u||!u.length||r>=c.length?null:u.reduce(function(d,h){var v,y=(v=h.props.data)!==null&&v!==void 0?v:n;y&&t.dataStartIndex+t.dataEndIndex!==0&&t.dataEndIndex-t.dataStartIndex>=r&&(y=y.slice(t.dataStartIndex,t.dataEndIndex+1));var g;if(s.dataKey&&!s.allowDuplicatedCategory){var b=y===void 0?c:y;g=Lf(b,s.dataKey,i)}else g=y&&y[r]||c[r];return g?[].concat(uu(d),[YN(h,g)]):d},[])},P2=function(t,n,r,i){var u=i||{x:t.chartX,y:t.chartY},s=rF(u,r),c=t.orderedTooltipTicks,d=t.tooltipAxis,h=t.tooltipTicks,v=M7(s,c,h,d);if(v>=0&&h){var y=h[v]&&h[v].value,g=_x(t,n,v,y),b=aF(r,c,v,u);return{activeTooltipIndex:v,activeLabel:y,activePayload:g,activeCoordinate:b}}return null},iF=function(t,n){var r=n.axes,i=n.graphicalItems,u=n.axisType,s=n.axisIdKey,c=n.stackGroups,d=n.dataStartIndex,h=n.dataEndIndex,v=t.layout,y=t.children,g=t.stackOffset,b=VN(v,u);return r.reduce(function(S,_){var x,E=_.type.defaultProps!==void 0?re(re({},_.type.defaultProps),_.props):_.props,M=E.type,A=E.dataKey,C=E.allowDataOverflow,w=E.allowDuplicatedCategory,T=E.scale,P=E.ticks,R=E.includeHidden,$=E[s];if(S[$])return S;var J=bh(t.data,{graphicalItems:i.filter(function(ee){var te,ce=s in ee.props?ee.props[s]:(te=ee.type.defaultProps)===null||te===void 0?void 0:te[s];return ce===$}),dataStartIndex:d,dataEndIndex:h}),B=J.length,G,W,ne;PW(E.domain,C,M)&&(G=$0(E.domain,null,C),b&&(M==="number"||T!=="auto")&&(ne=Nl(J,A,"category")));var Q=FD(M);if(!G||G.length===0){var D,H=(D=E.domain)!==null&&D!==void 0?D:Q;if(A){if(G=Nl(J,A,M),M==="category"&&b){var U=bL(G);w&&U?(W=G,G=_d(0,B)):w||(G=Vj(H,G,_).reduce(function(ee,te){return ee.indexOf(te)>=0?ee:[].concat(uu(ee),[te])},[]))}else if(M==="category")w?G=G.filter(function(ee){return ee!==""&&!ze(ee)}):G=Vj(H,G,_).reduce(function(ee,te){return ee.indexOf(te)>=0||te===""||ze(te)?ee:[].concat(uu(ee),[te])},[]);else if(M==="number"){var ue=N7(J,i.filter(function(ee){var te,ce,_e=s in ee.props?ee.props[s]:(te=ee.type.defaultProps)===null||te===void 0?void 0:te[s],be="hide"in ee.props?ee.props.hide:(ce=ee.type.defaultProps)===null||ce===void 0?void 0:ce.hide;return _e===$&&(R||!be)}),A,u,v);ue&&(G=ue)}b&&(M==="number"||T!=="auto")&&(ne=Nl(J,A,"category"))}else b?G=_d(0,B):c&&c[$]&&c[$].hasStack&&M==="number"?G=g==="expand"?[0,1]:ZN(c[$].stackGroups,d,h):G=GN(J,i.filter(function(ee){var te=s in ee.props?ee.props[s]:ee.type.defaultProps[s],ce="hide"in ee.props?ee.props.hide:ee.type.defaultProps.hide;return te===$&&(R||!ce)}),M,v,!0);if(M==="number")G=gx(y,G,$,u,P),H&&(G=$0(H,G,C));else if(M==="category"&&H){var N=H,X=G.every(function(ee){return N.indexOf(ee)>=0});X&&(G=N)}}return re(re({},S),{},Ee({},$,re(re({},E),{},{axisType:u,domain:G,categoricalDomain:ne,duplicateDomain:W,originalDomain:(x=E.domain)!==null&&x!==void 0?x:Q,isCategorical:b,layout:v})))},{})},oF=function(t,n){var r=n.graphicalItems,i=n.Axis,u=n.axisType,s=n.axisIdKey,c=n.stackGroups,d=n.dataStartIndex,h=n.dataEndIndex,v=t.layout,y=t.children,g=bh(t.data,{graphicalItems:r,dataStartIndex:d,dataEndIndex:h}),b=g.length,S=VN(v,u),_=-1;return r.reduce(function(x,E){var M=E.type.defaultProps!==void 0?re(re({},E.type.defaultProps),E.props):E.props,A=M[s],C=FD("number");if(!x[A]){_++;var w;return S?w=_d(0,b):c&&c[A]&&c[A].hasStack?(w=ZN(c[A].stackGroups,d,h),w=gx(y,w,A,u)):(w=$0(C,GN(g,r.filter(function(T){var P,R,$=s in T.props?T.props[s]:(P=T.type.defaultProps)===null||P===void 0?void 0:P[s],J="hide"in T.props?T.props.hide:(R=T.type.defaultProps)===null||R===void 0?void 0:R.hide;return $===A&&!J}),"number",v),i.defaultProps.allowDataOverflow),w=gx(y,w,A,u)),re(re({},x),{},Ee({},A,re(re({axisType:u},i.defaultProps),{},{hide:!0,orientation:Ln(tF,"".concat(u,".").concat(_%2),null),domain:w,originalDomain:C,isCategorical:S,layout:v})))}return x},{})},uF=function(t,n){var r=n.axisType,i=r===void 0?"xAxis":r,u=n.AxisComp,s=n.graphicalItems,c=n.stackGroups,d=n.dataStartIndex,h=n.dataEndIndex,v=t.children,y="".concat(i,"Id"),g=An(v,u),b={};return g&&g.length?b=iF(t,{axes:g,graphicalItems:s,axisType:i,axisIdKey:y,stackGroups:c,dataStartIndex:d,dataEndIndex:h}):s&&s.length&&(b=oF(t,{Axis:u,graphicalItems:s,axisType:i,axisIdKey:y,stackGroups:c,dataStartIndex:d,dataEndIndex:h})),b},lF=function(t){var n=Ao(t),r=hi(n,!1,!0);return{tooltipTicks:r,orderedTooltipTicks:Wx(r,function(i){return i.coordinate}),tooltipAxis:n,tooltipAxisBandSize:dd(n,r)}},R2=function(t){var n=t.children,r=t.defaultShowTooltip,i=Sn(n,Wo),u=0,s=0;return t.data&&t.data.length!==0&&(s=t.data.length-1),i&&i.props&&(i.props.startIndex>=0&&(u=i.props.startIndex),i.props.endIndex>=0&&(s=i.props.endIndex)),{chartX:0,chartY:0,dataStartIndex:u,dataEndIndex:s,activeTooltipIndex:-1,isTooltipActive:!!r}},sF=function(t){return!t||!t.length?!1:t.some(function(n){var r=Hr(n&&n.type);return r&&r.indexOf("Bar")>=0})},N2=function(t){return t==="horizontal"?{numericAxisName:"yAxis",cateAxisName:"xAxis"}:t==="vertical"?{numericAxisName:"xAxis",cateAxisName:"yAxis"}:t==="centric"?{numericAxisName:"radiusAxis",cateAxisName:"angleAxis"}:{numericAxisName:"angleAxis",cateAxisName:"radiusAxis"}},cF=function(t,n){var r=t.props,i=t.graphicalItems,u=t.xAxisMap,s=u===void 0?{}:u,c=t.yAxisMap,d=c===void 0?{}:c,h=r.width,v=r.height,y=r.children,g=r.margin||{},b=Sn(y,Wo),S=Sn(y,No),_=Object.keys(d).reduce(function(w,T){var P=d[T],R=P.orientation;return!P.mirror&&!P.hide?re(re({},w),{},Ee({},R,w[R]+P.width)):w},{left:g.left||0,right:g.right||0}),x=Object.keys(s).reduce(function(w,T){var P=s[T],R=P.orientation;return!P.mirror&&!P.hide?re(re({},w),{},Ee({},R,Ln(w,"".concat(R))+P.height)):w},{top:g.top||0,bottom:g.bottom||0}),E=re(re({},x),_),M=E.bottom;b&&(E.bottom+=b.props.height||Wo.defaultProps.height),S&&n&&(E=P7(E,i,r,n));var A=h-E.left-E.right,C=v-E.top-E.bottom;return re(re({brushBottom:M},E),{},{width:Math.max(A,0),height:Math.max(C,0)})},fF=function(t,n){if(n==="xAxis")return t[n].width;if(n==="yAxis")return t[n].height},dF=function(t){var n=t.chartName,r=t.GraphicalChild,i=t.defaultTooltipEventType,u=i===void 0?"axis":i,s=t.validateTooltipEventTypes,c=s===void 0?["axis"]:s,d=t.axisComponents,h=t.legendContent,v=t.formatAxisMap,y=t.defaultProps,g=function(E,M){var A=M.graphicalItems,C=M.stackGroups,w=M.offset,T=M.updateId,P=M.dataStartIndex,R=M.dataEndIndex,$=E.barSize,J=E.layout,B=E.barGap,G=E.barCategoryGap,W=E.maxBarSize,ne=N2(J),Q=ne.numericAxisName,D=ne.cateAxisName,H=sF(A),U=[];return A.forEach(function(ue,N){var X=bh(E.data,{graphicalItems:[ue],dataStartIndex:P,dataEndIndex:R}),ee=ue.type.defaultProps!==void 0?re(re({},ue.type.defaultProps),ue.props):ue.props,te=ee.dataKey,ce=ee.maxBarSize,_e=ee["".concat(Q,"Id")],be=ee["".concat(D,"Id")],ke={},Pe=d.reduce(function(En,_r){var Pi=M["".concat(_r.axisType,"Map")],Zs=ee["".concat(_r.axisType,"Id")];Pi&&Pi[Zs]||_r.axisType==="zAxis"||Ai();var Ys=Pi[Zs];return re(re({},En),{},Ee(Ee({},_r.axisType,Ys),"".concat(_r.axisType,"Ticks"),hi(Ys)))},ke),fe=Pe[D],Oe=Pe["".concat(D,"Ticks")],Me=C&&C[_e]&&C[_e].hasStack&&V7(ue,C[_e].stackGroups),le=Hr(ue.type).indexOf("Bar")>=0,st=dd(fe,Oe),Ie=[],St=H&&j7({barSize:$,stackGroups:C,totalSize:fF(Pe,D)});if(le){var wt,dt,rr=ze(ce)?W:ce,xr=(wt=(dt=dd(fe,Oe,!0))!==null&&dt!==void 0?dt:rr)!==null&&wt!==void 0?wt:0;Ie=C7({barGap:B,barCategoryGap:G,bandSize:xr!==st?xr:st,sizeList:St[be],maxBarSize:rr}),xr!==st&&(Ie=Ie.map(function(En){return re(re({},En),{},{position:re(re({},En.position),{},{offset:En.position.offset-xr/2})})}))}var Ia=ue&&ue.type&&ue.type.getComposedData;Ia&&U.push({props:re(re({},Ia(re(re({},Pe),{},{displayedData:X,props:E,dataKey:te,item:ue,bandSize:st,barPosition:Ie,offset:w,stackedData:Me,layout:J,dataStartIndex:P,dataEndIndex:R}))),{},Ee(Ee(Ee({key:ue.key||"item-".concat(N)},Q,Pe[Q]),D,Pe[D]),"animationId",T)),childIndex:PL(ue,E.children),item:ue})}),U},b=function(E,M){var A=E.props,C=E.dataStartIndex,w=E.dataEndIndex,T=E.updateId;if(!iE({props:A}))return null;var P=A.children,R=A.layout,$=A.stackOffset,J=A.data,B=A.reverseStackOrder,G=N2(R),W=G.numericAxisName,ne=G.cateAxisName,Q=An(P,r),D=U7(J,Q,"".concat(W,"Id"),"".concat(ne,"Id"),$,B),H=d.reduce(function(ee,te){var ce="".concat(te.axisType,"Map");return re(re({},ee),{},Ee({},ce,uF(A,re(re({},te),{},{graphicalItems:Q,stackGroups:te.axisType===W&&D,dataStartIndex:C,dataEndIndex:w}))))},{}),U=cF(re(re({},H),{},{props:A,graphicalItems:Q}),M==null?void 0:M.legendBBox);Object.keys(H).forEach(function(ee){H[ee]=v(A,H[ee],U,ee.replace("Map",""),n)});var ue=H["".concat(ne,"Map")],N=lF(ue),X=g(A,re(re({},H),{},{dataStartIndex:C,dataEndIndex:w,updateId:T,graphicalItems:Q,stackGroups:D,offset:U}));return re(re({formattedGraphicalItems:X,graphicalItems:Q,offset:U,stackGroups:D},N),H)},S=function(x){function E(M){var A,C,w;return GW(this,E),w=YW(this,E,[M]),Ee(w,"eventEmitterSymbol",Symbol("rechartsEventEmitter")),Ee(w,"accessibilityManager",new CW),Ee(w,"handleLegendBBoxUpdate",function(T){if(T){var P=w.state,R=P.dataStartIndex,$=P.dataEndIndex,J=P.updateId;w.setState(re({legendBBox:T},b({props:w.props,dataStartIndex:R,dataEndIndex:$,updateId:J},re(re({},w.state),{},{legendBBox:T}))))}}),Ee(w,"handleReceiveSyncEvent",function(T,P,R){if(w.props.syncId===T){if(R===w.eventEmitterSymbol&&typeof w.props.syncMethod!="function")return;w.applySyncEvent(P)}}),Ee(w,"handleBrushChange",function(T){var P=T.startIndex,R=T.endIndex;if(P!==w.state.dataStartIndex||R!==w.state.dataEndIndex){var $=w.state.updateId;w.setState(function(){return re({dataStartIndex:P,dataEndIndex:R},b({props:w.props,dataStartIndex:P,dataEndIndex:R,updateId:$},w.state))}),w.triggerSyncEvent({dataStartIndex:P,dataEndIndex:R})}}),Ee(w,"handleMouseEnter",function(T){var P=w.getMouseInfo(T);if(P){var R=re(re({},P),{},{isTooltipActive:!0});w.setState(R),w.triggerSyncEvent(R);var $=w.props.onMouseEnter;Ge($)&&$(R,T)}}),Ee(w,"triggeredAfterMouseMove",function(T){var P=w.getMouseInfo(T),R=P?re(re({},P),{},{isTooltipActive:!0}):{isTooltipActive:!1};w.setState(R),w.triggerSyncEvent(R);var $=w.props.onMouseMove;Ge($)&&$(R,T)}),Ee(w,"handleItemMouseEnter",function(T){w.setState(function(){return{isTooltipActive:!0,activeItem:T,activePayload:T.tooltipPayload,activeCoordinate:T.tooltipPosition||{x:T.cx,y:T.cy}}})}),Ee(w,"handleItemMouseLeave",function(){w.setState(function(){return{isTooltipActive:!1}})}),Ee(w,"handleMouseMove",function(T){T.persist(),w.throttleTriggeredAfterMouseMove(T)}),Ee(w,"handleMouseLeave",function(T){w.throttleTriggeredAfterMouseMove.cancel();var P={isTooltipActive:!1};w.setState(P),w.triggerSyncEvent(P);var R=w.props.onMouseLeave;Ge(R)&&R(P,T)}),Ee(w,"handleOuterEvent",function(T){var P=CL(T),R=Ln(w.props,"".concat(P));if(P&&Ge(R)){var $,J;/.*touch.*/i.test(P)?J=w.getMouseInfo(T.changedTouches[0]):J=w.getMouseInfo(T),R(($=J)!==null&&$!==void 0?$:{},T)}}),Ee(w,"handleClick",function(T){var P=w.getMouseInfo(T);if(P){var R=re(re({},P),{},{isTooltipActive:!0});w.setState(R),w.triggerSyncEvent(R);var $=w.props.onClick;Ge($)&&$(R,T)}}),Ee(w,"handleMouseDown",function(T){var P=w.props.onMouseDown;if(Ge(P)){var R=w.getMouseInfo(T);P(R,T)}}),Ee(w,"handleMouseUp",function(T){var P=w.props.onMouseUp;if(Ge(P)){var R=w.getMouseInfo(T);P(R,T)}}),Ee(w,"handleTouchMove",function(T){T.changedTouches!=null&&T.changedTouches.length>0&&w.throttleTriggeredAfterMouseMove(T.changedTouches[0])}),Ee(w,"handleTouchStart",function(T){T.changedTouches!=null&&T.changedTouches.length>0&&w.handleMouseDown(T.changedTouches[0])}),Ee(w,"handleTouchEnd",function(T){T.changedTouches!=null&&T.changedTouches.length>0&&w.handleMouseUp(T.changedTouches[0])}),Ee(w,"handleDoubleClick",function(T){var P=w.props.onDoubleClick;if(Ge(P)){var R=w.getMouseInfo(T);P(R,T)}}),Ee(w,"handleContextMenu",function(T){var P=w.props.onContextMenu;if(Ge(P)){var R=w.getMouseInfo(T);P(R,T)}}),Ee(w,"triggerSyncEvent",function(T){w.props.syncId!==void 0&&Vb.emit(Zb,w.props.syncId,T,w.eventEmitterSymbol)}),Ee(w,"applySyncEvent",function(T){var P=w.props,R=P.layout,$=P.syncMethod,J=w.state.updateId,B=T.dataStartIndex,G=T.dataEndIndex;if(T.dataStartIndex!==void 0||T.dataEndIndex!==void 0)w.setState(re({dataStartIndex:B,dataEndIndex:G},b({props:w.props,dataStartIndex:B,dataEndIndex:G,updateId:J},w.state)));else if(T.activeTooltipIndex!==void 0){var W=T.chartX,ne=T.chartY,Q=T.activeTooltipIndex,D=w.state,H=D.offset,U=D.tooltipTicks;if(!H)return;if(typeof $=="function")Q=$(U,T);else if($==="value"){Q=-1;for(var ue=0;ue<U.length;ue++)if(U[ue].value===T.activeLabel){Q=ue;break}}var N=re(re({},H),{},{x:H.left,y:H.top}),X=Math.min(W,N.x+N.width),ee=Math.min(ne,N.y+N.height),te=U[Q]&&U[Q].value,ce=_x(w.state,w.props.data,Q),_e=U[Q]?{x:R==="horizontal"?U[Q].coordinate:X,y:R==="horizontal"?ee:U[Q].coordinate}:WD;w.setState(re(re({},T),{},{activeLabel:te,activeCoordinate:_e,activePayload:ce,activeTooltipIndex:Q}))}else w.setState(T)}),Ee(w,"renderCursor",function(T){var P,R=w.state,$=R.isTooltipActive,J=R.activeCoordinate,B=R.activePayload,G=R.offset,W=R.activeTooltipIndex,ne=R.tooltipAxisBandSize,Q=w.getTooltipEventType(),D=(P=T.props.active)!==null&&P!==void 0?P:$,H=w.props.layout,U=T.key||"_recharts-cursor";return L.createElement($W,{key:U,activeCoordinate:J,activePayload:B,activeTooltipIndex:W,chartName:n,element:T,isActive:D,layout:H,offset:G,tooltipAxisBandSize:ne,tooltipEventType:Q})}),Ee(w,"renderPolarAxis",function(T,P,R){var $=Ln(T,"type.axisType"),J=Ln(w.state,"".concat($,"Map")),B=T.type.defaultProps,G=B!==void 0?re(re({},B),T.props):T.props,W=J&&J[G["".concat($,"Id")]];return q.cloneElement(T,re(re({},W),{},{className:Xe($,W.className),key:T.key||"".concat(P,"-").concat(R),ticks:hi(W,!0)}))}),Ee(w,"renderPolarGrid",function(T){var P=T.props,R=P.radialLines,$=P.polarAngles,J=P.polarRadius,B=w.state,G=B.radiusAxisMap,W=B.angleAxisMap,ne=Ao(G),Q=Ao(W),D=Q.cx,H=Q.cy,U=Q.innerRadius,ue=Q.outerRadius;return q.cloneElement(T,{polarAngles:Array.isArray($)?$:hi(Q,!0).map(function(N){return N.coordinate}),polarRadius:Array.isArray(J)?J:hi(ne,!0).map(function(N){return N.coordinate}),cx:D,cy:H,innerRadius:U,outerRadius:ue,key:T.key||"polar-grid",radialLines:R})}),Ee(w,"renderLegend",function(){var T=w.state.formattedGraphicalItems,P=w.props,R=P.children,$=P.width,J=P.height,B=w.props.margin||{},G=$-(B.left||0)-(B.right||0),W=UN({children:R,formattedGraphicalItems:T,legendWidth:G,legendContent:h});if(!W)return null;var ne=W.item,Q=j2(W,BW);return q.cloneElement(ne,re(re({},Q),{},{chartWidth:$,chartHeight:J,margin:B,onBBoxUpdate:w.handleLegendBBoxUpdate}))}),Ee(w,"renderTooltip",function(){var T,P=w.props,R=P.children,$=P.accessibilityLayer,J=Sn(R,sr);if(!J)return null;var B=w.state,G=B.isTooltipActive,W=B.activeCoordinate,ne=B.activePayload,Q=B.activeLabel,D=B.offset,H=(T=J.props.active)!==null&&T!==void 0?T:G;return q.cloneElement(J,{viewBox:re(re({},D),{},{x:D.left,y:D.top}),active:H,label:Q,payload:H?ne:[],coordinate:W,accessibilityLayer:$})}),Ee(w,"renderBrush",function(T){var P=w.props,R=P.margin,$=P.data,J=w.state,B=J.offset,G=J.dataStartIndex,W=J.dataEndIndex,ne=J.updateId;return q.cloneElement(T,{key:T.key||"_recharts-brush",onChange:wf(w.handleBrushChange,T.props.onChange),data:$,x:he(T.props.x)?T.props.x:B.left,y:he(T.props.y)?T.props.y:B.top+B.height+B.brushBottom-(R.bottom||0),width:he(T.props.width)?T.props.width:B.width,startIndex:G,endIndex:W,updateId:"brush-".concat(ne)})}),Ee(w,"renderReferenceElement",function(T,P,R){if(!T)return null;var $=w,J=$.clipPathId,B=w.state,G=B.xAxisMap,W=B.yAxisMap,ne=B.offset,Q=T.type.defaultProps||{},D=T.props,H=D.xAxisId,U=H===void 0?Q.xAxisId:H,ue=D.yAxisId,N=ue===void 0?Q.yAxisId:ue;return q.cloneElement(T,{key:T.key||"".concat(P,"-").concat(R),xAxis:G[U],yAxis:W[N],viewBox:{x:ne.left,y:ne.top,width:ne.width,height:ne.height},clipPathId:J})}),Ee(w,"renderActivePoints",function(T){var P=T.item,R=T.activePoint,$=T.basePoint,J=T.childIndex,B=T.isRange,G=[],W=P.props.key,ne=P.item.type.defaultProps!==void 0?re(re({},P.item.type.defaultProps),P.item.props):P.item.props,Q=ne.activeDot,D=ne.dataKey,H=re(re({index:J,dataKey:D,cx:R.x,cy:R.y,r:4,fill:w_(P.item),strokeWidth:2,stroke:"#fff",payload:R.payload,value:R.value},Ye(Q,!1)),If(Q));return G.push(E.renderActiveDot(Q,H,"".concat(W,"-activePoint-").concat(J))),$?G.push(E.renderActiveDot(Q,re(re({},H),{},{cx:$.x,cy:$.y}),"".concat(W,"-basePoint-").concat(J))):B&&G.push(null),G}),Ee(w,"renderGraphicChild",function(T,P,R){var $=w.filterFormatItem(T,P,R);if(!$)return null;var J=w.getTooltipEventType(),B=w.state,G=B.isTooltipActive,W=B.tooltipAxis,ne=B.activeTooltipIndex,Q=B.activeLabel,D=w.props.children,H=Sn(D,sr),U=$.props,ue=U.points,N=U.isRange,X=U.baseLine,ee=$.item.type.defaultProps!==void 0?re(re({},$.item.type.defaultProps),$.item.props):$.item.props,te=ee.activeDot,ce=ee.hide,_e=ee.activeBar,be=ee.activeShape,ke=!!(!ce&&G&&H&&(te||_e||be)),Pe={};J!=="axis"&&H&&H.props.trigger==="click"?Pe={onClick:wf(w.handleItemMouseEnter,T.props.onClick)}:J!=="axis"&&(Pe={onMouseLeave:wf(w.handleItemMouseLeave,T.props.onMouseLeave),onMouseEnter:wf(w.handleItemMouseEnter,T.props.onMouseEnter)});var fe=q.cloneElement(T,re(re({},$.props),Pe));function Oe(_r){return typeof W.dataKey=="function"?W.dataKey(_r.payload):null}if(ke)if(ne>=0){var Me,le;if(W.dataKey&&!W.allowDuplicatedCategory){var st=typeof W.dataKey=="function"?Oe:"payload.".concat(W.dataKey.toString());Me=Lf(ue,st,Q),le=N&&X&&Lf(X,st,Q)}else Me=ue==null?void 0:ue[ne],le=N&&X&&X[ne];if(be||_e){var Ie=T.props.activeIndex!==void 0?T.props.activeIndex:ne;return[q.cloneElement(T,re(re(re({},$.props),Pe),{},{activeIndex:Ie})),null,null]}if(!ze(Me))return[fe].concat(uu(w.renderActivePoints({item:$,activePoint:Me,basePoint:le,childIndex:ne,isRange:N})))}else{var St,wt=(St=w.getItemByXY(w.state.activeCoordinate))!==null&&St!==void 0?St:{graphicalItem:fe},dt=wt.graphicalItem,rr=dt.item,xr=rr===void 0?T:rr,Ia=dt.childIndex,En=re(re(re({},$.props),Pe),{},{activeIndex:Ia});return[q.cloneElement(xr,En),null,null]}return N?[fe,null,null]:[fe,null]}),Ee(w,"renderCustomized",function(T,P,R){return q.cloneElement(T,re(re({key:"recharts-customized-".concat(R)},w.props),w.state))}),Ee(w,"renderMap",{CartesianGrid:{handler:Mf,once:!0},ReferenceArea:{handler:w.renderReferenceElement},ReferenceLine:{handler:Mf},ReferenceDot:{handler:w.renderReferenceElement},XAxis:{handler:Mf},YAxis:{handler:Mf},Brush:{handler:w.renderBrush,once:!0},Bar:{handler:w.renderGraphicChild},Line:{handler:w.renderGraphicChild},Area:{handler:w.renderGraphicChild},Radar:{handler:w.renderGraphicChild},RadialBar:{handler:w.renderGraphicChild},Scatter:{handler:w.renderGraphicChild},Pie:{handler:w.renderGraphicChild},Funnel:{handler:w.renderGraphicChild},Tooltip:{handler:w.renderCursor,once:!0},PolarGrid:{handler:w.renderPolarGrid,once:!0},PolarAngleAxis:{handler:w.renderPolarAxis},PolarRadiusAxis:{handler:w.renderPolarAxis},Customized:{handler:w.renderCustomized}}),w.clipPathId="".concat((A=M.id)!==null&&A!==void 0?A:qs("recharts"),"-clip"),w.throttleTriggeredAfterMouseMove=UR(w.triggeredAfterMouseMove,(C=M.throttleDelay)!==null&&C!==void 0?C:1e3/60),w.state={},w}return WW(E,x),ZW(E,[{key:"componentDidMount",value:function(){var A,C;this.addListener(),this.accessibilityManager.setDetails({container:this.container,offset:{left:(A=this.props.margin.left)!==null&&A!==void 0?A:0,top:(C=this.props.margin.top)!==null&&C!==void 0?C:0},coordinateList:this.state.tooltipTicks,mouseHandlerCallback:this.triggeredAfterMouseMove,layout:this.props.layout}),this.displayDefaultTooltip()}},{key:"displayDefaultTooltip",value:function(){var A=this.props,C=A.children,w=A.data,T=A.height,P=A.layout,R=Sn(C,sr);if(R){var $=R.props.defaultIndex;if(!(typeof $!="number"||$<0||$>this.state.tooltipTicks.length-1)){var J=this.state.tooltipTicks[$]&&this.state.tooltipTicks[$].value,B=_x(this.state,w,$,J),G=this.state.tooltipTicks[$].coordinate,W=(this.state.offset.top+T)/2,ne=P==="horizontal",Q=ne?{x:G,y:W}:{y:G,x:W},D=this.state.formattedGraphicalItems.find(function(U){var ue=U.item;return ue.type.name==="Scatter"});D&&(Q=re(re({},Q),D.props.points[$].tooltipPosition),B=D.props.points[$].tooltipPayload);var H={activeTooltipIndex:$,isTooltipActive:!0,activeLabel:J,activePayload:B,activeCoordinate:Q};this.setState(H),this.renderCursor(R),this.accessibilityManager.setIndex($)}}}},{key:"getSnapshotBeforeUpdate",value:function(A,C){if(!this.props.accessibilityLayer)return null;if(this.state.tooltipTicks!==C.tooltipTicks&&this.accessibilityManager.setDetails({coordinateList:this.state.tooltipTicks}),this.props.layout!==A.layout&&this.accessibilityManager.setDetails({layout:this.props.layout}),this.props.margin!==A.margin){var w,T;this.accessibilityManager.setDetails({offset:{left:(w=this.props.margin.left)!==null&&w!==void 0?w:0,top:(T=this.props.margin.top)!==null&&T!==void 0?T:0}})}return null}},{key:"componentDidUpdate",value:function(A){i0([Sn(A.children,sr)],[Sn(this.props.children,sr)])||this.displayDefaultTooltip()}},{key:"componentWillUnmount",value:function(){this.removeListener(),this.throttleTriggeredAfterMouseMove.cancel()}},{key:"getTooltipEventType",value:function(){var A=Sn(this.props.children,sr);if(A&&typeof A.props.shared=="boolean"){var C=A.props.shared?"axis":"item";return c.indexOf(C)>=0?C:u}return u}},{key:"getMouseInfo",value:function(A){if(!this.container)return null;var C=this.container,w=C.getBoundingClientRect(),T=g8(w),P={chartX:Math.round(A.pageX-T.left),chartY:Math.round(A.pageY-T.top)},R=w.width/C.offsetWidth||1,$=this.inRange(P.chartX,P.chartY,R);if(!$)return null;var J=this.state,B=J.xAxisMap,G=J.yAxisMap,W=this.getTooltipEventType(),ne=P2(this.state,this.props.data,this.props.layout,$);if(W!=="axis"&&B&&G){var Q=Ao(B).scale,D=Ao(G).scale,H=Q&&Q.invert?Q.invert(P.chartX):null,U=D&&D.invert?D.invert(P.chartY):null;return re(re({},P),{},{xValue:H,yValue:U},ne)}return ne?re(re({},P),ne):null}},{key:"inRange",value:function(A,C){var w=arguments.length>2&&arguments[2]!==void 0?arguments[2]:1,T=this.props.layout,P=A/w,R=C/w;if(T==="horizontal"||T==="vertical"){var $=this.state.offset,J=P>=$.left&&P<=$.left+$.width&&R>=$.top&&R<=$.top+$.height;return J?{x:P,y:R}:null}var B=this.state,G=B.angleAxisMap,W=B.radiusAxisMap;if(G&&W){var ne=Ao(G);return Xj({x:P,y:R},ne)}return null}},{key:"parseEventsOfWrapper",value:function(){var A=this.props.children,C=this.getTooltipEventType(),w=Sn(A,sr),T={};w&&C==="axis"&&(w.props.trigger==="click"?T={onClick:this.handleClick}:T={onMouseEnter:this.handleMouseEnter,onDoubleClick:this.handleDoubleClick,onMouseMove:this.handleMouseMove,onMouseLeave:this.handleMouseLeave,onTouchMove:this.handleTouchMove,onTouchStart:this.handleTouchStart,onTouchEnd:this.handleTouchEnd,onContextMenu:this.handleContextMenu});var P=If(this.props,this.handleOuterEvent);return re(re({},P),T)}},{key:"addListener",value:function(){Vb.on(Zb,this.handleReceiveSyncEvent)}},{key:"removeListener",value:function(){Vb.removeListener(Zb,this.handleReceiveSyncEvent)}},{key:"filterFormatItem",value:function(A,C,w){for(var T=this.state.formattedGraphicalItems,P=0,R=T.length;P<R;P++){var $=T[P];if($.item===A||$.props.key===A.key||C===Hr($.item.type)&&w===$.childIndex)return $}return null}},{key:"renderClipPath",value:function(){var A=this.clipPathId,C=this.state.offset,w=C.left,T=C.top,P=C.height,R=C.width;return L.createElement("defs",null,L.createElement("clipPath",{id:A},L.createElement("rect",{x:w,y:T,height:P,width:R})))}},{key:"getXScales",value:function(){var A=this.state.xAxisMap;return A?Object.entries(A).reduce(function(C,w){var T=M2(w,2),P=T[0],R=T[1];return re(re({},C),{},Ee({},P,R.scale))},{}):null}},{key:"getYScales",value:function(){var A=this.state.yAxisMap;return A?Object.entries(A).reduce(function(C,w){var T=M2(w,2),P=T[0],R=T[1];return re(re({},C),{},Ee({},P,R.scale))},{}):null}},{key:"getXScaleByAxisId",value:function(A){var C;return(C=this.state.xAxisMap)===null||C===void 0||(C=C[A])===null||C===void 0?void 0:C.scale}},{key:"getYScaleByAxisId",value:function(A){var C;return(C=this.state.yAxisMap)===null||C===void 0||(C=C[A])===null||C===void 0?void 0:C.scale}},{key:"getItemByXY",value:function(A){var C=this.state,w=C.formattedGraphicalItems,T=C.activeItem;if(w&&w.length)for(var P=0,R=w.length;P<R;P++){var $=w[P],J=$.props,B=$.item,G=B.type.defaultProps!==void 0?re(re({},B.type.defaultProps),B.props):B.props,W=Hr(B.type);if(W==="Bar"){var ne=(J.data||[]).find(function(U){return kZ(A,U)});if(ne)return{graphicalItem:$,payload:ne}}else if(W==="RadialBar"){var Q=(J.data||[]).find(function(U){return Xj(A,U)});if(Q)return{graphicalItem:$,payload:Q}}else if(sh($,T)||ch($,T)||As($,T)){var D=OY({graphicalItem:$,activeTooltipItem:T,itemData:G.data}),H=G.activeIndex===void 0?D:G.activeIndex;return{graphicalItem:re(re({},$),{},{childIndex:H}),payload:As($,T)?G.data[D]:$.props.data[D]}}}return null}},{key:"render",value:function(){var A=this;if(!iE(this))return null;var C=this.props,w=C.children,T=C.className,P=C.width,R=C.height,$=C.style,J=C.compact,B=C.title,G=C.desc,W=j2(C,zW),ne=Ye(W,!1);if(J)return L.createElement(d2,{state:this.state,width:this.props.width,height:this.props.height,clipPathId:this.clipPathId},L.createElement(u0,jo({},ne,{width:P,height:R,title:B,desc:G}),this.renderClipPath(),uE(w,this.renderMap)));if(this.props.accessibilityLayer){var Q,D;ne.tabIndex=(Q=this.props.tabIndex)!==null&&Q!==void 0?Q:0,ne.role=(D=this.props.role)!==null&&D!==void 0?D:"application",ne.onKeyDown=function(U){A.accessibilityManager.keyboardEvent(U)},ne.onFocus=function(){A.accessibilityManager.focus()}}var H=this.parseEventsOfWrapper();return L.createElement(d2,{state:this.state,width:this.props.width,height:this.props.height,clipPathId:this.clipPathId},L.createElement("div",jo({className:Xe("recharts-wrapper",T),style:re({position:"relative",cursor:"default",width:P,height:R},$)},H,{ref:function(ue){A.container=ue}}),L.createElement(u0,jo({},ne,{width:P,height:R,title:B,desc:G,style:nF}),this.renderClipPath(),uE(w,this.renderMap)),this.renderLegend(),this.renderTooltip()))}}])}(q.Component);Ee(S,"displayName",n),Ee(S,"defaultProps",re({layout:"horizontal",stackOffset:"none",barCategoryGap:"10%",barGap:4,margin:{top:5,right:5,bottom:5,left:5},reverseStackOrder:!1,syncMethod:"index"},y)),Ee(S,"getDerivedStateFromProps",function(x,E){var M=x.dataKey,A=x.data,C=x.children,w=x.width,T=x.height,P=x.layout,R=x.stackOffset,$=x.margin,J=E.dataStartIndex,B=E.dataEndIndex;if(E.updateId===void 0){var G=R2(x);return re(re(re({},G),{},{updateId:0},b(re(re({props:x},G),{},{updateId:0}),E)),{},{prevDataKey:M,prevData:A,prevWidth:w,prevHeight:T,prevLayout:P,prevStackOffset:R,prevMargin:$,prevChildren:C})}if(M!==E.prevDataKey||A!==E.prevData||w!==E.prevWidth||T!==E.prevHeight||P!==E.prevLayout||R!==E.prevStackOffset||!Ro($,E.prevMargin)){var W=R2(x),ne={chartX:E.chartX,chartY:E.chartY,isTooltipActive:E.isTooltipActive},Q=re(re({},P2(E,A,P)),{},{updateId:E.updateId+1}),D=re(re(re({},W),ne),Q);return re(re(re({},D),b(re({props:x},D),E)),{},{prevDataKey:M,prevData:A,prevWidth:w,prevHeight:T,prevLayout:P,prevStackOffset:R,prevMargin:$,prevChildren:C})}if(!i0(C,E.prevChildren)){var H,U,ue,N,X=Sn(C,Wo),ee=X&&(H=(U=X.props)===null||U===void 0?void 0:U.startIndex)!==null&&H!==void 0?H:J,te=X&&(ue=(N=X.props)===null||N===void 0?void 0:N.endIndex)!==null&&ue!==void 0?ue:B,ce=ee!==J||te!==B,_e=!ze(A),be=_e&&!ce?E.updateId:E.updateId+1;return re(re({updateId:be},b(re(re({props:x},E),{},{updateId:be,dataStartIndex:ee,dataEndIndex:te}),E)),{},{prevChildren:C,dataStartIndex:ee,dataEndIndex:te})}return null}),Ee(S,"renderActiveDot",function(x,E,M){var A;return q.isValidElement(x)?A=q.cloneElement(x,E):Ge(x)?A=x(E):A=L.createElement(aD,E),L.createElement(gt,{className:"recharts-active-dot",key:M},A)});var _=q.forwardRef(function(E,M){return L.createElement(S,jo({},E,{ref:M}))});return _.displayName=S.displayName,_},hF=dF({chartName:"ScatterChart",GraphicalChild:Vs,defaultTooltipEventType:"item",validateTooltipEventTypes:["item"],axisComponents:[{axisType:"xAxis",AxisComp:mh},{axisType:"yAxis",AxisComp:gh},{axisType:"zAxis",AxisComp:Gs}],formatAxisMap:_X});const pF=({clusters:e})=>{const t=e.map(d=>({label:d.name,x:d.x_coord,y:d.y_coord,id:d.id})),{minX:n,maxX:r,minY:i,maxY:u,xRange:s,yRange:c}=q.useMemo(()=>{const d=t.map(S=>S.x),h=t.map(S=>S.y),v=Math.min(...d),y=Math.max(...d),g=Math.min(...h),b=Math.max(...h);return{minX:v,maxX:y,minY:g,maxY:b,xRange:y-v,yRange:b-g}},[t]);return V.jsx(f8,{children:V.jsxs(hF,{margin:{top:20,right:20,bottom:20,left:20},width:1e3,height:1e3,children:[V.jsx(sr,{cursor:{strokeDasharray:"3 3"},content:({payload:d})=>d&&d[0]?V.jsx("div",{className:"bg-white p-2 border rounded shadow",children:d[0].payload.label}):null}),V.jsx(mh,{type:"number",dataKey:"x",domain:[n-s*.05,r+s*.05],name:"X",tickFormatter:d=>d.toFixed(2)}),V.jsx(gh,{type:"number",dataKey:"y",domain:[i-c*.05,u+c*.05],name:"Y",tickFormatter:d=>d.toFixed(2)}),V.jsx(Gs,{range:[50,200]}),V.jsx(Vs,{name:"Clusters",data:t,fill:"#8884d8",cursor:"pointer"})]})})};function vF(){const[e,t]=q.useState(null),[n,r]=q.useState(null),[i,u]=q.useState(null),[s,c]=q.useState(new Map),[d,h]=q.useState(null),[v,y]=q.useState(null),[g,b]=q.useState([]),S=()=>{t(null),r(null),u(null),c(new Map),h(null),y(null)},_=()=>{var A;if(!i||!e)return;const x=new Map;for(const C of e){const w=(A=n==null?void 0:n.find(P=>P.chat_id===C.chat_id))==null?void 0:A.summary;if(!w)throw new Error(`No summary found for conversation ${C.chat_id}`);const T=uP.parse({...C,summary:w});x.set(C.chat_id,T)}c(x);const E=cP(i,null,0);h(E);const M=sP(E,[]);b(M)};return V.jsxs("div",{className:"min-h-screen flex flex-col",children:[!d&&V.jsx("div",{className:"p-4 ",children:V.jsx(DB,{setConversations:t,conversations:e,setSummaries:r,summaries:n,setClusters:u,clusters:i,handleVisualiseClusters:_})}),d&&V.jsxs("div",{className:"flex flex-1 h-screen",children:[V.jsxs("div",{className:"w-1/3 min-w-[300px] border-r flex flex-col",children:[V.jsx("div",{className:"p-4 border-b",children:V.jsxs("div",{className:"flex justify-between items-center",children:[V.jsx("h2",{className:"text-lg font-bold",children:"Cluster Hierarchy"}),V.jsxs(oP,{variant:"ghost",size:"sm",onClick:S,className:"text-slate-500 hover:text-slate-700",children:[V.jsx(dP,{className:"h-4 w-4 mr-1"}),"Reset"]})]})}),V.jsx("div",{className:"p-4 overflow-y-auto h-[40vh] border-b",children:V.jsx(t0,{clusterTree:d,onSelectCluster:y})}),v&&V.jsx("div",{className:"flex-1 overflow-y-auto",children:V.jsx(P4,{selectedCluster:v,conversationMetadataMap:s})})]}),V.jsx("div",{className:"flex-1 flex items-center justify-center text-slate-700 font-bold",children:v&&i&&V.jsx(pF,{clusters:g.filter(x=>x.level==v.level)})})]})]})}O$.createRoot(document.getElementById("root")).render(V.jsx(q.StrictMode,{children:V.jsx(vF,{})}));
